﻿Clear-Host

#Move to process path
$PmsSYSPath = Split-Path (Split-Path ($MyInvocation.MyCommand.Path) -Parent) -Parent
Set-Location $PmsSYSPath

#Declaration
$HostUrl = "http://localhost:8090"
$WebConfigLocation = Resolve-Path -Path './Web.config'
$CurrentDateFormat = Get-Date -UFormat %Y%m%d%T | ForEach-Object { $_ -replace ":", "" }
$ProcessStatus = "Fail"
$FileList = @('DRG3G041.C', 'DRG3G041.D', 'DRG43155.C', 'DRG43155.D', 'DRG43156.C', 'DRG43156.D', 'DRG43157.C', 'DRG43157.D', 'DRG43158.C', 'DRG43158.D')
$LogFilename = "$CurrentDateFormat.log"
$LogPath = "Logs/DRG/$LogFilename"

#FTP connection
$ftp = "ftp://10.192.32.135/" 
$user = 'snaprod' 
$pass = 'snaprod'

#Assign token
$ApiToken = "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJzc0NvbUNvZGUiOiIwMSIsInNzVXNlcklEIjoiMCIsInNzVXNlck5hbWUiOiJMTF9SdW5EcmFnb24iLCJHZW5lcmF0ZURhdGUiOiI3LzIvMjAyMSA2OjM3OjIxIFBNIiwic3NJUEFkZHJlc3MiOiI6OjEiLCJpc3MiOiJOTVQiLCJhdWQiOiJOTVQifQ.iDOVBaY3BuNffK8BC7NIdiyzxGDfb4WHTHXsQ-ae4n4"
$Token = ConvertTo-SecureString -String $ApiToken -AsPlainText -Force
$Headers = @{Authorization = "Bearer $ApiToken" }

#Functions
function VerifyFileList($ProcessPath, $writeInformation) {
    $Verified = 'P'
    $TotalFiles = 0
    foreach ($FileListName in $FileList) {
        $matchFileName = (Get-ChildItem -Path $ProcessPath -Filter $FileListName* | % { $_.Name }) -join ', '
        $matchFileCount = (Get-ChildItem -Path $ProcessPath -Filter $FileListName* | Measure-Object).Count

        if ($null -eq $matchFileName) {
            $Verified = 'F'

            if ($writeInformation -eq $true) {Write-Host "`tX $FileListName"}
        }
        elseif ($matchFileCount -gt 1) {
            $Verified = 'F'

            if ($writeInformation -eq $true) {Write-Host "`tX $matchFileName"}
        }
        else {
            if ($writeInformation -eq $true) {Write-Host "`t√ $matchFileName"}
            $TotalFiles = $TotalFiles + $matchFileCount
        }
    }

    if ($TotalFiles -ne 10) {
        $Verified = 'F'
    }

    return $Verified
}

function MoveFileToProcessPath ($File, $ProcessPath) {
    $FileCount = 0

    function Get-FtpDir ($url, $credentials) {
      $request = [Net.WebRequest]::Create($url)
      $request.Method = [System.Net.WebRequestMethods+FTP]::ListDirectory
      if ($credentials) { $request.Credentials = $credentials }
      $response = $request.GetResponse()
      $reader = New-Object IO.StreamReader $response.GetResponseStream() 
      while (-not $reader.EndOfStream) {
        $reader.ReadLine()
      }
      $reader.Close()
      $response.Close()
    }
    $credentials = New-Object System.Net.NetworkCredential($user, $pass)
    
    $folderPath = $ftp
    $files = Get-FTPDir -url $folderPath -credentials $credentials

    foreach ($filecontain in $FileList) {
        foreach ($file in ($files | Where-Object {$_ -like "$filecontain*"})){
            if($FileCount -eq 0) {
                CheckProcessPath
            }

            $source = $folderPath + $file
            $destination = "$ProcessPath$file"

            #Download method
            $webclient = New-Object System.Net.WebClient 
            $webclient.Credentials = $credentials
            $webclient.DownloadFile($source, $destination)
            
            #Delete method
            $ftpclient = [System.Net.FtpWebRequest]::Create($source)
            $ftpclient.Credentials = $credentials
            $ftpclient.Method = [System.Net.WebRequestMethods+Ftp]::DeleteFile
            $ftpclient.UseBinary = $true
            $ftpclient.UsePassive = $true
            $response = [System.Net.FtpWebResponse]$ftpclient.GetResponse()
            $response.Close()

            $FileCount++
        }
    }

    return $FileCount
}

function GetProcessLogs () {
    $dateFormat = Get-Date -Format "ddMMyyyy"
    $url = "$HostUrl/api/Batch/GetProcessLogs/NIN/$dateFormat"
    
    $response = Invoke-WebRequest -Method Get -Uri $url -ContentType "application/json" -Headers $Headers -UseBasicParsing
    
    $statusCode = $response.StatusCode
    $lines = $response.Content.Trim('"').Split([string[]]"\r\n", [StringSplitOptions]::None)

    Write-Host "***********************************"
    Write-Host "Start TPP Process logs."
    Write-Host "***********************************"

    if ($statusCode -eq 200) {
        foreach ($line in $lines) {
            Write-Host $line
        }
    }
    
    Write-Host "***********************************"
    Write-Host "End TPP Process logs."
    Write-Host "***********************************"
}

function ExitAndFail () {
    try {
        Stop-Transcript | Out-Null
    } catch [System.InvalidOperationException] {}
    
    $url = "$HostUrl/api/System/SendEmail/DRGFailed"
    $Body = @(
        @{
            path = "/$LogPath"
            name = $LogFilename
        }
    ) | ConvertTo-Json

    Invoke-WebRequest -Method Post -Uri $url -ContentType "application/json" -Headers $Headers -UseBasicParsing -Body $Body
    
    exit
}

function CheckProcessPath () {

    Write-Information "Info: `t- Checking process path." -InformationAction Continue
    $Verified = VerifyFileList $ProcessPath $false
    $EmptyStatus = Get-ChildItem $ProcessPath | ForEach-Object { if ($_.GetType().Name -eq "FileInfo") { "Not-Empty" } }

    if ($Verified -eq 'P' -and $EmptyStatus -eq "Not-Empty") {
        Write-Host "Error: `t- Process path is not empty." -InformationAction Continue
        ExitAndFail
    }
}


#Start logging
Start-Transcript -Path $LogPath

Write-Output "-----------------------------------"
Write-Output "Start Dragon process."
Write-Output "$(Get-Date)"
Write-Output "-----------------------------------"

Write-Output "Info: Get configuration."
$ProcessPath = Select-Xml -Path $WebConfigLocation -XPath '/configuration/appSettings/add' | ForEach-Object { if ($_.Node.key -eq "DRGProcessPath") { $_.Node.value } }
$BackupPath = (Join-Path -Path $ProcessPath -ChildPath "./Backup")

if ($null -eq $ProcessPath) {
    Write-Output "No Dragon process path be configured."
    ExitAndFail
}

#Test path
if (!(Test-Path -PathType Container -Path $ProcessPath)) {
    New-Item -ItemType Directory -Force -Path $ProcessPath
}
if (!(Test-Path -PathType Container -Path $BackupPath)) {
    New-Item -ItemType Directory -Force -Path $BackupPath
}

#Set path
$ProcessPath = Resolve-Path -Path $ProcessPath
$BackupPath = Resolve-Path -Path $BackupPath
Write-Output "Info: FTP path is $ftp"
Write-Output "Info: Process path is $ProcessPath"


#List DRG files from FTP
Write-Output "Info: Getting files from FTP."
$FileCount = MoveFileToProcessPath $File $ProcessPath
Write-Output "Info: Total $FileCount files found."


Write-Output "Info: Final preparing process directory."
$UnusedFiles = @(".DS_Store")
foreach ($File in $UnusedFiles) {
    $File = Join-Path -Path $ProcessPath -ChildPath $File

    if (Test-Path -Path $File) {
        Remove-Item -Path $File -Force
    }
}

Write-Output "Info: Verifing file list."
$Verified = VerifyFileList $ProcessPath $true

if ($Verified -eq 'F') {
    Write-Output "***********************************"
    Write-Output "Error: File list is not completed."
    Write-Output "***********************************"
    ExitAndFail
}

#Begin readfile process
$S1Url = "$HostUrl/api/Batch/DRG/ReadFileProcessPath"
$Body = @(
    @{
        ProcessPath = $ProcessPath.ToString()
    }
) | ConvertTo-Json

Write-Output "Info: (Step #1) Reading files into TPM_WORKFILE_DRG_..."
try {

    $response = Invoke-WebRequest -Method Post -Uri $S1Url -ContentType "application/json" -Headers $Headers -UseBasicParsing -Body $Body
}
finally {
    #another features eg. Send an email if error.
    $statusCode = $response.StatusCode
    if ($statusCode -eq 200) {
        Write-Output "Info: $response"
        # echo "Done"
    }
    else {
        Write-Output "Error: Process not completed ($statusCode)"
    }
}

#Begin receive dragon process
$S2Url = "$HostUrl/api/Batch/DRG/StartProcess"

Write-Output "Info: (Step #2) Begin Dragon process job."
try {
    $response = Invoke-WebRequest -Method Post -Uri $S2Url -ContentType "application/json" -Headers $Headers -UseBasicParsing
}
finally {
    #another features eg. Send an email if error.
    $statusCode = $response.StatusCode
    $result = $response | ConvertFrom-Json
    if ($statusCode -eq 200 -and $result.result -eq "Done") {
        Write-Output "Info: $response"
        $ProcessStatus = "Done"
    }
    else {
        Write-Output "Error: Process not completed. ""$($result.message.trim())"" ($statusCode)"
        GetProcessLogs
    }
}

#Backup processed files
if ($ProcessStatus -eq "Done") {

    Write-Output "Info: Backup file in process path."
    $ProcessedFiles = Get-ChildItem -Path $ProcessPath
    $FileCount = 0
    foreach ($File in $ProcessedFiles) {
        if ($File.GetType().Name -eq "FileInfo") {
            $FileCount++
            $FileName = $File.Name
            $NewFileName = ($FileName -replace $File.Extension, "") + "." + $CurrentDateFormat + ($File.Extension -replace "\.", "")
            $SourceLocation = Join-Path -Path $ProcessPath -ChildPath $FileName
            $DestinationLocation = Join-Path -Path $BackupPath -ChildPath $NewFileName

            Write-Output "Info: Backing up $SourceLocation to $DestinationLocation"
            Move-Item -Path $SourceLocation -Destination $DestinationLocation
        }
    }
}

Write-Output "-----------------------------------"
Write-Output "End Dragon process."
Write-Output "$(Get-Date)"
Write-Output "-----------------------------------"

#End logging
Stop-Transcript


if ($ProcessStatus -eq "Fail") {
    ExitAndFail
}
else {
    $url = "$HostUrl/api/System/SendEmail/DRGPassed"

    Invoke-WebRequest -Method Post -Uri $url -ContentType "application/json" -Headers $Headers -UseBasicParsing
}