Clear-Host

#Move to process path
$PmsSYSPath = Split-Path $MyInvocation.MyCommand.Path -Parent
Set-Location $PmsSYSPath
<#
Clear-Host

#Move to process path
$PmsSYSPath = Split-Path (Split-Path ($MyInvocation.MyCommand.Path) -Parent) -Parent
Set-Location $PmsSYSPath
Set-Location -Path ".\tempFile\ReportPath\PrintInvoices"
#>

#Set printing property
$ReportPrefix = "NPMS_INV_"
$ReportSuffix = ".PDF"
$PrinterName = "TestInvoicePrint"

$PrintTime = Get-Date -Format "yyyymmddHHMMss"

Function CallReport() {
    $HostUrl = "http://localhost:8090"
    $Url = "$HostUrl/api/Batch/PrintInvoices"
    $Body = @(
        @{
            ReportPrefix = $ReportPrefix
            ReportSuffix = $ReportSuffix
            PrinterName = $PrinterName
        }
    ) | ConvertTo-Json

    Invoke-WebRequest -Method Post -Uri $Url -ContentType "application/json" -Body $Body -UseBasicParsing -OutFile $ReportPrefix$PrintTime$ReportSuffix
}

Function MoveToBackup($File) {
    $BackupPath = Join-Path -Path . -ChildPath ".\backup"
    
    if (!(Test-Path -PathType Container -Path $BackupPath)) {
        New-Item -ItemType Directory -Force -Path $BackupPath
    }

    Move-Item -Path $File -Destination $BackupPath
}

Function GetPrinterJobCount() {
    $Printer = Get-Printer -Name $PrinterName
    
    $PrintJobs = Get-PrintJob -PrinterObject $Printer

    $JobCount = (($PrintJobs | Where-Object {$_.JobStatus -eq "Normal" -or $_.JobStatus -like "Printing*"}) | Measure-Object).Count
    Write-Output $JobCount
}

Function CheckRegisterJob($FirstJobCount) {

    $timeOut = new-timespan -Minutes 3
    $stopWatch = [System.Diagnostics.Stopwatch]::StartNew()

    While ($stopWatch.elapsed -lt $timeOut) {
    
        $CurrentJobCount = GetPrinterJobCount

        Write-Host $CurrentJobCount

        if($FirstJobCount -lt $CurrentJobCount){
            return $true
        }
        Start-Sleep -Seconds 10
    }

    Write-Host "TimeOut!!"

}

Function RegisterJob($PrintFile) {
    Write-Host "Printreport: "($PrintFile.Name)
    $Process = Start-Process -FilePath $PrintFile.Name -Verb PrintTo($PrinterName) -WindowStyle Hidden -PassThru

    Write-Host "Checking job change"

    if ((CheckRegisterJob -FirstJobCount $FirstJobCount) -eq $true) {
        $FirstJobCount = GetPrinterJobCount
        $Process.Close()
        try {
            MoveToBackup $PrintFile.Name
        } catch {
            Start-Sleep 10
            MoveToBackup $PrintFile.Name
        }
    }
}

$FirstJobCount = GetPrinterJobCount

Write-Host "Gettin Report"
CallReport

$Files = Get-ChildItem -Path . -Filter "$ReportPrefix*$ReportSuffix"
foreach($File in $Files) {
    RegisterJob -PrintFile $File
}
