using System;
using System.Collections.Generic;
using System.Configuration;
using System.Data;
using System.Globalization;
using System.IO;
using System.Net;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Threading.Tasks;
using System.Web;
using System.Web.Http;
using System.Web.Http.Cors;
using Oracle.ManagedDataAccess.Client;
using SYS_Core.aControl;
using SYS_Core.Controllers;
using SYSCore.aControl;
using SYSCore.ReportingService;
using static SYSCore.aControl.EmailSender;
using ClosedXML.Excel;

namespace SYSCore.Controllers
{
    [EnableCors("*", "*", "*")]
    [RoutePrefix("api/Batch")]
    public class BatchApiController : ApiController
    {
        private DBManager odb = new DBManager();
        private String gSQL = "";
        private String mDBName = ConfigurationManager.AppSettings["SYSDBName"];
        private DBOracle oOracledb = new DBOracle();

        private DataService dataService = new DataService();

        [HttpGet, Route("Allowance")]
        [Authorize]
        public IHttpActionResult RunAllowance()
        {
            try
            {
                if (DataService.GetIdentity(User, Cons.SS_USER_NAME) != "LL_RunAllowance")
                {
                    throw new Exception("Invalid");
                }

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_PRC_TEMP_ALLOWANCE";
                oOracledb.SqlQueryReturnValue(cmd);

                return Ok();

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "RunAllowance");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.BadRequest));
            }
        }

        [HttpGet, Route("PriceUpdate")]
        [Authorize]
        public async Task<IHttpActionResult> PriceUpdate()
        {
            try
            {
                if (DataService.GetIdentity(User, Cons.SS_USER_NAME) != "LL_SystemCheckup")
                {
                    throw new Exception("Invalid");
                }

                String DateFormat = DateTime.Now.AddDays(-1).ToString("dd-MMM-yyyy");
                string emailConfig = "PriceUpdate";
                string reportName = "PMS_RPT_TI_BASE_COST_MOVEMENT";
                string reportType = "EXCELOPENXML";
                string attachmentName = DateFormat + "_" + "Daily_Report_CostMovement.xlsx";
                string attachmentMime = "application/vnd.ms-excel";
                //DataService.GetIdentity(User, Cons.SS_COMPANY_CODE)
                List<ParameterValue> reportParams = new List<ParameterValue>();
                reportParams.Add(new ParameterValue { Name = "P_COMPANY_CODE", Value = "01", Label = "P_COMPANY_CODE" });
                byte[] reportBytes = await ReportServices.GenerateReportAsBytes(reportName, reportType, reportParams, attachmentName);

                List<EmailAttachment> emailAttachments = new List<EmailAttachment>();
                emailAttachments.Add(new EmailAttachment(reportBytes, attachmentName, attachmentMime));


                //Get email configuration
                EmailConfigure configure = dataService.GetEmailConfigure(emailConfig);
                configure.attachments = emailAttachments.ToArray();
                configure.content = configure.content
                                       .Replace("{PriceDate}", DateFormat);
                //Send email with attachments
                EmailSender sender = new EmailSender();
                sender.Email(configure);
                return Ok("OK");
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "CheckPriceUpdate");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.BadRequest));
            }
        }
        [HttpGet, Route("CheckReceivePONotUpdate")]
        [Authorize]
        public async Task<IHttpActionResult> CheckReceivePONotUpdate()
        {
            try
            {
                if (DataService.GetIdentity(User, Cons.SS_USER_NAME) != "LL_SystemCheckup")
                {
                    throw new Exception("Invalid");
                }
                String DateFormat = DateTime.Now.ToString("yyyyMMddhHHmmss");
                string emailConfig = "CheckNotUpdate";
                string attachmentName = DateFormat + "_" + "ReceivePONotUpdate.xlsx";
                string attachmentMime = "application/vnd.ms-excel";
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_CHECK_RECEIVE_ORDER_NOT_UPDATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                dt = ds.Tables[0];
                if(dt.Rows.Count > 0)
                {
                    using (XLWorkbook wb = new XLWorkbook())
                    {
                        wb.Worksheets.Add(dt, "sheet1");
                        using (MemoryStream stream = new MemoryStream())
                        {
                            wb.SaveAs(stream);
                            byte[] reportBytes = stream.ToArray();
                            List<EmailAttachment> emailAttachments = new List<EmailAttachment>();
                            emailAttachments.Add(new EmailAttachment(reportBytes, attachmentName, attachmentMime));
                            EmailConfigure configure = dataService.GetEmailConfigure(emailConfig);
                            configure.attachments = emailAttachments.ToArray();
                            EmailSender sender = new EmailSender();
                            sender.Email(configure);
                        }
                    }
                
                }

                return Ok("OK");

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "CheckPriceUpdate");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.BadRequest));
            }
        }


        [HttpGet, Route("Check")]
        [Authorize]
        public IHttpActionResult CheckBatch()
        {
            try
            {
                if (DataService.GetIdentity(User, Cons.SS_USER_NAME) != "LL_SystemCheckup")
                {
                    throw new Exception("Invalid");
                }

                String DateFormat = DateTime.Now.ToString("yyyyMMddhHHmmss");
                List<EmailAttachment> attachments = new List<EmailAttachment>();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_GET_BATCH_STATUS";
                cmd.Parameters.Add("R_PROGRAM", OracleDbType.Varchar2, 30).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 5).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string gPrgName = cmd.Parameters["R_PROGRAM"].Value.ToString();
                string gStatusCode = cmd.Parameters["R_STATUS"].Value.ToString();
                string gStatus = "";
                string emailConfig = "";

                switch (gStatusCode)
                {
                    case "P":
                        gStatus = "Passed";
                        emailConfig = "BatchPassed";
                        break;
                    case "R":
                        gStatus = "Running";
                        emailConfig = "BatchRunning";
                        break;
                    case "F1":
                        gStatus = "Failed";
                        emailConfig = "BatchFailedBroadcast";
                        break;
                    case "F":
                        gStatus = "Failed";
                        emailConfig = "BatchFailed";
                        break;
                }

                if (gStatusCode != "N")
                {
                    //Get email configuration
                    EmailConfigure configure = dataService.GetEmailConfigure(emailConfig);
                    configure.subject = configure.subject
                                            .Replace("{Status}", gStatus)
                                            .Replace("{PrgName}", gPrgName);

                    //Send email with attachments
                    EmailSender sender = new EmailSender();
                    sender.Email(configure);
                }

                return Ok(gStatusCode);

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "CheckBatch");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.BadRequest));
            }
        }


        [HttpPost, Route("DRG/ReadFileProcessPath")]
        [Authorize]
        public IHttpActionResult DRG_ReadFileProcessPath([FromBody] dynamic pData)
        {
            OracleCommand cmd;

            try
            {
                if (DataService.GetIdentity(User, Cons.SS_USER_NAME) == "LL_RunDragon")
                {
                    //Clear temp
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNPJ006_D0_CLEAR_TEMP";
                    oOracledb.SqlQueryReturnValue(cmd);

                    string processPath = pData.ProcessPath.ToString();
                    string line;
                    string[] fileEntries = Directory.GetFiles(processPath);
                    foreach (string filePath in fileEntries)
                    {
                        Console.WriteLine(filePath);
                        StreamReader reader = new StreamReader(filePath);
                        string[] paths = filePath.Split('\\');
                        string gFileName = paths[paths.Length - 1];
                        while ((line = reader.ReadLine()) != null)
                        {
                            cmd = new OracleCommand();
                            cmd.CommandText = "SP_NP_SNPJ006_D0_SNPP033";
                            cmd.Parameters.Add("P_DATA", OracleDbType.Varchar2).Value = line;
                            cmd.Parameters.Add("P_FNAME", OracleDbType.Varchar2).Value = gFileName;
                            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                            cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                            oOracledb.SqlQueryReturnValue(cmd);
                        }
                        reader.Close();
                    }

                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNPJ006_D0_BACKUP_DATA";
                    cmd.Parameters.Add("P_FILE_NAME", OracleDbType.Varchar2).Value = (Object)DBNull.Value;
                    cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                    DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                    DataTable dt = ds.Tables[0];
                    DataRow dr = dt.Rows[0];

                    return Ok(dr["MESSAGE"]);
                }

                throw new Exception("Invalid");
            }
            catch (Exception ex)
            {
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.BadRequest));
            }
        }

        [HttpPost, Route("DRG/StartProcess")]
        [Authorize]
        public IHttpActionResult DRG_StartProcess([FromBody] dynamic pData)
        {
            try
            {
                if (DataService.GetIdentity(User, Cons.SS_USER_NAME) == "LL_RunDragon")
                {
                    DataSet ds = new DataSet();
                    string returnMessage;
                    string status;
                    try
                    {
                        OracleCommand cmd = new OracleCommand();
                        cmd.CommandText = "SP_NP_SNPJ006_D1_START_JOB";
                        cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                        cmd.Parameters.Add("p_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                        cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                        ds = oOracledb.SqlQueryDS_CMD(cmd);
                        DataTable dtProcessStatus = ds.Tables[0];

                        returnMessage = dtProcessStatus.Rows[0]["MESSAGE"].ToString();
                        status = dtProcessStatus.Rows[0]["STATUS"].ToString();

                        if (status == "Fail")
                        {
                            return Json(new { result = "Fail", message = returnMessage });
                        }
                        else if (status == "Done" || status == "CALL_SNPJ006_D2")
                        {
                            return Json(new { result = "Done", message = returnMessage });
                        }
                    }
                    catch (OracleException ex)
                    {
                        if (ex.Number == 20001)
                        {
                            return Json(new { result = "Fail", message = oOracledb.GetErrorMessage(ex) });
                        }
                    }
                    catch (Exception ex)
                    {
                        oOracledb.WriteLog("ErrorHandle", ex.Message, "DRG_BeginProcess");
                        return Json(new { result = "Fail", message = ex.Message });
                    }

                    return Json(new { result = "Fail", message = "Something wrong in process." });
                }

                throw new Exception("Invalid");
            }
            catch (Exception ex)
            {
                return Json(new { result = "Fail", message = ex.Message });
            }
        }

        [HttpGet, Route("GetProcessLogs/{programName}/{procDate}")]
        [Authorize]
        public IHttpActionResult GetProcessLogs(string programName, string procDate)
        {
            try
            {
                List<string> messageList;
                String DateFormat = DateTime.Now.ToString("yyyyMMddhHHmmss");
                List<EmailAttachment> attachments = new List<EmailAttachment>();

                SystemApiController sysapi = new SystemApiController();
                DataTable dtOracle = sysapi.GetProcessLogs(programName, Convert.ToDateTime(DateTime.ParseExact(procDate, "ddMMyyyy", CultureInfo.InvariantCulture)));
                messageList = new List<string>();
                messageList.Add($"<LOG_DATE_TIME>: (<LOG_TYPE>) <MESSAGE> [<PROGRAM_NAME>]");
                foreach (DataRow dr in dtOracle.Rows)
                {
                    messageList.Add($"{dr["LOG_DATE_TIME"]}: ({dr["LOG_TYPE"]}) {dr["MESSAGE"]} [{dr["PROGRAM_NAME"]}]");
                }

                string message = string.Join("\r\n", messageList);

                return Ok(message);

            }
            catch (Exception ex)
            {
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.BadRequest));
            }
        }

        [HttpGet, Route("RunStoredProcedure/{spName}")]
        [Authorize]
        public IHttpActionResult RunStoredProcedure(string spName)
        {

            try
            {
                if (DataService.GetIdentity(User, Cons.SS_USER_NAME) != "LL_RunSP")
                {
                    throw new Exception("Invalid");
                }

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = spName;
                oOracledb.SqlExecute_CMD(cmd);

                return Ok();

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "RunStoredProcedure");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.BadRequest));
            }
        }

        /*
        [HttpPost, Route("PrintInvoices")]
        public async Task<IHttpActionResult> PrintInvoice([FromBody] dynamic pParams)
        {
            try
            {
                string printType = "INV";

                if (!AutoPrintPreProcess(printType))
                {
                    return Ok();
                }

                string reportPrefix = pParams.ReportPrefix.ToString();
                string reportSuffix = pParams.ReportSuffix.ToString();

                string dateFormat = DateTime.Now.ToString("yyyyMMddHHmmss");
                string reportType = "PDF";
                string fileName = String.Concat(reportPrefix, dateFormat, reportSuffix);
                string processPath = String.Concat(ConfigurationManager.AppSettings["DefaultReportOutputPath"], "PrintInvoices", "/");

                string reportName = "PMS_RPT_AUTO_PRINT_INVOICE";
                List<ParameterValue> reportParams = new List<ParameterValue>();
                reportParams.Add(new ParameterValue { Label = "P_COMPANY_CODE", Name = "P_COMPANY_CODE", Value = "01" });

                ReportOptions options = new ReportOptions();
                options.dpiX = pParams.ReportDPIX;
                options.dpiY = pParams.ReportDPIY;
                //Render reports
                ReportImageResult reportResult = await ReportServices.RenderReportAsImage(reportName, reportParams, options);
                await ReportServices.GenerateReport(reportName, reportType, reportParams, fileName, processPath);

                AutoPrintPostProcess(printType);

                //Page setting
                string printerName = pParams.PrinterName.ToString();
                int PaperWidth = Convert.ToInt32(pParams.Width.ToString());
                int PaperHeight = Convert.ToInt32(pParams.Height.ToString());

                await PrintProcess(processPath, fileName, reportResult, printerName, PaperWidth, PaperHeight);

                return Ok();
            }
            catch (Exception ex)
            {
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.BadRequest, ex.Message));
            }
        }

        [HttpPost, Route("PrintInvoicesBR")]
        public async Task<IHttpActionResult> PrintInvoiceBR([FromBody] dynamic pParams)
        {
            try
            {
                string printType = "INV_BR";

                if (!AutoPrintPreProcess(printType))
                {
                    return Ok();
                }

                string reportPrefix = pParams.ReportPrefix.ToString();
                string reportSuffix = pParams.ReportSuffix.ToString();

                string dateFormat = DateTime.Now.ToString("yyyyMMddHHmmss");
                string reportType = "PDF";
                string fileName = String.Concat(reportPrefix, dateFormat, reportSuffix);
                string processPath = String.Concat(ConfigurationManager.AppSettings["DefaultReportOutputPath"], "PrintInvoicesBR", "/");

                string reportName = "PMS_RPT_AUTO_PRINT_INVOICE_BR";
                List<ParameterValue> reportParams = new List<ParameterValue>();
                reportParams.Add(new ParameterValue { Label = "P_COMPANY_CODE", Name = "P_COMPANY_CODE", Value = "01" });

                //Render reports
                ReportImageResult reportResult = await ReportServices.RenderReportAsImage(reportName, reportParams);
                await ReportServices.GenerateReport(reportName, reportType, reportParams, fileName, processPath);

                AutoPrintPostProcess(printType);

                //Page setting
                string printerName = pParams.PrinterName.ToString();
                int PaperWidth = Convert.ToInt32(pParams.Width.ToString());
                int PaperHeight = Convert.ToInt32(pParams.Height.ToString());

                await PrintProcess(processPath, fileName, reportResult, printerName, PaperWidth, PaperHeight);

                return Ok();
            }
            catch (Exception ex)
            {
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.BadRequest, ex.Message));
            }
        }
        */


        [HttpPost, Route("PrintInvoices")]
        public async Task<HttpResponseMessage> PrintInvoice([FromBody] dynamic pParams)
        {
            try
            {
                string printType = "INV";

                if (!AutoPrintPreProcess(printType))
                {
                    return new HttpResponseMessage(HttpStatusCode.OK); 
                }

                string reportPrefix = pParams.ReportPrefix.ToString();
                string reportSuffix = pParams.ReportSuffix.ToString();

                string dateFormat = DateTime.Now.ToString("yyyyMMddHHmmss");
                string reportType = "PDF";
                string fileName = String.Concat(reportPrefix, dateFormat, reportSuffix);
                string processPath = String.Concat(ConfigurationManager.AppSettings["DefaultReportOutputPath"], "PrintInvoices", "/");

                string reportName = "PMS_RPT_AUTO_PRINT_INVOICE";
                List<ParameterValue> reportParams = new List<ParameterValue>();
                reportParams.Add(new ParameterValue { Label = "P_COMPANY_CODE", Name = "P_COMPANY_CODE", Value = "01" });

                byte[] bytes = await ReportServices.GenerateReportAsBytes(reportName, reportType, reportParams, fileName, processPath);
                Stream stream = new MemoryStream(bytes);

                AutoPrintPostProcess(printType);


                HttpResponseMessage response = new HttpResponseMessage(HttpStatusCode.OK);
                response.Content = new StreamContent(stream);
                response.Content.Headers.ContentDisposition = new System.Net.Http.Headers.ContentDispositionHeaderValue("attachment");
                response.Content.Headers.ContentDisposition.FileName = fileName;
                response.Content.Headers.ContentType = new MediaTypeHeaderValue("application/pdf");

                return response;
            }
            catch (Exception ex)
            {
                return new HttpResponseMessage(HttpStatusCode.BadRequest);
            }
        }

        [HttpPost, Route("RePrintRO")]
        public async Task<HttpResponseMessage> RePrintRO([FromBody] dynamic pParams)
        {
            try
            {
                string printType = "RO";

                if (!AutoPrintPreProcess(printType))
                {
                    return new HttpResponseMessage(HttpStatusCode.OK);
                }

                string reportPrefix = pParams.ReportPrefix.ToString();
                string reportSuffix = pParams.ReportSuffix.ToString();

                string dateFormat = DateTime.Now.ToString("yyyyMMddHHmmss");
                string reportType = "PDF";
                string fileName = String.Concat(reportPrefix, dateFormat, reportSuffix);
                string processPath = String.Concat(ConfigurationManager.AppSettings["DefaultReportOutputPath"], "RePrintRO", "/");

                string reportName = "PMS_RPT_SALES_SNSE039R_3_A4_Auto";
                List<ParameterValue> reportParams = new List<ParameterValue>();
                reportParams.Add(new ParameterValue { Label = "P_COMPANY_CODE", Name = "P_COMPANY_CODE", Value = "01" });

                byte[] bytes = await ReportServices.GenerateReportAsBytes(reportName, reportType, reportParams, fileName, processPath);
                Stream stream = new MemoryStream(bytes);

                AutoPrintPostProcess(printType);

                HttpResponseMessage response = new HttpResponseMessage(HttpStatusCode.OK);
                response.Content = new StreamContent(stream);
                response.Content.Headers.ContentDisposition = new System.Net.Http.Headers.ContentDispositionHeaderValue("attachment");
                response.Content.Headers.ContentDisposition.FileName = fileName;
                response.Content.Headers.ContentType = new MediaTypeHeaderValue("application/pdf");

                return response;
            }
            catch (Exception ex)
            {
                return new HttpResponseMessage(HttpStatusCode.BadRequest);
            }
        }

        [HttpPost, Route("RePrintRO2")]
        public async Task<HttpResponseMessage> RePrintRO2([FromBody] dynamic pParams)
        {
            try
            {
                string printType = "RO2";

                if (!AutoPrintPreProcess(printType))
                {
                    return new HttpResponseMessage(HttpStatusCode.OK);
                }

                string reportPrefix = pParams.ReportPrefix.ToString();
                string reportSuffix = pParams.ReportSuffix.ToString();

                string dateFormat = DateTime.Now.ToString("yyyyMMddHHmmss");
                string reportType = "PDF";
                string fileName = String.Concat(reportPrefix, dateFormat, reportSuffix);
                string processPath = String.Concat(ConfigurationManager.AppSettings["DefaultReportOutputPath"], "RePrintRO2", "/");

                string reportName = "PMS_RPT_SALES_SNSE039R_P_A4_Auto";
                List<ParameterValue> reportParams = new List<ParameterValue>();
                reportParams.Add(new ParameterValue { Label = "P_COMPANY_CODE", Name = "P_COMPANY_CODE", Value = "01" });

                byte[] bytes = await ReportServices.GenerateReportAsBytes(reportName, reportType, reportParams, fileName, processPath);
                Stream stream = new MemoryStream(bytes);

                AutoPrintPostProcess(printType);

                HttpResponseMessage response = new HttpResponseMessage(HttpStatusCode.OK);
                response.Content = new StreamContent(stream);
                response.Content.Headers.ContentDisposition = new System.Net.Http.Headers.ContentDispositionHeaderValue("attachment");
                response.Content.Headers.ContentDisposition.FileName = fileName;
                response.Content.Headers.ContentType = new MediaTypeHeaderValue("application/pdf");

                return response;
            }
            catch (Exception ex)
            {
                return new HttpResponseMessage(HttpStatusCode.BadRequest);
            }
        }

        [HttpPost, Route("PrintDN/{documentType}/{reportFormat}")]
        public async Task<HttpResponseMessage> PrintDN([FromBody] dynamic pParams, string documentType, string reportFormat)
        {
            try
            {
                string printType = "DN";

                if (reportFormat == "NMT")
                    printType = "DNN";
                if (reportFormat == "CUSTOMER")
                    printType = "DNC";

                if (!AutoPrintPreProcess(printType))
                {
                    return new HttpResponseMessage(HttpStatusCode.OK);
                }

                string reportPrefix = pParams.ReportPrefix.ToString();
                string reportSuffix = pParams.ReportSuffix.ToString();

                string dateFormat = DateTime.Now.ToString("yyyyMMddHHmmss");
                string reportType = "PDF";
                string fileName = String.Concat(reportPrefix, dateFormat, reportSuffix);
                string processPath = String.Concat(ConfigurationManager.AppSettings["DefaultReportOutputPath"], "PrintDN", "/", reportFormat, "/");

                string reportName = "PMS_RPT_SALES_SNSE063R_A4_Auto";
                List<ParameterValue> reportParams = new List<ParameterValue>();
                reportParams.Add(new ParameterValue { Label = "P_COMPANY_CODE", Name = "P_COMPANY_CODE", Value = "01" });
                reportParams.Add(new ParameterValue { Label = "P_DOCUMENT_TYPE", Name = "P_DOCUMENT_TYPE", Value = documentType });
                reportParams.Add(new ParameterValue { Label = "P_PRINT_FORMAT", Name = "P_PRINT_FORMAT", Value = reportFormat });

                byte[] bytes = await ReportServices.GenerateReportAsBytes(reportName, reportType, reportParams, fileName, processPath);
                Stream stream = new MemoryStream(bytes);

                AutoPrintPostProcess(printType);

                HttpResponseMessage response = new HttpResponseMessage(HttpStatusCode.OK);
                response.Content = new StreamContent(stream);
                response.Content.Headers.ContentDisposition = new System.Net.Http.Headers.ContentDispositionHeaderValue("attachment");
                response.Content.Headers.ContentDisposition.FileName = fileName;
                response.Content.Headers.ContentType = new MediaTypeHeaderValue("application/pdf");

                return response;
            }
            catch (Exception ex)
            {
                return new HttpResponseMessage(HttpStatusCode.BadRequest);
            }
        }

        [HttpPost, Route("PrintInvoicesBR")]
        public async Task<HttpResponseMessage> PrintInvoiceBR([FromBody] dynamic pParams)
        {
            try
            {
                string printType = "INV_BR";

                if (!AutoPrintPreProcess(printType))
                {
                    return new HttpResponseMessage(HttpStatusCode.OK); 
                }

                string reportPrefix = pParams.ReportPrefix.ToString();
                string reportSuffix = pParams.ReportSuffix.ToString();

                string dateFormat = DateTime.Now.ToString("yyyyMMddHHmmss");
                string reportType = "PDF";
                string fileName = String.Concat(reportPrefix, dateFormat, reportSuffix);
                string processPath = String.Concat(ConfigurationManager.AppSettings["DefaultReportOutputPath"], "PrintInvoicesBR", "/");

                string reportName = "PMS_RPT_AUTO_PRINT_INVOICE_BR";
                List<ParameterValue> reportParams = new List<ParameterValue>();
                reportParams.Add(new ParameterValue { Label = "P_COMPANY_CODE", Name = "P_COMPANY_CODE", Value = "01" });

                byte[] bytes = await ReportServices.GenerateReportAsBytes(reportName, reportType, reportParams, fileName, processPath);
                Stream stream = new MemoryStream(bytes);

                AutoPrintPostProcess(printType);


                HttpResponseMessage response = new HttpResponseMessage(HttpStatusCode.OK);
                response.Content = new StreamContent(stream);
                response.Content.Headers.ContentDisposition = new System.Net.Http.Headers.ContentDispositionHeaderValue("attachment");
                response.Content.Headers.ContentDisposition.FileName = fileName;
                response.Content.Headers.ContentType = new MediaTypeHeaderValue("application/pdf");

                return response;
            }
            catch (Exception ex)
            {
                return new HttpResponseMessage(HttpStatusCode.BadRequest);
            }
        }

        [HttpPost, Route("PrintPicking/{printType}")]
        public async Task<HttpResponseMessage> PrintPicking([FromBody] dynamic pParams, string printType)
        {
            try
            {
                if (!AutoPrintPreProcess(printType))
                {
                    return new HttpResponseMessage(HttpStatusCode.OK); 
                    //return Ok();
                }

                string reportPrefix = pParams.ReportPrefix.ToString();
                string reportSuffix = pParams.ReportSuffix.ToString();

                string dateFormat = DateTime.Now.ToString("yyyyMMddHHmmss");
                string reportType = "PDF";
                string fileName = String.Concat(reportPrefix, dateFormat, reportSuffix);
                string processPath = String.Concat(ConfigurationManager.AppSettings["DefaultReportOutputPath"], "PrintPicking", "/");

                string reportName = "PMS_RPT_AUTO_PRINT_PICKING";
                List<ParameterValue> reportParams = new List<ParameterValue>();
                reportParams.Add(new ParameterValue { Label = "P_COMPANY_CODE", Name = "P_COMPANY_CODE", Value = "01" });
                reportParams.Add(new ParameterValue { Label = "P_TYPE", Name = "P_TYPE", Value = printType });

                //Render reports
                //ReportImageResult reportResult = await ReportServices.RenderReportAsImage(reportName, reportParams);

                byte[] bytes = await ReportServices.GenerateReportAsBytes(reportName, reportType, reportParams, fileName, processPath);
                Stream stream = new MemoryStream(bytes);

                AutoPrintPostProcess(printType);


                HttpResponseMessage response = new HttpResponseMessage(HttpStatusCode.OK);
                response.Content = new StreamContent(stream);
                response.Content.Headers.ContentDisposition = new System.Net.Http.Headers.ContentDispositionHeaderValue("attachment");
                response.Content.Headers.ContentDisposition.FileName = fileName;
                response.Content.Headers.ContentType = new MediaTypeHeaderValue("application/pdf");

                return response;
            }
            catch (Exception ex)
            {
                return new HttpResponseMessage(HttpStatusCode.BadRequest);
                //return ResponseMessage(Request.CreateResponse(HttpStatusCode.BadRequest, ex.Message));
            }
        }

        [HttpPost, Route("PrintPickingBR")]
        public async Task<HttpResponseMessage> PrintPickingBR([FromBody] dynamic pParams)
        {
            try
            {
                string printType = "PICK_BR";

                if (!AutoPrintPreProcess(printType))
                {
                    return new HttpResponseMessage(HttpStatusCode.OK);
                    //return Ok();
                }

                string reportPrefix = pParams.ReportPrefix.ToString();
                string reportSuffix = pParams.ReportSuffix.ToString();

                string dateFormat = DateTime.Now.ToString("yyyyMMddHHmmss");
                string reportType = "PDF";
                string fileName = String.Concat(reportPrefix, dateFormat, reportSuffix);
                string processPath = String.Concat(ConfigurationManager.AppSettings["DefaultReportOutputPath"], "PrintPickingBR", "/");

                string reportName = "PMS_RPT_AUTO_PRINT_PICKING_BR";
                List<ParameterValue> reportParams = new List<ParameterValue>();
                reportParams.Add(new ParameterValue { Label = "P_COMPANY_CODE", Name = "P_COMPANY_CODE", Value = "01" });

                //Render reports
                //ReportImageResult reportResult = await ReportServices.RenderReportAsImage(reportName, reportParams);

                byte[] bytes = await ReportServices.GenerateReportAsBytes(reportName, reportType, reportParams, fileName, processPath);
                Stream stream = new MemoryStream(bytes);

                AutoPrintPostProcess(printType);


                HttpResponseMessage response = new HttpResponseMessage(HttpStatusCode.OK);
                response.Content = new StreamContent(stream);
                response.Content.Headers.ContentDisposition = new System.Net.Http.Headers.ContentDispositionHeaderValue("attachment");
                response.Content.Headers.ContentDisposition.FileName = fileName;
                response.Content.Headers.ContentType = new MediaTypeHeaderValue("application/pdf");

                return response;
            }
            catch (Exception ex)
            {
                return new HttpResponseMessage(HttpStatusCode.BadRequest);
                //return ResponseMessage(Request.CreateResponse(HttpStatusCode.BadRequest, ex.Message));
            }
        }

        [HttpPost, Route("PrintInvoicesDrop")]
        public async Task<HttpResponseMessage> PrintInvoicesDrop([FromBody] dynamic pParams)
        {
            try
            {

                string printType = "DROP";

                if (!AutoPrintPreProcess(printType))
                {
                    return new HttpResponseMessage(HttpStatusCode.OK);
                }

                string reportPrefix = pParams.ReportPrefix.ToString();
                string reportSuffix = pParams.ReportSuffix.ToString();

                string dateFormat = DateTime.Now.ToString("yyyyMMddHHmmss");
                string reportType = "PDF";
                string fileName = String.Concat(reportPrefix, dateFormat, reportSuffix);
                string processPath = String.Concat(ConfigurationManager.AppSettings["DefaultReportOutputPath"], "PrintInvoicesDrop", "/");

                string reportName = "PMS_RPT_ACC_SNPE019R2_AUTO_PRINT";
                List<ParameterValue> reportParams = new List<ParameterValue>();
                reportParams.Add(new ParameterValue { Label = "P_COMPANY_CODE", Name = "P_COMPANY_CODE", Value = "01" });

                byte[] bytes = await ReportServices.GenerateReportAsBytes(reportName, reportType, reportParams, fileName, processPath);
                Stream stream = new MemoryStream(bytes);

                HttpResponseMessage response = new HttpResponseMessage(HttpStatusCode.OK);
                response.Content = new StreamContent(stream);
                response.Content.Headers.ContentDisposition = new System.Net.Http.Headers.ContentDispositionHeaderValue("attachment");
                response.Content.Headers.ContentDisposition.FileName = fileName;
                response.Content.Headers.ContentType = new MediaTypeHeaderValue("application/pdf");

                return response;
            }
            catch (Exception ex)
            {
                return new HttpResponseMessage(HttpStatusCode.BadRequest);
            }
        }
        private bool AutoPrintPreProcess(string printType)
        {

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_AUTO_PRINT_PRE";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = "01";
            cmd.Parameters.Add("P_PRINT_TYPE", OracleDbType.Varchar2).Value = printType;
            cmd.Parameters.Add("P_PRINT_RECORD", OracleDbType.Int32).Direction = ParameterDirection.Output;

            cmd = oOracledb.SqlQueryReturnValue(cmd);
            int returnRecords;

            if (Int32.TryParse(cmd.Parameters["P_PRINT_RECORD"].Value.ToString(), out returnRecords))
            {
                return returnRecords > 0;
            }

            return false;

        }

        private void AutoPrintPostProcess(string printType)
        {

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_AUTO_PRINT_POST";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = "01";
            cmd.Parameters.Add("P_PRINT_TYPE", OracleDbType.Varchar2).Value = printType;

            oOracledb.SqlQueryReturnValue(cmd);

        }

        private async Task<bool> PrintProcess(string processPath, string fileName, ReportImageResult reportResult, string printerName, int paperWidth, int paperHeight)
        {
            string processLocation = HttpContext.Current.Server.MapPath(processPath);
            string printLocation = Path.Combine(processLocation, fileName);
            string backUpLocation = Path.Combine(processLocation, "./backup");

            PrintService printService = new PrintService();
            //if(await printService.CreatePrintRecord(printLocation, printerName))
            if (printService.PrintReport(reportResult, printerName, paperWidth, paperHeight))
            {
                if (!Directory.Exists(backUpLocation))
                    Directory.CreateDirectory(backUpLocation);

                int attempt = 0;
                while (attempt >= 0 && attempt < 3)
                {
                    try
                    {
                        File.Move(printLocation, Path.Combine(backUpLocation, fileName));
                        attempt = -1;
                    }
                    catch (Exception ex)
                    {
                        attempt++;
                        System.Threading.Thread.Sleep(15000);
                    }
                }
            }

            return true;
        }


        [HttpGet, Route("GenerateReport_Aging")]
        [Authorize]
        public async Task<IHttpActionResult> GenerateReport_Aging()
        {
            OracleCommand cmd = new OracleCommand();
            DateTime currentDate = DateTime.Now;
            DateTime reportDate = new DateTime(currentDate.Year, currentDate.Month, 1).AddDays(-1);
            try
            {
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_GEN_AGING_REPORT";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = "01";
                cmd.Parameters.Add("P_PROCESS_YEAR", OracleDbType.Varchar2).Value = reportDate.Year.ToString();
                cmd.Parameters.Add("P_PROCESS_MONTH", OracleDbType.Varchar2).Value = reportDate.Month.ToString().PadLeft(2, '0');
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 20).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;
                oOracledb.SqlQueryReturnValue(cmd);
                string status_generate = cmd.Parameters["R_STATUS"].Value.ToString();
                string message_generate = cmd.Parameters["R_MESSAGE"].Value.ToString();
                if (status_generate != "DONE")
                {
                    oOracledb.WriteLog("FAIL", message_generate, "SP_NP_GEN_AGING_REPORT");
                    //return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, "Gen Aging Report Error"));
                }
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("FAIL", ex.Message, "SP_NP_GEN_AGING_REPORT");
                //return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

            string MonthYear = reportDate.ToString("MMMM_yyyy");
            string AgingXLSXFileName = "Aging_Report_" + MonthYear + ".xlsx";
            string AgingPDFFileName = "Aging_Report_" + MonthYear + ".pdf";
            string Top20FileName = "Top_20_Aging_Rpt_" + MonthYear + ".pdf";
            //Get querystring
            //var queryStrings = Request.GetQueryNameValuePairs();

            var reportParams = new List<ParameterValue>();
            //Set report type EXCELOPENXML or PDF
            var fileTypePDF = "PDF";
            var fileTypeXLSX = "EXCELOPENXML";

            //Set output location & filename
            //var outputLocation = Path.Combine(ConfigurationManager.AppSettings["RootSharedPath"], "tempFile", "Aging_Report");
            var outputLocation = "~\\tempFile\\Aging_Report\\" + reportDate.ToString("yyyyMM") + "\\";

            //Create report parameters
            //foreach (var qs in queryStrings)

            reportParams.Add(new ParameterValue
            {
                Name = "P_COMPANY_CODE",
                Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE),
                Label = "P_COMPANY_CODE"
            });

            try
            {
                //Generate report
                await ReportServices.GenerateReport("PMS_RPT_CR_AGING_REPORT", fileTypeXLSX, reportParams, AgingXLSXFileName, outputLocation);
                await ReportServices.GenerateReport("PMS_RPT_CR_AGING_REPORT", fileTypePDF, reportParams, AgingPDFFileName, outputLocation);
                await ReportServices.GenerateReport("PMS_RPT_CR_AGING_REPORT_TOP20", fileTypePDF, reportParams, Top20FileName, outputLocation);

                //Get email configuration
                EmailConfigure configure = dataService.GetEmailConfigure("SendAgingReport");
                configure.subject = configure.subject.Replace("{ReportMonth}", reportDate.ToString("MMMM yyyy"));
                configure.content = configure.content
                                        .Replace("{ReportMonth}", reportDate.ToString("MMMM yyyy"))
                                        .Replace("{OutputLocation}", Path.Combine(ConfigurationManager.AppSettings["RootSharedPath"], outputLocation.Replace("~\\", "")));

                //Send email with attachments
                EmailSender sender = new EmailSender();
                sender.Email(configure);

                //oOracledb.WriteLog("PASS","PASS", "GenerateReport_Aging");
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("FAIL", ex.Message, "GenerateReport_Aging");
                //return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
            return Ok(outputLocation);
        }

    }
}
