using System;
using System.ComponentModel;
using System.Configuration;
using System.IO;
using System.Net;
using System.Net.Mail;
using System.Threading.Tasks;

namespace SYSCore.aControl
{
    public class EmailSender
    {
        public EmailSender()
        {
        }

        static bool mailSent = false;

        public void Email(EmailConfigure emailConfigure)
        {
            string smtpServer;
            int smtpPort;
            string senderEmail;
            string credentialUsername = null;
            string credentialPassword = null;

            if (ConfigurationManager.AppSettings["Environment"].ToUpper() == "PRODUCTION")
            {
                smtpServer = "InetGW.mail.nissan.biz";
                smtpPort = 25;
                senderEmail = "\"New PMS\" <noreply-npms@nissan.co.th>";
            }
            else if (ConfigurationManager.AppSettings["Environment"].ToUpper() == "DEVELOPMENT")
            {
                smtpServer = "smtp.office365.com";
                smtpPort = 587;
                senderEmail = "\"New PMS DEV\" <sutthiwat@smartgosd.com>";
                credentialUsername = "sutthiwat@smartgosd.com";
                credentialPassword = "Tumm@1234";
            }
            else
            {
                smtpServer = "";
                smtpPort = 25;
                senderEmail = "";
                return;
            }

            if (emailConfigure.to.Length == 0)
            {
                throw new Exception("No email configuration.");
            }

            //Configure SMTP server and credential
            SmtpClient smtp = new SmtpClient(smtpServer, smtpPort);
            if(credentialUsername != null)
            {
                smtp.Credentials = new NetworkCredential(credentialUsername, credentialPassword);
                smtp.EnableSsl = true;
            }

            //Compose email
            MailMessage mailMessage = new MailMessage();
            mailMessage.From = new MailAddress(senderEmail);

            foreach (var t in emailConfigure.to)
            {
                var emailAddress = t.Trim();
                if (emailAddress != "" && isValidEmailAddress(emailAddress))
                {
                    mailMessage.To.Add(emailAddress);
                }
            }

            foreach (var c in emailConfigure.cc)
            {
                var emailAddress = c.Trim();
                if (emailAddress != "" && isValidEmailAddress(emailAddress))
                {
                    mailMessage.CC.Add(emailAddress);
                }
            }

            foreach (var b in emailConfigure.bcc)
            {
                var emailAddress = b.Trim();
                if (emailAddress != "" && isValidEmailAddress(emailAddress))
                {
                    mailMessage.Bcc.Add(emailAddress);
                }
            }

            mailMessage.Subject = emailConfigure.subject;
            mailMessage.Body = emailConfigure.content;
            mailMessage.IsBodyHtml = true;

            if (emailConfigure.attachments != null) {
                foreach(var a in emailConfigure.attachments)
                {
                    if(a.contentBytes.Length > 0)
                    {
                        Attachment attachment = new Attachment(new MemoryStream(a.contentBytes), a.fileName, a.mediaType);
                        mailMessage.Attachments.Add(attachment);
                    }
                }
            }

            //Set high importance email
            if (emailConfigure.highImportant)
            {
                mailMessage.Priority = MailPriority.High;
            }

            if (mailMessage.To.Count > 0)
            {
                smtp.Send(mailMessage);
            }

            mailMessage.Dispose();

        }

        private static bool isValidEmailAddress(string emailAddress)
        {
            try
            {
                var email = new MailAddress(emailAddress);
                return true;
            }
            catch
            {
                return false;
            }
        }

        public class EmailAttachment
        {

            public EmailAttachment(byte[] _bytes, string _fileName, string _mediaType)
            {
                contentBytes = _bytes;
                fileName = _fileName;
                mediaType = _mediaType;
            }

            public byte[] contentBytes { get; set; }
            public string fileName { get; set; }
            public string mediaType { get; set; }
        }

        public class EmailConfigure
        {
            public EmailConfigure(string[] _to, string[] _cc, string[] _bcc, string _subject, string _content, EmailAttachment[] _attachments, bool _highImportant)
            {
                to = _to;

                cc = _cc;
                if (_cc == null)
                    cc = Array.Empty<string>();

                bcc = _bcc;
                if (_bcc == null)
                    bcc = Array.Empty<string>();

                subject = _subject;
                content = _content;
                attachments = _attachments;
                highImportant = _highImportant;
            }

            public string[] to { get; set; }
            public string[] cc { get; set; }
            public string[] bcc { get; set; }
            public string subject { get; set; }
            public string content { get; set; }
            public bool highImportant { get; set; }
            public EmailAttachment[] attachments { get; set; }
        }
    }
}
