﻿using SYS_Core.aControl;
using System;
using System.Web.Http;
using System.Web.Http.Cors;
using Oracle.ManagedDataAccess.Client;
using System.Data;
using System.Net.Http;
using System.Net;
using SYS_Core.aBiz;
using System.Threading.Tasks;
using System.Configuration;
using SYS_Core.Controllers;
using System.Collections;
using System.Web;
using System.IO;
using System.Linq;
using SYSCore.Attributes;
using Newtonsoft.Json;
using System.Collections.Generic;
using ClosedXML.Excel;
using System.Net.Http.Headers;
using RandomSolutions;
//using DocumentFormat.OpenXml.Drawing.Charts;

namespace SYSCore.Controllers
{
    [EnableCors("*", "*", "*")]
    [RoutePrefix("api/K9")]
    public class K9Controller : ApiController
    {

        private DBOracle oOracledb = new DBOracle();
      
        private DataModel DataModel = new DataModel();


        #region SNPP209

        [HttpGet, Route("SNPP209_GetVin/{vin}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNPP209_GetVin(string vin, int noofshowrec)
        {
            DataSet ds = new DataSet();

            vin = ismoUtil.escapeCharacter(vin);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPP209_GET_VIN";
                cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VIN", OracleDbType.Varchar2).Value = vin;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPP209_GetVin");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex));
            }
        }

        [HttpGet, Route("SNPP209_GetEndItem18/{enditem}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNPP209_GetEndItem18(string enditem, int noofshowrec)
        {
            DataSet ds = new DataSet();

            enditem = ismoUtil.escapeCharacter(enditem);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPP209_GET_END_ITEM_18";
                cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_END_ITEM_18", OracleDbType.Varchar2).Value = enditem;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPP209_GetEndItem18");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex));
            }
        }


        [HttpPost, Route("SNPP209_GetData")]
        [Authorize]
        public IHttpActionResult SNPP209_GetData([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPP209_GET_DATA";
                cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VIN", OracleDbType.Varchar2).Value = param.VIN;
                cmd.Parameters.Add("P_END_ITEM_18", OracleDbType.Varchar2).Value = param.END_ITEM_18;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPP209_GetData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex));
            }
        }

        [HttpPost, Route("SNPP209_Process")]
        [Authorize]
        public IHttpActionResult SNPP209_Process(List<SNPP209_CONFIRM> result)
        {
   

            DataSet ds = new DataSet();
            var json = JsonConvert.SerializeObject(result);
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPP209_PROCESS";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DATA", OracleDbType.Varchar2).Value = json;
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                oOracledb.SqlQueryReturnValue(cmd);
                string R_STATUS = cmd.Parameters["R_STATUS"].Value.ToString();

                return Json(new { STATUS = R_STATUS });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPP209_Process");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex));
            }
        }

        public class SNPP209_CONFIRM
        {
            public string END_ITEM_18 { get; set; }
        }


        #endregion SNPP209

        #region SNPP208

        [HttpPost, Route("SNPP208_UploadExcel")]
        [Authorize]
        public async Task<IHttpActionResult> SNPP208_UploadExcel(HttpRequestMessage request)
        {
            if (!request.Content.IsMimeMultipartContent())
            {
                oOracledb.WriteLog("ErrorHandle", "UnsupportedMediaType", "SNPP208_UploadExcel");
                throw new HttpResponseException(HttpStatusCode.UnsupportedMediaType);
            }
            var reqParts = await request.Content.ReadAsMultipartAsync();
            try
            {
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();
                string USER_ID = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                var file = reqParts.Contents[0];
                var buffer = await file.ReadAsByteArrayAsync();
                string gFileName = file.Headers.ContentDisposition.FileName.Trim('\"');
                string name = Path.GetFileNameWithoutExtension(gFileName);
                string ext = Path.GetExtension(gFileName);
                string gFileNameExtend = String.Concat(name, '_', USER_ID, ext);
                string gRootPath = ConfigurationManager.AppSettings["RootSharedPath"].ToString();
                string gUploadLication = System.IO.Path.Combine(gRootPath, "tempFile/SNPP208");
                string gFileLocation = System.IO.Path.Combine(gUploadLication, gFileName);
                DataService.SaveBytesToFileByLocation(buffer
                                                , gUploadLication
                                                , gFileLocation);
                SystemApiController sys = new SystemApiController();
                dt = SNPP208_Validate(gFileLocation);
                return Json(DTFM.convertToList(dt));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPP208_UploadExcel");
                throw new HttpResponseException(HttpStatusCode.NotAcceptable);
            }
            throw new Exception("Something wrong");
        }

        public DataTable SNPP208_Validate(string filelocation)
        {

            DataTable dt = new DataTable();
            SystemApiController sys = new SystemApiController();
            dt = sys.ReadDataExcel(filelocation);
            dt.Columns.Add("SEQ");
            dt.Columns.Add("MESSAGE");
            dt.Columns.Add("ERR_FLAG");
            int iCount = 0;
            Hashtable hTable = new Hashtable();
            OracleCommand cmd = new OracleCommand();
            DataColumnCollection columns = dt.Columns;
            foreach (DataRow d in dt.Rows)
            {
                if (!string.IsNullOrEmpty(d["VIN"].ToString()))
                {
                    iCount++;
                    try
                    {

                        cmd = new OracleCommand();
                        cmd.CommandText = "SP_NP_SNPP208_VALIDATE";
                        cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                        cmd.Parameters.Add("P_VIN", OracleDbType.Varchar2).Value = d["VIN"];
                        cmd.Parameters.Add("R_ERROR_MESSAGE", OracleDbType.Varchar2, 500).Direction = ParameterDirection.Output;
                        cmd.Parameters.Add("R_ERROR_FLAG", OracleDbType.Varchar2, 1).Direction = ParameterDirection.Output;

                        cmd = oOracledb.SqlQueryReturnValue(cmd);
                        string MSG = cmd.Parameters["R_ERROR_MESSAGE"].Value.ToString() == "null" ? "" : cmd.Parameters["R_ERROR_MESSAGE"].Value.ToString();
                        string ERROR_FLAG = cmd.Parameters["R_ERROR_FLAG"].Value.ToString();
                        
                        d["SEQ"] = iCount;
                        d["MESSAGE"] = MSG;
                        d["ERR_FLAG"] = ERROR_FLAG;

                        if (hTable.Contains(d["VIN"].ToString().Trim()))
                        {
                            d["SEQ"] = iCount;
                            d["MESSAGE"] = "Duplicate Part No at Seq " + hTable[d["VIN"]]; ;
                            d["ERR_FLAG"] = "Y";
                        }
                        else
                        {
                            hTable.Add(d["VIN"].ToString().Trim(), iCount);
                        }


                    }
                    catch (Exception ex)
                    {
                        //--- Roll back ---
                        oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPP208_Validate");
                        throw new HttpResponseException(HttpStatusCode.NotAcceptable);
                    }
                }
            }
            return dt;
        }

        [HttpPost, Route("SNPP208_Process")]
        [Authorize]
        public IHttpActionResult SNPP208_Process(List<SNPP208_ALLOCATE> param)
        {

            List<SNPP208_ALLOCATE> result = param.Where(x => x.ERR_FLAG == "N").ToList();
            string PROCESS_ID = DateTime.Now.ToString("yyyyMMdd-HHmmss-fff");
           
            DataSet ds = new DataSet();
            var json = JsonConvert.SerializeObject(result);
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPP208_PROCESS";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DATA", OracleDbType.Varchar2).Value = json;
                cmd.Parameters.Add("P_PROCESS_ID", OracleDbType.Varchar2).Value = PROCESS_ID;
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                oOracledb.SqlQueryReturnValue(cmd);
                string R_STATUS = cmd.Parameters["R_STATUS"].Value.ToString();

                return Json(new { STATUS = R_STATUS });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPP208_Process");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex));
            }
        }

        public class SNPP208_ALLOCATE
        {
            public string VIN { get; set; }
            public string OPTION_CODE { get; set; }
            public string ERR_FLAG { get; set; }
        }

        #endregion SNPP208

        #region SNPP207

        [HttpGet, Route("SNPP207_GetVin/{vin}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNPP207_GetVin(string vin, int noofshowrec)
        {
            DataSet ds = new DataSet();

            vin = ismoUtil.escapeCharacter(vin);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPP207_GET_VIN";
                cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VIN", OracleDbType.Varchar2).Value = vin;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPP207_GetVin");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex));
            }
        }

        [HttpPost, Route("SNPP207_GetEndItem18/{enditem}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNPP207_GetEndItem18(string enditem, int noofshowrec,[FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            enditem = ismoUtil.escapeCharacter(enditem);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPP207_GET_END_ITEM_18";
                cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VIN", OracleDbType.Varchar2).Value = param.VIN;
                cmd.Parameters.Add("P_END_ITEM_18", OracleDbType.Varchar2).Value = enditem;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPP207_GetEndItem18");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex));
            }
        }

        [HttpPost, Route("SNPP207_GetDestinationCode/{des}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNPP207_GetDestinationCode(string des, int noofshowrec, [FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            des = ismoUtil.escapeCharacter(des);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPP207_GET_DESTINATION_CODE";
                cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_END_ITEM_18", OracleDbType.Varchar2).Value = param.END_ITEM_18;
                cmd.Parameters.Add("P_DESTINATION_CODE", OracleDbType.Varchar2).Value = des != "null" ? des : "";
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPP207_GetDestinationCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex));
            }
        }

        [HttpPost, Route("SNPP207_GetData")]
        [Authorize]
        public IHttpActionResult SNPP207_GetData([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPP207_GET_DATA";
                cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VIN", OracleDbType.Varchar2).Value = param.VIN;
                cmd.Parameters.Add("P_END_ITEM_18", OracleDbType.Varchar2).Value = param.END_ITEM_18;
                cmd.Parameters.Add("P_DESTINATION_CODE", OracleDbType.Varchar2).Value = param.DESTINATION_CODE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPP207_GetData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex));
            }
        }

        [HttpPost, Route("SNPP207_Allocate")]
        [Authorize]
        public IHttpActionResult SNPP207_Allocate(List<SNPP207_ALLOCATE> param)
        {
            DataSet ds = new DataSet();
            var json = JsonConvert.SerializeObject(param);
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPP207_ALLOCATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DATA", OracleDbType.Varchar2).Value = json;
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                oOracledb.SqlQueryReturnValue(cmd);
                string R_STATUS = cmd.Parameters["R_STATUS"].Value.ToString();

                return Json(new { STATUS = R_STATUS });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPP207_Allocate");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex));
            }
        }


        public class SNPP207_ALLOCATE
        {
            public string VIN { get; set; }
            public string ITEM_NO { get; set; }
        }

        #endregion SNPP207

        #region SNM161

        [HttpGet, Route("CheckSapBillTo/{sapbillto}")]
        [Authorize]
        public IHttpActionResult CheckSapBillTo(string sapbillto)
        {
            DataSet ds = new DataSet();
            
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM161_CHECK_SAP_BILL_TO";
                cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_SAP_BILL_TO", OracleDbType.Varchar2).Value = sapbillto;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "CheckSapBillTo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex));
            }
        }

        [HttpPost, Route("SNM161_GetBranch/{param}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM161_GetBranch(string param, int noofshowrec,[FromBody] dynamic fparam)
        {
            DataSet ds = new DataSet();
            param = ismoUtil.escapeCharacter(param);
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM161_GET_BRANCH";
                cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = fparam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = param.ToUpper() == "NULL" ? (object)DBNull.Value : param;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM161_GetBranch");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex));
            }
        }

        [HttpPost, Route("SNM161_GetData")]
        [Authorize]
        public IHttpActionResult SNM161_GetData([FromBody] dynamic fparam)
        {
            DataSet ds = new DataSet();

            try
            {


                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM161_GET_DATA";
                cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = fparam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = fparam.BRANCH_CODE;
                cmd.Parameters.Add("P_SAP_SHIP_TO", OracleDbType.Varchar2).Value = fparam.SAP_SHIP_TO;
                cmd.Parameters.Add("P_SAP_BILL_TO", OracleDbType.Varchar2).Value = fparam.SAP_BILL_TO;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM161_GetData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex));
            }
        }

        [HttpPost, Route("SNM161_GetDataEdit")]
        [Authorize]
        public IHttpActionResult SNM161_GetDataEdit([FromBody] dynamic fparam)
        {
            DataSet ds = new DataSet();

            try
            {


                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM161_GET_DATA_EDIT";
                cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_SAP_SHIP_TO", OracleDbType.Varchar2).Value = fparam.SAP_SHIP_TO;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM161_GetData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex));
            }
        }

        [HttpPost, Route("SNM161_Insert")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM161_Insert([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM161_INSERT";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = param.PMS_CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRAANCH_CODE", OracleDbType.Varchar2).Value = param.PMS_BRANCH_CODE;
                cmd.Parameters.Add("P_SAP_SHIP_TO", OracleDbType.Varchar2).Value = param.SAP_SHIP_TO;
                cmd.Parameters.Add("P_SAP_BILL_TO", OracleDbType.Varchar2).Value = param.SAP_BILL_TO;
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string R_STATUS = cmd.Parameters["R_STATUS"].Value.ToString();
                string R_MESSAGE = cmd.Parameters["R_MESSAGE"].Value.ToString();

                if (R_STATUS == "FAIL")
                {
                    oOracledb.WriteLog("ErrorHandle", R_MESSAGE, "SNM161_Insert");
                }

                return Json(new { STATUS_MSG = R_STATUS, });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM161_Insert");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNM161_Update")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM161_Update([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM161_UPDATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = param.PMS_CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRAANCH_CODE", OracleDbType.Varchar2).Value = param.PSM_BRANCH_CODE;
                cmd.Parameters.Add("P_SAP_SHIP_TO", OracleDbType.Varchar2).Value = param.SAP_SHIP_TO;
                cmd.Parameters.Add("P_SAP_BILL_TO", OracleDbType.Varchar2).Value = param.SAP_BILL_TO;
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string R_STATUS = cmd.Parameters["R_STATUS"].Value.ToString();
                string R_MESSAGE = cmd.Parameters["R_MESSAGE"].Value.ToString();

                if (R_STATUS == "FAIL")
                {
                    oOracledb.WriteLog("ErrorHandle", R_MESSAGE, "SNM161_Update");
                }

                return Json(new { STATUS_MSG = R_STATUS, });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM161_Update");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }


        [HttpPost, Route("SNM161_Delete")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM130P_Delete([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM161_DELETE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_SAP_SHIP_TO", OracleDbType.Varchar2).Value = param.SAP_SHIP_TO;
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string R_STATUS = cmd.Parameters["R_STATUS"].Value.ToString();
                string R_MESSAGE = cmd.Parameters["R_MESSAGE"].Value.ToString();

                if (R_STATUS == "FAIL")
                {
                    oOracledb.WriteLog("ErrorHandle", R_MESSAGE, "SNM161_Delete");
                }

                return Json(new { STATUS_MSG = R_STATUS, });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM161_Delete");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        #endregion

        #region SNPE051

        [HttpGet, Route("SNPE051_GetPart/{PartNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNPE051_GetPart(string PartNo, string noofshowrec)
        {
            try
            {
                DataSet ds = new DataSet();
                PartNo = ismoUtil.escapeCharacter(PartNo);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE051_GET_PART";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = PartNo != "NULL" ? PartNo : (object)DBNull.Value;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE051_GetPart");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
                // return Json(new { Status = "Error" });

            }
        }


        [HttpPost, Route("SNPE051_GetDataForReceive")]
        [Authorize]
        public IHttpActionResult SNPE051_GetDataForReceive([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNPE051_GET_FOR_RECEIVE";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_DOCUMENT_NO", OracleDbType.Varchar2).Value = param.DOCUMENT_NO;
            cmd.Parameters.Add("P_DATE_FROM", OracleDbType.Date).Value = param.DATE_FROM;
            cmd.Parameters.Add("P_DATE_TO", OracleDbType.Date).Value = param.DATE_TO;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNPE051_GetDetailReceive")]
        [Authorize]
        public IHttpActionResult SNPE051_GetDetailReceive([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNPE051_GET_DETAIL_RECEIVE";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_DOCUMENT_NO", OracleDbType.Varchar2).Value = param.DOCUMENT_NO;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNPE051_ConfirmReceive")]
        [Authorize]
        public IHttpActionResult SNPE051_ConfirmReceive([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();
            try
            {
                
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE051_CONFIRM_RECEIVE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DOCUMENT_NO", OracleDbType.Varchar2).Value = param.DOCUMENT_NO;
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                oOracledb.SqlQueryReturnValue(cmd);
                string R_STATUS = cmd.Parameters["R_STATUS"].Value.ToString();

                return Json(new { STATUS = R_STATUS });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE051_ConfirmReceive");
                return Json(new { STATUS = "FAIL" });
            }

        }


        [HttpPost, Route("SNPE051_GetDataForProcess")]
        [Authorize]
        public IHttpActionResult SNPE051_GetDataForProcess([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNPE051_GET_FOR_PROCESS";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_DOCUMENT_NO", OracleDbType.Varchar2).Value = param.DOCUMENT_NO;
            cmd.Parameters.Add("P_DOCUMENT_DATE", OracleDbType.Date).Value = param.DOCUMENT_DATE;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNPE051_GetDetailProcess")]
        [Authorize]
        public IHttpActionResult SNPE051_GetDetailProcess([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNPE051_GET_DETAIL_PROCESS";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_DOCUMENT_NO", OracleDbType.Varchar2).Value = param.DOCUMENT_NO;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNPE051_ConfirmProcess")]
        [Authorize]
        public IHttpActionResult SNPE051_ConfirmProcess([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();
            try
            {

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE051_CONFIRM_PROCESS";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DOCUMENT_NO", OracleDbType.Varchar2).Value = param.DOCUMENT_NO;
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                oOracledb.SqlQueryReturnValue(cmd);
                string R_STATUS = cmd.Parameters["R_STATUS"].Value.ToString();

                return Json(new { STATUS = R_STATUS });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE051_ConfirmProcess");
                return Json(new { STATUS = "FAIL" });
            }

        }



        [HttpPost, Route("SNPE051_GetDataForInvoice")]
        [Authorize]
        public IHttpActionResult SNPE051_GetDataForInvoice([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNPE051_GET_FOR_INVOICE";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_DOCUMENT_NO", OracleDbType.Varchar2).Value = param.DOCUMENT_NO;
            cmd.Parameters.Add("P_DOCUMENT_DATE", OracleDbType.Date).Value = param.DOCUMENT_DATE;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNPE051_GetDetailInvoice")]
        [Authorize]
        public IHttpActionResult SNPE051_GetDetailInvoice([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNPE051_GET_DETAIL_INVOICE";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_DOCUMENT_NO", OracleDbType.Varchar2).Value = param.DOCUMENT_NO;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNPE051_ConfirmInvoice")]
        [Authorize]
        public IHttpActionResult SNPE051_ConfirmInvoice([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();
            try
            {

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE051_CONFIRM_INVOICE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DOCUMENT_NO", OracleDbType.Varchar2).Value = param.DOCUMENT_NO;
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                oOracledb.SqlQueryReturnValue(cmd);
                string R_STATUS = cmd.Parameters["R_STATUS"].Value.ToString();

                return Json(new { STATUS = R_STATUS });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE051_ConfirmInvoice");
                return Json(new { STATUS = "FAIL" });
            }

        }

        [HttpPost, Route("SNPE051_GetDataForCancel")]
        [Authorize]
        public IHttpActionResult SNPE051_GetDataForCancel([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNPE051_GET_FOR_CANCEL";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_DOCUMENT_NO", OracleDbType.Varchar2).Value = param.DOCUMENT_NO;
            cmd.Parameters.Add("P_DOCUMENT_DATE", OracleDbType.Date).Value = param.DOCUMENT_DATE;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNPE051_GetDetailCancel")]
        [Authorize]
        public IHttpActionResult SNPE051_GetDetailCancel([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNPE051_GET_DETAIL_CANCEL";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_DOCUMENT_NO", OracleDbType.Varchar2).Value = param.DOCUMENT_NO;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNPE051_ConfirmCancel")]
        [Authorize]
        public IHttpActionResult SNPE051_ConfirmCancel([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();
            try
            {

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE051_CONFIRM_CANCEL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DOCUMENT_NO", OracleDbType.Varchar2).Value = param.DOCUMENT_NO;
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                oOracledb.SqlQueryReturnValue(cmd);
                string R_STATUS = cmd.Parameters["R_STATUS"].Value.ToString();

                return Json(new { STATUS = R_STATUS });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE051_ConfirmCancel");
                return Json(new { STATUS = "FAIL" });
            }

        }

        [HttpGet, Route("SNPE051_Getdoc/{docno}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNPE051_Getdoc(string docno, int noofshowrec)
        {
            DataSet ds = new DataSet();

            docno = ismoUtil.escapeCharacter(docno);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNPE051_GET_DOC";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_DOCUMENT_NO", OracleDbType.Varchar2).Value = docno;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNPE051_UploadFileAssemble")]
        [Authorize]
        public async Task<IHttpActionResult> SNPE051_UploadFileAssemble(HttpRequestMessage request)
        {
            string company_code = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            string userId = DataService.GetIdentity(User, Cons.SS_USER_NAME);


            if (!request.Content.IsMimeMultipartContent())
            {
                throw new HttpResponseException(HttpStatusCode.UnsupportedMediaType);
            }
            var reqUpload = await request.Content.ReadAsMultipartAsync();
            try
            {
                string sqlDelete = String.Format("DELETE TPM_NP_SNPE051_UPLOAD_ASSEMBLE where company_code = '{0}' and create_by = '{1}'", company_code, userId);
                int row = oOracledb.SqlExecute(sqlDelete);

                DataSet ds = new DataSet();
                DataTable dt = new DataTable();
                string sql = @"SELECT   COMPANY_CODE,
                                        VIN     ,
                                        OPTION_CODE ,
                                        CREATE_BY
                                FROM TPM_NP_SNPE051_UPLOAD_ASSEMBLE";
                string SPPS_CODE = string.Empty;
                var file = reqUpload.Contents[0];
                var buffer = await file.ReadAsByteArrayAsync();
                using (MemoryStream stream = new MemoryStream(buffer))
                {
                    //dt.Columns.Add("COMPANY_CODE")
                    SystemApiController sys = new SystemApiController();
                    dt = sys.ReadDataExcel(stream);
                    dt.TableName = "Table";

                    DataTable newDt = new DataTable();
                    DataView dtv = new DataView();
                    dtv = dt.AsDataView();

                    newDt = dtv.ToTable();
                    DataColumn newColumn = new DataColumn();
                    newColumn = new DataColumn("CREATE_BY", typeof(System.String));
                    newColumn.DefaultValue = userId;
                    newDt.Columns.Add(newColumn);
                    newColumn = new DataColumn("COMPANY_CODE", typeof(System.String));
                    newColumn.DefaultValue = company_code;
                    newDt.Columns.Add(newColumn);
                    int rowCount = oOracledb.InsertDataTable(newDt, sql);
                }

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE051_VALIDATE_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE051_UploadFileAssemble");
                throw new HttpResponseException(HttpStatusCode.NotAcceptable);
            }
            throw new Exception("Something wrong");
        }


        [HttpPost, Route("SNPE051_UploadFileDisAssemble")]
        [Authorize]
        public async Task<IHttpActionResult> SNPE051_UploadFileDisAssemble(HttpRequestMessage request)
        {
            string company_code = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            string userId = DataService.GetIdentity(User, Cons.SS_USER_NAME);


            if (!request.Content.IsMimeMultipartContent())
            {
                throw new HttpResponseException(HttpStatusCode.UnsupportedMediaType);
            }
            var reqUpload = await request.Content.ReadAsMultipartAsync();
            try
            {
                string sqlDelete = String.Format("DELETE TPM_NP_SNPE051_UPLOAD_DISASSEMBLE where company_code = '{0}' and create_by = '{1}'", company_code, userId);
                int row = oOracledb.SqlExecute(sqlDelete);

                DataSet ds = new DataSet();
                DataTable dt = new DataTable();
                string sql = @"SELECT   COMPANY_CODE,
                                        VIN     ,
                                        OPTION_CODE ,
                                        CREATE_BY
                                FROM TPM_NP_SNPE051_UPLOAD_DISASSEMBLE";
                string SPPS_CODE = string.Empty;
                var file = reqUpload.Contents[0];
                var buffer = await file.ReadAsByteArrayAsync();
                using (MemoryStream stream = new MemoryStream(buffer))
                {
                    //dt.Columns.Add("COMPANY_CODE")
                    SystemApiController sys = new SystemApiController();
                    dt = sys.ReadDataExcel(stream);
                    dt.TableName = "Table";

                    DataTable newDt = new DataTable();
                    DataView dtv = new DataView();
                    dtv = dt.AsDataView();

                    newDt = dtv.ToTable();
                    DataColumn newColumn = new DataColumn();
                    newColumn = new DataColumn("CREATE_BY", typeof(System.String));
                    newColumn.DefaultValue = userId;
                    newDt.Columns.Add(newColumn);
                    newColumn = new DataColumn("COMPANY_CODE", typeof(System.String));
                    newColumn.DefaultValue = company_code;
                    newDt.Columns.Add(newColumn);
                    int rowCount = oOracledb.InsertDataTable(newDt, sql);
                }

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPE051_VALIDATE_DATA_DISASSEMBLE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPE051_UploadFileDisAssemble");
                throw new HttpResponseException(HttpStatusCode.NotAcceptable);
            }
            throw new Exception("Something wrong");
        }

        [HttpGet, Route("SNPE051_ProcessAssemble")]
        [Authorize]
        public IHttpActionResult SNPE051_ProcessAssemble()
        {
            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNPE051_PROCESS_ASSEMBLE";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
            cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
            oOracledb.SqlQueryReturnValue(cmd);
            string R_STATUS = cmd.Parameters["R_STATUS"].Value.ToString();

            return Json(new { STATUS = R_STATUS });
        }

        [HttpPost, Route("SNPE051_GetDataDisassemble")]
        [Authorize]
        public IHttpActionResult SNPE051_GetDataDisassemble([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNPE051_GET_TRAN_STOCK";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_VIN", OracleDbType.Varchar2).Value = param.VIN;
            cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNPE051_ProcessDisAssemble")]
        [Authorize]
        public IHttpActionResult SNPE051_ProcessDisAssemble(List<SNPE051_DISASSEMBLE> param)
        {
            DataSet ds = new DataSet();
            var json = JsonConvert.SerializeObject(param);
            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNPE051_PROCESS_DISASSEMBLE";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
            cmd.Parameters.Add("P_DATA", OracleDbType.Varchar2).Value = json;
            cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
            oOracledb.SqlQueryReturnValue(cmd);
            string R_STATUS = cmd.Parameters["R_STATUS"].Value.ToString();

            return Json(new { STATUS = R_STATUS });
        }

        public class SNPE051_DISASSEMBLE
        {
            public string VIN { get; set; }
        }
        #endregion
        #region SNM117P

        [HttpGet, Route("SNM117P_GetPDILocation/{locationCode?}")]
        [Authorize]
        public IHttpActionResult SNM117P_GetPDILocation(string locationCode = null)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();

            try
            {

                cmd.CommandText = "SP_NP_SNM117P_PDI_LOCATION";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_LOCATION_CODE", OracleDbType.Varchar2).Value = locationCode;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

            }
            catch (Exception ex)
            {
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex));
            }

            return Json(DTFM.convertToList(ds));

        }

        [HttpPost, Route("SNM117P_SavePDILocation/{locationCode?}")]
        [Authorize]
        public IHttpActionResult SNM117P_SavePDILocation([FromBody] dynamic pLocation, string locationCode = null)
        {
            OracleCommand cmd = new OracleCommand();

            try
            {
                if (locationCode == null)
                {
                    cmd.CommandText = "SP_NP_SNM117P_PDI_LOCAT_ADD";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_PDI_LOCATION_CODE", OracleDbType.Varchar2).Value = pLocation["PDI_LOCATION_CODE"];
                    cmd.Parameters.Add("P_PDI_LOCATION_DESC", OracleDbType.Varchar2).Value = pLocation["PDI_LOCATION_DESC"];
                    cmd.Parameters.Add("P_VENDOR_CODE_MAPPING", OracleDbType.Varchar2).Value = pLocation["VENDOR_CODE_MAPPING"];
                    cmd.Parameters.Add("P_REASON_CODE_MAPPING", OracleDbType.Decimal).Value = pLocation["REASON_CODE_MAPPING"];
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                }
                else
                {
                    cmd.CommandText = "SP_NP_SNM117P_PDI_LOCAT_SAVE";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_PDI_LOCATION_CODE", OracleDbType.Varchar2).Value = locationCode;
                    cmd.Parameters.Add("P_PDI_LOCATION_DESC", OracleDbType.Varchar2).Value = pLocation["PDI_LOCATION_DESC"];
                    cmd.Parameters.Add("P_VENDOR_CODE_MAPPING", OracleDbType.Varchar2).Value = pLocation["VENDOR_CODE_MAPPING"];
                    cmd.Parameters.Add("P_REASON_CODE_MAPPING", OracleDbType.Decimal).Value = pLocation["REASON_CODE_MAPPING"];
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                }

                oOracledb.SqlQueryReturnValue(cmd);

            }
            catch (Exception ex)
            {
                var result = ex.Message.Split('\n');
                var message = result[0].IndexOf("ORA-20001") == 0 ? result[0].Substring(11, result[0].Length - 11) : "Found error, please contact administrator to solve this issue.";
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, message));
            }

            return Ok();
        }


        [HttpPost, Route("SNM117P_DeletePDILocation/{locationCode}")]
        [Authorize]
        public IHttpActionResult SNM117P_DeletePDILocation(string locationCode)
        {
            OracleCommand cmd = new OracleCommand();

            try
            {

                cmd.CommandText = "SP_NP_SNM117P_PDI_LOCATION_DEL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PDI_LOCATION_CODE", OracleDbType.Varchar2).Value = locationCode;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                oOracledb.SqlQueryReturnValue(cmd);

            }
            catch (Exception ex)
            {
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex));
            }

            return Ok();
        }

        #endregion

        #region SNM118P

        [HttpGet, Route("SNM118P_GetModel/{endItem?}")]
        [Authorize]
        public IHttpActionResult SNM118P_GetModel(string endItem = null)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();

            try
            {

                cmd.CommandText = "SP_NP_SNM118P_MODEL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_END_ITEM_18", OracleDbType.Varchar2).Value = endItem;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

            }
            catch (Exception ex)
            {
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex));
            }

            return Json(DTFM.convertToList(ds));

        }

        [HttpPost, Route("SNM118P_SaveModel/{endItem?}")]
        [Authorize]
        public IHttpActionResult SNM118P_SaveModel([FromBody] dynamic pModel, string endItem = null)
        {
            OracleCommand cmd = new OracleCommand();

            try
            {
                if (endItem == null)
                {
                    cmd.CommandText = "SP_NP_SNM118P_MODEL_ADD";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_MAIN_MODEL", OracleDbType.Varchar2).Value = pModel["MAIN_MODEL"];
                    cmd.Parameters.Add("P_END_ITEM_18", OracleDbType.Varchar2).Value = pModel["END_ITEM_18"];
                    cmd.Parameters.Add("P_END_ITEM_DESC", OracleDbType.Varchar2).Value = pModel["END_ITEM_DESC"];
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                }
                else
                {
                    cmd.CommandText = "SP_NP_SNM118P_MODEL_SAVE";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_MAIN_MODEL", OracleDbType.Varchar2).Value = pModel["MAIN_MODEL"];
                    cmd.Parameters.Add("P_END_ITEM_18", OracleDbType.Varchar2).Value = endItem;
                    cmd.Parameters.Add("P_END_ITEM_DESC", OracleDbType.Varchar2).Value = pModel["END_ITEM_DESC"];
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                }

                oOracledb.SqlQueryReturnValue(cmd);

            }
            catch (Exception ex)
            {
                var result = ex.Message.Split('\n');
                var message = result[0].IndexOf("ORA-20001") == 0 ? result[0].Substring(11, result[0].Length - 11) : "Found error, please contact administrator to solve this issue.";
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, message));
            }

            return Ok();
        }


        [HttpPost, Route("SNM118P_DeleteModel/{endItem}")]
        [Authorize]
        public IHttpActionResult SNM118P_DeleteModel(string endItem)
        {
            OracleCommand cmd = new OracleCommand();

            try
            {

                cmd.CommandText = "SP_NP_SNM118P_MODEL_DEL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_END_ITEM_18", OracleDbType.Varchar2).Value = endItem;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                oOracledb.SqlQueryReturnValue(cmd);

            }
            catch (Exception ex)
            {
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex));
            }

            return Ok();
        }

        [HttpPost, Route("SNM118P_UploadFile")]
        [Authorize]
        public async Task<IHttpActionResult> SNM132P_UploadFile(HttpRequestMessage request)
        {
            if (!request.Content.IsMimeMultipartContent())
            {
                throw new HttpResponseException(HttpStatusCode.UnsupportedMediaType);
            }
            var reqParts = await request.Content.ReadAsMultipartAsync();

            try
            {
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();
                var file = reqParts.Contents[0];
                var buffer = await file.ReadAsByteArrayAsync();
                string gFileName = file.Headers.ContentDisposition.FileName.Trim('\"');
                string gRootPath = ConfigurationManager.AppSettings["RootSharedPath"].ToString();
                string gUploadLication = System.IO.Path.Combine(gRootPath, "tempFile/SNM118P");
                string gFileLocation = System.IO.Path.Combine(gUploadLication, gFileName);

                DataService.SaveBytesToFileByLocation(buffer
                                                , gUploadLication
                                                , gFileLocation);

                dt = SNM118P_ValidateData(gFileLocation);
                //ds.Tables.Add(dt);

                return Json(DTFM.convertToList(dt));

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM118P_UploadFile");
                throw new HttpResponseException(HttpStatusCode.NotAcceptable);
            }

            throw new Exception("Something wrong");

        }

        public DataTable SNM118P_ValidateData(string filelocation)
        {
            DataTable dt = new DataTable();
            SystemApiController sys = new SystemApiController();
            Hashtable hTable = new Hashtable();
            dt = sys.ReadDataExcel(filelocation);
        
            dt.Columns.Add("END_ITEM_18");
            dt.Columns.Add("DESCRIPTION");
            dt.Columns.Add("ERR_MSG");
            dt.Columns.Add("STATUS");

            int countrow = 1;
            OracleCommand cmd = new OracleCommand();
            foreach (DataRow row in dt.Rows)
            {

                if(!string.IsNullOrEmpty(row["End Item 18 digits"].ToString()))
                {
                    if (hTable.Contains(row["End Item 18 digits"].ToString().Trim()))
                    {
                        row["END_ITEM_18"] = row["End Item 18 digits"].ToString().Trim();
                        row["DESCRIPTION"] = row["Description"].ToString().Trim();
                        row["ERR_MSG"] = "Duplicate End Item 18 digits at Seq " + hTable[row["End Item 18 digits"]];
                        row["STATUS"] = "ERROR";
                    }
                    else
                    {
                        hTable.Add(row["End Item 18 digits"].ToString().Trim(), countrow);
                        row["END_ITEM_18"] = row["End Item 18 digits"].ToString().Trim();
                        row["DESCRIPTION"] = row["Description"].ToString().Trim();
                        

                        if (row["End Item 18 digits"].ToString().Trim().Length > 18)
                        {
                            row["ERR_MSG"] = "End Item 18 digits cannot be greater than 18 digits";
                            row["STATUS"] = "ERROR";
                        }
                        else if (row["End Item 18 digits"].ToString().Trim().Length < 18)
                        {
                            row["ERR_MSG"] = "End Item 18 digits cannot less than 18 digits";
                            row["STATUS"] = "ERROR";
                        }
                        else
                        {
                            cmd = new OracleCommand();
                            cmd.CommandText = "SP_NP_SNM118P_CHECK_DUP";
                            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                            cmd.Parameters.Add("P_END_ITEM_18", OracleDbType.Varchar2).Value = row["End Item 18 digits"].ToString().Trim();
                            cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;
                            cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;

                            cmd = oOracledb.SqlQueryReturnValue(cmd);

                            string R_STATUS = cmd.Parameters["R_STATUS"].Value.ToString();
                            string R_MESSAGE = cmd.Parameters["R_MESSAGE"].Value.ToString();


                            row["STATUS"] = R_STATUS ;
                            row["ERR_MSG"] = R_MESSAGE == "null" ? string.Empty : R_MESSAGE;


                        }
                    }
                }

                countrow++;
                
            }

            return dt;
        }

        [HttpPost, Route("SNM118P_SaveUpload")]
        [Authorize]
        public IHttpActionResult SNM118P_SaveUpload([FromBody] dynamic[] param)
        {
            DataSet ds = new DataSet();

            try
            {
                int iCount = 0;
                OracleCommand cmd = new OracleCommand();

                foreach (var detail in param)
                {
                    iCount++;

                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNM118P_SAVE_TEMP";
                    cmd.Parameters.Add("P_END_ITEM_18", OracleDbType.Varchar2).Value = detail["END_ITEM_18"];
                    cmd.Parameters.Add("P_END_ITEM_DESC", OracleDbType.Varchar2).Value = detail["DESCRIPTION"];
                    cmd.Parameters.Add("P_CLEAR_DATA", OracleDbType.Varchar2).Value = iCount == 1 ? "Y" : "N";
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                    oOracledb.SqlQueryReturnValue(cmd);

                }

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM118P_INSERT_UPLOAD";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string R_STATUS = cmd.Parameters["R_STATUS"].Value.ToString();
                string R_MESSAGE = cmd.Parameters["R_MESSAGE"].Value.ToString();

                if (R_STATUS == "FAIL")
                {
                    oOracledb.WriteLog("ErrorHandle", R_MESSAGE, "SNM118P_SaveUpload");
                }

                return Json(new { STATUS_MSG = R_STATUS, });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM118P_SaveUpload");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        #endregion

        #region SNM119P
        [HttpGet, Route("SNM119P_Search/{assemblyPartNo?}")]
        [Authorize]
        public IHttpActionResult SNM119P_Search(string assemblyPartNo = null)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd;
            try
            {
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM119P_BOM";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ASSEMBLY_PART_NO", OracleDbType.Varchar2).Value = assemblyPartNo;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
            } catch(Exception ex)
            {
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

            return Json(DTFM.convertToList(ds));
        }
        [HttpGet, Route("SNM119P_GetDetail/{assemblyPartNo}")]
        [Authorize]
        public IHttpActionResult SNM119P_GetDetail(string assemblyPartNo)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd;
            try
            {
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM119P_BOM_DETAIL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ASSEMBLY_PART_NO", OracleDbType.Varchar2).Value = assemblyPartNo;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
            }
            catch (Exception ex)
            {
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNM119P_Add")]
        [Authorize]
        public IHttpActionResult SNM119P_Add([FromBody] dynamic pBOMDetail)
        {
            OracleCommand cmd;
            try
            {
                int iCount = 0;
                foreach (var bomDetail in pBOMDetail.Detail)
                {
                        iCount++;
                        cmd = new OracleCommand();
                        cmd.CommandText = "SP_NP_SNM119P_SAVE_TEMP";
                        cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                        cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Int16).Value = iCount;
                        cmd.Parameters.Add("P_MATERIAL_PART_NO", OracleDbType.Varchar2).Value = bomDetail.MATERIAL_PART_NO;
                        cmd.Parameters.Add("P_MATERIAL_QTY", OracleDbType.Varchar2).Value = bomDetail.MATERIAL_QTY;
                        cmd.Parameters.Add("P_EXT_COLOR", OracleDbType.Varchar2).Value = bomDetail.EXT_COLOR;
                        cmd.Parameters.Add("P_INT_COLOR", OracleDbType.Varchar2).Value = bomDetail.INT_COLOR;
                        cmd.Parameters.Add("P_SUPER_SESSION_ON", OracleDbType.Varchar2).Value = bomDetail.SUPER_SESSION_ON;
                        cmd.Parameters.Add("P_FLAG", OracleDbType.Varchar2).Value = bomDetail.FLAG;
                        cmd.Parameters.Add("P_CLEAR_DATA", OracleDbType.Varchar2).Value = iCount == 1 ? "Y" : "N";
                        cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                      oOracledb.SqlQueryReturnValue(cmd);
                }

                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNM119P_BOM_ADD";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_INSTALL_PART_NO", OracleDbType.Varchar2).Value = pBOMDetail.INSTALL_PART_NO;
                    cmd.Parameters.Add("P_ASSEMBLY_PART_NO", OracleDbType.Varchar2).Value = pBOMDetail.ASSEMBLY_PART_NO;
                    cmd.Parameters.Add("P_DESCRIPTION", OracleDbType.Varchar2).Value = pBOMDetail.DESCRIPTION;
                    cmd.Parameters.Add("P_SPPS_NO", OracleDbType.Varchar2).Value = pBOMDetail.SPPS_NO;
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                    cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 100).Direction = ParameterDirection.Output;
                    cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;
                    cmd = oOracledb.SqlQueryReturnValue(cmd);

                    string R_STATUS = cmd.Parameters["R_STATUS"].Value.ToString();
                    string R_MESSAGE = cmd.Parameters["R_MESSAGE"].Value.ToString();

                    if (R_STATUS == "FAIL")
                    {
                        oOracledb.WriteLog("ErrorHandle", R_MESSAGE, "SNM119P_Add");
                    }

                    return Json(new { STATUS_MSG = R_STATUS, });

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM119P_Add");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }


      

        [HttpGet, Route("SNM119P_Delete/{installpart}")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM119P_Delete(string installpart)
        {
            OracleCommand cmd;

            try
            {
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM119P_BOM_DELETE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_INSTALL_PART_NO", OracleDbType.Varchar2).Value = installpart;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string R_STATUS = cmd.Parameters["R_STATUS"].Value.ToString();
                string R_MESSAGE = cmd.Parameters["R_MESSAGE"].Value.ToString();

                if (R_STATUS == "FAIL")
                {
                    oOracledb.WriteLog("ErrorHandle", R_MESSAGE, "SNM119P_Update");
                }

                return Json(new { STATUS_MSG = R_STATUS, });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM119P_Delete");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

            return Ok();

        }


        [HttpPost, Route("SNM119P_Update")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM119P_Update([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            try
            {
                int iCount = 0;
                OracleCommand cmd = new OracleCommand();

                foreach (var bomDetail in param.Detail)
                {
                    iCount++;
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNM119P_SAVE_TEMP";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Int16).Value = iCount;
                    cmd.Parameters.Add("P_MATERIAL_PART_NO", OracleDbType.Varchar2).Value = bomDetail.MATERIAL_PART_NO;
                    cmd.Parameters.Add("P_MATERIAL_QTY", OracleDbType.Varchar2).Value = bomDetail.MATERIAL_QTY;
                    cmd.Parameters.Add("P_EXT_COLOR", OracleDbType.Varchar2).Value = bomDetail.EXT_COLOR;
                    cmd.Parameters.Add("P_INT_COLOR", OracleDbType.Varchar2).Value = bomDetail.INT_COLOR;
                    cmd.Parameters.Add("P_SUPER_SESSION_ON", OracleDbType.Varchar2).Value = bomDetail.SUPER_SESSION_ON;
                    cmd.Parameters.Add("P_FLAG", OracleDbType.Varchar2).Value = bomDetail.FLAG;
                    cmd.Parameters.Add("P_CLEAR_DATA", OracleDbType.Varchar2).Value = iCount == 1 ? "Y" : "N";
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                    oOracledb.SqlQueryReturnValue(cmd);
                }

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM119P_UPDATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_INSTALL_PART_NO", OracleDbType.Varchar2).Value = param.INSTALL_PART_NO;
                cmd.Parameters.Add("P_SPPS_NO", OracleDbType.Varchar2).Value = param.SPPS_NO;
                cmd.Parameters.Add("P_DESCRIPTION", OracleDbType.Varchar2).Value = param.DESCRIPTION;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string R_STATUS = cmd.Parameters["R_STATUS"].Value.ToString();
                string R_MESSAGE = cmd.Parameters["R_MESSAGE"].Value.ToString();

                if (R_STATUS == "FAIL")
                {
                    oOracledb.WriteLog("ErrorHandle", R_MESSAGE, "SNM119P_Update");
                }

                return Json(new { STATUS_MSG = R_STATUS, });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM119P_Update");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }


        [HttpGet, Route("SNM119P_GetInstallPart/{part}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM121P_Family_Code(string part, int noofshowrec)
        {
            DataSet ds = new DataSet();

            part = ismoUtil.escapeCharacter(part);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM119P_GET_INSTALL_PART";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = part.ToUpper() == "NULL" ? (object)DBNull.Value : part;
            //cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }

        [HttpGet, Route("SNM119P_GetMaterialPart/{part}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM119P_GetMaterialPart(string part, int noofshowrec)
        {
            DataSet ds = new DataSet();

            part = ismoUtil.escapeCharacter(part);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM119P_GET_MATERIAL_P";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = part.ToUpper() == "NULL" ? (object)DBNull.Value : part;
            //cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }

        [HttpGet, Route("SNM119P_GetAssemblyPart/{part}")]
        [Authorize]
        public IHttpActionResult SNM119P_GetAssemblyPart(string part)
        {
            DataSet ds = new DataSet();

            part = ismoUtil.escapeCharacter(part);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM119P_GET_ASSEM_PART";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = part.ToUpper() == "NULL" ? (object)DBNull.Value : part;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }

        [HttpGet, Route("SNM119P_GetSppsCode/{spps}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM130P_Spps_Code(string spps, int noofshowrec)
        {
            DataSet ds = new DataSet();

            spps = ismoUtil.escapeCharacter(spps);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM119P_GET_SPPS";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_SPPS_CODE", OracleDbType.Varchar2).Value = spps.ToUpper() == "NULL" ? (object)DBNull.Value : spps;
            //cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNM119P_Upload")]
        [Authorize]
        public async Task<IHttpActionResult> SNM119P_Upload(HttpRequestMessage request)
        {
            if (!request.Content.IsMimeMultipartContent())
            {
                throw new HttpResponseException(HttpStatusCode.UnsupportedMediaType);
            }
            var reqParts = await request.Content.ReadAsMultipartAsync();

            try
            {
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();
                string USER_ID = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                var file = reqParts.Contents[0];

                var buffer = await file.ReadAsByteArrayAsync();
                string gFileName = file.Headers.ContentDisposition.FileName.Trim('\"');
                string name = Path.GetFileNameWithoutExtension(gFileName);
                string ext = Path.GetExtension(gFileName);
                string gFileNameExtend = String.Concat(name, '_', USER_ID, ext);
                string gRootPath = ConfigurationManager.AppSettings["RootSharedPath"].ToString();
                string gUploadLication = System.IO.Path.Combine(gRootPath, "tempFile/SNM119P");
                string gFileLocation = System.IO.Path.Combine(gUploadLication, gFileName);

                DataService.SaveBytesToFileByLocation(buffer
                                                , gUploadLication
                                                , gFileLocation);

                SystemApiController sys = new SystemApiController();
                //dt = sys.ReadDataExcel(gFileLocation);
                dt = SNM119P_ValidateData(gFileLocation);
                return Json(DTFM.convertToList(dt));

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM119P_Upload");
                throw new HttpResponseException(HttpStatusCode.NotAcceptable);
            }

            throw new Exception("Something wrong");

        }

        public DataTable SNM119P_ValidateData(string filelocation)
        {
            DataTable dt = new DataTable();
            SystemApiController sys = new SystemApiController();
            dt = sys.ReadDataExcel(filelocation);
            Hashtable hTableH = new Hashtable();
            Hashtable hTableD = new Hashtable();
            dt.Columns.Add("ERROR_MSG");
            dt.Columns.Add("ASSEMBLY_PART_NO");

            foreach (DataRow row in dt.Rows)
            {

                OracleCommand cmd = new OracleCommand();
                if (!string.IsNullOrEmpty(row["INSTALL_PART_NO"].ToString()))
                {

                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNM119P_VALIDATE_PART";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_INSTALL_PART_NO", OracleDbType.Varchar2).Value = row["INSTALL_PART_NO"];
                    cmd.Parameters.Add("P_MATERIAL_PART_NO", OracleDbType.Varchar2).Value = row["MATERIAL_PART_NO"];
                    cmd.Parameters.Add("P_MATERIAL_QTY", OracleDbType.Varchar2).Value = row["MATERIAL_QTY"];
                    cmd.Parameters.Add("P_EXT_COLOR", OracleDbType.Varchar2).Value = row["EXT_COLOR"];
                    cmd.Parameters.Add("P_INT_COLOR", OracleDbType.Varchar2).Value = row["INT_COLOR"];
                    cmd.Parameters.Add("C_RETURN_MSG", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;

                    cmd = oOracledb.SqlQueryReturnValue(cmd);

                    if (cmd.Parameters["C_RETURN_MSG"].Value.ToString() == "null")
                    {
                        row["ERROR_MSG"] = DBNull.Value;
                    }
                    else
                    {
                        row["ERROR_MSG"] = cmd.Parameters["C_RETURN_MSG"].Value.ToString();
                    }

                    row["ASSEMBLY_PART_NO"] = row["INSTALL_PART_NO"].ToString().Replace(row["INSTALL_PART_NO"].ToString().Substring(1, 1), "8");


                }
                else
                {
                    row["ERROR_MSG"] = "Install Part No not found";
                }


            }

            return dt;
        }


        [HttpPost, Route("SNM119P_SaveUpload")]
        [Authorize]
        public IHttpActionResult SNM119P_SaveUpload([FromBody] dynamic[] param)
        {
            DataSet ds = new DataSet();

            try
            {
                int iCount = 0;
                OracleCommand cmd = new OracleCommand();

                foreach (var bomDetail in param)
                {
                    iCount++;
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNM119P_SAVE_TEMP_UPD";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_INSTALL_PART_NO", OracleDbType.Varchar2).Value = bomDetail.INSTALL_PART_NO;
                    cmd.Parameters.Add("P_SPPS_NO", OracleDbType.Varchar2).Value = bomDetail.SPPS_NO;
                    cmd.Parameters.Add("P_DESCRIPTION", OracleDbType.Varchar2).Value = bomDetail.DESCRIPTION;
                    cmd.Parameters.Add("P_MATERIAL_PART_NO", OracleDbType.Varchar2).Value = bomDetail.MATERIAL_PART_NO;
                    cmd.Parameters.Add("P_MATERIAL_QTY", OracleDbType.Int16).Value = bomDetail.MATERIAL_QTY;
                    cmd.Parameters.Add("P_EXT_COLOR", OracleDbType.Varchar2).Value = bomDetail.EXT_COLOR;
                    cmd.Parameters.Add("P_INT_COLOR", OracleDbType.Varchar2).Value = bomDetail.INT_COLOR;
                    cmd.Parameters.Add("P_ASSEMBLY_PART_NO", OracleDbType.Varchar2).Value = bomDetail.ASSEMBLY_PART_NO;
                    cmd.Parameters.Add("P_CLEAR_DATA", OracleDbType.Varchar2).Value = iCount == 1 ? "Y" : "N";
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                    oOracledb.SqlQueryReturnValue(cmd);
                }

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM119P_INSERT_UPLOAD";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string R_STATUS = cmd.Parameters["R_STATUS"].Value.ToString();
                string R_MESSAGE = cmd.Parameters["R_MESSAGE"].Value.ToString();

                if (R_STATUS == "FAIL")
                {
                    oOracledb.WriteLog("ErrorHandle", R_MESSAGE, "SNM119P_Update");
                }

                return Json(new { STATUS_MSG = R_STATUS, });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM119P_Update");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }
        #endregion

        #region SNM120P

        [HttpGet, Route("SNM120P_Search/{endItem?}")]
        [Authorize]
        public IHttpActionResult SNM120P_Search(string endItem = null)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd;
            try
            {
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM120P_INSTALL_HEADER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_END_ITEM_18", OracleDbType.Varchar2).Value = endItem;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
            }
            catch (Exception ex)
            {
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

            return Json(DTFM.convertToList(ds));
        }
        [HttpGet, Route("SNM120P_GetDetail/{bomKey}")]
        [Authorize]
        public IHttpActionResult SNM120P_GetDetail(string bomKey)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd;
            try
            {
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM120P_INSTALL_DETAIL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_BOM_KEY", OracleDbType.Varchar2).Value = bomKey;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
            }
            catch (Exception ex)
            {
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

            return Json(DTFM.convertToList(ds));
        }


        [HttpPost, Route("SNM120P_GetBOMList/{partno}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM120P_GetBOMList(string partno, int noofshowrec, [FromBody] dynamic pColor)
        {
            DataSet ds = new DataSet();

            partno = ismoUtil.escapeCharacter(partno);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM120P_BOM_LIST";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = (partno != null && partno.ToUpper() == "NULL" ? "" : partno);
            cmd.Parameters.Add("P_EXT_COLOR", OracleDbType.Varchar2).Value = pColor["EXT_COLOR"];
            cmd.Parameters.Add("P_INT_COLOR", OracleDbType.Varchar2).Value = pColor["INT_COLOR"];
            cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }


        [HttpPost, Route("SNM120P_Add")]
        [Authorize]
        public IHttpActionResult SNM120P_Add([FromBody] dynamic pInstallDetail)
        {
            OracleCommand cmd;

            try
            {
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM120P_INSTALL_CHECK";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_END_ITEM_18", OracleDbType.Varchar2).Value = pInstallDetail.END_ITEM_18;
                cmd.Parameters.Add("P_EXT_COLOR", OracleDbType.Varchar2).Value = pInstallDetail.EXT_COLOR;
                cmd.Parameters.Add("P_INT_COLOR", OracleDbType.Varchar2).Value = pInstallDetail.INT_COLOR;
                cmd.Parameters.Add("P_PDI_LOCATION_CODE", OracleDbType.Varchar2).Value = pInstallDetail.PDI_LOCATION_CODE;
                cmd.Parameters.Add("P_DESTINATION_CODE", OracleDbType.Varchar2).Value = pInstallDetail.DESTINATION_CODE;
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 10).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                if (cmd.Parameters["R_STATUS"].Value.ToString() == "Exist")
                {
                    return ResponseMessage(Request.CreateResponse(HttpStatusCode.BadRequest, "Unable to add existing Install parts."));
                }
                else
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNM120P_INSTALL_H_ADD";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_END_ITEM_18", OracleDbType.Varchar2).Value = pInstallDetail.END_ITEM_18;
                    cmd.Parameters.Add("P_EXT_COLOR", OracleDbType.Varchar2).Value = pInstallDetail.EXT_COLOR;
                    cmd.Parameters.Add("P_INT_COLOR", OracleDbType.Varchar2).Value = pInstallDetail.INT_COLOR;
                    cmd.Parameters.Add("P_PDI_LOCATION_CODE", OracleDbType.Varchar2).Value = pInstallDetail.PDI_LOCATION_CODE;
                    cmd.Parameters.Add("P_DESTINATION_CODE", OracleDbType.Varchar2).Value = pInstallDetail.DESTINATION_CODE;
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("R_BOM_KEY", OracleDbType.Varchar2, 35).Direction = ParameterDirection.Output;

                    cmd = oOracledb.SqlQueryReturnValue(cmd);

                    string BOM_KEY = cmd.Parameters["R_BOM_KEY"].Value.ToString();

                    foreach (var installParts in pInstallDetail.INSTALL_PARTS)
                    {
                        cmd = new OracleCommand();
                        cmd.CommandText = "SP_NP_SNM120P_INSTALL_D_ADD";
                        cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                        cmd.Parameters.Add("P_BOM_KEY", OracleDbType.Varchar2).Value = BOM_KEY;
                        cmd.Parameters.Add("P_INSTALL_PART_NO", OracleDbType.Varchar2).Value = installParts.INSTALL_PART_NO;
                        cmd.Parameters.Add("P_INSTALL_QTY", OracleDbType.Decimal).Value = installParts.INSTALL_QTY;
                        cmd.Parameters.Add("P_TAKE_FROM", OracleDbType.Varchar2).Value = installParts.TAKE_FROM;
                        cmd.Parameters.Add("P_ISSUE_DOC_TYPE", OracleDbType.Varchar2).Value = installParts.ISSUE_DOC_TYPE;
                        cmd.Parameters.Add("P_CUSTOMER_TYPE", OracleDbType.Varchar2).Value = installParts.CUSTOMER_TYPE;
                        cmd.Parameters.Add("P_FIX_CUSTOMER_CODE", OracleDbType.Varchar2).Value = installParts.FIX_CUSTOMER_CODE;
                        cmd.Parameters.Add("P_FIX_BRANCH_CODE", OracleDbType.Varchar2).Value = installParts.FIX_BRANCH_CODE;
                        cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                        oOracledb.SqlQueryReturnValue(cmd);
                    }
                }
            }
            catch (Exception ex)
            {
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

            return Ok();

        }


        [HttpPost, Route("SNM120P_Save/{bomKey}")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM120P_Save([FromBody] dynamic pInstallDetail, string bomKey)
        {
            OracleCommand cmd;

            try
            {
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM120P_INSTALL_D_DELETE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_BOM_KEY", OracleDbType.Varchar2).Value = bomKey;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                oOracledb.SqlQueryReturnValue(cmd);

                foreach (var installParts in pInstallDetail.INSTALL_PARTS)
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNM120P_INSTALL_D_ADD";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_BOM_KEY", OracleDbType.Varchar2).Value = bomKey;
                    cmd.Parameters.Add("P_INSTALL_PART_NO", OracleDbType.Varchar2).Value = installParts.INSTALL_PART_NO;
                    cmd.Parameters.Add("P_INSTALL_QTY", OracleDbType.Decimal).Value = installParts.INSTALL_QTY;
                    cmd.Parameters.Add("P_TAKE_FROM", OracleDbType.Varchar2).Value = installParts.TAKE_FROM;
                    cmd.Parameters.Add("P_ISSUE_DOC_TYPE", OracleDbType.Varchar2).Value = installParts.ISSUE_DOC_TYPE;
                    cmd.Parameters.Add("P_CUSTOMER_TYPE", OracleDbType.Varchar2).Value = installParts.CUSTOMER_TYPE;
                    cmd.Parameters.Add("P_FIX_CUSTOMER_CODE", OracleDbType.Varchar2).Value = installParts.FIX_CUSTOMER_CODE;
                    cmd.Parameters.Add("P_FIX_BRANCH_CODE", OracleDbType.Varchar2).Value = installParts.FIX_BRANCH_CODE;
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                    oOracledb.SqlQueryReturnValue(cmd);
                }


                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM120P_INSTALL_H_SAVE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_BOM_KEY", OracleDbType.Varchar2).Value = bomKey;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                oOracledb.SqlQueryReturnValue(cmd);
            }
            catch (Exception ex)
            {
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

            return Ok();

        }


        [HttpPost, Route("SNM120P_Delete/{bomKey}")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM120P_Delete(string bomKey)
        {
            OracleCommand cmd;

            try
            {
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM120P_INSTALL_DELETE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_BOM_KEY", OracleDbType.Varchar2).Value = bomKey;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                oOracledb.SqlQueryReturnValue(cmd);
            }
            catch (Exception ex)
            {
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

            return Ok();

        }
        #endregion

        #region SNPP205
        [HttpPost, Route("SNPP205_GetData")]
        [Authorize]
        public IHttpActionResult SNPP203_GetData([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNPP205_GET_DATA";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = param.VENDOR_CODE;
            cmd.Parameters.Add("P_FROM_DATE", OracleDbType.Date).Value = param.FROM_DATE;
            cmd.Parameters.Add("P_TO_DATE", OracleDbType.Date).Value = param.TO_DATE;
            //cmd.Parameters.Add("P_PIC", OracleDbType.Varchar2).Value = param.PIC;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNPP205_GetDataPO")]
        [Authorize]
        public IHttpActionResult SNPP205_GetDataPO([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNPP205_GET_DATA_PO";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_CREATE_ID", OracleDbType.Varchar2).Value = param.CREATE_ID;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }

        

        [HttpGet, Route("SNPP205_GetStockInfo")]
        [Authorize]
        public IHttpActionResult SNPP205_GetStockInfo()
        {
            DataSet ds = new DataSet();
            //DataSet dt = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNPP205_GET_STOCK_INFO";
            //cmd.Parameters.Add(new OracleParameter("test", ds));
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNPP205_CreatePO")]
        [Authorize]
        public IHttpActionResult SNPP205_CreatePO(List<DataModel.SNPP203_CREATE_PO> param)
        {
            DataSet ds = new DataSet();
            string fromPO = "", toPO = "";
            try
            {
                var json = JsonConvert.SerializeObject(param);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPP205_CREATE_PO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PO_DATA", OracleDbType.Varchar2).Value = json;
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_FROM_PO", OracleDbType.Varchar2, 6).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_TO_PO", OracleDbType.Varchar2, 6).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_CREATE_ID", OracleDbType.Varchar2, 100).Direction = ParameterDirection.Output;
                oOracledb.SqlQueryReturnValue(cmd);
                string R_STATUS = cmd.Parameters["R_STATUS"].Value.ToString();
                string R_CREATE_ID = cmd.Parameters["R_CREATE_ID"].Value.ToString();
                if (R_STATUS == "DONE")
                {
                    fromPO = cmd.Parameters["R_FROM_PO"].Value.ToString();
                    toPO = cmd.Parameters["R_TO_PO"].Value.ToString();
                }

                return Json(new { STATUS = R_STATUS, FROM_PO = fromPO, TO_PO = toPO,CREATE_ID = R_CREATE_ID });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPP203_CreatePO");
                return Json(new { STATUS = "FAIL" });
            }
        }

        [HttpPost, Route("SNPP205_UploadFile")]
        [Authorize]
        public async Task<IHttpActionResult> SNPP205_UploadFile(HttpRequestMessage request)
        {
            string company_code  = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            string userId = DataService.GetIdentity(User, Cons.SS_USER_NAME);


            if (!request.Content.IsMimeMultipartContent())
            {
                throw new HttpResponseException(HttpStatusCode.UnsupportedMediaType);
            }
            var reqUpload = await request.Content.ReadAsMultipartAsync();
            try
            {
                string sqlDelete = String.Format("DELETE TPM_NP_PDI_PRODUCTION_PLAN_COLOR_UPLOAD where company_code = '{0}' and create_by = '{1}'", company_code, userId);
                int row = oOracledb.SqlExecute(sqlDelete);

                DataSet ds = new DataSet();
                DataTable dt = new DataTable();
                string sql = @"SELECT   COMPANY_CODE,
                                        EX_YEAR     ,
                                        EX_MONTH    ,
                                        END_ITEM_18 ,
                                        DESTINATION ,
                                        SPEC_CODE   ,
                                        COLOR       ,
                                        UNIT_QTY    ,
                                        CREATE_BY
                                FROM TPM_NP_PDI_PRODUCTION_PLAN_COLOR_UPLOAD";
                string SPPS_CODE = string.Empty;
                var file = reqUpload.Contents[0];
                var buffer = await file.ReadAsByteArrayAsync();
                using(MemoryStream stream = new MemoryStream(buffer))
                {
                    //dt.Columns.Add("COMPANY_CODE")
                    SystemApiController sys = new SystemApiController();
                    dt = sys.ReadDataExcel(stream);
                    dt.TableName = "Table";
                   
                    DataTable newDt = new DataTable();
                    DataView dtv = new DataView();
                    dtv = dt.AsDataView();
                    
                    newDt = dtv.ToTable();
                    DataColumn newColumn = new DataColumn();
                    newColumn = new DataColumn("CREATE_BY", typeof(System.String));
                    newColumn.DefaultValue = userId;
                    newDt.Columns.Add(newColumn);
                    newColumn = new DataColumn("COMPANY_CODE", typeof(System.String));
                    newColumn.DefaultValue = company_code;
                    newDt.Columns.Add(newColumn);
                    int rowCount = oOracledb.InsertDataTable(newDt, sql);
                }

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNPP205_VALIDATE_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPP205_UploadFile");
                throw new HttpResponseException(HttpStatusCode.NotAcceptable);
            }
            throw new Exception("Something wrong");
        }

        [HttpGet, Route("SNPP205_Process")]
        [Authorize]
        public IHttpActionResult SNPP205_Process()
        {
            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNPP205_START_PROCESS";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
            cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
            cmd = oOracledb.SqlQueryReturnValue(cmd);

            string R_STATUS = cmd.Parameters["R_STATUS"].Value.ToString();
            return Ok(R_STATUS);
        }

        [HttpPost, Route("SNPP205_DownloadRaw")]
        [Authorize]
        public HttpResponseMessage SNPP205_DownloadRaw([FromBody] dynamic param)
        {
            

            IEnumerable<dynamic> FileData = param.Table;

            //var List1 = JsonConvert.DeserializeObject<List<dynamic>>(param.DataTable);
            HttpResponseMessage response = new HttpResponseMessage(HttpStatusCode.OK);
            string gLocation = "~/tempFile/" + "SNPP205" + "/";
            string gWriteLocation = System.Web.HttpContext.Current.Server.MapPath(gLocation);
            FileInfo filePath = new FileInfo(gWriteLocation);
            filePath.Directory.Create();
            string gFileLocation = Path.Combine(gWriteLocation, "SNPP205_RAW.xlsx");
            string gReturnLocation = Path.Combine(gLocation, "SNPP205_RAW.xlsx");
            byte[] excel = FileData.ToExcel(x => x
            .AddColumn("VENDOR",z => z.VENDOR)
            .AddColumn("VENDOR_CODE",z => z.VENDOR_CODE)
            .AddColumn("TEMP_PO",z => z.PO_NO)
            .AddColumn("ACCEPT_PART_NO",z => z.ACCEPT_PART_NO)
            .AddColumn("ORDER_QTY", z => int.Parse(z.ORDER_QTY))
            );
            //byte[] excel = excel2;
            File.WriteAllBytes(gFileLocation, excel);
            string path = HttpContext.Current.Server.MapPath("~/" + gReturnLocation.Substring(1));
            if (!File.Exists(path))
            {
                return Request.CreateErrorResponse(HttpStatusCode.NotFound, "File not found.");
            }
            try
            {
                string mimeType = MimeMapping.GetMimeMapping("SNPP205_RAW.xlsx");
                StreamContent x = new StreamContent(new FileStream(path, FileMode.Open, FileAccess.Read));
                response.Content = x;
                response.Content.Headers.ContentType = new MediaTypeHeaderValue(mimeType);
                response.Content.Headers.ContentDisposition = new ContentDispositionHeaderValue("attachment");
                response.Content.Headers.ContentDisposition.FileName = "SNPP205_RAW.xlsx";
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPP205_DownloadRaw");
                return Request.CreateResponse(HttpStatusCode.BadRequest, ex);
            }
            return response;
        }

        [HttpPost, Route("SNPP205_DownloadPO")]
        [Authorize]
        public HttpResponseMessage SNPP205_DownloadPO([FromBody] dynamic param)
        {


            IEnumerable<dynamic> FileData = param.Table;

            //var List1 = JsonConvert.DeserializeObject<List<dynamic>>(param.DataTable);
            HttpResponseMessage response = new HttpResponseMessage(HttpStatusCode.OK);
            string gLocation = "~/tempFile/" + "SNPP205" + "/";
            string gWriteLocation = System.Web.HttpContext.Current.Server.MapPath(gLocation);
            FileInfo filePath = new FileInfo(gWriteLocation);
            filePath.Directory.Create();
            string gFileLocation = Path.Combine(gWriteLocation, "SNPP205_PO.xlsx");
            string gReturnLocation = Path.Combine(gLocation, "SNPP205_PO.xlsx");
            byte[] excel = FileData.ToExcel(x => x
            .AddColumn("VENDOR_CODE", z => z.VENDOR)
            .AddColumn("VENDOR_NAME", z => z.VENDOR_NAME_EN)
            .AddColumn("PO_NO", z => z.PO_NO)
            .AddColumn("PO_DATE", z => z.PO_DATE.ToString("dd/MMM/yyyy"))
            .AddColumn("ORDER_PART_NO", z => z.ORDER_PART_NO)
            .AddColumn("ACCEPT_PART_NO", z => z.ACCEPT_PART_NO)
            .AddColumn("ORDER_QTY", z => int.Parse(z.ORDER_QTY))
            .AddColumn("ACCEPT_QTY",z => int.Parse(z.ACCEPT_QTY))
            );
            //byte[] excel = excel2;
            File.WriteAllBytes(gFileLocation, excel);
            string path = HttpContext.Current.Server.MapPath("~/" + gReturnLocation.Substring(1));
            if (!File.Exists(path))
            {
                return Request.CreateErrorResponse(HttpStatusCode.NotFound, "File not found.");
            }
            try
            {
                string mimeType = MimeMapping.GetMimeMapping("SNPP205_PO.xlsx");
                StreamContent x = new StreamContent(new FileStream(path, FileMode.Open, FileAccess.Read));
                response.Content = x;
                response.Content.Headers.ContentType = new MediaTypeHeaderValue(mimeType);
                response.Content.Headers.ContentDisposition = new ContentDispositionHeaderValue("attachment");
                response.Content.Headers.ContentDisposition.FileName = "SNPP205_PO.xlsx";
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNPP205_DownloadPO");
                return Request.CreateResponse(HttpStatusCode.BadRequest, ex);
            }
            return response;
        }

        #endregion SNPP205

        #region SNM149

        [HttpGet, Route("SNM149_GetPDIBaseModelAll/{basemodel}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM149_GetPDIBaseModelAll(string basemodel, int noofshowrec)
        {
            DataSet ds = new DataSet();
            basemodel = ismoUtil.escapeCharacter(basemodel);
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM149_GET_PDI_BASE_MODEL_ALL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_BASE_MODEL", OracleDbType.Varchar2).Value = basemodel.ToUpper() == "NULL" ? (object)DBNull.Value : basemodel;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM149_GetPDIBaseModelAll");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNM149_GetPDIBaseModel/{basemodel}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM149_GetPDIBaseModel(string basemodel, int noofshowrec)
        {
            DataSet ds = new DataSet();
            basemodel = ismoUtil.escapeCharacter(basemodel);
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM149_GET_PDI_BASE_MODEL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_BASE_MODEL", OracleDbType.Varchar2).Value = basemodel.ToUpper() == "NULL" ? (object)DBNull.Value : basemodel;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM149_GetPDIBaseModel");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNM149_GetData/{basemodel}")]
        [Authorize]
        public IHttpActionResult SNM149_GetData(string basemodel)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM149_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_BASE_MODEL", OracleDbType.Varchar2).Value = basemodel.ToUpper() == "NULL" ? (object)DBNull.Value : basemodel;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(new { status = "Done", message = "Done", set = DTFM.convertToList(ds) });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM149_GetData");
                return Json(new { status = "Error", message = ex.Message });
            }
        }

        [HttpPost, Route("SNM149_Insert")]
        [Authorize]
        public IHttpActionResult SNM149_Insert([FromBody] dynamic pParam)
        {
            try
            {
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();

                OracleCommand cmd = new OracleCommand();
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM149_INSERT";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_BASE_MODEL", OracleDbType.Varchar2).Value = pParam["BASE_MODEL"];
                cmd.Parameters.Add("P_DESCRIPTIONS", OracleDbType.Varchar2).Value = pParam["DESCRIPTIONS"];
                cmd.Parameters.Add("P_STATUS", OracleDbType.Varchar2).Value = pParam["STATUS"];
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 100).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string r_status = cmd.Parameters["R_STATUS"].Value.ToString();
                string r_message = cmd.Parameters["R_MESSAGE"].Value.ToString();

                return Json(new { status = r_status, message = r_message });
            }
            catch (Exception ex)
            {
                //--- Roll back ---
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM149_Insert");
                return Json(new { status = "Fail", message = ex.Message });
            }
        }

        [HttpPost, Route("SNM149_Update")]
        [Authorize]
        public IHttpActionResult SNM149_Update([FromBody] dynamic pParam)
        {
            try
            {
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();

                OracleCommand cmd = new OracleCommand();
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM149_UPDATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_BASE_MODEL", OracleDbType.Varchar2).Value = pParam["BASE_MODEL"];
                cmd.Parameters.Add("P_DESCRIPTIONS", OracleDbType.Varchar2).Value = pParam["DESCRIPTIONS"];
                cmd.Parameters.Add("P_STATUS", OracleDbType.Varchar2).Value = pParam["STATUS"];
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 100).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string r_status = cmd.Parameters["R_STATUS"].Value.ToString();
                string r_message = cmd.Parameters["R_MESSAGE"].Value.ToString();

                return Json(new { status = r_status, message = r_message });
            }
            catch (Exception ex)
            {
                //--- Roll back ---
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM149_Update");
                return Json(new { status = "Fail", message = ex.Message });
            }
        }

        [HttpPost, Route("SNM149_Delete")]
        [Authorize]
        public IHttpActionResult SNM149_Delete([FromBody] dynamic pParam)
        {
            try
            {
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();

                OracleCommand cmd = new OracleCommand();
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM149_DELETE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_BASE_MODEL", OracleDbType.Varchar2).Value = pParam["BASE_MODEL"];
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 100).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string r_status = cmd.Parameters["R_STATUS"].Value.ToString();
                string r_message = cmd.Parameters["R_MESSAGE"].Value.ToString();

                return Json(new { status = r_status, message = r_message });
            }
            catch (Exception ex)
            {
                //--- Roll back ---
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM149_Delete");
                return Json(new { status = "Fail", message = ex.Message });
            }
        }

        [HttpPost, Route("SNM149_UploadFile")]
        [Authorize]
        public async Task<IHttpActionResult> SNM149_UploadFile(HttpRequestMessage request)
        {
            if (!request.Content.IsMimeMultipartContent())
            {
                throw new HttpResponseException(HttpStatusCode.UnsupportedMediaType);
            }
            var reqUpload = await request.Content.ReadAsMultipartAsync();
            try
            {
                OracleCommand cmd = new OracleCommand();
                SystemApiController sys = new SystemApiController();
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();
                DataTable dt_file = new DataTable();
                DataColumn dc = null;
                var file = reqUpload.Contents[0];
                var buffer = await file.ReadAsByteArrayAsync();
                MemoryStream stream = new MemoryStream(buffer);
                dt_file = sys.ReadDataExcel(stream);
                int colCount = dt_file.Columns.Count;
                string sql = @"SELECT * FROM TPM_NP_PDI_BASE_MODEL_MS_TEMPLATE";
                dc = new DataColumn("UPLOAD_USER", typeof(String));
                dc.DefaultValue = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                dt_file.Columns.Add(dc);
                dc = new DataColumn("COMPANY_CODE", typeof(String));
                dc.DefaultValue = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                dt_file.Columns.Add(dc);
                dt_file.Columns.Add("SEQ");
                DataView dv = dt_file.AsDataView();
                dt = dv.ToTable();

                for (int i = 0; i < dt.Rows.Count; i++)
                {
                    dt.Rows[i]["SEQ"] = i + 1;
                }

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM149_CLEAR_TEMP";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                oOracledb.SqlExecute_CMD(cmd);
                int count = oOracledb.InsertDataTable(dt, sql);
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM149_VALIDATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(new { status = "Done", message = "Done", set = DTFM.convertToList(ds) });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM149_UploadFile");
                throw new HttpResponseException(HttpStatusCode.NotAcceptable);
            }
            throw new Exception("Something wrong");
        }
        [HttpPost, Route("SNM149_SaveByUpload")]
        [Authorize]
        public IHttpActionResult SNM149_SaveByUpload([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM149_SAVE_BY_UPLOAD";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 10).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 100).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);
                string r_status = cmd.Parameters["R_STATUS"].Value.ToString();
                string r_message = cmd.Parameters["R_MESSAGE"].Value.ToString();
                return Json(new { status = r_status, message = r_message });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM149_SaveByUpload");
                return Json(new { status = "Fail", message = ex.Message });
            }
        }

        #endregion

        #region SNM150

        [HttpGet, Route("SNM150_GetInstallPartNo/{installpartno}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM150_GetInstallPartNo(string installpartno, int noofshowrec)
        {
            DataSet ds = new DataSet();
            installpartno = ismoUtil.escapeCharacter(installpartno);
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM150_GET_INSTALL_PART_NO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_INSTALL_PART_NO", OracleDbType.Varchar2).Value = installpartno.ToUpper() == "NULL" ? (object)DBNull.Value : installpartno;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM150_GetInstallPartNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNM150_GetComponentPartNo/{componentpartno}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM150_GetComponentPartNo(string componentpartno, int noofshowrec)
        {
            DataSet ds = new DataSet();
            componentpartno = ismoUtil.escapeCharacter(componentpartno);
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM150_GET_COMPONENT_PART_NO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_COMPONENT_PART_NO", OracleDbType.Varchar2).Value = componentpartno.ToUpper() == "NULL" ? (object)DBNull.Value : componentpartno;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM150_GetComponentPartNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNM150_GetData/{installpartno}/{componentpartno}")]
        [Authorize]
        public IHttpActionResult SNM150_GetData(string installpartno, string componentpartno)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM150_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_INSTALL_PART_NO", OracleDbType.Varchar2).Value = installpartno.ToUpper() == "NULL" ? (object)DBNull.Value : installpartno;
                cmd.Parameters.Add("P_COMPONENT_PART_NO", OracleDbType.Varchar2).Value = componentpartno.ToUpper() == "NULL" ? (object)DBNull.Value : componentpartno;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(new { status = "Done", message = "Done", set = DTFM.convertToList(ds) });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM150_GetData");
                return Json(new { status = "Error", message = ex.Message });
            }
        }

        [HttpPost, Route("SNM150_Insert")]
        [Authorize]
        public IHttpActionResult SNM150_Insert([FromBody] dynamic pParam)
        {
            try
            {
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();

                OracleCommand cmd = new OracleCommand();
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM150_INSERT";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_INSTALL_PART_NO", OracleDbType.Varchar2).Value = pParam["INSTALL_PART_NO"];
                cmd.Parameters.Add("P_POSITION_NO", OracleDbType.Varchar2).Value = pParam["POSITION_NO"];
                cmd.Parameters.Add("P_EXT_COLOR", OracleDbType.Varchar2).Value = pParam["EXT_COLOR"];
                cmd.Parameters.Add("P_INT_COLOR", OracleDbType.Varchar2).Value = pParam["INT_COLOR"];
                cmd.Parameters.Add("P_COMPONENT_PART_NO", OracleDbType.Varchar2).Value = pParam["COMPONENT_PART_NO"];
                cmd.Parameters.Add("P_COMPONENT_QTY", OracleDbType.Varchar2).Value = pParam["COMPONENT_QTY"];
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 400).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string r_status = cmd.Parameters["R_STATUS"].Value.ToString();
                string r_message = cmd.Parameters["R_MESSAGE"].Value.ToString();

                return Json(new { status = r_status, message = r_message });
            }
            catch (Exception ex)
            {
                //--- Roll back ---
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM150_Insert");
                return Json(new { status = "Fail", message = ex.Message });
            }
        }

        [HttpPost, Route("SNM150_Update")]
        [Authorize]
        public IHttpActionResult SNM150_Update([FromBody] dynamic pParam)
        {
            try
            {
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();

                OracleCommand cmd = new OracleCommand();
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM150_UPDATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ROWID", OracleDbType.Varchar2).Value = pParam["ID"];
                cmd.Parameters.Add("P_INSTALL_PART_NO", OracleDbType.Varchar2).Value = pParam["INSTALL_PART_NO"];
                cmd.Parameters.Add("P_POSITION_NO", OracleDbType.Varchar2).Value = pParam["POSITION_NO"];
                cmd.Parameters.Add("P_EXT_COLOR", OracleDbType.Varchar2).Value = pParam["EXT_COLOR"];
                cmd.Parameters.Add("P_INT_COLOR", OracleDbType.Varchar2).Value = pParam["INT_COLOR"];
                cmd.Parameters.Add("P_COMPONENT_PART_NO", OracleDbType.Varchar2).Value = pParam["COMPONENT_PART_NO"];
                cmd.Parameters.Add("P_COMPONENT_QTY", OracleDbType.Varchar2).Value = pParam["COMPONENT_QTY"];
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 400).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string r_status = cmd.Parameters["R_STATUS"].Value.ToString();
                string r_message = cmd.Parameters["R_MESSAGE"].Value.ToString();

                return Json(new { status = r_status, message = r_message });
            }
            catch (Exception ex)
            {
                //--- Roll back ---
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM150_Update");
                return Json(new { status = "Fail", message = ex.Message });
            }
        }

        [HttpPost, Route("SNM150_Delete")]
        [Authorize]
        public IHttpActionResult SNM150_Delete([FromBody] dynamic pParam)
        {
            try
            {
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();

                OracleCommand cmd = new OracleCommand();
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM150_DELETE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ROWID", OracleDbType.Varchar2).Value = pParam["ID"];
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 100).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string r_status = cmd.Parameters["R_STATUS"].Value.ToString();
                string r_message = cmd.Parameters["R_MESSAGE"].Value.ToString();

                return Json(new { status = r_status, message = r_message });
            }
            catch (Exception ex)
            {
                //--- Roll back ---
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM150_Delete");
                return Json(new { status = "Fail", message = ex.Message });
            }
        }

        [HttpPost, Route("SNM150_UploadFile")]
        [Authorize]
        public async Task<IHttpActionResult> SNM150_UploadFile(HttpRequestMessage request)
        {
            if (!request.Content.IsMimeMultipartContent())
            {
                throw new HttpResponseException(HttpStatusCode.UnsupportedMediaType);
            }
            var reqUpload = await request.Content.ReadAsMultipartAsync();
            try
            {
                OracleCommand cmd = new OracleCommand();
                SystemApiController sys = new SystemApiController();
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();
                DataTable dt_file = new DataTable();
                DataColumn dc = null;
                var file = reqUpload.Contents[0];
                var buffer = await file.ReadAsByteArrayAsync();
                MemoryStream stream = new MemoryStream(buffer);
                dt_file = sys.ReadDataExcel(stream);
                int colCount = dt_file.Columns.Count;
                string sql = @"SELECT * FROM TPM_NP_PDI_KIT_BOM_TEMPLATE";
                string column = @"INSTALL_PART_NO
                    ,POSITION_NO
                    ,COMPONENT_PART_NO
                    ,COMPONENT_QTY
                    ,EXT_COLOR
                    ,INT_COLOR";
                string[] array = column.Split(',');
                dc = new DataColumn("UPLOAD_USER", typeof(String));
                dc.DefaultValue = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                dt_file.Columns.Add(dc);
                dc = new DataColumn("COMPANY_CODE", typeof(String));
                dc.DefaultValue = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                dt_file.Columns.Add(dc);
                dt_file.Columns.Add("SEQ");
                DataView dv = dt_file.AsDataView();
                dt = dv.ToTable();

                for (int i = 0; i < colCount; i++)
                {
                    dt.Columns[i].ColumnName = array[i].Trim();
                }

                for (int i = 0; i < dt.Rows.Count; i++)
                {
                    dt.Rows[i]["SEQ"] = i + 1;
                }

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM150_CLEAR_TEMP";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                oOracledb.SqlExecute_CMD(cmd);
                int count = oOracledb.InsertDataTable(dt, sql);
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM150_VALIDATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(new { status = "Done", message = "Done", set = DTFM.convertToList(ds) });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM150_UploadFile");
                throw new HttpResponseException(HttpStatusCode.NotAcceptable);
            }
            throw new Exception("Something wrong");
        }

        [HttpPost, Route("SNM150_SaveByUpload")]
        [Authorize]
        public IHttpActionResult SNM150_SaveByUpload([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM150_SAVE_BY_UPLOAD";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 10).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 100).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);
                string r_status = cmd.Parameters["R_STATUS"].Value.ToString();
                string r_message = cmd.Parameters["R_MESSAGE"].Value.ToString();
                return Json(new { status = r_status, message = r_message });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM150_SaveByUpload");
                return Json(new { status = "Fail", message = ex.Message });
            }
        }

        #endregion

        #region SNM151

        [HttpGet, Route("SNM151_GetData/{optioncode}")]
        [Authorize]
        public IHttpActionResult SNM151_GetData(string optioncode)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM151_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_OPTION_CODE", OracleDbType.Varchar2).Value = optioncode.ToUpper() == "NULL" ? (object)DBNull.Value : optioncode;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(new { status = "Done", message = "Done", set = DTFM.convertToList(ds) });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM151_GetData");
                return Json(new { status = "Error", message = ex.Message });
            }
        }

        [HttpPost, Route("SNM151_Insert")]
        [Authorize]
        public IHttpActionResult SNM151_Insert([FromBody] dynamic pParam)
        {
            try
            {
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();

                OracleCommand cmd = new OracleCommand();
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM151_INSERT";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_OPTION_CODE", OracleDbType.Varchar2).Value = pParam["OPTION_CODE"];
                cmd.Parameters.Add("P_COMPLETE_PART_NO", OracleDbType.Varchar2).Value = pParam["COMPLETE_PART_NO"];
                cmd.Parameters.Add("P_INSTALL_PART_NO", OracleDbType.Varchar2).Value = pParam["INSTALL_PART_NO"];
                cmd.Parameters.Add("P_STATUS", OracleDbType.Varchar2).Value = pParam["STATUS"];
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 100).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string r_status = cmd.Parameters["R_STATUS"].Value.ToString();
                string r_message = cmd.Parameters["R_MESSAGE"].Value.ToString();

                return Json(new { status = r_status, message = r_message });
            }
            catch (Exception ex)
            {
                //--- Roll back ---
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM151_Insert");
                return Json(new { status = "Fail", message = ex.Message });
            }
        }

        [HttpPost, Route("SNM151_Update")]
        [Authorize]
        public IHttpActionResult SNM151_Update([FromBody] dynamic pParam)
        {
            try
            {
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();

                OracleCommand cmd = new OracleCommand();
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM151_UPDATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_OPTION_CODE", OracleDbType.Varchar2).Value = pParam["OPTION_CODE"];
                cmd.Parameters.Add("P_COMPLETE_PART_NO", OracleDbType.Varchar2).Value = pParam["COMPLETE_PART_NO"];
                cmd.Parameters.Add("P_INSTALL_PART_NO", OracleDbType.Varchar2).Value = pParam["INSTALL_PART_NO"];
                cmd.Parameters.Add("P_STATUS", OracleDbType.Varchar2).Value = pParam["STATUS"];
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 100).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string r_status = cmd.Parameters["R_STATUS"].Value.ToString();
                string r_message = cmd.Parameters["R_MESSAGE"].Value.ToString();

                return Json(new { status = r_status, message = r_message });
            }
            catch (Exception ex)
            {
                //--- Roll back ---
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM151_Update");
                return Json(new { status = "Fail", message = ex.Message });
            }
        }

        [HttpPost, Route("SNM151_Delete")]
        [Authorize]
        public IHttpActionResult SNM151_Delete([FromBody] dynamic pParam)
        {
            try
            {
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();

                OracleCommand cmd = new OracleCommand();
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM151_DELETE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_OPTION_CODE", OracleDbType.Varchar2).Value = pParam["OPTION_CODE"];
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 100).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string r_status = cmd.Parameters["R_STATUS"].Value.ToString();
                string r_message = cmd.Parameters["R_MESSAGE"].Value.ToString();

                return Json(new { status = r_status, message = r_message });
            }
            catch (Exception ex)
            {
                //--- Roll back ---
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM151_Delete");
                return Json(new { status = "Fail", message = ex.Message });
            }
        }

        [HttpPost, Route("SNM151_UploadFile")]
        [Authorize]
        public async Task<IHttpActionResult> SNM151_UploadFile(HttpRequestMessage request)
        {
            if (!request.Content.IsMimeMultipartContent())
            {
                throw new HttpResponseException(HttpStatusCode.UnsupportedMediaType);
            }
            var reqUpload = await request.Content.ReadAsMultipartAsync();
            try
            {
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();
                string SPPS_CODE = string.Empty;
                var file = reqUpload.Contents[0];
                var buffer = await file.ReadAsByteArrayAsync();
                MemoryStream stream = new MemoryStream(buffer);
                dt = SNM151_ValidateUpload(stream);
                stream.Flush();
                stream.Dispose();
                return Json(DTFM.convertToList(dt));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM151_UploadFile");
                throw new HttpResponseException(HttpStatusCode.NotAcceptable);
            }
            throw new Exception("Something wrong");
        }

        public DataTable SNM151_ValidateUpload(MemoryStream fileSteam)
        {
            DataTable dt = new DataTable();
            Hashtable hTable = new Hashtable();
            SystemApiController sys = new SystemApiController();
            dt = sys.ReadDataExcel(fileSteam);
            dt.Columns.Add("SEQ");
            dt.Columns.Add("ERR_FLAG");
            dt.Columns.Add("ERR_MSG");
            int iCount = 0;
            string key = string.Empty;
            OracleCommand cmd = new OracleCommand();
            foreach (DataRow row in dt.Rows)
            {
                iCount++;
                row["SEQ"] = iCount;
                key = row["OPTION_CODE"].ToString().Trim();
                if (hTable.Contains(key))
                {
                    row["ERR_FLAG"] = "Y";
                    row["ERR_MSG"] = "Duplicate Option code at Seq " + hTable[key];
                }
                else
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNM151_VALIDATE";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_OPTION_CODE", OracleDbType.Varchar2).Value = row["OPTION_CODE"].ToString().Trim();
                    cmd.Parameters.Add("P_COMPLETE_PART_NO", OracleDbType.Varchar2).Value = row["COMPLETE_PART_NO"].ToString().Trim();
                    cmd.Parameters.Add("P_INSTALL_PART_NO", OracleDbType.Varchar2).Value = row["INSTALL_PART_NO"].ToString().Trim();
                    cmd.Parameters.Add("R_ERR_FLAG", OracleDbType.Varchar2, 1).Direction = ParameterDirection.Output;
                    cmd.Parameters.Add("R_ERR_MSG", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;
                    cmd = oOracledb.SqlQueryReturnValue(cmd);

                    row["ERR_FLAG"] = cmd.Parameters["R_ERR_FLAG"].Value.ToString();
                    row["ERR_MSG"] = cmd.Parameters["R_ERR_MSG"].Value.ToString().ToUpper() == "NULL" ? "" : cmd.Parameters["R_ERR_MSG"].Value.ToString();
                    hTable.Add(key, iCount);
                }
            }
            return dt;
        }

        [HttpPost, Route("SNM151_SaveByUpload")]
        [Authorize]
        public IHttpActionResult SNM151_SaveByUpload([FromBody] dynamic[] pData)
        {
            try
            {
                var json = JsonConvert.SerializeObject(pData);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM151_SAVE_BY_UPLOAD";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_JSON_DATA", OracleDbType.Varchar2).Value = json;
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 10).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 100).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);
                string r_status = cmd.Parameters["R_STATUS"].Value.ToString();
                string r_message = cmd.Parameters["R_MESSAGE"].Value.ToString();
                return Json(new { status = r_status, message = r_message });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM151_SaveByUpload");
                return Json(new { status = "Fail", message = ex.Message });
            }
        }

        #endregion

        #region SNM152

        [HttpGet, Route("SNM152_GetCompletePartNo/{completepartno}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM152_GetCompletePartNo(string completepartno, int noofshowrec)
        {
            DataSet ds = new DataSet();
            completepartno = ismoUtil.escapeCharacter(completepartno);
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM152_GET_COMPLETE_PART_NO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_COMPLETE_PART_NO", OracleDbType.Varchar2).Value = completepartno.ToUpper() == "NULL" ? (object)DBNull.Value : completepartno;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM152_GetCompletePartNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNM152_GetWIPPartNo/{wippartno}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM152_GetWIPPartNo(string wippartno, int noofshowrec)
        {
            DataSet ds = new DataSet();
            wippartno = ismoUtil.escapeCharacter(wippartno);
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM152_GET_WIP_PART_NO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_WIP_PART_NO", OracleDbType.Varchar2).Value = wippartno.ToUpper() == "NULL" ? (object)DBNull.Value : wippartno;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM152_GetWIPPartNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNM152_GetInstallLocationCode/{installLocationCode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM152_GetInstallLocationCode(string installLocationCode, int noofshowrec)
        {
            DataSet ds = new DataSet();
            installLocationCode = ismoUtil.escapeCharacter(installLocationCode);
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM152_GET_INSTALL_LOCATION_CODE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_INSTALL_LOCATION_CODE", OracleDbType.Varchar2).Value = installLocationCode.ToUpper() == "NULL" ? (object)DBNull.Value : installLocationCode;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM152_GetInstallLocationCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNM152_GetBaseModel/{basemodel}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM152_GetBaseModel(string basemodel, int noofshowrec)
        {
            DataSet ds = new DataSet();
            basemodel = ismoUtil.escapeCharacter(basemodel);
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM152_GET_BASE_MODEL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_BASE_MODEL", OracleDbType.Varchar2).Value = basemodel.ToUpper() == "NULL" ? (object)DBNull.Value : basemodel;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM152_GetBaseModel");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpGet, Route("SNM152_GetActivityMaster")]
        [Authorize]
        public IHttpActionResult SNM152_GetActivityMaster()
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM152_GET_ACTIVITY_MASTER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(new { status = "Done", message = "Done", set = DTFM.convertToList(ds) });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM152_GetActivityMaster");
                return Json(new { status = "Error", message = ex.Message });
            }
        }

        [HttpGet, Route("SNM152_GetData/{optioncode}/{completePartno}/{wipPartno}")]
        [Authorize]
        public IHttpActionResult SNM152_GetData(string optioncode, string completePartno, string wipPartno)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM152_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_WIP_PART_NO", OracleDbType.Varchar2).Value = optioncode.ToUpper() == "NULL" ? (object)DBNull.Value : optioncode;
                cmd.Parameters.Add("P_COMPLETE_PART_NO", OracleDbType.Varchar2).Value = completePartno.ToUpper() == "NULL" ? (object)DBNull.Value : completePartno;
                cmd.Parameters.Add("P_WIP_PART_NO", OracleDbType.Varchar2).Value = wipPartno.ToUpper() == "NULL" ? (object)DBNull.Value : wipPartno;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(new { status = "Done", message = "Done", set = DTFM.convertToList(ds) });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM152_GetData");
                return Json(new { status = "Error", message = ex.Message });
            }
        }

        [HttpPost, Route("SNM152_Insert")]
        [Authorize]
        public IHttpActionResult SNM152_Insert([FromBody] dynamic pParam)
        {
            try
            {
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();

                OracleCommand cmd = new OracleCommand();
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM152_INSERT";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_BASE_MODEL", OracleDbType.Varchar2).Value = pParam["BASE_MODEL"];
                cmd.Parameters.Add("P_OPTION_CODE", OracleDbType.Varchar2).Value = pParam["OPTION_CODE"];
                cmd.Parameters.Add("P_OPTION_DESC", OracleDbType.Varchar2).Value = pParam["OPTION_DESC"];
                cmd.Parameters.Add("P_ACCESSORY_CODE", OracleDbType.Varchar2).Value = pParam["ACCESSORY_CODE"];
                cmd.Parameters.Add("P_ACCESSORY_DESC", OracleDbType.Varchar2).Value = pParam["ACCESSORY_DESC"];
                cmd.Parameters.Add("P_OPTION_TYPE", OracleDbType.Varchar2).Value = pParam["OPTION_TYPE"];
                cmd.Parameters.Add("P_INSTALL_LOCATION_CODE", OracleDbType.Varchar2).Value = pParam["INSTALL_LOCATION_CODE"];
                cmd.Parameters.Add("P_HAS_LABOR_COST", OracleDbType.Varchar2).Value = pParam["HAS_LABOR_COST"];
                cmd.Parameters.Add("P_COMPLETE_PART_NO", OracleDbType.Varchar2).Value = pParam["COMPLETE_PART_NO"];
                cmd.Parameters.Add("P_WIP_PART_NO", OracleDbType.Varchar2).Value = pParam["WIP_PART_NO"];
                cmd.Parameters.Add("P_STATUS", OracleDbType.Varchar2).Value = pParam["STATUS"];
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 100).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string r_status = cmd.Parameters["R_STATUS"].Value.ToString();
                string r_message = cmd.Parameters["R_MESSAGE"].Value.ToString();

                return Json(new { status = r_status, message = r_message });
            }
            catch (Exception ex)
            {
                //--- Roll back ---
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM152_Insert");
                return Json(new { status = "Fail", message = ex.Message });
            }
        }

        [HttpPost, Route("SNM152_Update")]
        [Authorize]
        public IHttpActionResult SNM152_Update([FromBody] dynamic pParam)
        {
            try
            {
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();

                OracleCommand cmd = new OracleCommand();
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM152_UPDATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_BASE_MODEL", OracleDbType.Varchar2).Value = pParam["BASE_MODEL"];
                cmd.Parameters.Add("P_OPTION_CODE", OracleDbType.Varchar2).Value = pParam["OPTION_CODE"];
                cmd.Parameters.Add("P_OPTION_DESC", OracleDbType.Varchar2).Value = pParam["OPTION_DESC"];
                cmd.Parameters.Add("P_ACCESSORY_CODE", OracleDbType.Varchar2).Value = pParam["ACCESSORY_CODE"];
                cmd.Parameters.Add("P_ACCESSORY_DESC", OracleDbType.Varchar2).Value = pParam["ACCESSORY_DESC"];
                cmd.Parameters.Add("P_OPTION_TYPE", OracleDbType.Varchar2).Value = pParam["OPTION_TYPE"];
                cmd.Parameters.Add("P_INSTALL_LOCATION_CODE", OracleDbType.Varchar2).Value = pParam["INSTALL_LOCATION_CODE"];
                cmd.Parameters.Add("P_HAS_LABOR_COST", OracleDbType.Varchar2).Value = pParam["HAS_LABOR_COST"];
                cmd.Parameters.Add("P_COMPLETE_PART_NO", OracleDbType.Varchar2).Value = pParam["COMPLETE_PART_NO"];
                cmd.Parameters.Add("P_WIP_PART_NO", OracleDbType.Varchar2).Value = pParam["WIP_PART_NO"];
                cmd.Parameters.Add("P_STATUS", OracleDbType.Varchar2).Value = pParam["STATUS"];
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 100).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string r_status = cmd.Parameters["R_STATUS"].Value.ToString();
                string r_message = cmd.Parameters["R_MESSAGE"].Value.ToString();

                return Json(new { status = r_status, message = r_message });
            }
            catch (Exception ex)
            {
                //--- Roll back ---
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM152_Update");
                return Json(new { status = "Fail", message = ex.Message });
            }
        }

        [HttpPost, Route("SNM152_Delete")]
        [Authorize]
        public IHttpActionResult SNM152_Delete([FromBody] dynamic pParam)
        {
            try
            {
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();

                OracleCommand cmd = new OracleCommand();
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM152_DELETE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_BASE_MODEL", OracleDbType.Varchar2).Value = pParam["BASE_MODEL"];
                cmd.Parameters.Add("P_OPTION_CODE", OracleDbType.Varchar2).Value = pParam["OPTION_CODE"];
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 100).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string r_status = cmd.Parameters["R_STATUS"].Value.ToString();
                string r_message = cmd.Parameters["R_MESSAGE"].Value.ToString();

                return Json(new { status = r_status, message = r_message });
            }
            catch (Exception ex)
            {
                //--- Roll back ---
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM152_Delete");
                return Json(new { status = "Fail", message = ex.Message });
            }
        }

        [HttpPost, Route("SNM152_UploadFile")]
        [Authorize]
        public async Task<IHttpActionResult> SNM152_UploadFile(HttpRequestMessage request)
        {
            if (!request.Content.IsMimeMultipartContent())
            {
                throw new HttpResponseException(HttpStatusCode.UnsupportedMediaType);
            }
            var reqUpload = await request.Content.ReadAsMultipartAsync();
            try
            {
                OracleCommand cmd = new OracleCommand();
                SystemApiController sys = new SystemApiController();
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();
                DataTable dt_file = new DataTable();
                DataColumn dc = null;
                var file = reqUpload.Contents[0];
                var buffer = await file.ReadAsByteArrayAsync();
                MemoryStream stream = new MemoryStream(buffer);
                dt_file = sys.ReadDataExcel(stream);
                int colCount = dt_file.Columns.Count;
                string sql = @"SELECT * FROM TPM_NP_PDI_OPTION_HEADER_TEMPLATE";
                dc = new DataColumn("UPLOAD_USER", typeof(String));
                dc.DefaultValue = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                dt_file.Columns.Add(dc);
                dc = new DataColumn("COMPANY_CODE", typeof(String));
                dc.DefaultValue = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                dt_file.Columns.Add(dc);
                dt_file.Columns.Add("SEQ");
                DataView dv = dt_file.AsDataView();
                dt = dv.ToTable();

                for (int i = 0; i < dt.Rows.Count; i++)
                {
                    dt.Rows[i]["SEQ"] = i + 1;
                }

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM152_CLEAR_TEMP";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                oOracledb.SqlExecute_CMD(cmd);
                int count = oOracledb.InsertDataTable(dt, sql);
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM152_VALIDATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(new { status = "Done", message = "Done", set = DTFM.convertToList(ds) });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM152_UploadFile");
                throw new HttpResponseException(HttpStatusCode.NotAcceptable);
            }
            throw new Exception("Something wrong");
        }

        [HttpPost, Route("SNM152_SaveByUpload")]
        [Authorize]
        public IHttpActionResult SNM152_SaveByUpload([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM152_SAVE_BY_UPLOAD";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 10).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 100).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);
                string r_status = cmd.Parameters["R_STATUS"].Value.ToString();
                string r_message = cmd.Parameters["R_MESSAGE"].Value.ToString();
                return Json(new { status = r_status, message = r_message });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM152_SaveByUpload");
                return Json(new { status = "Fail", message = ex.Message });
            }
        }

        #endregion

        #region SNM153

        [HttpGet, Route("SNM153_GetData/{optioncode}/{enditem18}/{vin}")]
        [Authorize]
        public IHttpActionResult SNM153_GetData(string optioncode, string enditem18, string vin)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM153_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_OPTION_CODE", OracleDbType.Varchar2).Value = optioncode.ToUpper() == "NULL" ? (object)DBNull.Value : optioncode;
                cmd.Parameters.Add("P_END_ITEM_18", OracleDbType.Varchar2).Value = enditem18.ToUpper() == "NULL" ? (object)DBNull.Value : enditem18;
                cmd.Parameters.Add("P_VIN", OracleDbType.Varchar2).Value = vin.ToUpper() == "NULL" ? (object)DBNull.Value : vin;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(new { status = "Done", message = "Done", set = DTFM.convertToList(ds) });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM153_GetData");
                return Json(new { status = "Error", message = ex.Message });
            }
        }

        [HttpPost, Route("SNM153_Insert")]
        [Authorize]
        public IHttpActionResult SNM153_Insert([FromBody] dynamic pParam)
        {
            try
            {
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();

                OracleCommand cmd = new OracleCommand();
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM153_INSERT";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_OPTION_CODE", OracleDbType.Varchar2).Value = pParam["OPTION_CODE"];
                cmd.Parameters.Add("P_END_ITEM_18", OracleDbType.Varchar2).Value = pParam["END_ITEM_18"];
                cmd.Parameters.Add("P_VIN", OracleDbType.Varchar2).Value = pParam["VIN"];
                cmd.Parameters.Add("P_WS_DLR_CODE", OracleDbType.Varchar2).Value = pParam["WS_DLR_CODE"];
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 100).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string r_status = cmd.Parameters["R_STATUS"].Value.ToString();
                string r_message = cmd.Parameters["R_MESSAGE"].Value.ToString();

                return Json(new { status = r_status, message = r_message });
            }
            catch (Exception ex)
            {
                //--- Roll back ---
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM153_Insert");
                return Json(new { status = "Fail", message = ex.Message });
            }
        }

        [HttpPost, Route("SNM153_Update")]
        [Authorize]
        public IHttpActionResult SNM153_Update([FromBody] dynamic pParam)
        {
            try
            {
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();

                OracleCommand cmd = new OracleCommand();
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM153_UPDATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ROWID", OracleDbType.Varchar2).Value = pParam["ID"];
                cmd.Parameters.Add("P_OPTION_CODE", OracleDbType.Varchar2).Value = pParam["OPTION_CODE"];
                cmd.Parameters.Add("P_END_ITEM_18", OracleDbType.Varchar2).Value = pParam["END_ITEM_18"];
                cmd.Parameters.Add("P_VIN", OracleDbType.Varchar2).Value = pParam["VIN"];
                cmd.Parameters.Add("P_WS_DLR_CODE", OracleDbType.Varchar2).Value = pParam["WS_DLR_CODE"];
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 100).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string r_status = cmd.Parameters["R_STATUS"].Value.ToString();
                string r_message = cmd.Parameters["R_MESSAGE"].Value.ToString();

                return Json(new { status = r_status, message = r_message });
            }
            catch (Exception ex)
            {
                //--- Roll back ---
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM153_Update");
                return Json(new { status = "Fail", message = ex.Message });
            }
        }

        [HttpPost, Route("SNM153_Delete")]
        [Authorize]
        public IHttpActionResult SNM153_Delete([FromBody] dynamic pParam)
        {
            try
            {
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();

                OracleCommand cmd = new OracleCommand();
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM153_DELETE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ROWID", OracleDbType.Varchar2).Value = pParam["ID"];
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 100).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string r_status = cmd.Parameters["R_STATUS"].Value.ToString();
                string r_message = cmd.Parameters["R_MESSAGE"].Value.ToString();

                return Json(new { status = r_status, message = r_message });
            }
            catch (Exception ex)
            {
                //--- Roll back ---
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM153_Delete");
                return Json(new { status = "Fail", message = ex.Message });
            }
        }

        [HttpPost, Route("SNM153_UploadFile")]
        [Authorize]
        public async Task<IHttpActionResult> SNM153_UploadFile(HttpRequestMessage request)
        {
            if (!request.Content.IsMimeMultipartContent())
            {
                throw new HttpResponseException(HttpStatusCode.UnsupportedMediaType);
            }
            var reqUpload = await request.Content.ReadAsMultipartAsync();
            try
            {
                OracleCommand cmd = new OracleCommand();
                SystemApiController sys = new SystemApiController();
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();
                DataTable dt_file = new DataTable();
                DataColumn dc = null;
                var file = reqUpload.Contents[0];
                var buffer = await file.ReadAsByteArrayAsync();
                MemoryStream stream = new MemoryStream(buffer);
                dt_file = sys.ReadDataExcel(stream);
                int colCount = dt_file.Columns.Count;
                string sql = @"SELECT * FROM TPM_NP_PDI_OPTION_SPECL_APPLY_TEMPLATE";
                dc = new DataColumn("UPLOAD_USER", typeof(String));
                dc.DefaultValue = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                dt_file.Columns.Add(dc);
                dc = new DataColumn("COMPANY_CODE", typeof(String));
                dc.DefaultValue = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                dt_file.Columns.Add(dc);
                dt_file.Columns.Add("SEQ");
                DataView dv = dt_file.AsDataView();
                dt = dv.ToTable();

                for (int i = 0; i < dt.Rows.Count; i++)
                {
                    dt.Rows[i]["SEQ"] = i + 1;
                }

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM153_CLEAR_TEMP";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                oOracledb.SqlExecute_CMD(cmd);
                int count = oOracledb.InsertDataTable(dt, sql);
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM153_VALIDATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(new { status = "Done", message = "Done", set = DTFM.convertToList(ds) });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM153_UploadFile");
                throw new HttpResponseException(HttpStatusCode.NotAcceptable);
            }
            throw new Exception("Something wrong");
        }

        [HttpPost, Route("SNM153_SaveByUpload")]
        [Authorize]
        public IHttpActionResult SNM153_SaveByUpload([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM153_SAVE_BY_UPLOAD";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 10).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 100).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);
                string r_status = cmd.Parameters["R_STATUS"].Value.ToString();
                string r_message = cmd.Parameters["R_MESSAGE"].Value.ToString();
                return Json(new { status = r_status, message = r_message });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM153_SaveByUpload");
                return Json(new { status = "Fail", message = ex.Message });
            }
        }

        #endregion

        #region SNM154

        [HttpGet, Route("SNM154_GetData/{optioncode}/{enditem18}/{destination}/{speccode}")]
        [Authorize]
        public IHttpActionResult SNM154_GetData(string optioncode, string enditem18, string destination, string speccode)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM154_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_OPTION_CODE", OracleDbType.Varchar2).Value = optioncode.ToUpper() == "NULL" ? (object)DBNull.Value : optioncode;
                cmd.Parameters.Add("P_END_ITEM_18", OracleDbType.Varchar2).Value = enditem18.ToUpper() == "NULL" ? (object)DBNull.Value : enditem18;
                cmd.Parameters.Add("P_DESTINATION", OracleDbType.Varchar2).Value = destination.ToUpper() == "NULL" ? (object)DBNull.Value : destination;
                cmd.Parameters.Add("P_SPEC_CODE", OracleDbType.Varchar2).Value = speccode.ToUpper() == "NULL" ? (object)DBNull.Value : speccode;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(new { status = "Done", message = "Done", set = DTFM.convertToList(ds) });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM154_GetData");
                return Json(new { status = "Error", message = ex.Message });
            }
        }

        [HttpGet, Route("SNM154_GetDestination/{destination}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM154_GetDestination(string destination, int noofshowrec)
        {
            DataSet ds = new DataSet();
            destination = ismoUtil.escapeCharacter(destination);
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM154_GET_DESTINATION";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DESTINATION", OracleDbType.Varchar2).Value = destination.ToUpper() == "NULL" ? (object)DBNull.Value : destination;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM154_GetDestination");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpGet, Route("SNM154_GetBaseModel/{basemodel}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM154_GetBaseModel(string basemodel, int noofshowrec)
        {
                DataSet ds = new DataSet();
            basemodel = ismoUtil.escapeCharacter(basemodel);
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM154_GET_BASE_MODEL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_BASE_MODEL", OracleDbType.Varchar2).Value = basemodel.ToUpper() == "NULL" ? (object)DBNull.Value : basemodel;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM154_GetBaseModel");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM154_Insert")]
        [Authorize]
        public IHttpActionResult SNM154_Insert([FromBody] dynamic pParam)
        {
            try
            {
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();

                OracleCommand cmd = new OracleCommand();
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM154_INSERT";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_OPTION_CODE", OracleDbType.Varchar2).Value = pParam["OPTION_CODE"];
                cmd.Parameters.Add("P_END_ITEM_18", OracleDbType.Varchar2).Value = pParam["END_ITEM_18"];
                cmd.Parameters.Add("P_DESTINATION", OracleDbType.Varchar2).Value = pParam["DESTINATION"];
                cmd.Parameters.Add("P_SPEC_CODE", OracleDbType.Varchar2).Value = pParam["SPEC_CODE"];
                cmd.Parameters.Add("P_BASE_MODEL", OracleDbType.Varchar2).Value = pParam["BASE_MODEL"];
                cmd.Parameters.Add("P_ADOPTED_DATE_FROM", OracleDbType.Date).Value = pParam["ADOPTED_DATE_FROM"] == null ? (object)DBNull.Value : Convert.ToDateTime(((string)pParam["ADOPTED_DATE_FROM"]).ToString());
                cmd.Parameters.Add("P_ADOPTED_DATE_TO", OracleDbType.Date).Value = pParam["ADOPTED_DATE_TO"] == null ? (object)DBNull.Value : Convert.ToDateTime(((string)pParam["ADOPTED_DATE_TO"]).ToString());
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 400).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string r_status = cmd.Parameters["R_STATUS"].Value.ToString();
                string r_message = cmd.Parameters["R_MESSAGE"].Value.ToString();

                return Json(new { status = r_status, message = r_message });
            }
            catch (Exception ex)
            {
                //--- Roll back ---
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM154_Insert");
                return Json(new { status = "Fail", message = ex.Message });
            }
        }

        [HttpPost, Route("SNM154_Update")]
        [Authorize]
        public IHttpActionResult SNM154_Update([FromBody] dynamic pParam)
        {
            try
            {
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();

                OracleCommand cmd = new OracleCommand();
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM154_UPDATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ROWID", OracleDbType.Varchar2).Value = pParam["ID"];
                cmd.Parameters.Add("P_OPTION_CODE", OracleDbType.Varchar2).Value = pParam["OPTION_CODE"];
                cmd.Parameters.Add("P_END_ITEM_18", OracleDbType.Varchar2).Value = pParam["END_ITEM_18"];
                cmd.Parameters.Add("P_DESTINATION", OracleDbType.Varchar2).Value = pParam["DESTINATION"];
                cmd.Parameters.Add("P_SPEC_CODE", OracleDbType.Varchar2).Value = pParam["SPEC_CODE"];
                cmd.Parameters.Add("P_BASE_MODEL", OracleDbType.Varchar2).Value = pParam["BASE_MODEL"];
                cmd.Parameters.Add("P_ADOPTED_DATE_FROM_AFTER", OracleDbType.Date).Value = pParam["ADOPTED_DATE_FROM"] == null ? (object)DBNull.Value : Convert.ToDateTime(((string)pParam["ADOPTED_DATE_FROM"]).ToString());
                cmd.Parameters.Add("P_ADOPTED_DATE_TO_AFTER", OracleDbType.Date).Value = pParam["ADOPTED_DATE_TO"] == null ? (object)DBNull.Value : Convert.ToDateTime(((string)pParam["ADOPTED_DATE_TO"]).ToString());
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 400).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string r_status = cmd.Parameters["R_STATUS"].Value.ToString();
                string r_message = cmd.Parameters["R_MESSAGE"].Value.ToString();

                return Json(new { status = r_status, message = r_message });
            }
            catch (Exception ex)
            {
                //--- Roll back ---
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM154_Update");
                return Json(new { status = "Fail", message = ex.Message });
            }
        }

        [HttpPost, Route("SNM154_Delete")]
        [Authorize]
        public IHttpActionResult SNM154_Delete([FromBody] dynamic pParam)
        {
            try
            {
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();

                OracleCommand cmd = new OracleCommand();
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM154_DELETE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ROWID", OracleDbType.Varchar2).Value = pParam["ID"];
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 100).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string r_status = cmd.Parameters["R_STATUS"].Value.ToString();
                string r_message = cmd.Parameters["R_MESSAGE"].Value.ToString();

                return Json(new { status = r_status, message = r_message });
            }
            catch (Exception ex)
            {
                //--- Roll back ---
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM154_Delete");
                return Json(new { status = "Fail", message = ex.Message });
            }
        }

        [HttpPost, Route("SNM154_UploadFile")]
        [Authorize]
        public async Task<IHttpActionResult> SNM154_UploadFile(HttpRequestMessage request)
        {
            if (!request.Content.IsMimeMultipartContent())
            {
                throw new HttpResponseException(HttpStatusCode.UnsupportedMediaType);
            }
            var reqUpload = await request.Content.ReadAsMultipartAsync();
            try
            {
                OracleCommand cmd = new OracleCommand();
                SystemApiController sys = new SystemApiController();
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();
                DataTable dt_file = new DataTable();
                DataColumn dc = null;
                var file = reqUpload.Contents[0];
                var buffer = await file.ReadAsByteArrayAsync();
                MemoryStream stream = new MemoryStream(buffer);
                dt_file = sys.ReadDataExcel(stream);
                int colCount = dt_file.Columns.Count;
                string sql = @"SELECT * FROM TPM_NP_PDI_OPTION_STD_APPLY_TEMPLATE";
                dc = new DataColumn("UPLOAD_USER", typeof(String));
                dc.DefaultValue = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                dt_file.Columns.Add(dc);
                dc = new DataColumn("COMPANY_CODE", typeof(String));
                dc.DefaultValue = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                dt_file.Columns.Add(dc);
                dt_file.Columns.Add("SEQ");
                DataView dv = dt_file.AsDataView();
                dt = dv.ToTable();

                for (int i = 0; i < dt.Rows.Count; i++)
                {
                    dt.Rows[i]["SEQ"] = i + 1;
                }

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM154_CLEAR_TEMP";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                oOracledb.SqlExecute_CMD(cmd);
                int count = oOracledb.InsertDataTable(dt, sql);
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM154_VALIDATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(new { status = "Done", message = "Done", set = DTFM.convertToList(ds) });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM154_UploadFile");
                throw new HttpResponseException(HttpStatusCode.NotAcceptable);
            }
            throw new Exception("Something wrong");
        }

        [HttpPost, Route("SNM154_SaveByUpload")]
        [Authorize]
        public IHttpActionResult SNM154_SaveByUpload([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM154_SAVE_BY_UPLOAD";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 10).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 100).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);
                string r_status = cmd.Parameters["R_STATUS"].Value.ToString();
                string r_message = cmd.Parameters["R_MESSAGE"].Value.ToString();
                return Json(new { status = r_status, message = r_message });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM154_SaveByUpload");
                return Json(new { status = "Fail", message = ex.Message });
            }
        }

        #endregion

        #region SNM155

        [HttpGet, Route("SNM155_GetData/{pdilocationcode}")]
        [Authorize]
        public IHttpActionResult SNM155_GetData(string pdilocationcode)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM155_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PDI_LOCATION_CODE", OracleDbType.Varchar2).Value = pdilocationcode.ToUpper() == "NULL" ? (object)DBNull.Value : pdilocationcode;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(new { status = "Done", message = "Done", set = DTFM.convertToList(ds) });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM155_GetData");
                return Json(new { status = "Error", message = ex.Message });
            }
        }

        [HttpPost, Route("SNM155_Insert")]
        [Authorize]
        public IHttpActionResult SNM155_Insert([FromBody] dynamic pParam)
        {
            try
            {
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();

                OracleCommand cmd = new OracleCommand();
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM155_INSERT";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PDI_LOCATION_CODE", OracleDbType.Varchar2).Value = pParam["PDI_LOCATION_CODE"];
                cmd.Parameters.Add("P_PDI_LOCATION_DESC", OracleDbType.Varchar2).Value = pParam["PDI_LOCATION_DESC"];
                cmd.Parameters.Add("P_VENDOR_CODE_MAPPING", OracleDbType.Varchar2).Value = pParam["VENDOR_CODE_MAPPING"];
                cmd.Parameters.Add("P_REASON_CODE_MAPPING", OracleDbType.Varchar2).Value = pParam["REASON_CODE_MAPPING"];
                cmd.Parameters.Add("P_VLM_SUB_INVENTORY_CODE", OracleDbType.Varchar2).Value = pParam["VLM_SUB_INVENTORY_CODE"];
                cmd.Parameters.Add("P_VLM_LOCATION_CODE", OracleDbType.Varchar2).Value = pParam["VLM_LOCATION_CODE"];
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 100).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string r_status = cmd.Parameters["R_STATUS"].Value.ToString();
                string r_message = cmd.Parameters["R_MESSAGE"].Value.ToString();

                return Json(new { status = r_status, message = r_message });
            }
            catch (Exception ex)
            {
                //--- Roll back ---
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM155_Insert");
                return Json(new { status = "Fail", message = ex.Message });
            }
        }

        [HttpPost, Route("SNM155_Update")]
        [Authorize]
        public IHttpActionResult SNM155_Update([FromBody] dynamic pParam)
        {
            try
            {
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();

                OracleCommand cmd = new OracleCommand();
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM155_UPDATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PDI_LOCATION_CODE", OracleDbType.Varchar2).Value = pParam["PDI_LOCATION_CODE"];
                cmd.Parameters.Add("P_PDI_LOCATION_DESC", OracleDbType.Varchar2).Value = pParam["PDI_LOCATION_DESC"];
                cmd.Parameters.Add("P_VENDOR_CODE_MAPPING", OracleDbType.Varchar2).Value = pParam["VENDOR_CODE_MAPPING"];
                cmd.Parameters.Add("P_REASON_CODE_MAPPING", OracleDbType.Varchar2).Value = pParam["REASON_CODE_MAPPING"];
                cmd.Parameters.Add("P_VLM_SUB_INVENTORY_CODE", OracleDbType.Varchar2).Value = pParam["VLM_SUB_INVENTORY_CODE"];
                cmd.Parameters.Add("P_VLM_LOCATION_CODE", OracleDbType.Varchar2).Value = pParam["VLM_LOCATION_CODE"];
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 100).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string r_status = cmd.Parameters["R_STATUS"].Value.ToString();
                string r_message = cmd.Parameters["R_MESSAGE"].Value.ToString();

                return Json(new { status = r_status, message = r_message });
            }
            catch (Exception ex)
            {
                //--- Roll back ---
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM155_Update");
                return Json(new { status = "Fail", message = ex.Message });
            }
        }

        [HttpPost, Route("SNM155_Delete")]
        [Authorize]
        public IHttpActionResult SNM155_Delete([FromBody] dynamic pParam)
        {
            try
            {
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();

                OracleCommand cmd = new OracleCommand();
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM155_DELETE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PDI_LOCATION_CODE", OracleDbType.Varchar2).Value = pParam["PDI_LOCATION_CODE"];
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 100).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string r_status = cmd.Parameters["R_STATUS"].Value.ToString();
                string r_message = cmd.Parameters["R_MESSAGE"].Value.ToString();

                return Json(new { status = r_status, message = r_message });
            }
            catch (Exception ex)
            {
                //--- Roll back ---
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM155_Delete");
                return Json(new { status = "Fail", message = ex.Message });
            }
        }

        [HttpPost, Route("SNM155_UploadFile")]
        [Authorize]
        public async Task<IHttpActionResult> SNM155_UploadFile(HttpRequestMessage request)
        {
            if (!request.Content.IsMimeMultipartContent())
            {
                throw new HttpResponseException(HttpStatusCode.UnsupportedMediaType);
            }
            var reqUpload = await request.Content.ReadAsMultipartAsync();
            try
            {
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();
                string SPPS_CODE = string.Empty;
                var file = reqUpload.Contents[0];
                var buffer = await file.ReadAsByteArrayAsync();
                MemoryStream stream = new MemoryStream(buffer);
                dt = SNM155_ValidateUpload(stream);
                stream.Flush();
                stream.Dispose();
                return Json(DTFM.convertToList(dt));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM155_UploadFile");
                throw new HttpResponseException(HttpStatusCode.NotAcceptable);
            }
            throw new Exception("Something wrong");
        }

        public DataTable SNM155_ValidateUpload(MemoryStream fileSteam)
        {
            DataTable dt = new DataTable();
            Hashtable hTable = new Hashtable();
            SystemApiController sys = new SystemApiController();
            dt = sys.ReadDataExcel(fileSteam);
            dt.Columns.Add("SEQ");
            dt.Columns.Add("ERR_FLAG");
            dt.Columns.Add("ERR_MSG");
            int iCount = 0;
            string key = string.Empty;
            OracleCommand cmd = new OracleCommand();
            foreach (DataRow row in dt.Rows)
            {
                iCount++;
                row["SEQ"] = iCount;
                key = row["PDI_LOCATION_CODE"].ToString().Trim();
                if (hTable.Contains(key))
                {
                    row["ERR_FLAG"] = "Y";
                    row["ERR_MSG"] = "Duplicate PDI Location code at Seq " + hTable[key];
                }
                else
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNM155_VALIDATE";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_PDI_LOCATION_CODE", OracleDbType.Varchar2).Value = row["PDI_LOCATION_CODE"].ToString().Trim();
                    cmd.Parameters.Add("P_PDI_LOCATION_DESC", OracleDbType.Varchar2).Value = row["PDI_LOCATION_DESC"].ToString().Trim();
                    cmd.Parameters.Add("P_VENDOR_CODE_MAPPING", OracleDbType.Varchar2).Value = row["VENDOR_CODE_MAPPING"].ToString().Trim();
                    cmd.Parameters.Add("P_REASON_CODE_MAPPING", OracleDbType.Varchar2).Value = row["REASON_CODE_MAPPING"].ToString().Trim();
                    cmd.Parameters.Add("P_VLM_SUB_INVENTORY_CODE", OracleDbType.Varchar2).Value = row["VLM_SUB_INVENTORY_CODE"].ToString().Trim();
                    cmd.Parameters.Add("P_VLM_LOCATION_CODE", OracleDbType.Varchar2).Value = row["VLM_LOCATION_CODE"].ToString().Trim();
                    cmd.Parameters.Add("R_ERR_FLAG", OracleDbType.Varchar2, 1).Direction = ParameterDirection.Output;
                    cmd.Parameters.Add("R_ERR_MSG", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;
                    cmd = oOracledb.SqlQueryReturnValue(cmd);

                    row["ERR_FLAG"] = cmd.Parameters["R_ERR_FLAG"].Value.ToString();
                    row["ERR_MSG"] = cmd.Parameters["R_ERR_MSG"].Value.ToString().ToUpper() == "NULL" ? "" : cmd.Parameters["R_ERR_MSG"].Value.ToString();
                    hTable.Add(key, iCount);
                }
            }
            return dt;
        }

        [HttpPost, Route("SNM155_SaveByUpload")]
        [Authorize]
        public IHttpActionResult SNM155_SaveByUpload([FromBody] dynamic[] pData)
        {
            try
            {
                var json = JsonConvert.SerializeObject(pData);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM155_SAVE_BY_UPLOAD";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_JSON_DATA", OracleDbType.Varchar2).Value = json;
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 10).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 100).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);
                string r_status = cmd.Parameters["R_STATUS"].Value.ToString();
                string r_message = cmd.Parameters["R_MESSAGE"].Value.ToString();
                return Json(new { status = r_status, message = r_message });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM155_SaveByUpload");
                return Json(new { status = "Fail", message = ex.Message });
            }
        }

        #endregion

        #region SNM156

        [HttpGet, Route("SNM156_GetData/{destinationcode}/{destinationdesc}")]
        [Authorize]
        public IHttpActionResult SNM156_GetData(string destinationcode, string destinationdesc)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM156_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DESTINATION_CODE", OracleDbType.Varchar2).Value = destinationcode.ToUpper() == "NULL" ? (object)DBNull.Value : destinationcode;
                cmd.Parameters.Add("P_DESTINATION_DESC", OracleDbType.Varchar2).Value = destinationdesc.ToUpper() == "NULL" ? (object)DBNull.Value : destinationdesc;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(new { status = "Done", message = "Done", set = DTFM.convertToList(ds) });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM156_GetData");
                return Json(new { status = "Error", message = ex.Message });
            }
        }

        [HttpPost, Route("SNM156_Insert")]
        [Authorize]
        public IHttpActionResult SNM156_Insert([FromBody] dynamic pParam)
        {
            try
            {
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();

                OracleCommand cmd = new OracleCommand();
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM156_INSERT";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DESTINATION_CODE", OracleDbType.Varchar2).Value = pParam["DESTINATION_CODE"];
                cmd.Parameters.Add("P_DESTINATION_DESC", OracleDbType.Varchar2).Value = pParam["DESTINATION_DESC"];
                cmd.Parameters.Add("P_OVERSEA_FLAG", OracleDbType.Varchar2).Value = pParam["OVERSEA_FLAG"];
                cmd.Parameters.Add("P_REGION", OracleDbType.Varchar2).Value = pParam["REGION"];
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 100).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string r_status = cmd.Parameters["R_STATUS"].Value.ToString();
                string r_message = cmd.Parameters["R_MESSAGE"].Value.ToString();

                return Json(new { status = r_status, message = r_message });
            }
            catch (Exception ex)
            {
                //--- Roll back ---
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM156_Insert");
                return Json(new { status = "Fail", message = ex.Message });
            }
        }

        [HttpPost, Route("SNM156_Update")]
        [Authorize]
        public IHttpActionResult SNM156_Update([FromBody] dynamic pParam)
        {
            try
            {
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();

                OracleCommand cmd = new OracleCommand();
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM156_UPDATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DESTINATION_CODE", OracleDbType.Varchar2).Value = pParam["DESTINATION_CODE"];
                cmd.Parameters.Add("P_DESTINATION_DESC", OracleDbType.Varchar2).Value = pParam["DESTINATION_DESC"];
                cmd.Parameters.Add("P_OVERSEA_FLAG", OracleDbType.Varchar2).Value = pParam["OVERSEA_FLAG"];
                cmd.Parameters.Add("P_REGION", OracleDbType.Varchar2).Value = pParam["REGION"];
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 100).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string r_status = cmd.Parameters["R_STATUS"].Value.ToString();
                string r_message = cmd.Parameters["R_MESSAGE"].Value.ToString();

                return Json(new { status = r_status, message = r_message });
            }
            catch (Exception ex)
            {
                //--- Roll back ---
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM156_Update");
                return Json(new { status = "Fail", message = ex.Message });
            }
        }

        [HttpPost, Route("SNM156_Delete")]
        [Authorize]
        public IHttpActionResult SNM156_Delete([FromBody] dynamic pParam)
        {
            try
            {
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();

                OracleCommand cmd = new OracleCommand();
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM156_DELETE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DESTINATION_CODE", OracleDbType.Varchar2).Value = pParam["DESTINATION_CODE"];
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 100).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string r_status = cmd.Parameters["R_STATUS"].Value.ToString();
                string r_message = cmd.Parameters["R_MESSAGE"].Value.ToString();

                return Json(new { status = r_status, message = r_message });
            }
            catch (Exception ex)
            {
                //--- Roll back ---
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM156_Delete");
                return Json(new { status = "Fail", message = ex.Message });
            }
        }

        [HttpPost, Route("SNM156_UploadFile")]
        [Authorize]
        public async Task<IHttpActionResult> SNM156_UploadFile(HttpRequestMessage request)
        {
            if (!request.Content.IsMimeMultipartContent())
            {
                throw new HttpResponseException(HttpStatusCode.UnsupportedMediaType);
            }
            var reqUpload = await request.Content.ReadAsMultipartAsync();
            try
            {
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();
                string SPPS_CODE = string.Empty;
                var file = reqUpload.Contents[0];
                var buffer = await file.ReadAsByteArrayAsync();
                MemoryStream stream = new MemoryStream(buffer);
                dt = SNM156_ValidateUpload(stream);
                stream.Flush();
                stream.Dispose();
                return Json(DTFM.convertToList(dt));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM156_UploadFile");
                throw new HttpResponseException(HttpStatusCode.NotAcceptable);
            }
            throw new Exception("Something wrong");
        }

        public DataTable SNM156_ValidateUpload(MemoryStream fileSteam)
        {
            DataTable dt = new DataTable();
            Hashtable hTable = new Hashtable();
            SystemApiController sys = new SystemApiController();
            dt = sys.ReadDataExcel(fileSteam);
            dt.Columns.Add("SEQ");
            dt.Columns.Add("ERR_FLAG");
            dt.Columns.Add("ERR_MSG");
            int iCount = 0;
            string key = string.Empty;
            OracleCommand cmd = new OracleCommand();
            foreach (DataRow row in dt.Rows)
            {
                iCount++;
                row["SEQ"] = iCount;
                key = row["DESTINATION_CODE"].ToString().Trim();
                if (hTable.Contains(key))
                {
                    row["ERR_FLAG"] = "Y";
                    row["ERR_MSG"] = "Duplicate Destination code at Seq " + hTable[key];
                }
                else
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNM156_VALIDATE";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_DESTINATION_CODE", OracleDbType.Varchar2).Value = row["DESTINATION_CODE"].ToString().Trim();
                    cmd.Parameters.Add("P_DESTINATION_DESC", OracleDbType.Varchar2).Value = row["DESTINATION_DESC"].ToString().Trim();
                    cmd.Parameters.Add("P_OVERSEA_FLAG", OracleDbType.Varchar2).Value = row["OVERSEA_FLAG"].ToString().Trim();
                    cmd.Parameters.Add("P_REGION", OracleDbType.Varchar2).Value = row["REGION"].ToString().Trim();
                    cmd.Parameters.Add("R_ERR_FLAG", OracleDbType.Varchar2, 1).Direction = ParameterDirection.Output;
                    cmd.Parameters.Add("R_ERR_MSG", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;
                    cmd = oOracledb.SqlQueryReturnValue(cmd);

                    row["ERR_FLAG"] = cmd.Parameters["R_ERR_FLAG"].Value.ToString();
                    row["ERR_MSG"] = cmd.Parameters["R_ERR_MSG"].Value.ToString().ToUpper() == "NULL" ? "" : cmd.Parameters["R_ERR_MSG"].Value.ToString();
                    hTable.Add(key, iCount);
                }
            }
            return dt;
        }

        [HttpPost, Route("SNM156_SaveByUpload")]
        [Authorize]
        public IHttpActionResult SNM156_SaveByUpload([FromBody] dynamic[] pData)
        {
            try
            {
                var json = JsonConvert.SerializeObject(pData);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM156_SAVE_BY_UPLOAD";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_JSON_DATA", OracleDbType.Varchar2).Value = json;
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 10).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 100).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);
                string r_status = cmd.Parameters["R_STATUS"].Value.ToString();
                string r_message = cmd.Parameters["R_MESSAGE"].Value.ToString();
                return Json(new { status = r_status, message = r_message });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM156_SaveByUpload");
                return Json(new { status = "Fail", message = ex.Message });
            }
        }

        #endregion

        #region SNM157

        [HttpGet, Route("SNM157_GetData/{chassisno}/{enditem18}/{optiontype}")]
        [Authorize]
        public IHttpActionResult SNM157_GetData(string chassisno, string enditem18, string optiontype)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM157_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CHASSIS_NO", OracleDbType.Varchar2).Value = chassisno.ToUpper() == "NULL" ? (object)DBNull.Value : chassisno;
                cmd.Parameters.Add("P_END_ITEM_18", OracleDbType.Varchar2).Value = enditem18.ToUpper() == "NULL" ? (object)DBNull.Value : enditem18;
                cmd.Parameters.Add("P_OPTION_TYPE", OracleDbType.Varchar2).Value = optiontype.ToUpper() == "NULL" ? (object)DBNull.Value : optiontype;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(new { status = "Done", message = "Done", set = DTFM.convertToList(ds) });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM157_GetData");
                return Json(new { status = "Error", message = ex.Message });
            }
        }

        [HttpPost, Route("SNM157_Insert")]
        [Authorize]
        public IHttpActionResult SNM157_Insert([FromBody] dynamic pParam)
        {
            try
            {
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();

                OracleCommand cmd = new OracleCommand();
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM157_INSERT";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CHASSIS_NO", OracleDbType.Varchar2).Value = pParam["CHASSIS_NO"];
                cmd.Parameters.Add("P_END_ITEM_18", OracleDbType.Varchar2).Value = pParam["END_ITEM_18"];
                cmd.Parameters.Add("P_OPTION_TYPE", OracleDbType.Varchar2).Value = pParam["OPTION_TYPE"];
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 100).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string r_status = cmd.Parameters["R_STATUS"].Value.ToString();
                string r_message = cmd.Parameters["R_MESSAGE"].Value.ToString();

                return Json(new { status = r_status, message = r_message });
            }
            catch (Exception ex)
            {
                //--- Roll back ---
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM157_Insert");
                return Json(new { status = "Fail", message = ex.Message });
            }
        }

        [HttpPost, Route("SNM157_Update")]
        [Authorize]
        public IHttpActionResult SNM157_Update([FromBody] dynamic pParam)
        {
            try
            {
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();

                OracleCommand cmd = new OracleCommand();
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM157_UPDATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CHASSIS_NO", OracleDbType.Varchar2).Value = pParam["CHASSIS_NO"];
                cmd.Parameters.Add("P_END_ITEM_18", OracleDbType.Varchar2).Value = pParam["END_ITEM_18"];
                cmd.Parameters.Add("P_OPTION_TYPE", OracleDbType.Varchar2).Value = pParam["OPTION_TYPE"];
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 100).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string r_status = cmd.Parameters["R_STATUS"].Value.ToString();
                string r_message = cmd.Parameters["R_MESSAGE"].Value.ToString();

                return Json(new { status = r_status, message = r_message });
            }
            catch (Exception ex)
            {
                //--- Roll back ---
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM157_Update");
                return Json(new { status = "Fail", message = ex.Message });
            }
        }

        [HttpPost, Route("SNM157_Delete")]
        [Authorize]
        public IHttpActionResult SNM157_Delete([FromBody] dynamic pParam)
        {
            try
            {
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();

                OracleCommand cmd = new OracleCommand();
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM157_DELETE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CHASSIS_NO", OracleDbType.Varchar2).Value = pParam["CHASSIS_NO"];
                cmd.Parameters.Add("P_END_ITEM_18", OracleDbType.Varchar2).Value = pParam["END_ITEM_18"];
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 100).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string r_status = cmd.Parameters["R_STATUS"].Value.ToString();
                string r_message = cmd.Parameters["R_MESSAGE"].Value.ToString();

                return Json(new { status = r_status, message = r_message });
            }
            catch (Exception ex)
            {
                //--- Roll back ---
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM157_Delete");
                return Json(new { status = "Fail", message = ex.Message });
            }
        }

        [HttpPost, Route("SNM157_UploadFile")]
        [Authorize]
        public async Task<IHttpActionResult> SNM157_UploadFile(HttpRequestMessage request)
        {
            if (!request.Content.IsMimeMultipartContent())
            {
                throw new HttpResponseException(HttpStatusCode.UnsupportedMediaType);
            }
            var reqUpload = await request.Content.ReadAsMultipartAsync();
            try
            {
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();
                string SPPS_CODE = string.Empty;
                var file = reqUpload.Contents[0];
                var buffer = await file.ReadAsByteArrayAsync();
                MemoryStream stream = new MemoryStream(buffer);
                dt = SNM157_ValidateUpload(stream);
                stream.Flush();
                stream.Dispose();
                return Json(DTFM.convertToList(dt));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM157_UploadFile");
                throw new HttpResponseException(HttpStatusCode.NotAcceptable);
            }
            throw new Exception("Something wrong");
        }

        public DataTable SNM157_ValidateUpload(MemoryStream fileSteam)
        {
            DataTable dt = new DataTable();
            Hashtable hTable = new Hashtable();
            SystemApiController sys = new SystemApiController();
            dt = sys.ReadDataExcel(fileSteam);
            dt.Columns.Add("SEQ");
            dt.Columns.Add("ERR_FLAG");
            dt.Columns.Add("ERR_MSG");
            int iCount = 0;
            string key = string.Empty;
            OracleCommand cmd = new OracleCommand();
            foreach (DataRow row in dt.Rows)
            {
                iCount++;
                row["SEQ"] = iCount;
                key = row["CHASSIS_NO"].ToString().Trim();
                if (hTable.Contains(key))
                {
                    row["ERR_FLAG"] = "Y";
                    row["ERR_MSG"] = "Duplicate Chassis no at Seq " + hTable[key];
                }
                else
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNM157_VALIDATE";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CHASSIS_NO", OracleDbType.Varchar2).Value = row["CHASSIS_NO"].ToString().Trim();
                    cmd.Parameters.Add("P_END_ITEM_18", OracleDbType.Varchar2).Value = row["END_ITEM_18"].ToString().Trim();
                    cmd.Parameters.Add("P_OPTION_TYPE", OracleDbType.Varchar2).Value = row["OPTION_TYPE"].ToString().Trim();
                    cmd.Parameters.Add("R_ERR_FLAG", OracleDbType.Varchar2, 1).Direction = ParameterDirection.Output;
                    cmd.Parameters.Add("R_ERR_MSG", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;
                    cmd = oOracledb.SqlQueryReturnValue(cmd);

                    row["ERR_FLAG"] = cmd.Parameters["R_ERR_FLAG"].Value.ToString();
                    row["ERR_MSG"] = cmd.Parameters["R_ERR_MSG"].Value.ToString().ToUpper() == "NULL" ? "" : cmd.Parameters["R_ERR_MSG"].Value.ToString();
                    hTable.Add(key, iCount);
                }
            }
            return dt;
        }

        [HttpPost, Route("SNM157_SaveByUpload")]
        [Authorize]
        public IHttpActionResult SNM157_SaveByUpload([FromBody] dynamic[] pData)
        {
            try
            {
                var json = JsonConvert.SerializeObject(pData);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM157_SAVE_BY_UPLOAD";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_JSON_DATA", OracleDbType.Varchar2).Value = json;
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 10).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 100).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);
                string r_status = cmd.Parameters["R_STATUS"].Value.ToString();
                string r_message = cmd.Parameters["R_MESSAGE"].Value.ToString();
                return Json(new { status = r_status, message = r_message });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM157_SaveByUpload");
                return Json(new { status = "Fail", message = ex.Message });
            }
        }

        #endregion

        #region SNM158

        [HttpGet, Route("SNM158_GetData/{exyear}/{exno}/{plant}/{domexp}")]
        [Authorize]
        public IHttpActionResult SNM158_GetData(string exyear, string exno, string plant, string domexp)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM158_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_EX_YEAR", OracleDbType.Varchar2).Value = exyear.ToUpper() == "NULL" ? (object)DBNull.Value : exyear;
                cmd.Parameters.Add("P_EX_NO", OracleDbType.Varchar2).Value = exno.ToUpper() == "NULL" ? (object)DBNull.Value : exno;
                cmd.Parameters.Add("P_PLANT", OracleDbType.Varchar2).Value = plant.ToUpper() == "NULL" ? (object)DBNull.Value : plant;
                cmd.Parameters.Add("P_DOM_EXP", OracleDbType.Varchar2).Value = domexp.ToUpper() == "NULL" ? (object)DBNull.Value : domexp;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(new { status = "Done", message = "Done", set = DTFM.convertToList(ds) });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM158_GetData");
                return Json(new { status = "Error", message = ex.Message });
            }
        }

        [HttpPost, Route("SNM158_Insert")]
        [Authorize]
        public IHttpActionResult SNM158_Insert([FromBody] dynamic pParam)
        {
            try
            {
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();

                OracleCommand cmd = new OracleCommand();
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM158_INSERT";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_EX_YEAR", OracleDbType.Varchar2).Value = pParam["EX_YEAR"];
                cmd.Parameters.Add("P_EX_NO", OracleDbType.Varchar2).Value = pParam["EX_NO"];
                cmd.Parameters.Add("P_PLANT", OracleDbType.Varchar2).Value = pParam["PLANT"];
                cmd.Parameters.Add("P_DOM_EXP", OracleDbType.Varchar2).Value = pParam["DOM_EXP"];
                cmd.Parameters.Add("P_BUYER_CODE", OracleDbType.Varchar2).Value = pParam["BUYER_CODE"];
                cmd.Parameters.Add("P_MODEL_PREFIX", OracleDbType.Varchar2).Value = pParam["MODEL_PREFIX"];
                cmd.Parameters.Add("P_MODEL_BASE", OracleDbType.Varchar2).Value = pParam["MODEL_BASE"];
                cmd.Parameters.Add("P_MODEL_SUFFIX", OracleDbType.Varchar2).Value = pParam["MODEL_SUFFIX"];
                cmd.Parameters.Add("P_SPEC_CODE", OracleDbType.Varchar2).Value = pParam["SPEC_CODE"];
                cmd.Parameters.Add("P_PRODUCTION_BUYER", OracleDbType.Varchar2).Value = pParam["PRODUCTION_BUYER"];
                cmd.Parameters.Add("P_PRODUCTION_PLAN_TOTAL", OracleDbType.Varchar2).Value = pParam["PRODUCTION_PLAN_TOTAL"];
                cmd.Parameters.Add("P_LIST_SEQ", OracleDbType.Varchar2).Value = pParam["LIST_SEQ"];
                cmd.Parameters.Add("P_DIMENSIONS", OracleDbType.Varchar2).Value = pParam["DIMENSIONS"];
                cmd.Parameters.Add("P_ACCESSORY_CODE_M", OracleDbType.Varchar2).Value = pParam["ACCESSORY_CODE_M"];
                cmd.Parameters.Add("P_ACCESSORY_NAME_M", OracleDbType.Varchar2).Value = pParam["ACCESSORY_NAME_M"];
                cmd.Parameters.Add("P_ACCESSORY_CODE_N", OracleDbType.Varchar2).Value = pParam["ACCESSORY_CODE_N"];
                cmd.Parameters.Add("P_ACCESSORY_NAME_N", OracleDbType.Varchar2).Value = pParam["ACCESSORY_NAME_N"];
                cmd.Parameters.Add("P_ACCESSORY_CODE_O", OracleDbType.Varchar2).Value = pParam["ACCESSORY_CODE_O"];
                cmd.Parameters.Add("P_ACCESSORY_NAME_O", OracleDbType.Varchar2).Value = pParam["ACCESSORY_NAME_O"];
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 400).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string r_status = cmd.Parameters["R_STATUS"].Value.ToString();
                string r_message = cmd.Parameters["R_MESSAGE"].Value.ToString();

                return Json(new { status = r_status, message = r_message });
            }
            catch (Exception ex)
            {
                //--- Roll back ---
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM158_Insert");
                return Json(new { status = "Fail", message = ex.Message });
            }
        }

        [HttpPost, Route("SNM158_Update")]
        [Authorize]
        public IHttpActionResult SNM158_Update([FromBody] dynamic pParam)
        {
            try
            {
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();

                OracleCommand cmd = new OracleCommand();
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM158_UPDATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ID", OracleDbType.Varchar2).Value = pParam["ID"];
                cmd.Parameters.Add("P_EX_YEAR", OracleDbType.Varchar2).Value = pParam["EX_YEAR"];
                cmd.Parameters.Add("P_EX_NO", OracleDbType.Varchar2).Value = pParam["EX_NO"];
                cmd.Parameters.Add("P_PLANT", OracleDbType.Varchar2).Value = pParam["PLANT"];
                cmd.Parameters.Add("P_DOM_EXP", OracleDbType.Varchar2).Value = pParam["DOM_EXP"];
                cmd.Parameters.Add("P_BUYER_CODE", OracleDbType.Varchar2).Value = pParam["BUYER_CODE"];
                cmd.Parameters.Add("P_MODEL_PREFIX", OracleDbType.Varchar2).Value = pParam["MODEL_PREFIX"];
                cmd.Parameters.Add("P_MODEL_BASE", OracleDbType.Varchar2).Value = pParam["MODEL_BASE"];
                cmd.Parameters.Add("P_MODEL_SUFFIX", OracleDbType.Varchar2).Value = pParam["MODEL_SUFFIX"];
                cmd.Parameters.Add("P_SPEC_CODE", OracleDbType.Varchar2).Value = pParam["SPEC_CODE"];
                cmd.Parameters.Add("P_PRODUCTION_BUYER", OracleDbType.Varchar2).Value = pParam["PRODUCTION_BUYER"];
                cmd.Parameters.Add("P_PRODUCTION_PLAN_TOTAL", OracleDbType.Varchar2).Value = pParam["PRODUCTION_PLAN_TOTAL"];
                cmd.Parameters.Add("P_LIST_SEQ", OracleDbType.Varchar2).Value = pParam["LIST_SEQ"];
                cmd.Parameters.Add("P_DIMENSIONS", OracleDbType.Varchar2).Value = pParam["DIMENSIONS"];
                cmd.Parameters.Add("P_ACCESSORY_CODE_M", OracleDbType.Varchar2).Value = pParam["ACCESSORY_CODE_M"];
                cmd.Parameters.Add("P_ACCESSORY_NAME_M", OracleDbType.Varchar2).Value = pParam["ACCESSORY_NAME_M"];
                cmd.Parameters.Add("P_ACCESSORY_CODE_N", OracleDbType.Varchar2).Value = pParam["ACCESSORY_CODE_N"];
                cmd.Parameters.Add("P_ACCESSORY_NAME_N", OracleDbType.Varchar2).Value = pParam["ACCESSORY_NAME_N"];
                cmd.Parameters.Add("P_ACCESSORY_CODE_O", OracleDbType.Varchar2).Value = pParam["ACCESSORY_CODE_O"];
                cmd.Parameters.Add("P_ACCESSORY_NAME_O", OracleDbType.Varchar2).Value = pParam["ACCESSORY_NAME_O"];
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 400).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string r_status = cmd.Parameters["R_STATUS"].Value.ToString();
                string r_message = cmd.Parameters["R_MESSAGE"].Value.ToString();

                return Json(new { status = r_status, message = r_message });
            }
            catch (Exception ex)
            {
                //--- Roll back ---
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM158_Update");
                return Json(new { status = "Fail", message = ex.Message });
            }
        }

        [HttpPost, Route("SNM158_Delete")]
        [Authorize]
        public IHttpActionResult SNM158_Delete([FromBody] dynamic pParam)
        {
            try
            {
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();

                OracleCommand cmd = new OracleCommand();
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM158_DELETE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ID", OracleDbType.Varchar2).Value = pParam["ID"];
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 100).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string r_status = cmd.Parameters["R_STATUS"].Value.ToString();
                string r_message = cmd.Parameters["R_MESSAGE"].Value.ToString();

                return Json(new { status = r_status, message = r_message });
            }
            catch (Exception ex)
            {
                //--- Roll back ---
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM158_Delete");
                return Json(new { status = "Fail", message = ex.Message });
            }
        }

        [HttpPost, Route("SNM158_UploadFile")]
        [Authorize]
        public async Task<IHttpActionResult> SNM158_UploadFile(HttpRequestMessage request)
        {
            if (!request.Content.IsMimeMultipartContent())
            {
                throw new HttpResponseException(HttpStatusCode.UnsupportedMediaType);
            }
            var reqUpload = await request.Content.ReadAsMultipartAsync();
            try
            {
                OracleCommand cmd = new OracleCommand();
                SystemApiController sys = new SystemApiController();
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();
                DataTable dt_file = new DataTable();
                DataColumn dc = null;
                var file = reqUpload.Contents[0];
                var buffer = await file.ReadAsByteArrayAsync();
                MemoryStream stream = new MemoryStream(buffer);
                dt_file = sys.ReadDataExcel(stream);
                int colCount = dt_file.Columns.Count;
                string sql = @"SELECT * FROM TPM_NP_PDI_PRODUCTION_PLAN_UPLOAD_TEMPLATE";
                string column = @"EX_YEAR,EX_NO,PLANT,DOM_EXP
                            ,BUYER_CODE,MODEL_PREFIX,MODEL_BASE,MODEL_SUFFIX,SPEC_CODE
                            ,PRODUCTION_BUYER,PRODUCTION_PLAN_TOTAL,LIST_SEQ,DIMENSIONS
                            ,ACCESSORY_CODE_M,ACCESSORY_NAME_M
                            ,ACCESSORY_CODE_N,ACCESSORY_NAME_N
                            ,ACCESSORY_CODE_O,ACCESSORY_NAME_O";
                string[] array = column.Split(',');

                dc = new DataColumn("UPLOAD_USER", typeof(String));
                dc.DefaultValue = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                dt_file.Columns.Add(dc);
                dt_file.Columns.Add("SEQ");
                DataView dv = dt_file.AsDataView();
                dt = dv.ToTable();

                for (int i = 0; i < colCount; i++)
                {
                    dt.Columns[i].ColumnName = array[i].Trim();
                }
                for (int i = 0; i < dt.Rows.Count; i++)
                {
                    dt.Rows[i]["SEQ"] = i + 1;
                }
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM158_CLEAR_TEMP";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                oOracledb.SqlExecute_CMD(cmd);
                int count = oOracledb.InsertDataTable(dt, sql);
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM158_VALIDATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(new { status = "Done", message = "Done", set = DTFM.convertToList(ds) });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM158_UploadFile");
                throw new HttpResponseException(HttpStatusCode.NotAcceptable);
            }
            throw new Exception("Something wrong");
        }

        [HttpPost, Route("SNM158_SaveByUpload")]
        [Authorize]
        public IHttpActionResult SNM158_SaveByUpload([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM158_SAVE_BY_UPLOAD";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 10).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 100).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);
                string r_status = cmd.Parameters["R_STATUS"].Value.ToString();
                string r_message = cmd.Parameters["R_MESSAGE"].Value.ToString();
                return Json(new { status = r_status, message = r_message });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM158_SaveByUpload");
                return Json(new { status = "Fail", message = ex.Message });
            }
        }

        #endregion

        #region SNM159

        [HttpGet, Route("SNM159_GetData/{vin}/{exno}/{plant}/{domexp}/{chasssino}")]
        [Authorize]
        public IHttpActionResult SNM159_GetData(string vin, string exno, string plant, string domexp, string chasssino)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM159_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VIN", OracleDbType.Varchar2).Value = vin.ToUpper() == "NULL" ? (object)DBNull.Value : vin;
                cmd.Parameters.Add("P_EX_NO", OracleDbType.Varchar2).Value = exno.ToUpper() == "NULL" ? (object)DBNull.Value : exno;
                cmd.Parameters.Add("P_PLANT", OracleDbType.Varchar2).Value = plant.ToUpper() == "NULL" ? (object)DBNull.Value : plant;
                cmd.Parameters.Add("P_DOM_EXP", OracleDbType.Varchar2).Value = domexp.ToUpper() == "NULL" ? (object)DBNull.Value : domexp;
                cmd.Parameters.Add("P_CHASSIS_NO", OracleDbType.Varchar2).Value = chasssino.ToUpper() == "NULL" ? (object)DBNull.Value : chasssino;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(new { status = "Done", message = "Done", set = DTFM.convertToList(ds) });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM159_GetData");
                return Json(new { status = "Error", message = ex.Message });
            }
        }

        [HttpPost, Route("SNM159_Insert")]
        [Authorize]
        public IHttpActionResult SNM159_Insert([FromBody] dynamic pParam)
        {
            try
            {
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();

                OracleCommand cmd = new OracleCommand();
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM159_INSERT";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VIN", OracleDbType.Varchar2).Value = pParam["VIN"];
                cmd.Parameters.Add("P_EX_NO", OracleDbType.Varchar2).Value = pParam["EX_NO"];
                cmd.Parameters.Add("P_CHASSIS_NO", OracleDbType.Varchar2).Value = pParam["CHASSIS_NO"];
                cmd.Parameters.Add("P_DOM_EXP", OracleDbType.Varchar2).Value = pParam["DOM_EXP"];
                cmd.Parameters.Add("P_PLANT", OracleDbType.Varchar2).Value = pParam["PLANT"];
                cmd.Parameters.Add("P_MODEL_PREFIX", OracleDbType.Varchar2).Value = pParam["MODEL_PREFIX"];
                cmd.Parameters.Add("P_MODEL_BASE", OracleDbType.Varchar2).Value = pParam["MODEL_BASE"];
                cmd.Parameters.Add("P_MODEL_SUFFIX", OracleDbType.Varchar2).Value = pParam["MODEL_SUFFIX"];
                cmd.Parameters.Add("P_BUYER_CODE", OracleDbType.Varchar2).Value = pParam["BUYER_CODE"];
                cmd.Parameters.Add("P_SPEC_CODE", OracleDbType.Varchar2).Value = pParam["SPEC_CODE"];
                cmd.Parameters.Add("P_COLOR", OracleDbType.Varchar2).Value = pParam["COLOR"];
                cmd.Parameters.Add("P_ENGINE_TYPE", OracleDbType.Varchar2).Value = pParam["ENGINE_TYPE"];
                cmd.Parameters.Add("P_ENGINE_NO", OracleDbType.Varchar2).Value = pParam["ENGINE_NO"];
                cmd.Parameters.Add("P_STAMPING_MODEL", OracleDbType.Varchar2).Value = pParam["STAMPING_MODEL"];
                cmd.Parameters.Add("P_PST", OracleDbType.Varchar2).Value = pParam["PST"];
                cmd.Parameters.Add("P_CPST", OracleDbType.Varchar2).Value = pParam["CPST"];
                cmd.Parameters.Add("P_NG", OracleDbType.Varchar2).Value = pParam["NG"];
                cmd.Parameters.Add("P_VEHICLE_LIST", OracleDbType.Varchar2).Value = pParam["VEHICLE_LIST"];
                cmd.Parameters.Add("P_PLANNED_OFFLINE_DATE", OracleDbType.Varchar2).Value = pParam["PLANNED_OFFLINE_DATE"];
                cmd.Parameters.Add("P_OFFLINE_DATE", OracleDbType.Varchar2).Value = pParam["OFFLINE_DATE"];
                cmd.Parameters.Add("P_FINAL_OK_DATE", OracleDbType.Varchar2).Value = pParam["FINAL_OK_DATE"];
                cmd.Parameters.Add("P_PLANNED_RECEIVE_DATE", OracleDbType.Varchar2).Value = pParam["PLANNED_RECEIVE_DATE"];
                cmd.Parameters.Add("P_RECEIVE_DATE", OracleDbType.Varchar2).Value = pParam["RECEIVE_DATE"];
                cmd.Parameters.Add("P_GATE_OUT_DATE", OracleDbType.Varchar2).Value = pParam["GATE_OUT_DATE"];
                cmd.Parameters.Add("P_STORAGE_YARD_IN_DATE", OracleDbType.Varchar2).Value = pParam["STORAGE_YARD_IN_DATE"];
                cmd.Parameters.Add("P_SI_DATE", OracleDbType.Varchar2).Value = pParam["SI_DATE"];
                cmd.Parameters.Add("P_TRANS_SHIPMENT_2ND", OracleDbType.Varchar2).Value = pParam["TRANS_SHIPMENT_2ND"];
                cmd.Parameters.Add("P_LOCATION", OracleDbType.Varchar2).Value = pParam["LOCATION"];
                cmd.Parameters.Add("P_PLANNED_LOCATION", OracleDbType.Varchar2).Value = pParam["PLANNED_LOCATION"];
                cmd.Parameters.Add("P_OLD_LOCATION", OracleDbType.Varchar2).Value = pParam["OLD_LOCATION"];
                cmd.Parameters.Add("P_SHIPPING_SYMBOL", OracleDbType.Varchar2).Value = pParam["SHIPPING_SYMBOL"];
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = pParam["CASE_NO"];
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pParam["INVOICE_NO"];
                cmd.Parameters.Add("P_TRIAL_SYMBOL", OracleDbType.Varchar2).Value = pParam["TRIAL_SYMBOL"];
                cmd.Parameters.Add("P_OPERATION_NAME_M", OracleDbType.Varchar2).Value = pParam["OPERATION_NAME_M"];
                cmd.Parameters.Add("P_OPERATION_NAME_N", OracleDbType.Varchar2).Value = pParam["OPERATION_NAME_N"];
                cmd.Parameters.Add("P_OPERATION_NAME_O", OracleDbType.Varchar2).Value = pParam["OPERATION_NAME_O"];
                cmd.Parameters.Add("P_ACCESSORY_CODE_M", OracleDbType.Varchar2).Value = pParam["ACCESSORY_CODE_M"];
                cmd.Parameters.Add("P_ACCESSORY_CODE_N", OracleDbType.Varchar2).Value = pParam["ACCESSORY_CODE_N"];
                cmd.Parameters.Add("P_ACCESSORY_CODE_O", OracleDbType.Varchar2).Value = pParam["ACCESSORY_CODE_O"];
                cmd.Parameters.Add("P_ACCOUNT_CODE", OracleDbType.Varchar2).Value = pParam["ACCOUNT_CODE"];
                cmd.Parameters.Add("P_VESSEL_CODE", OracleDbType.Varchar2).Value = pParam["VESSEL_CODE"];
                cmd.Parameters.Add("P_VESSEL_NAME", OracleDbType.Varchar2).Value = pParam["VESSEL_NAME"];
                cmd.Parameters.Add("P_LOADING_DATE", OracleDbType.Varchar2).Value = pParam["LOADING_DATE"];
                cmd.Parameters.Add("P_INVOICE_ISSUE_NO", OracleDbType.Varchar2).Value = pParam["INVOICE_ISSUE_NO"];
                cmd.Parameters.Add("P_INVOICE_ISSUE_DATE", OracleDbType.Varchar2).Value = pParam["INVOICE_ISSUE_DATE"];
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 100).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string r_status = cmd.Parameters["R_STATUS"].Value.ToString();
                string r_message = cmd.Parameters["R_MESSAGE"].Value.ToString();

                return Json(new { status = r_status, message = r_message });
            }
            catch (Exception ex)
            {
                //--- Roll back ---
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM159_Insert");
                return Json(new { status = "Fail", message = ex.Message });
            }
        }

        [HttpPost, Route("SNM159_Update")]
        [Authorize]
        public IHttpActionResult SNM159_Update([FromBody] dynamic pParam)
        {
            try
            {
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();

                OracleCommand cmd = new OracleCommand();
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM159_UPDATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VIN", OracleDbType.Varchar2).Value = pParam["VIN"];
                cmd.Parameters.Add("P_EX_NO", OracleDbType.Varchar2).Value = pParam["EX_NO"];
                cmd.Parameters.Add("P_CHASSIS_NO", OracleDbType.Varchar2).Value = pParam["CHASSIS_NO"];
                cmd.Parameters.Add("P_DOM_EXP", OracleDbType.Varchar2).Value = pParam["DOM_EXP"];
                cmd.Parameters.Add("P_PLANT", OracleDbType.Varchar2).Value = pParam["PLANT"];
                cmd.Parameters.Add("P_MODEL_PREFIX", OracleDbType.Varchar2).Value = pParam["MODEL_PREFIX"];
                cmd.Parameters.Add("P_MODEL_BASE", OracleDbType.Varchar2).Value = pParam["MODEL_BASE"];
                cmd.Parameters.Add("P_MODEL_SUFFIX", OracleDbType.Varchar2).Value = pParam["MODEL_SUFFIX"];
                cmd.Parameters.Add("P_BUYER_CODE", OracleDbType.Varchar2).Value = pParam["BUYER_CODE"];
                cmd.Parameters.Add("P_SPEC_CODE", OracleDbType.Varchar2).Value = pParam["SPEC_CODE"];
                cmd.Parameters.Add("P_COLOR", OracleDbType.Varchar2).Value = pParam["COLOR"];
                cmd.Parameters.Add("P_ENGINE_TYPE", OracleDbType.Varchar2).Value = pParam["ENGINE_TYPE"];
                cmd.Parameters.Add("P_ENGINE_NO", OracleDbType.Varchar2).Value = pParam["ENGINE_NO"];
                cmd.Parameters.Add("P_STAMPING_MODEL", OracleDbType.Varchar2).Value = pParam["STAMPING_MODEL"];
                cmd.Parameters.Add("P_PST", OracleDbType.Varchar2).Value = pParam["PST"];
                cmd.Parameters.Add("P_CPST", OracleDbType.Varchar2).Value = pParam["CPST"];
                cmd.Parameters.Add("P_NG", OracleDbType.Varchar2).Value = pParam["NG"];
                cmd.Parameters.Add("P_VEHICLE_LIST", OracleDbType.Varchar2).Value = pParam["VEHICLE_LIST"];
                cmd.Parameters.Add("P_PLANNED_OFFLINE_DATE", OracleDbType.Varchar2).Value = pParam["PLANNED_OFFLINE_DATE"];
                cmd.Parameters.Add("P_OFFLINE_DATE", OracleDbType.Varchar2).Value = pParam["OFFLINE_DATE"];
                cmd.Parameters.Add("P_FINAL_OK_DATE", OracleDbType.Varchar2).Value = pParam["FINAL_OK_DATE"];
                cmd.Parameters.Add("P_PLANNED_RECEIVE_DATE", OracleDbType.Varchar2).Value = pParam["PLANNED_RECEIVE_DATE"];
                cmd.Parameters.Add("P_RECEIVE_DATE", OracleDbType.Varchar2).Value = pParam["RECEIVE_DATE"];
                cmd.Parameters.Add("P_GATE_OUT_DATE", OracleDbType.Varchar2).Value = pParam["GATE_OUT_DATE"];
                cmd.Parameters.Add("P_STORAGE_YARD_IN_DATE", OracleDbType.Varchar2).Value = pParam["STORAGE_YARD_IN_DATE"];
                cmd.Parameters.Add("P_SI_DATE", OracleDbType.Varchar2).Value = pParam["SI_DATE"]; 
                cmd.Parameters.Add("P_TRANS_SHIPMENT_2ND", OracleDbType.Varchar2).Value = pParam["TRANS_SHIPMENT_2ND"];
                cmd.Parameters.Add("P_LOCATION", OracleDbType.Varchar2).Value = pParam["LOCATION"];
                cmd.Parameters.Add("P_PLANNED_LOCATION", OracleDbType.Varchar2).Value = pParam["PLANNED_LOCATION"];
                cmd.Parameters.Add("P_OLD_LOCATION", OracleDbType.Varchar2).Value = pParam["OLD_LOCATION"];
                cmd.Parameters.Add("P_SHIPPING_SYMBOL", OracleDbType.Varchar2).Value = pParam["SHIPPING_SYMBOL"];
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = pParam["CASE_NO"];
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pParam["INVOICE_NO"];
                cmd.Parameters.Add("P_TRIAL_SYMBOL", OracleDbType.Varchar2).Value = pParam["TRIAL_SYMBOL"];
                cmd.Parameters.Add("P_OPERATION_NAME_M", OracleDbType.Varchar2).Value = pParam["OPERATION_NAME_M"];
                cmd.Parameters.Add("P_OPERATION_NAME_N", OracleDbType.Varchar2).Value = pParam["OPERATION_NAME_N"];
                cmd.Parameters.Add("P_OPERATION_NAME_O", OracleDbType.Varchar2).Value = pParam["OPERATION_NAME_O"];
                cmd.Parameters.Add("P_ACCESSORY_CODE_M", OracleDbType.Varchar2).Value = pParam["ACCESSORY_CODE_M"];
                cmd.Parameters.Add("P_ACCESSORY_CODE_N", OracleDbType.Varchar2).Value = pParam["ACCESSORY_CODE_N"];
                cmd.Parameters.Add("P_ACCESSORY_CODE_O", OracleDbType.Varchar2).Value = pParam["ACCESSORY_CODE_O"];
                cmd.Parameters.Add("P_ACCOUNT_CODE", OracleDbType.Varchar2).Value = pParam["ACCOUNT_CODE"];
                cmd.Parameters.Add("P_VESSEL_CODE", OracleDbType.Varchar2).Value = pParam["VESSEL_CODE"];
                cmd.Parameters.Add("P_VESSEL_NAME", OracleDbType.Varchar2).Value = pParam["VESSEL_NAME"];
                cmd.Parameters.Add("P_LOADING_DATE", OracleDbType.Varchar2).Value = pParam["LOADING_DATE"];
                cmd.Parameters.Add("P_INVOICE_ISSUE_NO", OracleDbType.Varchar2).Value = pParam["INVOICE_ISSUE_NO"];
                cmd.Parameters.Add("P_INVOICE_ISSUE_DATE", OracleDbType.Varchar2).Value = pParam["INVOICE_ISSUE_DATE"];
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 100).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string r_status = cmd.Parameters["R_STATUS"].Value.ToString();
                string r_message = cmd.Parameters["R_MESSAGE"].Value.ToString();

                return Json(new { status = r_status, message = r_message });
            }
            catch (Exception ex)
            {
                //--- Roll back ---
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM159_Update");
                return Json(new { status = "Fail", message = ex.Message });
            }
        }

        [HttpPost, Route("SNM159_Delete")]
        [Authorize]
        public IHttpActionResult SNM159_Delete([FromBody] dynamic pParam)
        {
            try
            {
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();

                OracleCommand cmd = new OracleCommand();
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM159_DELETE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VIN", OracleDbType.Varchar2).Value = pParam["VIN"];
                cmd.Parameters.Add("P_EX_NO", OracleDbType.Varchar2).Value = pParam["EX_NO"];
                cmd.Parameters.Add("P_PLANT", OracleDbType.Varchar2).Value = pParam["PLANT"];
                cmd.Parameters.Add("P_DOM_EXP", OracleDbType.Varchar2).Value = pParam["DOM_EXP"];
                cmd.Parameters.Add("P_CHASSIS_NO", OracleDbType.Varchar2).Value = pParam["CHASSIS_NO"];
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 100).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string r_status = cmd.Parameters["R_STATUS"].Value.ToString();
                string r_message = cmd.Parameters["R_MESSAGE"].Value.ToString();

                return Json(new { status = r_status, message = r_message });
            }
            catch (Exception ex)
            {
                //--- Roll back ---
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM159_Delete");
                return Json(new { status = "Fail", message = ex.Message });
            }
        }

        [HttpPost, Route("SNM159_UploadFile")]
        [Authorize]
        public async Task<IHttpActionResult> SNM159_UploadFile(HttpRequestMessage request)
        {
            if (!request.Content.IsMimeMultipartContent())
            {
                throw new HttpResponseException(HttpStatusCode.UnsupportedMediaType);
            }
            var reqUpload = await request.Content.ReadAsMultipartAsync();
            try
            {
                OracleCommand cmd = new OracleCommand();
                SystemApiController sys = new SystemApiController();
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();
                DataTable dt_file = new DataTable();
                DataColumn dc = null;  
                var file = reqUpload.Contents[0];
                var buffer = await file.ReadAsByteArrayAsync();
                MemoryStream stream = new MemoryStream(buffer);
                dt_file = sys.ReadDataExcel(stream);
                int colCount = dt_file.Columns.Count;
                string sql = @"SELECT * FROM TPM_NP_PDI_PRODUCTION_ACTUAL_UPLOAD_TEMPLATE";
                string column = @"VIN,EX_NO,CHASSIS_NO,DOM_EXP,PLANT
                    ,MODEL_PREFIX,MODEL_BASE,MODEL_SUFFIX
                    ,BUYER_CODE,SPEC_CODE,COLOR
                    ,ENGINE_TYPE,ENGINE_NO,STAMPING_MODEL
                    ,PST,CPST,NG,VEHICLE_LIST
                    ,PLANNED_OFFLINE_DATE,OFFLINE_DATE
                    ,FINAL_OK_DATE,PLANNED_RECEIVE_DATE
                    ,RECEIVE_DATE,GATE_OUT_DATE
                    ,STORAGE_YARD_IN_DATE,SI_DATE
                    ,TRANS_SHIPMENT_2ND
                    ,LOCATION,PLANNED_LOCATION,OLD_LOCATION
                    ,SHIPPING_SYMBOL,CASE_NO
                    ,INVOICE_NO,TRIAL_SYMBOL
                    ,OPERATION_NAME_M,OPERATION_NAME_N,OPERATION_NAME_O
                    ,ACCESSORY_CODE_M,ACCESSORY_CODE_N,ACCESSORY_CODE_O
                    ,ACCOUNT_CODE,VESSEL_CODE,VESSEL_NAME
                    ,LOADING_DATE,INVOICE_ISSUE_NO,INVOICE_ISSUE_DATE";
                string[] array = column.Split(',');

                dc = new DataColumn("UPLOAD_USER", typeof(String));
                dc.DefaultValue = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                dt_file.Columns.Add(dc);
                dt_file.Columns.Add("SEQ");
                DataView dv = dt_file.AsDataView();
                dt = dv.ToTable();

                for (int i = 0; i < colCount; i++)
                {
                    dt.Columns[i].ColumnName = array[i].Trim();
                }
                for (int i = 0; i < dt.Rows.Count; i++)
                {
                    dt.Rows[i]["SEQ"] = i + 1;
                }

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM159_CLEAR_TEMP";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                oOracledb.SqlExecute_CMD(cmd);

                int count = oOracledb.InsertDataTable(dt, sql);

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM159_VALIDATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(new { status = "Done", message = "Done", set = DTFM.convertToList(ds)});
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM159_UploadFile");
                throw new HttpResponseException(HttpStatusCode.NotAcceptable);
            }
            throw new Exception("Something wrong");
        }

        [HttpPost, Route("SNM159_SaveByUpload")]
        [Authorize]
        public IHttpActionResult SNM159_SaveByUpload([FromBody]dynamic pParam)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM159_SAVE_BY_UPLOAD";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 10).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 100).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);
                string r_status = cmd.Parameters["R_STATUS"].Value.ToString();
                string r_message = cmd.Parameters["R_MESSAGE"].Value.ToString();
                return Json(new { status = r_status, message = r_message });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM159_SaveByUpload");
                return Json(new { status = "Fail", message = ex.Message });
            }
        }

        #endregion

        #region SNM160
        [HttpGet, Route("SNM160_GetActivityCode/{activitycode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM160_GetActivityCode(string activitycode, int noofshowrec)
        {
            DataSet ds = new DataSet();
            activitycode = ismoUtil.escapeCharacter(activitycode);
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM160_GET_ACTIVITY_CODE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ACTIVITY_CODE", OracleDbType.Varchar2).Value = activitycode.ToUpper() == "NULL" ? (object)DBNull.Value : activitycode;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM160_GetActivityCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNM160_GetData/{activitycode}/{enditem18}/{vin}")]
        [Authorize]
        public IHttpActionResult SNM160_GetData(string activitycode, string enditem18, string vin)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM160_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ACTIVITY_CODE", OracleDbType.Varchar2).Value = activitycode.ToUpper() == "NULL" ? (object)DBNull.Value : activitycode;
                cmd.Parameters.Add("P_END_ITEM_18", OracleDbType.Varchar2).Value = enditem18.ToUpper() == "NULL" ? (object)DBNull.Value : enditem18;
                cmd.Parameters.Add("P_VIN", OracleDbType.Varchar2).Value = vin.ToUpper() == "NULL" ? (object)DBNull.Value : vin;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(new { status = "Done", message = "Done", set = DTFM.convertToList(ds) });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM160_GetData");
                return Json(new { status = "Error", message = ex.Message });
            }
        }

        [HttpPost, Route("SNM160_Insert")]
        [Authorize]
        public IHttpActionResult SNM160_Insert([FromBody] dynamic pParam)
        {
            try
            {
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();

                OracleCommand cmd = new OracleCommand();
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM160_INSERT";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ACTIVITY_CODE", OracleDbType.Varchar2).Value = pParam["ACTIVITY_CODE"];
                cmd.Parameters.Add("P_END_ITEM_18", OracleDbType.Varchar2).Value = pParam["END_ITEM_18"];
                cmd.Parameters.Add("P_VIN", OracleDbType.Varchar2).Value = pParam["VIN"];
                cmd.Parameters.Add("P_WS_DEALER_CODE", OracleDbType.Varchar2).Value = pParam["WS_DEALER_CODE"];
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 100).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string r_status = cmd.Parameters["R_STATUS"].Value.ToString();
                string r_message = cmd.Parameters["R_MESSAGE"].Value.ToString();

                return Json(new { status = r_status, message = r_message });
            }
            catch (Exception ex)
            {
                //--- Roll back ---
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM160_Insert");
                return Json(new { status = "Fail", message = ex.Message });
            }
        }

        [HttpPost, Route("SNM160_Update")]
        [Authorize]
        public IHttpActionResult SNM160_Update([FromBody] dynamic pParam)
        {
            try
            {
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();

                OracleCommand cmd = new OracleCommand();
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM160_UPDATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ROWID", OracleDbType.Varchar2).Value = pParam["ID"];
                cmd.Parameters.Add("P_ACTIVITY_CODE", OracleDbType.Varchar2).Value = pParam["ACTIVITY_CODE"];
                cmd.Parameters.Add("P_END_ITEM_18", OracleDbType.Varchar2).Value = pParam["END_ITEM_18"];
                cmd.Parameters.Add("P_VIN", OracleDbType.Varchar2).Value = pParam["VIN"];
                cmd.Parameters.Add("P_WS_DEALER_CODE", OracleDbType.Varchar2).Value = pParam["WS_DEALER_CODE"];
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 100).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string r_status = cmd.Parameters["R_STATUS"].Value.ToString();
                string r_message = cmd.Parameters["R_MESSAGE"].Value.ToString();

                return Json(new { status = r_status, message = r_message });
            }
            catch (Exception ex)
            {
                //--- Roll back ---
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM160_Update");
                return Json(new { status = "Fail", message = ex.Message });
            }
        }

        [HttpPost, Route("SNM160_Delete")]
        [Authorize]
        public IHttpActionResult SNM160_Delete([FromBody] dynamic pParam)
        {
            try
            {
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();

                OracleCommand cmd = new OracleCommand();
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM160_DELETE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ROWID", OracleDbType.Varchar2).Value = pParam["ID"];
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 100).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string r_status = cmd.Parameters["R_STATUS"].Value.ToString();
                string r_message = cmd.Parameters["R_MESSAGE"].Value.ToString();

                return Json(new { status = r_status, message = r_message });
            }
            catch (Exception ex)
            {
                //--- Roll back ---
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM160_Delete");
                return Json(new { status = "Fail", message = ex.Message });
            }
        }

        [HttpPost, Route("SNM160_UploadFile")]
        [Authorize]
        public async Task<IHttpActionResult> SNM160_UploadFile(HttpRequestMessage request)
        {
            if (!request.Content.IsMimeMultipartContent())
            {
                throw new HttpResponseException(HttpStatusCode.UnsupportedMediaType);
            }
            var reqUpload = await request.Content.ReadAsMultipartAsync();
            try
            {

                OracleCommand cmd = new OracleCommand();
                SystemApiController sys = new SystemApiController();
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();
                DataTable dt_file = new DataTable();
                DataColumn dc = null;
                var file = reqUpload.Contents[0];
                var buffer = await file.ReadAsByteArrayAsync();
                MemoryStream stream = new MemoryStream(buffer);
                dt_file = sys.ReadDataExcel(stream);
                int colCount = dt_file.Columns.Count;
                string sql = @"SELECT * FROM TPM_NP_PDI_OPTION_ACT_APPLY_TEMPLATE";
                dc = new DataColumn("UPLOAD_USER", typeof(String));
                dc.DefaultValue = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                dt_file.Columns.Add(dc);
                dc = new DataColumn("COMPANY_CODE", typeof(String));
                dc.DefaultValue = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                dt_file.Columns.Add(dc);
                dt_file.Columns.Add("SEQ");
                DataView dv = dt_file.AsDataView();
                dt = dv.ToTable();

                for (int i = 0; i < dt.Rows.Count; i++)
                {
                    dt.Rows[i]["SEQ"] = i + 1;
                }

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM160_CLEAR_TEMP";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                oOracledb.SqlExecute_CMD(cmd);
                int count = oOracledb.InsertDataTable(dt, sql);
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM160_VALIDATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(new { status = "Done", message = "Done", set = DTFM.convertToList(ds) });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM160_UploadFile");
                throw new HttpResponseException(HttpStatusCode.NotAcceptable);
            }
            throw new Exception("Something wrong");
        }
        [HttpPost, Route("SNM160_SaveByUpload")]
        [Authorize]
        public IHttpActionResult SNM160_SaveByUpload([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM160_SAVE_BY_UPLOAD";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 10).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 100).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);
                string r_status = cmd.Parameters["R_STATUS"].Value.ToString();
                string r_message = cmd.Parameters["R_MESSAGE"].Value.ToString();
                return Json(new { status = r_status, message = r_message });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM160_SaveByUpload");
                return Json(new { status = "Fail", message = ex.Message });
            }
        }

        #endregion

        #region SNM162
        [HttpGet, Route("SNM162_GetEndItem18/{enditem18}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM162_GetEndItem18(string enditem18, int noofshowrec)
        {
            DataSet ds = new DataSet();
            enditem18 = ismoUtil.escapeCharacter(enditem18);
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM162_GET_END_ITEM_18";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_END_ITEM_18", OracleDbType.Varchar2).Value = enditem18.ToUpper() == "NULL" ? (object)DBNull.Value : enditem18;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM162_GetEndItem18");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpGet, Route("SNM162_GetEndItem18Sub/{enditem18}")]
        [Authorize]
        public IHttpActionResult SNM162_GetEndItem18Sub(string enditem18)
        {
            DataSet ds = new DataSet();
            enditem18 = ismoUtil.escapeCharacter(enditem18);
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM162_GET_END_ITEM_18_SUB";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_END_ITEM_18", OracleDbType.Varchar2).Value = enditem18.ToUpper() == "NULL" ? (object)DBNull.Value : enditem18;
                cmd.Parameters.Add("C_RETURN1", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN2", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN3", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM162_GetEndItem18Sub");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpGet, Route("SNM162_GetOptionApplied/{enditem18}/{destination}/{speccode}")]
        [Authorize]
        public IHttpActionResult SNM162_GetOptionApplied(string enditem18, string destination, string speccode)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM162_GET_OPTION_APPLIED";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_END_ITEM_18", OracleDbType.Varchar2).Value = enditem18.ToUpper() == "NULL" ? (object)DBNull.Value : enditem18;
                cmd.Parameters.Add("P_DESTINATION", OracleDbType.Varchar2).Value = destination.ToUpper() == "NULL" ? (object)DBNull.Value : destination;
                cmd.Parameters.Add("P_SPEC_CODE", OracleDbType.Varchar2).Value = speccode.ToUpper() == "NULL" ? (object)DBNull.Value : speccode;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(new { status = "Done", message = "Done", set = DTFM.convertToList(ds) });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM162_GetOptionApplied");
                return Json(new { status = "Error", message = ex.Message });
            }
        }
        [HttpGet, Route("SNM162_GetVINApplied/{optioncode}/{enditem18}")]
        [Authorize]
        public IHttpActionResult SNM162_GetVINApplied(string optioncode, string enditem18)
        {
            try
            {
            DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM162_GET_VIN_APPLIED";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_OPTION_CODE", OracleDbType.Varchar2).Value = optioncode.ToUpper() == "NULL" ? (object)DBNull.Value : optioncode;
                cmd.Parameters.Add("P_END_ITEM_18", OracleDbType.Varchar2).Value = enditem18.ToUpper() == "NULL" ? (object)DBNull.Value : enditem18;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(new { status = "Done", message = "Done", set = DTFM.convertToList(ds) });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM162_GetVINApplied");
                return Json(new { status = "Error", message = ex.Message });
            }
        }

        [HttpGet, Route("SNM162_GetVINActivity/{enditem18}")]
        [Authorize]
        public IHttpActionResult SNM162_GetVINActivity(string enditem18)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM162_GET_VIN_ACTIVITY";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_END_ITEM_18", OracleDbType.Varchar2).Value = enditem18.ToUpper() == "NULL" ? (object)DBNull.Value : enditem18;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(new { status = "Done", message = "Done", set = DTFM.convertToList(ds) });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM162_GetVINActivity");
                return Json(new { status = "Error", message = ex.Message });
            }
        }
        [HttpGet, Route("SNM162_GetBOMDetail/{optioncode}")]
        [Authorize]
        public IHttpActionResult SNM162_GetBOMDetail(string optioncode)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM162_GET_BOM_DETAIL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_OPTION_CODE", OracleDbType.Varchar2).Value = optioncode.ToUpper() == "NULL" ? (object)DBNull.Value : optioncode;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(new { status = "Done", message = "Done", set = DTFM.convertToList(ds) });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM162_GetBOMDetail");
                return Json(new { status = "Error", message = ex.Message });
            }
        }
        [HttpGet, Route("SNM162_GetVehicleStatus/{enditem18}")]
        [Authorize]
        public IHttpActionResult SNM162_GetVehicleStatus(string enditem18)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM162_GET_VEHICLE_STATUS";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_END_ITEM_18", OracleDbType.Varchar2).Value = enditem18.ToUpper() == "NULL" ? (object)DBNull.Value : enditem18;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(new { status = "Done", message = "Done", set = DTFM.convertToList(ds) });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM162_GetVehicleStatus");
                return Json(new { status = "Error", message = ex.Message });
            }
        }

        [HttpGet, Route("SNM162_GetPopUpComponent/{vin}")]
        [Authorize]
        public IHttpActionResult SNM162_GetPopUpComponent(string vin)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM162_GET_POPUP_COMPONENT";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VIN", OracleDbType.Varchar2).Value = vin.ToUpper() == "NULL" ? (object)DBNull.Value : vin;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(new { status = "Done", message = "Done", set = DTFM.convertToList(ds) });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM162_GetPopUpComponent");
                return Json(new { status = "Error", message = ex.Message });
            }
        }

        [HttpGet, Route("SNM162_GetPopUpWholeSales/{vin}")]
        [Authorize]
        public IHttpActionResult SNM162_GetPopUpWholeSales(string vin)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM162_GET_POPUP_WHOLESALES";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VIN", OracleDbType.Varchar2).Value = vin.ToUpper() == "NULL" ? (object)DBNull.Value : vin;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(new { status = "Done", message = "Done", set = DTFM.convertToList(ds) });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM162_GetPopUpWholeSales");
                return Json(new { status = "Error", message = ex.Message });
            }
        }
        [HttpGet, Route("SNM162_GetPopUpErrorLog/{vin}")]
        [Authorize]
        public IHttpActionResult SNM162_GetPopUpErrorLog(string vin)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM162_GET_POPUP_ERROR_LOG";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VIN", OracleDbType.Varchar2).Value = vin.ToUpper() == "NULL" ? (object)DBNull.Value : vin;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(new { status = "Done", message = "Done", set = DTFM.convertToList(ds) });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM162_GetPopUpErrorLog");
                return Json(new { status = "Error", message = ex.Message });
            }
        }
        #endregion
        #region SNM164
        [HttpGet, Route("SNM164_GetOrderTabData/{orderdate}/{optioncode}/{enditem18}/{vin}/{pmscustomer}/{vomstatus}")]
        [Authorize]
        public IHttpActionResult SNM164_GetOrderTabData(string orderdate, string optioncode, string enditem18, string vin, string pmscustomer, string vomstatus)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM164_GET_VOM_ORDER_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_OPTION_CODE", OracleDbType.Varchar2).Value = orderdate.ToUpper() == "NULL" ? (object)DBNull.Value : orderdate;
                cmd.Parameters.Add("P_ORDER_DATE", OracleDbType.Varchar2).Value = optioncode.ToUpper() == "NULL" ? (object)DBNull.Value : optioncode;
                cmd.Parameters.Add("P_END_ITEM_18", OracleDbType.Varchar2).Value = enditem18.ToUpper() == "NULL" ? (object)DBNull.Value : enditem18;
                cmd.Parameters.Add("P_VIN", OracleDbType.Varchar2).Value = vin.ToUpper() == "NULL" ? (object)DBNull.Value : vin;
                cmd.Parameters.Add("P_PMS_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pmscustomer.ToUpper() == "NULL" ? (object)DBNull.Value : pmscustomer;
                cmd.Parameters.Add("P_VOM_STATUS", OracleDbType.Varchar2).Value = vomstatus.ToUpper() == "NULL" ? (object)DBNull.Value : vomstatus;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN2", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(new { status = "Done", message = "Done", set = DTFM.convertToList(ds) });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM164_GetOrderTabData");
                return Json(new { status = "Error", message = ex.Message });
            }
        }
        [HttpGet, Route("SNM164_GetComponentTabData/{vin}/{optioncode}")]
        [Authorize]
        public IHttpActionResult SNM164_GetComponentTabData(string vin, string optioncode)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM164_GET_VOM_COMPONENT_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VIN", OracleDbType.Varchar2).Value = vin.ToUpper() == "NULL" ? (object)DBNull.Value : vin;
                cmd.Parameters.Add("P_OPTION_CODE", OracleDbType.Varchar2).Value = optioncode.ToUpper() == "NULL" ? (object)DBNull.Value : optioncode;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(new { status = "Done", message = "Done", set = DTFM.convertToList(ds) });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM164_GetComponentTabData");
                return Json(new { status = "Error", message = ex.Message });
            }
        }

        [HttpPost, Route("SNM164_IssueTF")]
        [Authorize]
        public IHttpActionResult SNM164_IssueTF(List<IssueTfModel> param)
        {
            try
            {
                var json = JsonConvert.SerializeObject(param);
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM164_START_ISSUE_TF";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VIN_DATA", OracleDbType.Varchar2).Value = json;
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_DOC_FROM", OracleDbType.Varchar2,20).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("P_DOC_TO", OracleDbType.Varchar2,20).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("P_STATUS", OracleDbType.Varchar2, 20).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string doc_from = cmd.Parameters["P_DOC_FROM"].Value.ToString() == "null" ? "" : cmd.Parameters["P_DOC_FROM"].Value.ToString();
                string doc_to = cmd.Parameters["P_DOC_TO"].Value.ToString() == "null" ? "" : cmd.Parameters["P_DOC_TO"].Value.ToString();
                string staus = cmd.Parameters["P_STATUS"].Value.ToString();
                return Json(new { STATUS = staus, DOC_FROM = doc_from, DOC_TO = doc_to });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM164_GetComponentTabData");
                return Json(new { STATUS = "Error", message = ex.Message });
            }
        }
		
		[HttpGet, Route("SNM164_GetTransferData/{vin}/{optioncode}")]
        [Authorize]
        public IHttpActionResult SNM164_GetTransferData(string vin, string optioncode)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM164_GET_VOM_TRANSFER_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VIN", OracleDbType.Varchar2).Value = vin.ToUpper() == "NULL" ? (object)DBNull.Value : vin;
                cmd.Parameters.Add("P_OPTION_CODE", OracleDbType.Varchar2).Value = optioncode.ToUpper() == "NULL" ? (object)DBNull.Value : optioncode;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(new { status = "Done", message = "Done", set = DTFM.convertToList(ds) });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM164_GetTransferData");
                return Json(new { status = "Error", message = ex.Message });
            }
        }
        [HttpGet, Route("SNM164_GetVOMStatus")]
        [Authorize]
        public IHttpActionResult SNM164_GetVOMStatus()
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM164_GET_VOM_STATUS";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(new { status = "Done", message = "Done", set = DTFM.convertToList(ds) });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM164_GetVOMStatus");
                return Json(new { status = "Error", message = ex.Message });
            }
        }
        public class IssueTfModel
        {
            public string VIN { get; set; }
        }
        #endregion
    }
}
