﻿using Oracle.ManagedDataAccess.Client;
using RandomSolutions;
using SYS_Core.aBiz;
using SYS_Core.aControl;
using SYSCore.aControl;
using SYSCore.ReportingService;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Configuration;
using System.Data;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Text;
using System.Threading.Tasks;
using System.Web;
using System.Web.Http;
using System.Web.Http.Cors;
using SYSCore.Attributes;

namespace SYS_Core.Controllers
{
    [EnableCors("*", "*", "*")]
    [RoutePrefix("api/SL")]
    public class SalesController : ApiController
    {
        private DBManager odb = new DBManager();
        private DBOracle oOracledb = new DBOracle();

        private DataModel DataModel = new DataModel();

        //private string PMS_FILE_PATH = "//192.168.1.250/P_Shared/PMS_Upload/";
        #region SNSE221


        [HttpGet, Route("SNSE221_GetCustomer/{customer}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE221_GetCustomer(string customer, int noofshowrec)
        {
            DataSet ds = new DataSet();
            customer = ismoUtil.escapeCharacter(customer);
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE221_CUSTOMER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = customer.ToUpper() == "NULL" ? (object)DBNull.Value : customer;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE221_GetCustomer");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE221_GetBranch/{branch}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE221_GetBranch(string branch, int noofshowrec, [FromBody] dynamic param)
        {
            DataSet ds = new DataSet();
            branch = ismoUtil.escapeCharacter(branch);
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE221_GET_BRANCH";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = param.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = branch.ToUpper() == "NULL" ? (object)DBNull.Value : branch;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE221_GetBranch");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpPost, Route("SNSE221_GetOrderNo/{orderNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE221_GetOrderNo(string orderNo, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            orderNo = ismoUtil.escapeCharacter(orderNo);
            //var pData = pParam["DATA"];

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE221_ORDER_NO";
                cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = orderNo.ToUpper() == "NULL" ? (object)DBNull.Value : orderNo;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam["BRANCH_CODE"];
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE221_GetOrderNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }


        }

        [HttpPost, Route("SNSE221_GetOrderData")]
        [Authorize]
        public IHttpActionResult SNSE221_GetOrderData([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE221_GET_ORDER";
                cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pParam["ORDER_NO"];
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam["BRANCH_CODE"];
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE221_GetOrderData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }


        }




        [HttpGet, Route("SNSE221_GetData/{part_no}")]
        [Authorize]
        public IHttpActionResult SNSE221_GetData(string part_no)
        {
            DataSet ds = new DataSet();
            part_no = ismoUtil.escapeCharacter(part_no);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE221_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = part_no;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE221_GetData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE221_Reverse")]
        [Authorize]
        public IHttpActionResult SNSE221_Reverse([FromBody] dynamic data)
        {

            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            int iCount = 0;
            try
            {

                foreach (dynamic d in data)
                {
                    iCount++;
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE221_SAVE_TEMP_REVERSE";
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = d["CUSTOMER_CODE"];
                    cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = d["ORDER_NO"];
                    cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Varchar2).Value = d["ITEM_NO"];
                    cmd.Parameters.Add("P_SUB_ITEM_NO", OracleDbType.Varchar2).Value = d["SUB_ITEM_NO"];
                    cmd.Parameters.Add("P_ACCEPT_PART_NO", OracleDbType.Varchar2).Value = d["ACCEPT_PART_NO"];
                    cmd.Parameters.Add("P_REVERSE_QTY", OracleDbType.Varchar2).Value = d["REVERSE_QTY"];
                    cmd.Parameters.Add("P_DOCUMENT_TYPE", OracleDbType.Varchar2).Value = d["DOCUMENT_TYPE"];
                    cmd.Parameters.Add("P_DOCUMENT_NO", OracleDbType.Varchar2).Value = d["DOCUMENT_NO"];
                    cmd.Parameters.Add("P_DOCUMENT_DATE", OracleDbType.Date).Value = d["DOCUMENT_DATE"];
                    cmd.Parameters.Add("P_REASON_CODE", OracleDbType.Varchar2).Value = d["REASON_CODE"];
                    cmd.Parameters.Add("P_ITEM_NO_ODS", OracleDbType.Varchar2).Value = d["ITEM_NO_ORDER_STATUS"];
                    cmd.Parameters.Add("P_SUB_ITEM_NO_ODS", OracleDbType.Varchar2).Value = d["SUB_ITEM_NO_ORDER_STATUS"];
                    cmd.Parameters.Add("P_RNO", OracleDbType.Varchar2).Value = d["RNO"];
                    cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_CLEAR_DATA", OracleDbType.Varchar2).Value = iCount == 1 ? "Y" : "N";
                    cmd = oOracledb.SqlQueryReturnValue(cmd);
                }
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE221_TRANSACTION_REVERSE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                //--- Roll back ---
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE221_Reverse");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }



        [HttpPost, Route("SNSE221_Allocate")]
        [Authorize]
        public IHttpActionResult SNSE221_Allocate([FromBody] dynamic data)
        {

            DataSet ds = new DataSet();
            DataTable dt = new DataTable();

            OracleCommand cmd = new OracleCommand();
            int iCount = 0;
            try
            {

                foreach (dynamic d in data.DETAIL)
                {
                    iCount++;
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE221_SAVE_TEMP_ALLOCATE";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = d["CUSTOMER_CODE"];
                    cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = d["ORDER_NO"];
                    cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Varchar2).Value = d["ITEM_NO"];
                    cmd.Parameters.Add("P_SUB_ITEM_NO", OracleDbType.Varchar2).Value = d["SUB_ITEM_NO"];
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = d["BRANCH_CODE"];
                    cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = d["ORDER_PART_NO"];
                    cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_CLEAR_DATA", OracleDbType.Varchar2).Value = iCount == 1 ? "Y" : "N";
                    cmd = oOracledb.SqlQueryReturnValue(cmd);
                }
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE221_ALLOCATE_MANUAL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = data["BRANCH_CODE"];
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = data["ORDER_NO"];
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);
                string r_status = cmd.Parameters["R_STATUS"].Value.ToString();
                string r_msg = cmd.Parameters["R_MESSAGE"].Value.ToString();

                if (r_status != "DONE")
                {
                    oOracledb.WriteLog("ErrorHandle", r_msg, "SNSE221_Allocate");
                }

                return Json(new { STATUS = r_status, MESSAGE = r_msg });
            }
            catch (Exception ex)
            {
                //--- Roll back ---
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE221_Allocate");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

            //return Json(data);
        }

        #endregion SNSE221
        #region SNSE040_P

        [HttpGet, Route("SNSE040_P_GetSentBy")]
        [Authorize]
        public IHttpActionResult SNSE040_GetSentBy()
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE040_P_GET_SENT_BY";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE040_P_GetSentBy");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNSE040_P_GetCustomer/{customer}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE040_P_GetCustomer(string customer, int noofshowrec)
        {
            DataSet ds = new DataSet();
            customer = ismoUtil.escapeCharacter(customer);
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE040_P_GET_CUSTOMER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = customer.ToUpper() == "NULL" ? (object)DBNull.Value : customer;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE040_P_GetCustomer");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE040_P_GetBranch/{branch}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE040_P_GetBranch(string branch, int noofshowrec, [FromBody] dynamic param)
        {
            DataSet ds = new DataSet();
            branch = ismoUtil.escapeCharacter(branch);
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE040_P_GET_BRANCH";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = param.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = branch.ToUpper() == "NULL" ? (object)DBNull.Value : branch;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE040_P_GetBranch");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpPost, Route("SNSE040_P_GetInstruction/{instruction}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE040_P_GetInstruction(string instruction, int noofshowrec, [FromBody] dynamic param)
        {
            DataSet ds = new DataSet();
            instruction = ismoUtil.escapeCharacter(instruction);
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE040_P_INSTRUCTION";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_INSTRUCTION", OracleDbType.Varchar2).Value = instruction.ToUpper() == "NULL" ? (object)DBNull.Value : instruction;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = param.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = param.BRANCH_CODE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE040_P_GetInstruction");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpPost, Route("SNSE040_P_GetHeader")]
        [Authorize]
        public IHttpActionResult SNSE040_P_GetHeader([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE040_P_GET_HEADER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = param.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = param.BRANCH_CODE;
                cmd.Parameters.Add("P_PACK_DATE", OracleDbType.Varchar2).Value = param.PACK_DATE;
                //cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = param.SENT_BY;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE040_P_GetHeader");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE040_P_GetDetail")]
        [Authorize]
        public IHttpActionResult SNSE040_P_GetDetail([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE040_P_GET_DETAIL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = param.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = param.BRANCH_CODE;
                cmd.Parameters.Add("P_PACK_DATE", OracleDbType.Date).Value = param.PACK_DATE;
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = param.CASE_NO;
                cmd.Parameters.Add("P_CASE_SEQ", OracleDbType.Int16).Value = param.CASE_SEQ;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE040_P_GetDetail");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE040_P_FindCaseSeq")]
        [Authorize]
        public IHttpActionResult SNSE040_P_FindCaseSeq([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE040_P_FIND_CASE_SEQ";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PACK_DATE", OracleDbType.Varchar2).Value = param.PACK_DATE_STRING;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = param.CUSTOMER_CODE;
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = param.CASE_NO;
                cmd.Parameters.Add("R_CASE_SEQ", OracleDbType.Int16).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string R_CASE_SEQ = cmd.Parameters["R_CASE_SEQ"].Value.ToString();
                return Json(new { CASE_SEQ = R_CASE_SEQ, });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE040_P_FindCaseSeq");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE040_P_Save")]
        [Authorize]
        public IHttpActionResult SNSE040_P_Save([FromBody] dynamic data)
        {

            DataSet ds = new DataSet();
            DataTable dt = new DataTable();

            OracleCommand cmd = new OracleCommand();
            int iCount = 0;
            try
            {

                foreach (dynamic d in data.DETAIL)
                {
                    iCount++;
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE040_P_SAVE_TEMP";
                    cmd.Parameters.Add("P_INSTUCTION_NO", OracleDbType.Varchar2).Value = d["INSTRUCTION_NO"];
                    cmd.Parameters.Add("P_PICK_LOT_DATE", OracleDbType.Date).Value = d["PICK_LOT_DATE"];
                    cmd.Parameters.Add("P_PICK_LOT_SEQ", OracleDbType.Varchar2).Value = d["PICK_LOT_SEQ"];
                    cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = d["ORDER_NO"];
                    cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Varchar2).Value = d["ITEM_NO"];
                    cmd.Parameters.Add("P_SUB_ITEM_NO", OracleDbType.Varchar2).Value = d["SUB_ITEM_NO"];
                    cmd.Parameters.Add("P_ACCEPT_PART_NO", OracleDbType.Varchar2).Value = d["ACCEPT_PART_NO"];
                    cmd.Parameters.Add("P_PACK_QTY", OracleDbType.Varchar2).Value = d["PACK_QTY"];
                    cmd.Parameters.Add("P_UPDATE_FLAG", OracleDbType.Varchar2).Value = "INS";
                    cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = d["ORDER_PART_NO"];
                    cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_CLEAR_DATA", OracleDbType.Varchar2).Value = iCount == 1 ? "Y" : "N";
                    cmd = oOracledb.SqlQueryReturnValue(cmd);
                }
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE040_P_SAVE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = data["BRANCH_CODE"];
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = data["CASE_NO"];
                cmd.Parameters.Add("P_CASE_SEQ", OracleDbType.Varchar2).Value = data["CASE_SEQ"];
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 100).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);
                string r_status = cmd.Parameters["R_STATUS"].Value.ToString();
                string r_msg = cmd.Parameters["R_MESSAGE"].Value.ToString();

                return Json(new { STATUS = r_status, MESSAGE = r_msg });
            }
            catch (Exception ex)
            {
                //--- Roll back ---
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE040_P_Save");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

            //return Json(data);
        }


        [HttpPost, Route("SNSE040_P_Save_Update")]
        [Authorize]
        [RunBatchGuard]
        public IHttpActionResult SNSE040_P_Save_Update([FromBody] dynamic data)
        {

            DataSet ds = new DataSet();
            DataTable dt = new DataTable();

            OracleCommand cmd = new OracleCommand();
            int iCount = 0;
            try
            {

                foreach (dynamic d in data.DETAIL)
                {
                    iCount++;
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE040_P_SAVE_TEMP";
                    cmd.Parameters.Add("P_INSTUCTION_NO", OracleDbType.Varchar2).Value = d["INSTRUCTION_NO"];
                    cmd.Parameters.Add("P_PICK_LOT_DATE", OracleDbType.Date).Value = d["PICK_LOT_DATE"];
                    cmd.Parameters.Add("P_PICK_LOT_SEQ", OracleDbType.Varchar2).Value = d["PICK_LOT_SEQ"];
                    cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = d["ORDER_NO"];
                    cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Varchar2).Value = d["ITEM_NO"];
                    cmd.Parameters.Add("P_SUB_ITEM_NO", OracleDbType.Varchar2).Value = d["SUB_ITEM_NO"];
                    cmd.Parameters.Add("P_ACCEPT_PART_NO", OracleDbType.Varchar2).Value = d["ACCEPT_PART_NO"];
                    cmd.Parameters.Add("P_PACK_QTY", OracleDbType.Varchar2).Value = d["PACK_QTY"];
                    cmd.Parameters.Add("P_UPDATE_FLAG", OracleDbType.Varchar2).Value = d["UPDATE_FLAG"];
                    cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = d["ORDER_PART_NO"];
                    cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_CLEAR_DATA", OracleDbType.Varchar2).Value = iCount == 1 ? "Y" : "N";
                    cmd = oOracledb.SqlQueryReturnValue(cmd);
                }
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE040_P_SAVE_UPDATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = data["BRANCH_CODE"];
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = data["CASE_NO"];
                cmd.Parameters.Add("P_CASE_SEQ", OracleDbType.Varchar2).Value = data["CASE_SEQ"];
                cmd.Parameters.Add("P_PACK_DATE", OracleDbType.Varchar2).Value = data["PACK_DATE"];
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 100).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);
                string r_status = cmd.Parameters["R_STATUS"].Value.ToString();
                string r_msg = cmd.Parameters["R_MESSAGE"].Value.ToString();

                return Json(new { STATUS = r_status, MESSAGE = r_msg });
            }
            catch (Exception ex)
            {
                //--- Roll back ---
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE040_P_Save");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

            //return Json(data);
        }
        #endregion
        #region SNSQ033
        [HttpGet, Route("SNSQ033_GetVendor/{vendor}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSQ033_GetVendor(string vendor, int noofshowrec)
        {
            DataSet ds = new DataSet();
            vendor = ismoUtil.escapeCharacter(vendor);
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ033_GET_VENDOR";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = vendor.ToUpper() == "NULL" ? (object)DBNull.Value : vendor;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE204_P_GetBrNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNSQ033_GetPart/{part}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSQ033_GetPart(string part, int noofshowrec, [FromBody] dynamic param)
        {
            DataSet ds = new DataSet();
            part = ismoUtil.escapeCharacter(part);
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ033_GET_PART";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_FROM_DATE", OracleDbType.Date).Value = param.FROM_DATE;
                cmd.Parameters.Add("P_TO_DATE", OracleDbType.Date).Value = param.TO_DATE;
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = param.VENDOR_CODE;
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = part.ToUpper() == "NULL" ? (object)DBNull.Value : part;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE204_P_GetBrNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNSQ033_Search")]
        [Authorize]
        public IHttpActionResult SNSQ033_Search([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ033_SEARCH";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_FROM_DATE", OracleDbType.Date).Value = param["FROM_DATE"];
                cmd.Parameters.Add("P_TO_DATE", OracleDbType.Date).Value = param["TO_DATE"];
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = param["VENDOR_CODE"];
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = param["PART_NO"];
                cmd.Parameters.Add("P_BASE_PRICE", OracleDbType.Varchar2).Value = param["BASE_PRICE"];
                cmd.Parameters.Add("P_MIGRATION", OracleDbType.Varchar2).Value = param["MIGRATION"];
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE204_D_GetOrderNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNSQ033_Export_DAT")]
        [Authorize]
        public HttpResponseMessage SNSQ033_Export_DAT([FromBody] dynamic[] paramData)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();
            int iCount = 0;
            try
            {
                OracleCommand cmd = new OracleCommand();
                foreach (var detail in paramData)
                {
                    iCount++;
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSQ033_SAVE_TEMP";
                    cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = detail["PART_NO"];
                    cmd.Parameters.Add("P_PART_NAME_EN", OracleDbType.Varchar2).Value = detail["PART_NAME_EN"];
                    cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = detail["VENDOR_CODE"];
                    cmd.Parameters.Add("P_BASE_PRICE", OracleDbType.Double).Value = detail["BASE_PRICE"];
                    cmd.Parameters.Add("P_CLEAR_DATA", OracleDbType.Varchar2).Value = iCount == 1 ? "Y" : "N";
                    cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    oOracledb.SqlQueryReturnValue(cmd);
                }
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ033_GEN_FILE_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                dt = ds.Tables[0];
                string filename = "GTTCSVAS" + DataService.GetIdentity(User, Cons.SS_USER_NAME).ToString() + ".DAT";
                string x = DataService.WriteCSVNoColumn(dt, filename, "SNSQ033");
                Controllers.SystemApiController response = new Controllers.SystemApiController();
                var File = response.downloadFile(x.Substring(1), filename);
                return File;
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ033_Export_DAT");
                return Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message);
            }
        }
        [HttpPost, Route("SNSQ033_Export_CSV")]
        public HttpResponseMessage SNM105P_Download(List<DataModel.SNSQ033> dModel)
        {
            DataTable dt = new DataTable();
            try
            {
                string x = string.Empty;
                string filename = "SNSQ033" + DataService.GetIdentity(User, Cons.SS_USER_NAME).ToString() + ".csv";
                string floderLocation = "SNSQ033";
                dt = DTFM.ToDatatable<DataModel.SNSQ033>(dModel);
                x = DataService.WriteCSV(dt, filename, floderLocation, "SNSQ033");
                Controllers.SystemApiController response = new Controllers.SystemApiController();
                var File = response.downloadFile(x.Substring(1), filename);
                return File;
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ033_Export_CSV");
                return Request.CreateResponse(HttpStatusCode.BadRequest);
            }
        }
        [HttpGet, Route("SNSQ033_GetProcessLog")]
        [Authorize]
        public IHttpActionResult SNSQ033_GetProcessLog()
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ033_GET_PROCESS_LOG";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ033_GetProcessLog");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        #endregion SNSQ033
        #region SNSE208
        [HttpPost, Route("SNSE208_Upload")]
        [Authorize]
        public async Task<IHttpActionResult> SNSE208_Upload(HttpRequestMessage request)
        {
            if (!request.Content.IsMimeMultipartContent())
            {
                throw new HttpResponseException(HttpStatusCode.UnsupportedMediaType);
            }
            var reqParts = await request.Content.ReadAsMultipartAsync();
            var name = reqParts.Contents[1].ReadAsByteArrayAsync();
            var option = Encoding.UTF8.GetString(name.Result, 0, name.Result.Length);
            List<string> ErrorList = new List<string>();
            try
            {
                string fileLocation = string.Empty;
                var file = reqParts.Contents[0];
                var buffer = await file.ReadAsByteArrayAsync();
                string fileName = Path.GetFileNameWithoutExtension(file.Headers.ContentDisposition.FileName.Trim('\"'));
                string ext = Path.GetExtension(file.Headers.ContentDisposition.FileName.Trim('\"'));
                string FullfileName = String.Concat(fileName, "-", DateTime.Now.ToString("yyyyMMddHHmm"), ext);
                string outputLocation = "~/tempFile/" + "SNSE208" + "/";
                string outputPath = System.Web.HttpContext.Current.Server.MapPath(outputLocation);
                fileLocation = String.Concat(outputPath, FullfileName);
                DataService.SaveBytesToFileByLocation(buffer
                                                , fileLocation
                                                , fileLocation);
                ErrorList = SNSE208_InsertDataFromExcelFile(fileLocation, option);
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE208_Upload");
                throw new HttpResponseException(HttpStatusCode.NotAcceptable);
            }
            return Json(new { Error = ErrorList });
        }
        public string SNSE208_ClearTemp()
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSE208_CLEAR_TEMP";
            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return "DONE";
        }
        public List<string> SNSE208_InsertDataFromExcelFile(string fileLocation, string option)
        {
            string fileLocationN = fileLocation;
            string optionN = option;
            DataTable dt = new DataTable();
            SystemApiController sys = new SystemApiController();
            dt = sys.ReadDataExcel(fileLocationN);
            List<string> ErrList = new List<string>();
            SNSE208_ClearTemp();
            foreach (DataRow row in dt.Rows)
            {
                int count = 0;
                string p1 = null;
                string p2 = null;
                string p3 = null;
                string p4 = null;
                if (optionN == "L1")
                {
                    p1 = null;
                    p2 = null;
                    p3 = row["Order Part No"].ToString().Trim();
                    p4 = row["Approved Qty"].ToString().Trim();
                }
                else
                {
                    p1 = row["Customer Code"].ToString().Trim();
                    p2 = row["Order No"].ToString().Trim();
                    p3 = row["Order Part No"].ToString().Trim();
                    p4 = row["Approved Qty"].ToString().Trim();
                }
                if (p1 == null && p2 == null && p3 != null && p4 != null)
                {
                    count = 2;
                }
                else if (p1 != null && p2 != null && p3 != null && p4 != null)
                {
                    count = 4;
                }
                else
                {
                    if (p1 != null) { count = count + 1; }
                    if (p2 != null) { count = count + 1; }
                    if (p3 != null) { count = count + 1; }
                    if (p4 != null) { count = count + 1; }
                }
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE208_VALID_QTY_UPL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = p1;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = p2;
                cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = p3;
                cmd.Parameters.Add("P_APPROVED_QTY", OracleDbType.Varchar2).Value = p4;
                cmd.Parameters.Add("P_OPT_LAYOUT", OracleDbType.Varchar2).Value = optionN;
                cmd.Parameters.Add("P_COUNT_COLUMN", OracleDbType.Varchar2).Value = count;
                cmd = oOracledb.SqlQueryReturnValue(cmd);
            }
            return ErrList;
        }
        [HttpGet, Route("SNSE208_GetTempData")]
        [Authorize]
        public IHttpActionResult SNSE208_GetTempData()
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE208_GET_TEMP_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE208_GetTempData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNSE208_Save")]
        [Authorize]
        public IHttpActionResult SNSE208_Save([FromBody] dynamic[] pDataList)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();
            try
            {
                foreach (var data in pDataList)
                {
                    OracleCommand cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE208_SAVE_UPL_APV_QTY";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = data.ORDER_NO;
                    cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = data.ORDER_PART_NO;
                    cmd.Parameters.Add("P_CONV_APPROVE_QTY", OracleDbType.Varchar2).Value = data.CONV_APPROVED_QTY;
                    cmd.Parameters.Add("P_SEL_UPLOAD", OracleDbType.Varchar2).Value = data.SEL_UPLOAD;
                    cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE208";
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                    ds = oOracledb.SqlQueryDS_CMD(cmd);
                }
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE208_Save");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNSE208_getDataForExport")]
        [Authorize]
        public HttpResponseMessage SNSE208_getDataForExport([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSE208_GET_TEMP_DATA";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("C_RE", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
            ds = oOracledb.SqlQueryDS_CMD(cmd);
            string x = SNSE208_ExportFile(ds.Tables[0], "SNSE208.csv", "SNSE208");
            SystemApiController sys = new SystemApiController();
            var respons = sys.downloadFile(x.Substring(1), "SNSE208.csv");
            return respons;
        }
        public static string SNSE208_ExportFile(DataTable dt, string fileName, string FloderLocation)
        {
            bool Succes = false;
            string gLocation = "~/tempFile/" + FloderLocation + "/";
            string gWriteLocation = System.Web.HttpContext.Current.Server.MapPath(gLocation);
            FileInfo filePath = new FileInfo(gWriteLocation);
            filePath.Directory.Create();
            string gFileLocation = Path.Combine(gWriteLocation, fileName);
            string gReturnLocation = Path.Combine(gLocation, fileName);
            SystemApiController sys = new SystemApiController();
            try
            {
                StringBuilder sb = new StringBuilder();
                string[] HeaderForExport = new string[]
                {
                    "Seq.",
                    "Customer Code",
                    "Order No",
                    "Order Part No",
                    "Approved Qty",
                    "Message",
                };
                IEnumerable<string> columnNames = HeaderForExport.Select(h => h.ToString());
                sb.AppendLine(string.Join(",", columnNames));
                foreach (DataRow row in dt.Rows)
                {
                    sb.Append(CsvEscape(row[0].ToString()));
                    for (int i = 0; i < dt.Columns.Count - 10; i++)
                    {
                        sb.Append(CsvEscape(row[i].ToString()));
                        sb.Append(i == dt.Columns.Count - 11 ? "\n" : ",");
                    }
                }
                File.WriteAllText(gFileLocation, sb.ToString());
                Succes = true;
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
            }
            if (Succes)
            {
                return gReturnLocation;
            }
            else
            {
                return "";
            }
        }
        #endregion

        #region SNSE211
        [HttpGet, Route("SNSE211_GetData")]
        [Authorize]
        public IHttpActionResult SNSE211_GetData()
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            try
            {
                cmd.CommandText = "SP_NP_SNSE211_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE211_GetData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNSE211_Upload")]
        [Authorize]
        public async Task<IHttpActionResult> SNSE211_Upload(HttpRequestMessage request)
        {

            if (!request.Content.IsMimeMultipartContent())
            {
                throw new HttpResponseException(HttpStatusCode.UnsupportedMediaType);
            }

            var reqParts = await request.Content.ReadAsMultipartAsync();
            List<string> ErrorList = new List<string>();
            try
            {
                string fileLocation = string.Empty;
                string gFileName = string.Empty;


                var file = reqParts.Contents[0];
                var buffer = await file.ReadAsByteArrayAsync();
                string fileName = Path.GetFileNameWithoutExtension(file.Headers.ContentDisposition.FileName.Trim('\"'));
                string ext = Path.GetExtension(file.Headers.ContentDisposition.FileName.Trim('\"'));
                string FullfileName = String.Concat(fileName, "-", DateTime.Now.ToString("yyyyMMddHHmm"), ext);
                string outputLocation = "~/tempFile/" + "SNSE211" + "/";
                string outputPath = System.Web.HttpContext.Current.Server.MapPath(outputLocation);
                fileLocation = String.Concat(outputPath, FullfileName);
                gFileName = file.Headers.ContentDisposition.FileName.Trim('\"');

                DataService.SaveBytesToFileByLocation(buffer
                                                , fileLocation
                                                , fileLocation);

                ErrorList = SNSE211_InsertDataFromExcelFile(fileLocation, gFileName, FullfileName);
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE211_Upload");
                throw new HttpResponseException(HttpStatusCode.NotAcceptable);
            }

            return Json(new { Error = ErrorList });
        }

        public List<string> SNSE211_InsertDataFromExcelFile(string fileLocation, string gFileName, string FullfileName)
        {
            string fileLocationN = fileLocation;
            string gFileNameN = gFileName;
            string FullfileNameN = FullfileName;
            DataTable dt = new DataTable();
            SystemApiController sys = new SystemApiController();
            dt = sys.ReadDataExcel(fileLocationN);
            List<string> ErrList = new List<string>();
            int countrow = 0;
            OracleCommand cmd = new OracleCommand();
            DataSet ds = new DataSet();

            try
            {
                cmd.CommandText = "SP_NP_SNSE211_DEL_TEMP";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE211_InsertDataFromExcelFile");
                ErrList.Add("Error : SP_NP_SNSE211_DEL_TEMP please recheck");
                return ErrList;
            }

            foreach (DataRow row in dt.Rows)
            {
                countrow++;

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE211_INS_TEMP_UPLOAD";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = row["Customer Code"].ToString().Trim();
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = row["Branch Code"].ToString().Trim();
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_SEQ", OracleDbType.Varchar2).Value = countrow;
                cmd = oOracledb.SqlQueryReturnValue(cmd);
            }
            return ErrList;
        }

        [HttpGet, Route("SNSE211_GetTempData")]
        [Authorize]
        public IHttpActionResult SNSE211_GetTempData()
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();

            try
            {
                cmd.CommandText = "SP_NP_SNSE211_GET_TEMP_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE211_GetTempData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE211_Update")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNSE211_Update([FromBody] dynamic[] pDataList)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();
            OracleCommand cmd = new OracleCommand();
            try
            {
                cmd.CommandText = "SP_NP_SNSE211_CLEAR_OLD_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                oOracledb.SqlQueryReturnValue(cmd);
                foreach (var data in pDataList)
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE211_UPDATE";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = data.BRANCH_CODE;
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE211";
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                    ds = oOracledb.SqlQueryDS_CMD(cmd);
                }
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE211_Update");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE211_getDataForExport")]
        [Authorize]
        public HttpResponseMessage SNSE211_getDataForExport([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSE211_GET_TEMP_DATA";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
            ds = oOracledb.SqlQueryDS_CMD(cmd);
            string x = SNSE211_ExportFile(ds.Tables[0], "SNSE211.csv", "SNSE211");
            SystemApiController sys = new SystemApiController();
            var respons = sys.downloadFile(x.Substring(1), "SNSE211.csv");
            return respons;
        }

        public static string SNSE211_ExportFile(DataTable dt, string fileName, string FloderLocation)
        {
            bool Succes = false;
            string gLocation = "~/tempFile/" + FloderLocation + "/";
            string gWriteLocation = System.Web.HttpContext.Current.Server.MapPath(gLocation);
            FileInfo filePath = new FileInfo(gWriteLocation);
            filePath.Directory.Create();
            string gFileLocation = Path.Combine(gWriteLocation, fileName);
            string gReturnLocation = Path.Combine(gLocation, fileName);
            SystemApiController sys = new SystemApiController();
            try
            {
                StringBuilder sb = new StringBuilder();
                string[] HeaderForExport = new string[]
                {
                    "[Seq.]",
                    "[CUSTOMER CODE.]",
                    "[BRANCH CODE]",
                    "[Message]"
                };




                IEnumerable<string> columnNames = HeaderForExport.Select(h => h.ToString());
                sb.AppendLine(string.Join(",", columnNames));
                foreach (DataRow row in dt.Rows)
                {
                    for (int i = 0; i < dt.Columns.Count - 2; i++)
                    {
                        sb.Append(CsvEscape(row[i].ToString()));
                        sb.Append(i == dt.Columns.Count - 3 ? "\n" : ",");
                    }
                }
                File.WriteAllText(gFileLocation, sb.ToString());
                Succes = true;
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
            }
            if (Succes)
            {
                return gReturnLocation;
            }
            else
            {
                return "";
            }
        }

        [HttpGet, Route("SNSE211_ClearOldData")]
        [Authorize]
        public IHttpActionResult SNSE211_ClearOldData()
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();

            try
            {
                cmd.CommandText = "SP_NP_SNSE211_CLEAR_OLD_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE211_ClearOldData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        #endregion

        #region SNSE204
        [HttpPost, Route("SNSE204_GetPart/{part}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE204_GetPart(string part, int noofshowrec, [FromBody] dynamic param)
        {
            DataSet ds = new DataSet();
            //var pData = pParam["DATA"];
            part = ismoUtil.escapeCharacter(part);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE204_GET_PART";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = param["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = param["BRANCH_CODE"];
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = param["ORDER_NO"];
                cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = part.ToUpper() == "NULL" ? (object)DBNull.Value : part;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE204_D_GetPart");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNSE204_GetInvoice/{invoice}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE204_GetInvoice(string invoice, int noofshowrec, [FromBody] dynamic param)
        {
            DataSet ds = new DataSet();
            //var pData = pParam["DATA"];
            invoice = ismoUtil.escapeCharacter(invoice);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE204_GET_INVOICE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = param["ORDER_PART_NO"];
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = invoice.ToUpper() == "NULL" ? (object)DBNull.Value : invoice;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE204_GetInvoice");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE204_GetPO/{po}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE204_GetPO(string po, int noofshowrec, [FromBody] dynamic param)
        {
            DataSet ds = new DataSet();
            //var pData = pParam["DATA"];
            po = ismoUtil.escapeCharacter(po);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE204_GET_PO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = param["ORDER_PART_NO"];
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = param["INVOICE_NO"];
                cmd.Parameters.Add("P_PO_NO", OracleDbType.Varchar2).Value = po.ToUpper() == "NULL" ? (object)DBNull.Value : po;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE204_GetPO");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE204_GetCase/{caseno}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE204_GetCase(string caseno, int noofshowrec, [FromBody] dynamic param)
        {
            DataSet ds = new DataSet();
            //var pData = pParam["DATA"];
            caseno = ismoUtil.escapeCharacter(caseno);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE204_GET_CASE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = param["ORDER_PART_NO"];
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = param["INVOICE_NO"];
                cmd.Parameters.Add("P_PO_NO", OracleDbType.Varchar2).Value = param["PO_NO"];
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = caseno.ToUpper() == "NULL" ? (object)DBNull.Value : caseno;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE204_GetCase");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE204_GetSubCase/{subcase}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE204_GetSubCase(string subcase, int noofshowrec, [FromBody] dynamic param)
        {
            DataSet ds = new DataSet();
            //var pData = pParam["DATA"];
            subcase = ismoUtil.escapeCharacter(subcase);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE204_GET_SUB_CASE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = param["ORDER_PART_NO"];
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = param["INVOICE_NO"];
                cmd.Parameters.Add("P_PO_NO", OracleDbType.Varchar2).Value = param["PO_NO"];
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = param["CASE_NO"]; ;
                cmd.Parameters.Add("P_SUB_CASE_NO", OracleDbType.Varchar2).Value = subcase.ToUpper() == "NULL" ? (object)DBNull.Value : subcase;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE204_GetSubCase");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE204_CheckInvoice")]
        [Authorize]
        public IHttpActionResult SNSE204_CheckInvoice([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE204_CHECK_INVOICE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = param["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = param["BRANCH_CODE"];
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = param["ORDER_NO"];
                cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = param["ORDER_PART_NO"];
                cmd.Parameters.Add("C_RETURN1", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN2", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE204_CheckInvoice");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE204_CheckQTY")]
        [Authorize]
        public IHttpActionResult SNSE204_CheckQTY([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE204_CHECK_QTY";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PO_NO", OracleDbType.Varchar2).Value = param["PO_NO"];
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = param["INVOICE_NO"];
                cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = param["ORDER_PART_NO"];
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = param["CASE_NO"];
                cmd.Parameters.Add("P_SUB_CASE_NO", OracleDbType.Varchar2).Value = param["SUB_CASE_NO"];
                cmd.Parameters.Add("R_SUM_QTY", OracleDbType.Int16).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_REF_INDENT", OracleDbType.Int16).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                Int16 R_SUM_QTY = Int16.Parse(cmd.Parameters["R_SUM_QTY"].Value.ToString());
                Int16 R_REF_INDENT = Int16.Parse(cmd.Parameters["R_REF_INDENT"].Value.ToString());

                return Json(new { SUM_QTY = R_SUM_QTY, REF_INDENT = R_REF_INDENT });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE204_CheckQTY");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpPost, Route("SNSE204_CheckQTYBySubcase")]
        [Authorize]
        public IHttpActionResult SNSE204_CheckQTYBySubcase([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE204_CK_QTY_SUBCASE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PO_NO", OracleDbType.Varchar2).Value = param["PO_NO"];
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = param["INVOICE_NO"];
                cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = param["ORDER_PART_NO"];
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = param["CASE_NO"];
                cmd.Parameters.Add("P_SUB_CASE_NO", OracleDbType.Varchar2).Value = param["SUB_CASE_NO"];
                cmd.Parameters.Add("R_REF_INDENT", OracleDbType.Int16).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);


                Int16 R_REF_INDENT = Int16.Parse(cmd.Parameters["R_REF_INDENT"].Value.ToString());

                return Json(new { REF_INDENT = R_REF_INDENT });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE204_CheckQTY");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE204_Save")]
        [Authorize]
        public IHttpActionResult SNSE204_Save([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();
            int iCount = 0;
            OracleCommand cmd = new OracleCommand();
            try
            {

                //Run insert detail
                foreach (var detail in param.Detail)
                {
                    iCount++;

                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE204_SAVETEMP";
                    cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = detail["ORDER_PART_NO"];
                    cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Int32).Value = detail["ITEM_NO"];
                    cmd.Parameters.Add("P_ORDER_PART_NAME", OracleDbType.Varchar2).Value = detail["ORDER_PART_NAME"];
                    cmd.Parameters.Add("P_LOCATION_NO", OracleDbType.Varchar2).Value = detail["LOCATION_NO"];
                    cmd.Parameters.Add("P_PO_NO", OracleDbType.Varchar2).Value = detail["PO_NO"];
                    cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = detail["INVOICE_NO"];
                    cmd.Parameters.Add("P_ORDER_QTY", OracleDbType.Int16).Value = detail["ORDER_QTY"];
                    cmd.Parameters.Add("P_LEFT_INDENT", OracleDbType.Int16).Value = detail["LEFT_INDENT"];
                    cmd.Parameters.Add("P_REF_ORDER_ITEM_NO", OracleDbType.Int16).Value = detail["REF_ORDER_ITEM_NO"];
                    cmd.Parameters.Add("P_REF_ORDER_SUB_ITEM_NO", OracleDbType.Varchar2).Value = detail["REF_ORDER_SUB_ITEM_NO"];
                    cmd.Parameters.Add("P_REF_VOR_STATUS", OracleDbType.Varchar2).Value = detail["REF_VOR_STATUS"];
                    cmd.Parameters.Add("P_REF_ORDER_QTY", OracleDbType.Int16).Value = detail["REF_ORDER_QTY"];
                    cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = detail["VENDOR_CODE"];
                    cmd.Parameters.Add("P_VENDOR_TYPE", OracleDbType.Varchar2).Value = detail["VENDOR_TYPE"];
                    cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = detail["CASE_NO"];
                    cmd.Parameters.Add("P_SUB_CASE_NO", OracleDbType.Varchar2).Value = detail["SUB_CASE_NO"];
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_CLEAR_DATA", OracleDbType.Varchar2).Value = iCount == 1 ? "Y" : "N";

                    oOracledb.SqlQueryReturnValue(cmd);

                }

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE204_SAVE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = param["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = param["BRANCH_CODE"];
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = param["ORDER_NO"];
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_FORM", OracleDbType.Varchar2).Value = "SNSE024";
                cmd.Parameters.Add("P_VOR_STATUS", OracleDbType.Varchar2).Value = param["VOR_STATUS"];
                cmd.Parameters.Add("P_BR_LOCATION", OracleDbType.Varchar2).Value = param["BR_LOCATION"];
                cmd.Parameters.Add("P_BR_LOCATION_OTHER", OracleDbType.Varchar2).Value = param["BR_LOCATION_OTHER"];
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 255).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string BR_NO = cmd.Parameters["R_STATUS"].Value.ToString();


                return Json(new { status = "done", br_no = BR_NO });
            }
            catch (OracleException oex)
            {

                string err_massage = string.Empty;
                string err_log = string.Empty;

                if (oex.Number == 20003)
                {
                    err_massage = "PO Qty is now 0 !";
                    err_log = oex.Message;
                }
                else if (oex.Number == 20004)
                {
                    err_massage = "PO Qty has been chagned Please Re-enter Form!";
                    err_log = oex.Message;
                }
                else if (oex.Number == 20005)
                {
                    err_massage = "Can not insert into Order Status!";
                    err_log = oex.Message;
                }
                else if (oex.Number == 20006)
                {
                    err_massage = "Can not insert into VOR Status!";
                    err_log = oex.Message;
                }
                else
                {
                    err_massage = "Found an error, please contract support team";
                    err_log = oex.Message;
                }
                oOracledb.WriteLog("ErrorHandle", oex.Number + " - " + err_log, "SNSE204_Save");
                return Ok(new { status = "fail", massage = err_massage });
            }
            catch (Exception ex)
            {


                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE204_Save");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.NotAcceptable, ex.Message));
            }
        }
        #endregion

        #region SNSE204_P

        [HttpGet, Route("SNSE204_P_GetBrNo/{br}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE204_P_GetBrNo(string br, int noofshowrec)
        {
            DataSet ds = new DataSet();
            //var pData = pParam["DATA"];
            br = ismoUtil.escapeCharacter(br);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE204_P_GET_BR_NO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = br.ToUpper() == "NULL" ? (object)DBNull.Value : br;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE204_P_GetBrNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        #endregion

        #region SNSE204_D
        [HttpGet, Route("SNSE204_D_GetCustomer/{cusID}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE204_D_GetCustomer(string cusID, int noofshowrec)
        {
            DataSet ds = new DataSet();

            cusID = ismoUtil.escapeCharacter(cusID);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE204_D_CUSTOMER";
                cmd.Parameters.Add("PCOMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("PCUSTOMER_CODE", OracleDbType.Varchar2).Value = cusID.ToUpper() == "NULL" ? (object)DBNull.Value : cusID;
                //cmd.Parameters.Add("PSHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("CRETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE204_D_GetCustomer");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE204_D_GetBranchCode/{branchCode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE204_D_GetBranchCode(string branchCode, int noofshowrec, [FromBody] dynamic pCustomer)
        {
            DataSet ds = new DataSet();
            //var pData = pParam["DATA"];
            branchCode = ismoUtil.escapeCharacter(branchCode);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE204_D_GET_BRANCH";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pCustomer["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = branchCode.ToUpper() == "NULL" ? (object)DBNull.Value : branchCode;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE204_D_GetBranchCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE204_D_GetSearchOrderNo/{order}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE204_D_GetSearchOrderNo(string order, int noofshowrec, [FromBody] dynamic param)
        {
            DataSet ds = new DataSet();
            order = ismoUtil.escapeCharacter(order);
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE204_D_SEARCH_ORDER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = param["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = param["BRANCH_CODE"];
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = order.ToUpper() == "NULL" ? (object)DBNull.Value : order;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE204_D_GetSearchOrderNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE204_D_GetOrderNo/{order}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE204_D_GetOrderNo(string order, int noofshowrec, [FromBody] dynamic param)
        {
            DataSet ds = new DataSet();
            //var pData = pParam["DATA"];
            order = ismoUtil.escapeCharacter(order);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE204_D_ORDER_NO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = param["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = param["BRANCH_CODE"];
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = order.ToUpper() == "NULL" ? (object)DBNull.Value : order;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE204_D_GetOrderNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpGet, Route("SNSE204_D_GetBrNo/{br}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE204_D_GetBrNo(string br, int noofshowrec)
        {
            DataSet ds = new DataSet();
            //var pData = pParam["DATA"];
            br = ismoUtil.escapeCharacter(br);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE204_D_GET_BR_NO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = br.ToUpper() == "NULL" ? (object)DBNull.Value : br;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE204_D_GetBrNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE204_D_GetPart/{part}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE204_D_GetPart(string part, int noofshowrec, [FromBody] dynamic param)
        {
            DataSet ds = new DataSet();
            //var pData = pParam["DATA"];
            part = ismoUtil.escapeCharacter(part);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE204_D_GET_PART";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = param["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = param["BRANCH_CODE"];
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = param["ORDER_NO"];
                cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = part.ToUpper() == "NULL" ? (object)DBNull.Value : part;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE204_D_GetPart");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE204_D_GetPO/{po}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE204_D_GetPO(string po, int noofshowrec, [FromBody] dynamic param)
        {
            DataSet ds = new DataSet();
            //var pData = pParam["DATA"];
            po = ismoUtil.escapeCharacter(po);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE204_D_GET_PO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = param["ORDER_PART_NO"];

                cmd.Parameters.Add("P_PO", OracleDbType.Varchar2).Value = po.ToUpper() == "NULL" ? (object)DBNull.Value : po;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE204_D_GetPO");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE204_D_GetInvoice/{invoice}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE204_D_GetInvoice(string invoice, int noofshowrec, [FromBody] dynamic param)
        {
            DataSet ds = new DataSet();
            //var pData = pParam["DATA"];
            invoice = ismoUtil.escapeCharacter(invoice);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE204_D_GET_INVOICE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = param["ORDER_PART_NO"];
                cmd.Parameters.Add("P_PO", OracleDbType.Varchar2).Value = param["PO_NO"];
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = invoice.ToUpper() == "NULL" ? (object)DBNull.Value : invoice;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE204_D_GetInvoice");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpPost, Route("SNSE204_D_CheckPO")]
        [Authorize]
        public IHttpActionResult SNSE204_D_CheckPO([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE204_D_CHECK_PO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = param["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = param["BRANCH_CODE"];
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = param["ORDER_NO"];
                cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = param["ORDER_PART_NO"];
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE204_D_CheckPO");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE204_D_CheckQTY")]
        [Authorize]
        public IHttpActionResult SNSE204_D_CheckQTY([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE204_D_CHECK_QTY";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PO_NO", OracleDbType.Varchar2).Value = param["PO_NO"];
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = param["INVOICE_NO"];
                cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = param["ORDER_PART_NO"];
                cmd.Parameters.Add("R_SUM_QTY", OracleDbType.Int16).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_REF_INDENT", OracleDbType.Int16).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                Int16 R_SUM_QTY = Int16.Parse(cmd.Parameters["R_SUM_QTY"].Value.ToString());
                Int16 R_REF_INDENT = Int16.Parse(cmd.Parameters["R_REF_INDENT"].Value.ToString());

                return Json(new { SUM_QTY = R_SUM_QTY, REF_INDENT = R_REF_INDENT });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE204_D_CheckQTY");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE204_D_GetData")]
        [Authorize]
        public IHttpActionResult SNSE204_D_GetData([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE204_D_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_BR_NO", OracleDbType.Varchar2).Value = param["BR_NO"];
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = param["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = param["BRANCH_CODE"];
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = param["ORDER_NO"];
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE204_D_GetData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE204_D_GetDetail")]
        [Authorize]
        public IHttpActionResult SNSE204_D_GetDetail([FromBody] dynamic pdata)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE204_D_GET_DETAIL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_BR_NO", OracleDbType.Varchar2).Value = pdata.BR_NO;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pdata.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pdata.BRANCH_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pdata.ORDER_NO;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE204_D_GetDetail");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpPost, Route("SNSE204_D_Save")]
        [Authorize]
        public IHttpActionResult SNSE204_D_Save([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();
            int iCount = 0;
            OracleCommand cmd = new OracleCommand();
            try
            {

                //Run insert detail
                foreach (var detail in param.Detail)
                {
                    iCount++;

                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE204_D_SAVETEMP";
                    cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = detail["ORDER_PART_NO"];
                    cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Int32).Value = detail["ITEM_NO"];
                    cmd.Parameters.Add("P_ORDER_PART_NAME", OracleDbType.Varchar2).Value = detail["ORDER_PART_NAME"];
                    cmd.Parameters.Add("P_LOCATION_NO", OracleDbType.Varchar2).Value = detail["LOCATION_NO"];
                    cmd.Parameters.Add("P_PO_NO", OracleDbType.Varchar2).Value = detail["PO_NO"];
                    cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = detail["INVOICE_NO"];
                    cmd.Parameters.Add("P_ORDER_QTY", OracleDbType.Int16).Value = detail["ORDER_QTY"];
                    cmd.Parameters.Add("P_LEFT_INDENT", OracleDbType.Int16).Value = detail["LEFT_INDENT"];
                    cmd.Parameters.Add("P_REF_ORDER_ITEM_NO", OracleDbType.Int16).Value = detail["REF_ORDER_ITEM_NO"];
                    cmd.Parameters.Add("P_REF_ORDER_SUB_ITEM_NO", OracleDbType.Varchar2).Value = detail["REF_ORDER_SUB_ITEM_NO"];
                    cmd.Parameters.Add("P_REF_VOR_STATUS", OracleDbType.Varchar2).Value = detail["REF_VOR_STATUS"];
                    cmd.Parameters.Add("P_REF_ORDER_QTY", OracleDbType.Int16).Value = detail["REF_ORDER_QTY"];
                    cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = detail["VENDOR_CODE"];
                    cmd.Parameters.Add("P_VENDOR_TYPE", OracleDbType.Varchar2).Value = detail["VENDOR_TYPE"];
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_CLEAR_DATA", OracleDbType.Varchar2).Value = iCount == 1 ? "Y" : "N";

                    oOracledb.SqlQueryReturnValue(cmd);

                }

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE204_D_SAVE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = param["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = param["BRANCH_CODE"];
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = param["ORDER_NO"];
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_FORM", OracleDbType.Varchar2).Value = "SNSE024_D";
                cmd.Parameters.Add("P_VOR_STATUS", OracleDbType.Varchar2).Value = param["VOR_STATUS"];
                cmd.Parameters.Add("P_BR_LOCATION", OracleDbType.Varchar2).Value = param["BR_LOCATION"];
                cmd.Parameters.Add("P_BR_LOCATION_OTHER", OracleDbType.Varchar2).Value = param["BR_LOCATION_OTHER"];
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 255).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string BR_NO = cmd.Parameters["R_STATUS"].Value.ToString();


                return Json(new { status = "done", br_no = BR_NO });
            }
            catch (OracleException oex)
            {
                string err_massage = string.Empty;
                if (oex.Number == 20001)
                {
                    err_massage = "Borrow No is exist in system";
                }
                else if (oex.Number == 20002)
                {
                    err_massage = "Someone is working on this order, please try again!";
                }
                else if (oex.Number == 20003)
                {
                    err_massage = "PO Qty is now 0 !";
                }
                else if (oex.Number == 20004)
                {
                    err_massage = "PO Qty has been chagned Please Re-enter Form!";
                }
                else if (oex.Number == 20005)
                {
                    err_massage = "Can not insert into Order Status!";
                }
                else if (oex.Number == 20006)
                {
                    err_massage = "Can not insert into VOR Status!";
                }
                else if (oex.Number == 20007)
                {
                    err_massage = "Found an error, please contract support team";
                }
                oOracledb.WriteLog("ErrorHandle", oex.Number + " - " + err_massage, "SNSE204_D_Save");
                return Ok(new { status = "fail", massage = err_massage });
            }
            catch (Exception ex)
            {


                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE204_D_Save");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.NotAcceptable, ex.Message));
            }
        }

        #endregion

        #region SNSE205

        [HttpGet, Route("SNSE205_GetCustomer/{customerCode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE205_GetCustomer(string customerCode, int noofshowrec)
        {
            DataSet ds = new DataSet();

            customerCode = ismoUtil.escapeCharacter(customerCode);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE205_GET_CUST_CODE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = customerCode.ToUpper() == "NULL" ? (object)DBNull.Value : customerCode;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE205_GetCustomer");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE205_GetBranch/{branchCode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE205_GetBranch(string branchCode, int noofshowrec, [FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            branchCode = ismoUtil.escapeCharacter(branchCode);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE205_GET_BRANCH_CODE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = param["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = branchCode.ToUpper() == "NULL" ? (object)DBNull.Value : branchCode;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE205_GetBranch");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE205_GetOrderNo/{orderNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE205_GetOrderNo(string orderNo, int noofshowrec, [FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            orderNo = ismoUtil.escapeCharacter(orderNo);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE205_GET_ORDER_NO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = param["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = param["BRANCH_CODE"];
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = orderNo.ToUpper() == "NULL" ? (object)DBNull.Value : orderNo;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE205_GetOrderNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNSE205_GetBorrowNo/{brNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE205_GetBorrowNo(string brNo, int noofshowrec)
        {
            DataSet ds = new DataSet();

            brNo = ismoUtil.escapeCharacter(brNo);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE205_GET_BR_NO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_BR_NO", OracleDbType.Varchar2).Value = brNo.ToUpper() == "NULL" ? (object)DBNull.Value : brNo;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE205_GetBorrowNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE205_GetBRList")]
        [Authorize]
        public IHttpActionResult SNSE205_GetBRList([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE205_GET_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = param["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = param["BRANCH_CODE"];
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = param["ORDER_NO"];
                cmd.Parameters.Add("P_BR_NO", OracleDbType.Varchar2).Value = param["BR_NO"];
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE205_GetData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNSE205_GetData")]
        [Authorize]
        public IHttpActionResult SNSE205_GetData([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE205_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = param["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = param["BRANCH_CODE"];
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = param["ORDER_NO"];
                cmd.Parameters.Add("P_BR_NO", OracleDbType.Varchar2).Value = param["BR_NO"];
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE205_GetData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE205_Save")]
        [Authorize]
        public IHttpActionResult SNSE205_Save([FromBody] dynamic[] pDataList)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();

            try
            {
                foreach (var data in pDataList)
                {
                    OracleCommand cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE205_SAVE";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = data.BRANCH_CODE;
                    cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = data.ORDER_NO;
                    cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Varchar2).Value = data.ITEM_NO;
                    cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = data.ORDER_PART_NO;
                    cmd.Parameters.Add("P_BR_NO", OracleDbType.Varchar2).Value = data.BR_NO;
                    cmd.Parameters.Add("P_BR_TYPE", OracleDbType.Varchar2).Value = data.BR_TYPE;
                    cmd.Parameters.Add("P_ORDER_QTY", OracleDbType.Varchar2).Value = data.ORDER_QTY;
                    cmd.Parameters.Add("P_ACCEPT_QTY", OracleDbType.Varchar2).Value = data.ACCEPT_QTY;
                    cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE205";
                    cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                    ds = oOracledb.SqlQueryDS_CMD(cmd);
                    dt = ds.Tables[0];
                }
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE205_Save");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        #endregion
        #region SNM100P
        [HttpPost, Route("SNM100P_CheckDuplicate")]
        [Authorize]
        public IHttpActionResult SNM100P_CheckDuplicate([FromBody] dynamic data)
        {
            try
            {
                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM100P_CHECK_DUPLICATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = data.ORDER_CLASS;
                cmd.Parameters.Add("C_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM100P_CheckDuplicate");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpPost, Route("SNM100P_Getdata")]
        [Authorize]
        public IHttpActionResult SNM100P_Getdata([FromBody] dynamic pParam)
        {
            try
            {
                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM100P_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2, DataService.GetIdentity(User, Cons.SS_COMPANY_CODE), ParameterDirection.Input);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("cReturn", OracleDbType.RefCursor).Direction = ParameterDirection.Output;


                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM100P_Getdata");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpGet, Route("SNM100P_GetOrderClass")]
        [Authorize]
        public IHttpActionResult SNM100P_GetOrderClass()
        {

            try
            {
                DataSet ds = new DataSet();

                string gSQL = "SP_NP_SNM100P_GET_ORDER_CLASS";

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = gSQL;
                cmd.CommandType = CommandType.StoredProcedure;
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2, DataService.GetIdentity(User, Cons.SS_COMPANY_CODE), ParameterDirection.Input);
                cmd.Parameters.Add("cReturn", OracleDbType.RefCursor).Direction = ParameterDirection.Output;


                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM100P_GetOrderClass");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }


        }

        [HttpGet, Route("SNM100P_GetCustomerCode/{customerCode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM100P_GetCustomerCode(string customerCode, int noofshowrec)
        {
            try
            {
                DataSet ds = new DataSet();
                customerCode = ismoUtil.escapeCharacter(customerCode);

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM100P_GET_CUSTOMER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = customerCode.ToUpper() == "NULL" ? (object)DBNull.Value : customerCode;
                //cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM100P_GetCustomerCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM100P_ETE_Factor_Save")]
        [Authorize]
        public IHttpActionResult SNM100P_ETE_Factor_Save([FromBody] dynamic pDataList)
        {
            try
            {
                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM100P_SAVE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pDataList.CUSTOMER_CODE;
                cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = pDataList.ORDER_CLASS;
                cmd.Parameters.Add("P_LEAD_TIME_ETE", OracleDbType.Varchar2).Value = pDataList.LEAD_TIME_ETE;
                cmd.Parameters.Add("P_CREATE_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM100P";
                cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_LAST_IP_ADDR", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("C_RE", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM100P_ETE_Factor_Save");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNM100P_ETE_Factor_Delete")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM100P_ETE_Factor_Delete([FromBody] dynamic pDataList)
        {
            try
            {
                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM100P_DELETE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pDataList.CUSTOMER_CODE;
                cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = pDataList.ORDER_CLASS;
                cmd.Parameters.Add("P_LEAD_TIME_ETE", OracleDbType.Varchar2).Value = pDataList.LEAD_TIME_ETE;
                cmd.Parameters.Add("P_CREATE_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM100P";
                cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_LAST_IP_ADDR", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("C_RE", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM100P_ETE_Factor_Delete");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }


        }
        #endregion

        #region SNM049P

        [HttpPost, Route("SNM049P_GetData")]
        [Authorize]
        public IHttpActionResult SNM049P_GetData([FromBody] dynamic data)
        {
            try
            {
                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM049P_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_COUNTRY_CODE", OracleDbType.Varchar2).Value = data["COUNTRY_CODE"];
                cmd.Parameters.Add("C_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM049P_GetData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM049P_CheckDuplicate")]
        [Authorize]
        public IHttpActionResult SNM049P_CheckDuplicate([FromBody] dynamic data)
        {
            try
            {
                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM049P_CHECK_DUPLICATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_COUNTRY_CODE", OracleDbType.Varchar2).Value = data["COUNTRY_CODE"];
                cmd.Parameters.Add("C_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM049P_CheckDuplicate");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM049P_Update")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM049P_Update([FromBody] dynamic pDataList)
        {
            try
            {
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();
                var x = HttpContext.Current.Request.UserHostAddress.ToString();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM049P_UPDATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_COUNTRY_CODE", OracleDbType.Varchar2).Value = pDataList.COUNTRY_CODE;
                cmd.Parameters.Add("P_COUNTRY_NAME_EN", OracleDbType.Varchar2).Value = pDataList.COUNTRY_NAME_EN;
                cmd.Parameters.Add("P_COUNTRY_NAME_TH", OracleDbType.Varchar2).Value = pDataList.COUNTRY_NAME_TH;
                cmd.Parameters.Add("P_CREATE_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM049P";
                cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_LAST_IP_ADDR", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("C_RE", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM049P_Update");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNM049P_Delete")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM049P_Delete([FromBody] dynamic pDataList)
        {
            try
            {
                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM049P_DELETE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_COUNTRY_CODE", OracleDbType.Varchar2).Value = pDataList.COUNTRY_CODE;
                cmd.Parameters.Add("P_CREATE_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM049P";
                cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_LAST_IP_ADDR", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("C_RE", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM049P_Delete");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpGet, Route("SNM049P_GetCountryCode/{countryCode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM049P_GetCountryCode(string countryCode, int noofshowrec)
        {
            try
            {
                DataSet ds = new DataSet();

                countryCode = ismoUtil.escapeCharacter(countryCode);

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM049P_GET_COUNTRY_CODE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_COUNTRY_CODE", OracleDbType.Varchar2).Value = countryCode.ToUpper() == "NULL" ? (object)DBNull.Value : countryCode;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM049P_GetCountryCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        #endregion

        #region NTAG_Z


        [HttpGet, Route("NTAG_Z_START_PROCESS/{floc}/{tloc}")]
        [Authorize]
        public IHttpActionResult NTAG_Z_START_PROCESS(string floc, string tloc)
        {

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_NTAGZ_START_PROCESS";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_FLOC", OracleDbType.Varchar2).Value = floc;
                cmd.Parameters.Add("P_TLOC", OracleDbType.Varchar2).Value = tloc;
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 100).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                return Json(new
                {

                    STATUS = cmd.Parameters["R_STATUS"].Value.ToString(),

                });

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "NTAG_Z_START_PROCESS");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }
        #endregion

        #region SNSE009M
        [HttpGet, Route("SNSE009M_GetCustomerCode/{customerCode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE009M_GetCustomerCode(string customerCode, int noofshowrec)
        {
            DataSet ds = new DataSet();

            customerCode = ismoUtil.escapeCharacter(customerCode);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE009M_CUSTOMER_CODE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = customerCode.ToUpper() == "NULL" ? (object)DBNull.Value : customerCode;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE009M_GetCustomerCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE009M_GetOrderNo/{orderNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE009M_GetOrderNo(string orderNo, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            orderNo = ismoUtil.escapeCharacter(orderNo);
            var pData = pParam["DATA"];

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE009M_ORDER_NO";
                cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = orderNo.ToUpper() == "NULL" ? (object)DBNull.Value : orderNo;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE009M_GetOrderNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE009M_getData")]
        [Authorize]
        public IHttpActionResult SNSE009M_getData([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE009M_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pParam.ORDER_NO;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE009M_getData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE009M_GetOrderNoRptFrom/{orderNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE009M_GetOrderNoRptFrom(string orderNo, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            orderNo = ismoUtil.escapeCharacter(orderNo);
            var pData = pParam["DATA"];

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE009M_ORDER_NO";
                cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = orderNo.ToUpper() == "NULL" ? (object)DBNull.Value : orderNo;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData["CUSTOMER_CODE_FROM"];
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE009M_GetOrderNoRptFrom");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE009M_GetOrderNoRptTo/{orderNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE009M_GetOrderNoRptTo(string orderNo, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            orderNo = ismoUtil.escapeCharacter(orderNo);
            var pData = pParam["DATA"];

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE009M_ORDER_NO";
                cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = orderNo.ToUpper() == "NULL" ? (object)DBNull.Value : orderNo;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData["CUSTOMER_CODE_TO"];
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE009M_GetOrderNoRptTo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpGet, Route("SNSE009M_GetPartNo/{partNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE009M_GetPartNo(string partNo, int noofshowrec)
        {
            DataSet ds = new DataSet();

            partNo = ismoUtil.escapeCharacter(partNo);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE009M_PART_NO";
                cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = partNo.ToUpper() == "NULL" ? (object)DBNull.Value : partNo;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE009M_GetPartNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE009M_ApproveOrder")]
        [Authorize]
        public IHttpActionResult SNSE009M_ApproveOrder([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE009M_APPROVE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pParam.ORDER_NO;
                cmd.Parameters.Add("P_CHK_CONF", OracleDbType.Varchar2).Value = pParam.CHK_CONF;
                cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE009_M";
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE009M_ApproveOrder");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE009M_Cancel")]
        [Authorize]
        public IHttpActionResult SNSE009M_Cancel([FromBody] dynamic[] pDataList)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();

            try
            {
                foreach (var data in pDataList)
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE009M_UPDATE_OS";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = data.ORDER_NO;
                    cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = data.ORDER_PART_NO;
                    cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Varchar2).Value = data.ITEM_NO;
                    cmd.Parameters.Add("P_RETAIL_PRICE_B", OracleDbType.Varchar2).Value = data.RETAIL_PRICE_B;
                    cmd.Parameters.Add("P_UNIT_PRICE_B", OracleDbType.Varchar2).Value = data.UNIT_PRICE_B;
                    cmd.Parameters.Add("P_CB_CANCEL", OracleDbType.Varchar2).Value = data.CB_CANCEL;
                    cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE009M";
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                    ds = oOracledb.SqlQueryDS_CMD(cmd);
                }

                if (pDataList.Length > 0)
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE009M_UPDATE_OH";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pDataList[0].CUSTOMER_CODE;
                    cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pDataList[0].ORDER_NO;
                    cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE009M";
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                    ds = oOracledb.SqlQueryDS_CMD(cmd);
                }

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE009M_Cancel");
                return Ok("Error : " + ex.Message);
            }

            return Ok("DONE");
        }


        #endregion

        #region SNM100P
        [HttpGet, Route("SNM100P_GetMasterCode/{masterCode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM100P_GetMasterCode(string masterCode, int noofshowrec)
        {
            DataSet ds = new DataSet();

            masterCode = ismoUtil.escapeCharacter(masterCode);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM100P_GET_MASTER_CODE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_MASTER_CODE", OracleDbType.Varchar2).Value = masterCode.ToUpper() == "NULL" ? (object)DBNull.Value : masterCode;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM100P_GetMasterCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        #endregion

        #region SNSE212

        [HttpGet, Route("SNSE212GetCustomerCode/{cusID}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE212GetCustomerCode(string cusID, int noofshowrec)
        {
            DataSet ds = new DataSet();

            cusID = ismoUtil.escapeCharacter(cusID);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE212_CUSTOMER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = cusID.ToUpper() == "NULL" ? (object)DBNull.Value : cusID;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE212GetCustomerCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE212GetOrderNo/{orderNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE212GetOrderNo(string orderNo, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            orderNo = ismoUtil.escapeCharacter(orderNo);
            var pData = pParam["DATA"];

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE212_ORDER_NO";
                cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = orderNo.ToUpper() == "NULL" ? (object)DBNull.Value : orderNo;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE212GetOrderNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE212getData")]
        [Authorize]
        public IHttpActionResult SNSE212getData([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE212_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pParam.ORDER_NO;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE212getData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }


        }

        [HttpPost, Route("SNSE212AllocateOrder")]
        [Authorize]
        public IHttpActionResult SNSE212AllocateOrder([FromBody] dynamic[] pDataList)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();

            try
            {
                foreach (var data in pDataList)
                {
                    OracleCommand cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE212_UNLOCK";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = data.ORDER_NO;
                    cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = data.ORDER_PART_NO;
                    cmd.Parameters.Add("P_ORDER_DATE", OracleDbType.Date).Value = data.ORDER_DATE;
                    cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE212";
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                    ds = oOracledb.SqlQueryDS_CMD(cmd);
                    dt = ds.Tables[0];
                }
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE212AllocateOrder");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }


        }
        #endregion

        #region SNSE019
        [HttpGet, Route("SNSE019_GetCustomer/{cusID}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE019_GetCustomer(string cusID, int noofshowrec)
        {
            DataSet ds = new DataSet();

            cusID = ismoUtil.escapeCharacter(cusID);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE019_CUSTOMER_CODE";
                cmd.Parameters.Add("pCOMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("pCUSTOMER_CODE", OracleDbType.Varchar2).Value = cusID.ToUpper() == "NULL" ? (object)DBNull.Value : cusID;
                cmd.Parameters.Add("pSHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE019_GetCustomer");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }


        }

        [HttpPost, Route("SNSE019_GetOrderNo/{orderNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE019_GetOrderNo(string orderNo, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            orderNo = ismoUtil.escapeCharacter(orderNo);
            var pData = pParam["DATA"];

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE019_ORDER_NO";
                cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = orderNo.ToUpper() == "NULL" ? (object)DBNull.Value : orderNo;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE019_GetOrderNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }


        }

        [HttpPost, Route("SNSE019_GetData")]
        [Authorize]
        public IHttpActionResult SNSE019_GetData([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNSE019_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_ORDER_NO_START", OracleDbType.Varchar2).Value = pParam.ORDER_NO_START;
                cmd.Parameters.Add("P_ORDER_NO_END", OracleDbType.Varchar2).Value = pParam.ORDER_NO_END;
                cmd.Parameters.Add("P_IP_ADDRESS", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE019_GetData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNSE019_GetData2")]
        [Authorize]
        public IHttpActionResult SNSE019_GetData2([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {

                OracleCommand cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNSE019_GET_DATA_2";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_IP_ADDRESS", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE019_GetData2");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpGet, Route("SNSE019_GetPickSeq")]
        [Authorize]
        public IHttpActionResult SNSE019_GetPickSeq()
        {
            DataSet ds = new DataSet();

            try
            {

                OracleCommand cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNSE019_PICK_LOT_SEQ";
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE019_GetPickSeq");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE019_Process")]
        [Authorize]
        public IHttpActionResult SNSE019_Process([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNSE019_PROCESS";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_IP_ADDRESS", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE019_Process");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }


        #endregion

        #region SNSE215
        [HttpGet, Route("SNSE215_GetCustomer/{cusID}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE215_GetCustomer(string cusID, int noofshowrec)
        {
            DataSet ds = new DataSet();

            cusID = ismoUtil.escapeCharacter(cusID);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE215_GET_CUSTOMER";
                cmd.Parameters.Add("pCOMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("pCUSTOMER_CODE", OracleDbType.Varchar2).Value = cusID.ToUpper() == "NULL" ? (object)DBNull.Value : cusID;
                cmd.Parameters.Add("pSHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE215_GetCustomer");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpGet, Route("SNSE215_GetCustomer2/{cusID}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE215_GetCustomer2(string cusID, int noofshowrec)
        {
            DataSet ds = new DataSet();

            cusID = ismoUtil.escapeCharacter(cusID);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE215_GET_CUSTOMER2";
                cmd.Parameters.Add("pCOMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("pCUSTOMER_CODE", OracleDbType.Varchar2).Value = cusID.ToUpper() == "NULL" ? (object)DBNull.Value : cusID;
                cmd.Parameters.Add("pSHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE215_GetCustomer2");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }


        }

        [HttpPost, Route("SNSE215_GetOrderNo/{orderNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE215_GetOrderNo(string orderNo, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            orderNo = ismoUtil.escapeCharacter(orderNo);
            var pData = pParam["DATA"];

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE215_GET_ORDER";
                cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = orderNo.ToUpper() == "NULL" ? (object)DBNull.Value : orderNo;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE215_GetOrderNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE215_GetData")]
        [Authorize]
        public IHttpActionResult SNSE215_GetData([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE215_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pParam.ORDER_NO;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE215_GetData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE215_Save")]
        [Authorize]
        [RunBatchGuard]
        public IHttpActionResult SNSE215_Save([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNSE215_INSERT";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pParam.ORDER_NO;
                cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE215_Save");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNSE215_Delete")]
        [Authorize]
        [RunBatchGuard]
        public IHttpActionResult SNSE215_Delete([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE215_DELETE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pParam.ORDER_NO;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE215_GetData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        #endregion

        #region SNSE010R
        [HttpGet, Route("SNSE010R_GetCustomer/{cusID}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE010R_GetCustomer(string cusID, int noofshowrec)
        {
            DataSet ds = new DataSet();

            cusID = ismoUtil.escapeCharacter(cusID);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE010R_CUSTOMER_CODE";
                cmd.Parameters.Add("pCOMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("pCUSTOMER_CODE", OracleDbType.Varchar2).Value = cusID.ToUpper() == "NULL" ? (object)DBNull.Value : cusID;
                cmd.Parameters.Add("pSHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE010R_GetCustomer");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }


        }

        [HttpPost, Route("SNSE010R_GetOrderNo/{orderNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE010R_GetOrderNo(string orderNo, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            orderNo = ismoUtil.escapeCharacter(orderNo);
            var pData = pParam["DATA"];

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE010R_ORDER_NO";
                cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = orderNo.ToUpper() == "NULL" ? (object)DBNull.Value : orderNo;
                cmd.Parameters.Add("P_CUSTOMER_CODE_START", OracleDbType.Varchar2).Value = pData["CUSTOMER_CODE_START"];
                cmd.Parameters.Add("P_CUSTOMER_CODE_END", OracleDbType.Varchar2).Value = pData["CUSTOMER_CODE_END"];
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE010R_GetOrderNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        #endregion

        #region SNM013P

        [HttpGet, Route("SNM013P_GetSalesChannel/{salesChannel}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM013P_GetSalesChannel(string salesChannel, int noofshowrec)
        {
            DataSet ds = new DataSet();
            salesChannel = ismoUtil.escapeCharacter(salesChannel);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM013P_GET_SALE_CHANNEL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_SALES_CHANNEL", OracleDbType.Varchar2).Value = salesChannel.ToUpper() == "NULL" ? (object)DBNull.Value : salesChannel;
                cmd.Parameters.Add("pSHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("CRETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM013P_GetSalesChannel");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM013P_GetData")]
        [Authorize]
        public IHttpActionResult SNM013P_GetData([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM013P_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_SALES_CHANNEL", OracleDbType.Varchar2).Value = pParam.SALES_CHANNEL;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM013P_GetData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM013P_Delete")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM013P_Delete([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM013P_DELETE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_SALES_CHANNEL", OracleDbType.Varchar2).Value = param.SALES_CHANNEL;
                cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM013P";
                cmd.Parameters.Add("P_IP_ADDRESS", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM013P_Delete");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM013P_Save")]
        [Authorize]
        public IHttpActionResult SNM013P_Save([FromBody] dynamic pDataList)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM013P_SAVE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_SALES_CHANNEL", OracleDbType.Varchar2).Value = pDataList.SALES_CHANNEL;
                cmd.Parameters.Add("P_CHANNEL_NAME", OracleDbType.Varchar2).Value = pDataList.CHANNEL_NAME;
                cmd.Parameters.Add("P_GROUP_CHANNEL", OracleDbType.Varchar2).Value = pDataList.GROUP_CHANNEL;
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM013P";
                cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_IP_ADDRESS", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("C_RE", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM013P_Save");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }


        }

        #endregion

        #region SNSE001_N
        [HttpPost, Route("SNSE215_ChangeStatus")]
        [Authorize]
        public IHttpActionResult SNSE215_ChangeStatus([FromBody] dynamic[] pDataList)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();

            try
            {
                foreach (var data in pDataList)
                {
                    OracleCommand cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE215_CHANGE_STATUS";
                    cmd.Parameters.Add("P_FILE_NAME", OracleDbType.Varchar2).Value = data.FILE_NAME;
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                    ds = oOracledb.SqlQueryDS_CMD(cmd);
                }
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE215_ChangeStatus");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        public string SNSE001_N_ClearTemp()
        {
            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSE001_N_DEL_TEMP";
            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return "DONE";
        }

        public string SNSE001_N_InsertTemp(string fileLocation)
        {
            string fileLocationN = fileLocation;
            var fileName = fileLocationN.Split('\\').Last();
            DataTable dt = new DataTable();
            SystemApiController sys = new SystemApiController();
            dt = sys.ReadDataExcel(fileLocationN);
            List<string> ErrList = new List<string>();
            int countrow = 1;



            foreach (DataRow row in dt.Rows)
            {

                string p1 = null;
                string p2 = null;
                string p3 = null;
                string p4 = null;
                string p5 = null;
                string p6 = null;
                string p7 = null;
                string p8 = null;
                string p9 = null;
                string p10 = null;
                string p11 = null;
                string p12 = null;
                string p13 = null;
                string p14 = null;
                string p15 = null;
                string p16 = null;
                string p17 = null;

                p1 = row["HEADER CARD NO"].ToString().Trim();
                p2 = row["HEADER COMPANY CODE1"].ToString().Trim();
                p3 = row["HEADER COMPANY CODE2"].ToString().Trim();
                p4 = row["HEADER BUYER CODE"].ToString().Trim();
                p5 = row["HEADER SERIAL NO"].ToString().Trim();
                p6 = row["HEADER INDENT NO"].ToString().Trim();
                p7 = row["HEADER SHIP CODE"].ToString().Trim();
                p8 = row["HEADER DRAGON ORDER DATE"].ToString().Trim();
                p9 = row["CARD NO"].ToString().Trim();
                p10 = row["COMPANY CODE1"].ToString().Trim();
                p11 = row["COMPANY CODE2"].ToString().Trim();
                p12 = row["BUYER CODE"].ToString().Trim();
                p13 = row["SERIAL NO"].ToString().Trim();
                p14 = row["ITEM"].ToString().Trim();
                p15 = row["PART NO"].ToString().Trim();
                p16 = row["ORDER QTY"].ToString().Trim();
                p17 = row["UNIT PRICE"].ToString().Trim();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE001_N_INS_TEMP";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_HEADER_CARD_NO", OracleDbType.Varchar2).Value = p1;
                cmd.Parameters.Add("P_HEADER_COMPANY_CODE_1", OracleDbType.Varchar2).Value = p2;
                cmd.Parameters.Add("P_HEADER_COMPANY_CODE_2", OracleDbType.Varchar2).Value = p3;
                cmd.Parameters.Add("P_HEADER_BUYER_CODE", OracleDbType.Varchar2).Value = p4;
                cmd.Parameters.Add("P_HEADER_SERIAL_NO", OracleDbType.Varchar2).Value = p5;
                cmd.Parameters.Add("P_HEADER_INDENT_NO", OracleDbType.Varchar2).Value = p6;
                cmd.Parameters.Add("P_HEADER_SHIP_CODE", OracleDbType.Varchar2).Value = p7;
                cmd.Parameters.Add("P_HEADER_DRAGON_ORDER_DATE", OracleDbType.Varchar2).Value = p8;
                cmd.Parameters.Add("P_CARD_NO", OracleDbType.Varchar2).Value = p9;
                cmd.Parameters.Add("P_COMPANY_CODE_1", OracleDbType.Varchar2).Value = p10;
                cmd.Parameters.Add("P_COMPANY_CODE_2", OracleDbType.Varchar2).Value = p11;
                cmd.Parameters.Add("P_BUYER_CODE", OracleDbType.Varchar2).Value = p12;
                cmd.Parameters.Add("P_SERIAL_NO", OracleDbType.Varchar2).Value = p13;
                cmd.Parameters.Add("P_ITEM", OracleDbType.Varchar2).Value = p14;
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = p15;
                cmd.Parameters.Add("P_ORDER_QTY", OracleDbType.Varchar2).Value = p16;
                cmd.Parameters.Add("P_UNIT_PRICE", OracleDbType.Varchar2).Value = p17;
                cmd.Parameters.Add("P_FILE_NAME", OracleDbType.Varchar2).Value = fileName;
                cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                countrow++;
            }

            return fileName;

        }

        [HttpGet, Route("SNSE001_N_GetTempData")]
        [Authorize]
        public IHttpActionResult SNSE001_N_GetTempData()
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE001_N_GET_TEMP_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE001_N_GetTempData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNSE001_N_Upload")]
        [Authorize]
        public IHttpActionResult SNSE001_N_Upload([FromBody] dynamic pDataList)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE001_N_RECEIVE_DRAGON";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE001_N_Upload");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNSE001N_getControlDate")]
        [Authorize]
        public IHttpActionResult SNSE001N_getControlDate()
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE001_N_CHK_DATE_DRG";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_TYPE_DG", OracleDbType.Varchar2).Value = "01";
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE001N_getControlDate");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpPost, Route("SNSE001N_UploadFiles")]
        [Authorize]
        public async Task<IHttpActionResult> SNSE001N_UploadFiles(HttpRequestMessage request)
        {
            if (!request.Content.IsMimeMultipartContent())
            {
                throw new HttpResponseException(HttpStatusCode.UnsupportedMediaType);
            }

            var reqParts = await request.Content.ReadAsMultipartAsync();
            //List<string> ErrorList = new List<string>();
            List<List<string>> ErrorList = new List<List<string>>();
            List<string> MessageList = new List<string>();

            try
            {
                string fileLocation = string.Empty;
                string fileLocation2 = string.Empty;

                SNSE001_N_ClearTemp();

                foreach (var file in reqParts.Contents)
                {
                    var buffer = await file.ReadAsByteArrayAsync();
                    string fileName = Path.GetFileNameWithoutExtension(file.Headers.ContentDisposition.FileName.Trim('\"'));
                    string ext = Path.GetExtension(file.Headers.ContentDisposition.FileName.Trim('\"'));
                    string FullfileName = String.Concat(fileName, "-", DateTime.Now.ToString("yyyyMMddHHmm"), ext);
                    string outputLocation = "~/tempFile/" + "SNSE001_N" + "/Process/";
                    string outputPath = System.Web.HttpContext.Current.Server.MapPath(outputLocation);
                    fileLocation = String.Concat(outputPath, FullfileName);
                    DataService.SaveBytesToFileByLocation(buffer
                                                    , fileLocation
                                                    , fileLocation);

                    string outputLocation2 = "~/tempFile/" + "SNSE001_N" + "/Backup/";
                    string outputPath2 = System.Web.HttpContext.Current.Server.MapPath(outputLocation2);
                    fileLocation2 = String.Concat(outputPath2, FullfileName);
                    DataService.SaveBytesToFileByLocation(buffer
                                                    , fileLocation2
                                                    , fileLocation2);
                    //PROCESS
                    //MessageList.Add(SNSE001_N_InsertDataFromExcelFile(fileLocation));
                    MessageList.Add(SNSE001_N_InsertTemp(fileLocation));
                    ErrorList.Add(MessageList);

                    //DELETE FILE IN PROCESS FOLDER AFTER PROCESSED.
                    //SNSE001_N_DeleteFileAfterProcess(fileLocation);
                }
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE001N_UploadFiles");
                throw new HttpResponseException(HttpStatusCode.NotAcceptable);
            }

            return Json(new { Error = ErrorList });
        }

        public string SNSE001_N_InsertDataFromExcelFile(string fileLocation)
        {
            string fileLocationN = fileLocation;
            var fileName = fileLocationN.Split('\\').Last();
            DataTable dt = new DataTable();
            SystemApiController sys = new SystemApiController();
            dt = sys.ReadDataExcel(fileLocationN);
            List<string> w1ErrList = new List<string>();
            int countrow = 1;
            foreach (DataRow row in dt.Rows)
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE001_N_RECEIVE_DRAGON";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_H_CARD_NO", OracleDbType.Varchar2).Value = row["HEADER CARD NO"].ToString().Trim();
                cmd.Parameters.Add("P_H_COMPANY2_CODE", OracleDbType.Varchar2).Value = (row["HEADER COMPANY CODE1"].ToString().Trim() + row["HEADER COMPANY CODE2"].ToString().Trim()).Trim();
                cmd.Parameters.Add("P_H_SUPP_SOURCE_COCE", OracleDbType.Varchar2).Value = row["HEADER SUPPLY CODE"].ToString().Trim();
                cmd.Parameters.Add("P_H_BUYER_CODE", OracleDbType.Varchar2).Value = row["HEADER BUYER CODE"].ToString().Trim();
                cmd.Parameters.Add("P_H_SERIAL_NO", OracleDbType.Varchar2).Value = row["HEADER SERIAL NO"].ToString().Trim();
                cmd.Parameters.Add("P_H_INDENT_NO", OracleDbType.Varchar2).Value = row["HEADER INDENT NO"].ToString().Trim();
                cmd.Parameters.Add("P_H_SHIP_M_CODE", OracleDbType.Varchar2).Value = row["HEADER SHIP CODE"].ToString().Trim();
                cmd.Parameters.Add("P_H_DRAGON_ORDER_DATE", OracleDbType.Varchar2).Value = row["HEADER DRAGON ORDER DATE"].ToString().Trim();
                cmd.Parameters.Add("P_H_SCHEDULE_RESERVER", OracleDbType.Varchar2).Value = row["HEADER SCHEDULE RESERVER"].ToString().Trim();
                cmd.Parameters.Add("P_H_ORDER_QTY", OracleDbType.Varchar2).Value = row["HEADER ORDER QTY"].ToString().Trim();
                cmd.Parameters.Add("P_H_PRICE_MONTH_DATE", OracleDbType.Varchar2).Value = row["HEADER PRICE MONTH DATE"].ToString().Trim();
                cmd.Parameters.Add("P_H_LARGE_VOLUMN", OracleDbType.Varchar2).Value = row["HEADER LARGE VOLUMN"].ToString().Trim();
                cmd.Parameters.Add("P_H_BO_REQUEST", OracleDbType.Varchar2).Value = row["HEADER BO REQUEST"].ToString().Trim();
                cmd.Parameters.Add("P_D_CARD_NO", OracleDbType.Varchar2).Value = row["CARD NO"].ToString().Trim();
                cmd.Parameters.Add("P_D_COMPANY2_CODE", OracleDbType.Varchar2).Value = (row["COMPANY CODE1"].ToString().Trim() + row["COMPANY CODE2"].ToString().Trim()).Trim();
                cmd.Parameters.Add("P_D_SUPP_SOURCE_COCE", OracleDbType.Varchar2).Value = row["SUPPLY CODE"].ToString().Trim();
                cmd.Parameters.Add("P_D_BUYER_CODE", OracleDbType.Varchar2).Value = row["BUYER CODE"].ToString().Trim();
                cmd.Parameters.Add("P_D_SERIAL_NO", OracleDbType.Varchar2).Value = row["SERIAL NO"].ToString().Trim();
                cmd.Parameters.Add("P_D_ITEM_NO", OracleDbType.Varchar2).Value = row["ITEM"].ToString().Trim();
                cmd.Parameters.Add("P_D_PART_NO", OracleDbType.Varchar2).Value = row["PART NO"].ToString().Trim();
                cmd.Parameters.Add("P_D_QUANTITY", OracleDbType.Varchar2).Value = row["ORDER QTY"].ToString().Trim();
                cmd.Parameters.Add("P_D_ICA_REQUEST", OracleDbType.Varchar2).Value = row["ICA REQUEST"].ToString().Trim();
                cmd.Parameters.Add("P_D_LOCAL_LOCATION_NO", OracleDbType.Varchar2).Value = row["LOCAL LOCATION NO"].ToString().Trim();
                cmd.Parameters.Add("P_D_CURRENCY_CODE", OracleDbType.Varchar2).Value = row["CURRENCY CODE"].ToString().Trim();
                cmd.Parameters.Add("P_D_UNIT_PRICE", OracleDbType.Varchar2).Value = row["UNIT PRICE"].ToString().Trim();
                cmd.Parameters.Add("P_D_HOLD_DATE", OracleDbType.Varchar2).Value = row["HOLD DATE"].ToString().Trim();
                cmd.Parameters.Add("P_D_FILLER", OracleDbType.Varchar2).Value = row["FILLER"].ToString().Trim();
                cmd.Parameters.Add("P_COMFIRM_FLAG", OracleDbType.Varchar2).Value = row["CONFIRM FLAG (Y/N)"].ToString().Trim();
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE001_N";
                cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_FILE_NAME", OracleDbType.Varchar2).Value = fileName;
                cmd.Parameters.Add("P_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("P_MESSAGE", OracleDbType.Varchar2, 100).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                countrow++;
            }

            return fileName;
        }

        [HttpPost, Route("SNSE001_N_getErrorData")]
        [Authorize]
        public IHttpActionResult SNSE001_N_getErrorData([FromBody] dynamic[] pDataList)
        {
            DataSet ds = new DataSet();
            DataSet ds2 = new DataSet();

            DataTable dt = new DataTable();
            DataTable dtCopy = new DataTable();
            var i = 1;
            foreach (var data in pDataList)
            {

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE001_N_GET_ERROR_MSG";
                cmd.Parameters.Add("P_FILE_NAME", OracleDbType.Varchar2).Value = data.FILE_NAME;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                dt = ds.Tables[0];
                dtCopy = dt.Copy();
                dtCopy.TableName = "Table" + i;

                ds2.Tables.Add(dtCopy);
                i++;
            }
            return Json(DTFM.convertToList(ds2));
        }

        public void SNSE001_N_DeleteFileAfterProcess(string fileLocation)
        {
            string fileLocationN = fileLocation;
            Directory.Delete(fileLocationN, true);
        }
        #endregion

        #region SNSE015_D
        [HttpGet, Route("SNSE015_D_GetCustomerCode/{customer}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE015_D_GetCustomerCode(string customer, int noofshowrec)
        {
            DataSet ds = new DataSet();
            customer = ismoUtil.escapeCharacter(customer);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE015_D_CUSTOMER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = customer.ToUpper() == "NULL" ? (object)DBNull.Value : customer;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("CRETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE001N_getControlDate");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }


        }

        [HttpPost, Route("SNSE015_D_GetOrderNo/{orderNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE015_D_GetOrderNo(string orderNo, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            orderNo = ismoUtil.escapeCharacter(orderNo);
            //var pData = pParam["DATA"];

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE015_D_ORDER_NO";
                cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = orderNo.ToUpper() == "NULL" ? (object)DBNull.Value : orderNo;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam["BRANCH_CODE"];
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE015_D_GetOrderNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }


        }

        [HttpPost, Route("SNSE015_D_getData")]
        [Authorize]
        public IHttpActionResult SNSE015_D_getData([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();

            try
            {
                cmd.CommandText = "SP_NP_SNSE015_D_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pParam.ORDER_NO;
                cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE015_D_getData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpPost, Route("SNSE015_D_Allocate")]
        [Authorize]
        public IHttpActionResult SNSE015_D_Allocate([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();

            try
            {
                cmd.CommandText = "SP_NP_SNSE015_D_ALLOCATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam.BRANCH_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pParam.ORDER_NO;
                cmd.Parameters.Add("P_ORDER_DATE", OracleDbType.Date).Value = pParam.ORDER_DATE;
                cmd.Parameters.Add("P_DSP_BRANCH_STATUS", OracleDbType.Varchar2).Value = pParam.DSP_BRANCH_STATUS;
                cmd.Parameters.Add("P_CHK_PRINT_PICKING_CARD", OracleDbType.Varchar2).Value = pParam.CHK_PRINT_PICKING_CARD;
                cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE015_D";
                cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = pParam.ORDER_CLASS;
                cmd.Parameters.Add("P_CUR_ROUTE_CODE", OracleDbType.Varchar2).Value = pParam.CUR_ROUTE_CODE;
                cmd.Parameters.Add("P_CUR_TRANSPORT_CODE", OracleDbType.Varchar2).Value = pParam.CUR_TRANSPORT_CODE;
                cmd.Parameters.Add("P_CHANGE_ROUTE_CUSTOMER", OracleDbType.Date).Value = pParam.CHANGE_ROUTE_CUSTOMER;
                cmd.Parameters.Add("P_CHANGE_ROUTE_BRANCH", OracleDbType.Varchar2).Value = pParam.CHANGE_ROUTE_BRANCH;
                cmd.Parameters.Add("P_ROUTE_CODE", OracleDbType.Varchar2).Value = pParam.ROUTE_CODE;
                cmd.Parameters.Add("P_TRANSPORT_CODE", OracleDbType.Varchar2).Value = pParam.TRANSPORT_CODE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE015_D_Allocate");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE015_D_GetChangeRouteCust/{cust}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE015_D_GetChangeRouteCust(string cust, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            cust = ismoUtil.escapeCharacter(cust);
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE015_D_GET_CHG_ROUTE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CHANGE_ROUTE_CUSTOMER", OracleDbType.Varchar2).Value = cust.ToUpper() == "NULL" ? (object)DBNull.Value : cust;
                cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = pParam["ORDER_CLASS"];
                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pParam["ORDER_TYPE"];
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE015_D_GetChangeRouteCust");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        #endregion

        #region SNSE015_W
        [HttpGet, Route("SNSE015_W_GetCustomerCode/{customer}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE015_W_GetCustomerCode(string customer, int noofshowrec)
        {
            DataSet ds = new DataSet();
            customer = ismoUtil.escapeCharacter(customer);
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE015_W_CUSTOMER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = customer.ToUpper() == "NULL" ? (object)DBNull.Value : customer;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("CRETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE001N_getControlDate");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNSE015_W_GetOrderNo/{orderNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE015_W_GetOrderNo(string orderNo, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            orderNo = ismoUtil.escapeCharacter(orderNo);
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE015_W_ORDER_NO";
                cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = orderNo.ToUpper() == "NULL" ? (object)DBNull.Value : orderNo;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam["BRANCH_CODE"];
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE015_W_GetOrderNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNSE015_W_getData")]
        [Authorize]
        public IHttpActionResult SNSE015_W_getData([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            try
            {
                cmd.CommandText = "SP_NP_SNSE015_W_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pParam.ORDER_NO;
                cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE015_W_getData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNSE015_W_Allocate")]
        [Authorize]
        public IHttpActionResult SNSE015_W_Allocate([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            try
            {
                cmd.CommandText = "SP_NP_SNSE015_W_ALLOCATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam.BRANCH_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pParam.ORDER_NO;
                cmd.Parameters.Add("P_ORDER_DATE", OracleDbType.Date).Value = pParam.ORDER_DATE;
                cmd.Parameters.Add("P_DSP_BRANCH_STATUS", OracleDbType.Varchar2).Value = pParam.DSP_BRANCH_STATUS;
                cmd.Parameters.Add("P_CHK_PRINT_PICKING_CARD", OracleDbType.Varchar2).Value = pParam.CHK_PRINT_PICKING_CARD;
                cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE015_W";
                cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = pParam.ORDER_CLASS;
                cmd.Parameters.Add("P_CUR_ROUTE_CODE", OracleDbType.Varchar2).Value = pParam.CUR_ROUTE_CODE;
                cmd.Parameters.Add("P_CUR_TRANSPORT_CODE", OracleDbType.Varchar2).Value = pParam.CUR_TRANSPORT_CODE;
                cmd.Parameters.Add("P_CHANGE_ROUTE_CUSTOMER", OracleDbType.Date).Value = pParam.CHANGE_ROUTE_CUSTOMER;
                cmd.Parameters.Add("P_CHANGE_ROUTE_BRANCH", OracleDbType.Varchar2).Value = pParam.CHANGE_ROUTE_BRANCH;
                cmd.Parameters.Add("P_ROUTE_CODE", OracleDbType.Varchar2).Value = pParam.ROUTE_CODE;
                cmd.Parameters.Add("P_TRANSPORT_CODE", OracleDbType.Varchar2).Value = pParam.TRANSPORT_CODE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE015_W_Allocate");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNSE015_W_GetChangeRouteCust/{cust}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE015_W_GetChangeRouteCust(string cust, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            cust = ismoUtil.escapeCharacter(cust);
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE015_W_GET_CHG_ROUTE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CHANGE_ROUTE_CUSTOMER", OracleDbType.Varchar2).Value = cust.ToUpper() == "NULL" ? (object)DBNull.Value : cust;
                cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = pParam["ORDER_CLASS"];
                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pParam["ORDER_TYPE"];
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE015_W_GetChangeRouteCust");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        #endregion
        #region SNJF003
        [HttpGet, Route("SNJF003_GetCustomerCode/{customer}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNJF003_GetCustomerCode(string customer, int noofshowrec)
        {
            DataSet ds = new DataSet();
            customer = ismoUtil.escapeCharacter(customer);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNJF003_GET_CUSTOMER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = customer.ToUpper() == "NULL" ? (object)DBNull.Value : customer;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNJF003_GetCustomerCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }


        }

        [HttpPost, Route("SNJF003_getInvoiceNo/{invoiceNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNJF003_getInvoiceNo(string invoiceNo, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            invoiceNo = ismoUtil.escapeCharacter(invoiceNo);
            var pData = pParam["DATA"];

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNJF003_GET_INOVICE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = invoiceNo.ToUpper() == "NULL" ? (object)DBNull.Value : invoiceNo;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pData["BRANCH_CODE"];
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNJF003_getInvoiceNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNJF003_getPreInvoice")]
        [Authorize]
        public IHttpActionResult SNJF003_getPreInvoice([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNJF003_GET_PRE_INOVICE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam["BRANCH_CODE"];
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pParam["INVOICE_NO"];
                cmd.Parameters.Add("P_INVOICE_DATE", OracleDbType.Date).Value = pParam["INVOICE_DATE"];
                cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = pParam["SENT_BY"];
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNJF003_getPreInvoice");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }


        }

        [HttpPost, Route("SNJF003_getData")]
        [Authorize]
        public IHttpActionResult SNJF003_getData([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNJF003_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam["BRANCH_CODE"];
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pParam["INVOICE_NO"];
                cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = pParam["SENT_BY"];
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNJF003_getData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNJF003_getDataDetail")]
        [Authorize]
        public IHttpActionResult SNJF003_getDataDetail([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNJF003_GET_DATA_DETAIL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam["BRANCH_CODE"];
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = pParam["CASE_NO"];
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNJF003_getDataDetail");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }


        }

        [HttpGet, Route("SNJF003_getFileName")]
        [Authorize]
        public IHttpActionResult SNJF003_getFileName()
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNJF003_GET_FILE_NAME";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PGM", OracleDbType.Varchar2).Value = "SNJF003";
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNJF003_getFileName");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }


        }

        [HttpPost, Route("SNJF003_UpdateInvoiceHeader")]
        [Authorize]
        public IHttpActionResult SNJF003_UpdateInvoiceHeader([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNJF003_UPD_TRANS_HEADER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pParam.INVOICE_NO;
                cmd.Parameters.Add("P_BL_AWB_DATE", OracleDbType.Date).Value = pParam.BL_AWB_DATE;
                cmd.Parameters.Add("P_BL_AWB_NO", OracleDbType.Varchar2).Value = pParam.BL_AWB_NO;
                cmd.Parameters.Add("P_VESSEL_FLIGHT_N", OracleDbType.Varchar2).Value = pParam.VESSEL_FLIGHT_N;
                cmd.Parameters.Add("P_EXCHANGE_RATE", OracleDbType.Varchar2).Value = pParam.EXCHANGE_RATE;
                cmd.Parameters.Add("P_INSURANCE_C", OracleDbType.Varchar2).Value = pParam.INSURANCE_C;
                cmd.Parameters.Add("P_FWD_CHARGE_C", OracleDbType.Varchar2).Value = pParam.FWD_CHARGE_C;
                cmd.Parameters.Add("P_FREIGH_CHARGE_C", OracleDbType.Varchar2).Value = pParam.FREIGH_CHARGE_C;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNJF003_UpdateInvoiceHeader");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNJF003_UpdateInvoiceDetail")]
        [Authorize]
        [RunBatchGuard]
        public IHttpActionResult SNJF003_UpdateInvoiceDetail([FromBody] dynamic[] pDataList)
        {
            DataSet ds = new DataSet();

            try
            {
                foreach (var data in pDataList)
                {
                    OracleCommand cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNJF003_PROCESS_UPDATE";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = data.BRANCH_CODE;
                    cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = data.CASE_NO;
                    cmd.Parameters.Add("P_PACK_DATE", OracleDbType.Date).Value = data.PACK_DATE;
                    cmd.Parameters.Add("P_CONTAINER_NO", OracleDbType.Varchar2).Value = data.CONTAINER_NO;
                    cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = data.INVOICE_NO;
                    cmd.Parameters.Add("P_INVOICE_DATE", OracleDbType.Date).Value = data.INVOICE_DATE;
                    cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = data.SENT_BY;
                    cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                    ds = oOracledb.SqlQueryDS_CMD(cmd);

                }
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNJF003_UpdateInvoiceDetail");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNJF003_Dragon_Generate")]
        [Authorize]
        public IHttpActionResult SNJF003_Dragon_Generate([FromBody] dynamic pData)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNJF003_INS_DRAGON";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
            cmd.Parameters.Add("P_DOCUMENT_NO", OracleDbType.Varchar2).Value = pData.INVOICE_NO;
            cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = pData.SENT_BY;
            cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
            cmd.Parameters.Add("C_DRAGON_HEADER", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
            cmd.Parameters.Add("C_DRAGON_DETAIL", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            DataTable dtProcessStatus = ds.Tables[0];

            if (dtProcessStatus.Rows[0]["STATUS"].ToString() == "DONE")
            {
                //Set data table for generate order file
                DataTable dtDragonFile = ds.Tables[1];
                DataTable dtDragonData = ds.Tables[2];

                string gFileDetailName = dtDragonFile.Rows[0]["FILE_NAME"].ToString();

                string gCategory = "SNJF003/DRG";

                //Write and get file path.
                string gDetailPath = DataService.WriteFile(dtDragonData, "TEXT", gCategory, gFileDetailName);

                var fileDetail = DataService.GetFileStream(gDetailPath, gFileDetailName);

                return Ok(new { Detail = new { filename = gFileDetailName, file = fileDetail } });
            }
            else
            {
                string errString = dtProcessStatus.Rows[0]["MESSAGE"].ToString();
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, errString));
            }
        }
        #endregion

        #region SNSE005_N

        [HttpPost, Route("SNSE005_N_Upload")]
        [Authorize]
        public async Task<IHttpActionResult> SNSE005_N_Upload(HttpRequestMessage request)
        {

            if (!request.Content.IsMimeMultipartContent())
            {
                throw new HttpResponseException(HttpStatusCode.UnsupportedMediaType);
            }

            var reqParts = await request.Content.ReadAsMultipartAsync();
            List<string> ErrorList = new List<string>();
            try
            {
                string fileLocation = string.Empty;
                foreach (var file in reqParts.Contents)
                {
                    var buffer = await file.ReadAsByteArrayAsync();
                    string fileName = Path.GetFileNameWithoutExtension(file.Headers.ContentDisposition.FileName.Trim('\"'));
                    string ext = Path.GetExtension(file.Headers.ContentDisposition.FileName.Trim('\"'));
                    string FullfileName = String.Concat(fileName, "-", DateTime.Now.ToString("yyyyMMddHHmm"), ext);
                    string outputLocation = "~/tempFile/" + "SNSE005_N" + "/";
                    string outputPath = System.Web.HttpContext.Current.Server.MapPath(outputLocation);
                    fileLocation = String.Concat(outputPath, FullfileName);

                    DataService.SaveBytesToFileByLocation(buffer
                                                    , fileLocation
                                                    , fileLocation);
                }


                ErrorList = SNSE005_N_InsertDataFromExcelFile(fileLocation);
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE005_N_Upload");
                return Ok("Error : " + ex.Message);
            }

            return Json(new { Error = ErrorList });
        }

        public string SNSE005_N_ClearTemp()
        {
            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSE005_N_DEL_TEMP";
            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return "DONE";
        }


        public List<string> SNSE005_N_InsertDataFromExcelFile(string fileLocation)
        {

            string fileLocationN = fileLocation;
            DataTable dt = new DataTable();
            SystemApiController sys = new SystemApiController();
            dt = sys.ReadDataExcel(fileLocationN);
            List<string> ErrList = new List<string>();
            int countrow = 1;

            SNSE005_N_ClearTemp();

            int IntValue;
            decimal DecimalValue;

            foreach (DataRow row in dt.Rows)
            {
                string p1 = null;
                string p2 = null;
                string p3 = null;
                string p4 = null;
                p1 = row["ITEM_NO"].ToString().Trim();
                p2 = row["PART_NO"].ToString().Trim();
                p3 = row["ORDER_QTY"].ToString().Trim();
                p4 = row["UNIT_PRICE"].ToString().Trim();

                //Check Validate
                if (p3 != "" || p3 != null)
                {
                    if (int.TryParse(p3, out IntValue))
                    {
                        if (Int32.Parse(p3) < 0)
                        {
                            p3 = "0";
                        }
                    }
                    else if (Decimal.TryParse(p3, NumberStyles.Any, new CultureInfo("en-US"), out DecimalValue))
                    {
                        if (Decimal.Parse(p3) < 0)
                        {
                            p3 = "0";
                        }
                    }
                    else
                    {
                        p3 = "0";
                    }
                }
                else
                {
                    p3 = "0";
                }

                if (p4 != "" || p4 != null)
                {
                    if (int.TryParse(p4, out IntValue))
                    {
                        if (Int32.Parse(p4) < 0)
                        {
                            p4 = "0";
                        }
                    }
                    else if (Decimal.TryParse(p4, NumberStyles.Any, new CultureInfo("en-US"), out DecimalValue))
                    {
                        if (Decimal.Parse(p4) < 0)
                        {
                            p4 = "0";
                        }
                    }
                    else
                    {
                        p4 = "0";
                    }
                }
                else
                {
                    p4 = "0";
                }

                if (p1 != null && p2 != null && p3 != null)
                {
                    OracleCommand cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE005_N_INS_TEMP";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Varchar2).Value = p1;
                    cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = p2;
                    cmd.Parameters.Add("P_ORDER_QTY", OracleDbType.Varchar2).Value = p3;
                    cmd.Parameters.Add("P_UNIT_PRICE", OracleDbType.Varchar2).Value = p4;
                    cmd.Parameters.Add("P_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;

                    cmd = oOracledb.SqlQueryReturnValue(cmd);

                    //string status = cmd.Parameters["P_STATUS"].Value.ToString();

                    //if (status == "DONE")
                    //{
                    //    ErrList.Add("Row " + countrow + " Upload complete");
                    //}
                    //else if (status == "FAIL")
                    //{
                    //    ErrList.Add("Row " + countrow + " " + "Part No. :" + " " + row["Part No."].ToString().Trim() + " error please recheck");
                    //}
                }
                else
                {
                    ErrList.Add("Row " + countrow + " " + "Item No. :" + " " + row["ITEM_NO"].ToString().Trim() + " error please recheck");
                }

                countrow++;
            }

            return ErrList;

        }


        [HttpGet, Route("SNSE005_N_GetTempData")]
        [Authorize]
        public IHttpActionResult SNSE005_N_GetTempData()
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE005_N_GET_TEMP_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE005_N_GetTempData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }


        }

        [HttpGet, Route("SNSE005_N_GetCustomer/{customer}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE005_N_GetCustomer(string customer, int noofshowrec)
        {
            DataSet ds = new DataSet();
            customer = ismoUtil.escapeCharacter(customer);
            customer = customer.Trim();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE005_N_CUSTOMER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = customer.ToUpper() == "NULL" ? (object)DBNull.Value : customer;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE005_N_GetCustomer");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE005_N_GetOrder/{order}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE005_N_GetOrder(string order, int noofshowrec, [FromBody] dynamic param)
        {
            DataSet ds = new DataSet();
            order = ismoUtil.escapeCharacter(order);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE005_N_ORDER_NO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = param["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = order.ToUpper() == "NULL" ? (object)DBNull.Value : order;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE005_N_GetOrder");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpPost, Route("SNSE005_N_GetOrderInfo")]
        [Authorize]
        public IHttpActionResult SNSE005_N_GetOrderInfo([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE005_N_ORDER_NO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = param["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = param["ORDER_NO"];
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = 10;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE005_N_GetOrderInfo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }


        }



        //[HttpPost, Route("SNSE005_N_GetBranch/{branch}/{noofshowrec}")]
        //[Authorize]
        //public IHttpActionResult SNSE005_N_GetBranch(string branch, int noofshowrec, [FromBody] dynamic param)
        //{
        //    DataSet ds = new DataSet();
        //    branch = ismoUtil.escapeCharacter(branch);

        //    OracleCommand cmd = new OracleCommand();
        //    cmd.CommandText = "SP_NP_SNSE005_N_BRANCH";
        //    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
        //    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = param["CUSTOMER_CODE"];
        //    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = branch.ToUpper() == "NULL" ? (object)DBNull.Value : branch;
        //    //cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
        //    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
        //    ds = oOracledb.SqlQueryDS_CMD(cmd);
        //    return Json(DTFM.convertToList(ds));
        //}

        [HttpPost, Route("SNSE005_N_GetHeader")]
        [Authorize]
        public IHttpActionResult SNSE005_N_GetHeader([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE005_N_GET_HEADER";
                cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam.BRANCH_CODE;
                cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = pParam.ORDER_CLASS;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pParam.ORDER_NO;
                cmd.Parameters.Add("P_BO_FLAG", OracleDbType.Varchar2).Value = pParam.BO_FLAG;
                cmd.Parameters.Add("P_DELIVERY_CODE", OracleDbType.Varchar2).Value = pParam.DELIVERY_CODE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE005_N_GetHeader");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNSE005_N_GetHeaderInfo")]
        [Authorize]
        public IHttpActionResult SNSE005_N_GetHeaderInfo([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE005_N_HEADER_INFO";
                cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam.BRANCH_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pParam.ORDER_NO;
                cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = pParam.ORDER_CLASS;
                cmd.Parameters.Add("P_BO_FLAG", OracleDbType.Varchar2).Value = pParam.BO_FLAG;
                cmd.Parameters.Add("P_DELIVERY_CODE", OracleDbType.Varchar2).Value = pParam.DELIVERY_CODE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE005_N_GetHeaderInfo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE005_N_GetData")]
        [Authorize]
        public IHttpActionResult SNSE005_N_GetData([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE005_N_GET_DATA";
                cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam.BRANCH_CODE;

                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pParam.ORDER_NO;

                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                // cmd.Parameters.Add("C_RETURN2", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE005_N_GetData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE005_N_GetPartNo/{partNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE005_N_GetPartNo(string partNo, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            partNo = ismoUtil.escapeCharacter(partNo);
            var pData = pParam["DATA"];

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE005_N_PART";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_ORDER_DATE", OracleDbType.Varchar2).Value = pData.ORDER_DATE.ToString("dd-MM-yyyy");
                cmd.Parameters.Add("P_CURRENCY_CODE", OracleDbType.Varchar2).Value = pData.CURRENCY_CODE;
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = partNo.ToUpper() == "NULL" ? (object)DBNull.Value : partNo;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE005_N_GetPartNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }


        [HttpPost, Route("SNSE005_N_GetPartNoAdd/{partNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE005_N_GetPartNoAdd(string partNo, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            partNo = ismoUtil.escapeCharacter(partNo);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE005_N_GET_PART_NO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_ORDER_DATE", OracleDbType.Varchar2).Value = pParam.ORDER_DATE.ToString("yyyy-MM-dd");
                cmd.Parameters.Add("P_CURRENCY_CODE", OracleDbType.Varchar2).Value = pParam.CURRENCY_CODE;
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = partNo.ToUpper() == "NULL" ? (object)DBNull.Value : partNo;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE005_N_GetPartNoAdd");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }


        [HttpPost, Route("SNSE005_N_GetPartInfo")]
        [Authorize]
        public IHttpActionResult SNSE005_N_GetPartInfo([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE005_N_GET_PART_INFO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_ORDER_DATE", OracleDbType.Varchar2).Value = pParam.ORDER_DATE.ToString("yyyy-MM-dd");
                cmd.Parameters.Add("P_CURRENCY_CODE", OracleDbType.Varchar2).Value = pParam.CURRENCY_CODE;
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pParam.PART_NO;
                cmd.Parameters.Add("C_RETURN1", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN2", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE005_N_GetPartInfo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }


        }

        [HttpPost, Route("SNSE005_N_GetPartInfoUpload")]
        [Authorize]
        public IHttpActionResult SNSE005_N_GetPartInfoUpload([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE005_N_PART_INFO_UL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_ORDER_DATE", OracleDbType.Varchar2).Value = pParam.ORDER_DATE.ToString("yyyy-MM-dd");
                cmd.Parameters.Add("P_CURRENCY_CODE", OracleDbType.Varchar2).Value = pParam.CURRENCY_CODE;
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pParam.PART_NO;
                cmd.Parameters.Add("C_RETURN1", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN2", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE005_N_GetPartInfoUpload");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }


        }


        [HttpPost, Route("SNSE005_N_CheckUploadPart")]
        [Authorize]
        public IHttpActionResult SNSE005_N_CheckUploadPart([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE005_N_CHK_PART_UL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pParam.ORDER_NO;
                cmd.Parameters.Add("P_ORDER_DATE", OracleDbType.Varchar2).Value = pParam.ORDER_DATE.ToString("yyyy-MM-dd");
                cmd.Parameters.Add("P_CURRENCY_CODE", OracleDbType.Varchar2).Value = pParam.CURRENCY_CODE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE005_N_CheckUploadPart");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }


        }

        [HttpPost, Route("SNSE005_N_Add")]
        [Authorize]
        public IHttpActionResult SNSE005_N_Add([FromBody] dynamic[] pDataList)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();


            foreach (var data in pDataList)
            {
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE005_N_ADD_DETAIL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pDataList[0].CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pDataList[0].BRANCH_CODE;
                cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = pDataList[0].ORDER_CLASS;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pDataList[0].ORDER_NO;
                cmd.Parameters.Add("P_BO_FLAG", OracleDbType.Varchar2).Value = pDataList[0].BO_FLAG;
                cmd.Parameters.Add("P_DELIVERY_CODE", OracleDbType.Varchar2).Value = pDataList[0].DELIVERY_CODE;

                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pDataList[0].ORDER_TYPE;
                cmd.Parameters.Add("P_ORDER_DATE", OracleDbType.Date).Value = pDataList[0].ORDER_DATE;
                cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = pDataList[0].SENT_BY;

                cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Varchar2).Value = data.ITEM_NO;
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = data.PART_NO;
                cmd.Parameters.Add("P_ORDER_QTY", OracleDbType.Varchar2).Value = data.ORDER_QTY;
                cmd.Parameters.Add("P_STATUS", OracleDbType.Varchar2).Value = data.STATUS;
                cmd.Parameters.Add("P_RETAIL_PRICE_B", OracleDbType.Varchar2).Value = data.RETAIL_PRICE_B;
                cmd.Parameters.Add("P_RETAIL_PRICE_C", OracleDbType.Varchar2).Value = data.RETAIL_PRICE_C;
                cmd.Parameters.Add("P_UNIT_PRICE_B", OracleDbType.Varchar2).Value = data.UNIT_PRICE_B;
                cmd.Parameters.Add("P_UNIT_PRICE_C", OracleDbType.Varchar2).Value = data.UNIT_PRICE;
                cmd.Parameters.Add("P_RETAIL_AMT_B", OracleDbType.Varchar2).Value = data.RETAIL_AMT_B;
                cmd.Parameters.Add("P_RETAIL_AMT_C", OracleDbType.Varchar2).Value = data.RETAIL_AMT_C;
                cmd.Parameters.Add("P_NET_AMT_B", OracleDbType.Varchar2).Value = data.NET_AMT_B;
                cmd.Parameters.Add("P_NET_AMT_C", OracleDbType.Varchar2).Value = data.NET_AMT_C;
                cmd.Parameters.Add("P_AVERAGE_COST", OracleDbType.Varchar2).Value = data.AVERAGE_COST;
                cmd.Parameters.Add("P_STANDARD_COST", OracleDbType.Varchar2).Value = data.STANDARD_COST;
                cmd.Parameters.Add("P_AVERAGE_AMT", OracleDbType.Varchar2).Value = data.AVERAGE_AMT;
                cmd.Parameters.Add("P_STANDARD_AMT", OracleDbType.Varchar2).Value = data.STANDARD_AMT;
                cmd.Parameters.Add("P_LOCK_SALES", OracleDbType.Varchar2).Value = data.LOCK_SALES;

                cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE005_N";
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                if (ds.Tables.Count > 0)
                {
                    if (ds.Tables[0].Rows[0]["STATUS"].ToString() != "DONE")
                    {
                        string errString = "Error : " + ds.Tables[0].Rows[0]["STATUS"].ToString();
                        oOracledb.WriteLog("ErrorHandle", errString, "SNSE005_N_Add");
                        return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, errString));
                    }
                }
                else
                {
                    string errString = "Error : No data found";
                    oOracledb.WriteLog("ErrorHandle", errString, "SNSE005_N_Add");
                    return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, errString));
                }
            }

            try
            {
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE005_N_ADD_HEADER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pDataList[0].CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pDataList[0].BRANCH_CODE;
                cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = pDataList[0].ORDER_CLASS;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pDataList[0].ORDER_NO;
                cmd.Parameters.Add("P_BO_FLAG", OracleDbType.Varchar2).Value = pDataList[0].BO_FLAG;
                cmd.Parameters.Add("P_DELIVERY_CODE", OracleDbType.Varchar2).Value = pDataList[0].DELIVERY_CODE;

                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pDataList[0].ORDER_TYPE;
                cmd.Parameters.Add("P_ORDER_DATE", OracleDbType.Date).Value = pDataList[0].ORDER_DATE;

                cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE005_N";
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);



                if (ds.Tables.Count > 0)
                {
                    if (ds.Tables[0].Rows[0]["STATUS"].ToString() != "DONE")
                    {
                        string errString = "Error : " + ds.Tables[0].Rows[0]["STATUS"].ToString();
                        oOracledb.WriteLog("ErrorHandle", errString, "SNSE005_N_Add");
                        return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, errString));
                    }
                }
                else
                {
                    string errString = "Error : No data found";
                    oOracledb.WriteLog("ErrorHandle", errString, "SNSE005_N_Add");
                    return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, errString));
                }

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE005_N_Add");
                return Ok("Error : " + ex.Message);
            }

            return Ok("DONE");
        }

        [HttpPost, Route("SNSE215_ChangeStatusByOrder")]
        [Authorize]
        public IHttpActionResult SNSE215_ChangeStatusByOrder([FromBody] dynamic pData)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE215_CHANGE_STA_2";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pData.ORDER_NO;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE215_ChangeStatusByOrder");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }


        }

        [HttpPost, Route("SNSE005_N_Save")]
        [Authorize]
        public IHttpActionResult SNSE005_N_Save([FromBody] dynamic[] pDataList)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();

            try
            {
                foreach (var data in pDataList)
                {
                    OracleCommand cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE005_N_SAVE";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = data.BRANCH_CODE;
                    cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = data.ORDER_CLASS;
                    cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = data.ORDER_NO;
                    cmd.Parameters.Add("P_BO_FLAG", OracleDbType.Varchar2).Value = data.BO_FLAG;
                    cmd.Parameters.Add("P_DELIVERY_CODE", OracleDbType.Varchar2).Value = data.DELIVERY_CODE;
                    cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Varchar2).Value = data.ITEM_NO;
                    cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = data.PART_NO;
                    cmd.Parameters.Add("P_ORDER_QTY", OracleDbType.Varchar2).Value = data.ORDER_QTY;
                    cmd.Parameters.Add("P_STATUS", OracleDbType.Varchar2).Value = data.STATUS;
                    cmd.Parameters.Add("P_REASON_CODE", OracleDbType.Varchar2).Value = data.REASON_CODE;
                    cmd.Parameters.Add("P_RETAIL_PRICE_B", OracleDbType.Varchar2).Value = data.RETAIL_PRICE_B;
                    cmd.Parameters.Add("P_RETAIL_PRICE_C", OracleDbType.Varchar2).Value = data.RETAIL_PRICE_C;
                    cmd.Parameters.Add("P_UNIT_PRICE_B", OracleDbType.Varchar2).Value = data.UNIT_PRICE_B;
                    cmd.Parameters.Add("P_UNIT_PRICE_C", OracleDbType.Varchar2).Value = data.UNIT_PRICE;
                    cmd.Parameters.Add("P_RETAIL_AMT_B", OracleDbType.Varchar2).Value = data.RETAIL_AMT_B;
                    cmd.Parameters.Add("P_RETAIL_AMT_C", OracleDbType.Varchar2).Value = data.RETAIL_AMT_C;
                    cmd.Parameters.Add("P_NET_AMT_B", OracleDbType.Varchar2).Value = data.NET_AMT_B;
                    cmd.Parameters.Add("P_NET_AMT_C", OracleDbType.Varchar2).Value = data.NET_AMT_C;
                    cmd.Parameters.Add("P_AVERAGE_COST", OracleDbType.Varchar2).Value = data.AVERAGE_COST;
                    cmd.Parameters.Add("P_STANDARD_COST", OracleDbType.Varchar2).Value = data.STANDARD_COST;
                    cmd.Parameters.Add("P_AVERAGE_AMT", OracleDbType.Varchar2).Value = data.AVERAGE_AMT;
                    cmd.Parameters.Add("P_STANDARD_AMT", OracleDbType.Varchar2).Value = data.STANDARD_AMT;
                    cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE005_N";
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                    ds = oOracledb.SqlQueryDS_CMD(cmd);
                    dt = ds.Tables[0];
                }
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE005_N_Save");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }


        }

        [HttpPost, Route("SNSE005_N_Delete")]
        [Authorize]
        public IHttpActionResult SNSE005_N_Delete([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE005_N_DELETE_PART";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = param.CUSTOMER_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = param.ORDER_NO;
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = param.PART_NO;
                cmd.Parameters.Add("P_ORDER_QTY", OracleDbType.Varchar2).Value = param.ORDER_QTY;
                cmd.Parameters.Add("P_RETAIL_AMT_B", OracleDbType.Varchar2).Value = param.RETAIL_AMT_B;
                cmd.Parameters.Add("P_RETAIL_AMT_C", OracleDbType.Varchar2).Value = param.RETAIL_AMT_C;
                cmd.Parameters.Add("P_NET_AMT_B", OracleDbType.Varchar2).Value = param.NET_AMT_B;
                cmd.Parameters.Add("P_NET_AMT_C", OracleDbType.Varchar2).Value = param.NET_AMT_C;
                cmd.Parameters.Add("P_AVERAGE_AMT", OracleDbType.Varchar2).Value = param.AVERAGE_AMT;
                cmd.Parameters.Add("P_STANDARD_AMT", OracleDbType.Varchar2).Value = param.STANDARD_AMT;
                cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE005_N";
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE005_N_Delete");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }


        }


        [HttpPost, Route("SNSE005_Upload")]
        [Authorize]
        public async Task<IHttpActionResult> SNSE005_Upload(HttpRequestMessage request)
        {

            DataTable dt = new DataTable();

            if (!request.Content.IsMimeMultipartContent())
            {
                throw new HttpResponseException(HttpStatusCode.UnsupportedMediaType);
            }

            var reqParts = await request.Content.ReadAsMultipartAsync();
            List<string> ErrorList = new List<string>();
            try
            {


                string fileLocation = string.Empty;
                foreach (var file in reqParts.Contents)
                {
                    var buffer = await file.ReadAsByteArrayAsync();
                    string fileName = Path.GetFileNameWithoutExtension(file.Headers.ContentDisposition.FileName.Trim('\"'));
                    string ext = Path.GetExtension(file.Headers.ContentDisposition.FileName.Trim('\"'));
                    string FullfileName = String.Concat(fileName, "-", DateTime.Now.ToString("yyyyMMddHHmm"), ext);
                    string outputLocation = "~/tempFile/" + "SNSE005" + "/";
                    string outputPath = System.Web.HttpContext.Current.Server.MapPath(outputLocation);
                    fileLocation = String.Concat(outputPath, FullfileName);

                    DataService.SaveBytesToFileByLocation(buffer
                                                    , fileLocation
                                                    , fileLocation);
                }

                //ErrorList = SNSE005_InsertDataFromExcelFile(fileLocation);


                SystemApiController sys = new SystemApiController();
                dt = sys.ReadDataExcel(fileLocation);

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE005_Upload");
                throw new HttpResponseException(HttpStatusCode.NotAcceptable);
            }

            return Json(DTFM.convertToList(dt));
        }



        public List<string> SNSE005_InsertDataFromExcelFile(string fileLocation)
        {
            string fileLocationN = fileLocation;
            DataTable dt = new DataTable();
            SystemApiController sys = new SystemApiController();
            dt = sys.ReadDataExcel(fileLocationN);
            List<string> ErrList = new List<string>();


            return ErrList;

        }


        #endregion

        #region SNBA007_A
        [HttpGet, Route("SNBA007_A_LockOrder/{jobID}/{batchID}")]
        [Authorize]
        public IHttpActionResult SNBA007_A_LockOrder(string jobID, string batchID)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNBA007_A_LOCK_WEB";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_JOB_ID", OracleDbType.Varchar2).Value = jobID;
                cmd.Parameters.Add("P_batchID", OracleDbType.Varchar2).Value = batchID;
                oOracledb.SqlQueryReturnValue(cmd);

            }
            catch (Exception ex)
            {
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

            return Ok();
        }
        [HttpGet, Route("SNBA007_A_UnLockOrder/{jobID}/{batchID}")]
        [Authorize]
        public IHttpActionResult SNBA007_A_UnLockOrder(string jobID, string batchID)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNBA007_A_UNLOCK_WEB";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_JOB_ID", OracleDbType.Varchar2).Value = jobID;
                cmd.Parameters.Add("P_batchID", OracleDbType.Varchar2).Value = batchID;
                oOracledb.SqlQueryReturnValue(cmd);
            }
            catch (Exception ex)
            {
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

            return Ok();
        }
        [HttpGet, Route("SNBA007_A_LockOrderCheckStock")]
        [Authorize]
        public IHttpActionResult SNBA007_A_LockOrderCheckStock()
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNBA007_A_LCK_WEB_CHK";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                oOracledb.SqlQueryReturnValue(cmd);
            }
            catch (Exception ex)
            {
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
            return Ok();
        }
        [HttpGet, Route("SNBA007_A_UnLockOrderCheckStock")]
        [Authorize]
        public IHttpActionResult SNBA007_A_UnLockOrderCheckStock()
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNBA007_A_ULCK_WEB_CHK";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                oOracledb.SqlQueryReturnValue(cmd);
            }
            catch (Exception ex)
            {
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
            return Ok();
        }
        #endregion

        #region SNSE003

        [HttpPost, Route("SNSE003_GetOrderList")]
        [Authorize]
        public IHttpActionResult SNSE003_GetOrderList([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE003_ORDER_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam.BRANCH_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pParam.ORDER_NO;
                cmd.Parameters.Add("P_ORDER_DATE", OracleDbType.Date).Value = pParam.ORDER_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(pParam.ORDER_DATE);
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pParam.INVOICE_NO;
                cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE003_ORDER_LIST");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE003_GetOrderDetail")]
        [Authorize]
        public IHttpActionResult SNSE003_GetOrderDetail([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE003_GET_DETAIL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pParam.ORDER_NO;
                cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("c_Return2", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE003_GET_DETAIL");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNSE003_GetCustomer/{cusID}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE003_GetCustomer(string cusID, int noofshowrec)
        {
            DataSet ds = new DataSet();

            cusID = ismoUtil.escapeCharacter(cusID);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE003_CUSTOMER";
                cmd.Parameters.Add("pCOMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("pCUSTOMER_CODE", OracleDbType.Varchar2).Value = cusID.ToUpper() == "NULL" ? (object)DBNull.Value : cusID;
                cmd.Parameters.Add("pSHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE003_CUSTOMER");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpGet, Route("SNSE003_GetSearchInv/{InvNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE003_GetSearchInv(string InvNo, int noofshowrec)
        {
            DataSet ds = new DataSet();
            InvNo = ismoUtil.escapeCharacter(InvNo);
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE003_GET_INVOICE_NO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = InvNo.ToUpper() == "NULL" ? (object)DBNull.Value : InvNo;
                cmd.Parameters.Add("P_SHOW_REC", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE003_GetSearchInv");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNSE003_GetSearchCust/{cusID}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE003_GetSearchCust(string cusID, int noofshowrec)
        {
            DataSet ds = new DataSet();

            cusID = ismoUtil.escapeCharacter(cusID);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE003_GET_CUST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = cusID.ToUpper() == "NULL" ? (object)DBNull.Value : cusID;
                cmd.Parameters.Add("P_SHOW_REC", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE003_GET_CUST");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE003_GetSearchBranch/{branchCode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE003_GetSearchBranch(string branchCode, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            var pData = pParam["DATA"];
            branchCode = ismoUtil.escapeCharacter(branchCode);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE003_GET_BRANCH";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = branchCode.ToUpper() == "NULL" ? (object)DBNull.Value : branchCode;
                cmd.Parameters.Add("P_SHOW_REC", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE003_GET_BRANCH");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE003_GetSearchOrder/{Order}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE003_GetSearchOrder(string Order, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            var pData = pParam["DATA"];
            Order = ismoUtil.escapeCharacter(Order);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE003_GET_ORDER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pData["BRANCH_CODE"];
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = Order.ToUpper() == "NULL" ? (object)DBNull.Value : Order;
                cmd.Parameters.Add("P_SHOW_REC", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE003_GET_ORDER");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNSE003_GetParts/{partNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE003_GetParts(string partNo, int noofshowrec)
        {
            DataSet ds = new DataSet();

            partNo = ismoUtil.escapeCharacter(partNo);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE003_GETPARTS";
                cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("p_PART_NO", OracleDbType.Varchar2).Value = (partNo != null && partNo == "null" ? "" : partNo);
                cmd.Parameters.Add("p_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE003_GETPARTS");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE003_GetCustomerBranch/{branchCode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE003_GetCustomerBranch(string branchCode, int noofshowrec, [FromBody] dynamic pCustomer)
        {
            DataSet ds = new DataSet();

            branchCode = ismoUtil.escapeCharacter(branchCode);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE003_CUSTOMER_BRANCH";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pCustomer["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = branchCode.ToUpper() == "NULL" ? (object)DBNull.Value : branchCode;
                cmd.Parameters.Add("P_SHOW_REC", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE003_CUSTOMER_BRANCH");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE003_CheckOrder")]
        [Authorize]
        public IHttpActionResult SNSE003_CheckOrder([FromBody] dynamic pParam)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE003_CHECK_ORDER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pParam.ORDER_NO;
                cmd.Parameters.Add("R_ERROR_MSG", OracleDbType.Varchar2, 255).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                if (cmd.Parameters["R_ERROR_MSG"].Value.ToString() != "Done")
                {
                    return Json(new
                    {
                        Result = "Fail",
                        Message = cmd.Parameters["R_ERROR_MSG"].Value.ToString()
                    });
                }

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE003_CHECK_ORDER");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

            return Ok();
        }

        [HttpGet, Route("SNSE003_SNSC003/{customerCode}/{orderDate}/{partNo}/{orderClass}/{vendorCode}")]
        [Authorize]
        public IHttpActionResult SNSE003_SNSC003(string customerCode, string orderDate, string partNo, string orderClass, string vendorCode)
        {
            decimal RETAIL_D
                , RETAIL_E
                , UNIT_D
                , UNIT_E
                , DISC_1
                , DISC_2
                , DISC_3
                , AVG_COST
                , STD_COST;
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SNSC003A";
                cmd.Parameters.Add("CUST_CODE", OracleDbType.Varchar2).Value = customerCode;
                cmd.Parameters.Add("ORDER_DATE", OracleDbType.Date).Value = Convert.ToDateTime(orderDate);
                cmd.Parameters.Add("PARTNO", OracleDbType.Varchar2).Value = partNo;
                cmd.Parameters.Add("OR_CLASS", OracleDbType.Varchar2).Value = orderClass;
                cmd.Parameters.Add("VD_CODE", OracleDbType.Varchar2).Value = vendorCode;
                cmd.Parameters.Add("OVERSEAFLAG", OracleDbType.Varchar2, 1).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("RETAIL_D", OracleDbType.Decimal).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("RETAIL_E", OracleDbType.Decimal).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("UNIT_D", OracleDbType.Decimal).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("UNIT_E", OracleDbType.Decimal).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("DISC_1", OracleDbType.Decimal).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("DISC_2", OracleDbType.Decimal).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("DISC_3", OracleDbType.Decimal).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("DISC_GROUP", OracleDbType.Varchar2, 3).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("DISC_CODE", OracleDbType.Varchar2, 2).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("AVG_COST", OracleDbType.Decimal).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("STD_COST", OracleDbType.Decimal).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                return Json(new
                {
                    OVERSEAFLAG = cmd.Parameters["OVERSEAFLAG"].Value.ToString(),
                    RETAIL_D = Decimal.TryParse(cmd.Parameters["RETAIL_D"].Value.ToString(), out RETAIL_D) ? RETAIL_D : 0,
                    RETAIL_E = Decimal.TryParse(cmd.Parameters["RETAIL_E"].Value.ToString(), out RETAIL_E) ? RETAIL_E : 0,
                    UNIT_D = Decimal.TryParse(cmd.Parameters["UNIT_D"].Value.ToString(), out UNIT_D) ? UNIT_D : 0,
                    UNIT_E = Decimal.TryParse(cmd.Parameters["UNIT_E"].Value.ToString(), out UNIT_E) ? UNIT_E : 0,
                    DISC_1 = Decimal.TryParse(cmd.Parameters["DISC_1"].Value.ToString(), out DISC_1) ? DISC_1 : 0,
                    DISC_2 = Decimal.TryParse(cmd.Parameters["DISC_2"].Value.ToString(), out DISC_2) ? DISC_2 : 0,
                    DISC_3 = Decimal.TryParse(cmd.Parameters["DISC_3"].Value.ToString(), out DISC_3) ? DISC_3 : 0,
                    DISC_GROUP = cmd.Parameters["DISC_GROUP"].Value.ToString(),
                    DISC_CODE = cmd.Parameters["DISC_CODE"].Value.ToString(),
                    AVG_COST = Decimal.TryParse(cmd.Parameters["AVG_COST"].Value.ToString(), out AVG_COST) ? AVG_COST : 0,
                    STD_COST = Decimal.TryParse(cmd.Parameters["STD_COST"].Value.ToString(), out STD_COST) ? STD_COST : 0
                });

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSC003A");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNSE003_GetCampaignSpecialDiscount")]
        [Authorize]
        public IHttpActionResult SNSE003_GetCampaignSpecialDiscount([FromBody] dynamic pData)
        {
            OracleCommand cmd;

            try
            {

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE_SALES_CMP_DEL_TEMP";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pData["ORDER_NO"];

                oOracledb.SqlQueryReturnValue(cmd);

                var partList = new List<dynamic>(pData.PART_LIST);
                string gPartListString = string.Join(",", partList.Select(p => p["ORDER_PART_NO"]));

                foreach (var part in partList)
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE_SALES_CMP_INS_TEMP";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData["CUSTOMER_CODE"];
                    cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pData["ORDER_NO"];
                    cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = part["ORDER_PART_NO"];
                    cmd.Parameters.Add("P_QTY", OracleDbType.Decimal).Value = part["ORDER_QTY"] ?? 0;

                    oOracledb.SqlQueryReturnValue(cmd);
                }

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE003_GET_CMP_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pData["ORDER_NO"];
                cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = pData["ORDER_CLASS"];
                cmd.Parameters.Add("P_ORDER_DATE", OracleDbType.Date).Value = Convert.ToDateTime(pData["ORDER_DATE"]);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pData["BRANCH_CODE"];
                cmd.Parameters.Add("P_PART_LIST", OracleDbType.Varchar2).Value = gPartListString;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE003_GetCampaignSpecialDiscount");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNSE003_SaveDocument")]
        [Authorize]
        [DateControlGuard, RunBatchGuard]
        public IHttpActionResult SNSE003_SaveDocument([FromBody] dynamic pData)
        {
            int iCount = 0;
            OracleCommand cmd = new OracleCommand();

            var partList = new List<dynamic>(pData.PART_LIST);

            if (partList.Count > 0)
            {
                //Run insert detail
                foreach (var detail in partList)
                {
                    iCount++;

                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE003_SAVE_DETAIL_TEMP";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pData.ORDER_NO;
                    cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Decimal).Value = iCount;
                    cmd.Parameters.Add("P_ORDER_QTY", OracleDbType.Decimal).Value = detail.ORDER_QTY;
                    cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = detail.ORDER_PART_NO;
                    cmd.Parameters.Add("P_OVERSEA_FLAG_C", OracleDbType.Varchar2).Value = detail.OVERSEA_FLAG_C;
                    cmd.Parameters.Add("P_RETAIL_PRICE_B", OracleDbType.Decimal).Value = detail.RETAIL_PRICE_B;
                    cmd.Parameters.Add("P_RETAIL_PRICE_C", OracleDbType.Decimal).Value = detail.RETAIL_PRICE_C;
                    cmd.Parameters.Add("P_UNIT_PRICE_B", OracleDbType.Decimal).Value = detail.UNIT_PRICE;
                    cmd.Parameters.Add("P_UNIT_PRICE_C", OracleDbType.Decimal).Value = detail.UNIT_PRICE_C;
                    cmd.Parameters.Add("P_UNIT_PRICE_EDP", OracleDbType.Decimal).Value = detail.UNIT_EDP;
                    cmd.Parameters.Add("P_DISCOUNT_RATE1", OracleDbType.Decimal).Value = detail.DISCOUNT_RATE1;
                    cmd.Parameters.Add("P_DISCOUNT_RATE2", OracleDbType.Decimal).Value = detail.DISCOUNT_RATE2;
                    cmd.Parameters.Add("P_DISCOUNT_RATE3", OracleDbType.Decimal).Value = detail.DISCOUNT_RATE3;
                    cmd.Parameters.Add("P_DISCOUNT_GROUP", OracleDbType.Varchar2).Value = detail.DISCOUNT_GROUP;
                    cmd.Parameters.Add("P_DISCOUNT_CODE", OracleDbType.Varchar2).Value = detail.DISCOUNT_CODE;
                    cmd.Parameters.Add("P_AVERAGE_COST", OracleDbType.Decimal).Value = detail.AVERAGE_COST;
                    cmd.Parameters.Add("P_STANDARD_COST", OracleDbType.Decimal).Value = detail.STANDARD_COST;
                    cmd.Parameters.Add("P_STATUS", OracleDbType.Varchar2).Value = detail.STATUS;
                    cmd.Parameters.Add("P_REASON_CODE", OracleDbType.Decimal).Value = detail.REASON_CODE;
                    cmd.Parameters.Add("P_CAMPAIGN_ID", OracleDbType.Varchar2).Value = detail.CAMPAIGN_ID;
                    cmd.Parameters.Add("P_CAMPAIGN_REF", OracleDbType.Varchar2).Value = detail.CAMPAIGN_REF;
                    cmd.Parameters.Add("P_CLEAR_DATA", OracleDbType.Varchar2).Value = iCount == 1 ? "Y" : "N";
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    oOracledb.SqlQueryReturnValue(cmd);

                }

                try
                {
                    //Insert transaction
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE003_SAVE_TRANSACTION";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_CUSTOMER_BRANCH", OracleDbType.Varchar2).Value = pData.BRANCH_CODE;
                    cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pData.ORDER_NO;
                    cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = pData.ORDER_CLASS;
                    cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pData.ORDER_TYPE;
                    cmd.Parameters.Add("P_ORDER_DATE", OracleDbType.Date).Value = Convert.ToDateTime(pData._ORDER_DATE);
                    cmd.Parameters.Add("P_DELIVERY_CODE", OracleDbType.Varchar2).Value = pData.DELIVERY_CODE;
                    cmd.Parameters.Add("P_ROUTE_CODE", OracleDbType.Varchar2).Value = pData.ROUTE_CODE;
                    cmd.Parameters.Add("P_TRANSPORT_CODE", OracleDbType.Varchar2).Value = pData.TRANSPORT_CODE;
                    cmd.Parameters.Add("P_DISCOUNT_GROUP", OracleDbType.Varchar2).Value = pData.DISCOUNT_GROUP;
                    cmd.Parameters.Add("P_BO_FLAG", OracleDbType.Varchar2).Value = pData.BO_FLAG;
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                    oOracledb.SqlQueryReturnValue(cmd);

                    return Ok();

                }
                catch (Exception ex)
                {
                    oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE003_SAVE_TRANSACTION");
                    return ResponseMessage(Request.CreateResponse(HttpStatusCode.BadRequest, "Something went wrong."));
                }

            }

            return ResponseMessage(Request.CreateResponse(HttpStatusCode.BadRequest, "No row to process."));

        }

        [HttpPost, Route("SNSE003_CheckOilPart")]
        [Authorize]
        public IHttpActionResult SNSE003_CheckOilPart([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE003_CHECK_OIL_PART";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CustomerCode;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pData.BranchCode;
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pData.PartNo;
                cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = pData.OrderClass;
                cmd.Parameters.Add("R_MSG", OracleDbType.Varchar2, 500).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                return Json(new
                {
                    result = cmd.Parameters["R_MSG"].Value.ToString()
                });

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE003_CheckOilPart");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        #endregion

        #region SNSE004

        [HttpPost, Route("SNSE004_GetOrderList")]
        [Authorize]
        public IHttpActionResult SNSE004_GetOrderList([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE004_ORDER_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam.BRANCH_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pParam.ORDER_NO;
                cmd.Parameters.Add("P_ORDER_DATE", OracleDbType.Date).Value = pParam.ORDER_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(pParam.ORDER_DATE);
                cmd.Parameters.Add("P_DOCUMENT_NO", OracleDbType.Varchar2).Value = pParam.DOCUMENT_NO;
                cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE004_ORDER_LIST");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE004_GetOrderDetail")]
        [Authorize]
        public IHttpActionResult SNSE004_GetOrderDetail([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE004_GET_DETAIL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam.BRANCH_CODE;

                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pParam.ORDER_NO;
                cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("c_Return2", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE004_GET_DETAIL");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNSE004_GetBOList/{customerCode}/{branchCode}/{orderNo}/{orderDate}")]
        [Authorize]
        public IHttpActionResult SNSE004_GetBOList(string customerCode, string branchCode, string orderNo, string orderDate)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE004_BO_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = customerCode;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = branchCode;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = orderNo;
                cmd.Parameters.Add("P_ORDER_DATE", OracleDbType.Date).Value = Convert.ToDateTime(orderDate);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE004_BO_LIST");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNSE004_GetETPList/{customerCode}/{branchCode}/{orderNo}/{orderDate}")]
        [Authorize]
        public IHttpActionResult SNSE004_GetETPList(string customerCode, string branchCode, string orderNo, string orderDate)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE004_ETP_List";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = customerCode;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = branchCode;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = orderNo;
                cmd.Parameters.Add("P_ORDER_DATE", OracleDbType.Date).Value = Convert.ToDateTime(orderDate);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE004_ETP_List");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNSE004_GetCustomer/{cusID}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE004_GetCustomer(string cusID, int noofshowrec)
        {
            DataSet ds = new DataSet();

            cusID = ismoUtil.escapeCharacter(cusID);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE004_CUSTOMER";
                cmd.Parameters.Add("PCOMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("PCUSTOMER_CODE", OracleDbType.Varchar2).Value = cusID.ToUpper() == "NULL" ? (object)DBNull.Value : cusID;
                cmd.Parameters.Add("PSHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("CRETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE004_CUSTOMER");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNSE004_GetSearchCust/{cusID}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE004_GetSearchCust(string cusID, int noofshowrec)
        {
            DataSet ds = new DataSet();

            cusID = ismoUtil.escapeCharacter(cusID);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE004_GET_CUSTOMER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = cusID.ToUpper() == "NULL" ? (object)DBNull.Value : cusID;
                cmd.Parameters.Add("P_SHOW_REC", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE004_GET_CUSTOMER");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpPost, Route("SNSE004_GetParts/{partNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE004_GetParts(string partNo, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            partNo = ismoUtil.escapeCharacter(partNo);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE004_GETPARTS";
                cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam["BRANCH_CODE"];
                cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = pParam["ORDER_CLASS"];
                cmd.Parameters.Add("p_PART_NO", OracleDbType.Varchar2).Value = (partNo != null && partNo == "null" ? "" : partNo);
                cmd.Parameters.Add("p_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE004_GETPARTS");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        //[HttpGet, Route("SNSE004_GetParts/{partNo}/{noofshowrec}")]
        //[Authorize]
        //public IHttpActionResult SNSE004_GetParts(string partNo, int noofshowrec)
        //{
        //    DataSet ds = new DataSet();

        //    partNo = ismoUtil.escapeCharacter(partNo);

        //    try
        //    {
        //        OracleCommand cmd = new OracleCommand();
        //        cmd.CommandText = "SP_NP_SNSE004_GETPARTS";
        //        cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
        //        cmd.Parameters.Add("p_PART_NO", OracleDbType.Varchar2).Value = (partNo != null && partNo == "null" ? "" : partNo);
        //        cmd.Parameters.Add("p_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
        //        cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

        //        ds = oOracledb.SqlQueryDS_CMD(cmd);

        //        return Json(DTFM.convertToList(ds));
        //    }
        //    catch (Exception ex)
        //    {
        //        oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE004_GETPARTS");
        //        return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
        //    }
        //}

        [HttpPost, Route("SNSE004_GetSearchBrach/{branchCode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE004_GetSearchBrach(string branchCode, int noofshowrec, [FromBody] dynamic pData)
        {
            DataSet ds = new DataSet();
            branchCode = ismoUtil.escapeCharacter(branchCode);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE004_GET_BRANCH";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = branchCode.ToUpper() == "NULL" ? (object)DBNull.Value : branchCode;
                cmd.Parameters.Add("P_SHOW_REC", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE004_GET_BRANCH");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpPost, Route("SNSE004_GetBranchCode/{branchCode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE004_GetBranchCode(string branchCode, int noofshowrec, [FromBody] dynamic pCustomer)
        {
            DataSet ds = new DataSet();
            //var pData = pParam["DATA"];
            branchCode = ismoUtil.escapeCharacter(branchCode);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE004_BRANCH_CODE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pCustomer["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = branchCode.ToUpper() == "NULL" ? (object)DBNull.Value : branchCode;
                cmd.Parameters.Add("P_SHOW_REC", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE004_BRANCH_CODE");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE004_GetCustCodeOrderNo/{OrderNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE004_GetCustCodeOrderNo(string OrderNo, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            var pData = pParam["DATA"];
            OrderNo = ismoUtil.escapeCharacter(OrderNo);


            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE004_ORDER_NO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pData["BRANCH_CODE"];
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = OrderNo.ToUpper() == "NULL" ? (object)DBNull.Value : OrderNo;
                cmd.Parameters.Add("P_SHOW_REC", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE004_ORDER_NO");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNSE004_GetDocumentNo/{DocumentNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE004_GetDocumentNo(string DocumentNo, int noofshowrec)
        {
            DataSet ds = new DataSet();
            DocumentNo = ismoUtil.escapeCharacter(DocumentNo);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE004_GET_DOC";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DOCUMENT_NO", OracleDbType.Varchar2).Value = DocumentNo.ToUpper() == "NULL" ? (object)DBNull.Value : DocumentNo;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE004_GET_DOC");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE004_CheckOrder")]
        [Authorize]
        public IHttpActionResult SNSE004_CheckOrder([FromBody] dynamic pParam)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE004_CHECK_ORDER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pParam.ORDER_NO;
                cmd.Parameters.Add("R_ERROR_MSG", OracleDbType.Varchar2, 255).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                if (cmd.Parameters["R_ERROR_MSG"].Value.ToString() != "Done")
                {
                    return Json(new
                    {
                        Result = "Fail",
                        Message = cmd.Parameters["R_ERROR_MSG"].Value.ToString()
                    });
                }

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE004_CHECK_ORDER");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

            return Ok();
        }

        [HttpGet, Route("SNSE004_SNSC003/{customerCode}/{orderDate}/{partNo}/{orderClass}/{vendorCode}")]
        [Authorize]
        public IHttpActionResult SNSE004_SNSC003(string customerCode, string orderDate, string partNo, string orderClass, string vendorCode)
        {
            decimal RETAIL_D
                , RETAIL_E
                , UNIT_D
                , UNIT_E
                , DISC_1
                , DISC_2
                , DISC_3
                , AVG_COST
                , STD_COST;
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SNSC003A";
                cmd.Parameters.Add("CUST_CODE", OracleDbType.Varchar2).Value = customerCode;
                cmd.Parameters.Add("ORDER_DATE", OracleDbType.Date).Value = Convert.ToDateTime(orderDate);
                cmd.Parameters.Add("PARTNO", OracleDbType.Varchar2).Value = partNo;
                cmd.Parameters.Add("OR_CLASS", OracleDbType.Varchar2).Value = orderClass;
                cmd.Parameters.Add("VD_CODE", OracleDbType.Varchar2).Value = vendorCode;
                cmd.Parameters.Add("OVERSEAFLAG", OracleDbType.Varchar2, 1).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("RETAIL_D", OracleDbType.Decimal).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("RETAIL_E", OracleDbType.Decimal).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("UNIT_D", OracleDbType.Decimal).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("UNIT_E", OracleDbType.Decimal).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("DISC_1", OracleDbType.Decimal).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("DISC_2", OracleDbType.Decimal).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("DISC_3", OracleDbType.Decimal).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("DISC_GROUP", OracleDbType.Varchar2, 3).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("DISC_CODE", OracleDbType.Varchar2, 2).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("AVG_COST", OracleDbType.Decimal).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("STD_COST", OracleDbType.Decimal).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                return Json(new
                {
                    OVERSEAFLAG = cmd.Parameters["OVERSEAFLAG"].Value.ToString(),
                    RETAIL_D = Decimal.TryParse(cmd.Parameters["RETAIL_D"].Value.ToString(), out RETAIL_D) ? RETAIL_D : 0,
                    RETAIL_E = Decimal.TryParse(cmd.Parameters["RETAIL_E"].Value.ToString(), out RETAIL_E) ? RETAIL_E : 0,
                    UNIT_D = Decimal.TryParse(cmd.Parameters["UNIT_D"].Value.ToString(), out UNIT_D) ? UNIT_D : 0,
                    UNIT_E = Decimal.TryParse(cmd.Parameters["UNIT_E"].Value.ToString(), out UNIT_E) ? UNIT_E : 0,
                    DISC_1 = Decimal.TryParse(cmd.Parameters["DISC_1"].Value.ToString(), out DISC_1) ? DISC_1 : 0,
                    DISC_2 = Decimal.TryParse(cmd.Parameters["DISC_2"].Value.ToString(), out DISC_2) ? DISC_2 : 0,
                    DISC_3 = Decimal.TryParse(cmd.Parameters["DISC_3"].Value.ToString(), out DISC_3) ? DISC_3 : 0,
                    DISC_GROUP = cmd.Parameters["DISC_GROUP"].Value.ToString(),
                    DISC_CODE = cmd.Parameters["DISC_CODE"].Value.ToString(),
                    AVG_COST = Decimal.TryParse(cmd.Parameters["AVG_COST"].Value.ToString(), out AVG_COST) ? AVG_COST : 0,
                    STD_COST = Decimal.TryParse(cmd.Parameters["STD_COST"].Value.ToString(), out STD_COST) ? STD_COST : 0
                });

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE004_SNSC003A");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }


        [HttpPost, Route("SNSE004_GetCampaignSpecialDiscount")]
        [Authorize]
        public IHttpActionResult SNSE004_GetCampaignSpecialDiscount([FromBody] dynamic pParams)
        {
            decimal DISCOUNT_RATE1
                , DISCOUNT_RATE2
                , DISCOUNT_RATE3
                , UNIT_PRICE_B
                , UNIT_PRICE;

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE004_GET_CMP_DIS";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pParams.ORDER_NO;
                cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = pParams.ORDER_CLASS;
                cmd.Parameters.Add("P_ORDER_DATE", OracleDbType.Date).Value = Convert.ToDateTime(pParams.ORDER_DATE);
                cmd.Parameters.Add("P_SALES_CHANNEL", OracleDbType.Varchar2).Value = pParams.SALES_CHANNEL;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParams.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParams.BRANCH_CODE;
                cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = pParams.ORDER_PART_NO;
                cmd.Parameters.Add("P_ORDER_QTY", OracleDbType.Decimal).Value = pParams.ORDER_QTY;
                cmd.Parameters.Add("P_PART_LIST", OracleDbType.Varchar2).Value = pParams.PART_LIST;
                cmd.Parameters.Add("R_DISCOUNT_RATE1", OracleDbType.Decimal).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_DISCOUNT_RATE2", OracleDbType.Decimal).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_DISCOUNT_RATE3", OracleDbType.Decimal).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_UNIT_PRICE_B", OracleDbType.Decimal).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_UNIT_PRICE", OracleDbType.Decimal).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_DISCOUNT_CODE", OracleDbType.Varchar2, 10).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_CAMPAIGN_ID", OracleDbType.Varchar2, 10).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_CAMPAIGN_TYPE", OracleDbType.Varchar2, 1).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_CAMPAIGN_REF", OracleDbType.Varchar2, 10).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_DSP_CAMPAIGN_NAME", OracleDbType.Varchar2, 100).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                return Json(new
                {
                    DISCOUNT_RATE1 = Decimal.TryParse(cmd.Parameters["R_DISCOUNT_RATE1"].Value.ToString(), out DISCOUNT_RATE1) ? DISCOUNT_RATE1 : 0,
                    DISCOUNT_RATE2 = Decimal.TryParse(cmd.Parameters["R_DISCOUNT_RATE2"].Value.ToString(), out DISCOUNT_RATE2) ? DISCOUNT_RATE2 : 0,
                    DISCOUNT_RATE3 = Decimal.TryParse(cmd.Parameters["R_DISCOUNT_RATE3"].Value.ToString(), out DISCOUNT_RATE3) ? DISCOUNT_RATE3 : 0,
                    UNIT_PRICE_B = Decimal.TryParse(cmd.Parameters["R_UNIT_PRICE_B"].Value.ToString(), out UNIT_PRICE_B) ? UNIT_PRICE_B : 0,
                    UNIT_PRICE = Decimal.TryParse(cmd.Parameters["R_UNIT_PRICE"].Value.ToString(), out UNIT_PRICE) ? UNIT_PRICE : 0,
                    DISCOUNT_CODE = cmd.Parameters["R_DISCOUNT_CODE"].Value.ToString(),
                    CAMPAIGN_ID = cmd.Parameters["R_CAMPAIGN_ID"].Value.ToString(),
                    CAMPAIGN_TYPE = cmd.Parameters["R_CAMPAIGN_TYPE"].Value.ToString(),
                    CAMPAIGN_REF = cmd.Parameters["R_CAMPAIGN_REF"].Value.ToString(),
                    DSP_CAMPAIGN_NAME = cmd.Parameters["R_DSP_CAMPAIGN_NAME"].Value.ToString()
                });

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE004_GET_CMP_DIS");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNSE004_SaveDocument")]
        [Authorize]
        [DateControlGuard, RunBatchGuard]
        public IHttpActionResult SNSE004_SaveDocument([FromBody] dynamic pData)
        {
            int iCount = 0;
            OracleCommand cmd = new OracleCommand();

            var partList = new List<dynamic>(pData.PART_LIST);

            if (partList.Count > 0)
            {

                try
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE004_CHG_ROUTE";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pData.BRANCH_CODE;
                    cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pData.ORDER_NO;
                    cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = pData.ORDER_CLASS;
                    cmd.Parameters.Add("P_ORDER_DATE", OracleDbType.Date).Value = Convert.ToDateTime(pData._ORDER_DATE);
                    cmd.Parameters.Add("P_CUR_ROUTE_CODE", OracleDbType.Varchar2).Value = pData.CUR_ROUTE_CODE;
                    cmd.Parameters.Add("P_CUR_TRANSPORT_CODE", OracleDbType.Varchar2).Value = pData.CUR_TRANSPORT_CODE;
                    cmd.Parameters.Add("P_CHANGE_ROUTE_CUSTOMER", OracleDbType.Varchar2).Value = pData.CHANGE_ROUTE_CUSTOMER;
                    cmd.Parameters.Add("P_CHANGE_ROUTE_BRANCH", OracleDbType.Varchar2).Value = pData.CHANGE_ROUTE_BRANCH;
                    cmd.Parameters.Add("P_ROUTE_CODE", OracleDbType.Varchar2).Value = pData.ROUTE_CODE;
                    cmd.Parameters.Add("P_TRANSPORT_CODE", OracleDbType.Varchar2).Value = pData.TRANSPORT_CODE;
                    cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE004_M";
                    oOracledb.SqlQueryReturnValue(cmd);
                    //Run insert detail
                    foreach (var detail in partList)
                    {
                        iCount++;

                        cmd = new OracleCommand();
                        cmd.CommandText = "SP_NP_SNSE004_SAVE_DETAIL_TEMP";
                        cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                        cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                        cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pData.ORDER_NO;
                        cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Decimal).Value = iCount;
                        cmd.Parameters.Add("P_ORDER_QTY", OracleDbType.Decimal).Value = detail.ORDER_QTY;
                        cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = detail.ORDER_PART_NO;
                        cmd.Parameters.Add("P_SALES_PACKAGE", OracleDbType.Varchar2).Value = detail.SALES_PACKAGE;
                        cmd.Parameters.Add("P_OH_QTY", OracleDbType.Varchar2).Value = detail.OH_QTY;
                        cmd.Parameters.Add("P_OVERSEA_FLAG_C", OracleDbType.Varchar2).Value = detail.OVERSEA_FLAG_C;
                        cmd.Parameters.Add("P_RETAIL_PRICE_B", OracleDbType.Decimal).Value = detail.RETAIL_PRICE_B;
                        cmd.Parameters.Add("P_RETAIL_PRICE_C", OracleDbType.Decimal).Value = detail.RETAIL_PRICE_C;
                        cmd.Parameters.Add("P_UNIT_PRICE_B", OracleDbType.Decimal).Value = detail.UNIT_PRICE_B;
                        cmd.Parameters.Add("P_UNIT_PRICE_C", OracleDbType.Decimal).Value = detail.UNIT_PRICE_C;
                        cmd.Parameters.Add("P_UNIT_PRICE_EDP", OracleDbType.Decimal).Value = detail.UNIT_EDP;
                        cmd.Parameters.Add("P_DISCOUNT_RATE1", OracleDbType.Decimal).Value = detail.DISCOUNT_RATE1;
                        cmd.Parameters.Add("P_DISCOUNT_RATE2", OracleDbType.Decimal).Value = detail.DISCOUNT_RATE2;
                        cmd.Parameters.Add("P_DISCOUNT_RATE3", OracleDbType.Decimal).Value = detail.DISCOUNT_RATE3;
                        cmd.Parameters.Add("P_DISCOUNT_GROUP", OracleDbType.Varchar2).Value = detail.DISCOUNT_GROUP;
                        cmd.Parameters.Add("P_DISCOUNT_CODE", OracleDbType.Varchar2).Value = detail.DISCOUNT_CODE;
                        cmd.Parameters.Add("P_AVERAGE_COST", OracleDbType.Decimal).Value = detail.AVERAGE_COST;
                        cmd.Parameters.Add("P_STANDARD_COST", OracleDbType.Decimal).Value = detail.STANDARD_COST;
                        cmd.Parameters.Add("P_STATUS", OracleDbType.Varchar2).Value = detail.STATUS;
                        cmd.Parameters.Add("P_REASON_CODE", OracleDbType.Decimal).Value = detail.REASON_CODE;
                        cmd.Parameters.Add("P_CAMPAIGN_ID", OracleDbType.Varchar2).Value = detail.CAMPAIGN_ID;
                        cmd.Parameters.Add("P_CAMPAIGN_REF", OracleDbType.Varchar2).Value = detail.CAMPAIGN_REF;
                        cmd.Parameters.Add("P_CLEAR_DATA", OracleDbType.Varchar2).Value = iCount == 1 ? "Y" : "N";
                        cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                        oOracledb.SqlQueryReturnValue(cmd);

                    }

                    //Insert transaction
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE004_SAVE_TRANSACTION";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_CUSTOMER_BRANCH", OracleDbType.Varchar2).Value = pData.BRANCH_CODE;
                    cmd.Parameters.Add("P_CUSTOMER_NAME", OracleDbType.Varchar2).Value = pData.CUSTOMER_NAME;
                    cmd.Parameters.Add("P_CUST_ADDR1", OracleDbType.Varchar2).Value = pData.CUST_ADDR1;
                    cmd.Parameters.Add("P_CUST_ADDR2", OracleDbType.Varchar2).Value = pData.CUST_ADDR2;
                    cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pData.ORDER_NO;
                    cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = pData.ORDER_CLASS;
                    cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pData.ORDER_TYPE;
                    cmd.Parameters.Add("P_ORDER_DATE", OracleDbType.Date).Value = Convert.ToDateTime(pData._ORDER_DATE);
                    cmd.Parameters.Add("P_DELIVERY_CODE", OracleDbType.Varchar2).Value = pData.DELIVERY_CODE;
                    cmd.Parameters.Add("P_ROUTE_CODE", OracleDbType.Varchar2).Value = pData.ROUTE_CODE;
                    cmd.Parameters.Add("P_TRANSPORT_CODE", OracleDbType.Varchar2).Value = pData.TRANSPORT_CODE;
                    cmd.Parameters.Add("P_DISCOUNT_GROUP", OracleDbType.Varchar2).Value = pData.DISCOUNT_GROUP;
                    cmd.Parameters.Add("P_BO_FLAG", OracleDbType.Varchar2).Value = pData.BO_FLAG;
                    cmd.Parameters.Add("P_PRINT_PICKING_CARD", OracleDbType.Varchar2).Value = (pData.PRINT_PICKING_CARD == "true" ? "Y" : "N");
                    cmd.Parameters.Add("P_PRINT_REMARK", OracleDbType.Varchar2).Value = (pData.PRINT_REMARK == "true" ? "Y" : "N");
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_DOC_NO", OracleDbType.Varchar2, 20).Direction = ParameterDirection.Output;

                    oOracledb.SqlQueryReturnValue(cmd);

                    return Json(new
                    {
                        DOCUMENT_NO = cmd.Parameters["P_DOC_NO"].Value.ToString()
                    });


                }
                catch (Exception ex)
                {
                    oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE004_SAVE_TRANSACTION");
                    return ResponseMessage(Request.CreateResponse(HttpStatusCode.BadRequest, "Something went wrong."));
                }

            }

            return ResponseMessage(Request.CreateResponse(HttpStatusCode.BadRequest, "No row to process."));

        }


        [HttpPost, Route("SNSE004_A_SaveDocument")]
        [Authorize]
        [DateControlGuard, RunBatchGuard]
        public IHttpActionResult SNSE004_A_SaveDocument([FromBody] dynamic pData)
        {
            int iCount = 0;
            OracleCommand cmd = new OracleCommand();

            var partList = new List<dynamic>(pData.PART_LIST);

            if (partList.Count > 0)
            {

                try
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE004_CHG_ROUTE";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pData.BRANCH_CODE;
                    cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pData.ORDER_NO;
                    cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = pData.ORDER_CLASS;
                    cmd.Parameters.Add("P_ORDER_DATE", OracleDbType.Date).Value = Convert.ToDateTime(pData._ORDER_DATE);
                    cmd.Parameters.Add("P_CUR_ROUTE_CODE", OracleDbType.Varchar2).Value = pData.CUR_ROUTE_CODE;
                    cmd.Parameters.Add("P_CUR_TRANSPORT_CODE", OracleDbType.Varchar2).Value = pData.CUR_TRANSPORT_CODE;
                    cmd.Parameters.Add("P_CHANGE_ROUTE_CUSTOMER", OracleDbType.Varchar2).Value = pData.CHANGE_ROUTE_CUSTOMER;
                    cmd.Parameters.Add("P_CHANGE_ROUTE_BRANCH", OracleDbType.Varchar2).Value = pData.CHANGE_ROUTE_BRANCH;
                    cmd.Parameters.Add("P_ROUTE_CODE", OracleDbType.Varchar2).Value = pData.ROUTE_CODE;
                    cmd.Parameters.Add("P_TRANSPORT_CODE", OracleDbType.Varchar2).Value = pData.TRANSPORT_CODE;
                    cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE004_M";
                    oOracledb.SqlQueryReturnValue(cmd);
                    //Run insert detail
                    foreach (var detail in partList)
                    {
                        iCount++;

                        cmd = new OracleCommand();
                        cmd.CommandText = "SP_NP_SNSE004_SAVE_DETAIL_TEMP";
                        cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                        cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                        cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pData.ORDER_NO;
                        cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Decimal).Value = iCount;
                        cmd.Parameters.Add("P_ORDER_QTY", OracleDbType.Decimal).Value = detail.ORDER_QTY;
                        cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = detail.ORDER_PART_NO;
                        cmd.Parameters.Add("P_SALES_PACKAGE", OracleDbType.Varchar2).Value = detail.SALES_PACKAGE;
                        cmd.Parameters.Add("P_OH_QTY", OracleDbType.Varchar2).Value = detail.OH_QTY;
                        cmd.Parameters.Add("P_OVERSEA_FLAG_C", OracleDbType.Varchar2).Value = detail.OVERSEA_FLAG_C;
                        cmd.Parameters.Add("P_RETAIL_PRICE_B", OracleDbType.Decimal).Value = detail.RETAIL_PRICE_B;
                        cmd.Parameters.Add("P_RETAIL_PRICE_C", OracleDbType.Decimal).Value = detail.RETAIL_PRICE_C;
                        cmd.Parameters.Add("P_UNIT_PRICE_B", OracleDbType.Decimal).Value = detail.UNIT_PRICE_B;
                        cmd.Parameters.Add("P_UNIT_PRICE_C", OracleDbType.Decimal).Value = detail.UNIT_PRICE_C;
                        cmd.Parameters.Add("P_UNIT_PRICE_EDP", OracleDbType.Decimal).Value = detail.UNIT_EDP;
                        cmd.Parameters.Add("P_DISCOUNT_RATE1", OracleDbType.Decimal).Value = detail.DISCOUNT_RATE1;
                        cmd.Parameters.Add("P_DISCOUNT_RATE2", OracleDbType.Decimal).Value = detail.DISCOUNT_RATE2;
                        cmd.Parameters.Add("P_DISCOUNT_RATE3", OracleDbType.Decimal).Value = detail.DISCOUNT_RATE3;
                        cmd.Parameters.Add("P_DISCOUNT_GROUP", OracleDbType.Varchar2).Value = detail.DISCOUNT_GROUP;
                        cmd.Parameters.Add("P_DISCOUNT_CODE", OracleDbType.Varchar2).Value = detail.DISCOUNT_CODE;
                        cmd.Parameters.Add("P_AVERAGE_COST", OracleDbType.Decimal).Value = detail.AVERAGE_COST;
                        cmd.Parameters.Add("P_STANDARD_COST", OracleDbType.Decimal).Value = detail.STANDARD_COST;
                        cmd.Parameters.Add("P_STATUS", OracleDbType.Varchar2).Value = detail.STATUS;
                        cmd.Parameters.Add("P_REASON_CODE", OracleDbType.Decimal).Value = detail.REASON_CODE;
                        cmd.Parameters.Add("P_CAMPAIGN_ID", OracleDbType.Varchar2).Value = detail.CAMPAIGN_ID;
                        cmd.Parameters.Add("P_CAMPAIGN_REF", OracleDbType.Varchar2).Value = detail.CAMPAIGN_REF;
                        cmd.Parameters.Add("P_CLEAR_DATA", OracleDbType.Varchar2).Value = iCount == 1 ? "Y" : "N";
                        cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                        oOracledb.SqlQueryReturnValue(cmd);

                    }

                    //Insert transaction
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE004_A_SAVE_TRANS";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_CUSTOMER_BRANCH", OracleDbType.Varchar2).Value = pData.BRANCH_CODE;
                    cmd.Parameters.Add("P_CUSTOMER_NAME", OracleDbType.Varchar2).Value = pData.CUSTOMER_NAME;
                    cmd.Parameters.Add("P_CUST_ADDR1", OracleDbType.Varchar2).Value = pData.CUST_ADDR1;
                    cmd.Parameters.Add("P_CUST_ADDR2", OracleDbType.Varchar2).Value = pData.CUST_ADDR2;
                    cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pData.ORDER_NO;
                    cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = pData.ORDER_CLASS;
                    cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pData.ORDER_TYPE;
                    cmd.Parameters.Add("P_ORDER_DATE", OracleDbType.Date).Value = Convert.ToDateTime(pData._ORDER_DATE);
                    cmd.Parameters.Add("P_DELIVERY_CODE", OracleDbType.Varchar2).Value = pData.DELIVERY_CODE;
                    cmd.Parameters.Add("P_ROUTE_CODE", OracleDbType.Varchar2).Value = pData.ROUTE_CODE;
                    cmd.Parameters.Add("P_TRANSPORT_CODE", OracleDbType.Varchar2).Value = pData.TRANSPORT_CODE;
                    cmd.Parameters.Add("P_DISCOUNT_GROUP", OracleDbType.Varchar2).Value = pData.DISCOUNT_GROUP;
                    cmd.Parameters.Add("P_BO_FLAG", OracleDbType.Varchar2).Value = pData.BO_FLAG;
                    cmd.Parameters.Add("P_PRINT_PICKING_CARD", OracleDbType.Varchar2).Value = (pData.PRINT_PICKING_CARD == "true" ? "Y" : "N");
                    // cmd.Parameters.Add("P_PRINT_REMARK", OracleDbType.Varchar2).Value = (pData.PRINT_REMARK == "true" ? "Y" : "N");
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_STATUS", OracleDbType.Varchar2, 20).Direction = ParameterDirection.Output;

                    oOracledb.SqlQueryReturnValue(cmd);

                    return Json(new
                    {
                        STATUS_TEXT = cmd.Parameters["P_STATUS"].Value.ToString()
                    });


                }
                catch (Exception ex)
                {
                    oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE004_A_SaveDocument");
                    return ResponseMessage(Request.CreateResponse(HttpStatusCode.BadRequest, "Something went wrong."));
                }

            }

            return ResponseMessage(Request.CreateResponse(HttpStatusCode.BadRequest, "No row to process."));

        }


        [HttpPost, Route("SNSE004_CheckAcceptQuantity")]
        [Authorize]
        public IHttpActionResult SNSE004_CheckAcceptQuantity([FromBody] dynamic pParams)
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE004_SNSC001_SCR";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pParams.PART_NO;
                cmd.Parameters.Add("P_ORDER_QTY", OracleDbType.Varchar2).Value = pParams.ORDER_QTY;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParams.CUSTOMER_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pParams.ORDER_NO;
                cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Varchar2).Value = pParams.ITEM_NO;
                cmd.Parameters.Add("P_SUB_ITEM", OracleDbType.Varchar2).Value = pParams.SUB_ITEM;
                cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = pParams.ORDER_CLASS;
                cmd.Parameters.Add("P_STATUS", OracleDbType.Varchar2).Value = pParams.STATUS;
                cmd.Parameters.Add("P_BO_FLAG", OracleDbType.Varchar2).Value = pParams.BO_FLAG;
                cmd.Parameters.Add("P_AUTO_FLAG", OracleDbType.Varchar2).Value = pParams.AUTO_FLAG;
                cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE004_M";
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE004_CheckAcceptQuantity");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        //for SNSE004_D
        [HttpPost, Route("SNSE004_D_SaveDocument")]
        [Authorize]
        [DateControlGuard, RunBatchGuard]
        public IHttpActionResult SNSE004_D_SaveDocument([FromBody] dynamic pData)
        {
            int iCount = 0;
            OracleCommand cmd = new OracleCommand();

            var partList = new List<dynamic>(pData.PART_LIST);

            if (partList.Count > 0)
            {

                try
                {
                    //Run insert detail
                    foreach (var detail in partList)
                    {
                        iCount++;

                        cmd = new OracleCommand();
                        cmd.CommandText = "SP_NP_SNSE004_SAVE_DETAIL_TEMP";
                        cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                        cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                        cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pData.ORDER_NO;
                        cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Decimal).Value = iCount;
                        cmd.Parameters.Add("P_ORDER_QTY", OracleDbType.Decimal).Value = detail.ORDER_QTY;
                        cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = detail.ORDER_PART_NO;
                        cmd.Parameters.Add("P_SALES_PACKAGE", OracleDbType.Varchar2).Value = detail.SALES_PACKAGE;
                        cmd.Parameters.Add("P_OH_QTY", OracleDbType.Varchar2).Value = detail.OH_QTY;
                        cmd.Parameters.Add("P_OVERSEA_FLAG_C", OracleDbType.Varchar2).Value = detail.OVERSEA_FLAG_C;
                        cmd.Parameters.Add("P_RETAIL_PRICE_B", OracleDbType.Decimal).Value = detail.RETAIL_PRICE_B;
                        cmd.Parameters.Add("P_RETAIL_PRICE_C", OracleDbType.Decimal).Value = detail.RETAIL_PRICE_C;
                        cmd.Parameters.Add("P_UNIT_PRICE_B", OracleDbType.Decimal).Value = detail.UNIT_PRICE_B;
                        cmd.Parameters.Add("P_UNIT_PRICE_C", OracleDbType.Decimal).Value = detail.UNIT_PRICE_C;
                        cmd.Parameters.Add("P_UNIT_PRICE_EDP", OracleDbType.Decimal).Value = detail.UNIT_EDP;
                        cmd.Parameters.Add("P_DISCOUNT_RATE1", OracleDbType.Decimal).Value = detail.DISCOUNT_RATE1;
                        cmd.Parameters.Add("P_DISCOUNT_RATE2", OracleDbType.Decimal).Value = detail.DISCOUNT_RATE2;
                        cmd.Parameters.Add("P_DISCOUNT_RATE3", OracleDbType.Decimal).Value = detail.DISCOUNT_RATE3;
                        cmd.Parameters.Add("P_DISCOUNT_GROUP", OracleDbType.Varchar2).Value = detail.DISCOUNT_GROUP;
                        cmd.Parameters.Add("P_DISCOUNT_CODE", OracleDbType.Varchar2).Value = detail.DISCOUNT_CODE;
                        cmd.Parameters.Add("P_AVERAGE_COST", OracleDbType.Decimal).Value = detail.AVERAGE_COST;
                        cmd.Parameters.Add("P_STANDARD_COST", OracleDbType.Decimal).Value = detail.STANDARD_COST;
                        cmd.Parameters.Add("P_STATUS", OracleDbType.Varchar2).Value = detail.STATUS;
                        cmd.Parameters.Add("P_REASON_CODE", OracleDbType.Decimal).Value = detail.REASON_CODE;
                        cmd.Parameters.Add("P_CAMPAIGN_ID", OracleDbType.Varchar2).Value = detail.CAMPAIGN_ID;
                        cmd.Parameters.Add("P_CAMPAIGN_REF", OracleDbType.Varchar2).Value = detail.CAMPAIGN_REF;
                        cmd.Parameters.Add("P_CLEAR_DATA", OracleDbType.Varchar2).Value = iCount == 1 ? "Y" : "N";
                        cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                        oOracledb.SqlQueryReturnValue(cmd);

                    }

                    //Insert transaction
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE004_D_SAVE_TRANS";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_CUSTOMER_BRANCH", OracleDbType.Varchar2).Value = pData.BRANCH_CODE;
                    cmd.Parameters.Add("P_CUSTOMER_NAME", OracleDbType.Varchar2).Value = pData.CUSTOMER_NAME;
                    cmd.Parameters.Add("P_CUST_ADDR1", OracleDbType.Varchar2).Value = pData.CUST_ADDR1;
                    cmd.Parameters.Add("P_CUST_ADDR2", OracleDbType.Varchar2).Value = pData.CUST_ADDR2;
                    cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pData.ORDER_NO;
                    cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = pData.ORDER_CLASS;
                    cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pData.ORDER_TYPE;
                    cmd.Parameters.Add("P_ORDER_DATE", OracleDbType.Date).Value = Convert.ToDateTime(pData._ORDER_DATE);
                    cmd.Parameters.Add("P_DELIVERY_CODE", OracleDbType.Varchar2).Value = pData.DELIVERY_CODE;
                    cmd.Parameters.Add("P_ROUTE_CODE", OracleDbType.Varchar2).Value = pData.ROUTE_CODE;
                    cmd.Parameters.Add("P_TRANSPORT_CODE", OracleDbType.Varchar2).Value = pData.TRANSPORT_CODE;
                    cmd.Parameters.Add("P_DISCOUNT_GROUP", OracleDbType.Varchar2).Value = pData.DISCOUNT_GROUP;
                    cmd.Parameters.Add("P_BO_FLAG", OracleDbType.Varchar2).Value = pData.BO_FLAG;
                    cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pData.INVOICE_NO;
                    cmd.Parameters.Add("P_INVOICE_DATE", OracleDbType.Date).Value = Convert.ToDateTime(pData._INVOICE_DATE);
                    cmd.Parameters.Add("P_PRINT_PICKING_CARD", OracleDbType.Varchar2).Value = (pData.PRINT_PICKING_CARD == "true" ? "Y" : "N");
                    cmd.Parameters.Add("P_PRINT_REMARK", OracleDbType.Varchar2).Value = (pData.PRINT_REMARK == "true" ? "Y" : "N");
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_DOC_NO", OracleDbType.Varchar2, 20).Direction = ParameterDirection.Output;

                    oOracledb.SqlQueryReturnValue(cmd);

                    return Json(new
                    {
                        DOCUMENT_NO = cmd.Parameters["P_DOC_NO"].Value.ToString()
                    });


                }
                catch (Exception ex)
                {
                    oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE004_D_SAVE_TRANSACTION");
                    return ResponseMessage(Request.CreateResponse(HttpStatusCode.BadRequest, "Something went wrong."));
                }

            }

            return ResponseMessage(Request.CreateResponse(HttpStatusCode.BadRequest, "No row to process."));

        }
        [HttpPost, Route("SNSE004_GetChangeRouteCust/{cust}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE004_GetChangeRouteCust(string cust, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            cust = ismoUtil.escapeCharacter(cust);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE004_GET_CHG_ROUTE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CHANGE_ROUTE_CUSTOMER", OracleDbType.Varchar2).Value = cust.ToUpper() == "NULL" ? (object)DBNull.Value : cust;
                cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = pParam["ORDER_CLASS"];
                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pParam["ORDER_TYPE"];
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE004_GetChangeRouteCust");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNSE004_CheckDocument")]
        [Authorize]
        public IHttpActionResult SNSE004_CheckDocument([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE004_D_CHK_DO_EXIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DOCUMENT_NO", OracleDbType.Varchar2).Value = pParam.INVOICE_NO;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE004_CheckDocument");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        #endregion

        #region SNSQ022
        [HttpGet, Route("SNSQ022_GetCustomer/{cusID}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSQ022_GetCustomer(string cusID, int noofshowrec)
        {
            DataSet ds = new DataSet();

            cusID = ismoUtil.escapeCharacter(cusID);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ022_GET_CUSTOMER";
                cmd.Parameters.Add("pCOMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("pCUSTOMER_CODE", OracleDbType.Varchar2).Value = cusID.ToUpper() == "NULL" ? (object)DBNull.Value : cusID;
                cmd.Parameters.Add("pSHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ022_GetCustomer");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }


        }

        [HttpPost, Route("SNSQ022_GetOrderNo/{orderNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSQ022_GetOrderNo(string orderNo, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            orderNo = ismoUtil.escapeCharacter(orderNo);
            var pData = pParam["DATA"];

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ022_GET_ORDER_NO";
                cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = orderNo.ToUpper() == "NULL" ? (object)DBNull.Value : orderNo;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ022_GetOrderNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }


        }

        [HttpPost, Route("SNSQ022_GetOrderPartNo/{orderPartNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSQ022_GetOrderPartNo(string orderPartNo, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            orderPartNo = ismoUtil.escapeCharacter(orderPartNo);
            var pData = pParam["DATA"];

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ022_ORDER_PART_NO";
                cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = orderPartNo.ToUpper() == "NULL" ? (object)DBNull.Value : orderPartNo;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pData["ORDER_NO"];
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ022_GetOrderPartNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }


        }

        [HttpPost, Route("SNSQ022_GetOrderClass/{orderClass}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSQ022_GetOrderClass(string orderClass, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            orderClass = ismoUtil.escapeCharacter(orderClass);
            var pData = pParam["DATA"];

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ022_ORDER_CLASS";
                cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = orderClass.ToUpper() == "NULL" ? (object)DBNull.Value : orderClass;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pData["ORDER_NO"];
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ022_GetOrderClass");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNSQ022_ExportSummary")]
        public HttpResponseMessage SNSQ022_ExportSummary(List<DataModel.SNSQ022_SUMMARY> Data)
        {
            HttpResponseMessage response = new HttpResponseMessage(HttpStatusCode.OK);
            string gLocation = "~/tempFile/" + "SNSQ022" + "/";
            string gWriteLocation = System.Web.HttpContext.Current.Server.MapPath(gLocation);
            FileInfo filePath = new FileInfo(gWriteLocation);
            filePath.Directory.Create();
            string gFileLocation = Path.Combine(gWriteLocation, "SNSQ022_SUMMARY.xlsx");
            string gReturnLocation = Path.Combine(gLocation, "SNSQ022_SUMMARY.xlsx");

            var excel2 = Data.ToExcel(x => x
            .AddColumn("Seq", z => z.V_SEQ)
            .AddColumn("Customer Code", z => z.CUSTOMER_CODE)
            .AddColumn("Customer Name", z => z.CUSTOMER_NAME)
            .AddColumn("Sum Order No", z => z.CNT_ORDER_NO)
            .AddColumn("Sum Order Qty", z => z.SUM_ORDER_QTY)
            .AddColumn("Sum Accept Qty", z => z.SUM_ACCEPT_QTY)
            .AddColumn("Sum Net Weight (KG)", z => z.SUM_NET_WEIGHT)
            .AddColumn("Sum Net Volume (M3)", z => z.SUM_NET_DIMENSION)
            .AddColumn("Sum Amount", z => z.SUM_AMOUNT)
            .AddColumn("Status", z => z.DISPLAY_STATUS)
            .AddColumn("Unit Price", z => z.DISPLAY_UNIT_PRICE)
            );

            byte[] excel = excel2;
            File.WriteAllBytes(gFileLocation, excel);
            string path = HttpContext.Current.Server.MapPath("~/" + gReturnLocation.Substring(1));
            if (!File.Exists(path))
            {
                return Request.CreateErrorResponse(HttpStatusCode.NotFound, "File not found.");
            }
            try
            {
                string mimeType = MimeMapping.GetMimeMapping("SNSQ022_SUMMARY.xlsx");
                StreamContent x = new StreamContent(new FileStream(path, FileMode.Open, FileAccess.Read));
                response.Content = x;
                response.Content.Headers.ContentType = new MediaTypeHeaderValue(mimeType);
                response.Content.Headers.ContentDisposition = new ContentDispositionHeaderValue("attachment");
                response.Content.Headers.ContentDisposition.FileName = "SNSQ022_SUMMARY.xlsx";
            }
            catch (Exception ex)
            {
                return Request.CreateResponse(HttpStatusCode.BadRequest, ex);
            }
            return response;
        }

        [HttpPost, Route("SNSQ022_ExportIncomplete")]
        public HttpResponseMessage SNSQ022_ExportIncomplete(List<DataModel.SNSQ022_INCOMPLETE> Data)
        {
            HttpResponseMessage response = new HttpResponseMessage(HttpStatusCode.OK);
            string gLocation = "~/tempFile/" + "SNSQ022" + "/";
            string gWriteLocation = System.Web.HttpContext.Current.Server.MapPath(gLocation);
            FileInfo filePath = new FileInfo(gWriteLocation);
            filePath.Directory.Create();
            string gFileLocation = Path.Combine(gWriteLocation, "SNSQ022_INCOMPLETE.xlsx");
            string gReturnLocation = Path.Combine(gLocation, "SNSQ022_INCOMPLETE.xlsx");

            var excel2 = Data.ToExcel(x => x
            .AddColumn("Seq", z => z.V_SEQ)
            .AddColumn("Customer Code", z => z.CUSTOMER_CODE)
            .AddColumn("Order No.", z => z.ORDER_NO)
            .AddColumn("Serial No.", z => z.SERIAL_NO)
            .AddColumn("Item No.", z => z.V_ITEM_NO)
            .AddColumn("Accept Part No.", z => z.ACCEPT_PART_NO)
            .AddColumn("H.S. Code", z => z.TARIFF_CODE)
            .AddColumn("Orgin Name", z => z.ORIGIN_NAME)
            .AddColumn("Dangerous Goods", z => z.DG)
            .AddColumn("DIU Flag", z => z.DIU_FLAG)
            .AddColumn("Net Weight", z => z.NET_WEIGHT)
            .AddColumn("Dimension Max", z => z.DIMENSION_MAX)
            .AddColumn("Dimension Mid", z => z.DIMENSION_MID)
            .AddColumn("Dimension Min", z => z.DIMENSION_MIN)
            .AddColumn("Total Net Weight (KG)", z => z.AMT_NET_WEIGHT)
            .AddColumn("Total Volume (M3)", z => z.AMT_NET_DIMENSION)
            .AddColumn("Order Qty", z => z.ORDER_QTY)
            .AddColumn("Accept Qty", z => z.ACCEPT_QTY)
            .AddColumn("Accept Part Name", z => z.ACCEPT_PART_NAME)
            .AddColumn("Order Part No", z => z.ORDER_PART_NO)
            .AddColumn("Order Part Name", z => z.ORDER_PART_NAME)
            .AddColumn("Order Class", z => z.ORDER_CLASS)
            .AddColumn("Order Date", z => z.ORDER_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(z.ORDER_DATE).ToString("dd-MM-yyyy"))
            .AddColumn("Allocate Date", z => z.ALLOCATE_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(z.ALLOCATE_DATE).ToString("dd-MM-yyyy"))
            .AddColumn("ETD Date", z => z.ETD_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(z.ETD_DATE).ToString("dd-MM-yyyy"))
            );

            byte[] excel = excel2;
            //byte[] excel = Data.ToExcel();
            File.WriteAllBytes(gFileLocation, excel);
            string path = HttpContext.Current.Server.MapPath("~/" + gReturnLocation.Substring(1));
            if (!File.Exists(path))
            {
                return Request.CreateErrorResponse(HttpStatusCode.NotFound, "File not found.");
            }
            try
            {
                string mimeType = MimeMapping.GetMimeMapping("SNSQ022_INCOMPLETE.xlsx");
                StreamContent x = new StreamContent(new FileStream(path, FileMode.Open, FileAccess.Read));
                response.Content = x;
                response.Content.Headers.ContentType = new MediaTypeHeaderValue(mimeType);
                response.Content.Headers.ContentDisposition = new ContentDispositionHeaderValue("attachment");
                response.Content.Headers.ContentDisposition.FileName = "SNSQ022_INCOMPLETE.xlsx";
            }
            catch (Exception ex)
            {
                return Request.CreateResponse(HttpStatusCode.BadRequest, ex);
            }
            return response;
        }

        [HttpPost, Route("SNSQ022_ExportComplete")]
        public HttpResponseMessage SNSQ022_ExportComplete(List<DataModel.SNSQ022_COMPLETE> Data)
        {
            HttpResponseMessage response = new HttpResponseMessage(HttpStatusCode.OK);
            string gLocation = "~/tempFile/" + "SNSQ022" + "/";
            string gWriteLocation = System.Web.HttpContext.Current.Server.MapPath(gLocation);
            FileInfo filePath = new FileInfo(gWriteLocation);
            filePath.Directory.Create();
            string gFileLocation = Path.Combine(gWriteLocation, "SNSQ022_COMPLETE.xlsx");
            string gReturnLocation = Path.Combine(gLocation, "SNSQ022_COMPLETE.xlsx");

            var excel2 = Data.ToExcel(x => x
            .AddColumn("Seq", z => z.V_SEQ)
            .AddColumn("Customer Code", z => z.CUSTOMER_CODE)
            .AddColumn("Order No.", z => z.ORDER_NO)
            .AddColumn("Serial No.", z => z.SERIAL_NO)
            .AddColumn("Item No.", z => z.V_ITEM_NO)
            .AddColumn("Accept Part No.", z => z.ACCEPT_PART_NO)
            .AddColumn("H.S. Code", z => z.TARIFF_CODE)
            .AddColumn("Orgin Name", z => z.ORIGIN_NAME)
            .AddColumn("Dangerous Goods", z => z.DG)
            .AddColumn("DIU Flag", z => z.DIU_FLAG)
            .AddColumn("Net Weight", z => z.NET_WEIGHT)
            .AddColumn("Dimension Max", z => z.DIMENSION_MAX)
            .AddColumn("Dimension Mid", z => z.DIMENSION_MID)
            .AddColumn("Dimension Min", z => z.DIMENSION_MIN)
            .AddColumn("Total Net Weight (KG)", z => z.AMT_NET_WEIGHT)
            .AddColumn("Total Volume (M3)", z => z.AMT_NET_DIMENSION)
            .AddColumn("Order Qty", z => z.ORDER_QTY)
            .AddColumn("Accept Qty", z => z.ACCEPT_QTY)
            .AddColumn("Accept Part Name", z => z.ACCEPT_PART_NAME)
            .AddColumn("Order Part No", z => z.ORDER_PART_NO)
            .AddColumn("Order Part Name", z => z.ORDER_PART_NAME)
            .AddColumn("Order Class", z => z.ORDER_CLASS)
            .AddColumn("Order Date", z => z.ORDER_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(z.ORDER_DATE).ToString("dd-MM-yyyy"))
            .AddColumn("Allocate Date", z => z.ALLOCATE_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(z.ALLOCATE_DATE).ToString("dd-MM-yyyy"))
            .AddColumn("ETD Date", z => z.ETD_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(z.ETD_DATE).ToString("dd-MM-yyyy"))
            );

            byte[] excel = excel2;
            //byte[] excel = Data.ToExcel();
            File.WriteAllBytes(gFileLocation, excel);
            string path = HttpContext.Current.Server.MapPath("~/" + gReturnLocation.Substring(1));
            if (!File.Exists(path))
            {
                return Request.CreateErrorResponse(HttpStatusCode.NotFound, "File not found.");
            }
            try
            {
                string mimeType = MimeMapping.GetMimeMapping("SNSQ022_COMPLETE.xlsx");
                StreamContent x = new StreamContent(new FileStream(path, FileMode.Open, FileAccess.Read));
                response.Content = x;
                response.Content.Headers.ContentType = new MediaTypeHeaderValue(mimeType);
                response.Content.Headers.ContentDisposition = new ContentDispositionHeaderValue("attachment");
                response.Content.Headers.ContentDisposition.FileName = "SNSQ022_COMPLETE.xlsx";
            }
            catch (Exception ex)
            {
                return Request.CreateResponse(HttpStatusCode.BadRequest, ex);
            }
            return response;
        }

        [HttpPost, Route("SNSQ022_ExportUnitPriceFail")]
        public HttpResponseMessage SNSQ022_ExportUnitPriceFail(List<DataModel.SNSQ022_UNIT_FAIL> Data)
        {
            HttpResponseMessage response = new HttpResponseMessage(HttpStatusCode.OK);
            string gLocation = "~/tempFile/" + "SNSQ022" + "/";
            string gWriteLocation = System.Web.HttpContext.Current.Server.MapPath(gLocation);
            FileInfo filePath = new FileInfo(gWriteLocation);
            filePath.Directory.Create();
            string gFileLocation = Path.Combine(gWriteLocation, "SNSQ022_UNIT_FAIL.xlsx");
            string gReturnLocation = Path.Combine(gLocation, "SNSQ022_UNIT_FAIL.xlsx");

            var excel2 = Data.ToExcel(x => x
            .AddColumn("Seq.", z => z.V_SEQ)
            .AddColumn("Customer Code", z => z.CUSTOMER_CODE)
            .AddColumn("Order No.", z => z.ORDER_NO)
            .AddColumn("Order Class", z => z.ORDER_CLASS)
            .AddColumn("Serial No.", z => z.SERIAL_NO)
            .AddColumn("Item No.", z => z.V_ITEM_NO)
            .AddColumn("Accept Part No.", z => z.ACCEPT_PART_NO)
            .AddColumn("Vendor Code", z => z.VENDOR_CODE)
            .AddColumn("Order Qty", z => z.ORDER_QTY)
            .AddColumn("Accept Qty", z => z.ACCEPT_QTY)
            .AddColumn("Unit Price (Master)", z => z.UNIT_PRICE_MASTER)
            .AddColumn("Unit Price", z => z.UNIT_PRICE_C)
            .AddColumn("Amount Price", z => z.NET_AMT_C)
            .AddColumn("Accept Part Name", z => z.ACCEPT_PART_NAME)
            .AddColumn("Order Part No", z => z.ORDER_PART_NO)
            .AddColumn("Order Part Name", z => z.ORDER_PART_NAME)
            .AddColumn("Order Class", z => z.ORDER_CLASS)
            .AddColumn("Order Date", z => z.ORDER_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(z.ORDER_DATE).ToString("dd-MM-yyyy"))
            .AddColumn("Allocate Date", z => z.ALLOCATE_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(z.ALLOCATE_DATE).ToString("dd-MM-yyyy"))
            );



            byte[] excel = excel2;
            //byte[] excel = Data.ToExcel();
            File.WriteAllBytes(gFileLocation, excel);
            string path = HttpContext.Current.Server.MapPath("~/" + gReturnLocation.Substring(1));
            if (!File.Exists(path))
            {
                return Request.CreateErrorResponse(HttpStatusCode.NotFound, "File not found.");
            }
            try
            {
                string mimeType = MimeMapping.GetMimeMapping("SNSQ022_COMPLETE.xlsx");
                StreamContent x = new StreamContent(new FileStream(path, FileMode.Open, FileAccess.Read));
                response.Content = x;
                response.Content.Headers.ContentType = new MediaTypeHeaderValue(mimeType);
                response.Content.Headers.ContentDisposition = new ContentDispositionHeaderValue("attachment");
                response.Content.Headers.ContentDisposition.FileName = "SNSQ022_COMPLETE.xlsx";
            }
            catch (Exception ex)
            {
                return Request.CreateResponse(HttpStatusCode.BadRequest, ex);
            }
            return response;
        }

        [HttpPost, Route("SNSQ022_ExportIncompleteTariff")]
        public HttpResponseMessage SNSQ022_ExportIncompleteTariff(List<DataModel.SNSQ022_INCOMPLETE_TARIFF> Data)
        {
            HttpResponseMessage response = new HttpResponseMessage(HttpStatusCode.OK);
            string gLocation = "~/tempFile/" + "SNSQ022" + "/";
            string gWriteLocation = System.Web.HttpContext.Current.Server.MapPath(gLocation);
            FileInfo filePath = new FileInfo(gWriteLocation);
            filePath.Directory.Create();
            string gFileLocation = Path.Combine(gWriteLocation, "SNSQ022_INCOMPLETE_TARIFF.xlsx");
            string gReturnLocation = Path.Combine(gLocation, "SNSQ022_INCOMPLETE_TARIFF.xlsx");

            var excel2 = Data.ToExcel(x => x
            .AddColumn("Seq.", z => z.V_SEQ)
            .AddColumn("Customer Code", z => z.CUSTOMER_CODE)
            .AddColumn("Order No.", z => z.ORDER_NO)
            .AddColumn("Serial No.", z => z.SERIAL_NO)
            .AddColumn("Item No.", z => z.V_ITEM_NO)
            .AddColumn("Accept Part No.", z => z.ACCEPT_PART_NO)
            .AddColumn("Tariff Master", z => z.TARIFF_CODE)
            .AddColumn("Tariff Export", z => z.TARIFF_MASTER)
            .AddColumn("Remark", z => z.REMARK)
            );


            byte[] excel = excel2;
            //byte[] excel = Data.ToExcel();
            File.WriteAllBytes(gFileLocation, excel);
            string path = HttpContext.Current.Server.MapPath("~/" + gReturnLocation.Substring(1));
            if (!File.Exists(path))
            {
                return Request.CreateErrorResponse(HttpStatusCode.NotFound, "File not found.");
            }
            try
            {
                string mimeType = MimeMapping.GetMimeMapping("SNSQ022_INCOMPLETE_TARIFF.xlsx");
                StreamContent x = new StreamContent(new FileStream(path, FileMode.Open, FileAccess.Read));
                response.Content = x;
                response.Content.Headers.ContentType = new MediaTypeHeaderValue(mimeType);
                response.Content.Headers.ContentDisposition = new ContentDispositionHeaderValue("attachment");
                response.Content.Headers.ContentDisposition.FileName = "SNSQ022_INCOMPLETE_TARIFF.xlsx";
            }
            catch (Exception ex)
            {
                return Request.CreateResponse(HttpStatusCode.BadRequest, ex);
            }
            return response;
        }

        [HttpPost, Route("SNSQ022_ExportImproperMaster")]
        public HttpResponseMessage SNSQ022_ExportImproperMaster(List<DataModel.SNSQ022_IMPROPER_MASTER> Data)
        {
            HttpResponseMessage response = new HttpResponseMessage(HttpStatusCode.OK);
            string gLocation = "~/tempFile/" + "SNSQ022" + "/";
            string gWriteLocation = System.Web.HttpContext.Current.Server.MapPath(gLocation);
            FileInfo filePath = new FileInfo(gWriteLocation);
            filePath.Directory.Create();
            string gFileLocation = Path.Combine(gWriteLocation, "SNSQ022_IMPROPER_MASTER.xlsx");
            string gReturnLocation = Path.Combine(gLocation, "SNSQ022_IMPROPER_MASTER.xlsx");

            var excel2 = Data.ToExcel(x => x
            .AddColumn("Seq.", z => z.V_SEQ)
            .AddColumn("Part No.", z => z.PART_NO)
            .AddColumn("Tariff Master", z => z.TARIFF_CODE)
            .AddColumn("Tariff Export", z => z.TARIFF_MASTER)
            );

            byte[] excel = excel2;
            //byte[] excel = Data.ToExcel();
            File.WriteAllBytes(gFileLocation, excel);
            string path = HttpContext.Current.Server.MapPath("~/" + gReturnLocation.Substring(1));
            if (!File.Exists(path))
            {
                return Request.CreateErrorResponse(HttpStatusCode.NotFound, "File not found.");
            }
            try
            {
                string mimeType = MimeMapping.GetMimeMapping("SNSQ022_IMPROPER_MASTER.xlsx");
                StreamContent x = new StreamContent(new FileStream(path, FileMode.Open, FileAccess.Read));
                response.Content = x;
                response.Content.Headers.ContentType = new MediaTypeHeaderValue(mimeType);
                response.Content.Headers.ContentDisposition = new ContentDispositionHeaderValue("attachment");
                response.Content.Headers.ContentDisposition.FileName = "SNSQ022_IMPROPER_MASTER.xlsx";
            }
            catch (Exception ex)
            {
                return Request.CreateResponse(HttpStatusCode.BadRequest, ex);
            }
            return response;
        }


        [HttpPost, Route("SNSQ022_UpdateUnitPriceFail")]
        [Authorize]
        public IHttpActionResult SNSQ022_UpdateUnitPriceFail([FromBody] dynamic pDataList)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ022_UPD_UPF";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pDataList.CUSTOMER_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pDataList.ORDER_NO;
                cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Varchar2).Value = pDataList.ITEM_NO;
                cmd.Parameters.Add("P_UNIT_PRICE_MASTER", OracleDbType.Varchar2).Value = pDataList.UNIT_PRICE_MASTER;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ022_UpdateUnitPriceFail");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNSQ022_UpdateUnitPrice")]
        [Authorize]
        public IHttpActionResult SNSQ022_UpdateUnitPrice([FromBody] dynamic pDataList)
        {
            DataSet ds = new DataSet();

            try
            {
                foreach (var data in pDataList)
                {
                    OracleCommand cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSQ022_UPD_UPF";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = data.ORDER_NO;
                    cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Varchar2).Value = data.ITEM_NO;
                    cmd.Parameters.Add("P_UNIT_PRICE_MASTER", OracleDbType.Varchar2).Value = data.UNIT_PRICE_MASTER;

                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                    ds = oOracledb.SqlQueryDS_CMD(cmd);

                    // return Json(DTFM.convertToList(ds));

                    ds = oOracledb.SqlQueryDS_CMD(cmd);

                    if (ds.Tables.Count > 0)
                    {
                        if (ds.Tables[0].Rows[0]["STATUS"].ToString() != "DONE")
                        {
                            string errString = "Error : " + ds.Tables[0].Rows[0]["STATUS"].ToString();
                            oOracledb.WriteLog("ErrorHandle", data.ITEM_NO + " : " + errString, "SNSQ022_UpdateUnitPrice");
                            return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, errString));
                        }
                    }
                    else
                    {
                        string errString = "Error : No data found";
                        oOracledb.WriteLog("ErrorHandle", errString, "SNSQ022_UpdateUnitPrice");
                        return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, errString));
                    }
                }

                return Ok("DONE");
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ022_UpdateUnitPrice");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpPost, Route("SNSQ022_UpdateAllUnitPriceFail")]
        [Authorize]
        public IHttpActionResult SNSQ022_UpdateAllUnitPriceFail([FromBody] dynamic[] pDataList)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();

            try
            {
                foreach (var data in pDataList)
                {
                    OracleCommand cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSQ022_UPD_UPF";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = data.ORDER_NO;
                    cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Varchar2).Value = data.ITEM_NO;
                    cmd.Parameters.Add("P_UNIT_PRICE_MASTER", OracleDbType.Varchar2).Value = data.UNIT_PRICE_MASTER;
                    cmd.Parameters.Add("P_ACCEPT_QTY", OracleDbType.Varchar2).Value = data.ACCEPT_QTY;
                    cmd.Parameters.Add("P_EXCHANGE_RATE", OracleDbType.Varchar2).Value = data.EXCHANGE_RATE;
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                    ds = oOracledb.SqlQueryDS_CMD(cmd);
                    dt = ds.Tables[0];
                }
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ022_UpdateAllUnitPriceFail");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSQ022_UpdateTariffCode")]
        [Authorize]
        public IHttpActionResult SNSQ022_UpdateTariffCode([FromBody] dynamic[] pDataList)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();

            try
            {
                foreach (var data in pDataList)
                {
                    OracleCommand cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSQ022_UPD_TARIFF_CODE";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_TARIFF_MASTER", OracleDbType.Varchar2).Value = data.TARIFF_MASTER;
                    cmd.Parameters.Add("P_ACCEPT_PART_NO", OracleDbType.Varchar2).Value = data.ACCEPT_PART_NO;
                    cmd.Parameters.Add("P_SERIAL_NO", OracleDbType.Varchar2).Value = data.SERIAL_NO;
                    cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Varchar2).Value = data.ITEM_NO;
                    cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = data.ORDER_NO;
                    cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSQ022";
                    cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("C_RE", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                    ds = oOracledb.SqlQueryDS_CMD(cmd);
                    dt = ds.Tables[0];
                }
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ022_UpdateTariffCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }


        }

        [HttpPost, Route("SNSQ022_GetSummary")]
        [Authorize]
        public IHttpActionResult SNSQ022_GetSummary([FromBody] dynamic pDataList)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ022_GET_SUM";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pDataList.CUSTOMER_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pDataList.ORDER_NO;
                cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = pDataList.ORDER_PART_NO;
                cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = pDataList.ORDER_CLASS;
                cmd.Parameters.Add("P_FROM_DATE", OracleDbType.Date).Value = pDataList.FROM_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(pDataList.FROM_DATE);
                cmd.Parameters.Add("P_TO_DATE", OracleDbType.Date).Value = pDataList.TO_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(pDataList.TO_DATE);
                cmd.Parameters.Add("cReturn", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ022_GetSummary");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSQ022_GetInComplete")]
        [Authorize]
        public IHttpActionResult SNSQ022_GetInComplete([FromBody] dynamic pDataList)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ022_GET_INCOM";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pDataList.CUSTOMER_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pDataList.ORDER_NO;
                cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = pDataList.ORDER_PART_NO;
                cmd.Parameters.Add("P_FROM_DATE", OracleDbType.Date).Value = pDataList.FROM_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(pDataList.FROM_DATE);
                cmd.Parameters.Add("P_TO_DATE", OracleDbType.Date).Value = pDataList.TO_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(pDataList.TO_DATE);
                cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = pDataList.ORDER_CLASS;
                cmd.Parameters.Add("cReturn", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ022_GetInComplete");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSQ022_GetUnitPrice")]
        [Authorize]
        public IHttpActionResult SNSQ022_GetUnitPrice([FromBody] dynamic pDataList)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ022_GET_UNIP";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pDataList.CUSTOMER_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pDataList.ORDER_NO;
                cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = pDataList.ORDER_PART_NO;
                cmd.Parameters.Add("P_FROM_DATE", OracleDbType.Date).Value = pDataList.FROM_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(pDataList.FROM_DATE);
                cmd.Parameters.Add("P_TO_DATE", OracleDbType.Date).Value = pDataList.TO_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(pDataList.TO_DATE);
                cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = pDataList.ORDER_CLASS;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ022_GetUnitPrice");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }


        }

        [HttpPost, Route("SNSQ022_GetUnitPriceFail")]
        [Authorize]
        public IHttpActionResult SNSQ022_GetUnitPriceFail([FromBody] dynamic pDataList)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ022_GET_UNIP_FAIL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pDataList.CUSTOMER_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pDataList.ORDER_NO;
                cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = pDataList.ORDER_PART_NO;
                cmd.Parameters.Add("P_FROM_DATE", OracleDbType.Date).Value = pDataList.FROM_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(pDataList.FROM_DATE);
                cmd.Parameters.Add("P_TO_DATE", OracleDbType.Date).Value = pDataList.TO_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(pDataList.TO_DATE);
                cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = pDataList.ORDER_CLASS;
                cmd.Parameters.Add("cReturn", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ022_GetUnitPriceFail");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNSQ022_GetInCompleteTariff")]
        [Authorize]
        public IHttpActionResult SNSQ022_GetInCompleteTariff([FromBody] dynamic pDataList)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ022_GET_INCOM_TARIFF";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pDataList.CUSTOMER_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pDataList.ORDER_NO;
                cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = pDataList.ORDER_PART_NO;
                cmd.Parameters.Add("P_FROM_DATE", OracleDbType.Date).Value = pDataList.FROM_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(pDataList.FROM_DATE);
                cmd.Parameters.Add("P_TO_DATE", OracleDbType.Date).Value = pDataList.TO_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(pDataList.TO_DATE);
                cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = pDataList.ORDER_CLASS;
                cmd.Parameters.Add("cReturn", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ022_GetInCompleteTariff");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }


        }

        [HttpPost, Route("SNSQ022_GetComplete")]
        [Authorize]
        public IHttpActionResult SNSQ022_GetComplete([FromBody] dynamic pDataList)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ022_GET_COM";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pDataList.CUSTOMER_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pDataList.ORDER_NO;
                cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = pDataList.ORDER_PART_NO;
                cmd.Parameters.Add("P_FROM_DATE", OracleDbType.Date).Value = pDataList.FROM_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(pDataList.FROM_DATE);
                cmd.Parameters.Add("P_TO_DATE", OracleDbType.Date).Value = pDataList.TO_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(pDataList.TO_DATE);
                cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = pDataList.ORDER_CLASS;
                cmd.Parameters.Add("cReturn", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ022_GetComplete");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }
        #endregion

        #region SNSQ01E_UPLOAD
        [HttpGet, Route("SNSQ01E_GetPartNo/{partNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSQ01E_GetPartNo(string partNo, int noofshowrec)
        {
            DataSet ds = new DataSet();

            partNo = ismoUtil.escapeCharacter(partNo);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ01E_PART_NO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = partNo.ToUpper() == "NULL" ? (object)DBNull.Value : partNo;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ01E_GetPartNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }


        }

        [HttpGet, Route("SNSQ01E_GetTariffCode/{tariffCode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSQ01E_GetTariffCode(string tariffCode, int noofshowrec)
        {
            DataSet ds = new DataSet();

            tariffCode = ismoUtil.escapeCharacter(tariffCode);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ01E_TARIFF_CODE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_TARIFF_CODE", OracleDbType.Varchar2).Value = tariffCode.ToUpper() == "NULL" ? (object)DBNull.Value : tariffCode;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ01E_GetTariffCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNSQ01E_Save")]
        [Authorize]
        public IHttpActionResult SNSQ01E_Save([FromBody] dynamic pDataList)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ01E_SAVE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pDataList.PART_NO;
                cmd.Parameters.Add("P_PART_NAME_EN", OracleDbType.Varchar2).Value = pDataList.PART_NAME_EN;
                cmd.Parameters.Add("P_PART_NAME_TH", OracleDbType.Varchar2).Value = pDataList.PART_NAME_TH;
                cmd.Parameters.Add("P_DIU_FLAG", OracleDbType.Varchar2).Value = pDataList.DIU_FLAG;
                cmd.Parameters.Add("P_TARIFF_CODE", OracleDbType.Varchar2).Value = pDataList.TARIFF_CODE;
                cmd.Parameters.Add("P_NEW_DIU_FLAG", OracleDbType.Varchar2).Value = pDataList.NEW_DIU_FLAG;
                cmd.Parameters.Add("P_NEW_TARIFF_CODE", OracleDbType.Varchar2).Value = pDataList.NEW_TARIFF_CODE;
                cmd.Parameters.Add("P_CREATE_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSQ01E";
                cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RE", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ01E_Save");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSQ01E_UpdateSO")]
        [Authorize]
        public IHttpActionResult SNSQ01E_UpdateSO([FromBody] dynamic pDataList)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ01E_UPD_TARIFF_CODE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pDataList.PART_NO;
                cmd.Parameters.Add("P_TARIFF_CODE", OracleDbType.Varchar2).Value = pDataList.TARIFF_CODE;
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSQ01E";
                cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RE", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ01E_UpdateSO");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSQ01E_SaveList")]
        [Authorize]
        public IHttpActionResult SNSQ01E_SaveList([FromBody] dynamic[] pDataList)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();

            try
            {
                foreach (var data in pDataList)
                {
                    OracleCommand cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSQ01E_SAVE";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = data.PART_NO;
                    cmd.Parameters.Add("P_PART_NAME_EN", OracleDbType.Varchar2).Value = data.PART_NAME_EN;
                    cmd.Parameters.Add("P_PART_NAME_TH", OracleDbType.Varchar2).Value = data.PART_NAME_TH;
                    cmd.Parameters.Add("P_DIU_FLAG", OracleDbType.Varchar2).Value = data.DIU_FLAG;
                    cmd.Parameters.Add("P_TARIFF_CODE", OracleDbType.Varchar2).Value = data.FORMER_TARIFF_CODE;
                    cmd.Parameters.Add("P_NEW_DIU_FLAG", OracleDbType.Varchar2).Value = data.DIU_FLAG;
                    cmd.Parameters.Add("P_NEW_TARIFF_CODE", OracleDbType.Varchar2).Value = data.TARIFF_CODE;
                    cmd.Parameters.Add("P_CREATE_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSQ01E";
                    cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("C_RE", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                    ds = oOracledb.SqlQueryDS_CMD(cmd);

                    ds = oOracledb.SqlQueryDS_CMD(cmd);
                    dt = ds.Tables[0];
                }
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ01E_SaveList");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSQ01E_UpdateSOList")]
        [Authorize]
        public IHttpActionResult SNSQ01E_UpdateSOList([FromBody] dynamic[] pDataList)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();

            try
            {
                foreach (var data in pDataList)
                {
                    OracleCommand cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSQ01E_UPD_TARIFF_CODE";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = data.PART_NO;
                    cmd.Parameters.Add("P_TARIFF_CODE", OracleDbType.Varchar2).Value = data.TARIFF_CODE;
                    cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSQ01E";
                    cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("C_RE", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                    ds = oOracledb.SqlQueryDS_CMD(cmd);

                    ds = oOracledb.SqlQueryDS_CMD(cmd);
                    dt = ds.Tables[0];
                }
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ01E_UpdateSOList");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }


        }

        [HttpPost, Route("SNSQ01E_Upload")]
        [Authorize]
        public async Task<IHttpActionResult> SNSQ01E_Upload(HttpRequestMessage request)
        {

            if (!request.Content.IsMimeMultipartContent())
            {
                throw new HttpResponseException(HttpStatusCode.UnsupportedMediaType);
            }

            var reqParts = await request.Content.ReadAsMultipartAsync();
            List<string> ErrorList = new List<string>();
            try
            {
                string fileLocation = string.Empty;
                foreach (var file in reqParts.Contents)
                {
                    var buffer = await file.ReadAsByteArrayAsync();
                    string fileName = Path.GetFileNameWithoutExtension(file.Headers.ContentDisposition.FileName.Trim('\"'));
                    string ext = Path.GetExtension(file.Headers.ContentDisposition.FileName.Trim('\"'));
                    string FullfileName = String.Concat(fileName, "-", DateTime.Now.ToString("yyyyMMddHHmm"), ext);
                    string outputLocation = "~/tempFile/" + "SNSQ01E" + "/";
                    string outputPath = System.Web.HttpContext.Current.Server.MapPath(outputLocation);
                    fileLocation = String.Concat(outputPath, FullfileName);

                    DataService.SaveBytesToFileByLocation(buffer
                                                    , fileLocation
                                                    , fileLocation);
                }


                ErrorList = SNSQ01E_InsertDataFromExcelFile(fileLocation);
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ01E_Upload");
                throw new HttpResponseException(HttpStatusCode.NotAcceptable);
            }

            return Json(new { Error = ErrorList });
        }

        public string SNSQ01E_ClearTemp()
        {
            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSQ01E_DEL_TEMP";
            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return "DONE";
        }


        public List<string> SNSQ01E_InsertDataFromExcelFile(string fileLocation)
        {

            string fileLocationN = fileLocation;
            DataTable dt = new DataTable();
            SystemApiController sys = new SystemApiController();
            dt = sys.ReadDataExcel(fileLocationN);
            List<string> ErrList = new List<string>();
            int countrow = 1;

            SNSQ01E_ClearTemp();

            foreach (DataRow row in dt.Rows)
            {
                string p1 = null;
                string p2 = null;
                string p3 = null;

                p1 = row["Part No."].ToString().Trim();
                p2 = row["New Tariff Code"].ToString().Trim();
                p3 = row["DIU Flag (Y/N)"].ToString().Trim();



                if (p1 != null && p2 != null && p3 != null)
                {
                    OracleCommand cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSQ01E_INS_TEMP_UPLOAD";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = p1;
                    cmd.Parameters.Add("P_NEW_TARIFF_CODE", OracleDbType.Varchar2).Value = p2;
                    cmd.Parameters.Add("P_DIU_FLAG", OracleDbType.Varchar2).Value = p3;
                    cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;

                    cmd = oOracledb.SqlQueryReturnValue(cmd);

                    string status = cmd.Parameters["P_STATUS"].Value.ToString();
                    if (status == "DONE")
                    {
                        ErrList.Add("Row " + countrow + " Upload complete");
                    }
                    else if (status == "FAIL")
                    {
                        ErrList.Add("Row " + countrow + " " + "Part No. :" + " " + row["Part No."].ToString().Trim() + " error please recheck");
                    }

                }
                else
                {
                    ErrList.Add("Row " + countrow + " " + "Part No. :" + " " + row["Part No."].ToString().Trim() + " error please recheck");
                }

                countrow++;
            }

            return ErrList;

        }


        [HttpGet, Route("SNSQ01E_GetTempData")]
        [Authorize]
        public IHttpActionResult SNSQ01E_GetTempData()
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ01E_GET_TEMP_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ01E_GetTempData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        #endregion

        #region SNM106P

        [HttpGet, Route("SNM106P_GetUser2/{userID}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM106P_GetUser2(string userID, int noofshowrec)
        {
            DataSet ds = new DataSet();

            userID = ismoUtil.escapeCharacter(userID);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM106P_GET_USER2";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = userID.ToUpper() == "NULL" ? (object)DBNull.Value : userID;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM106P_GetUser2");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }



        [HttpGet, Route("SNM106P_GetUser/{userID}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM106P_GetUser(string userID, int noofshowrec)
        {
            DataSet ds = new DataSet();

            userID = ismoUtil.escapeCharacter(userID);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM106P_GET_USER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = userID.ToUpper() == "NULL" ? (object)DBNull.Value : userID;
                //cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM106P_GetUser");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM106P_GetData")]
        [Authorize]
        public IHttpActionResult SNM106P_GetData([FromBody] dynamic pDataList)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM106P_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = pDataList.USER_ID;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM106P_GetData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNM106P_SAVE")]
        [Authorize]
        public IHttpActionResult SNM106P_SAVE([FromBody] dynamic pDataList)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM106P_SAVE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = pDataList.USER_ID;
                cmd.Parameters.Add("P_REMARK", OracleDbType.Varchar2).Value = pDataList.REMARK;
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM106P";
                cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM106P_SAVE");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM106P_Delete")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM106P_Delete([FromBody] dynamic pDataList)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM106P_DELETE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = pDataList.USER_ID;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM106P_Delete");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        #endregion

        #region SNSQ043
        [HttpGet, Route("SNSQ043_GetPartNo/{partNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSQ043_GetPartNo(string partNo, int noofshowrec)
        {
            DataSet ds = new DataSet();

            partNo = ismoUtil.escapeCharacter(partNo);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ043_GET_PART";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = partNo.ToUpper() == "NULL" ? (object)DBNull.Value : partNo;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ043_GetPartNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpPost, Route("SNSQ043_getData")]
        [Authorize]
        public IHttpActionResult SNSQ043_getData([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ043_GET_DATA";
                cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pParam.PART_NO;
                cmd.Parameters.Add("P_FROM_DATE", OracleDbType.Date).Value = pParam.FROM_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(pParam.FROM_DATE);
                cmd.Parameters.Add("P_TO_DATE", OracleDbType.Date).Value = pParam.TO_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(pParam.TO_DATE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ043_getData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }


        }

        [HttpPost, Route("SNSQ043_getData_Export")]
        [Authorize]
        public HttpResponseMessage SNSQ043_getData_Export([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSQ043_GET_DATA_EXPORT";
            cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pParam.PART_NO;
            cmd.Parameters.Add("P_FROM_DATE", OracleDbType.Date).Value = pParam.FROM_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(pParam.FROM_DATE);
            cmd.Parameters.Add("P_TO_DATE", OracleDbType.Date).Value = pParam.TO_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(pParam.TO_DATE);
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            string x = SNSQ043_Export(ds.Tables[0], "SNSQ043.csv", "SNSQ043");
            SystemApiController sys = new SystemApiController();
            var respons = sys.downloadFile(x.Substring(1), "SNSQ043.csv");
            return respons;
        }

        //[HttpPost, Route("SNSQ043_Export")]
        //public HttpResponseMessage SNSQ043_Export(List<DataModel.SNSQ043> Data)
        //{
        //    HttpResponseMessage response = new HttpResponseMessage(HttpStatusCode.OK);
        //    string gLocation = "~/tempFile/" + "SNSQ043" + "/";
        //    string gWriteLocation = System.Web.HttpContext.Current.Server.MapPath(gLocation);
        //    FileInfo filePath = new FileInfo(gWriteLocation);
        //    filePath.Directory.Create();
        //    string gFileLocation = Path.Combine(gWriteLocation, "SNSQ043.xlsx");
        //    string gReturnLocation = Path.Combine(gLocation, "SNSQ043.xlsx");

        //    var excel2 = Data.ToExcel(x => x
        //    .AddColumn("Seq", z => z.SEQ)
        //    .AddColumn("Part No.", z => z.PART_NUMBER)
        //    .AddColumn("Price App", z => z.PRICE_APP)
        //    .AddColumn("Part Name", z => z.PART_NAME)
        //    .AddColumn("GLP (JP)", z => z.GLP)
        //    .AddColumn("Exchange Rate", z => z.EXCHANGE_RATE)
        //    .AddColumn("GLP (TH)", z => z.GLP_TH)
        //    .AddColumn("D/C", z => z.DESCRIPTION_CODE)
        //    .AddColumn("S/C", z => z.SALES_CLASSIFICATION_CODE)
        //    .AddColumn("GLP Establisment Catagory", z => z.GLP_ESTABLISHMENT_CATEGORY)
        //    .AddColumn("Status", z => z.STATUS)
        //     .AddColumn("Create By User", z => z.CREATE_BY)
        //    .AddColumn("Create Date", z => z.CREATE_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(z.CREATE_DATE))
        //    .AddColumn("Last Update By Program", z => z.LAST_UPD_PGM)
        //    .AddColumn("Last Update By User", z => z.LAST_UPD_BY)
        //    .AddColumn("Last Update Date", z => z.LAST_UPD_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(z.LAST_UPD_DATE))


        //    );

        //    byte[] excel = excel2;
        //    File.WriteAllBytes(gFileLocation, excel);
        //    string path = HttpContext.Current.Server.MapPath("~/" + gReturnLocation.Substring(1));
        //    if (!File.Exists(path))
        //    {
        //        return Request.CreateErrorResponse(HttpStatusCode.NotFound, "File not found.");
        //    }
        //    try
        //    {
        //        string mimeType = MimeMapping.GetMimeMapping("SNSQ043.xlsx");
        //        StreamContent x = new StreamContent(new FileStream(path, FileMode.Open, FileAccess.Read));
        //        response.Content = x;
        //        response.Content.Headers.ContentType = new MediaTypeHeaderValue(mimeType);
        //        response.Content.Headers.ContentDisposition = new ContentDispositionHeaderValue("attachment");
        //        response.Content.Headers.ContentDisposition.FileName = "SNSQ043.xlsx";
        //    }
        //    catch (Exception ex)
        //    {
        //        return Request.CreateResponse(HttpStatusCode.BadRequest, ex);
        //    }
        //    return response;
        //}




        public static string SNSQ043_Export(DataTable dt, string fileName, string FloderLocation)
        {

            bool Succes = false;

            string gLocation = "~/tempFile/" + FloderLocation + "/";
            string gWriteLocation = System.Web.HttpContext.Current.Server.MapPath(gLocation);

            FileInfo filePath = new FileInfo(gWriteLocation);
            filePath.Directory.Create();
            string gFileLocation = Path.Combine(gWriteLocation, fileName);
            string gReturnLocation = Path.Combine(gLocation, fileName);
            SystemApiController sys = new SystemApiController();
            try
            {
                StringBuilder sb = new StringBuilder();

                string[] HeaderForExport = new string[]
                {   "Seq",
                    "Part number",
                    "Part name",
                    "GLP",
                    "GLP(THB)",
                    "Exchange Rate",
                    "Price App",
                    "Description code",
                    "Sales classification code",
                    "GLP establishment category",
                    "Status",
                    "Create by",
                    "Create date",
                    "Last update PGM",
                    "Last update by",
                    "Last update date",
                };
                IEnumerable<string> columnNames = HeaderForExport.Select(h => h.ToString());
                sb.AppendLine(string.Join(",", columnNames));

                foreach (DataRow row in dt.Rows)
                {
                    for (int i = 0; i < dt.Columns.Count; i++)
                    {

                        sb.Append(CsvEscape(row[i].ToString()));
                        sb.Append(i == dt.Columns.Count - 1 ? "\n" : ",");
                    }
                }

                File.WriteAllText(gFileLocation, sb.ToString());

                Succes = true;
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
            }

            if (Succes)
            {
                return gReturnLocation;
            }
            else
            {

                return "";
            }

        }

        static string CsvEscape(string value)
        {
            if (value.Contains(","))
            {
                return "\"" + value.Replace("\"", "\"\"") + "\"";
            }
            return value;
        }

        #endregion

        #region SNSQ045

        [HttpGet, Route("SNSQ045_getBuyerCode/{buyerCode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSQ045_getBuyerCode(string buyerCode, int noofshowrec)
        {
            DataSet ds = new DataSet();

            buyerCode = ismoUtil.escapeCharacter(buyerCode);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ045_GET_BUYER_CODE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_BUYER_CODE", OracleDbType.Varchar2).Value = buyerCode.ToUpper() == "NULL" ? (object)DBNull.Value : buyerCode;
                //cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ045_getBuyerCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }


        }

        [HttpPost, Route("SNSQ045_getData")]
        [Authorize]
        public IHttpActionResult SNSQ045_getData([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ045_GET_DATA";
                cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_BUYER_CODE", OracleDbType.Varchar2).Value = pParam.BUYER_CODE;
                cmd.Parameters.Add("P_FROM_DATE", OracleDbType.Date).Value = pParam.FROM_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(pParam.FROM_DATE);
                cmd.Parameters.Add("P_TO_DATE", OracleDbType.Date).Value = pParam.TO_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(pParam.TO_DATE);
                cmd.Parameters.Add("P_GROUP", OracleDbType.Int16).Value = pParam.GROUP;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ045_getData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNSQ045_getData_Export")]
        [Authorize]
        public HttpResponseMessage SNSQ045_getData_Export([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSQ045_GET_DATA_EXPORT";
            cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_BUYER_CODE", OracleDbType.Varchar2).Value = pParam.BUYER_CODE;
            cmd.Parameters.Add("P_FROM_DATE", OracleDbType.Date).Value = pParam.FROM_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(pParam.FROM_DATE);
            cmd.Parameters.Add("P_TO_DATE", OracleDbType.Date).Value = pParam.TO_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(pParam.TO_DATE);
            cmd.Parameters.Add("P_GROUP", OracleDbType.Int16).Value = pParam.GROUP;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            string x = SNSQ045_Export(ds.Tables[0], "SNSQ045.csv", "SNSQ045");
            SystemApiController sys = new SystemApiController();
            var respons = sys.downloadFile(x.Substring(1), "SNSQ045.csv");
            return respons;
        }

        public static string SNSQ045_Export(DataTable dt, string fileName, string FloderLocation)
        {

            bool Succes = false;

            string gLocation = "~/tempFile/" + FloderLocation + "/";
            string gWriteLocation = System.Web.HttpContext.Current.Server.MapPath(gLocation);

            FileInfo filePath = new FileInfo(gWriteLocation);
            filePath.Directory.Create();
            string gFileLocation = Path.Combine(gWriteLocation, fileName);
            string gReturnLocation = Path.Combine(gLocation, fileName);
            SystemApiController sys = new SystemApiController();
            try
            {
                StringBuilder sb = new StringBuilder();

                string[] HeaderForExport = new string[]
                {       "Seq",
                        "Buyer Code",
                        "D / C",
                        "S / C",
                        "Part Number",
                        "Part Name",
                        "Price App YM 01",
                        "Price Code 01",
                        "Disc Rate /Unit Price 01",
                        "Price App YM 02",
                        "Price Code 02",
                        "Disc Rate /Unit Price 02",
                        "Price App YM 03",
                        "Price Code 03",
                        "Disc Rate /Unit Price 03",
                        "Price App YM 04",
                        "Price Code 04",
                        "Disc Rate /Unit Price 04",
                        "Price App YM 05",
                        "Price Code 05",
                        "Disc Rate /Unit Price 05",
                        "Price App YM 06",
                        "Price Code 06",
                        "Disc Rate /Unit Price 06",
                        "Price App YM 07",
                        "Price Code 07",
                        "Disc Rate /Unit Price 07",
                        "Price App YM 08",
                        "Price Code 08",
                        "Disc Rate /Unit Price 08",
                        "Price App YM 09",
                        "Price Code 09",
                        "Disc Rate /Unit Price 09",
                        "Price App YM 10",
                        "Price Code 10",
                        "Disc Rate /Unit Price 10",
                        "Price App YM 11",
                        "Price Code 11",
                        "Disc Rate /Unit Price 11",
                        "Price App YM 12",
                        "Price Code 12",
                        "Disc Rate /Unit Price 12",
                        "Price App YM 13",
                        "Price Code 13",
                        "Disc Rate /Unit Price 13",
                        "Price App YM 14",
                        "Price Code 14",
                        "Disc Rate /Unit Price 14",
                        "Price App YM 15",
                        "Price Code 15",
                        "Disc Rate /Unit Price 15",
                        "Start YM",
                        "End YM",
                        "New Part Number",
                        "Interchangeability",
                        "Filler",
                        "Create By",
                        "Create Date",
                        "Last Update Program",
                        "Last Update By",
                        "Last Update Date",
                };
                IEnumerable<string> columnNames = HeaderForExport.Select(h => h.ToString());
                sb.AppendLine(string.Join(",", columnNames));

                foreach (DataRow row in dt.Rows)
                {
                    for (int i = 0; i < dt.Columns.Count; i++)
                    {

                        sb.Append(CsvEscape(row[i].ToString()));
                        sb.Append(i == dt.Columns.Count - 1 ? "\n" : ",");
                    }
                }

                File.WriteAllText(gFileLocation, sb.ToString());

                Succes = true;
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
            }

            if (Succes)
            {
                return gReturnLocation;
            }
            else
            {

                return "";
            }

        }
        #endregion

        #region SNSQ023_001
        [HttpPost, Route("SNSQ023_GetCustomerCode/{customerCode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSQ023_GetCustomerCode(string customerCode, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            customerCode = ismoUtil.escapeCharacter(customerCode);
            var pData = pParam["DATA"];

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ023_GET_CUSTOMER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = customerCode.ToUpper() == "NULL" ? (object)DBNull.Value : customerCode;
                cmd.Parameters.Add("P_FROM_DATE", OracleDbType.Date).Value = pData.FROM_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(pData.FROM_DATE);
                cmd.Parameters.Add("P_TO_DATE", OracleDbType.Date).Value = pData.TO_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(pData.TO_DATE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ023_GetCustomerCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }


        }

        [HttpPost, Route("SNSQ023_GetOrderNo/{orderNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSQ023_GetOrderNo(string orderNo, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            orderNo = ismoUtil.escapeCharacter(orderNo);
            var pData = pParam["DATA"];

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ023_GET_ORDER_NO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = orderNo.ToUpper() == "NULL" ? (object)DBNull.Value : orderNo;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_FROM_DATE", OracleDbType.Date).Value = pData.FROM_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(pData.FROM_DATE);
                cmd.Parameters.Add("P_TO_DATE", OracleDbType.Date).Value = pData.TO_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(pData.TO_DATE);
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ023_GetOrderNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }


        }

        [HttpPost, Route("SNSQ023_GetOrderPartNo/{orderPartNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSQ023_GetOrderPartNo(string orderPartNo, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            orderPartNo = ismoUtil.escapeCharacter(orderPartNo);
            var pData = pParam["DATA"];

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ023_GET_ORDER_PART";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pData.ORDER_NO;
                cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = orderPartNo.ToUpper() == "NULL" ? (object)DBNull.Value : orderPartNo;
                cmd.Parameters.Add("P_FROM_DATE", OracleDbType.Date).Value = pData.FROM_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(pData.FROM_DATE);
                cmd.Parameters.Add("P_TO_DATE", OracleDbType.Date).Value = pData.TO_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(pData.TO_DATE);
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ023_GetOrderPartNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }


        }


        [HttpPost, Route("SNSQ023_GetData")]
        [Authorize]
        public IHttpActionResult SNSQ023_GetData([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ023_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pParam.ORDER_NO;
                cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = pParam.ORDER_PART_NO;
                cmd.Parameters.Add("P_FROM_DATE", OracleDbType.Date).Value = pParam.FROM_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(pParam.FROM_DATE);
                cmd.Parameters.Add("P_TO_DATE", OracleDbType.Date).Value = pParam.TO_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(pParam.TO_DATE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ023_GetData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }


        }


        //[HttpPost, Route("SNSQ023_Export")]
        //public HttpResponseMessage SNSQ023_Export(List<DataModel.SNSQ023> Data)
        //{
        //    HttpResponseMessage response = new HttpResponseMessage(HttpStatusCode.OK);
        //    string gLocation = "~/tempFile/" + "SNSQ023" + "/";
        //    string gWriteLocation = System.Web.HttpContext.Current.Server.MapPath(gLocation);
        //    FileInfo filePath = new FileInfo(gWriteLocation);
        //    filePath.Directory.Create();
        //    string gFileLocation = Path.Combine(gWriteLocation, "SNSQ023.xlsx");
        //    string gReturnLocation = Path.Combine(gLocation, "SNSQ023.xlsx");

        //    var excel2 = Data.ToExcel(x => x
        //    .AddColumn("Seq.", z => z.V_SEQ)
        //    .AddColumn("Customer Code", z => z.CUSTOMER_CODE)
        //    .AddColumn("Order No", z => z.ORDER_NO)
        //    .AddColumn("Order Date", z => z.ORDER_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(z.ORDER_DATE).ToString("dd-MM-yyyy"))
        //    .AddColumn("Order Part No", z => z.ORDER_PART_NO)
        //    .AddColumn("Order Status Vendor Code", z => z.V_VENDOR_CODE)
        //    .AddColumn("Parts Master Vendor Code", z => z.P_VENDOR_CODE)
        //    .AddColumn("Item No", z => z.V_ITEM_NO)
        //    .AddColumn("Accept Part No", z => z.ACCEPT_PART_NO)
        //    .AddColumn("Order Qty", z => z.ORDER_QTY)
        //    .AddColumn("Allocation Date", z => z.ALLOCATE_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(z.ALLOCATE_DATE).ToString("dd-MM-yyyy"))
        //    .AddColumn("1st Allocated Qty", z => z.V_F_ACCEPT_QTY)
        //    .AddColumn("After 1st Allocated Qty", z => z.V_AFTER_F_ACCEPT_QTY)
        //    .AddColumn("B/O Qty", z => z.V_BO_QTY)
        //    .AddColumn("Picking Card Issue Date", z => z.V_PICK_LOT_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(z.V_PICK_LOT_DATE).ToString("dd-MM-yyyy"))
        //    .AddColumn("L/T from Allocation to Picking Card Issue", z => z.V_LT_ORDER_PICK)
        //    .AddColumn("Invoice Date", z => z.INVOICE_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(z.INVOICE_DATE).ToString("dd-MM-yyyy"))
        //    .AddColumn("L/T from Picking Card Issue to Invoice", z => z.V_LT_PICK_INVOICE)
        //    .AddColumn("ETD Date", z => z.ETD_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(z.ETD_DATE).ToString("dd-MM-yyyy"))
        //    .AddColumn("Customer Name", z => z.CUST_NAME_EN)
        //    .AddColumn("Order Class", z => z.ORDER_CLASS)
        //    );

        //    byte[] excel = excel2;
        //    File.WriteAllBytes(gFileLocation, excel);
        //    string path = HttpContext.Current.Server.MapPath("~/" + gReturnLocation.Substring(1));
        //    if (!File.Exists(path))
        //    {
        //        return Request.CreateErrorResponse(HttpStatusCode.NotFound, "File not found.");
        //    }
        //    try
        //    {
        //        string mimeType = MimeMapping.GetMimeMapping("SNSQ023.xlsx");
        //        StreamContent x = new StreamContent(new FileStream(path, FileMode.Open, FileAccess.Read));
        //        response.Content = x;
        //        response.Content.Headers.ContentType = new MediaTypeHeaderValue(mimeType);
        //        response.Content.Headers.ContentDisposition = new ContentDispositionHeaderValue("attachment");
        //        response.Content.Headers.ContentDisposition.FileName = "SNSQ023.xlsx";
        //    }
        //    catch (Exception ex)
        //    {
        //        return Request.CreateResponse(HttpStatusCode.BadRequest, ex);
        //    }
        //    return response;
        //}

        [HttpPost, Route("SNSQ023_getDataForExport")]
        [Authorize]
        public HttpResponseMessage SNSQ023_getDataForExport([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSQ023_GET_DATA";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
            cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pParam.ORDER_NO;
            cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = pParam.ORDER_PART_NO;
            cmd.Parameters.Add("P_FROM_DATE", OracleDbType.Date).Value = pParam.FROM_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(pParam.FROM_DATE);
            cmd.Parameters.Add("P_TO_DATE", OracleDbType.Date).Value = pParam.TO_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(pParam.TO_DATE);
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            string x = SNSQ023_ExportExcel(ds.Tables[0], "SNSQ023.csv", "SNSQ023");
            if (x == "")
            {
                Request.CreateResponse(HttpStatusCode.NotAcceptable, "Export excel error");
            }
            SystemApiController sys = new SystemApiController();
            var respons = sys.downloadFile(x.Substring(1), "SNSQ023.csv");
            return respons;

        }

        public string SNSQ023_ExportExcel(DataTable dt, string fileName, string FloderLocation)
        {
            bool Succes = false;

            string custCode = "AABBBBBBBCCCCCCCCCCCCCCCCC";
            string orderNo = "AABBBBBBBCCCCCCCCCCCCCCCCC";
            string orderPartNo = "AABBBBBBBCCCCCCCCCCCCCCCCC";
            string itemNo = "AABBBBBBBCCCCCCCCCCCCCCCCC";
            int boQTY = 0;
            string gLocation = "~/tempFile/" + FloderLocation + "/";
            string gWriteLocation = System.Web.HttpContext.Current.Server.MapPath(gLocation);

            FileInfo filePath = new FileInfo(gWriteLocation);
            filePath.Directory.Create();
            string gFileLocation = Path.Combine(gWriteLocation, fileName);
            string gReturnLocation = Path.Combine(gLocation, fileName);
            SystemApiController sys = new SystemApiController();
            try
            {
                StringBuilder sb = new StringBuilder();

                string[] HeaderForExport = new string[]
                {
                    "Seq.",
                    "Customer Code",
                    "Order No",
                    "Order Date",
                    "Order Part No",
                    "Order Status Vendor Code",
                    "Parts Master Vendor Code",
                    "Item No",
                    "Accept Part No",
                    "Order Qty",
                    "Allocation Date",
                    "1st Allocated Qty",
                    "After 1st Allocated Qty",
                    "B/O Qty",
                    "Currency Code",
                    "Exchange Rate",
                    "Unit Price C",
                    "Retail Price C",
                    "Picking Card Issue Date",
                    "L/T from Allocation to Picking Card Issue",
                    "Invoice Date",
                    "L/T from Picking Card Issue to Invoice",
                    "ETD Date",
                    "Customer Name",
                    "Order Class",
                };

                IEnumerable<string> columnNames = HeaderForExport.Select(h => h.ToString());
                sb.AppendLine(string.Join(",", columnNames));

                foreach (DataRow row in dt.Rows)
                {
                    if (custCode == row[1].ToString() && orderNo == row[2].ToString() && itemNo == row[25].ToString() && orderPartNo == row[4].ToString())
                    {
                        row[2] = row[2].ToString() != null ? (object)DBNull.Value : "";
                        row[3] = row[3].ToString() != null ? (object)DBNull.Value : "";
                        row[4] = row[4].ToString() != null ? (object)DBNull.Value : "";
                        row[5] = row[5].ToString() != null ? (object)DBNull.Value : "";
                        row[6] = row[6].ToString() != null ? (object)DBNull.Value : "";
                        row[9] = row[9].ToString() != null ? (object)DBNull.Value : "";
                        row[23] = row[23].ToString() != null ? (object)DBNull.Value : "";

                        if (row[10].ToString() == row[27].ToString() && row[10].ToString() == row[32].ToString())
                        {
                            //row[11] = Int32.Parse(row[33].ToString());
                            row[11] = row[33].ToString() != null ? Int32.Parse(row[33].ToString()) : 0;
                            row[12] = row[12].ToString() != null ? (object)DBNull.Value : "";
                            row[13] = boQTY - Int32.Parse(row[11].ToString());
                        }
                        else
                        {
                            row[11] = row[11].ToString() != null ? (object)DBNull.Value : "";
                            // row[12] = Int32.Parse(row[33].ToString());
                            row[12] = row[33].ToString() != null ? Int32.Parse(row[33].ToString()) : 0;
                            row[13] = boQTY - Int32.Parse(row[12].ToString());
                        };

                        row[18] = row[28].ToString() != null ? (object)DBNull.Value : "";
                        row[20] = row[29].ToString() != null ? (object)DBNull.Value : "";
                        row[21] = row[30].ToString() != null ? (object)DBNull.Value : "";
                        row[19] = row[31].ToString() != null ? (object)DBNull.Value : "";

                        boQTY = row[13].ToString() != null ? Int32.Parse(row[13].ToString()) : 0;
                    }
                    else
                    {
                        custCode = row[1].ToString() != null ? String.Empty : "";
                        orderNo = row[2].ToString() != null ? String.Empty : "";
                        orderPartNo = row[4].ToString() != null ? String.Empty : "";
                        itemNo = row[25].ToString() != null ? String.Empty : "";
                        boQTY = row[13].ToString() != null ? Int32.Parse(row[13].ToString()) : 0;
                    }

                    for (int i = 0; i < dt.Columns.Count - 9; i++)
                    {
                        var j = i;
                        if (i == 3 || i == 10 || i == 18 || i == 20 || i == 22)
                        {
                            sb.Append(row[i].ToString() == "" ? "" : Convert.ToDateTime(row[i]).ToString("dd-MMM-yyyy"));
                        }
                        else if (i == 23)
                        {
                            sb.Append(CsvEscape(row[i].ToString().Replace(',', ' ')));
                        }
                        else
                        {
                            sb.Append("=\"" + CsvEscape(row[i].ToString()) + "\"");

                        }
                        sb.Append(i == dt.Columns.Count - 10 ? "\n" : ",");
                    }
                }
                File.WriteAllText(gFileLocation, sb.ToString());

                Succes = true;
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ023_ExportExcel");

            }

            if (Succes)
            {
                return gReturnLocation;
            }
            else
            {
                return "";
            }
        }
        #endregion

        #region SNSE011
        [HttpGet, Route("SNSE011_getCustomerCode/{customerCode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE011_getCustomerCode(string customerCode, int noofshowrec)
        {
            DataSet ds = new DataSet();

            customerCode = ismoUtil.escapeCharacter(customerCode);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE011_GET_CUSTOMER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = customerCode.ToUpper() == "NULL" ? (object)DBNull.Value : customerCode;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE011_getCustomerCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }


        }

        [HttpGet, Route("SNSE011_GetCustSearch/{cusID}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE011_GetCustSearch(string cusID, int noofshowrec)
        {
            DataSet ds = new DataSet();

            cusID = ismoUtil.escapeCharacter(cusID);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE011_N_GET_CUST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = cusID.ToUpper() == "NULL" ? (object)DBNull.Value : cusID;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE011_GetCustSearch");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE011_GetSearchBranch/{branchCode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE011_GetSearchBranch(string branchCode, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            var pData = pParam["DATA"];
            branchCode = ismoUtil.escapeCharacter(branchCode);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE011_N_GET_BRANCH";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = branchCode.ToUpper() == "NULL" ? (object)DBNull.Value : branchCode;
                cmd.Parameters.Add("P_SHOW_REC", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE011_GetSearchBranch");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpPost, Route("SNSE011_GetSearchOrder/{OrderNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE011_GetSearchOrder(string OrderNo, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            var pData = pParam["DATA"];
            OrderNo = ismoUtil.escapeCharacter(OrderNo);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE011_GET_ORDER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pData["BRANCH_CODE"];
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = OrderNo.ToUpper() == "NULL" ? (object)DBNull.Value : OrderNo;
                cmd.Parameters.Add("P_SHOW_REC", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE011_GetSearchOrder");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE011_getOrderNo/{orderNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE011_getOrderNo(string orderNo, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            orderNo = ismoUtil.escapeCharacter(orderNo);
            var pData = pParam["DATA"];

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE011_GET_ORDER_NO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = orderNo.ToUpper() == "NULL" ? (object)DBNull.Value : orderNo;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE011_getOrderNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNSE011_getOrderNoRpt/{orderNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE011_getOrderNoRpt(string orderNo, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            orderNo = ismoUtil.escapeCharacter(orderNo);
            var pData = pParam["DATA"];

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE011_GET_ORDER_NO_RPT";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = orderNo.ToUpper() == "NULL" ? (object)DBNull.Value : orderNo;
                cmd.Parameters.Add("P_CUSTOMER_CODE_S", OracleDbType.Varchar2).Value = pData["CUSTOMER_CODE_FROM"];
                cmd.Parameters.Add("P_CUSTOMER_CODE_E", OracleDbType.Varchar2).Value = pData["CUSTOMER_CODE_TO"];
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE011_getOrderNoRpt");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNSE011_getData")]
        [Authorize]
        public IHttpActionResult SNSE011_getData([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE011_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pParam.ORDER_NO;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE011_getData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNSE011_UpdateOrder")]
        [Authorize]
        public IHttpActionResult SNSE011_UpdateOrder([FromBody] dynamic[] pDataList)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();

            try
            {
                foreach (var data in pDataList)
                {
                    OracleCommand cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE011_UPD_ORDER";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = data.ORDER_NO;
                    cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Varchar2).Value = data.ITEM_NO;
                    cmd.Parameters.Add("P_SUB_ITEM_NO", OracleDbType.Varchar2).Value = data.SUB_ITEM_NO;
                    cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = data.ORDER_PART_NO;
                    cmd.Parameters.Add("P_ACCEPT_QTY", OracleDbType.Varchar2).Value = data.ACCEPT_QTY;
                    cmd.Parameters.Add("P_RETAIL_AMT_B", OracleDbType.Varchar2).Value = data.RETAIL_AMT_B;
                    cmd.Parameters.Add("P_RETAIL_AMT_C", OracleDbType.Varchar2).Value = data.RETAIL_AMT_C;
                    cmd.Parameters.Add("P_NET_AMT_B", OracleDbType.Varchar2).Value = data.NET_AMT_B;
                    cmd.Parameters.Add("P_NET_AMT_C", OracleDbType.Varchar2).Value = data.NET_AMT_C;
                    cmd.Parameters.Add("P_AVERAGE_AMT", OracleDbType.Varchar2).Value = data.AVERAGE_AMT;
                    cmd.Parameters.Add("P_STANDARD_AMT", OracleDbType.Varchar2).Value = data.STANDARD_AMT;
                    cmd.Parameters.Add("P_DISCOUNT_AMT", OracleDbType.Varchar2).Value = data.DISCOUNT_AMT;
                    cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE011";
                    cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = data.ORDER_CLASS;
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                    ds = oOracledb.SqlQueryDS_CMD(cmd);
                    dt = ds.Tables[0];
                }
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE011_UpdateOrder");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }


        }
        #endregion

        #region SNSE010
        [HttpGet, Route("SNSE010_GetCustomerCode/{customerCode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE010_GetCustomerCode(string customerCode, int noofshowrec)
        {
            DataSet ds = new DataSet();

            customerCode = ismoUtil.escapeCharacter(customerCode);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE010_GET_CUSTOMER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = customerCode.ToUpper() == "NULL" ? (object)DBNull.Value : customerCode;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE010_getCustomerCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNSE010_GetOrderNo/{orderNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE010_GetOrderNo(string orderNo, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            orderNo = ismoUtil.escapeCharacter(orderNo);
            var pData = pParam["DATA"];

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE010_GET_ORDER_NO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = orderNo.ToUpper() == "NULL" ? (object)DBNull.Value : orderNo;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pData["BRANCH_CODE"];
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE010_GetOrderNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }


        }

        [HttpPost, Route("SNSE010_getOrderNoRpt/{orderNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE010_getOrderNoRpt(string orderNo, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            orderNo = ismoUtil.escapeCharacter(orderNo);
            var pData = pParam["DATA"];

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE010_GET_ORDER_NO_RPT";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = orderNo.ToUpper() == "NULL" ? (object)DBNull.Value : orderNo;
                cmd.Parameters.Add("P_CUSTOMER_CODE_S", OracleDbType.Varchar2).Value = pData["CUSTOMER_CODE_FROM"];
                cmd.Parameters.Add("P_CUSTOMER_CODE_E", OracleDbType.Varchar2).Value = pData["CUSTOMER_CODE_TO"];
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE010_getOrderNoRpt");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNSE010_GetOrderList")]
        [Authorize]
        public IHttpActionResult SNSE010_GetOrderList([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE010_GET_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam.BRANCH_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pParam.ORDER_NO;
                cmd.Parameters.Add("P_ORDER_DATE", OracleDbType.Date).Value = pParam.ORDER_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(pParam.ORDER_DATE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE010_GetOrderList");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE010_getData")]
        [Authorize]
        public IHttpActionResult SNSE010_getData([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE010_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pParam.ORDER_NO;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE010_getData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNSE010_UpdateOrder")]
        [Authorize]
        public IHttpActionResult SNSE010_UpdateOrder([FromBody] dynamic[] pDataList)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();

            try
            {
                foreach (var data in pDataList)
                {
                    OracleCommand cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE010_UPD_ORDER";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = data.ORDER_NO;
                    cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Varchar2).Value = data.ITEM_NO;
                    cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = data.ORDER_PART_NO;
                    cmd.Parameters.Add("P_ORDER_QTY", OracleDbType.Varchar2).Value = data.ORDER_QTY;
                    cmd.Parameters.Add("P_RETAIL_AMT_B", OracleDbType.Varchar2).Value = data.RETAIL_AMT_B;
                    cmd.Parameters.Add("P_RETAIL_AMT_C", OracleDbType.Varchar2).Value = data.RETAIL_AMT_C;
                    cmd.Parameters.Add("P_NET_AMT_B", OracleDbType.Varchar2).Value = data.NET_AMT_B;
                    cmd.Parameters.Add("P_NET_AMT_C", OracleDbType.Varchar2).Value = data.NET_AMT_C;
                    cmd.Parameters.Add("P_AVERAGE_AMT", OracleDbType.Varchar2).Value = data.AVERAGE_AMT;
                    cmd.Parameters.Add("P_STANDARD_AMT", OracleDbType.Varchar2).Value = data.STANDARD_AMT;
                    cmd.Parameters.Add("P_DISCOUNT_AMT", OracleDbType.Varchar2).Value = data.DISCOUNT_AMT;
                    cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE010";
                    cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                    ds = oOracledb.SqlQueryDS_CMD(cmd);
                    dt = ds.Tables[0];
                }
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE010_UpdateOrder");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        #endregion

        #region SNSQ074E

        [HttpGet, Route("SNSQ074E_GetBuyerCode/{buyerCode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSQ074E_GetBuyerCode(string buyerCode, int noofshowrec)
        {
            DataSet ds = new DataSet();

            buyerCode = ismoUtil.escapeCharacter(buyerCode);
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ074E_GET_BUYER_CODE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_BUYER_CODE", OracleDbType.Varchar2).Value = buyerCode.ToUpper() == "NULL" ? (object)DBNull.Value : buyerCode;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ074E_GetBuyerCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSQ074E_GetBranchCode/{branchCode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSQ074E_GetBranchCode(string branchCode, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            branchCode = ismoUtil.escapeCharacter(branchCode);
            var pData = pParam["DATA"];

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ074E_GET_BRANCH_CODE";
                cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = branchCode.ToUpper() == "NULL" ? (object)DBNull.Value : branchCode;
                cmd.Parameters.Add("P_BUYER_CODE", OracleDbType.Varchar2).Value = pData["BUYER_CODE"];
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ074E_GetBranchCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSQ074E_GetInvoice")]
        [Authorize]
        public IHttpActionResult SNSQ074E_GetInvoice([FromBody] dynamic pDataList)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ074E_GET_INVOICE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_FROM_DATE", OracleDbType.Date).Value = pDataList.FROM_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(pDataList.FROM_DATE);
                cmd.Parameters.Add("P_TO_DATE", OracleDbType.Date).Value = pDataList.TO_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(pDataList.TO_DATE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ074E_GetInvoice");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNSQ074E_GetPickList")]
        [Authorize]
        public IHttpActionResult SNSQ074E_GetPickList()
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ074E_GET_PICKLIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ074E_GetPickList");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSQ074E_GetSummary")]
        [Authorize]
        public IHttpActionResult SNSQ074E_GetSummary([FromBody] dynamic pDataList)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ074E_SUMMARY";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_BUYER_CODE", OracleDbType.Varchar2).Value = pDataList.BUYER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pDataList.BRANCH_CODE;
                cmd.Parameters.Add("P_FROM_DATE", OracleDbType.Date).Value = pDataList.FROM_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(pDataList.FROM_DATE);
                cmd.Parameters.Add("P_TO_DATE", OracleDbType.Date).Value = pDataList.TO_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(pDataList.TO_DATE);
                cmd.Parameters.Add("cReturn", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ074E_GetSummary");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSQ074E_GetInvoiceDetail")]
        [Authorize]
        public IHttpActionResult SNSQ074E_GetInvoiceDetail([FromBody] dynamic pDataList)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ074E_INOVICE_DETAIL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_BUYER_CODE", OracleDbType.Varchar2).Value = pDataList.BUYER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pDataList.BRANCH_CODE;
                cmd.Parameters.Add("P_FROM_DATE", OracleDbType.Date).Value = pDataList.FROM_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(pDataList.FROM_DATE);
                cmd.Parameters.Add("P_TO_DATE", OracleDbType.Date).Value = pDataList.TO_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(pDataList.TO_DATE);
                cmd.Parameters.Add("cReturn", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ074E_GetInvoiceDetail");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSQ074E_GetPickingDetail")]
        [Authorize]
        public IHttpActionResult SNSQ074E_GetPickingDetail([FromBody] dynamic pDataList)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ074E_PICKING_DETAIL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_BUYER_CODE", OracleDbType.Varchar2).Value = pDataList.BUYER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pDataList.BRANCH_CODE;
                cmd.Parameters.Add("cReturn", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ074E_GetPickingDetail");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSQ074E_GetExchangeRateDetail")]
        [Authorize]
        public IHttpActionResult SNSQ074E_GetExchangeRateDetail([FromBody] dynamic pDataList)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ074E_EXCHANGE_DETAIL";
                cmd.Parameters.Add("P_DATA_MONTH", OracleDbType.Varchar2).Value = pDataList["DATA_MONTH"].ToString("dd-MM-yyyy");
                cmd.Parameters.Add("cReturn", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ074E_GetExchangeRateDetail");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSQ074E_AddExchangeRate")]
        [Authorize]
        public IHttpActionResult SNSQ074E_AddExchangeRate([FromBody] dynamic pDataList)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ074E_SAVE_EXC_RATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_EFFECTIVE_DATE", OracleDbType.Varchar2).Value = pDataList["EFFECTIVE_DATE"].ToString("dd-MM-yyyy");
                cmd.Parameters.Add("P_CURRENCY_CODE", OracleDbType.Varchar2).Value = pDataList["CURRENCY"];
                cmd.Parameters.Add("P_EXCHANGE_RATE", OracleDbType.Int32).Value = pDataList["EXCHANGE_RATE"];
                cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSQ074E";
                cmd.Parameters.Add("cReturn", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ074E_AddExchangeRate");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSQ074E_SummaryExport")]
        public HttpResponseMessage SNSQ074E_SummaryExport(List<DataModel.SNSQ074E> dModel)
        {
            DataTable dt = new DataTable();
            string x = string.Empty;
            string filename = "SNSQ074E.csv";
            string floderLocation = "SNSQ074E";
            dt.Columns.Add("colum1");

            dt.Rows.Add("ID;PO2C");
            dt.Rows.Add("B");

            //table1
            dt.Rows.Add("C;X3;Y1;K\"On Process\"");
            dt.Rows.Add("C;X4;Y1;K\"BP\"");
            dt.Rows.Add("C;X5;Y1;K\"Att%\"");
            dt.Rows.Add("C;X2;Y2;K" + '"' + Math.Round(Convert.ToDecimal(dModel[0].T_PART_SALE), 2) + '"');
            dt.Rows.Add("C;X2;Y3;K" + '"' + Math.Round(Convert.ToDecimal(dModel[0].T_PART_GP), 2) + '"');
            dt.Rows.Add("C;X2;Y4;K" + '"' + Math.Round(Convert.ToDecimal(dModel[0].T_GPR_PARTS_PERCENT), 2) + '"');
            dt.Rows.Add("C;X2;Y5;K" + '"' + Math.Round(Convert.ToDecimal(dModel[0].T_ACC_SALE), 2) + '"');
            dt.Rows.Add("C;X2;Y6;K" + '"' + Math.Round(Convert.ToDecimal(dModel[0].T_ACC_GP), 2) + '"');
            dt.Rows.Add("C;X2;Y7;K" + '"' + Math.Round(Convert.ToDecimal(dModel[0].T_GPR_ACC_PERCENT), 2) + '"');
            dt.Rows.Add("C;X2;Y8;K" + '"' + Math.Round(Convert.ToDecimal(dModel[0].T_ACC_DLR_OPT_SALE), 2) + '"');
            dt.Rows.Add("C;X2;Y9;K" + '"' + Math.Round(Convert.ToDecimal(dModel[0].T_ACC_DLR_OPT_SALE_GP), 2) + '"');
            dt.Rows.Add("C;X2;Y10;K" + '"' + Math.Round(Convert.ToDecimal(dModel[0].T_ACC_PDI_SALE), 2) + '"');
            dt.Rows.Add("C;X2;Y11;K" + '"' + Math.Round(Convert.ToDecimal(dModel[0].T_ACC_PDI_SALE_GP), 2) + '"');
            dt.Rows.Add("C;X2;Y12;K" + '"' + Math.Round(Convert.ToDecimal(dModel[0].T_EXP_SALES), 2) + '"');
            dt.Rows.Add("C;X2;Y13;K" + '"' + Math.Round(Convert.ToDecimal(dModel[0].T_EXP_GP), 2) + '"');
            dt.Rows.Add("C;X2;Y14;K" + '"' + Math.Round(Convert.ToDecimal(dModel[0].T_GPR_PERCENT), 2) + '"');

            dt.Rows.Add("C;X3;Y2;K" + '"' + Math.Round(Convert.ToDecimal(dModel[0].OP_PART_SALE), 2) + '"');
            dt.Rows.Add("C;X3;Y3;K" + '"' + Math.Round(Convert.ToDecimal(dModel[0].OP_PART_GP), 2) + '"');
            dt.Rows.Add("C;X3;Y4;K" + '"' + Math.Round(Convert.ToDecimal(dModel[0].OP_GPR_PART_PERCENT), 2) + '"');
            dt.Rows.Add("C;X3;Y5;K" + '"' + Math.Round(Convert.ToDecimal(dModel[0].OP_ACC_SALE), 2) + '"');
            dt.Rows.Add("C;X3;Y6;K" + '"' + Math.Round(Convert.ToDecimal(dModel[0].OP_ACC_GP), 2) + '"');
            dt.Rows.Add("C;X3;Y7;K" + '"' + Math.Round(Convert.ToDecimal(dModel[0].OP_GPR_ACC_PERCENT), 2) + '"');
            dt.Rows.Add("C;X3;Y12;K" + '"' + Math.Round(Convert.ToDecimal(dModel[0].OP_EXP_SALES), 2) + '"');
            dt.Rows.Add("C;X3;Y13;K" + '"' + Math.Round(Convert.ToDecimal(dModel[0].OP_EXP_GP), 2) + '"');
            dt.Rows.Add("C;X3;Y14;K" + '"' + Math.Round(Convert.ToDecimal(dModel[0].OP_T_GPR_PERCENT), 2) + '"');

            dt.Rows.Add("C;X4;Y2;K" + '"' + Math.Round(Convert.ToDecimal(dModel[0].BP_PART_SALE), 2) + '"');
            dt.Rows.Add("C;X4;Y3;K" + '"' + Math.Round(Convert.ToDecimal(dModel[0].BP_PART_GP), 2) + '"');
            dt.Rows.Add("C;X4;Y4;K" + '"' + Math.Round(Convert.ToDecimal(dModel[0].BP_GPR_PART_PERCENT), 2) + '"');
            dt.Rows.Add("C;X4;Y5;K" + '"' + Math.Round(Convert.ToDecimal(dModel[0].BP_ACC_SALE), 2) + '"');
            dt.Rows.Add("C;X4;Y6;K" + '"' + Math.Round(Convert.ToDecimal(dModel[0].BP_ACC_GP), 2) + '"');
            dt.Rows.Add("C;X4;Y7;K" + '"' + Math.Round(Convert.ToDecimal(dModel[0].BP_GPR_ACC_PERCENT), 2) + '"');
            dt.Rows.Add("C;X4;Y12;K" + '"' + Math.Round(Convert.ToDecimal(dModel[0].BP_EXP_SALES), 2) + '"');
            dt.Rows.Add("C;X4;Y13;K" + '"' + Math.Round(Convert.ToDecimal(dModel[0].BP_EXP_GP), 2) + '"');
            dt.Rows.Add("C;X4;Y14;K" + '"' + Math.Round(Convert.ToDecimal(dModel[0].BP_T_GPR_PERCENT), 2) + '"');

            dt.Rows.Add("C;X5;Y2;K" + '"' + Math.Round(Convert.ToDecimal(dModel[0].ATT_PART_SALE), 2) + '"');
            dt.Rows.Add("C;X5;Y3;K" + '"' + Math.Round(Convert.ToDecimal(dModel[0].ATT_PART_GP), 2) + '"');
            dt.Rows.Add("C;X5;Y4;K" + '"' + Math.Round(Convert.ToDecimal(dModel[0].ATT_GPR_PART_PERCENT), 2) + '"');
            dt.Rows.Add("C;X5;Y5;K" + '"' + Math.Round(Convert.ToDecimal(dModel[0].ATT_ACC_SALE), 2) + '"');
            dt.Rows.Add("C;X5;Y6;K" + '"' + Math.Round(Convert.ToDecimal(dModel[0].ATT_ACC_GP), 2) + '"');
            dt.Rows.Add("C;X5;Y7;K" + '"' + Math.Round(Convert.ToDecimal(dModel[0].ATT_GPR_ACC_PERCENT), 2) + '"');
            dt.Rows.Add("C;X5;Y12;K" + '"' + Math.Round(Convert.ToDecimal(dModel[0].ATT_EXP_SALES), 2) + '"');
            dt.Rows.Add("C;X5;Y13;K" + '"' + Math.Round(Convert.ToDecimal(dModel[0].ATT_EXP_GP), 2) + '"');
            dt.Rows.Add("C;X5;Y14;K" + '"' + Math.Round(Convert.ToDecimal(dModel[0].ATT_T_GPR_PERCENT), 2) + '"');


            dt.Rows.Add("C;X1;Y1;K\"Actual as\"");
            dt.Rows.Add("C;X1;Y2;K\"Part Sale\"");
            dt.Rows.Add("C;X1;Y3;K\"Part GP\"");
            dt.Rows.Add("C;X1;Y4;K\"%GPR Parts\"");
            dt.Rows.Add("C;X1;Y5;K\"Acc Sale\"");
            dt.Rows.Add("C;X1;Y6;K\"Acc GP\"");
            dt.Rows.Add("C;X1;Y7;K\"%GPR Acc\"");
            dt.Rows.Add("C;X1;Y8;K\"Acc Dealer Option Sale\"");
            dt.Rows.Add("C;X1;Y9;K\"Acc Dealer Option Sale GP\"");
            dt.Rows.Add("C;X1;Y10;K\"Acc PDI Sale\"");
            dt.Rows.Add("C;X1;Y11;K\"Acc PDI Sale GP\"");
            dt.Rows.Add("C;X1;Y12;K\"Exp Sales Grand Total\"");
            dt.Rows.Add("C;X1;Y13;K\"Exp GP Grand Total\"");
            dt.Rows.Add("C;X1;Y14;K\"%GPR\"");

            dt.Rows.Add("E");

            x = DataService.WriteCSVNoColumn(dt, filename, floderLocation);
            Controllers.SystemApiController response = new Controllers.SystemApiController();
            var File = response.downloadFile(x.Substring(1), filename);
            return File;
        }

        [HttpPost, Route("SNSQ074E_GetInvoiceExport")]
        [Authorize]
        public HttpResponseMessage SNSQ074E_GetInvoiceExport([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSQ074E_INV_DETAIL_EXP";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_BUYER_CODE", OracleDbType.Varchar2).Value = pParam.BUYER_CODE;
            cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam.BRANCH_CODE;
            cmd.Parameters.Add("P_FROM_DATE", OracleDbType.Date).Value = pParam.FROM_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(pParam.FROM_DATE);
            cmd.Parameters.Add("P_TO_DATE", OracleDbType.Date).Value = pParam.TO_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(pParam.TO_DATE);
            cmd.Parameters.Add("cReturn", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            string x = SNSQ074E_InvoiceExport(ds.Tables[0], "SNSQ074E_INVOICE.csv", "SNSQ074E");
            SystemApiController sys = new SystemApiController();
            var respons = sys.downloadFile(x.Substring(1), "SNSQ074E_INVOICE.csv");
            return respons;

        }

        public static string SNSQ074E_InvoiceExport(DataTable dt, string fileName, string FloderLocation)
        {
            bool Succes = false;

            string gLocation = "~/tempFile/" + FloderLocation + "/";
            string gWriteLocation = System.Web.HttpContext.Current.Server.MapPath(gLocation);

            FileInfo filePath = new FileInfo(gWriteLocation);
            filePath.Directory.Create();
            string gFileLocation = Path.Combine(gWriteLocation, fileName);
            string gReturnLocation = Path.Combine(gLocation, fileName);
            SystemApiController sys = new SystemApiController();
            try
            {
                StringBuilder sb = new StringBuilder();

                string[] HeaderForExport = new string[]
                {
                    "Seq.",
                    "Buyer Code",
                    "Invoice Number",
                    "Issue Date",
                    "B/L Date or Delivery Date",
                    "Exchnage Rate",
                    "Currency",
                    "Qty",
                    "FOB",
                    "Freight",
                    "Insurance",
                    "CIF(Currency)",
                    "CIF(Baht)",
                    "CIF(M Baht)",
                    "Recheck",
                    "Remark",
                    "Flight No/Vessel",
                    "Part Sale",
                    "Part THB",
                    "Parts STD",
                    "Parts STD in THB-K",
                    "Parts GP in THB",
                    "%",
                    "Parts AVG",
                    "Parts AVG in THB-K",
                    "Parts AVG GP in THB",
                    "%",
                    "ACC Sale",
                    "ACC THB",
                    "ACC STD",
                    "ACC STD in THB-K",
                    "ACC GP in THB",
                    "%",
                    "ACC AVG",
                    "ACC AVG in THB-K",
                    "ACC AVG GP in THB",
                    "%"
                };

                IEnumerable<string> columnNames = HeaderForExport.Select(h => h.ToString());
                sb.AppendLine(string.Join(",", columnNames));

                foreach (DataRow row in dt.Rows)
                {
                    for (int i = 0; i < dt.Columns.Count - 0; i++)
                    {
                        sb.Append(CsvEscape(row[i].ToString()));
                        sb.Append(i == dt.Columns.Count - 1 ? "\n" : ",");
                    }
                }
                File.WriteAllText(gFileLocation, sb.ToString());

                Succes = true;
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
            }

            if (Succes)
            {
                return gReturnLocation;
            }
            else
            {
                return "";
            }
        }

        [HttpPost, Route("SNSQ074E_GetPickingExport")]
        [Authorize]
        public HttpResponseMessage SNSQ074E_GetPickingExport([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSQ074E_PICK_DETAIL_EXP";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_BUYER_CODE", OracleDbType.Varchar2).Value = pParam.BUYER_CODE;
            cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam.BRANCH_CODE;
            cmd.Parameters.Add("cReturn", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            string x = SNSQ074E_PickingExport(ds.Tables[0], "SNSQ074E_PICKING.csv", "SNSQ074E");
            SystemApiController sys = new SystemApiController();
            var respons = sys.downloadFile(x.Substring(1), "SNSQ074E_PICKING.csv");
            return respons;

        }

        public static string SNSQ074E_PickingExport(DataTable dt, string fileName, string FloderLocation)
        {
            bool Succes = false;

            string gLocation = "~/tempFile/" + FloderLocation + "/";
            string gWriteLocation = System.Web.HttpContext.Current.Server.MapPath(gLocation);

            FileInfo filePath = new FileInfo(gWriteLocation);
            filePath.Directory.Create();
            string gFileLocation = Path.Combine(gWriteLocation, fileName);
            string gReturnLocation = Path.Combine(gLocation, fileName);
            SystemApiController sys = new SystemApiController();
            try
            {
                StringBuilder sb = new StringBuilder();

                string[] HeaderForExport = new string[]
                {
                    "Seq.",
                    "Customer Code",
                    "PL No",
                    "PL Date",
                    "Order Class",
                    "Exchnage Rate",
                    "Currency",
                    "Qty",
                    "FOB",
                    "CIF(Currency)",
                    "CIF(Baht)",
                    "CIF(M Baht)",
                    "Recheck",
                    "Remark",
                    "Part Sale",
                    "Part THB",
                    "Parts AVG",
                    "Parts AVG in THB-K",
                    "Parts GP in THB",
                    "%",
                    "ACC Sale",
                    "ACC THB",
                    "ACC AVG",
                    "ACC AVG in THB-K",
                    "ACC GP in THB",
                    "%"
                };

                IEnumerable<string> columnNames = HeaderForExport.Select(h => h.ToString());
                sb.AppendLine(string.Join(",", columnNames));

                foreach (DataRow row in dt.Rows)
                {
                    for (int i = 0; i < dt.Columns.Count - 0; i++)
                    {
                        sb.Append(CsvEscape(row[i].ToString()));
                        sb.Append(i == dt.Columns.Count - 1 ? "\n" : ",");
                    }
                }
                File.WriteAllText(gFileLocation, sb.ToString());

                Succes = true;
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
            }

            if (Succes)
            {
                return gReturnLocation;
            }
            else
            {
                return "";
            }
        }

        [HttpPost, Route("SNSQ074E_ExchangeExport")]
        public HttpResponseMessage SNSQ074E_ExchangeExport(List<DataModel.SNSQ074E_EXCHANGE> dModel)
        {
            DataTable dt = new DataTable();
            string x = string.Empty;
            string filename = "SNSQ074E_EXCHANGE.csv";
            string floderLocation = "SNSQ074E_EXCHANGE";
            dt.Columns.Add("colum1");

            dt.Rows.Add("ID;PO2C");
            dt.Rows.Add("B");

            //table4
            dt.Rows.Add("C;X1;Y1;K\"Date\"");
            dt.Rows.Add("C;X2;Y1;K\"USD\"");
            dt.Rows.Add("C;X3;Y1;K\"NZD\"");
            dt.Rows.Add("C;X4;Y1;K\"AUD\"");

            int countrow = 0;
            int itemno = 0;
            for (var i = 0; i < dModel.Count; i++)
            {
                countrow = i + 2;
                itemno = i + 1;
                dt.Rows.Add("C;X1;Y" + countrow + ";K" + '"' + Convert.ToDateTime(dModel[i].EFFECTIVE_DATE).ToString("dd-MM-yyyy") + '"');
                dt.Rows.Add("C;X2;Y" + countrow + ";K" + '"' + dModel[i].USD_RATE + '"');
                dt.Rows.Add("C;X3;Y" + countrow + ";K" + '"' + dModel[i].NZD_RATE + '"');
                dt.Rows.Add("C;X4;Y" + countrow + ";K" + '"' + dModel[i].AUD_RATE + '"');
            }

            dt.Rows.Add("E");

            x = DataService.WriteCSVNoColumn(dt, filename, floderLocation);
            Controllers.SystemApiController response = new Controllers.SystemApiController();
            var File = response.downloadFile(x.Substring(1), filename);
            return File;
        }
        #endregion

        #region SNSE199
        [HttpGet, Route("SNSE199_GetCustomerCode/{cusID}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE199_GetCustomerCode(string cusID, int noofshowrec)
        {
            DataSet ds = new DataSet();

            cusID = ismoUtil.escapeCharacter(cusID);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE199_CUSTOMER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = cusID.ToUpper() == "NULL" ? (object)DBNull.Value : cusID;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE199_GetCustomerCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE199_getData")]
        [Authorize]
        public IHttpActionResult SNSE199_getData([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE199_GET_DATA";
                cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_IP_ADDRESS", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE199_getData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }


        [HttpPost, Route("SNSE199_Export")]
        [Authorize]
        public HttpResponseMessage SNSE199_Export(List<DataModel.SNSE199> Data)
        {
            HttpResponseMessage response = new HttpResponseMessage(HttpStatusCode.OK);
            string gLocation = "~/tempFile/" + "SNSE199" + "/";
            string gWriteLocation = System.Web.HttpContext.Current.Server.MapPath(gLocation);
            FileInfo filePath = new FileInfo(gWriteLocation);
            filePath.Directory.Create();
            string gFileLocation = Path.Combine(gWriteLocation, "SNSE199.xlsx");
            string gReturnLocation = Path.Combine(gLocation, "SNSE199.xlsx");

            var excel2 = Data.ToExcel(x => x
            .AddColumn("No.", z => z.SEQ)
            .AddColumn("Customer Code", z => z.CUSTOMER_CODE)
            .AddColumn("Order No.", z => z.ORDER_NO)
            .AddColumn("Order Class", z => z.ORDER_CLASS)
            .AddColumn("Order Date", z => z.ORDER_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(z.ORDER_DATE).ToString("yyyy-MMM-dd"))
            .AddColumn("Item No.", z => z.ITEM_NO)
            .AddColumn("Sub No.", z => z.SUB_ITEM_NO)
            .AddColumn("Order Part No.", z => z.ORDER_PART_NO)
            .AddColumn("Order Qty", z => z.ORDER_QTY)
            .AddColumn("Accept Part No", z => z.ACCEPT_PART_NO)
            .AddColumn("Accept Qty", z => z.ACCEPT_QTY)
            .AddColumn("Batch ETD", z => z.BATCH_ETD == null ? (object)DBNull.Value : Convert.ToDateTime(z.BATCH_ETD).ToString("yyyy-MMM-dd"))
            .AddColumn("Issue Picking Date", z => z.ISSUE_PICK_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(z.ISSUE_PICK_DATE).ToString("yyyy-MMM-dd"))
            .AddColumn("Working Leadtime", z => z.WORKING_LEADTIME)
            .AddColumn("Export ETD", z => z.EXPORT_ETD == null ? (object)DBNull.Value : Convert.ToDateTime(z.EXPORT_ETD).ToString("yyyy-MMM-dd"))
            .AddColumn("Shipment Leadtime", z => z.SHIPMENT_LEADTIME)
            .AddColumn("Export ETA", z => z.EXPORT_ETA == null ? (object)DBNull.Value : Convert.ToDateTime(z.EXPORT_ETA).ToString("yyyy-MMM-dd"))
            .AddColumn("Message", z => z.MESSAGE));


            byte[] excel = excel2;
            // byte[] excel = dt.ToExcel();
            File.WriteAllBytes(gFileLocation, excel);
            string path = HttpContext.Current.Server.MapPath("~/" + gReturnLocation.Substring(1));
            if (!File.Exists(path))
            {
                return Request.CreateErrorResponse(HttpStatusCode.NotFound, "File not found.");
            }
            try
            {
                string mimeType = MimeMapping.GetMimeMapping("SNSE199.xlsx");
                StreamContent x = new StreamContent(new FileStream(path, FileMode.Open, FileAccess.Read));
                response.Content = x;
                response.Content.Headers.ContentType = new MediaTypeHeaderValue(mimeType);
                response.Content.Headers.ContentDisposition = new ContentDispositionHeaderValue("attachment");
                response.Content.Headers.ContentDisposition.FileName = "SNSE199.xlsx";
            }
            catch (Exception ex)
            {
                return Request.CreateResponse(HttpStatusCode.BadRequest, ex);
            }
            return response;
        }

        //[HttpPost, Route("SNSE199_Export")]
        //public HttpResponseMessage SNSE199_Export(List<DataModel.SNSE199> dModel)
        //{
        //    DataTable dt = new DataTable();
        //    string x = string.Empty;
        //    string filename = "SNSE199.csv";
        //    string floderLocation = "SNSE199";
        //    dt.Columns.Add("colum1");

        //    dt.Rows.Add("ID;PO2C");
        //    dt.Rows.Add("B");
        //    dt.Rows.Add("C;X1;Y1;K\"No.\"");
        //    dt.Rows.Add("C;X2;Y1;K\"Customer Code\"");
        //    dt.Rows.Add("C;X3;Y1;K\"Order No\"");
        //    dt.Rows.Add("C;X4;Y1;K\"Order Class\"");
        //    dt.Rows.Add("C;X5;Y1;K\"Order Date\"");
        //    dt.Rows.Add("C;X6;Y1;K\"Item No.\"");
        //    dt.Rows.Add("C;X7;Y1;K\"Sub No.\"");
        //    dt.Rows.Add("C;X8;Y1;K\"Order Part No.\"");
        //    dt.Rows.Add("C;X9;Y1;K\"Order Qty\"");
        //    dt.Rows.Add("C;X10;Y1;K\"Accept Part No\"");
        //    dt.Rows.Add("C;X11;Y1;K\"Accept Qty\"");
        //    dt.Rows.Add("C;X12;Y1;K\"Batch ETD\"");
        //    dt.Rows.Add("C;X13;Y1;K\"Issue Picking Date\"");
        //    dt.Rows.Add("C;X14;Y1;K\"Working Leadtime\"");
        //    dt.Rows.Add("C;X15;Y1;K\"Export ETD\"");
        //    dt.Rows.Add("C;X16;Y1;K\"Shipment Leadtime\"");
        //    dt.Rows.Add("C;X17;Y1;K\"Export ETA\"");
        //    dt.Rows.Add("C;X18;Y1;K\"Message\"");

        //    int countrow = 0;
        //    int itemno = 0;
        //    for (var i = 0; i < dModel.Count; i++)
        //    {
        //        countrow = i + 2;
        //        itemno = i + 1;
        //        dt.Rows.Add("C;X1;Y" + countrow + ";K" + '"' + dModel[i].SEQ + '"');
        //        dt.Rows.Add("C;X2;Y" + countrow + ";K" + '"' + dModel[i].CUSTOMER_CODE + '"');
        //        dt.Rows.Add("C;X3;Y" + countrow + ";K" + '"' + dModel[i].ORDER_NO + '"');
        //        dt.Rows.Add("C;X4;Y" + countrow + ";K" + '"' + dModel[i].ORDER_CLASS + '"');
        //        dt.Rows.Add("C;X5;Y" + countrow + ";K" + '"' + Convert.ToDateTime(dModel[i].ORDER_DATE).ToString("yyyy-MMM-dd") + '"');
        //        dt.Rows.Add("C;X6;Y" + countrow + ";K" + '"' + dModel[i].ITEM_NO + '"');
        //        dt.Rows.Add("C;X7;Y" + countrow + ";K" + '"' + dModel[i].SUB_ITEM_NO + '"');
        //        dt.Rows.Add("C;X8;Y" + countrow + ";K" + '"' + dModel[i].ORDER_PART_NO + '"');
        //        dt.Rows.Add("C;X9;Y" + countrow + ";K" + '"' + dModel[i].ORDER_QTY + '"');
        //        dt.Rows.Add("C;X10;Y" + countrow + ";K" + '"' + dModel[i].ACCEPT_PART_NO + '"');
        //        dt.Rows.Add("C;X11;Y" + countrow + ";K" + '"' + dModel[i].ACCEPT_QTY + '"');
        //        if (dModel[i].BATCH_ETD != null && dModel[i].BATCH_ETD != "")
        //        {
        //            dt.Rows.Add("C;X12;Y" + countrow + ";K" + '"' + Convert.ToDateTime(dModel[i].BATCH_ETD).ToString("yyyy-MMM-dd") + '"');
        //        }
        //        else
        //        {
        //            dt.Rows.Add("C;X12;Y" + countrow + ";K" + '"' + dModel[i].BATCH_ETD + '"');
        //        }
        //        if (dModel[i].ISSUE_PICK_DATE != null && dModel[i].ISSUE_PICK_DATE != "")
        //        {
        //            dt.Rows.Add("C;X13;Y" + countrow + ";K" + '"' + Convert.ToDateTime(dModel[i].ISSUE_PICK_DATE).ToString("yyyy-MMM-dd") + '"');
        //        }
        //        else
        //        {
        //            dt.Rows.Add("C;X13;Y" + countrow + ";K" + '"' + dModel[i].ISSUE_PICK_DATE + '"');
        //        }
        //        dt.Rows.Add("C;X14;Y" + countrow + ";K" + '"' + dModel[i].WORKING_LEADTIME + '"');
        //        if (dModel[i].EXPORT_ETD != null && dModel[i].EXPORT_ETD != "")
        //        {
        //            dt.Rows.Add("C;X15;Y" + countrow + ";K" + '"' + Convert.ToDateTime(dModel[i].EXPORT_ETD).ToString("yyyy-MMM-dd") + '"');
        //        }
        //        else
        //        {
        //            dt.Rows.Add("C;X15;Y" + countrow + ";K" + '"' + dModel[i].EXPORT_ETD + '"');
        //        }
        //        dt.Rows.Add("C;X16;Y" + countrow + ";K" + '"' + dModel[i].SHIPMENT_LEADTIME + '"');
        //        if (dModel[i].EXPORT_ETA != null && dModel[i].EXPORT_ETA != "")
        //        {
        //            dt.Rows.Add("C;X17;Y" + countrow + ";K" + '"' + Convert.ToDateTime(dModel[i].EXPORT_ETA).ToString("yyyy-MMM-dd") + '"');
        //        }
        //        else
        //        {
        //            dt.Rows.Add("C;X17;Y" + countrow + ";K" + '"' + dModel[i].EXPORT_ETA + '"');
        //        }
        //        dt.Rows.Add("C;X18;Y" + countrow + ";K" + '"' + dModel[i].MESSAGE + '"');
        //    }

        //    dt.Rows.Add("E");

        //    x = DataService.WriteCSVNoColumn(dt, filename, floderLocation);
        //    Controllers.SystemApiController response = new Controllers.SystemApiController();
        //    var File = response.downloadFile(x.Substring(1), filename);
        //    return File;
        //}

        [HttpPost, Route("SNSE199_ETD_Provide")]
        [Authorize]
        public IHttpActionResult SNSE199_ETD_Provide([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE199_PROVIDE_ETD";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_CREATE_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_IP_ADDRESS", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE199_ETD_Provide");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        #endregion

        #region SNSE034

        [HttpGet, Route("SNSE034_GetCustomerCode/{cusID}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE034_GetCustomerCode(string cusID, int noofshowrec)
        {
            DataSet ds = new DataSet();

            cusID = ismoUtil.escapeCharacter(cusID);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE034_GET_CUSTOMER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = cusID.ToUpper() == "NULL" ? (object)DBNull.Value : cusID;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE034_GetCustomerCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        //[HttpPost, Route("SNSE034_GetBranchCode/{branchCode}/{noofshowrec}")]
        //[Authorize]
        //public IHttpActionResult SNSE034_GetBranchCode(string branchCode, int noofshowrec, [FromBody] dynamic pParam)
        //{
        //    DataSet ds = new DataSet();

        //    branchCode = ismoUtil.escapeCharacter(branchCode);
        //    var pData = pParam["DATA"];

        //    OracleCommand cmd = new OracleCommand();
        //    cmd.CommandText = "SP_NP_SNSE034_BRANCH";
        //    cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
        //    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = branchCode.ToUpper() == "NULL" ? (object)DBNull.Value : branchCode;
        //    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData["CUSTOMER_CODE"];
        //    cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
        //    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

        //    ds = oOracledb.SqlQueryDS_CMD(cmd);

        //    return Json(DTFM.convertToList(ds));
        //}

        [HttpPost, Route("SNSE034_getData")]
        [Authorize]
        public IHttpActionResult SNSE034_getData([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE034_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam.BRANCH_CODE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE034_getData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNSE034_Update")]
        [Authorize]
        [RunBatchGuard]
        public IHttpActionResult SNSE034_Update([FromBody] dynamic[] pDataList)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();

            try
            {
                foreach (var data in pDataList)
                {
                    OracleCommand cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE034_UPDATE";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = data.BRANCH_CODE;
                    cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = data.CASE_NO;
                    cmd.Parameters.Add("P_CONTAINER_NO", OracleDbType.Varchar2).Value = data.CONTAINER_NO;
                    cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE034";
                    cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                    ds = oOracledb.SqlQueryDS_CMD(cmd);
                    dt = ds.Tables[0];
                }
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE034_Update");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNSE034_Delete")]
        [Authorize]
        [RunBatchGuard]
        public IHttpActionResult SNSE034_Delete([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE034_DELETE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = param.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = param.BRANCH_CODE;
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = param.CASE_NO;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE034_Delete");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        #endregion

        #region SNSE015

        [HttpGet, Route("SNSE015_GetCustomerCode/{cusID}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE015_GetCustomerCode(string cusID, int noofshowrec)
        {
            DataSet ds = new DataSet();

            cusID = ismoUtil.escapeCharacter(cusID);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE015_N_CUSTOMER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = cusID.ToUpper() == "NULL" ? (object)DBNull.Value : cusID;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE015_GetCustomerCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE015_GetOrderNo/{orderNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE015_GetOrderNo(string orderNo, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            orderNo = ismoUtil.escapeCharacter(orderNo);
            var pData = pParam["DATA"];

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE015_N_ORDER_NO";
                cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = orderNo.ToUpper() == "NULL" ? (object)DBNull.Value : orderNo;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE015_GetOrderNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNSE015_GetOrderNoRpt/{orderNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE015_GetOrderNoRpt(string orderNo, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            orderNo = ismoUtil.escapeCharacter(orderNo);
            var pData = pParam["DATA"];

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE015_N_ORDER_NO_RPT";
                cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = orderNo.ToUpper() == "NULL" ? (object)DBNull.Value : orderNo;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE015_GetOrderNoRpt");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }


        }

        [HttpPost, Route("SNSE015_getData")]
        [Authorize]
        public IHttpActionResult SNSE015_getData([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE015_N_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pParam.ORDER_NO;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE015_getData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNSE015_AllocateOrder")]
        [Authorize]
        [RunBatchGuard]
        public IHttpActionResult SNSE015_AllocateOrder([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE015_N_ALLOCATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pParam.ORDER_NO;
                cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = pParam.ORDER_CLASS;
                cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE015_N";
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE015_AllocateOrder");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }


        }
        #endregion

        #region SNSE025N
        [HttpGet, Route("SNSE025_N_GetCustomerCode/{cusID}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE025_N_GetCustomerCode(string cusID, int noofshowrec)
        {
            DataSet ds = new DataSet();

            cusID = ismoUtil.escapeCharacter(cusID);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE025N_CUSTOMER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = cusID.ToUpper() == "NULL" ? (object)DBNull.Value : cusID;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE025_N_GetCustomerCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE025_N_GetBranchCode/{branchCode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE025_N_GetBranchCode(string branchCode, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            branchCode = ismoUtil.escapeCharacter(branchCode);
            var pData = pParam["DATA"];

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE025N_BRANCH";
                cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = branchCode.ToUpper() == "NULL" ? (object)DBNull.Value : branchCode;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE025_N_GetBranchCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNSE025_N_GetCaseNo/{caseNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE025_N_GetCaseNo(string caseNo, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            caseNo = ismoUtil.escapeCharacter(caseNo);
            var pData = pParam["DATA"];

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE025N_CASE_NO";
                cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pData["BRANCH_CODE"];
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = caseNo.ToUpper() == "NULL" ? (object)DBNull.Value : caseNo;
                cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = pData["SENT_BY"];
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE025_N_GetCaseNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNSE025_N_GetCaseNo2")]
        [Authorize]
        public IHttpActionResult SNSE025_N_GetCaseNo2([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE025N_CASE_NO2";
                cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam["BRANCH_CODE"];
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = pParam["SENT_BY"];
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE025_N_GetCaseNo2");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNSE025_N_GetInstructionNo/{instructionNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE025_N_GetInstructionNo(string instructionNo, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            instructionNo = ismoUtil.escapeCharacter(instructionNo);
            var pData = pParam["DATA"];

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE025N_INSTRUCTION_NO";
                cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pData["BRANCH_CODE"];
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = pData["SENT_BY"];
                cmd.Parameters.Add("P_INSTRUCTION_NO", OracleDbType.Varchar2).Value = instructionNo.ToUpper() == "NULL" ? (object)DBNull.Value : instructionNo;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE025_N_GetInstructionNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNSE025_N_GetInstructionNo2/{instructionNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE025_N_GetInstructionNo2(string instructionNo, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            instructionNo = ismoUtil.escapeCharacter(instructionNo);
            var pData = pParam["DATA"];

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE025N_INSTRUCTION_NO";
                cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pData["BRANCH_CODE"];
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = pData["SENT_BY"];
                cmd.Parameters.Add("P_INSTRUCTION_NO", OracleDbType.Varchar2).Value = instructionNo.ToUpper() == "NULL" ? (object)DBNull.Value : instructionNo;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE025_N_GetInstructionNo2");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }


        }

        [HttpPost, Route("SNSE025_N_getData")]
        [Authorize]
        public IHttpActionResult SNSE025_N_getData([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE025N_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam.BRANCH_CODE;
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = pParam.CASE_NO;
                cmd.Parameters.Add("P_PACK_DATE", OracleDbType.Date).Value = pParam.PACK_DATE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE025_N_getData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE025_N_Insert")]
        [Authorize]
        [RunBatchGuard]
        public IHttpActionResult SNSE025_N_Insert([FromBody] dynamic[] pDataList)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();

            try
            {
                foreach (var data in pDataList)
                {
                    OracleCommand cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE025N_INSERT";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = data.BRANCH_CODE;
                    cmd.Parameters.Add("P_PACK_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_PACK_DATE", OracleDbType.Date).Value = data.PACK_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(data.PACK_DATE);
                    cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = data.SENT_BY;
                    cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = data.CASE_NO;
                    cmd.Parameters.Add("P_CASE_NO_OLD", OracleDbType.Varchar2).Value = data.CASE_NO_OLD;
                    cmd.Parameters.Add("P_SUB_CASE_NO", OracleDbType.Varchar2).Value = data.SUB_CASE_NO;
                    cmd.Parameters.Add("P_CASE_SEQ", OracleDbType.Varchar2).Value = data.CASE_SEQ;
                    cmd.Parameters.Add("P_INSTRUCTION_NO", OracleDbType.Varchar2).Value = data.INSTRUCTION_NO;
                    cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = data.ORDER_NO;
                    cmd.Parameters.Add("P_ORDER_DATE", OracleDbType.Date).Value = data.ORDER_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(data.ORDER_DATE);
                    cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Varchar2).Value = data.ITEM_NO;
                    cmd.Parameters.Add("P_SUB_ITEM_NO", OracleDbType.Varchar2).Value = data.SUB_ITEM_NO;
                    cmd.Parameters.Add("P_OLD_SUB_ITEM_NO", OracleDbType.Varchar2).Value = data.OLD_SUB_ITEM_NO;
                    cmd.Parameters.Add("P_OVERSEA_FLAG_C", OracleDbType.Varchar2).Value = data.OVERSEA_FLAG_C;
                    cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = data.ORDER_CLASS;
                    cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = data.ORDER_PART_NO;
                    cmd.Parameters.Add("P_ACCEPT_PART_NO", OracleDbType.Varchar2).Value = data.ACCEPT_PART_NO;
                    cmd.Parameters.Add("P_NET_WEIGHT", OracleDbType.Varchar2).Value = data.NET_WEIGHT;
                    cmd.Parameters.Add("P_ACCEPT_QTY", OracleDbType.Varchar2).Value = data.ACCEPT_QTY;
                    cmd.Parameters.Add("P_PACK_QTY", OracleDbType.Int32).Value = data.PACK_QTY;
                    cmd.Parameters.Add("P_TARIFF_CODE", OracleDbType.Varchar2).Value = data.TARIFF_CODE;
                    cmd.Parameters.Add("P_DUTY_RATE", OracleDbType.Varchar2).Value = data.DUTY_RATE;
                    cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = data.INVOICE_NO;
                    cmd.Parameters.Add("P_INVOICE_DATE", OracleDbType.Date).Value = data.INVOICE_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(data.INVOICE_DATE);
                    cmd.Parameters.Add("P_PICK_LOT_SEQ", OracleDbType.Varchar2).Value = data.PICK_LOT_SEQ;
                    cmd.Parameters.Add("P_PICK_LOT_DATE", OracleDbType.Date).Value = data.PICK_LOT_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(data.PICK_LOT_DATE);
                    cmd.Parameters.Add("P_CREATE_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE025_N";
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                    ds = oOracledb.SqlQueryDS_CMD(cmd);
                    dt = ds.Tables[0];
                }
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE025_N_Insert");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }




        [HttpPost, Route("SNSE025_N_Update")]
        [Authorize]
        [RunBatchGuard]
        public IHttpActionResult SNSE025_N_Update([FromBody] dynamic[] pDataList)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();

            try
            {
                foreach (var data in pDataList)
                {
                    OracleCommand cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE025N_UPDATE";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = data.BRANCH_CODE;
                    cmd.Parameters.Add("P_PACK_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_PACK_DATE", OracleDbType.Date).Value = data.PACK_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(data.PACK_DATE);
                    cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = data.SENT_BY;
                    cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = data.CASE_NO;
                    cmd.Parameters.Add("P_SUB_CASE_NO", OracleDbType.Varchar2).Value = data.SUB_CASE_NO;
                    cmd.Parameters.Add("P_CASE_SEQ", OracleDbType.Varchar2).Value = data.CASE_SEQ;
                    cmd.Parameters.Add("P_INSTRUCTION_NO", OracleDbType.Varchar2).Value = data.INSTRUCTION_NO;
                    cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = data.ORDER_NO;
                    cmd.Parameters.Add("P_ORDER_DATE", OracleDbType.Date).Value = data.ORDER_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(data.ORDER_DATE);
                    cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Varchar2).Value = data.ITEM_NO;
                    cmd.Parameters.Add("P_SUB_ITEM_NO", OracleDbType.Varchar2).Value = data.SUB_ITEM_NO;
                    cmd.Parameters.Add("P_OLD_SUB_ITEM_NO", OracleDbType.Varchar2).Value = data.OLD_SUB_ITEM_NO;
                    cmd.Parameters.Add("P_OVERSEA_FLAG_C", OracleDbType.Varchar2).Value = data.OVERSEA_FLAG_C;
                    cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = data.ORDER_CLASS;
                    cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = data.ORDER_PART_NO;
                    cmd.Parameters.Add("P_ACCEPT_PART_NO", OracleDbType.Varchar2).Value = data.ACCEPT_PART_NO;
                    cmd.Parameters.Add("P_NET_WEIGHT", OracleDbType.Varchar2).Value = data.NET_WEIGHT;
                    cmd.Parameters.Add("P_ACCEPT_QTY", OracleDbType.Varchar2).Value = data.ACCEPT_QTY;
                    cmd.Parameters.Add("P_PACK_QTY", OracleDbType.Int32).Value = data.PACK_QTY;
                    cmd.Parameters.Add("P_TARIFF_CODE", OracleDbType.Varchar2).Value = data.TARIFF_CODE;
                    cmd.Parameters.Add("P_DUTY_RATE", OracleDbType.Varchar2).Value = data.DUTY_RATE;
                    cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = data.INVOICE_NO;
                    cmd.Parameters.Add("P_INVOICE_DATE", OracleDbType.Date).Value = data.INVOICE_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(data.INVOICE_DATE);
                    cmd.Parameters.Add("P_PICK_LOT_SEQ", OracleDbType.Varchar2).Value = data.PICK_LOT_SEQ;
                    cmd.Parameters.Add("P_PICK_LOT_DATE", OracleDbType.Date).Value = data.PICK_LOT_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(data.PICK_LOT_DATE);
                    cmd.Parameters.Add("P_CREATE_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE025_N";
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                    ds = oOracledb.SqlQueryDS_CMD(cmd);
                    dt = ds.Tables[0];
                }
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE025_N_Update");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }


        }

        [HttpPost, Route("SNSE025N_Delete")]
        [Authorize]
        [RunBatchGuard]
        public IHttpActionResult SNSE025N_Delete([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE025N_DELETE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = param.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = param.BRANCH_CODE;
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = param.CASE_NO;
                cmd.Parameters.Add("P_INSTRUCTION_NO", OracleDbType.Varchar2).Value = param.INSTRUCTION_NO;
                cmd.Parameters.Add("P_PICK_LOT_DATE", OracleDbType.Date).Value = param.PICK_LOT_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(param.PICK_LOT_DATE);
                cmd.Parameters.Add("P_PICK_LOT_SEQ", OracleDbType.Varchar2).Value = param.PICK_LOT_SEQ;
                cmd.Parameters.Add("P_PACK_QTY", OracleDbType.Int32).Value = param.PACK_QTY;
                cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE025_N";
                cmd.Parameters.Add("C_RE", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE025N_Delete");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }


        }
        #endregion

        #region SNSE031

        [HttpPost, Route("SNSE031_GetData")]
        [Authorize]
        public IHttpActionResult SNSE031_GetData([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE031_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam.BRANCH_CODE;

                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = pParam.CASE_NO;
                cmd.Parameters.Add("P_PACK_DATE", OracleDbType.Date).Value = pParam.PACK_DATE;
                cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = pParam.SENT_BY;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE031_GetData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNSE031_NetWeightList")]
        [Authorize]
        public IHttpActionResult SNSE031_NetWeightList([FromBody] dynamic pParam)
        {
            try
            {
                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE031_GET_NET_WEIGHT";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam.BRANCH_CODE;
                cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = pParam.CASE_NO;
                cmd.Parameters.Add("P_PACK_DATE", OracleDbType.Date).Value = pParam.PACK_DATE;
                cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = pParam.SENT_BY;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE031_GET_NET_WEIGHT");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }


        [HttpPost, Route("SNSE031_Update")]
        [Authorize]
        [RunBatchGuard]
        public IHttpActionResult SNSE031_Update([FromBody] dynamic[] pDataList)
        {
            DataSet ds = new DataSet();
            String gCaseNo = "";
            try
            {
                foreach (var data in pDataList)
                {
                    OracleCommand cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE031_UPDATE";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = data.BRANCH_CODE;
                    cmd.Parameters.Add("P_PACK_DATE", OracleDbType.Date).Value = data.PACK_DATE;
                    cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = data.SENT_BY;
                    cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = data.CASE_NO;
                    cmd.Parameters.Add("P_INVOICE_STATUS", OracleDbType.Varchar2).Value = data.INVOICE_STATUS;

                    cmd.Parameters.Add("P_GROSS_WEIGHT", OracleDbType.Varchar2).Value = data.GROSS_WEIGHT;
                    cmd.Parameters.Add("P_GROSS_MM", OracleDbType.Varchar2).Value = data.GROSS_MM;
                    cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE031";
                    cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                    gCaseNo = data.CASE_NO;
                    ds = oOracledb.SqlQueryDS_CMD(cmd);

                }
                return Json(DTFM.convertToList(ds));

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE031_Update");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, "Case no : " + gCaseNo + " : " + ex.Message));
            }
        }

        [HttpPost, Route("SNSE031_UpdateNetWeight")]
        [Authorize]
        public IHttpActionResult SNSE031_UpdateNetWeight([FromBody] dynamic[] pDataList)
        {
            DataSet ds = new DataSet();

            try
            {
                foreach (var data in pDataList)
                {
                    OracleCommand cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE031_UPD_NETWEIGHT";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = data.ORDER_PART_NO;
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = data.BRANCH_CODE;
                    cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = data.CASE_NO;
                    cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = data.SENT_BY;
                    cmd.Parameters.Add("P_NET_WEIGHT", OracleDbType.Int32).Value = data.NET_WEIGHT_SAVE;
                    cmd.Parameters.Add("P_PACK_DATE", OracleDbType.Date).Value = data.PACK_DATE;
                    cmd.Parameters.Add("P_ACCEPT_QTY", OracleDbType.Varchar2).Value = data.ACCEPT_QTY;

                    cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE031";
                    cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                    ds = oOracledb.SqlQueryDS_CMD(cmd);

                }
                return Json(DTFM.convertToList(ds));

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE031_UpdateNetWeight");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        #endregion

        #region SNSE032

        [HttpGet, Route("SNSE032_GetCustomerCode/{cusID}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE032_GetCustomerCode(string cusID, int noofshowrec)
        {
            DataSet ds = new DataSet();

            cusID = ismoUtil.escapeCharacter(cusID);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE032_CUSTOMER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = cusID.ToUpper() == "NULL" ? (object)DBNull.Value : cusID;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE032_GetCustomerCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpPost, Route("SNSE032_GetData")]
        [Authorize]
        public IHttpActionResult SNSE032_GetData([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE032_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam.BRANCH_CODE;
                cmd.Parameters.Add("C_RETURN1", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN2", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE032_GetData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }


        [HttpPost, Route("SNSE032_GetDataModify")]
        [Authorize]
        public IHttpActionResult SNSE032_GetDataModify([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE032_GET_DATA_MODIFY";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam.BRANCH_CODE;
                cmd.Parameters.Add("P_PINVOICE_NO", OracleDbType.Varchar2).Value = pParam.PINVOICE_NO;
                cmd.Parameters.Add("C_RETURN1", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN2", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE032_GetDataModify");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }


        [HttpPost, Route("SNSE032_ConfirmPre")]
        [Authorize]
        public IHttpActionResult SNSE032_ConfirmPre([FromBody] dynamic[] dataList)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            string gPreInvoice = "";
            try
            {
                cmd.CommandText = "SP_NP_SNSE032_GEN_DOC_NO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                if (ds.Tables.Count > 0)
                {
                    gPreInvoice = ds.Tables[0].Rows[0]["PRE_INV_NO"].ToString();

                }
            }
            catch (Exception ex)
            {
                gPreInvoice = "";
                return Ok("Error : " + ex.Message);
            }

            try
            {
                foreach (var data in dataList)
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE032_CONFIRM";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_PINVOICE_NO", OracleDbType.Varchar2).Value = gPreInvoice;
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = data.BRANCH_CODE;
                    cmd.Parameters.Add("P_PACK_DATE", OracleDbType.Date).Value = data.PACK_DATE;
                    cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = data.CASE_NO;

                    cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE032";
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                    ds = oOracledb.SqlQueryDS_CMD(cmd);

                    if (ds.Tables.Count > 0)
                    {
                        if (ds.Tables[0].Rows[0]["STATUS"].ToString() != "DONE")
                        {
                            string errString = "Error : " + ds.Tables[0].Rows[0]["STATUS"].ToString();
                            return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, errString));
                        }
                    }

                };

                return Ok("DONE-" + gPreInvoice);
            }
            catch (Exception ex)
            {
                return Ok("Error : " + ex.Message);
            }
        }

        [HttpPost, Route("SNSE032_ConfirmDelete")]
        [Authorize]
        [RunBatchGuard]
        public IHttpActionResult SNSE032_ConfirmDelete([FromBody] dynamic[] dataList)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();

            try
            {
                foreach (var data in dataList)
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE032_DELETE";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);

                    // cmd.Parameters.Add("P_PINVOICE_NO", OracleDbType.Varchar2).Value = data.PINVOICE_NO;
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = data.BRANCH_CODE;
                    cmd.Parameters.Add("P_PACK_DATE", OracleDbType.Date).Value = data.PACK_DATE;
                    cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = data.SENT_BY;
                    cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = data.CASE_NO;

                    cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE032";
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                    ds = oOracledb.SqlQueryDS_CMD(cmd);

                    if (ds.Tables.Count > 0)
                    {
                        if (ds.Tables[0].Rows[0]["STATUS"].ToString() != "DONE")
                        {
                            string errString = "Error : " + ds.Tables[0].Rows[0]["STATUS"].ToString();
                            return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, errString));
                        }
                    }

                };

                return Ok("DONE");
            }
            catch (Exception ex)
            {
                return Ok("Error : " + ex.Message);
            }
        }


        [HttpPost, Route("SNSE032_GetPreInvoiceAir/{preInvoiceNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE032_GetPreInvoiceAir(string preInvoiceNo, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            preInvoiceNo = ismoUtil.escapeCharacter(preInvoiceNo);

            try
            {
                OracleCommand cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNSE032_GET_PINVOICE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PRE_INVOICE", OracleDbType.Varchar2).Value = preInvoiceNo.ToUpper() == "NULL" ? (object)DBNull.Value : preInvoiceNo;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam["BRANCH_CODE"];
                cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = "A";
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE032_GetPreInvoiceAir");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNSE032_ModifyPre")]
        [Authorize]
        public IHttpActionResult SNSE032_ModifyPre([FromBody] dynamic[] dataList)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            String pPInvoiceDate = "";
            try
            {
                pPInvoiceDate = DateTime.Now.ToString("MM-dd-yyyy HH:mm:ss");
                foreach (var data in dataList)
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE032_MODIFY";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = data.BRANCH_CODE;
                    cmd.Parameters.Add("P_PACK_DATE", OracleDbType.Date).Value = data.PACK_DATE;
                    cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = data.CASE_NO;
                    cmd.Parameters.Add("P_PINVOICE_NO", OracleDbType.Varchar2).Value = data.PINVOICE_NO;
                    cmd.Parameters.Add("P_PINVOICE_DATE", OracleDbType.Varchar2).Value = pPInvoiceDate;
                    cmd.Parameters.Add("P_CHK_BOX", OracleDbType.Varchar2).Value = data.CHK_BOX;
                    cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE032";
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                    ds = oOracledb.SqlQueryDS_CMD(cmd);
                    if (ds.Tables.Count > 0)
                    {
                        if (ds.Tables[0].Rows[0]["STATUS"].ToString() != "DONE")
                        {
                            string errString = "Error : " + ds.Tables[0].Rows[0]["STATUS"].ToString();
                            return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, errString));
                        }
                    }
                };
                return Ok("DONE");
            }
            catch (Exception ex)
            {
                return Ok("Error : " + ex.Message);
            }
        }
        #endregion

        #region SNSE033N

        [HttpGet, Route("SNSE033_GetCustomerCodeAir/{cusID}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE033_GetCustomerCodeAir(string cusID, int noofshowrec)
        {
            DataSet ds = new DataSet();

            cusID = ismoUtil.escapeCharacter(cusID);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE033_CUSTOMER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = cusID.ToUpper() == "NULL" ? (object)DBNull.Value : cusID;
                cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = "A";
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE033_GetCustomerCodeAir");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNSE033_GetCustomerCodeSea/{cusID}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE033_GetCustomerCodeSea(string cusID, int noofshowrec)
        {
            DataSet ds = new DataSet();

            cusID = ismoUtil.escapeCharacter(cusID);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE033_CUSTOMER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = cusID.ToUpper() == "NULL" ? (object)DBNull.Value : cusID;
                cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = "S";
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE033_GetCustomerCodeSea");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE033_GetPreInvoiceAir/{preInvoiceNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE033_GetPreInvoiceAir(string preInvoiceNo, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            preInvoiceNo = ismoUtil.escapeCharacter(preInvoiceNo);

            try
            {
                OracleCommand cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNSE033_GET_PINVOICE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PRE_INVOICE", OracleDbType.Varchar2).Value = preInvoiceNo.ToUpper() == "NULL" ? (object)DBNull.Value : preInvoiceNo;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam["BRANCH_CODE"];
                cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = "A";
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE033_GetPreInvoiceAir");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNSE033_GetPreInvoiceSea/{preInvoiceNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE033_GetPreInvoiceSea(string preInvoiceNo, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            preInvoiceNo = ismoUtil.escapeCharacter(preInvoiceNo);

            try
            {
                OracleCommand cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNSE033_GET_PINVOICE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PRE_INVOICE", OracleDbType.Varchar2).Value = preInvoiceNo.ToUpper() == "NULL" ? (object)DBNull.Value : preInvoiceNo;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam["BRANCH_CODE"];
                cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = "S";
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE033_GetPreInvoiceSea");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }


        [HttpPost, Route("SNSE033_GetDataAir")]
        [Authorize]
        public IHttpActionResult SNSE033_GetDataAir([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNSE033_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam["BRANCH_CODE"];
                cmd.Parameters.Add("P_PINVOICE_NO", OracleDbType.Varchar2).Value = pParam["PINVOICE_NO"];
                cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = "A";
                cmd.Parameters.Add("C_RETURN1", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN2", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE033_GetDataAir");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNSE033_GetDataSea")]
        [Authorize]
        public IHttpActionResult SNSE033_GetDataSea([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNSE033_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam["BRANCH_CODE"];
                cmd.Parameters.Add("P_PINVOICE_NO", OracleDbType.Varchar2).Value = pParam["PINVOICE_NO"];
                cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = "S";
                cmd.Parameters.Add("C_RETURN1", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN2", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE033_GetDataSea");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }


        [HttpPost, Route("SNSE033_ConfirmAir")]
        [Authorize]
        [DateControlGuard, RunBatchGuard]
        public IHttpActionResult SNSE033_ConfirmAir([FromBody] dynamic[] dataList)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            string gInvoiceNo = "";
            string gTracking = "";
            string gDocType = "";
            try
            {
                cmd.CommandText = "SP_NP_SNSE033_GEN_INVOICE_NO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = dataList[0].SENT_BY;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                if (ds.Tables.Count > 0)
                {
                    gInvoiceNo = ds.Tables[0].Rows[0]["INVOICE_NO"].ToString();
                    gDocType = gInvoiceNo.Substring(0, 2);
                }
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE033_ConfirmAir");
                return Ok("Error : " + ex.Message);
            }

            try
            {
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_UPD_STAT_ISSUE_INV";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = dataList[0].CUSTOMER_CODE;
                cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = dataList[0].SENT_BY;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_TYPE", OracleDbType.Varchar2).Value = "START";
                oOracledb.SqlQueryReturnValue(cmd);
                foreach (var data in dataList)
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE033_CONFIRM_AIR";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = data.BRANCH_CODE;
                    cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = data.SENT_BY;

                    cmd.Parameters.Add("P_PINVOICE_NO", OracleDbType.Varchar2).Value = data.PINVOICE_NO;
                    cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = gInvoiceNo;
                    cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = data.CASE_NO;
                    cmd.Parameters.Add("P_PACK_DATE", OracleDbType.Date).Value = data.PACK_DATE;
                    cmd.Parameters.Add("P_CONTAINER_NO", OracleDbType.Varchar2).Value = data.CONTAINER_NO;
                    cmd.Parameters.Add("P_AWB_NO", OracleDbType.Varchar2).Value = dataList[0].AWB_NO;

                    cmd.Parameters.Add("P_AWB_DATE", OracleDbType.Date).Value = dataList[0].AWB_DATE;
                    cmd.Parameters.Add("P_FRC", OracleDbType.Varchar2).Value = dataList[0].FRC;
                    cmd.Parameters.Add("P_FWC", OracleDbType.Varchar2).Value = dataList[0].FWC;
                    cmd.Parameters.Add("P_INS", OracleDbType.Varchar2).Value = dataList[0].INS;
                    cmd.Parameters.Add("P_FLIGHT_NO", OracleDbType.Varchar2).Value = dataList[0].FLIGHT_NO;
                    cmd.Parameters.Add("P_TOTAL_CASE", OracleDbType.Varchar2).Value = dataList[0].TOTAL_CASE;

                    cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE033";
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                    gTracking = data.CASE_NO;
                    ds = oOracledb.SqlQueryDS_CMD(cmd);

                    if (ds.Tables.Count > 0)
                    {
                        if (ds.Tables[0].Rows[0]["STATUS"].ToString() != "DONE")
                        {
                            string errString = "Error : " + ds.Tables[0].Rows[0]["STATUS"].ToString();
                            return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, errString));
                        }
                    }

                };

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", "Case no : " + gTracking + " - " + ex.Message, "SNSE033_ConfirmAir");
                return Ok("Error : " + ex.Message);
            }


            try
            {
                cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNSE033_UPDATE_TRANS_H";

                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = dataList[0].CUSTOMER_CODE;
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = gInvoiceNo;
                cmd.Parameters.Add("P_DOC_TYPE", OracleDbType.Varchar2).Value = gDocType;

                cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE033";
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                if (ds.Tables.Count > 0)
                {
                    if (ds.Tables[0].Rows[0]["STATUS"].ToString() != "DONE")
                    {
                        string errString = "Error : " + ds.Tables[0].Rows[0]["STATUS"].ToString();
                        return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, errString));
                    }
                }
            }
            catch (Exception ex)
            {
                gInvoiceNo = "";
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE033_ConfirmAir");
                return Ok("Error : " + ex.Message);
            }
            cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_UPD_STAT_ISSUE_INV";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = dataList[0].CUSTOMER_CODE;
            cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = dataList[0].SENT_BY;
            cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
            cmd.Parameters.Add("P_TYPE", OracleDbType.Varchar2).Value = "END";
            oOracledb.SqlQueryReturnValue(cmd);
            return Ok("DONE-" + gInvoiceNo);
        }

        [HttpPost, Route("SNSE033_ConfirmSea")]
        [Authorize]
        [DateControlGuard, RunBatchGuard]
        public IHttpActionResult SNSE033_ConfirmSea([FromBody] dynamic[] dataList)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            string gInvoiceNo = "";
            string gTracking = "";
            string gDocType = "";
            try
            {
                cmd.CommandText = "SP_NP_SNSE033_GEN_INVOICE_NO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = dataList[0].SENT_BY;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                if (ds.Tables.Count > 0)
                {
                    gInvoiceNo = ds.Tables[0].Rows[0]["INVOICE_NO"].ToString();
                    gDocType = gInvoiceNo.Substring(0, 2);
                }
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE033_ConfirmSea");
                return Ok("Error : " + ex.Message);
            }

            try
            {
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_UPD_STAT_ISSUE_INV";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = dataList[0].CUSTOMER_CODE;
                cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = dataList[0].SENT_BY;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_TYPE", OracleDbType.Varchar2).Value = "START";
                oOracledb.SqlQueryReturnValue(cmd);
                foreach (var data in dataList)
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE033_CONFIRM_SEA";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = data.BRANCH_CODE;
                    cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = data.SENT_BY;

                    cmd.Parameters.Add("P_PINVOICE_NO", OracleDbType.Varchar2).Value = data.PINVOICE_NO;
                    cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = gInvoiceNo;
                    cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = data.CASE_NO;
                    cmd.Parameters.Add("P_PACK_DATE", OracleDbType.Date).Value = data.PACK_DATE;
                    cmd.Parameters.Add("P_CONTAINER_NO", OracleDbType.Varchar2).Value = data.CONTAINER_NO;
                    cmd.Parameters.Add("P_BL_NO", OracleDbType.Varchar2).Value = dataList[0].BL_NO;

                    cmd.Parameters.Add("P_BL_DATE", OracleDbType.Date).Value = dataList[0].BL_DATE;
                    cmd.Parameters.Add("P_FRC", OracleDbType.Varchar2).Value = dataList[0].FRC;
                    cmd.Parameters.Add("P_FWC", OracleDbType.Varchar2).Value = dataList[0].FWC;
                    cmd.Parameters.Add("P_INS", OracleDbType.Varchar2).Value = dataList[0].INS;
                    cmd.Parameters.Add("P_VESSEL", OracleDbType.Varchar2).Value = dataList[0].VESSEL;
                    cmd.Parameters.Add("P_TOTAL_CASE", OracleDbType.Varchar2).Value = dataList[0].TOTAL_CASE;

                    cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE033";
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                    gTracking = data.CASE_NO;
                    ds = oOracledb.SqlQueryDS_CMD(cmd);

                    if (ds.Tables.Count > 0)
                    {
                        if (ds.Tables[0].Rows[0]["STATUS"].ToString() != "DONE")
                        {
                            string errString = "Error : " + ds.Tables[0].Rows[0]["STATUS"].ToString();
                            return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, errString));
                        }
                    }

                };

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", "Case no : " + gTracking + " - " + ex.Message, "SNSE033_ConfirmSea");
                return Ok("Error : " + ex.Message);
            }


            try
            {
                cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNSE033_UPDATE_TRANS_H";

                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = dataList[0].CUSTOMER_CODE;
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = gInvoiceNo;
                cmd.Parameters.Add("P_DOC_TYPE", OracleDbType.Varchar2).Value = gDocType;

                cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE033";
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                if (ds.Tables.Count > 0)
                {
                    if (ds.Tables[0].Rows[0]["STATUS"].ToString() != "DONE")
                    {
                        string errString = "Error : " + ds.Tables[0].Rows[0]["STATUS"].ToString();
                        return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, errString));
                    }
                }
            }
            catch (Exception ex)
            {
                gInvoiceNo = "";
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE033_ConfirmSea");
                return Ok("Error : " + ex.Message);
            }
            cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_UPD_STAT_ISSUE_INV";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = dataList[0].CUSTOMER_CODE;
            cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = dataList[0].SENT_BY;
            cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
            cmd.Parameters.Add("P_TYPE", OracleDbType.Varchar2).Value = "END";
            oOracledb.SqlQueryReturnValue(cmd);
            return Ok("DONE-" + gInvoiceNo);
        }


        #endregion

        #region SNSE053

        [HttpGet, Route("SNSE053_GetCustomerCode/{cusID}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE053_GetCustomerCode(string cusID, int noofshowrec)
        {
            DataSet ds = new DataSet();

            cusID = ismoUtil.escapeCharacter(cusID);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE053_CUSTOMER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = cusID.ToUpper() == "NULL" ? (object)DBNull.Value : cusID;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE053_GetCustomerCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpPost, Route("SNSE053_GetData")]
        [Authorize]
        public IHttpActionResult SNSE053_GetData([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE053_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam.BRANCH_CODE;
                cmd.Parameters.Add("C_RETURN1", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN2", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE053_GetData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }


        [HttpPost, Route("SNSE053_GetDataModify")]
        [Authorize]
        public IHttpActionResult SNSE053_GetDataModify([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE053_GET_DATA_MODIFY";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam.BRANCH_CODE;
                cmd.Parameters.Add("P_PINVOICE_NO", OracleDbType.Varchar2).Value = pParam.PINVOICE_NO;
                cmd.Parameters.Add("C_RETURN1", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN2", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE053_GetDataModify");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }



        [HttpPost, Route("SNSE053_ConfirmPre")]
        [Authorize]
        [DateControlGuard, RunBatchGuard]
        public IHttpActionResult SNSE053_ConfirmPre([FromBody] dynamic[] dataList)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            string gPreInvoice = "";
            try
            {
                cmd.CommandText = "SP_NP_SNSE053_GEN_DOC_NO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                if (ds.Tables.Count > 0)
                {
                    gPreInvoice = ds.Tables[0].Rows[0]["PRE_INV_NO"].ToString();

                }
            }
            catch (Exception ex)
            {
                gPreInvoice = "";
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE053_ConfirmPre");
                return Ok("Error : " + ex.Message);
            }

            try
            {
                foreach (var data in dataList)
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE053_CONFIRM";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_PINVOICE_NO", OracleDbType.Varchar2).Value = gPreInvoice;
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = data.BRANCH_CODE;
                    cmd.Parameters.Add("P_PACK_DATE", OracleDbType.Date).Value = data.PACK_DATE;
                    cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = data.CASE_NO;

                    cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE032";
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                    ds = oOracledb.SqlQueryDS_CMD(cmd);

                    if (ds.Tables.Count > 0)
                    {
                        if (ds.Tables[0].Rows[0]["STATUS"].ToString() != "DONE")
                        {
                            string errString = "Error : " + ds.Tables[0].Rows[0]["STATUS"].ToString();
                            return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, errString));
                        }
                    }

                };

                return Ok("DONE-" + gPreInvoice);
            }
            catch (Exception ex)
            {
                return Ok("Error : " + ex.Message);
            }
        }


        [HttpPost, Route("SNSE053_ConfirmDelete")]
        [Authorize]
        [RunBatchGuard]
        public IHttpActionResult SNSE053_ConfirmDelete([FromBody] dynamic[] dataList)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();

            try
            {
                foreach (var data in dataList)
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE032_DELETE";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);

                    // cmd.Parameters.Add("P_PINVOICE_NO", OracleDbType.Varchar2).Value = data.PINVOICE_NO;
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = data.BRANCH_CODE;
                    cmd.Parameters.Add("P_PACK_DATE", OracleDbType.Date).Value = data.PACK_DATE;
                    cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = data.SENT_BY;
                    cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = data.CASE_NO;

                    cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE053";
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                    ds = oOracledb.SqlQueryDS_CMD(cmd);

                    if (ds.Tables.Count > 0)
                    {
                        if (ds.Tables[0].Rows[0]["STATUS"].ToString() != "DONE")
                        {
                            string errString = "Error : " + ds.Tables[0].Rows[0]["STATUS"].ToString();
                            return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, errString));
                        }
                    }

                };

                return Ok("DONE");
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE053_ConfirmDelete");
                return Ok("Error : " + ex.Message);
            }
        }

        [HttpPost, Route("SNSE053_GetPreInvoiceSea/{preInvoiceNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE053_GetPreInvoiceSea(string preInvoiceNo, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            preInvoiceNo = ismoUtil.escapeCharacter(preInvoiceNo);

            try
            {
                OracleCommand cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNSE032_GET_PINVOICE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PRE_INVOICE", OracleDbType.Varchar2).Value = preInvoiceNo.ToUpper() == "NULL" ? (object)DBNull.Value : preInvoiceNo;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam["BRANCH_CODE"];
                cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = "S";
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE053_GetPreInvoiceSea");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE053_ModifyPre")]
        [Authorize]
        public IHttpActionResult SNSE053_ModifyPre([FromBody] dynamic[] dataList)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            String pPInvoiceDate = "";
            try
            {
                pPInvoiceDate = DateTime.Now.ToString("MM-dd-yyyy HH:mm:ss");
                foreach (var data in dataList)
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE053_MODIFY";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = data.BRANCH_CODE;
                    cmd.Parameters.Add("P_PACK_DATE", OracleDbType.Date).Value = data.PACK_DATE;
                    cmd.Parameters.Add("P_CASE_NO", OracleDbType.Varchar2).Value = data.CASE_NO;
                    cmd.Parameters.Add("P_PINVOICE_NO", OracleDbType.Varchar2).Value = data.PINVOICE_NO;
                    cmd.Parameters.Add("P_PINVOICE_DATE", OracleDbType.Varchar2).Value = pPInvoiceDate;
                    cmd.Parameters.Add("P_CHK_BOX", OracleDbType.Varchar2).Value = data.CHK_BOX;
                    cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE053";
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                    ds = oOracledb.SqlQueryDS_CMD(cmd);
                    if (ds.Tables.Count > 0)
                    {
                        if (ds.Tables[0].Rows[0]["STATUS"].ToString() != "DONE")
                        {
                            string errString = "Error : " + ds.Tables[0].Rows[0]["STATUS"].ToString();
                            return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, errString));
                        }
                    }
                };
                return Ok("DONE");
            }
            catch (Exception ex)
            {
                return Ok("Error : " + ex.Message);
            }
        }
        #endregion

        #region SNSE038


        [HttpGet, Route("SNSE038_GetCustomerCode/{cusID}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE038_GetCustomerCode(string cusID, int noofshowrec)
        {
            DataSet ds = new DataSet();

            cusID = ismoUtil.escapeCharacter(cusID);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE038_CUSTOMER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = cusID.ToUpper() == "NULL" ? (object)DBNull.Value : cusID;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE038_GetCustomerCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE038_GetOrderNo/{orderNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE038_GetOrderNo(string orderNo, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            orderNo = ismoUtil.escapeCharacter(orderNo);
            var pData = pParam["DATA"];

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE038_ORDER_NO";
                cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = orderNo.ToUpper() == "NULL" ? (object)DBNull.Value : orderNo;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE038_GetOrderNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE038_GetData")]
        [Authorize]
        public IHttpActionResult SNSE038_GetData([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE038_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pParam.ORDER_NO;
                cmd.Parameters.Add("P_ORDER_NO_TO", OracleDbType.Varchar2).Value = pParam.ORDER_NO_TO;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE038_GetData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNSE038_GetPickLotSeq")]
        [Authorize]
        public IHttpActionResult SNSE038_GetPickLotSeq()
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE038_GET_PICK_LOT_SEQ";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE038_GetPickLotSeq");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpPost, Route("SNSE038_Process")]
        [Authorize]
        public IHttpActionResult SNSE038_Process([FromBody] dynamic[] dataList)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            string gPickLotSeq = "";

            try
            {
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE038_DELETE_WORKFILE";
                cmd.Parameters.Add("P_IP_ADDRESS", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                ds = oOracledb.SqlQueryDS_CMD(cmd);
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE038_Process");
                return Ok("Error : " + ex.Message);
            }

            try
            {
                foreach (var data in dataList)
                {

                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE038_PROCESS_WORKFILE";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = data.ORDER_NO;
                    cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Varchar2).Value = data.ITEM_NO;
                    cmd.Parameters.Add("P_SUB_ITEM_NO", OracleDbType.Varchar2).Value = data.SUB_ITEM_NO;
                    cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = data.ACCEPT_PART_NO;
                    cmd.Parameters.Add("P_INSTRUCTION_NO", OracleDbType.Varchar2).Value = data.INSTRUCTION_NO;
                    cmd.Parameters.Add("P_PICK_LOT_DATE", OracleDbType.Date).Value = dataList[0].PICK_LOT_DATE;
                    // cmd.Parameters.Add("P_PICK_LOT_SEQ", OracleDbType.Varchar2).Value = gPickLotSeq;
                    cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE038";
                    cmd.Parameters.Add("P_IP_ADDRESS", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                    ds = oOracledb.SqlQueryDS_CMD(cmd);

                    if (ds.Tables.Count > 0)
                    {
                        if (ds.Tables[0].Rows[0]["STATUS"].ToString() != "DONE")
                        {
                            string errString = "Error : " + ds.Tables[0].Rows[0]["STATUS"].ToString();
                            return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, errString));
                        }
                    }

                };

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE038_Process");
                return Ok("Error : " + ex.Message);
            }


            try
            {
                cmd = new OracleCommand();

                cmd.CommandText = "SP_NP_SNSE038_PROCESS_SUBMIT";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = dataList[0].CUSTOMER_CODE;
                cmd.Parameters.Add("P_PICK_LOT_DATE", OracleDbType.Date).Value = dataList[0].PICK_LOT_DATE;
                // cmd.Parameters.Add("P_PICK_LOT_SEQ", OracleDbType.Varchar2).Value = gPickLotSeq;
                cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE038";
                cmd.Parameters.Add("P_IP_ADDRESS", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                if (ds.Tables.Count > 0)
                {
                    if (ds.Tables[0].Rows[0]["STATUS"].ToString() != "DONE")
                    {
                        string errString = "Error : " + ds.Tables[0].Rows[0]["STATUS"].ToString();
                        return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, errString));
                    }
                    else
                    {
                        gPickLotSeq = ds.Tables[0].Rows[0]["PICK_LOT_SEQ"].ToString();
                    }
                }
            }
            catch (Exception ex)
            {

                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE038_Process");
                return Ok("Error : " + ex.Message);
            }
            return Ok("DONE-" + gPickLotSeq);
        }


        #endregion

        #region SNSE038_W
        [HttpGet, Route("SNSE038_W_GetCustomerCode/{cusID}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE038_W_GetCustomerCode(string cusID, int noofshowrec)
        {
            DataSet ds = new DataSet();
            cusID = ismoUtil.escapeCharacter(cusID);
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE038_W_CUSTOMER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = cusID.ToUpper() == "NULL" ? (object)DBNull.Value : cusID;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE038_W_GetCustomerCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNSE038_W_GetOrderNo/{orderNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE038_W_GetOrderNo(string orderNo, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            orderNo = ismoUtil.escapeCharacter(orderNo);
            var pData = pParam["DATA"];
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE038_W_ORDER_NO";
                cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = orderNo.ToUpper() == "NULL" ? (object)DBNull.Value : orderNo;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE038_W_GetOrderNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNSE038_W_GetData")]
        [Authorize]
        public IHttpActionResult SNSE038_W_GetData([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE038_W_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pParam.ORDER_NO;
                cmd.Parameters.Add("P_ORDER_NO_TO", OracleDbType.Varchar2).Value = pParam.ORDER_NO_TO;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE038_W_GetData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpGet, Route("SNSE038_W_GetPickLotSeq")]
        [Authorize]
        public IHttpActionResult SNSE038_W_GetPickLotSeq()
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE038_W_GET_PICK_LOT_SEQ";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE038_W_GetPickLotSeq");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNSE038_W_Process")]
        [Authorize]
        public IHttpActionResult SNSE038_W_Process([FromBody] dynamic[] dataList)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            string gPickLotSeq = "";
            try
            {
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE038_W_DELETE_WORKFILE";
                cmd.Parameters.Add("P_IP_ADDRESS", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                ds = oOracledb.SqlQueryDS_CMD(cmd);
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE038_W_Process");
                return Ok("Error : " + ex.Message);
            }
            try
            {
                foreach (var data in dataList)
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE038_W_PROCESS_WORKFILE";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = data.ORDER_NO;
                    cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Varchar2).Value = data.ITEM_NO;
                    cmd.Parameters.Add("P_SUB_ITEM_NO", OracleDbType.Varchar2).Value = data.SUB_ITEM_NO;
                    cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = data.ACCEPT_PART_NO;
                    cmd.Parameters.Add("P_INSTRUCTION_NO", OracleDbType.Varchar2).Value = data.INSTRUCTION_NO;
                    cmd.Parameters.Add("P_PICK_LOT_DATE", OracleDbType.Date).Value = dataList[0].PICK_LOT_DATE;
                    cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE038_W";
                    cmd.Parameters.Add("P_IP_ADDRESS", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                    ds = oOracledb.SqlQueryDS_CMD(cmd);
                    if (ds.Tables.Count > 0)
                    {
                        if (ds.Tables[0].Rows[0]["STATUS"].ToString() != "DONE")
                        {
                            string errString = "Error : " + ds.Tables[0].Rows[0]["STATUS"].ToString();
                            return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, errString));
                        }
                    }
                };
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE038_W_Process");
                return Ok("Error : " + ex.Message);
            }
            try
            {
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE038_W_PROCESS_SUBMIT";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = dataList[0].CUSTOMER_CODE;
                cmd.Parameters.Add("P_PICK_LOT_DATE", OracleDbType.Date).Value = dataList[0].PICK_LOT_DATE;
                cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE038_W";
                cmd.Parameters.Add("P_IP_ADDRESS", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                if (ds.Tables.Count > 0)
                {
                    if (ds.Tables[0].Rows[0]["STATUS"].ToString() != "DONE")
                    {
                        string errString = "Error : " + ds.Tables[0].Rows[0]["STATUS"].ToString();
                        return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, errString));
                    }
                    else
                    {
                        gPickLotSeq = ds.Tables[0].Rows[0]["PICK_LOT_SEQ"].ToString();
                    }
                }
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE038_W_Process");
                return Ok("Error : " + ex.Message);
            }
            return Ok("DONE-" + gPickLotSeq);
        }
        #endregion
        #region SNSE011_N

        [HttpPost, Route("SNSE011_N_GetOrderList")]
        [Authorize]
        public IHttpActionResult SNSE011_N_GetOrderList([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE011_N_GET_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam.BRANCH_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pParam.ORDER_NO;
                cmd.Parameters.Add("P_ORDER_DATE", OracleDbType.Date).Value = pParam.ORDER_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(pParam.ORDER_DATE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE011_N_GetOrderList");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }



        [HttpPost, Route("SNSE011_N_GetDetail")]
        [Authorize]
        public IHttpActionResult SNSE011_N_GetDetail([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE011_N_GET_DETAIL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pParam.ORDER_NO;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE011_N_GetDetail");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNSE011_N_Process")]
        [Authorize]
        public IHttpActionResult SNSE011_N_Process([FromBody] dynamic[] pPartList)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();

            try
            {
                foreach (var part in pPartList)
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE011_N_PROCESS";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pPartList[0].HEADER_CUSTOMER_CODE;
                    cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pPartList[0].HEADER_ORDER_NO;
                    cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Decimal).Value = part.ITEM_NO;
                    cmd.Parameters.Add("P_SUB_ITEM_NO", OracleDbType.Varchar2).Value = part.SUB_ITEM_NO;
                    cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = part.ORDER_PART_NO;
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                    ds = oOracledb.SqlQueryDS_CMD(cmd);
                };

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE011_N_Process");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }
        #endregion

        #region SNSE068

        [HttpGet, Route("SNSE068_GetCustSearch/{cusID}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE068_GetCustSearch(string cusID, int noofshowrec)
        {
            DataSet ds = new DataSet();

            cusID = ismoUtil.escapeCharacter(cusID);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE068_GET_CUST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = cusID.ToUpper() == "NULL" ? (object)DBNull.Value : cusID;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE068_GET_CUST");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpPost, Route("SNSE068_GetSearchBranch/{branchCode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE068_GetSearchBranch(string branchCode, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            var pData = pParam["DATA"];
            branchCode = ismoUtil.escapeCharacter(branchCode);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE068_GET_BRANCH";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = branchCode.ToUpper() == "NULL" ? (object)DBNull.Value : branchCode;
                cmd.Parameters.Add("P_SHOW_REC", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE068_GET_BRANCH");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE068_GetSearchOrder/{OrderNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE068_GetSearchOrder(string OrderNo, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            var pData = pParam["DATA"];
            OrderNo = ismoUtil.escapeCharacter(OrderNo);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE068_GET_ORDER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pData["BRANCH_CODE"];
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = OrderNo.ToUpper() == "NULL" ? (object)DBNull.Value : OrderNo;
                cmd.Parameters.Add("P_SHOW_REC", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE068_GET_ORDER");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE068_GetOrderList")]
        [Authorize]
        public IHttpActionResult SNSE068_GetOrderList([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE068_ORDER_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam.BRANCH_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pParam.ORDER_NO;
                cmd.Parameters.Add("P_ORDER_DATE", OracleDbType.Date).Value = pParam.ORDER_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(pParam.ORDER_DATE);
                cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE068_ORDER_LIST");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE068_GetOrderDetail")]
        [Authorize]
        public IHttpActionResult SNSE068_GetOrderDetail([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE068_GET_DETAIL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pParam.ORDER_NO;
                cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("c_Return2", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE068_GET_DETAIL");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNSE068_GetCustomer/{cusID}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE068_GetCustomer(string cusID, int noofshowrec)
        {
            DataSet ds = new DataSet();

            cusID = ismoUtil.escapeCharacter(cusID);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE068_CUSTOMER";
                cmd.Parameters.Add("pCOMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("pCUSTOMER_CODE", OracleDbType.Varchar2).Value = cusID.ToUpper() == "NULL" ? (object)DBNull.Value : cusID;
                cmd.Parameters.Add("pSHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE068_CUSTOMER");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNSE068_GetParts/{partNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE068_GetParts(string partNo, int noofshowrec)
        {
            DataSet ds = new DataSet();

            partNo = ismoUtil.escapeCharacter(partNo);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE068_GETPARTS";
                cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("p_PART_NO", OracleDbType.Varchar2).Value = (partNo != null && partNo == "null" ? "" : partNo);
                cmd.Parameters.Add("p_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE068_GETPARTS");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE068_GetCustomerBranch/{branchCode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE068_GetCustomerBranch(string branchCode, int noofshowrec, [FromBody] dynamic pCustomer)
        {
            DataSet ds = new DataSet();

            branchCode = ismoUtil.escapeCharacter(branchCode);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE068_CUSTOMER_BRANCH";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pCustomer["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = branchCode.ToUpper() == "NULL" ? (object)DBNull.Value : branchCode;
                cmd.Parameters.Add("P_SHOW_REC", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE068_CUSTOMER_BRANCH");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE068_CheckOrder")]
        [Authorize]
        public IHttpActionResult SNSE068_CheckOrder([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE068_CHECK_ORDER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pData["ORDER_NO"];
                cmd.Parameters.Add("R_ERROR_MSG", OracleDbType.Varchar2, 255).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                if (cmd.Parameters["R_ERROR_MSG"].Value.ToString() != "Done")
                {
                    return Json(new
                    {
                        Result = "Fail",
                        Message = cmd.Parameters["R_ERROR_MSG"].Value.ToString()
                    });
                }

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE068_CHECK_ORDER");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

            return Ok();
        }

        [HttpGet, Route("SNSE068_SNSC003/{customerCode}/{orderDate}/{partNo}/{orderClass}/{vendorCode}")]
        [Authorize]
        public IHttpActionResult SNSE068_SNSC003(string customerCode, string orderDate, string partNo, string orderClass, string vendorCode)
        {
            decimal RETAIL_D
                , RETAIL_E
                , UNIT_D
                , UNIT_E
                , DISC_1
                , DISC_2
                , DISC_3
                , AVG_COST
                , STD_COST;
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SNSC003A";
                cmd.Parameters.Add("CUST_CODE", OracleDbType.Varchar2).Value = customerCode;
                cmd.Parameters.Add("ORDER_DATE", OracleDbType.Date).Value = Convert.ToDateTime(orderDate);
                cmd.Parameters.Add("PARTNO", OracleDbType.Varchar2).Value = partNo;
                cmd.Parameters.Add("OR_CLASS", OracleDbType.Varchar2).Value = orderClass;
                cmd.Parameters.Add("VD_CODE", OracleDbType.Varchar2).Value = vendorCode;
                cmd.Parameters.Add("OVERSEAFLAG", OracleDbType.Varchar2, 1).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("RETAIL_D", OracleDbType.Decimal).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("RETAIL_E", OracleDbType.Decimal).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("UNIT_D", OracleDbType.Decimal).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("UNIT_E", OracleDbType.Decimal).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("DISC_1", OracleDbType.Decimal).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("DISC_2", OracleDbType.Decimal).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("DISC_3", OracleDbType.Decimal).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("DISC_GROUP", OracleDbType.Varchar2, 3).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("DISC_CODE", OracleDbType.Varchar2, 2).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("AVG_COST", OracleDbType.Decimal).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("STD_COST", OracleDbType.Decimal).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                return Json(new
                {
                    OVERSEAFLAG = cmd.Parameters["OVERSEAFLAG"].Value.ToString(),
                    RETAIL_D = Decimal.TryParse(cmd.Parameters["RETAIL_D"].Value.ToString(), out RETAIL_D) ? RETAIL_D : 0,
                    RETAIL_E = Decimal.TryParse(cmd.Parameters["RETAIL_E"].Value.ToString(), out RETAIL_E) ? RETAIL_E : 0,
                    UNIT_D = Decimal.TryParse(cmd.Parameters["UNIT_D"].Value.ToString(), out UNIT_D) ? UNIT_D : 0,
                    UNIT_E = Decimal.TryParse(cmd.Parameters["UNIT_E"].Value.ToString(), out UNIT_E) ? UNIT_E : 0,
                    DISC_1 = Decimal.TryParse(cmd.Parameters["DISC_1"].Value.ToString(), out DISC_1) ? DISC_1 : 0,
                    DISC_2 = Decimal.TryParse(cmd.Parameters["DISC_2"].Value.ToString(), out DISC_2) ? DISC_2 : 0,
                    DISC_3 = Decimal.TryParse(cmd.Parameters["DISC_3"].Value.ToString(), out DISC_3) ? DISC_3 : 0,
                    DISC_GROUP = cmd.Parameters["DISC_GROUP"].Value.ToString(),
                    DISC_CODE = cmd.Parameters["DISC_CODE"].Value.ToString(),
                    AVG_COST = Decimal.TryParse(cmd.Parameters["AVG_COST"].Value.ToString(), out AVG_COST) ? AVG_COST : 0,
                    STD_COST = Decimal.TryParse(cmd.Parameters["STD_COST"].Value.ToString(), out STD_COST) ? STD_COST : 0
                });

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSC003A");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }


        [HttpPost, Route("SNSE068_GetCampaignSpecialDiscount")]
        [Authorize]
        public IHttpActionResult SNSE068_GetCampaignSpecialDiscount([FromBody] dynamic pParams)
        {
            decimal DISCOUNT_RATE1
                , DISCOUNT_RATE2
                , DISCOUNT_RATE3
                , UNIT_PRICE_B
                , UNIT_PRICE;

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE068_GET_CMP_DIS";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pParams.ORDER_NO;
                cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = pParams.ORDER_CLASS;
                cmd.Parameters.Add("P_ORDER_DATE", OracleDbType.Date).Value = Convert.ToDateTime(pParams.ORDER_DATE);
                cmd.Parameters.Add("P_SALES_CHANNEL", OracleDbType.Varchar2).Value = pParams.SALES_CHANNEL;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParams.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParams.BRANCH_CODE;
                cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = pParams.ORDER_PART_NO;
                cmd.Parameters.Add("P_ORDER_QTY", OracleDbType.Decimal).Value = pParams.ORDER_QTY;
                cmd.Parameters.Add("P_PART_LIST", OracleDbType.Varchar2).Value = pParams.PART_LIST;
                cmd.Parameters.Add("R_DISCOUNT_RATE1", OracleDbType.Decimal).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_DISCOUNT_RATE2", OracleDbType.Decimal).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_DISCOUNT_RATE3", OracleDbType.Decimal).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_UNIT_PRICE_B", OracleDbType.Decimal).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_UNIT_PRICE", OracleDbType.Decimal).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_DISCOUNT_CODE", OracleDbType.Varchar2, 10).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_CAMPAIGN_ID", OracleDbType.Varchar2, 10).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_CAMPAIGN_TYPE", OracleDbType.Varchar2, 1).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_CAMPAIGN_REF", OracleDbType.Varchar2, 10).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_DSP_CAMPAIGN_NAME", OracleDbType.Varchar2, 100).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                return Json(new
                {
                    DISCOUNT_RATE1 = Decimal.TryParse(cmd.Parameters["R_DISCOUNT_RATE1"].Value.ToString(), out DISCOUNT_RATE1) ? DISCOUNT_RATE1 : 0,
                    DISCOUNT_RATE2 = Decimal.TryParse(cmd.Parameters["R_DISCOUNT_RATE2"].Value.ToString(), out DISCOUNT_RATE2) ? DISCOUNT_RATE2 : 0,
                    DISCOUNT_RATE3 = Decimal.TryParse(cmd.Parameters["R_DISCOUNT_RATE3"].Value.ToString(), out DISCOUNT_RATE3) ? DISCOUNT_RATE3 : 0,
                    UNIT_PRICE_B = Decimal.TryParse(cmd.Parameters["R_UNIT_PRICE_B"].Value.ToString(), out UNIT_PRICE_B) ? UNIT_PRICE_B : 0,
                    UNIT_PRICE = Decimal.TryParse(cmd.Parameters["R_UNIT_PRICE"].Value.ToString(), out UNIT_PRICE) ? UNIT_PRICE : 0,
                    DISCOUNT_CODE = cmd.Parameters["R_DISCOUNT_CODE"].Value.ToString(),
                    CAMPAIGN_ID = cmd.Parameters["R_CAMPAIGN_ID"].Value.ToString(),
                    CAMPAIGN_TYPE = cmd.Parameters["R_CAMPAIGN_TYPE"].Value.ToString(),
                    CAMPAIGN_REF = cmd.Parameters["R_CAMPAIGN_REF"].Value.ToString(),
                    DSP_CAMPAIGN_NAME = cmd.Parameters["R_DSP_CAMPAIGN_NAME"].Value.ToString()
                });

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE068_GET_CMP_DIS");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNSE068_SaveDocument")]
        [Authorize]
        public IHttpActionResult SNSE068_SaveDocument([FromBody] dynamic pData)
        {
            int iCount = 0;
            OracleCommand cmd = new OracleCommand();

            var partList = new List<dynamic>(pData.PART_LIST);

            if (partList.Count > 0)
            {
                //Run insert detail
                foreach (var detail in partList)
                {
                    iCount++;

                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE068_SAVE_DETAIL_TEMP";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pData.ORDER_NO;
                    cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Decimal).Value = iCount;
                    cmd.Parameters.Add("P_ORDER_QTY", OracleDbType.Decimal).Value = detail.ORDER_QTY;
                    cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = detail.ORDER_PART_NO;
                    cmd.Parameters.Add("P_OVERSEA_FLAG_C", OracleDbType.Varchar2).Value = detail.OVERSEA_FLAG_C;
                    cmd.Parameters.Add("P_RETAIL_PRICE_B", OracleDbType.Decimal).Value = detail.RETAIL_PRICE_B;
                    cmd.Parameters.Add("P_RETAIL_PRICE_C", OracleDbType.Decimal).Value = detail.RETAIL_PRICE_C;
                    cmd.Parameters.Add("P_UNIT_PRICE_B", OracleDbType.Decimal).Value = detail.UNIT_PRICE_B;
                    cmd.Parameters.Add("P_UNIT_PRICE_C", OracleDbType.Decimal).Value = detail.UNIT_PRICE_C;
                    cmd.Parameters.Add("P_UNIT_PRICE_EDP", OracleDbType.Decimal).Value = detail.UNIT_EDP;
                    cmd.Parameters.Add("P_DISCOUNT_RATE1", OracleDbType.Decimal).Value = detail.DISCOUNT_RATE1;
                    cmd.Parameters.Add("P_DISCOUNT_RATE2", OracleDbType.Decimal).Value = detail.DISCOUNT_RATE2;
                    cmd.Parameters.Add("P_DISCOUNT_RATE3", OracleDbType.Decimal).Value = detail.DISCOUNT_RATE3;
                    cmd.Parameters.Add("P_DISCOUNT_GROUP", OracleDbType.Varchar2).Value = detail.DISCOUNT_GROUP;
                    cmd.Parameters.Add("P_DISCOUNT_CODE", OracleDbType.Varchar2).Value = detail.DISCOUNT_CODE;
                    cmd.Parameters.Add("P_AVERAGE_COST", OracleDbType.Decimal).Value = detail.AVERAGE_COST;
                    cmd.Parameters.Add("P_STANDARD_COST", OracleDbType.Decimal).Value = detail.STANDARD_COST;
                    cmd.Parameters.Add("P_STATUS", OracleDbType.Varchar2).Value = detail.STATUS;
                    cmd.Parameters.Add("P_REASON_CODE", OracleDbType.Decimal).Value = detail.REASON_CODE;
                    cmd.Parameters.Add("P_CAMPAIGN_ID", OracleDbType.Varchar2).Value = detail.CAMPAIGN_ID;
                    cmd.Parameters.Add("P_CAMPAIGN_REF", OracleDbType.Varchar2).Value = detail.CAMPAIGN_REF;
                    cmd.Parameters.Add("P_CLEAR_DATA", OracleDbType.Varchar2).Value = iCount == 1 ? "Y" : "N";
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    oOracledb.SqlQueryReturnValue(cmd);

                }

                try
                {
                    //Insert transaction
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE068_SAVE_TRANSACTION";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_CUSTOMER_BRANCH", OracleDbType.Varchar2).Value = pData.BRANCH_CODE;
                    cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pData.ORDER_NO;
                    cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = pData.ORDER_CLASS;
                    cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pData.ORDER_TYPE;
                    cmd.Parameters.Add("P_ORDER_DATE", OracleDbType.Date).Value = Convert.ToDateTime(pData._ORDER_DATE);
                    cmd.Parameters.Add("P_DELIVERY_CODE", OracleDbType.Varchar2).Value = pData.DELIVERY_CODE;
                    cmd.Parameters.Add("P_ROUTE_CODE", OracleDbType.Varchar2).Value = pData.ROUTE_CODE;
                    cmd.Parameters.Add("P_TRANSPORT_CODE", OracleDbType.Varchar2).Value = pData.TRANSPORT_CODE;
                    cmd.Parameters.Add("P_DISCOUNT_GROUP", OracleDbType.Varchar2).Value = pData.DISCOUNT_GROUP;
                    cmd.Parameters.Add("P_BO_FLAG", OracleDbType.Varchar2).Value = pData.BO_FLAG;
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                    oOracledb.SqlQueryReturnValue(cmd);

                    return Ok();

                }
                catch (Exception ex)
                {
                    oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE068_SAVE_DETAIL_TEMP");
                    return ResponseMessage(Request.CreateResponse(HttpStatusCode.BadRequest, "Something went wrong."));
                }

            }

            return ResponseMessage(Request.CreateResponse(HttpStatusCode.BadRequest, "No row to process."));

        }
        #endregion

        #region SNSE073
        [HttpGet, Route("SNSE073_GetCustomerCode/{cusID}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE073_GetCustomerCode(string cusID, int noofshowrec)
        {
            DataSet ds = new DataSet();

            cusID = ismoUtil.escapeCharacter(cusID);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE073_GET_CUSTOMER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = cusID.ToUpper() == "NULL" ? (object)DBNull.Value : cusID;
                cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE073_GetCustomerCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE073_GetOrderNo/{orderNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE073_GetOrderNo(string orderNo, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            orderNo = ismoUtil.escapeCharacter(orderNo);
            var pData = pParam["DATA"];

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE073_GET_ORDER_NO";
                cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = orderNo.ToUpper() == "NULL" ? (object)DBNull.Value : orderNo;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE073_GetOrderNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE073_GetOrderPartNo/{orderPartNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE073_GetOrderPartNo(string orderPartNo, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            orderPartNo = ismoUtil.escapeCharacter(orderPartNo);
            var pData = pParam["DATA"];

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE073_ORDER_PART_NO";
                cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = orderPartNo.ToUpper() == "NULL" ? (object)DBNull.Value : orderPartNo;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pData["ORDER_NO"];
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE073_GetOrderPartNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE073_getData")]
        [Authorize]
        public IHttpActionResult SNSE073_getData([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE073_GET_DATA";
                cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pParam.ORDER_NO;

                cmd.Parameters.Add("P_VT_J", OracleDbType.Int16).Value = pParam.VT_J;
                cmd.Parameters.Add("P_VT_K", OracleDbType.Int16).Value = pParam.VT_K;
                cmd.Parameters.Add("P_VT_L", OracleDbType.Int16).Value = pParam.VT_L;
                cmd.Parameters.Add("P_VT_O", OracleDbType.Int16).Value = pParam.VT_O;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE073_getData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE073_Save")]
        [Authorize]
        [RunBatchGuard]
        public IHttpActionResult SNSE073_Save([FromBody] dynamic[] pDataList)
        {
            DataSet ds = new DataSet();

            try
            {
                foreach (var data in pDataList)
                {
                    OracleCommand cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE073_UPDATE";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = data.ORDER_NO;
                    cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = data.ORDER_PART_NO;
                    cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Varchar2).Value = data.ITEM_NO;
                    cmd.Parameters.Add("P_CHK", OracleDbType.Varchar2).Value = data.CHK;
                    cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE073";
                    cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                    ds = oOracledb.SqlQueryDS_CMD(cmd);

                }
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                return Ok("Error : " + ex.Message);
            }




        }
        #endregion

        #region SNSE026_M

        [HttpPost, Route("SNSE026M_Process")]
        [Authorize]
        //[IssueCardGuard]
        public IHttpActionResult SNSE026M_Process([FromBody] dynamic[] pDataList)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd;

            try
            {
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE026M_DEL_TEMP";
                cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                oOracledb.SqlQueryReturnValue(cmd);

                int i = 0;
                foreach (var data in pDataList)
                {
                    i = i + 1;
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE026M_INSERT_TEMP";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = data.ORDER_NO;
                    cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Varchar2).Value = data.ITEM_NO;
                    cmd.Parameters.Add("P_SUB_ITEM_NO", OracleDbType.Varchar2).Value = data.SUB_ITEM_NO;
                    cmd.Parameters.Add("P_OLD_SUB_ITEM_NO", OracleDbType.Varchar2).Value = data.OLD_SUB_ITEM_NO;
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = data.BRANCH_CODE;
                    cmd.Parameters.Add("P_OVERSEA_FLAG_C", OracleDbType.Varchar2).Value = data.OVERSEA_FLAG_C;
                    cmd.Parameters.Add("P_DELIVERY_CODE", OracleDbType.Varchar2).Value = data.DELIVERY_CODE;
                    cmd.Parameters.Add("P_ROUTE_CODE", OracleDbType.Varchar2).Value = data.ROUTE_CODE;
                    cmd.Parameters.Add("P_TRANSPORT_CODE", OracleDbType.Varchar2).Value = data.TRANSPORT_CODE;
                    cmd.Parameters.Add("P_ZONE_AREA", OracleDbType.Varchar2).Value = data.ZONE_AREA;
                    cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = data.ORDER_CLASS;
                    cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = data.ORDER_PART_NO;
                    cmd.Parameters.Add("P_ACCEPT_PART_NO", OracleDbType.Varchar2).Value = data.ACCEPT_PART_NO;
                    cmd.Parameters.Add("P_ORDER_QTY", OracleDbType.Varchar2).Value = data.ORDER_QTY;
                    cmd.Parameters.Add("P_ACCEPT_QTY", OracleDbType.Varchar2).Value = data.ACCEPT_QTY;
                    cmd.Parameters.Add("P_ORDER_DATE", OracleDbType.Date).Value = data.ORDER_DATE;
                    cmd.Parameters.Add("P_ALLOCATE_DATE", OracleDbType.Date).Value = data.ALLOCATE_DATE;
                    cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = data.INVOICE_NO;
                    cmd.Parameters.Add("P_INVOICE_DATE", OracleDbType.Varchar2).Value = data.INVOICE_DATE;
                    cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = data.SENT_BY;
                    cmd.Parameters.Add("P_PICK_LOT_DATE", OracleDbType.Date).Value = pDataList[0].PICK_LOT_DATE;
                    cmd.Parameters.Add("P_PICK_LOT_SEQ", OracleDbType.Varchar2).Value = pDataList[0].PICK_LOT_SEQ;
                    cmd.Parameters.Add("P_PICK_LIST_NO", OracleDbType.Varchar2).Value = pDataList[0].PICKING_LIST_NO;
                    cmd.Parameters.Add("P_CREATE_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE026_M";
                    cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_LAST_UPD_DATE", OracleDbType.Date).Value = data.LAST_UPD_DATE;
                    cmd.Parameters.Add("P_CHK_PROCESS", OracleDbType.Varchar2).Value = i;
                    cmd.Parameters.Add("P_IP_ADDRESS", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                    ds = oOracledb.SqlQueryDS_CMD(cmd);
                }

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE026M_START_PROCESS";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PICK_LOT_DATE", OracleDbType.Date).Value = pDataList[0].PICK_LOT_DATE;
                cmd.Parameters.Add("P_PICK_LOT_SEQ", OracleDbType.Varchar2).Value = pDataList[0].PICK_LOT_SEQ;
                cmd.Parameters.Add("P_PICK_LIST_NO", OracleDbType.Varchar2).Value = pDataList[0].PICKING_LIST_NO;
                cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(new { status = "DONE" });
            }
            catch (OracleException oex)
            {
                string err_massage = string.Empty;
                if (oex.Number == 20002)
                {
                    err_massage = "Someone is printing picking card, please try again later!";
                }
                else
                {
                    err_massage = "Found an error, please contract support team";
                }
                return Ok(new { status = "fail", massage = err_massage, err_code = oex.Number });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE026M_Process");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.ExpectationFailed, ex.Message));
            }
        }
        [HttpPost, Route("SNSE026M_getData")]
        [Authorize]
        public IHttpActionResult SNSE026M_getData([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE026M_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pParam.ORDER_NO;
                cmd.Parameters.Add("P_ACCEPT_PART_NO", OracleDbType.Varchar2).Value = pParam.ACCEPT_PART_NO;
                cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = pParam.ORDER_CLASS;
                cmd.Parameters.Add("P_FROM_DATE", OracleDbType.Date).Value = pParam.ALLOCATE_DATE_FROM == null ? (object)DBNull.Value : Convert.ToDateTime(pParam.ALLOCATE_DATE_FROM);
                cmd.Parameters.Add("P_TO_DATE", OracleDbType.Date).Value = pParam.ALLOCATE_DATE_TO == null ? (object)DBNull.Value : Convert.ToDateTime(pParam.ALLOCATE_DATE_TO);
                cmd.Parameters.Add("P_DG", OracleDbType.Varchar2).Value = pParam.DG;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE026M_getData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE026M_getAllocateFromDate/{fromDate}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE026M_getAllocateFromDate(string fromDate, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            fromDate = ismoUtil.escapeCharacter(fromDate);
            var pData = pParam["DATA"];

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE026M_ALLOCATE_DATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pData.ORDER_NO;
                cmd.Parameters.Add("P_ACCEPT_PART_NO", OracleDbType.Varchar2).Value = pData.ACCEPT_PART_NO;
                cmd.Parameters.Add("P_ALLOCATE_DATE", OracleDbType.Varchar2).Value = "FROM";
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE026M_getAllocateFromDate");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE026M_getAllocateToDate/{fromDate}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE026M_getAllocateToDate(string fromDate, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            fromDate = ismoUtil.escapeCharacter(fromDate);
            var pData = pParam["DATA"];

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE026M_ALLOCATE_DATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pData.ORDER_NO;
                cmd.Parameters.Add("P_ACCEPT_PART_NO", OracleDbType.Varchar2).Value = pData.ACCEPT_PART_NO;
                cmd.Parameters.Add("P_ALLOCATE_DATE", OracleDbType.Varchar2).Value = "TO";
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE026M_getAllocateToDate");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE026M_getCustomerCode/{cusID}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE026M_getCustomerCode(string cusID, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            cusID = ismoUtil.escapeCharacter(cusID);
            var pData = pParam["DATA"];

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE026M_CUSTOMER_CODE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = cusID.ToUpper() == "NULL" ? (object)DBNull.Value : cusID;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pData.ORDER_NO;
                cmd.Parameters.Add("P_ACCEPT_PART_NO", OracleDbType.Varchar2).Value = pData.ACCEPT_PART_NO;
                cmd.Parameters.Add("P_FROM_DATE", OracleDbType.Date).Value = pData.ALLOCATE_DATE_FROM == null ? (object)DBNull.Value : Convert.ToDateTime(pData.ALLOCATE_DATE_FROM);
                cmd.Parameters.Add("P_TO_DATE", OracleDbType.Date).Value = pData.ALLOCATE_DATE_TO == null ? (object)DBNull.Value : Convert.ToDateTime(pData.ALLOCATE_DATE_TO);
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE026M_getCustomerCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }
        [HttpPost, Route("SNSE026M_GetOrderNo/{orderNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE026M_GetOrderNo(string orderNo, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            orderNo = ismoUtil.escapeCharacter(orderNo);
            var pData = pParam["DATA"];

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE026M_ORDER_NO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = orderNo.ToUpper() == "NULL" ? (object)DBNull.Value : orderNo;
                cmd.Parameters.Add("P_ACCEPT_PART_NO", OracleDbType.Varchar2).Value = pData.ACCEPT_PART_NO;
                cmd.Parameters.Add("P_FROM_DATE", OracleDbType.Date).Value = pData.ALLOCATE_DATE_FROM == null ? (object)DBNull.Value : Convert.ToDateTime(pData.ALLOCATE_DATE_FROM);
                cmd.Parameters.Add("P_TO_DATE", OracleDbType.Date).Value = pData.ALLOCATE_DATE_TO == null ? (object)DBNull.Value : Convert.ToDateTime(pData.ALLOCATE_DATE_TO);
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE026M_GetOrderNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpPost, Route("SNSE026M_FirstLast_Date")]
        [Authorize]
        public IHttpActionResult SNSE026M_FirstLast_Date([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE026M_FIRST_LAST_DATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ALLOCATE_DATE", OracleDbType.Varchar2).Value = pParam.ALLOCATE_DATE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE026M_FirstLast_Date");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE026M_GetAcceptPartNo/{acceptPartNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE026M_GetAcceptPartNo(string acceptPartNo, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            acceptPartNo = ismoUtil.escapeCharacter(acceptPartNo);
            var pData = pParam["DATA"];

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE026M_ACCEPT_PART_NO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pData.ORDER_NO;
                cmd.Parameters.Add("P_ACCEPT_PART_NO", OracleDbType.Varchar2).Value = acceptPartNo.ToUpper() == "NULL" ? (object)DBNull.Value : acceptPartNo;
                cmd.Parameters.Add("P_FROM_DATE", OracleDbType.Date).Value = pData.ALLOCATE_DATE_FROM == null ? (object)DBNull.Value : Convert.ToDateTime(pData.ALLOCATE_DATE_FROM);
                cmd.Parameters.Add("P_TO_DATE", OracleDbType.Date).Value = pData.ALLOCATE_DATE_TO == null ? (object)DBNull.Value : Convert.ToDateTime(pData.ALLOCATE_DATE_TO);
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE026M_GetAcceptPartNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNSE026M_GetPickLotSeq")]
        [Authorize]
        public IHttpActionResult SNSE026M_GetPickLotSeq()
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE026M_GET_PICKLOT_SEQ";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE026M_GetPickLotSeq");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        //[HttpPost, Route("SNSE026M_Process")]
        //[Authorize]
        //public IHttpActionResult SNSE026M_Process([FromBody] dynamic[] pDataList)
        //{
        //    try
        //    {
        //        DataSet ds = new DataSet();
        //        int i = 0;
        //        foreach (var data in pDataList)
        //        {
        //            i = i + 1;
        //            OracleCommand cmd = new OracleCommand();
        //            cmd.CommandText = "SP_NP_SNSE026M_START_PROCESS";
        //            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
        //            cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
        //            cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = data.ORDER_NO;
        //            cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Varchar2).Value = data.ITEM_NO;
        //            cmd.Parameters.Add("P_SUB_ITEM_NO", OracleDbType.Varchar2).Value = data.SUB_ITEM_NO;
        //            cmd.Parameters.Add("P_OLD_SUB_ITEM_NO", OracleDbType.Varchar2).Value = data.OLD_SUB_ITEM_NO;
        //            cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = data.BRANCH_CODE;
        //            cmd.Parameters.Add("P_OVERSEA_FLAG_C", OracleDbType.Varchar2).Value = data.OVERSEA_FLAG_C;
        //            cmd.Parameters.Add("P_DELIVERY_CODE", OracleDbType.Varchar2).Value = data.DELIVERY_CODE;
        //            cmd.Parameters.Add("P_ROUTE_CODE", OracleDbType.Varchar2).Value = data.ROUTE_CODE;
        //            cmd.Parameters.Add("P_TRANSPORT_CODE", OracleDbType.Varchar2).Value = data.TRANSPORT_CODE;
        //            cmd.Parameters.Add("P_ZONE_AREA", OracleDbType.Varchar2).Value = data.ZONE_AREA;
        //            cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = data.ORDER_CLASS;
        //            cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = data.ORDER_PART_NO;
        //            cmd.Parameters.Add("P_ACCEPT_PART_NO", OracleDbType.Varchar2).Value = data.ACCEPT_PART_NO;
        //            cmd.Parameters.Add("P_ORDER_QTY", OracleDbType.Varchar2).Value = data.ORDER_QTY;
        //            cmd.Parameters.Add("P_ACCEPT_QTY", OracleDbType.Varchar2).Value = data.ACCEPT_QTY;
        //            cmd.Parameters.Add("P_ORDER_DATE", OracleDbType.Date).Value = data.ORDER_DATE;
        //            cmd.Parameters.Add("P_ALLOCATE_DATE", OracleDbType.Date).Value = data.ALLOCATE_DATE;
        //            cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = data.INVOICE_NO;
        //            cmd.Parameters.Add("P_INVOICE_DATE", OracleDbType.Varchar2).Value = data.INVOICE_DATE;
        //            cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = data.SENT_BY;
        //            cmd.Parameters.Add("P_PICK_LOT_DATE", OracleDbType.Date).Value = pDataList[0].PICK_LOT_DATE;
        //            cmd.Parameters.Add("P_PICK_LOT_SEQ", OracleDbType.Varchar2).Value = pDataList[0].PICK_LOT_SEQ;
        //            cmd.Parameters.Add("P_PICK_LIST_NO", OracleDbType.Varchar2).Value = pDataList[0].PICKING_LIST_NO;
        //            cmd.Parameters.Add("P_CREATE_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
        //            cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE026_M";
        //            cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
        //            cmd.Parameters.Add("P_LAST_UPD_DATE", OracleDbType.Date).Value = data.LAST_UPD_DATE;
        //            cmd.Parameters.Add("P_CHK_PROCESS", OracleDbType.Varchar2).Value = i;
        //            cmd.Parameters.Add("P_IP_ADDRESS", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
        //            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

        //            ds = oOracledb.SqlQueryDS_CMD(cmd);

        //        }
        //        return Json(DTFM.convertToList(ds));
        //    }
        //    catch (Exception ex)
        //    {
        //        oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE026M_Process");
        //        return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
        //    }
        //}

        [HttpGet, Route("SNSE026M_GetPickingNo")]
        [Authorize]
        public IHttpActionResult SNSE026M_GetPickingNo()
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE026M_GET_PICKING_NO";
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE026M_GetPickingNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }



        [HttpPost, Route("SNSE026M_UpdateStatus")]
        [Authorize]
        public IHttpActionResult SNSE026M_UpdateStatus([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE026M_UPDATE_STATUS";
                cmd.Parameters.Add("P_LOCATION", OracleDbType.Varchar2).Value = pParam.LOCATION;
                cmd.Parameters.Add("P_LOCATION_FROM", OracleDbType.Varchar2).Value = pParam.LOCATION_FROM;
                cmd.Parameters.Add("P_LOCATION_TO", OracleDbType.Varchar2).Value = pParam.LOCATION_TO;
                cmd.Parameters.Add("P_IP_ADDRESS", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE026M_UpdateStatus");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        #endregion

        #region SNSE052M
        [HttpPost, Route("SNSE052M_FirstLast_Date")]
        [Authorize]
        public IHttpActionResult SNSE052M_FirstLast_Date([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE052M_FIRST_LAST_DATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ALLOCATE_DATE", OracleDbType.Varchar2).Value = pParam.ALLOCATE_DATE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE052M_FirstLast_Date");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE052M_getAllocateFromDate/{fromDate}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE052M_getAllocateFromDate(string fromDate, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            fromDate = ismoUtil.escapeCharacter(fromDate);
            var pData = pParam["DATA"];

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE052M_ALLOCATE_DATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pData.ORDER_NO;
                cmd.Parameters.Add("P_ACCEPT_PART_NO", OracleDbType.Varchar2).Value = pData.ACCEPT_PART_NO;
                cmd.Parameters.Add("P_ALLOCATE_DATE", OracleDbType.Varchar2).Value = "FROM";
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE052M_getAllocateFromDate");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE052M_getAllocateToDate/{fromDate}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE052M_getAllocateToDate(string fromDate, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            fromDate = ismoUtil.escapeCharacter(fromDate);
            var pData = pParam["DATA"];

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE052M_ALLOCATE_DATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pData.ORDER_NO;
                cmd.Parameters.Add("P_ACCEPT_PART_NO", OracleDbType.Varchar2).Value = pData.ACCEPT_PART_NO;
                cmd.Parameters.Add("P_ALLOCATE_DATE", OracleDbType.Varchar2).Value = "TO";
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE052M_getAllocateToDate");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE052M_getCustomerCode/{cusID}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE052M_getCustomerCode(string cusID, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            cusID = ismoUtil.escapeCharacter(cusID);
            var pData = pParam["DATA"];

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE052M_CUSTOMER_CODE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = cusID.ToUpper() == "NULL" ? (object)DBNull.Value : cusID;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pData.ORDER_NO;
                cmd.Parameters.Add("P_ACCEPT_PART_NO", OracleDbType.Varchar2).Value = pData.ACCEPT_PART_NO;
                cmd.Parameters.Add("P_FROM_DATE", OracleDbType.Date).Value = pData.ALLOCATE_DATE_FROM;
                cmd.Parameters.Add("P_TO_DATE", OracleDbType.Date).Value = pData.ALLOCATE_DATE_TO;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE052M_getCustomerCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE052M_GetOrderNo/{orderNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE052M_GetOrderNo(string orderNo, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            orderNo = ismoUtil.escapeCharacter(orderNo);
            var pData = pParam["DATA"];

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE052M_ORDER_NO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = orderNo.ToUpper() == "NULL" ? (object)DBNull.Value : orderNo;
                cmd.Parameters.Add("P_ACCEPT_PART_NO", OracleDbType.Varchar2).Value = pData.ACCEPT_PART_NO;
                cmd.Parameters.Add("P_FROM_DATE", OracleDbType.Date).Value = pData.ALLOCATE_DATE_FROM;
                cmd.Parameters.Add("P_TO_DATE", OracleDbType.Date).Value = pData.ALLOCATE_DATE_TO;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE052M_GetOrderNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE052M_GetAcceptPartNo/{acceptPartNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE052M_GetAcceptPartNo(string acceptPartNo, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            acceptPartNo = ismoUtil.escapeCharacter(acceptPartNo);
            var pData = pParam["DATA"];

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE052M_ACCEPT_PART_NO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pData.ORDER_NO;
                cmd.Parameters.Add("P_ACCEPT_PART_NO", OracleDbType.Varchar2).Value = acceptPartNo.ToUpper() == "NULL" ? (object)DBNull.Value : acceptPartNo;
                cmd.Parameters.Add("P_FROM_DATE", OracleDbType.Date).Value = pData.ALLOCATE_DATE_FROM;
                cmd.Parameters.Add("P_TO_DATE", OracleDbType.Date).Value = pData.ALLOCATE_DATE_TO;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE052M_GetAcceptPartNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE052M_GetPickingListNoFrom/{pickingListNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE052M_GetPickingListNoFrom(string pickingListNo, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            pickingListNo = ismoUtil.escapeCharacter(pickingListNo);
            var pData = pParam["DATA"];

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE052M_PICKING_NO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pData.ORDER_NO;
                cmd.Parameters.Add("P_ACCEPT_PART_NO", OracleDbType.Varchar2).Value = pData.ACCEPT_PART_NO;
                cmd.Parameters.Add("P_PICK_LIST_NO", OracleDbType.Varchar2).Value = pickingListNo.ToUpper() == "NULL" ? (object)DBNull.Value : pickingListNo;
                cmd.Parameters.Add("P_TYPE", OracleDbType.Varchar2).Value = "FROM";
                cmd.Parameters.Add("P_FROM_DATE", OracleDbType.Date).Value = pData.ALLOCATE_DATE_FROM;
                cmd.Parameters.Add("P_TO_DATE", OracleDbType.Date).Value = pData.ALLOCATE_DATE_TO;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE052M_GetPickingListNoFrom");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE052M_GetPickingListNoTo/{pickingListNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE052M_GetPickingListNoTo(string pickingListNo, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            pickingListNo = ismoUtil.escapeCharacter(pickingListNo);
            var pData = pParam["DATA"];

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE052M_PICKING_NO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pData.ORDER_NO;
                cmd.Parameters.Add("P_ACCEPT_PART_NO", OracleDbType.Varchar2).Value = pData.ACCEPT_PART_NO;
                cmd.Parameters.Add("P_PICK_LIST_NO", OracleDbType.Varchar2).Value = pickingListNo.ToUpper() == "NULL" ? (object)DBNull.Value : pickingListNo;
                cmd.Parameters.Add("P_TYPE", OracleDbType.Varchar2).Value = "TO";
                cmd.Parameters.Add("P_FROM_DATE", OracleDbType.Date).Value = pData.ALLOCATE_DATE_FROM;
                cmd.Parameters.Add("P_TO_DATE", OracleDbType.Date).Value = pData.ALLOCATE_DATE_TO;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE052M_GetPickingListNoTo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE052M_getData")]
        [Authorize]
        public IHttpActionResult SNSE052M_getData([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE052M_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pParam.ORDER_NO;
                cmd.Parameters.Add("P_ACCEPT_PART_NO", OracleDbType.Varchar2).Value = pParam.ACCEPT_PART_NO;
                cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = pParam.ORDER_CLASS;
                cmd.Parameters.Add("P_FROM_DATE", OracleDbType.Date).Value = pParam.ALLOCATE_DATE_FROM;
                cmd.Parameters.Add("P_TO_DATE", OracleDbType.Date).Value = pParam.ALLOCATE_DATE_TO;
                cmd.Parameters.Add("P_FROM_PICK_NO", OracleDbType.Varchar2).Value = pParam.FROM_PICK_LIST_NO;
                cmd.Parameters.Add("P_TO_PICK_NO", OracleDbType.Varchar2).Value = pParam.TO_PICK_LIST_NO;
                cmd.Parameters.Add("P_INSTRUCTION_NO", OracleDbType.Varchar2).Value = pParam.INSTRUCTION_NO;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE052M_getData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNSE052M_getCombineCustomerCode/{cusID}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE052M_getCombineCustomerCode(string cusID, int noofshowrec)
        {
            DataSet ds = new DataSet();
            cusID = ismoUtil.escapeCharacter(cusID);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE052M_COM_CUST_CODE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = cusID.ToUpper() == "NULL" ? (object)DBNull.Value : cusID;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE052M_getCombineCustomerCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }
        [HttpPost, Route("SNSE052M_getCombinePickNo/{pickNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE052M_getCombinePickNo(string pickNo, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            pickNo = ismoUtil.escapeCharacter(pickNo);
            var pData = pParam["DATA"];

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE052M_COM_PICK_NO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_PICKING_LIST_NO", OracleDbType.Varchar2).Value = pickNo.ToUpper() == "NULL" ? (object)DBNull.Value : pickNo;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE052M_getCombinePickNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE052M_combinePL")]
        [Authorize]
        public IHttpActionResult SNSE052M_combinePL([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE052M_GEN_COMBINE_SEQ";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_IP_ADDRESS", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE052M_combinePL");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        public string SNSE052M_DelWorkFile()
        {
            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSE052M_DEL_WORKFILE";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
            cmd.Parameters.Add("P_IP_ADDRESS", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return "DONE";
        }

        [HttpPost, Route("SNSE052M_genCondition")]
        [Authorize]
        public IHttpActionResult SNSE052M_genCondition([FromBody] dynamic[] pDataList)
        {
            SNSE052M_DelWorkFile();

            DataSet ds = new DataSet();

            try
            {
                foreach (var data in pDataList)
                {
                    OracleCommand cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE052M_GEN_CONDITION";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_LOCATION", OracleDbType.Varchar2).Value = data.LOCATION;
                    cmd.Parameters.Add("P_PICK_LIST_NO", OracleDbType.Varchar2).Value = data.PICK_LIST_NO;
                    cmd.Parameters.Add("P_INSTRUCTION_NO", OracleDbType.Varchar2).Value = data.INSTRUCTION_NO;
                    cmd.Parameters.Add("P_LOCATION_NO", OracleDbType.Varchar2).Value = data.LOCATION_NO;
                    cmd.Parameters.Add("P_ACCEPT_PART_NO", OracleDbType.Varchar2).Value = data.ACCEPT_PART_NO;
                    cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = data.ORDER_PART_NO;
                    cmd.Parameters.Add("P_IP_ADDRESS", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                    ds = oOracledb.SqlQueryDS_CMD(cmd);
                }
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE052M_genCondition");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE052M_getDataCombinePL")]
        [Authorize]
        public IHttpActionResult SNSE052M_getDataCombinePL([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE052M_GET_DATA_CB";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE_0;
                cmd.Parameters.Add("P_CUSTOMER_CODE_2", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE_1;
                cmd.Parameters.Add("P_CUSTOMER_CODE_3", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE_2;
                cmd.Parameters.Add("P_CUSTOMER_CODE_4", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE_3;
                cmd.Parameters.Add("P_CUSTOMER_CODE_5", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE_4;
                cmd.Parameters.Add("P_PICKING_LIST_NO", OracleDbType.Varchar2).Value = pParam.PICKING_LIST_NO_0;
                cmd.Parameters.Add("P_PICKING_LIST_NO_2", OracleDbType.Varchar2).Value = pParam.PICKING_LIST_NO_1;
                cmd.Parameters.Add("P_PICKING_LIST_NO_3", OracleDbType.Varchar2).Value = pParam.PICKING_LIST_NO_2;
                cmd.Parameters.Add("P_PICKING_LIST_NO_4", OracleDbType.Varchar2).Value = pParam.PICKING_LIST_NO_3;
                cmd.Parameters.Add("P_PICKING_LIST_NO_5", OracleDbType.Varchar2).Value = pParam.PICKING_LIST_NO_4;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE052M_getDataCombinePL");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        #endregion

        #region SNIE001_N

        [HttpGet, Route("SNIE001_N_GetDocList/{DocNo=null}/{NoOfShowRecords=50}")]
        [Authorize]
        public IHttpActionResult SNIE001_N_GetDocList(string DocNo = null, string NoOfShowRecords = "50")
        {
            try
            {
                DataSet ds = new DataSet();
                DocNo = ismoUtil.escapeCharacter(DocNo);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNIE001_N_GET_DOC";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DOCUMEN_NO", OracleDbType.Varchar2).Value = DocNo != "null" ? DocNo : "";
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = NoOfShowRecords;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNIE001_N_GET_DOC");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));

            }
        }


        [HttpGet, Route("SNIE001_N_GetDocDetail/{DocNo=null}/{NoOfShowRecords=50}")]
        [Authorize]
        public IHttpActionResult SNIE001_N_GetDocDetail(string DocNo = null, string NoOfShowRecords = "50")
        {
            try
            {
                DataSet ds = new DataSet();
                DocNo = ismoUtil.escapeCharacter(DocNo);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNIE001_N_GET_DOC_DETAIL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DOCUMEN_NO", OracleDbType.Varchar2).Value = DocNo != "null" ? DocNo : "";
                //  cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = NoOfShowRecords;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNIE001_N_GET_DOC");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));

            }
        }


        [HttpPost, Route("SNIE001_N_GetList")]
        [Authorize]
        public IHttpActionResult SNIE001_N_GetList([FromBody] dynamic pData)
        {
            try
            {

                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNIE001_N_GET_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DOCUMENT_TYPE", OracleDbType.Varchar2).Value = pData.DOCUMENT_TYPE;
                cmd.Parameters.Add("P_REASON_CODE", OracleDbType.Varchar2).Value = pData.REASON_CODE;
                cmd.Parameters.Add("P_DOCUMENT_NO", OracleDbType.Varchar2).Value = pData.DOCUMENT_NO;
                cmd.Parameters.Add("P_DOCUMENT_DATE", OracleDbType.Date).Value = pData._DOCUMENT_DATE == null ? (Object)DBNull.Value : Convert.ToDateTime(pData._DOCUMENT_DATE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pData.PART_NO;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNIE001_N_GET_LIST");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpGet, Route("SNIE001_N_GetHead/{DocNo=null}")]
        [Authorize]
        public IHttpActionResult SNIE001_N_GetHead(string DocNo)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNIE001_N_GET_HEADER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DOCUMENT_NO", OracleDbType.Varchar2).Value = DocNo;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNIE001_N_GET_HEADER");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpGet, Route("SNIE001_N_GetDocType/{DocType=null}/{noofshowrec=null}")]
        [Authorize]
        public IHttpActionResult SNIE001_N_GetDocType(string DocType = null, string noofshowrec = null)
        {
            try
            {
                DataSet ds = new DataSet();
                DocType = ismoUtil.escapeCharacter(DocType);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNIE001_N_DOC_TYPE_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DOCUMENT_TYPE", OracleDbType.Varchar2).Value = DocType.ToUpper() != "NULL" ? DocType : "";
                cmd.Parameters.Add("p_SHOW_RECORD", OracleDbType.Varchar2).Value = noofshowrec != "null" ? noofshowrec : "20";
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNIE001_N_DOC_TYPE_LIST");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpGet, Route("SNIE001_N_GetReason/{ReasonCode=null}/{noofshowrec=null}")]
        [Authorize]
        public IHttpActionResult SNIE001_N_GetReason(string ReasonCode = null, string noofshowrec = null)
        {
            try
            {
                DataSet ds = new DataSet();
                ReasonCode = ismoUtil.escapeCharacter(ReasonCode);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNIE001_N_REASON_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_REASON_CODE", OracleDbType.Varchar2).Value = ReasonCode != "null" ? ReasonCode : "";
                cmd.Parameters.Add("P_JOB_TYPE", OracleDbType.Varchar2).Value = "";
                cmd.Parameters.Add("p_SHOW_RECORD", OracleDbType.Varchar2).Value = noofshowrec != "null" ? noofshowrec : "100";
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNIE001_N_REASON_LIST");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNIE001_N_GetReason/{ReasonCode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNIE001_N_GetReason(string ReasonCode, string noofshowrec, [FromBody] dynamic pData)
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNIE001_N_REASON_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_REASON_CODE", OracleDbType.Varchar2).Value = ReasonCode.ToUpper() != "NULL" ? ReasonCode : "";
                cmd.Parameters.Add("P_JOB_TYPE", OracleDbType.Varchar2).Value = pData.JOB_TYPE;
                cmd.Parameters.Add("p_SHOW_RECORD", OracleDbType.Varchar2).Value = "20";
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SP_NP_SNIE001_N_REASON_LIST");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNIE001_N_GetCus/{CusCode=null}/{noofshowrec=null}")]
        [Authorize]
        public IHttpActionResult SNIE001_N_GetCus(string CusCode = null, string noofshowrec = null)
        {
            try
            {
                DataSet ds = new DataSet();
                CusCode = ismoUtil.escapeCharacter(CusCode);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNIE001_N_CUS_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = CusCode != "null" ? CusCode : "";
                cmd.Parameters.Add("p_SHOW_RECORD", OracleDbType.Varchar2).Value = noofshowrec != "null" ? noofshowrec : "20";
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNIE001_N_CUS_LIST");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNIE001_N_GetPart/{PartCode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNIE001_N_GetPart(string PartCode, string noofshowrec, [FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();
                PartCode = ismoUtil.escapeCharacter(PartCode);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNIE001_N_PART_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = PartCode != "null" ? PartCode : "";
                cmd.Parameters.Add("P_REF_DOC_TYPE", OracleDbType.Varchar2).Value = pData.REF_DOC_TYPE != null ? pData.REF_DOC_TYPE : null;
                cmd.Parameters.Add("P_REF_DOC_NO", OracleDbType.Varchar2).Value = pData.REF_DOC_NO != null ? pData.REF_DOC_NO : null;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNIE001_N_PART_LIST");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpGet, Route("SNIE001_N_GetPartList/{DocumentNo}")]
        [Authorize]
        public IHttpActionResult SNIE001_N_GetPartList(string DocumentNo)
        {
            try
            {
                DataSet ds = new DataSet();
                DocumentNo = ismoUtil.escapeCharacter(DocumentNo);// REF_DOC_NO
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNIE001_N_GET_DETAIL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DOCUMENT_NO", OracleDbType.Varchar2).Value = DocumentNo;
                cmd.Parameters.Add("C_RETURN1", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN2", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNIE001_N_GET_DETAIL");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNIE001_N_Save")]
        [Authorize]
        public IHttpActionResult SNIE001_N_Save([FromBody] dynamic pData)
        {

            int iCount = 0;
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();

            var partList = new List<dynamic>(pData.PART_LIST);

            if (partList.Count > 0)
            {
                foreach (var item in pData.PART_LIST)
                {
                    if (item.EDIT == "EDIT")
                    {
                        iCount++;
                        cmd = new OracleCommand();
                        cmd.CommandText = "SP_NP_SNIE001_N_SAVE_TEMP";
                        cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Int16).Value = iCount;
                        cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = item["PART_NO"];
                        cmd.Parameters.Add("P_QUANTITY", OracleDbType.Double).Value = item["QUANTITY"];
                        cmd.Parameters.Add("P_UNIT_PRICE", OracleDbType.Double).Value = item["UNIT_PRICE"];
                        cmd.Parameters.Add("P_CLEAR_DATA", OracleDbType.Varchar2).Value = iCount == 1 ? "Y" : "N";
                        cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                        oOracledb.SqlQueryReturnValue(cmd);
                    }
                }
                try
                {
                    //Insert transaction
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNIE001_N_SAVE_TRX";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_DOCUMENT_TYPE", OracleDbType.Varchar2).Value = pData.DOCUMENT_TYPE;
                    cmd.Parameters.Add("P_DOCUMENT_DATE", OracleDbType.Date).Value = Convert.ToDateTime(pData._DOCUMENT_DATE);
                    cmd.Parameters.Add("P_JOB_TYPE", OracleDbType.Varchar2).Value = pData.JOB_TYPE;
                    cmd.Parameters.Add("P_REF_DOC_NO", OracleDbType.Varchar2).Value = pData.REF_DOC_NO;
                    cmd.Parameters.Add("P_REF_DOC_TYPE", OracleDbType.Varchar2).Value = pData.REF_DOC_TYPE;
                    cmd.Parameters.Add("P_REASON_CODE", OracleDbType.Varchar2).Value = pData.REASON_CODE;
                    cmd.Parameters.Add("P_REMARK", OracleDbType.Varchar2).Value = pData.REMARK;
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("R_DOCUMENT_NO", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;

                    cmd = oOracledb.SqlQueryReturnValue(cmd);

                    string documentNo = cmd.Parameters["R_DOCUMENT_NO"].Value.ToString();

                    return Json(new { DOCUMENT_NO = documentNo });
                }
                catch (Exception ex)
                {
                    oOracledb.WriteLog("ErrorHandle", ex.Message, "SNIE001_N_SAVE_TRX");
                    ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
                }

            }

            return ResponseMessage(Request.CreateResponse(HttpStatusCode.BadRequest, "No row to process."));
        }
        #endregion

        #region SNSE163
        [HttpGet, Route("SNSE163_getCustomerCode/{cusID}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE163_getCustomerCode(string cusID, int noofshowrec)
        {
            DataSet ds = new DataSet();
            cusID = ismoUtil.escapeCharacter(cusID);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE163_GET_CUSTOMER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = cusID.ToUpper() == "NULL" ? (object)DBNull.Value : cusID;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE163_getCustomerCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNSE163_getPartNo/{partNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE163_getPartNo(string partNo, int noofshowrec)
        {
            DataSet ds = new DataSet();
            partNo = ismoUtil.escapeCharacter(partNo);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE163_GET_PART_NO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = partNo.ToUpper() == "NULL" ? (object)DBNull.Value : partNo;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE163_getPartNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNSE163_getModelCode/{modelCode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE163_getModelCode(string modelCode, int noofshowrec)
        {
            DataSet ds = new DataSet();
            modelCode = ismoUtil.escapeCharacter(modelCode);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE163_GET_MODEL_CODE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_MODEL_CODE", OracleDbType.Varchar2).Value = modelCode.ToUpper() == "NULL" ? (object)DBNull.Value : modelCode;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE163_getModelCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE163_getData")]
        [Authorize]
        public IHttpActionResult SNSE163_getData([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE163_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pParam.PART_NO;
                cmd.Parameters.Add("P_MODEL_CODE", OracleDbType.Varchar2).Value = pParam.MODEL_CODE;
                cmd.Parameters.Add("P_GP_RATE", OracleDbType.Int32).Value = pParam.GP_RATE == "" ? pParam.GP_RATE = null : pParam.GP_RATE;
                cmd.Parameters.Add("P_CAL_DIS_RATE", OracleDbType.Varchar2).Value = pParam.CAL_DIS_RATE;
                cmd.Parameters.Add("P_SETTING_TYPE", OracleDbType.Varchar2).Value = pParam.SETTING_TYPE == "" ? pParam.SETTING_TYPE = null : pParam.SETTING_TYPE;
                cmd.Parameters.Add("P_CONFIRM_STATUS", OracleDbType.Varchar2).Value = pParam.CONFIRM_STATUS == "" ? pParam.CONFIRM_STATUS = null : pParam.CONFIRM_STATUS;
                cmd.Parameters.Add("P_OPP_GR_RATE", OracleDbType.Varchar2).Value = pParam.OPP_GR_RATE;
                cmd.Parameters.Add("P_COMPLETE_STATUS", OracleDbType.Varchar2).Value = pParam.COMPLETE_STATUS == "" ? pParam.COMPLETE_STATUS = null : pParam.COMPLETE_STATUS;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE163_getData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE163_CalAdjRate")]
        [Authorize]
        public IHttpActionResult SNSE163_CalAdjRate([FromBody] dynamic[] pDataList)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();

            try
            {
                foreach (var data in pDataList)
                {
                    OracleCommand cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE0163_CAL_ADJ_RATE";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_PART_ON", OracleDbType.Varchar2).Value = data.PART_NO;
                    cmd.Parameters.Add("P_PMS_ADJ_RATE", OracleDbType.Varchar2).Value = data.PMS_ADJ_RATE;
                    cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE163";
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                    ds = oOracledb.SqlQueryDS_CMD(cmd);
                    dt = ds.Tables[0];
                }
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE163_CalAdjRate");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpPost, Route("SNSE163_CalDisRate")]
        [Authorize]
        public IHttpActionResult SNSE163_CalDisRate([FromBody] dynamic[] pDataList)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();

            try
            {
                foreach (var data in pDataList)
                {
                    OracleCommand cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE163_CAL_DC_SETTING";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_BUYER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = data.PART_NO;
                    cmd.Parameters.Add("P_DESCRIPTION_CODE", OracleDbType.Varchar2).Value = data.DESCRIPTION_CODE;
                    cmd.Parameters.Add("P_SALES_CLASS_CODE", OracleDbType.Varchar2).Value = data.SALES_CLASS_CODE;
                    cmd.Parameters.Add("P_CAL_DIS_RATE", OracleDbType.Varchar2).Value = data.CAL_DIS_RATE;
                    cmd.Parameters.Add("P_RETAIL_PRICE", OracleDbType.Varchar2).Value = data.RETAIL_PRICE;
                    cmd.Parameters.Add("P_GDR_TYPE", OracleDbType.Varchar2).Value = data.GDR_TYPE;
                    cmd.Parameters.Add("P_GLOBAL_DISCOUNT_RATE", OracleDbType.Varchar2).Value = data.GLOBAL_DISCOUNT_RATE;
                    cmd.Parameters.Add("P_COMPLETE_STATUS", OracleDbType.Varchar2).Value = data.COMPLETE_STATUS;
                    cmd.Parameters.Add("P_PMS_FOB_PRICE", OracleDbType.Varchar2).Value = data.PMS_FOB_PRICE;
                    cmd.Parameters.Add("P_STANDARD_COST", OracleDbType.Varchar2).Value = data.STANDARD_COST;
                    cmd.Parameters.Add("P_PMS_ADJ_RATE", OracleDbType.Varchar2).Value = data.PMS_ADJ_RATE;
                    cmd.Parameters.Add("P_GLOBAL_LIST_PRICE_JPY", OracleDbType.Varchar2).Value = data.GLOBAL_LIST_PRICE_JPY;
                    cmd.Parameters.Add("P_GLOBAL_FOB_PRICE", OracleDbType.Varchar2).Value = data.GLOBAL_FOB_PRICE;
                    cmd.Parameters.Add("P_EXCHANGE_RATE_JPY", OracleDbType.Varchar2).Value = data.EXCHANGE_RATE_JPY;
                    cmd.Parameters.Add("P_TEMP_COST", OracleDbType.Varchar2).Value = data.TEMP_COST;
                    cmd.Parameters.Add("P_ITP_PARTS_FLAG", OracleDbType.Varchar2).Value = data.ITP_PARTS_FLAG;
                    cmd.Parameters.Add("P_ADJ_FOB_PRICE", OracleDbType.Varchar2).Value = data.ADJ_FOB_PRICE;
                    cmd.Parameters.Add("P_CONFIRM_STATUS", OracleDbType.Varchar2).Value = data.CONFIRM_STATUS;
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                    ds = oOracledb.SqlQueryDS_CMD(cmd);
                    dt = ds.Tables[0];
                }
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE163_CalDisRate");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE163_RegisConfirm")]
        [Authorize]
        public IHttpActionResult SNSE163_RegisConfirm([FromBody] dynamic[] pDataList)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();

            try
            {
                foreach (var data in pDataList)
                {
                    OracleCommand cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE163_REGIS_COMFIRM";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_BUYER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = data.PART_NO;
                    cmd.Parameters.Add("P_COMPLETE_STATUS", OracleDbType.Varchar2).Value = data.COMPLETE_STATUS;
                    cmd.Parameters.Add("P_STANDARD_COST", OracleDbType.Varchar2).Value = data.STANDARD_COST;
                    cmd.Parameters.Add("P_GLOBAL_LIST_PRICE", OracleDbType.Varchar2).Value = data.GLOBAL_LIST_PRICE;
                    cmd.Parameters.Add("P_TEMP_COST", OracleDbType.Varchar2).Value = data.TEMP_COST;
                    cmd.Parameters.Add("P_ITP_PARTS_FLAG", OracleDbType.Varchar2).Value = data.ITP_PARTS_FLAG;
                    cmd.Parameters.Add("P_CONFIRM_STATUS", OracleDbType.Varchar2).Value = data.CONFIRM_STATUS;
                    cmd.Parameters.Add("P_FOB_PRICE_FC", OracleDbType.Varchar2).Value = data.FOB_PRICE_FC;
                    cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_PRICE_APP", OracleDbType.Date).Value = data.EFFECTIVE_DATE;
                    cmd.Parameters.Add("P_CURRENCY_CODE", OracleDbType.Varchar2).Value = data.CURRENCY_CODE;
                    cmd.Parameters.Add("P_BASE_PRICE", OracleDbType.Varchar2).Value = data.BASE_PRICE;
                    cmd.Parameters.Add("P_GP_RATE", OracleDbType.Varchar2).Value = data.GP_RATE;
                    cmd.Parameters.Add("P_FOB_PRICE_FLAG", OracleDbType.Varchar2).Value = data.FOB_PRICE_FLAG;
                    cmd.Parameters.Add("P_PMS_GP_RATE", OracleDbType.Varchar2).Value = data.PMS_GP_RATE;
                    cmd.Parameters.Add("P_ADJ_GP_RATE", OracleDbType.Varchar2).Value = data.ADJ_GP_RATE;
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                    ds = oOracledb.SqlQueryDS_CMD(cmd);
                    dt = ds.Tables[0];
                }
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE163_RegisConfirm");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpPost, Route("SNSE163_ExportExcel")]
        [Authorize]
        public HttpResponseMessage SNSE163_ExportExcel(List<DataModel.SNSE163> Data)
        {
            HttpResponseMessage response = new HttpResponseMessage(HttpStatusCode.OK);
            string gLocation = "~/tempFile/" + "SNSE163" + "/";
            string gWriteLocation = System.Web.HttpContext.Current.Server.MapPath(gLocation);
            FileInfo filePath = new FileInfo(gWriteLocation);
            filePath.Directory.Create();
            string gFileLocation = Path.Combine(gWriteLocation, "SNSE163.xlsx");
            string gReturnLocation = Path.Combine(gLocation, "SNSE163.xlsx");

            var excel2 = Data.ToExcel(x => x
            .AddColumn("Seq.", z => z.SEQ)
            .AddColumn("Customer Code", z => z.CUSTOMER_CODE)
            .AddColumn("Part No.", z => z.PART_NO)
            .AddColumn("Part Name", z => z.PART_NAME)
            .AddColumn("ITP Part Flag", z => z.ITP_PARTS_FLAG)
            .AddColumn("Calculate By", z => z.CALCULATE_BY)
            .AddColumn("In Store Cost(THB)", z => z.STANDARD_COST)
            .AddColumn("Average Cost(THB)", z => z.AVERAGE_COST)
            .AddColumn("Retail Price(THB)", z => z.RETAIL_PRICE)
            .AddColumn("Temp Cost(THB)", z => z.TEMP_COST)
            .AddColumn("Global List Price(THB)", z => z.GLOBAL_LIST_PRICE)
            .AddColumn("Global Discount Rate(%)", z => z.GLOBAL_DISCOUNT_RATE)
            .AddColumn("GDR Type", z => z.DSP_GDR_TYPE)
            .AddColumn("Model Code", z => z.MODEL_CODE)
            .AddColumn("FOB Price(THB)", z => z.GLOBAL_FOB_PRICE)
            .AddColumn("GP Rate(%)", z => z.GP_RATE)
            .AddColumn("GP Amount(THB)", z => z.DIF_GLOBAL_COST)
            .AddColumn("FOB Adjust(THB)", z => z.PMS_FOB_PRICE)
            .AddColumn("GP Rate(%)", z => z.PMS_GP_RATE)
            .AddColumn("(%) Adjust GP", z => z.PMS_ADJ_RATE)
            .AddColumn("Amount(THB)", z => z.DIF_PMS_COST)
            .AddColumn("FOB Price Customer", z => z.FOB_PRICE_FC)
            .AddColumn("Currency Code Customer", z => z.CURRENCY_CODE)
            .AddColumn("Exchange Rate(THB-->??)", z => z.EXCHANGE_RATE)
            .AddColumn("Effective Date", z => z.PRICE_APP == null ? (object)DBNull.Value : Convert.ToDateTime(z.PRICE_APP).ToString("dd-MM-yyyy"))
            .AddColumn("Global List Price(JPY)", z => z.GLOBAL_LIST_PRICE_JPY)
            .AddColumn("Exchange Rate(JPY -->THB)", z => z.EXCHANGE_RATE_JPY)
            .AddColumn("FOB Price Flag", z => z.DSC_FOB_PRICE_FLAG)
            .AddColumn("FOB Price Registered", z => z.FOB_PRICE_CONFIRM)
            .AddColumn("GP Rate(%) Registered", z => z.GP_RATE_COMFIRM)
            .AddColumn("S/C", z => z.SALES_CLASS_CODE)
            .AddColumn("D/C", z => z.DESCRIPTION_CODE)
            .AddColumn("Price Info Type", z => z.SETTING_TYPE_D)
            .AddColumn("Price Info Due Date", z => z.SETTING_DUE_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(z.SETTING_DUE_DATE).ToString("dd-MM-yyyy"))
            .AddColumn("Export Net Price", z => z.EXPORT_NET_PRICE)
            .AddColumn("Export Currency Code", z => z.EXPORT_CURRENCY_CODE)
            .AddColumn("Registration Status", z => z.DSP_CONFIRM_STATUS)
            .AddColumn("Create By", z => z.CREATE_BY)
            .AddColumn("Create Date", z => z.CREATE_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(z.CREATE_DATE).ToString("dd-MM-yyyy"))
            .AddColumn("Last Update by Program ", z => z.LAST_UPD_PGM)
            .AddColumn("Last Update by User ", z => z.LAST_UPD_BY)
            .AddColumn("Last Update Date ", z => z.LAST_UPD_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(z.LAST_UPD_DATE).ToString("dd-MM-yyyy"))
            .AddColumn("Message Error", z => z.MESSAGE_ERROR)
            );

            byte[] excel = excel2;
            // byte[] excel = dt.ToExcel();
            File.WriteAllBytes(gFileLocation, excel);
            string path = HttpContext.Current.Server.MapPath("~/" + gReturnLocation.Substring(1));
            if (!File.Exists(path))
            {
                return Request.CreateErrorResponse(HttpStatusCode.NotFound, "File not found.");
            }
            try
            {
                string mimeType = MimeMapping.GetMimeMapping("SNSE163.xlsx");
                StreamContent x = new StreamContent(new FileStream(path, FileMode.Open, FileAccess.Read));
                response.Content = x;
                response.Content.Headers.ContentType = new MediaTypeHeaderValue(mimeType);
                response.Content.Headers.ContentDisposition = new ContentDispositionHeaderValue("attachment");
                response.Content.Headers.ContentDisposition.FileName = "SNSE163.xlsx";
            }
            catch (Exception ex)
            {
                return Request.CreateResponse(HttpStatusCode.BadRequest, ex);
            }
            return response;
        }


        //[HttpPost, Route("SNSE163_ExportExcel")]
        //public HttpResponseMessage SNSE163_ExportExcel(List<DataModel.SNSE163> dModel)
        //{
        //    DataTable dt = new DataTable();
        //    string x = string.Empty;
        //    string filename = "SNSE163.csv";
        //    string floderLocation = "SNSE163";
        //    dt.Columns.Add("colum1");

        //    dt.Rows.Add("ID;PO2C");
        //    dt.Rows.Add("B");
        //    dt.Rows.Add("C;X1;Y1;K\"Seq.\"");
        //    dt.Rows.Add("C;X2;Y1;K\"Customer Code\"");
        //    dt.Rows.Add("C;X3;Y1;K\"Part No.\"");
        //    dt.Rows.Add("C;X4;Y1;K\"Part Name\"");
        //    dt.Rows.Add("C;X5;Y1;K\"ITP Part Flag\"");
        //    dt.Rows.Add("C;X6;Y1;K\"Calculate By\"");
        //    dt.Rows.Add("C;X7;Y1;K\"In Store Cost(THB)\"");
        //    dt.Rows.Add("C;X8;Y1;K\"Average Cost(THB)\"");
        //    dt.Rows.Add("C;X9;Y1;K\"Retail Price(THB)\"");
        //    dt.Rows.Add("C;X10;Y1;K\"Temp Cost(THB)\"");
        //    dt.Rows.Add("C;X11;Y1;K\"Global List Price(THB)\"");
        //    dt.Rows.Add("C;X12;Y1;K\"Global Discount Rate(%)\"");
        //    dt.Rows.Add("C;X13;Y1;K\"GDR Type\"");
        //    dt.Rows.Add("C;X14;Y1;K\"Model Code\"");
        //    dt.Rows.Add("C;X15;Y1;K\"FOB Price(THB)\"");
        //    dt.Rows.Add("C;X16;Y1;K\"GP Rate(%)\"");
        //    dt.Rows.Add("C;X17;Y1;K\"GP Amount(THB)\"");
        //    dt.Rows.Add("C;X18;Y1;K\"FOB Adjust(THB)\"");
        //    dt.Rows.Add("C;X19;Y1;K\"GP Rate(%)\"");
        //    dt.Rows.Add("C;X20;Y1;K\"(%) Adjust GP\"");
        //    dt.Rows.Add("C;X21;Y1;K\"Amount(THB)\"");
        //    dt.Rows.Add("C;X22;Y1;K\"FOB Price Customer\"");
        //    dt.Rows.Add("C;X23;Y1;K\"Currency Code Customer\"");
        //    dt.Rows.Add("C;X24;Y1;K\"Exchange Rate(THB-->??)\"");
        //    dt.Rows.Add("C;X25;Y1;K\"Effective Date\"");
        //    dt.Rows.Add("C;X26;Y1;K\"Global List Price(JPY)\"");
        //    dt.Rows.Add("C;X27;Y1;K\"Exchange Rate(JPY -->THB)\"");
        //    dt.Rows.Add("C;X28;Y1;K\"FOB Price Flag\"");
        //    dt.Rows.Add("C;X29;Y1;K\"FOB Price Registered\"");
        //    dt.Rows.Add("C;X30;Y1;K\"GP Rate(%) Registered\"");
        //    dt.Rows.Add("C;X31;Y1;K\"S/C\"");
        //    dt.Rows.Add("C;X32;Y1;K\"D/C\"");
        //    dt.Rows.Add("C;X33;Y1;K\"Price Info Type\"");
        //    dt.Rows.Add("C;X34;Y1;K\"Price Info Due Date\"");
        //    dt.Rows.Add("C;X35;Y1;K\"Export Net Price\"");
        //    dt.Rows.Add("C;X36;Y1;K\"Export Currency Code\"");
        //    dt.Rows.Add("C;X37;Y1;K\"Registration Status\"");
        //    dt.Rows.Add("C;X38;Y1;K\"Create By\"");
        //    dt.Rows.Add("C;X39;Y1;K\"Create Date\"");
        //    dt.Rows.Add("C;X40;Y1;K\"Last Update by Program \"");
        //    dt.Rows.Add("C;X41;Y1;K\"Last Update by User \"");
        //    dt.Rows.Add("C;X42;Y1;K\"Last Update Date \"");
        //    dt.Rows.Add("C;X43;Y1;K\"Message Error\"");

        //    int countrow = 0;
        //    int itemno = 0;
        //    for (var i = 0; i < dModel.Count; i++)
        //    {
        //        countrow = i + 2;
        //        itemno = i + 1;
        //        dt.Rows.Add("C;X1;Y" + countrow + ";K" + '"' + dModel[i].SEQ + '"');
        //        dt.Rows.Add("C;X2;Y" + countrow + ";K" + '"' + dModel[i].CUSTOMER_CODE + '"');
        //        dt.Rows.Add("C;X3;Y" + countrow + ";K" + '"' + dModel[i].PART_NO + '"');
        //        dt.Rows.Add("C;X4;Y" + countrow + ";K" + '"' + dModel[i].PART_NAME + '"');
        //        dt.Rows.Add("C;X5;Y" + countrow + ";K" + '"' + dModel[i].ITP_PARTS_FLAG + '"');
        //        dt.Rows.Add("C;X6;Y" + countrow + ";K" + '"' + dModel[i].CALCULATE_BY + '"');
        //        dt.Rows.Add("C;X7;Y" + countrow + ";K" + '"' + dModel[i].STANDARD_COST + '"');
        //        dt.Rows.Add("C;X8;Y" + countrow + ";K" + '"' + dModel[i].AVERAGE_COST + '"');
        //        dt.Rows.Add("C;X9;Y" + countrow + ";K" + '"' + dModel[i].RETAIL_PRICE + '"');
        //        dt.Rows.Add("C;X10;Y" + countrow + ";K" + '"' + dModel[i].TEMP_COST + '"');
        //        dt.Rows.Add("C;X11;Y" + countrow + ";K" + '"' + dModel[i].GLOBAL_LIST_PRICE + '"');
        //        dt.Rows.Add("C;X12;Y" + countrow + ";K" + '"' + dModel[i].GLOBAL_DISCOUNT_RATE + '"');
        //        dt.Rows.Add("C;X13;Y" + countrow + ";K" + '"' + dModel[i].DSP_GDR_TYPE + '"');
        //        dt.Rows.Add("C;X14;Y" + countrow + ";K" + '"' + dModel[i].MODEL_CODE + '"');
        //        dt.Rows.Add("C;X15;Y" + countrow + ";K" + '"' + dModel[i].GLOBAL_FOB_PRICE + '"');
        //        dt.Rows.Add("C;X16;Y" + countrow + ";K" + '"' + dModel[i].GP_RATE + '"');
        //        dt.Rows.Add("C;X17;Y" + countrow + ";K" + '"' + dModel[i].DIF_GLOBAL_COST + '"');
        //        dt.Rows.Add("C;X18;Y" + countrow + ";K" + '"' + dModel[i].PMS_FOB_PRICE + '"');
        //        dt.Rows.Add("C;X19;Y" + countrow + ";K" + '"' + dModel[i].PMS_GP_RATE + '"');
        //        dt.Rows.Add("C;X20;Y" + countrow + ";K" + '"' + dModel[i].PMS_ADJ_RATE + '"');
        //        dt.Rows.Add("C;X21;Y" + countrow + ";K" + '"' + dModel[i].DIF_PMS_COST + '"');
        //        dt.Rows.Add("C;X22;Y" + countrow + ";K" + '"' + dModel[i].FOB_PRICE_FC + '"');
        //        dt.Rows.Add("C;X23;Y" + countrow + ";K" + '"' + dModel[i].CURRENCY_CODE + '"');
        //        dt.Rows.Add("C;X24;Y" + countrow + ";K" + '"' + dModel[i].EXCHANGE_RATE + '"');
        //        dt.Rows.Add("C;X25;Y" + countrow + ";K" + '"' + Convert.ToDateTime(dModel[i].PRICE_APP) + '"');
        //        dt.Rows.Add("C;X26;Y" + countrow + ";K" + '"' + dModel[i].GLOBAL_LIST_PRICE_JPY + '"');
        //        dt.Rows.Add("C;X27;Y" + countrow + ";K" + '"' + dModel[i].EXCHANGE_RATE_JPY + '"');
        //        dt.Rows.Add("C;X28;Y" + countrow + ";K" + '"' + dModel[i].DSC_FOB_PRICE_FLAG + '"');
        //        dt.Rows.Add("C;X29;Y" + countrow + ";K" + '"' + dModel[i].FOB_PRICE_CONFIRM + '"');
        //        dt.Rows.Add("C;X30;Y" + countrow + ";K" + '"' + dModel[i].GP_RATE_COMFIRM + '"');
        //        dt.Rows.Add("C;X31;Y" + countrow + ";K" + '"' + dModel[i].SALES_CLASS_CODE + '"');
        //        dt.Rows.Add("C;X32;Y" + countrow + ";K" + '"' + dModel[i].DESCRIPTION_CODE + '"');
        //        dt.Rows.Add("C;X33;Y" + countrow + ";K" + '"' + dModel[i].SETTING_TYPE_D + '"');
        //        dt.Rows.Add("C;X34;Y" + countrow + ";K" + '"' + Convert.ToDateTime(dModel[i].SETTING_DUE_DATE) + '"');
        //        dt.Rows.Add("C;X35;Y" + countrow + ";K" + '"' + dModel[i].EXPORT_NET_PRICE + '"');
        //        dt.Rows.Add("C;X36;Y" + countrow + ";K" + '"' + dModel[i].EXPORT_CURRENCY_CODE + '"');
        //        dt.Rows.Add("C;X37;Y" + countrow + ";K" + '"' + dModel[i].DSP_CONFIRM_STATUS + '"');
        //        dt.Rows.Add("C;X38;Y" + countrow + ";K" + '"' + dModel[i].CREATE_BY + '"');
        //        dt.Rows.Add("C;X39;Y" + countrow + ";K" + '"' + Convert.ToDateTime(dModel[i].CREATE_DATE) + '"');
        //        dt.Rows.Add("C;X40;Y" + countrow + ";K" + '"' + dModel[i].LAST_UPD_PGM + '"');
        //        dt.Rows.Add("C;X41;Y" + countrow + ";K" + '"' + dModel[i].LAST_UPD_BY + '"');
        //        dt.Rows.Add("C;X42;Y" + countrow + ";K" + '"' + Convert.ToDateTime(dModel[i].LAST_UPD_DATE) + '"');
        //        dt.Rows.Add("C;X43;Y" + countrow + ";K" + '"' + dModel[i].MESSAGE_ERROR + '"');
        //    }

        //    dt.Rows.Add("E");

        //    x = DataService.WriteCSVNoColumn(dt, filename, floderLocation);
        //    Controllers.SystemApiController response = new Controllers.SystemApiController();
        //    var File = response.downloadFile(x.Substring(1), filename);
        //    return File;
        //}
        #endregion

        #region SNSE043N
        [HttpGet, Route("SNSE043N_getCustomerCode/{cusID}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE043N_getCustomerCode(string cusID, int noofshowrec)
        {
            DataSet ds = new DataSet();
            cusID = ismoUtil.escapeCharacter(cusID);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE043N_GET_CUSTOMER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = cusID.ToUpper() == "NULL" ? (object)DBNull.Value : cusID;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE043N_getCustomerCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE043N_getInvoiceDate/{invoiceDate}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE043N_getInvoiceDate(string invoiceDate, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            invoiceDate = ismoUtil.escapeCharacter(invoiceDate);
            var pData = pParam["DATA"];

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE043N_INVOICE_DATE";
                cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE_FROM", OracleDbType.Varchar2).Value = pData["CUSTOMER_CODE_FROM"];
                cmd.Parameters.Add("P_CUSTOMER_CODE_TO", OracleDbType.Varchar2).Value = pData["CUSTOMER_CODE_TO"];
                cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = pData["SENT_BY"];
                cmd.Parameters.Add("P_INVOICE_DATE", OracleDbType.Varchar2).Value = invoiceDate.ToUpper() == "NULL" ? (object)DBNull.Value : invoiceDate;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE043N_getInvoiceDate");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE043N_getDocNo/{DocNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE043N_getDocNo(string DocNo, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            DocNo = ismoUtil.escapeCharacter(DocNo);
            var pData = pParam["DATA"];

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE043N_DOC_NO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE_FROM", OracleDbType.Varchar2).Value = pData["CUSTOMER_CODE_FROM"];
                cmd.Parameters.Add("P_CUSTOMER_CODE_TO", OracleDbType.Varchar2).Value = pData["CUSTOMER_CODE_TO"];
                cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = pData["SENT_BY"];
                //cmd.Parameters.Add("P_INVOICE_DATE_FROM", OracleDbType.Varchar2).Value = pData["INVOICE_DATE_FROM"];
                //cmd.Parameters.Add("P_INVOICE_DATE_TO", OracleDbType.Varchar2).Value = pData["INVOICE_DATE_TO"];

                cmd.Parameters.Add("P_INVOICE_DATE_FROM", OracleDbType.Varchar2).Value = pData["INVOICE_DATE_FROM"].ToString("dd-MMM-yyyy");
                cmd.Parameters.Add("P_INVOICE_DATE_TO", OracleDbType.Varchar2).Value = pData["INVOICE_DATE_TO"].ToString("dd-MMM-yyyy");


                cmd.Parameters.Add("P_DOC_NO", OracleDbType.Varchar2).Value = DocNo.ToUpper() == "NULL" ? (object)DBNull.Value : DocNo;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE043N_getDocNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpPost, Route("SNSE43N_getData")]
        [Authorize]
        public IHttpActionResult SNSE43N_getData([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE043N_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE_FROM", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE_FROM;
                cmd.Parameters.Add("P_CUSTOMER_CODE_TO", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE_TO;
                cmd.Parameters.Add("P_DOC_NO_FROM", OracleDbType.Varchar2).Value = pParam.DOC_NO_FROM;
                cmd.Parameters.Add("P_DOC_NO_TO", OracleDbType.Varchar2).Value = pParam.DOC_NO_TO;
                cmd.Parameters.Add("P_INVOICE_DATE_FROM", OracleDbType.Varchar2).Value = pParam["INVOICE_DATE_FROM"].ToString("dd-MM-yyyy");
                cmd.Parameters.Add("P_INVOICE_DATE_TO", OracleDbType.Varchar2).Value = pParam["INVOICE_DATE_TO"].ToString("dd-MM-yyyy");
                cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = pParam.SENT_BY;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE43N_getData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE043N_ExportExcel")]
        public HttpResponseMessage SNSE043N_ExportExcel(List<DataModel.SNSE043N> dModel)
        {
            DataTable dt = new DataTable();
            string x = string.Empty;
            string filename = "SNSE043N.csv";
            string floderLocation = "SNSE043N";
            dt.Columns.Add("colum1");

            dt.Rows.Add("ID;PO2C");
            dt.Rows.Add("B");
            dt.Rows.Add("C;X1;Y1;K\"Pre-Invoice Attached Sheet\"");
            dt.Rows.Add("C;X1;Y2;K\"Pre-Invoice No\"");
            dt.Rows.Add("C;X2;Y2;K\"Buyer Code\"");
            dt.Rows.Add("C;X3;Y2;K\"Destination\"");
            dt.Rows.Add("C;X4;Y2;K\"Date (yyyy-mm-dd)\"");
            dt.Rows.Add("C;X11;Y2;K" + '"' + dModel[0].COMPANY_NAME + '"');
            dt.Rows.Add("C;X1;Y3;K" + '"' + dModel[0].PINVOICE_NO + '"');
            dt.Rows.Add("C;X2;Y3;K" + '"' + dModel[0].CUSTOMER_CODE + '"');
            dt.Rows.Add("C;X3;Y3;K" + '"' + dModel[0].BRANCH_NAME_EN + '"');
            dt.Rows.Add("C;X4;Y3;K" + '"' + Convert.ToDateTime(dModel[0].PINVOICE_DATE).ToString("dd-MM-yyyy") + '"');
            dt.Rows.Add("C;X1;Y4;K\"Container\"");
            dt.Rows.Add("C;X3;Y4;K\"Serial\"");
            dt.Rows.Add("C;X11;Y4;K" + '"' + dModel[0].CURRENCY_CODE + '"');
            dt.Rows.Add("C;X13;Y4;K\"Total\"");
            dt.Rows.Add("C;X13;Y5;K\"N/WT\"");
            dt.Rows.Add("C;X14;Y5;K\"M3\"");
            dt.Rows.Add("C;X15;Y5;K\"G/W\"");
            dt.Rows.Add("C;X1;Y6;K\"Case No.\"");
            dt.Rows.Add("C;X2;Y6;K\"Sub No.\"");
            dt.Rows.Add("C;X3;Y6;K\"Serial No.\"");
            dt.Rows.Add("C;X4;Y6;K\"Indent No.\"");
            dt.Rows.Add("C;X5;Y6;K\"Item No.\"");
            dt.Rows.Add("C;X6;Y6;K\"Parts No.\"");
            dt.Rows.Add("C;X7;Y6;K\"Description\"");
            dt.Rows.Add("C;X8;Y6;K\"Country of Origin\"");
            dt.Rows.Add("C;X9;Y6;K\"H.S. Code\"");
            dt.Rows.Add("C;X10;Y6;K\"QTY\"");
            dt.Rows.Add("C;X11;Y6;K\"Unit Price\"");
            dt.Rows.Add("C;X12;Y6;K\"Amount\"");
            dt.Rows.Add("C;X13;Y6;K\"(0.001KG)\"");
            dt.Rows.Add("C;X14;Y6;K\"(0.001m3)\"");
            dt.Rows.Add("C;X15;Y6;K\"(0.1kg.)\"");

            int countrow = 0;
            int itemno = 0;
            for (var i = 0; i < dModel.Count; i++)
            {
                countrow = i + 2;
                itemno = 7 + i;
                dt.Rows.Add("C;X1;Y" + itemno + ";K" + '"' + dModel[i].CASE_NO + '"');
                dt.Rows.Add("C;X2;Y" + itemno + ";K" + '"' + dModel[i].SUB_CASE_NO + '"');
                dt.Rows.Add("C;X3;Y" + itemno + ";K" + '"' + dModel[i].SERIAL_NO + '"');
                dt.Rows.Add("C;X4;Y" + itemno + ";K" + '"' + dModel[i].INDENT_NO + '"');
                dt.Rows.Add("C;X5;Y" + itemno + ";K" + '"' + dModel[i].V_ITEM_NO + '"');
                dt.Rows.Add("C;X6;Y" + itemno + ";K" + '"' + dModel[i].PART_NO + '"');
                dt.Rows.Add("C;X7;Y" + itemno + ";K" + '"' + dModel[i].PART_NAME_EN + '"');
                dt.Rows.Add("C;X8;Y" + itemno + ";K" + '"' + dModel[i].ORIGIN_NAME_EN + '"');
                dt.Rows.Add("C;X9;Y" + itemno + ";K" + '"' + dModel[i].TARIFF_CODE + '"');
                dt.Rows.Add("C;X10;Y" + itemno + ";K" + '"' + dModel[i].PACK_QTY + '"');
                dt.Rows.Add("C;X11;Y" + itemno + ";K" + '"' + dModel[i].UNIT_PRICE_C + '"');
                dt.Rows.Add("C;X12;Y" + itemno + ";K" + '"' + dModel[i].AAX + '"');
                dt.Rows.Add("C;X13;Y" + itemno + ";K" + '"' + dModel[i].N_WT + '"');
                dt.Rows.Add("C;X14;Y" + itemno + ";K" + '"' + dModel[i].GROSS_MM + '"');
                dt.Rows.Add("C;X15;Y" + itemno + ";K" + '"' + dModel[i].GROSS_WEIGHT + '"');
            }

            dt.Rows.Add("E");

            x = DataService.WriteCSVNoColumn(dt, filename, floderLocation);
            Controllers.SystemApiController response = new Controllers.SystemApiController();
            var File = response.downloadFile(x.Substring(1), filename);
            return File;
        }
        #endregion

        #region SNSE044N
        [HttpGet, Route("SNSE44N_getFileName")]
        [Authorize]
        public IHttpActionResult SNSE44N_getFileName()
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE044N_GET_FILE_NAME";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE44N_getFileName");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE44N_getDocType")]
        [Authorize]
        public IHttpActionResult SNSE44N_getDocType([FromBody] dynamic data)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE044N_GET_DOC_TYPE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = data.SENT_BY;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE44N_getDocType");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNSE044N_getCustomerCode/{cusID}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE044N_getCustomerCode(string cusID, int noofshowrec)
        {
            DataSet ds = new DataSet();
            cusID = ismoUtil.escapeCharacter(cusID);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE044N_GET_CUSTOMER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = cusID.ToUpper() == "NULL" ? (object)DBNull.Value : cusID;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE044N_getCustomerCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE044N_getDocNo/{DocNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE044N_getDocNo(string DocNo, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            DocNo = ismoUtil.escapeCharacter(DocNo);
            var pData = pParam["DATA"];

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE044N_DOC_NO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE_FROM", OracleDbType.Varchar2).Value = pData["CUSTOMER_CODE_FROM"];
                cmd.Parameters.Add("P_CUSTOMER_CODE_TO", OracleDbType.Varchar2).Value = pData["CUSTOMER_CODE_TO"];
                cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = pData["SENT_BY"];
                cmd.Parameters.Add("P_DOC_TYPE", OracleDbType.Varchar2).Value = pData["DOC_TYPE"];
                cmd.Parameters.Add("P_DOCUMENT_DATE_FROM", OracleDbType.Varchar2).Value = pData["INVOICE_DATE_FROM"] == null ? (object)DBNull.Value : pData["INVOICE_DATE_FROM"].ToString("dd-MM-yyyy");
                cmd.Parameters.Add("P_DOCUMENT_DATE_TO", OracleDbType.Varchar2).Value = pData["INVOICE_DATE_TO"] == null ? (object)DBNull.Value : pData["INVOICE_DATE_TO"].ToString("dd-MM-yyyy");
                cmd.Parameters.Add("P_DOC_NO", OracleDbType.Varchar2).Value = DocNo.ToUpper() == "NULL" ? (object)DBNull.Value : DocNo;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE044N_getDocNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE044N_getData")]
        [Authorize]
        public HttpResponseMessage SNSE044N_getData([FromBody] dynamic data)
        {
            DataSet ds = new DataSet();

            string xxx = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSE044N_GET_DATA";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_CUSTOMER_CODE_FROM", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE_FROM;
            cmd.Parameters.Add("P_CUSTOMER_CODE_TO", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE_TO;
            cmd.Parameters.Add("P_DOC_NO_FROM", OracleDbType.Varchar2).Value = data.DOC_NO_FROM;
            cmd.Parameters.Add("P_DOC_NO_TO", OracleDbType.Varchar2).Value = data.DOC_NO_TO;
            cmd.Parameters.Add("P_INVOICE_DATE_FROM", OracleDbType.Varchar2).Value = data["INVOICE_DATE_FROM"] == null ? (object)DBNull.Value : data["INVOICE_DATE_FROM"].ToString("dd-MM-yyyy");
            cmd.Parameters.Add("P_INVOICE_DATE_TO", OracleDbType.Varchar2).Value = data["INVOICE_DATE_TO"] == null ? (object)DBNull.Value : data["INVOICE_DATE_TO"].ToString("dd-MM-yyyy");
            cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = data.SENT_BY;
            cmd.Parameters.Add("P_DOC_TYPE", OracleDbType.Varchar2).Value = data.DOC_TYPE;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            string x = SNSE044N_ExportExcel(ds.Tables[0], "SNSE044N.csv", "SNSE044N");
            SystemApiController sys = new SystemApiController();
            var respons = sys.downloadFile(x.Substring(1), "SNSE044N.csv");
            return respons;

        }

        public static string SNSE044N_ExportExcel(DataTable dt, string fileName, string FloderLocation)
        {
            string invoiceNo = "AABBBBBBBCCCCCCCCCCCCCCCCC";
            string caseNo = "AABBBBBBBCCCCCCCCCCCCCCCCC";
            bool Succes = false;
            bool firstInsert = false;
            decimal totalCaseM3 = 0;
            decimal totalCaseGW = 0;
            decimal totalCaseQTY = 0;
            decimal totalCaseAMT = 0;
            decimal totalCaseNW = 0;

            decimal totalConM3 = 0;
            decimal totalConGW = 0;
            decimal totalConQTY = 0;
            decimal totalConAMT = 0;
            decimal totalConNW = 0;


            string gLocation = "~/tempFile/" + FloderLocation + "/";
            string gWriteLocation = System.Web.HttpContext.Current.Server.MapPath(gLocation);

            FileInfo filePath = new FileInfo(gWriteLocation);
            filePath.Directory.Create();
            string gFileLocation = Path.Combine(gWriteLocation, fileName);
            string gReturnLocation = Path.Combine(gLocation, fileName);
            SystemApiController sys = new SystemApiController();
            try
            {
                StringBuilder sb = new StringBuilder();

                foreach (DataRow row in dt.Rows)
                {

                    if (row[0].ToString() != invoiceNo)
                    {
                        invoiceNo = row[0].ToString();
                        caseNo = row[8].ToString();

                        if (firstInsert)
                        {
                            string RowTotalCase = "," + "," + "," + "," + "," + "," + "Total By Case" + "," + "," + "," + totalCaseQTY + "," + "," + totalCaseAMT + "," + totalCaseNW + "," + totalCaseM3 + "," + totalCaseGW;
                            sb.AppendLine(RowTotalCase);
                            totalConQTY = totalConQTY + totalCaseQTY;
                            totalConAMT = totalConAMT + totalCaseAMT;
                            totalConNW = totalConNW + totalCaseNW;
                            totalConM3 = totalConM3 + totalCaseM3;
                            totalConGW = totalConGW + totalCaseGW;
                            totalCaseQTY = 0;
                            totalCaseAMT = 0;
                            totalCaseNW = 0;
                            totalCaseM3 = 0;
                            totalCaseGW = 0;

                            string RowTotalContainer = "," + "," + "," + "," + "," + "," + "Total By Container" + "," + "," + "," + totalConQTY + "," + "," + totalConAMT + "," + totalConNW + "," + totalConM3 + "," + totalConGW;
                            sb.AppendLine(RowTotalContainer);
                            totalConQTY = 0;
                            totalConAMT = 0;
                            totalConNW = 0;
                            totalConM3 = 0;
                            totalConGW = 0;
                        }

                        string firstRow = "Invoice Attached Sheet";
                        sb.AppendLine(firstRow);

                        string secondRow = "Invoice No" + "," + "Buyer Code" + "," + "Destination" + "," + "Date (yyyy-mm-dd)" + "," + "," + "," + "," + "," + "," + "," + "NISSAN MOTOR (THAILAND) CO. LTD.";
                        sb.AppendLine(secondRow);

                        string thirdRow = row[0].ToString() + "," + row[1].ToString() + "," + row[2].ToString() + "," + row[3].ToString() + "," + "," + "," + "," + "," + "," + "," + "," + "," + ",";
                        sb.AppendLine(thirdRow);

                        string fouthRow = "Container" + "," + "," + "Serial" + "," + "," + "," + "," + "," + "," + "," + "," + row[4].ToString() + "," + "," + "Total";
                        sb.AppendLine(fouthRow);

                        string fifthRow = "," + "," + "," + "," + "," + "," + "," + "," + "," + "," + "," + "," + "N/WT" + "," + "M3" + "," + "G/W";
                        sb.AppendLine(fifthRow);

                        string[] HeaderForExport = new string[]
                    {   "Case No.",
                        "Sub No.",
                        "Serial No.",
                        "Indent No.",
                        "Item No.",
                        "Parts No.",
                        "Description",
                        "Country of Origin",
                        "H.S. Code",
                        "QTY",
                        "Unit Price",
                        "Amount",
                        "(0.001KG)",
                        "(0.001m3)",
                        "(0.1kg.)"
                    };

                        IEnumerable<string> columnNames = HeaderForExport.Select(h => h.ToString());
                        sb.AppendLine(string.Join(",", columnNames));

                        for (int i = 8; i < dt.Columns.Count - 19; i++)
                        {
                            sb.Append(CsvEscape(row[i].ToString()));
                            sb.Append(i == dt.Columns.Count - 20 ? "\n" : ",");
                        }


                        totalCaseQTY = decimal.Parse(row[23].ToString());
                        totalCaseAMT = decimal.Parse(row[24].ToString());
                        totalCaseNW = decimal.Parse(row[25].ToString());

                        totalCaseM3 = decimal.Parse(row[21].ToString());
                        totalCaseGW = decimal.Parse(row[22].ToString());


                        //totalConQTY = decimal.Parse(row[26].ToString());
                        //totalConAMT = decimal.Parse(row[27].ToString());
                        //totalConNW = decimal.Parse(row[28].ToString());

                        //totalCaseM3 = decimal.Parse(row[21].ToString());
                        //totalCaseGW = decimal.Parse(row[22].ToString());

                        //totalConM3 = decimal.Parse(row[31].ToString());
                        //totalConGW = decimal.Parse(row[32].ToString());
                    }
                    else
                    {
                        firstInsert = true;

                        if (row[8].ToString() != caseNo && row[0].ToString() == invoiceNo)
                        {
                            invoiceNo = row[0].ToString();
                            caseNo = row[8].ToString();

                            string RowTotalCase = "," + "," + "," + "," + "," + "," + "Total By Case" + "," + "," + "," + totalCaseQTY + "," + "," + totalCaseAMT + "," + totalCaseNW + "," + totalCaseM3 + "," + totalCaseGW;
                            sb.AppendLine(RowTotalCase);
                            totalConQTY = totalConQTY + totalCaseQTY;
                            totalConAMT = totalConAMT + totalCaseAMT;
                            totalConNW = totalConNW + totalCaseNW;
                            totalConM3 = totalConM3 + totalCaseM3;
                            totalConGW = totalConGW + totalCaseGW;
                            totalCaseQTY = 0;
                            totalCaseAMT = 0;
                            totalCaseNW = 0;
                            totalCaseM3 = 0;
                            totalCaseGW = 0;

                            for (int i = 8; i < dt.Columns.Count - 19; i++)
                            {
                                sb.Append(CsvEscape(row[i].ToString()));
                                sb.Append(i == dt.Columns.Count - 20 ? "\n" : ",");
                            }

                            totalCaseQTY = decimal.Parse(row[23].ToString());
                            totalCaseAMT = decimal.Parse(row[24].ToString());
                            totalCaseNW = decimal.Parse(row[25].ToString());
                            totalCaseM3 = decimal.Parse(row[21].ToString());
                            totalCaseGW = decimal.Parse(row[22].ToString());

                        }
                        else
                        {
                            sb.Append("," + ",");
                            for (int i = 10; i < dt.Columns.Count - 19; i++)
                            {
                                sb.Append(CsvEscape(row[i].ToString()));
                                sb.Append(i == dt.Columns.Count - 20 ? "\n" : ",");
                            }

                            totalCaseQTY = decimal.Parse(row[23].ToString());
                            totalCaseAMT = decimal.Parse(row[24].ToString());
                            totalCaseNW = decimal.Parse(row[25].ToString());
                            totalCaseM3 = decimal.Parse(row[21].ToString());
                            totalCaseGW = decimal.Parse(row[22].ToString());

                        }
                    }
                }
                if (dt.Rows.Count > 0)
                {
                    string RowTotalCaseLast = "," + "," + "," + "," + "," + "," + "Total By Case" + "," + "," + "," + totalCaseQTY + "," + "," + totalCaseAMT + "," + totalCaseNW + "," + totalCaseM3 + "," + totalCaseGW;
                    sb.AppendLine(RowTotalCaseLast);
                    totalConQTY = totalConQTY + totalCaseQTY;
                    totalConAMT = totalConAMT + totalCaseAMT;
                    totalConNW = totalConNW + totalCaseNW;
                    totalConM3 = totalConM3 + totalCaseM3;
                    totalConGW = totalConGW + totalCaseGW;
                    totalCaseQTY = 0;
                    totalCaseAMT = 0;
                    totalCaseNW = 0;
                    totalCaseM3 = 0;
                    totalCaseGW = 0;

                    string RowTotalContainerLast = "," + "," + "," + "," + "," + "," + "Total By Container" + "," + "," + "," + totalConQTY + "," + "," + totalConAMT + "," + totalConNW + "," + totalConM3 + "," + totalConGW;
                    sb.AppendLine(RowTotalContainerLast);
                    totalConQTY = 0;
                    totalConAMT = 0;
                    totalConNW = 0;
                    totalConM3 = 0;
                    totalConGW = 0;
                }
                File.WriteAllText(gFileLocation, sb.ToString());

                Succes = true;
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
            }

            if (Succes)
            {
                return gReturnLocation;
            }
            else
            {

                return "";
            }
        }

        [HttpPost, Route("SNSE044N_Paperless")]
        [Authorize]
        public IHttpActionResult SNSE044N_Paperless([FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE044N_INS_PAPERLESS";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE_FROM", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE_FROM;
                cmd.Parameters.Add("P_CUSTOMER_CODE_TO", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE_TO;
                cmd.Parameters.Add("P_DOC_NO_FROM", OracleDbType.Varchar2).Value = pData.DOC_NO_FROM;
                cmd.Parameters.Add("P_DOC_NO_TO", OracleDbType.Varchar2).Value = pData.DOC_NO_TO;
                cmd.Parameters.Add("P_INVOICE_DATE_FROM", OracleDbType.Varchar2).Value = pData.INVOICE_DATE_FROM.ToString("dd-MM-yyyy");
                cmd.Parameters.Add("P_DOCUMENT_DATE_TO", OracleDbType.Varchar2).Value = pData.INVOICE_DATE_TO.ToString("dd-MM-yyyy");
                cmd.Parameters.Add("P_DOC_TYPE", OracleDbType.Varchar2).Value = pData.DOC_TYPE;
                cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = pData.SENT_BY;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE044N_Paperless");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE044N_Dragon")]
        [Authorize]
        public IHttpActionResult SNSE044N_Dragon([FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE044N_INS_DRAGON";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE_FROM", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE_FROM;
                cmd.Parameters.Add("P_CUSTOMER_CODE_TO", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE_TO;
                cmd.Parameters.Add("P_DOC_NO_FROM", OracleDbType.Varchar2).Value = pData.DOC_NO_FROM;
                cmd.Parameters.Add("P_DOC_NO_TO", OracleDbType.Varchar2).Value = pData.DOC_NO_TO;
                cmd.Parameters.Add("P_INVOICE_DATE_FROM", OracleDbType.Varchar2).Value = pData.INVOICE_DATE_FROM.ToString("dd-MM-yyyy");
                cmd.Parameters.Add("P_DOCUMENT_DATE_TO", OracleDbType.Varchar2).Value = pData.INVOICE_DATE_TO.ToString("dd-MM-yyyy");
                cmd.Parameters.Add("P_DOC_TYPE", OracleDbType.Varchar2).Value = pData.DOC_TYPE;
                cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = pData.SENT_BY;
                cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE044N_Dragon");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpPost, Route("SNSE044N_Dragon_Generate")]
        [Authorize]
        public IHttpActionResult SNSE044N_Dragon_Generate([FromBody] dynamic pData)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSE044N_INS_DRAGON";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_CUSTOMER_CODE_FROM", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE_FROM;
            cmd.Parameters.Add("P_CUSTOMER_CODE_TO", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE_TO;
            cmd.Parameters.Add("P_DOC_NO_FROM", OracleDbType.Varchar2).Value = pData.DOC_NO_FROM;
            cmd.Parameters.Add("P_DOC_NO_TO", OracleDbType.Varchar2).Value = pData.DOC_NO_TO;
            cmd.Parameters.Add("P_INVOICE_DATE_FROM", OracleDbType.Varchar2).Value = pData.INVOICE_DATE_FROM.ToString("dd-MM-yyyy");
            cmd.Parameters.Add("P_DOCUMENT_DATE_TO", OracleDbType.Varchar2).Value = pData.INVOICE_DATE_TO.ToString("dd-MM-yyyy");
            cmd.Parameters.Add("P_DOC_TYPE", OracleDbType.Varchar2).Value = pData.DOC_TYPE;
            cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = pData.SENT_BY;
            cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
            cmd.Parameters.Add("C_DRAGON_HEADER", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
            cmd.Parameters.Add("C_DRAGON_DETAIL", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            DataTable dtProcessStatus = ds.Tables[0];

            if (dtProcessStatus.Rows[0]["STATUS"].ToString() == "DONE")
            {
                //Set data table for generate order file
                DataTable dtDragonFile = ds.Tables[1];
                DataTable dtDragonData = ds.Tables[2];

                string gFileDetailName = dtDragonFile.Rows[0]["FILE_NAME"].ToString();

                string gCategory = "SNSE044N/DRG";

                //Write and get file path.
                string gDetailPath = DataService.WriteFile(dtDragonData, "TEXT", gCategory, gFileDetailName);

                var fileDetail = DataService.GetFileStream(gDetailPath, gFileDetailName);

                return Ok(new { Detail = new { filename = gFileDetailName, file = fileDetail } });
            }
            else
            {
                string errString = dtProcessStatus.Rows[0]["MESSAGE"].ToString();
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, errString));
            }
        }


        #endregion

        #region SNSQ025_001

        [HttpPost, Route("SNSQ025_001_GET_LIST")]
        [Authorize]
        public IHttpActionResult SNSQ025_001_GET_LIST([FromBody] dynamic pData)
        {
            try
            {

                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ025_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = pData.ORDER_PART_NO;
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData.VENDOR_CODE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ025_001_GET_LIST");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNSQ025_GetVendor/{VendorCode=null}/{NoOfShowRecords=null}")]
        [Authorize]
        public IHttpActionResult SNSQ025_GetVendor(string VendorCode = null, string NoOfShowRecords = null)
        {
            try
            {
                DataSet ds = new DataSet();
                VendorCode = ismoUtil.escapeCharacter(VendorCode);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ025_GET_VENDOR";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = VendorCode != "null" ? VendorCode : "";
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = NoOfShowRecords;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ025_GetVendor");
                return Json(new { Status = "Error" });

            }
        }

        [HttpGet, Route("SNSQ025_GetPartNo/{OrderPartNo=null}/{NoOfShowRecords=null}")]
        [Authorize]
        public IHttpActionResult SNSQ025_GetPartNo(string OrderPartNo = null, string NoOfShowRecords = null)
        {
            try
            {
                DataSet ds = new DataSet();
                OrderPartNo = ismoUtil.escapeCharacter(OrderPartNo);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ025_GET_ORDER_PART";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = OrderPartNo.ToUpper() == "NULL" ? (object)DBNull.Value : OrderPartNo;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = NoOfShowRecords;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ025_GetPartNo");
                return Json(new { Status = "Error" });

            }
        }

        [HttpPost, Route("SNSQ025_Export_Excle")]
        public HttpResponseMessage SNSQ025_Export_Excle(List<DataModel.SNSQ025_EXPORT> dModel)
        {

            DataTable dt = new DataTable();
            string x = string.Empty;
            Console.WriteLine();
            string filename = "SNSQ025.csv";
            string floderLocation = "SNSQ025";

            ///get date of week
            string[] date_show = new string[8];
            for (var i = 0; i < 8; i++)
            {

                var Date_curr = DateTime.Today.AddDays(i);
                var Date_week = Date_curr.DayOfWeek;
                var Day = Date_week + " " + Date_curr.ToString("dd-MM-yyyy");

                date_show[i] = Day;
            }


            dt.Columns.Add("colum1");

            dt.Rows.Add("ID;PO2C");
            dt.Rows.Add("B");
            dt.Rows.Add("C;X1;Y1;K\"Seq.\"");
            dt.Rows.Add("C;X2;Y1;K\"Part No\"");
            dt.Rows.Add("C;X3;Y1;K\"Part Name\"");
            dt.Rows.Add("C;X4;Y1;K\"Staff Inchange\"");
            dt.Rows.Add("C;X5;Y1;K\"Vender Code\"");
            dt.Rows.Add("C;X6;Y1;K\"Model\"");
            dt.Rows.Add("C;X7;Y1;K\"BO Qty (Domestic)\"");
            dt.Rows.Add("C;X8;Y1;K\"BO Line (Domestic)\"");
            dt.Rows.Add("C;X9;Y1;K\"BO Qty (Export)\"");
            dt.Rows.Add("C;X10;Y1;K\"BO Line (Export)\"");
            dt.Rows.Add("C;X11;Y1;K\"Sum Bo Qty\"");
            dt.Rows.Add("C;X12;Y1;K\"Sum Bo Line\"");
            dt.Rows.Add("C;X13;Y1;K\"" + date_show[0] + "\"");
            dt.Rows.Add("C;X14;Y1;K\"" + date_show[1] + "\"");
            dt.Rows.Add("C;X15;Y1;K\"" + date_show[2] + "\"");
            dt.Rows.Add("C;X16;Y1;K\"" + date_show[3] + "\"");
            dt.Rows.Add("C;X17;Y1;K\"" + date_show[4] + "\"");
            dt.Rows.Add("C;X18;Y1;K\"" + date_show[5] + "\"");
            dt.Rows.Add("C;X19;Y1;K\"" + date_show[6] + "\"");
            dt.Rows.Add("C;X20;Y1;K\"" + date_show[7] + "\"");
            dt.Rows.Add("C;X21;Y1;K\"Sum Delivery Qty\"");

            int countrow = 0;
            int itemno = 0;
            for (var i = 0; i < dModel.Count; i++)
            {
                countrow = i + 2;
                itemno = i + 1;
                dt.Rows.Add("C;X1;Y" + countrow + ";K" + '"' + dModel[i].SEQ + '"');
                dt.Rows.Add("C;X2;Y" + countrow + ";K" + '"' + dModel[i].ORDER_PART_NO + '"');
                dt.Rows.Add("C;X3;Y" + countrow + ";K" + '"' + dModel[i].PART_NAME_EN + '"');
                dt.Rows.Add("C;X4;Y" + countrow + ";K" + '"' + dModel[i].VENDOR_STAFF + '"');
                dt.Rows.Add("C;X5;Y" + countrow + ";K" + '"' + dModel[i].VENDOR_CODE + '"');
                dt.Rows.Add("C;X6;Y" + countrow + ";K" + '"' + dModel[i].MAIN_MODEL + '"');
                dt.Rows.Add("C;X7;Y" + countrow + ";K" + '"' + dModel[i].SUM_QTY_D + '"');
                dt.Rows.Add("C;X8;Y" + countrow + ";K" + '"' + dModel[i].SUM_LINE_D + '"');
                dt.Rows.Add("C;X9;Y" + countrow + ";K" + '"' + dModel[i].SUM_QTY_E + '"');
                dt.Rows.Add("C;X10;Y" + countrow + ";K" + '"' + dModel[i].SUM_LINE_E + '"');
                dt.Rows.Add("C;X11;Y" + countrow + ";K" + '"' + dModel[i].SUM_BO_QTY + '"');
                dt.Rows.Add("C;X12;Y" + countrow + ";K" + '"' + dModel[i].SUM_BO_LINE + '"');
                dt.Rows.Add("C;X13;Y" + countrow + ";K" + '"' + dModel[i].DEL_QTY_1 + '"');
                dt.Rows.Add("C;X14;Y" + countrow + ";K" + '"' + dModel[i].DEL_QTY_2 + '"');
                dt.Rows.Add("C;X15;Y" + countrow + ";K" + '"' + dModel[i].DEL_QTY_3 + '"');
                dt.Rows.Add("C;X16;Y" + countrow + ";K" + '"' + dModel[i].DEL_QTY_4 + '"');
                dt.Rows.Add("C;X17;Y" + countrow + ";K" + '"' + dModel[i].DEL_QTY_5 + '"');
                dt.Rows.Add("C;X18;Y" + countrow + ";K" + '"' + dModel[i].DEL_QTY_6 + '"');
                dt.Rows.Add("C;X19;Y" + countrow + ";K" + '"' + dModel[i].DEL_QTY_7 + '"');
                dt.Rows.Add("C;X20;Y" + countrow + ";K" + '"' + dModel[i].DEL_QTY_8 + '"');
                dt.Rows.Add("C;X21;Y" + countrow + ";K" + '"' + dModel[i].SUM_DEL_QTY + '"');
            }

            dt.Rows.Add("E");

            x = DataService.WriteCSVNoColumn(dt, filename, floderLocation);
            Controllers.SystemApiController response = new Controllers.SystemApiController();
            var File = response.downloadFile(x.Substring(1), filename);
            return File;
        }

        #endregion

        #region SNSQ092

        [HttpPost, Route("SNSQ092_GET_LIST")]
        [Authorize]
        public IHttpActionResult SNSQ092_GET_LIST([FromBody] dynamic pData)
        {

            try
            {

                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ092_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_FROM_DATE", OracleDbType.Date).Value = pData.FROM_ORDER_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(pData.FROM_ORDER_DATE);
                cmd.Parameters.Add("P_TO_DATE", OracleDbType.Date).Value = pData.ORDER_DATE_TO == null ? (object)DBNull.Value : Convert.ToDateTime(pData.ORDER_DATE_TO);
                cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = pData.ORDER_PART_NO;
                cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = pData.ORDER_CLASS;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pData.ORDER_NO;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_CHECK_QTY", OracleDbType.Varchar2).Value = pData.CHECK_QTY;
                cmd.Parameters.Add("P_LOCK_SALES", OracleDbType.Varchar2).Value = pData.LOCK_SALES;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ092_GET_LIST");
                return Json(new { Status = "Error" });
            }
        }


        [HttpGet, Route("SNSQ092_GetPartNo/{PartNo=null}/{NoOfShowRecords=null}")]
        [Authorize]
        public IHttpActionResult SNSQ092_GetPartNo(string PartNo = null, string NoOfShowRecords = null)
        {
            try
            {
                DataSet ds = new DataSet();
                PartNo = ismoUtil.escapeCharacter(PartNo);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ092_PART_NO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = PartNo.ToUpper() == "NULL" ? (object)DBNull.Value : PartNo;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = NoOfShowRecords;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ092_GetPartNo");
                return Json(new { Status = "Error" });

            }
        }

        [HttpGet, Route("SNSQ092_GetCustomerCode/{CustomerCode=null}/{NoOfShowRecords=null}")]
        [Authorize]
        public IHttpActionResult SNSQ092_GetCustomerCode(string CustomerCode = null, string NoOfShowRecords = null)
        {
            try
            {
                DataSet ds = new DataSet();
                CustomerCode = ismoUtil.escapeCharacter(CustomerCode);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ092_CUSTOMER_CODE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = CustomerCode.ToUpper() == "NULL" ? (object)DBNull.Value : CustomerCode;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = NoOfShowRecords;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ092_GetCustomerCode");
                return Json(new { Status = "Error" });

            }
        }

        [HttpGet, Route("SNSQ092_GetOrderNo/{OrderNo=null}/{NoOfShowRecords=null}")]
        [Authorize]
        public IHttpActionResult SNSQ092_GetOrderNo(string OrderNo = null, string NoOfShowRecords = null)
        {
            try
            {
                DataSet ds = new DataSet();
                OrderNo = ismoUtil.escapeCharacter(OrderNo);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ092_ORDER_NO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = OrderNo.ToUpper() == "NULL" ? (object)DBNull.Value : OrderNo;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = NoOfShowRecords;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ092_GetOrderNo");
                return Json(new { Status = "Error" });

            }
        }

        [HttpPost, Route("SNSQ092_Export_Excle")]
        public HttpResponseMessage SNSQ092_Export_Excle(List<DataModel.SNSQ092_EXPORT> Data)
        {

            HttpResponseMessage response = new HttpResponseMessage(HttpStatusCode.OK);
            string gLocation = "~/tempFile/" + "SNSQ092" + "/";
            string gWriteLocation = System.Web.HttpContext.Current.Server.MapPath(gLocation);
            FileInfo filePath = new FileInfo(gWriteLocation);
            filePath.Directory.Create();
            string gFileLocation = Path.Combine(gWriteLocation, "SNSQ092.xlsx");
            string gReturnLocation = Path.Combine(gLocation, "SNSQ092.xlsx");

            var excel2 = Data.ToExcel(x => x
            .AddColumn("Seq.", z => z.SEQ)
            .AddColumn("Part No", z => z.PART_NO)
            .AddColumn("Part Name EN", z => z.PART_NAME_EN)
            .AddColumn("Part Name Thai", z => z.PART_NAME_TH)
            .AddColumn("Order Type", z => z.OVERSEA_FLAG_C)
            .AddColumn("Order Date", z => Convert.ToDateTime(z.ORDER_DATE).ToString("dd-MMM-yyyy"))
            .AddColumn("Customer Code", z => z.CUSTOMER_CODE)
            .AddColumn("Order Qty", z => z.ORDER_QTY)
            .AddColumn("Accept Qty", z => z.ACCEPT_QTY)
            .AddColumn("Order No", z => z.ORDER_NO)
            .AddColumn("Item No", z => z.ITEM_NO)
            .AddColumn("Sub Item", z => z.SUB_ITEM_NO)
            .AddColumn("Order Class", z => z.ORDER_CLASS)
            .AddColumn("Reason Code", z => z.REASON_CODE)
            .AddColumn("Lock Sales", z => z.LOCK_SALES)
            .AddColumn("Total OH", z => z.TOTAL_OH_QTY)
            .AddColumn("OH Qty", z => z.OH_QTY)
            .AddColumn("Indent Qty", z => z.INDENT_QTY)
            .AddColumn("Allocate Qty", z => z.ALLOCATE_QTY)
            .AddColumn("Transfer Qty", z => z.TRANSFER_QTY)
            .AddColumn("Suspend Qty", z => z.SUSPEND_QTY)
            );

            byte[] excel = excel2;
            //byte[] excel = Data.ToExcel();
            File.WriteAllBytes(gFileLocation, excel);
            string path = HttpContext.Current.Server.MapPath("~/" + gReturnLocation.Substring(1));
            if (!File.Exists(path))
            {
                return Request.CreateErrorResponse(HttpStatusCode.NotFound, "File not found.");
            }
            try
            {
                string mimeType = MimeMapping.GetMimeMapping("SNSQ092.xlsx");
                StreamContent x = new StreamContent(new FileStream(path, FileMode.Open, FileAccess.Read));
                response.Content = x;
                response.Content.Headers.ContentType = new MediaTypeHeaderValue(mimeType);
                response.Content.Headers.ContentDisposition = new ContentDispositionHeaderValue("attachment");
                response.Content.Headers.ContentDisposition.FileName = "SNSQ092.xlsx";
            }
            catch (Exception ex)
            {
                return Request.CreateResponse(HttpStatusCode.BadRequest, ex);
            }
            return response;
        }

        #endregion

        #region WEEKLYREP

        [HttpGet, Route("WEEKLYREP/{currentDate}")]
        public async Task<IHttpActionResult> WEEKLYREP(string currentDate)
        {

            //Get querystring
            var queryStrings = Request.GetQueryNameValuePairs();
            string dateLocation = Convert.ToDateTime(currentDate).ToString("yyyyMMdd");

            var baseLocation = String.Concat(ConfigurationManager.AppSettings["DefaultReportOutputPath"], dateLocation, "/");

            //Get report list
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_WEEKLY_REPORT_LIST";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = "01";
            cmd.Parameters.Add("P_PROCESS_DATE", OracleDbType.Date).Value = Convert.ToDateTime(currentDate);
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
            ds = oOracledb.SqlQueryDS_CMD(cmd);

            DataTable reportTable = ds.Tables[0];
            foreach (DataRow reportRow in reportTable.Rows)
            {
                var reportParams = new List<ParameterValue>();
                var reportName = reportRow["REPORT_NAME"].ToString();

                //Set output location & filename
                var confFileLocation = reportRow["FILE_LOCATION"].ToString();
                var fileLocation = confFileLocation.Substring(0, confFileLocation.LastIndexOf("/"));
                var outputLocation = String.Concat(baseLocation, fileLocation, "/");
                var fileName = confFileLocation.Substring(confFileLocation.LastIndexOf("/"));
                var paramList = reportRow["PARAMETERS"].ToString().Split('&');

                //Set report type EXCELOPENXML or PDF
                var fileType = reportRow["FILE_TYPE"].ToString();

                //Create report parameters
                foreach (var param in paramList)
                {
                    var p = param.Split('=');
                    reportParams.Add(new ParameterValue
                    {
                        Name = p[0],
                        Value = p[1],
                        Label = p[0]
                    });
                }

                try
                {
                    //Generate report
                    await ReportServices.GenerateReport(reportName, fileType, reportParams, fileName, outputLocation);

                    odb.WriteLog("WEEKLYREP", "Pass", reportName);
                }
                catch (Exception ex)
                {
                    odb.WriteLog("WEEKLYREP", ex.Message, reportName);
                    //return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
                }
            }

            return Ok(baseLocation);
        }


        #endregion

        #region SNSR063F_A4

        [HttpGet, Route("SNSR063F_A4_GetDocStart/{DocumentNoStart=null}/{NoOfShowRecords=null}")]
        [Authorize]
        public IHttpActionResult SNSR063F_A4_GetDocStart(string DocumentNoStart = null, string NoOfShowRecords = null)
        {
            try
            {
                DataSet ds = new DataSet();
                DocumentNoStart = ismoUtil.escapeCharacter(DocumentNoStart);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSR063F_GET_DOCSTART";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DOCUMENT_NO_START", OracleDbType.Varchar2).Value = DocumentNoStart.ToUpper() == "NULL" ? (object)DBNull.Value : DocumentNoStart;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = NoOfShowRecords;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSR063F_A4_GetDocStart");
                return Json(new { Status = "Error" });

            }
        }

        [HttpGet, Route("SNSR063F_A4_GetDocEnd/{DocumentNoEnd=null}/{NoOfShowRecords=null}")]
        [Authorize]
        public IHttpActionResult SNSR063F_A4_GetDocEnd(string DocumentNoEnd = null, string NoOfShowRecords = null)
        {
            try
            {
                DataSet ds = new DataSet();
                DocumentNoEnd = ismoUtil.escapeCharacter(DocumentNoEnd);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSR063F_GET_DOCEND";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DOCUMENT_NO_END", OracleDbType.Varchar2).Value = DocumentNoEnd.ToUpper() == "NULL" ? (object)DBNull.Value : DocumentNoEnd;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = NoOfShowRecords;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSR063F_A4_GetDocEnd");
                return Json(new { Status = "Error" });

            }
        }

        [HttpGet, Route("SNSR063F_A4_GetPartStart/{PartNoStart=null}/{NoOfShowRecords=null}")]
        [Authorize]
        public IHttpActionResult SNSR063F_A4_GetPartStart(string PartNoStart = null, string NoOfShowRecords = null)
        {
            try
            {
                DataSet ds = new DataSet();
                PartNoStart = ismoUtil.escapeCharacter(PartNoStart);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSR063F_GET_PART_START";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO_SATRT", OracleDbType.Varchar2).Value = PartNoStart.ToUpper() == "NULL" ? (object)DBNull.Value : PartNoStart;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = NoOfShowRecords;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSR063F_A4_GetPartStart");
                return Json(new { Status = "Error" });

            }
        }

        [HttpGet, Route("SNSR063F_A4_GetPartEnd/{PartNoEnd=null}/{NoOfShowRecords=null}")]
        [Authorize]
        public IHttpActionResult SNSR063F_A4_GetPartEnd(string PartNoEnd = null, string NoOfShowRecords = null)
        {
            try
            {
                DataSet ds = new DataSet();
                PartNoEnd = ismoUtil.escapeCharacter(PartNoEnd);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSR063F_GET_PARTNO_END";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO_END", OracleDbType.Varchar2).Value = PartNoEnd.ToUpper() == "NULL" ? (object)DBNull.Value : PartNoEnd;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = NoOfShowRecords;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSR063F_A4_GetPartEnd");
                return Json(new { Status = "Error" });

            }
        }

        [HttpGet, Route("SNSR063F_A4_GetCusStart/{CustomerStart=null}/{NoOfShowRecords=null}")]
        [Authorize]
        public IHttpActionResult SNSR063F_A4_GetCusStart(string CustomerStart = null, string NoOfShowRecords = null)
        {
            try
            {
                DataSet ds = new DataSet();
                CustomerStart = ismoUtil.escapeCharacter(CustomerStart);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSR063F_GET_CUS_START";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUS_CODE_START", OracleDbType.Varchar2).Value = CustomerStart.ToUpper() == "NULL" ? (object)DBNull.Value : CustomerStart;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = NoOfShowRecords;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSR063F_A4_GetCusStart");
                return Json(new { Status = "Error" });

            }
        }

        [HttpGet, Route("SNSR063F_A4_GetCusEnd/{CustomerEnd=null}/{NoOfShowRecords=null}")]
        [Authorize]
        public IHttpActionResult SNSR063F_A4_GetCusEnd(string CustomerEnd = null, string NoOfShowRecords = null)
        {
            try
            {
                DataSet ds = new DataSet();
                CustomerEnd = ismoUtil.escapeCharacter(CustomerEnd);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSR063F_GET_CUS_END";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUS_CODE_END", OracleDbType.Varchar2).Value = CustomerEnd.ToUpper() == "NULL" ? (object)DBNull.Value : CustomerEnd;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = NoOfShowRecords;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSR063F_A4_GetCusEnd");
                return Json(new { Status = "Error" });

            }
        }


        #endregion

        #region SNSE063R_A4
        [HttpPost, Route("SNSE063R_A4_GetDocumentStart/{documentStart}/{NoOfShowRecords}")]
        [Authorize]
        public IHttpActionResult SNSE063R_A4_GetDocumentStart(string documentStart, string NoOfShowRecords, [FromBody] dynamic param)
        {
            try
            {
                DataSet ds = new DataSet();
                documentStart = ismoUtil.escapeCharacter(documentStart);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE063R_A4_GET_DOC_S";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DOCUMENT_FROM", OracleDbType.Varchar2).Value = documentStart.ToUpper() == "NULL" ? (object)DBNull.Value : documentStart;
                cmd.Parameters.Add("P_DOCUMENT_TYPE", OracleDbType.Varchar2).Value = param.DOCUMENT_TYPE;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = NoOfShowRecords;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE063R_A4_GetDocumentStart");
                return Json(new { Status = "Error" });
            }
        }
        [HttpPost, Route("SNSE063R_A4_GetDocumentEnd/{documentEnd}/{NoOfShowRecords}")]
        [Authorize]
        public IHttpActionResult SNSE063R_A4_GetDocumentEnd(string documentEnd, string NoOfShowRecords, [FromBody] dynamic param)
        {
            try
            {
                DataSet ds = new DataSet();
                documentEnd = ismoUtil.escapeCharacter(documentEnd);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE063R_A4_GET_DOC_E";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DOCUMENT_TO", OracleDbType.Varchar2).Value = documentEnd.ToUpper() == "NULL" ? (object)DBNull.Value : documentEnd;
                cmd.Parameters.Add("P_DOCUMENT_TYPE", OracleDbType.Varchar2).Value = param.DOCUMENT_TYPE;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = NoOfShowRecords;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE063R_A4_GetDocumentEnd");
                return Json(new { Status = "Error" });
            }
        }

        [HttpPost, Route("SNSE063R_A4_SetPrintData")]
        [Authorize]
        public IHttpActionResult SNSE063R_A4_SetPrintData([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE063R_A4_SET_PRINT";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DOCUMENT_TYPE", OracleDbType.Varchar2).Value = pParam.DOCUMENT_TYPE;
                cmd.Parameters.Add("P_DOCUMENT_FROM", OracleDbType.Varchar2).Value = pParam.FROM_DOCUMENT_NO;
                cmd.Parameters.Add("P_DOCUMENT_TO", OracleDbType.Varchar2).Value = pParam.TO_DOCUMENT_NO;
                cmd.Parameters.Add("P_PRINT_FORMAT", OracleDbType.Varchar2).Value = pParam.PRINT_FORMAT;
                cmd.Parameters.Add("P_PRINT_RECORD", OracleDbType.Int32).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);
                int returnRecords;

                if (Int32.TryParse(cmd.Parameters["P_PRINT_RECORD"].Value.ToString(), out returnRecords))
                {
                    if (returnRecords > 0)
                    {
                        return Ok("Exists");
                    }
                    return Ok("No record");
                }

                return Ok("Unable to print");
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE039P_SetPrintData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        #endregion
        #region SNSE063_A4

        [HttpPost, Route("SNSE063_A4_SaveDocument")]
        [Authorize]
        public IHttpActionResult SNSE063_A4_SaveDocument([FromBody] dynamic pDetail)
        {
            int iCount = 0;
            OracleCommand cmd = new OracleCommand();


            try
            {
                //Run insert detail
                foreach (var detail in pDetail.Detail)
                {
                    iCount++;

                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE063_A4_SAVE_DT_TEMP";
                    cmd.Parameters.Add("P_ACCEPT_PART_NO", OracleDbType.Varchar2).Value = detail["PART_NO"];
                    cmd.Parameters.Add("P_ORDER_QTY", OracleDbType.Int16).Value = detail["ORDER_QTY"];
                    cmd.Parameters.Add("P_A_QTY", OracleDbType.Int16).Value = detail["A_QTY"];
                    cmd.Parameters.Add("P_UNIT_PRICE_B", OracleDbType.Double).Value = detail["UNIT_PRICE_B"];
                    cmd.Parameters.Add("P_RETAIL_PRICE", OracleDbType.Double).Value = detail["RETAIL_PRICE"];
                    cmd.Parameters.Add("P_AMOUNT", OracleDbType.Double).Value = detail["AMOUNT"];
                    cmd.Parameters.Add("P_DISCOUNT_RATE1", OracleDbType.Double).Value = detail["DISCOUNT_RATE1"];
                    cmd.Parameters.Add("P_DISCOUNT_RATE2", OracleDbType.Double).Value = detail["DISCOUNT_RATE2"];
                    cmd.Parameters.Add("P_DISCOUNT_RATE3", OracleDbType.Double).Value = detail["DISCOUNT_RATE3"];
                    cmd.Parameters.Add("P_FLAG_CHAIN", OracleDbType.Varchar2).Value = detail["FLAG_CHAIN"];
                    cmd.Parameters.Add("P_PART_NAME", OracleDbType.Varchar2).Value = detail["PART_NAME"];

                    cmd.Parameters.Add("P_CLEAR_DATA", OracleDbType.Varchar2).Value = iCount == 1 ? "Y" : "N";
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                    oOracledb.SqlQueryReturnValue(cmd);

                }

                //Insert transaction
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE063_A4_SAVE_TRAN";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pDetail["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_CUSTOMER_NAME", OracleDbType.Varchar2).Value = pDetail["CUSTOMER_NAME"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pDetail["BRANCH_CODE"];

                cmd.Parameters.Add("P_CUST_ADDR1", OracleDbType.Varchar2).Value = pDetail["CUST_ADDR1"];
                cmd.Parameters.Add("P_CUST_ADDR2", OracleDbType.Varchar2).Value = pDetail["CUST_ADDR2"];

                cmd.Parameters.Add("P_CUSTOMER_CODE_ST", OracleDbType.Varchar2).Value = pDetail["CUSTOMER_CODE_ST"];
                cmd.Parameters.Add("P_CUSTOMER_NAME_ST", OracleDbType.Varchar2).Value = pDetail["CUSTOMER_NAME_ST"];
                cmd.Parameters.Add("P_BRANCH_CODE_ST", OracleDbType.Varchar2).Value = pDetail["BRANCH_CODE_ST"];

                cmd.Parameters.Add("P_CUST_ADDR1_ST", OracleDbType.Varchar2).Value = pDetail["CUST_ADDR1_ST"];
                cmd.Parameters.Add("P_CUST_ADDR2_ST", OracleDbType.Varchar2).Value = pDetail["CUST_ADDR2_ST"];
                cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pDetail["TEMP_ORDER_TYPE"];
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pDetail["ORDER_NO"];
                cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = pDetail["ORDER_CLASS"];
                cmd.Parameters.Add("P_ORDER_DATE", OracleDbType.Date).Value = Convert.ToDateTime(pDetail["ORDER_DATE"]);
                cmd.Parameters.Add("P_VAT_FLAG", OracleDbType.Varchar2).Value = pDetail["VAT_FLAG"];
                cmd.Parameters.Add("P_VAT_RATE", OracleDbType.Varchar2).Value = pDetail["VAT_RATE"];
                cmd.Parameters.Add("P_DELIVERY_CODE", OracleDbType.Varchar2).Value = pDetail["DELIVERY_CODE"];
                cmd.Parameters.Add("P_ROUTE_CODE", OracleDbType.Varchar2).Value = pDetail["ROUTE_CODE"];
                cmd.Parameters.Add("P_TRANSPORT_CODE", OracleDbType.Varchar2).Value = pDetail["TRANSPORT_CODE"];
                cmd.Parameters.Add("P_REF_OTHER_DOC", OracleDbType.Varchar2).Value = pDetail["REF_OTHER_DOC"];
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_DOCUMENT_NO", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string documentNo = cmd.Parameters["R_DOCUMENT_NO"].Value.ToString();
                //string documentNo = string.Empty;
                return Json(new { DOCUMENT_NO = documentNo });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE063_A4_SaveDocument");
                Console.WriteLine(ex);
            }


            return ResponseMessage(Request.CreateResponse(HttpStatusCode.BadRequest, "No row to process."));

        }

        [HttpGet, Route("SNSE063A4_FindUnitPrice/{part_no}/{doc_date}/{customer_code}")]
        [Authorize]
        public IHttpActionResult SNSE063A4_FindUnitPrice(string part_no, string doc_date, string customer_code)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE063_A4_CUST_CODE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = part_no;
                cmd.Parameters.Add("P_DOCUMENT_DATE", OracleDbType.Varchar2).Value = doc_date;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = customer_code;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE063A4_FindUnitPrice");
                return Json(new { Status = "Error" });

            }
        }

        [HttpGet, Route("SNSE063A4_GetCuscode/{CustomerCodeBT=null}/{NoOfShowRecords=null}")]
        [Authorize]
        public IHttpActionResult SNSE063A4_GetCuscode(string CustomerCodeBT = null, string NoOfShowRecords = null)
        {
            try
            {
                DataSet ds = new DataSet();
                CustomerCodeBT = ismoUtil.escapeCharacter(CustomerCodeBT);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE063_A4_CUST_CODE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = CustomerCodeBT.ToUpper() == "NULL" ? (object)DBNull.Value : CustomerCodeBT;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = NoOfShowRecords;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE063A4_GetCuscode");
                return Json(new { Status = "Error" });

            }
        }


        [HttpGet, Route("SNSE063A4_GetOrderSearch/{order}/{NoOfShowRecords}")]
        [Authorize]
        public IHttpActionResult SNSE063A4_GetOrderSearch(string order, string NoOfShowRecords)
        {
            try
            {
                DataSet ds = new DataSet();
                order = ismoUtil.escapeCharacter(order);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE063_A4_ORDER_S";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ORDER", OracleDbType.Varchar2).Value = order.ToUpper() == "NULL" ? (object)DBNull.Value : order;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = NoOfShowRecords;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE063A4_GetOrderSearch");
                return Json(new { Status = "Error" });

            }
        }

        [HttpPost, Route("SNSE063A4_GetBranchCode/{BranchCode}/{NoOfShowRecords}")]
        [Authorize]
        public IHttpActionResult SNSE063A4_GetBranchCode(string BranchCode, int NoOfShowRecords, [FromBody] dynamic pCustomer)
        {
            DataSet ds = new DataSet();

            BranchCode = ismoUtil.escapeCharacter(BranchCode);
            var pData = pCustomer["DATA"];

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE063_A4_BRANCH_CODE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = BranchCode.ToUpper() == "NULL" ? (object)DBNull.Value : BranchCode;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = NoOfShowRecords;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE063A4_GetBranchCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE063A4_GetShipToCusCode/{CustomerCodeST}/{NoOfShowRecords}")]
        [Authorize]
        public IHttpActionResult SNSE063A4_GetShipToCusCode(string CustomerCodeST, int NoOfShowRecords, [FromBody] dynamic pCustomer)
        {
            DataSet ds = new DataSet();

            CustomerCodeST = ismoUtil.escapeCharacter(CustomerCodeST);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE063_A4_BRANCH_CODE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = CustomerCodeST;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pCustomer["BRANCH_CODE"];
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = NoOfShowRecords;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE063A4_GetShipToCusCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE063A4_GetDocList/{DocumentNo}/{NoOfShowRecords}")]
        [Authorize]
        public IHttpActionResult SNSE063A4_GetDocList(string DocumentNo, string NoOfShowRecords, [FromBody] dynamic pParams)
        {
            try
            {
                DataSet ds = new DataSet();
                DocumentNo = ismoUtil.escapeCharacter(DocumentNo);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE063_A4_DOC_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DOCUMENT_NO", OracleDbType.Varchar2).Value = DocumentNo.ToUpper() == "NULL" ? (object)DBNull.Value : DocumentNo;
                cmd.Parameters.Add("P_DOCUMENT_DATE", OracleDbType.Date).Value = pParams.DOCUMENT_DATE;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = NoOfShowRecords;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE063A4_GetDocList");
                return Json(new { Status = "Error" });

            }
        }

        [HttpGet, Route("SNSE063_A4_GETLIST/{customerCode}/{branchCode}/{orderNo}/{orderDate}")]
        [Authorize]
        public IHttpActionResult SNSE063_A4_GETLIST(string customerCode, string branchCode, string orderNo, string orderDate)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE063_A4_GETLIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = customerCode.ToUpper() == "NULL" ? (object)DBNull.Value : customerCode;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = branchCode.ToUpper() == "NULL" ? (object)DBNull.Value : branchCode;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = orderNo.ToUpper() == "NULL" ? (object)DBNull.Value : orderNo;
                cmd.Parameters.Add("P_ORDER_DATE", OracleDbType.Date).Value = orderDate.ToUpper() == "NULL" ? (object)DBNull.Value : Convert.ToDateTime(orderDate);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE063_A4_GETLIST");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNSE063A4_GetCustomerCodeList/{CustomerCode=null}/{NoOfShowRecords=null}")]
        [Authorize]
        public IHttpActionResult SNSE063A4_GetCustomerCodeList(string CustomerCode = null, string NoOfShowRecords = null)
        {
            try
            {
                DataSet ds = new DataSet();
                CustomerCode = ismoUtil.escapeCharacter(CustomerCode);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE063_A4_CUSCODE_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = CustomerCode.ToUpper() == "NULL" ? (object)DBNull.Value : CustomerCode;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = NoOfShowRecords;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE063A4_GetCustomerCodeList");
                return Json(new { Status = "Error" });

            }
        }

        [HttpGet, Route("SNSE063A4_GetOrderNoList/{OrderNo=null}/{NoOfShowRecords=null}")]
        [Authorize]
        public IHttpActionResult SNSE063A4_GetOrderNoList(string OrderNo = null, string NoOfShowRecords = null)
        {
            try
            {
                DataSet ds = new DataSet();
                OrderNo = ismoUtil.escapeCharacter(OrderNo);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE063_A4_ORDER_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = OrderNo.ToUpper() == "NULL" ? (object)DBNull.Value : OrderNo;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = NoOfShowRecords;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE063A4_GetOrderNoList");
                return Json(new { Status = "Error" });

            }
        }

        [HttpGet, Route("SNSE063A4_GETPART/{PartNo=null}/{NoOfShowRecords=null}")]
        [Authorize]
        public IHttpActionResult SNSE063A4_GETPART(string PartNo = null, string NoOfShowRecords = null)
        {
            try
            {
                DataSet ds = new DataSet();
                PartNo = ismoUtil.escapeCharacter(PartNo);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE063_A4_GET_PARTNO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = PartNo.ToUpper() == "NULL" ? (object)DBNull.Value : PartNo;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = NoOfShowRecords;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE063A4_GETPART");
                return Json(new { Status = "Error" });

            }
        }

        [HttpGet, Route("SNSE063_A4_SNSC003/{customerCode}/{orderDate}/{partNo}/{orderClass}/{vendorCode}")]
        [Authorize]
        public IHttpActionResult SNSE063_A4_SNSC003(string customerCode, string orderDate, string partNo, string orderClass, string vendorCode)
        {
            decimal RETAIL_D
                , RETAIL_E
                , UNIT_D
                , UNIT_E
                , DISC_1
                , DISC_2
                , DISC_3
                , AVG_COST
                , STD_COST;
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SNSC063";
                cmd.Parameters.Add("CUST_CODE", OracleDbType.Varchar2).Value = customerCode;
                cmd.Parameters.Add("ORDER_DATE", OracleDbType.Date).Value = Convert.ToDateTime(orderDate);
                cmd.Parameters.Add("PARTNO", OracleDbType.Varchar2).Value = partNo;
                cmd.Parameters.Add("OR_CLASS", OracleDbType.Varchar2).Value = orderClass;
                cmd.Parameters.Add("VD_CODE", OracleDbType.Varchar2).Value = vendorCode;
                cmd.Parameters.Add("OVERSEAFLAG", OracleDbType.Varchar2, 1).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("RETAIL_D", OracleDbType.Decimal).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("RETAIL_E", OracleDbType.Decimal).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("UNIT_D", OracleDbType.Decimal).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("UNIT_E", OracleDbType.Decimal).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("DISC_1", OracleDbType.Decimal).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("DISC_2", OracleDbType.Decimal).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("DISC_3", OracleDbType.Decimal).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("DISC_GROUP", OracleDbType.Varchar2, 3).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("DISC_CODE", OracleDbType.Varchar2, 2).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("AVG_COST", OracleDbType.Decimal).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("STD_COST", OracleDbType.Decimal).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                return Json(new
                {
                    OVERSEAFLAG = cmd.Parameters["OVERSEAFLAG"].Value.ToString(),
                    RETAIL_D = Decimal.TryParse(cmd.Parameters["RETAIL_D"].Value.ToString(), out RETAIL_D) ? RETAIL_D : 0,
                    RETAIL_E = Decimal.TryParse(cmd.Parameters["RETAIL_E"].Value.ToString(), out RETAIL_E) ? RETAIL_E : 0,
                    UNIT_D = Decimal.TryParse(cmd.Parameters["UNIT_D"].Value.ToString(), out UNIT_D) ? UNIT_D : 0,
                    UNIT_E = Decimal.TryParse(cmd.Parameters["UNIT_E"].Value.ToString(), out UNIT_E) ? UNIT_E : 0,
                    DISC_1 = Decimal.TryParse(cmd.Parameters["DISC_1"].Value.ToString(), out DISC_1) ? DISC_1 : 0,
                    DISC_2 = Decimal.TryParse(cmd.Parameters["DISC_2"].Value.ToString(), out DISC_2) ? DISC_2 : 0,
                    DISC_3 = Decimal.TryParse(cmd.Parameters["DISC_3"].Value.ToString(), out DISC_3) ? DISC_3 : 0,
                    DISC_GROUP = cmd.Parameters["DISC_GROUP"].Value.ToString(),
                    DISC_CODE = cmd.Parameters["DISC_CODE"].Value.ToString(),
                    AVG_COST = Decimal.TryParse(cmd.Parameters["AVG_COST"].Value.ToString(), out AVG_COST) ? AVG_COST : 0,
                    STD_COST = Decimal.TryParse(cmd.Parameters["STD_COST"].Value.ToString(), out STD_COST) ? STD_COST : 0
                });

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE063_A4_SNSC003");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpGet, Route("SNSE063A4_GetRouteCode/{RouteCode=null}/{NoOfShowRecords=null}")]
        [Authorize]
        public IHttpActionResult SNSE063A4_GetRouteCode(string RouteCode = null, string NoOfShowRecords = null)
        {
            try
            {
                DataSet ds = new DataSet();
                RouteCode = ismoUtil.escapeCharacter(RouteCode);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE063_A4_ROUTECODE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ROUTE_CODE", OracleDbType.Varchar2).Value = RouteCode.ToUpper() == "NULL" ? (object)DBNull.Value : RouteCode;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = NoOfShowRecords;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE063A4_GetRouteCode");
                return Json(new { Status = "Error" });

            }
        }

        [HttpGet, Route("SNSE063A4_GetTransCode/{TransCode=null}/{NoOfShowRecords=null}")]
        [Authorize]
        public IHttpActionResult SNSE063A4_GetTransCode(string TransCode = null, string NoOfShowRecords = null)
        {
            try
            {
                DataSet ds = new DataSet();
                TransCode = ismoUtil.escapeCharacter(TransCode);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE063_A4_TRANCODE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_TRANSPORT_CODE", OracleDbType.Varchar2).Value = TransCode.ToUpper() == "NULL" ? (object)DBNull.Value : TransCode;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = NoOfShowRecords;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE063A4_GetTransCode");
                return Json(new { Status = "Error" });

            }
        }


        [HttpPost, Route("SNSE063_A4_FindUnitPrice")]
        [Authorize]
        public IHttpActionResult SNSE063_A4_FindUnitPrice([FromBody] dynamic param)
        {
            try
            {
                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE063_A4_FIND_UNIT_P";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = param.PART_NO;
                cmd.Parameters.Add("P_DOCUMENT_DATE", OracleDbType.Varchar2).Value = param.ORDER_DATE.ToString("yyyy-MM-dd");
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = param.CUSTOMER_CODE;
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = param.VENDOR_CODE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE063_A4_FindUnitPrice");
                return Json(new { Status = "Error" });

            }
        }



        [HttpPost, Route("SNSE063_A4_FIndChainPart")]
        [Authorize]
        public IHttpActionResult SNSE063_A4_FIndChainPart([FromBody] dynamic param)
        {
            decimal RETAIL_PRICE_B, AVERAGE_COST, UNIT_PRICE_B, STANDARD_COST;
            Int16 B_OH_QTY_CHAIN, LOCK_SALES;
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE063_A4_FIND_CHAIN";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = param.PART_NO;
                cmd.Parameters.Add("P_DOCUMENT_DATE", OracleDbType.Date).Value = Convert.ToDateTime(param.DOCUMENT_DATE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = param.CUSTOMER_CODE;
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = param.VENDOR_CODE;
                cmd.Parameters.Add("R_RETAIL_PRICE_B", OracleDbType.Decimal).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_AVERAGE_COST", OracleDbType.Decimal).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_UNIT_PRICE_B", OracleDbType.Decimal).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_STANDARD_COST", OracleDbType.Decimal).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_DISCOUNT_GROUP", OracleDbType.Varchar2, 10).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_DISCOUNT_CODE", OracleDbType.Varchar2, 10).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_OVERSEA_FLAG", OracleDbType.Varchar2, 10).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_B_OH_QTY_CHAIN", OracleDbType.Int16).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_FLAG_CHAIN", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_NEW_PART", OracleDbType.Varchar2, 100).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_LOCK_SALES", OracleDbType.Varchar2, 100).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("P_COUNT", OracleDbType.Int16).Value = 0;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                //RETAIL_PRICE_B = Decimal.TryParse(cmd.Parameters["R_RETAIL_PRICE_B"].Value.ToString(), out RETAIL_PRICE_B) ? RETAIL_PRICE_B : 0;
                //AVERAGE_COST = Decimal.TryParse(cmd.Parameters["R_AVERAGE_COST"].Value.ToString(), out AVERAGE_COST) ? AVERAGE_COST : 0;
                //UNIT_PRICE_B = Decimal.TryParse(cmd.Parameters["R_UNIT_PRICE_B"].Value.ToString(), out UNIT_PRICE_B) ? UNIT_PRICE_B : 0;
                //STANDARD_COST = Decimal.TryParse(cmd.Parameters["R_STANDARD_COST"].Value.ToString(), out STANDARD_COST) ? STANDARD_COST : 0;
                //B_OH_QTY_CHAIN = Int16.TryParse(cmd.Parameters["B_OH_QTY_CHAIN"].Value.ToString(), out B_OH_QTY_CHAIN) ? B_OH_QTY_CHAIN : 0;

                return Json(new
                {

                    RETAIL_PRICE_B = Decimal.TryParse(cmd.Parameters["R_RETAIL_PRICE_B"].Value.ToString(), out RETAIL_PRICE_B) ? RETAIL_PRICE_B : 0,
                    AVERAGE_COST = Decimal.TryParse(cmd.Parameters["R_AVERAGE_COST"].Value.ToString(), out AVERAGE_COST) ? AVERAGE_COST : 0,
                    UNIT_PRICE_B = Decimal.TryParse(cmd.Parameters["R_UNIT_PRICE_B"].Value.ToString(), out UNIT_PRICE_B) ? UNIT_PRICE_B : 0,
                    STANDARD_COST = Decimal.TryParse(cmd.Parameters["R_STANDARD_COST"].Value.ToString(), out STANDARD_COST) ? STANDARD_COST : 0,
                    B_OH_QTY_CHAIN = Int16.TryParse(cmd.Parameters["R_B_OH_QTY_CHAIN"].Value.ToString(), out B_OH_QTY_CHAIN) ? B_OH_QTY_CHAIN : 0,
                    LOCK_SALES = Int16.TryParse(cmd.Parameters["R_LOCK_SALES"].Value.ToString(), out LOCK_SALES) ? LOCK_SALES : 0,
                    DISCOUNT_GROUP = cmd.Parameters["R_DISCOUNT_GROUP"].Value.ToString(),
                    DISCOUNT_CODE = cmd.Parameters["R_DISCOUNT_CODE"].Value.ToString(),
                    OVERSEA_FLAG = cmd.Parameters["R_OVERSEA_FLAG"].Value.ToString(),
                    FLAG_CHAIN = cmd.Parameters["R_FLAG_CHAIN"].Value.ToString(),
                    NEW_PART = cmd.Parameters["R_NEW_PART"].Value.ToString(),

                });

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE063_A4_FIndChainPart");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }


        [HttpPost, Route("SNSE063_A4_SearchData")]
        [Authorize]
        public IHttpActionResult SNSE063_A4_SearchData([FromBody] dynamic param)
        {
            try
            {
                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE063A4_DOC_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DOCUMENT_NO", OracleDbType.Varchar2).Value = param.DOCUMENT_NO;
                cmd.Parameters.Add("P_DOCUMENT_DATE", OracleDbType.Date).Value = param.DOCUMENT_DATE.ToString().ToUpper() == "NULL" ? (object)DBNull.Value : Convert.ToDateTime(param.DOCUMENT_DATE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = param.CUSTOMER_CODE;
                cmd.Parameters.Add("P_ORDER", OracleDbType.Varchar2).Value = param.ORDER_NO;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE063_A4_SearchData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));

            }
        }


        [HttpPost, Route("SNSE063_A4_GetDetail")]
        [Authorize]
        public IHttpActionResult SNSE063_A4_GetDetail([FromBody] dynamic param)
        {
            try
            {
                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE063A4_DOC_DETAIL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DOCUMENT_NO", OracleDbType.Varchar2).Value = param.DOCUMENT_NO;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE063_A4_GetDetail");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));

            }
        }



        [HttpPost, Route("SNSE063A4_CheckOrderNo")]
        [Authorize]
        public IHttpActionResult SNSE063A4_CheckOrderNo([FromBody] dynamic pParams)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE063_A4_CHECK_ORDER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_SHIPTO_CUST_CODE", OracleDbType.Varchar2).Value = pParams.CUSTOMER_CODE_ST;
                cmd.Parameters.Add("P_REF_DOC_NO", OracleDbType.Varchar2).Value = pParams.REF_DOC_NO;
                cmd.Parameters.Add("P_DOCUMENT_DATE", OracleDbType.Date).Value = pParams.ORDER_DATE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE063A4_CheckOrderNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNSE063A4_GetVatRate")]
        [Authorize]
        public IHttpActionResult SNSE063A4_GetVatRate([FromBody] dynamic pParams)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE063_A4_FIND_VAT_RATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VAT_FLAG", OracleDbType.Varchar2).Value = pParams.VAT_FLAG;
                cmd.Parameters.Add("P_DOCUMENT_DATE", OracleDbType.Date).Value = pParams.ORDER_DATE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE063A4_GetVatRate");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNSE063_A4_CheckRefOtherDoc")]
        [Authorize]
        public IHttpActionResult SNSE063_A4_CheckRefOtherDoc([FromBody] dynamic param)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE063_A4_CHECK_REF";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DOCUMENT_TYPE", OracleDbType.Varchar2).Value = "DN";
                cmd.Parameters.Add("P_REF_OTHER_DOC", OracleDbType.Varchar2).Value = param.REF_OTHER_DOC;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE063_A4_CheckRefOtherDoc");
                return Json(new { Status = "Error" });
            }
        }
        #endregion

        #region SNSE206

        [HttpGet, Route("SNSE206_GetDocNo/{CancelDocID=null}/{NoOfShowRecords=null}")]
        [Authorize]
        public IHttpActionResult SNSE206_GetDocNo(string CancelDocID = null, string NoOfShowRecords = null)
        {
            try
            {
                DataSet ds = new DataSet();
                CancelDocID = ismoUtil.escapeCharacter(CancelDocID);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE206_GET_DOC_NO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CANCEL_DOC_ID", OracleDbType.Varchar2).Value = CancelDocID.ToUpper() == "NULL" ? (object)DBNull.Value : CancelDocID;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = NoOfShowRecords;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE206_GetDocNo");
                return Json(new { Status = "Error" });

            }
        }


        [HttpGet, Route("SNSE206_GetCustCode/{CustomerCode=null}/{NoOfShowRecords=null}")]
        [Authorize]
        public IHttpActionResult SNSE206_GetCustCode(string CustomerCode = null, string NoOfShowRecords = null)
        {
            try
            {
                DataSet ds = new DataSet();
                CustomerCode = ismoUtil.escapeCharacter(CustomerCode);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE206_GET_CUST_CODE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = CustomerCode.ToUpper() == "NULL" ? (object)DBNull.Value : CustomerCode;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = NoOfShowRecords;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE206_GetCustCode");
                return Json(new { Status = "Error" });

            }
        }

        [HttpPost, Route("SNSE206_UPDATE")]
        [Authorize]
        [RunBatchGuard]
        public IHttpActionResult SNSE206_UPDATE([FromBody] dynamic[] pDataList)
        {
            DataSet ds = new DataSet();
            String gPartNo = "";
            try
            {
                foreach (var data in pDataList)
                {
                    OracleCommand cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE206_UPDATE";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = data.BRANCH_CODE;
                    cmd.Parameters.Add("P_CANCEL_DOC_ID", OracleDbType.Varchar2).Value = data.CANCEL_DOC_ID;
                    cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Varchar2).Value = data.ITEM_NO;
                    cmd.Parameters.Add("P_SUB_ITEM_NO", OracleDbType.Varchar2).Value = data.ATTRIBUTE4;
                    cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = data.ORDER_NO;
                    cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = data.ORDER_PART_NO;
                    cmd.Parameters.Add("P_ORDER_QTY", OracleDbType.Varchar2).Value = data.ORDER_QTY;
                    cmd.Parameters.Add("P_CANCEL_REASON", OracleDbType.Varchar2).Value = data.CANCEL_REASON;

                    cmd.Parameters.Add("P_REASON_CODE", OracleDbType.Varchar2).Value = data.REASON_CODE;
                    cmd.Parameters.Add("P_APPROVE_STATUS", OracleDbType.Varchar2).Value = data.APPROVE_STATUS;
                    cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE206";
                    cmd.Parameters.Add("P_LAST_UPD_BY_AUTH", OracleDbType.Varchar2).Value = data.USERNAME_AUTH;
                    cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                    gPartNo = data.ORDER_PART_NO;
                    ds = oOracledb.SqlQueryDS_CMD(cmd);

                }
                return Json(DTFM.convertToList(ds));

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE206_UPDATE");
                return Json(new { Status = "Error" });
            }
        }

        [HttpPost, Route("SNSE206_GET_DATA")]
        [Authorize]
        public IHttpActionResult SNSE206_GET_DATA([FromBody] dynamic pData)
        {

            try
            {

                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE206_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DOCUMENT_NO", OracleDbType.Varchar2).Value = pData.CANCEL_DOC_ID;
                cmd.Parameters.Add("P_ORDER_DATE", OracleDbType.Date).Value = pData.ORDER_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(pData.ORDER_DATE);
                cmd.Parameters.Add("P_CANCEL_REQUEST_DATE", OracleDbType.Date).Value = pData.CANCEL_REQUEST_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(pData.CANCEL_REQUEST_DATE);
                cmd.Parameters.Add("P_FLAG", OracleDbType.Varchar2).Value = pData.STATUS;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE206_GET_DATA");
                return Json(new { Status = "Error" });
            }
        }

        [HttpGet, Route("SNSE206_Reason_Code/{ReasonCode=null}/{NoOfShowRecords=null}")]
        [Authorize]
        public IHttpActionResult SNSE206_Reason_Code(string ReasonCode = null, string NoOfShowRecords = null)
        {
            try
            {
                DataSet ds = new DataSet();
                ReasonCode = ismoUtil.escapeCharacter(ReasonCode);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE206_REASON_CODE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_REASON_CODE", OracleDbType.Varchar2).Value = ReasonCode.ToUpper() == "NULL" ? (object)DBNull.Value : ReasonCode;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = NoOfShowRecords;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE206_Reason_Code");
                return Json(new { Status = "Error" });

            }
        }


        [HttpGet, Route("SNSE206_GetMaxApprove")]
        [Authorize]
        public IHttpActionResult SNSE206_GetMaxApprove()
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE206_GET_MAX_APPROVE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE206_GetMaxApprove");
                return Json(new { Status = "Error" });

            }
        }

        [HttpPost, Route("SNSE206_ApproveAuth")]
        [Authorize]
        public IHttpActionResult SNSE206_ApproveAuth([FromBody] dynamic pData)
        {

            try
            {

                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE206_APPROVE_AUTH";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USERNAME", OracleDbType.Varchar2).Value = pData.USERNAME;
                cmd.Parameters.Add("P_PASSWORD", OracleDbType.Varchar2).Value = pData.PASSWORD;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE206_ApproveAuth");
                return Json(new { Status = "Error" });
            }
        }

        [HttpPost, Route("SNSE206_ExportExcel")]
        [Authorize]
        public HttpResponseMessage SNSE206_ExportExcel([FromBody] dynamic Data)
        {
            IEnumerable<dynamic> FileData = Data;
            string fileName = "SNSE206.xlsx";
            HttpResponseMessage response = new HttpResponseMessage(HttpStatusCode.OK);
            var excel2 = FileData.ToExcel(x => x
            .AddColumn("Customer", z => z.CUSTOMER_CODE)
            .AddColumn("Branch", z => z.BRANCH_CODE)
            .AddColumn("Cancel No.", z => z.CANCEL_DOC_ID)
            .AddColumn("Order No", z => z.ORDER_NO)
            .AddColumn("Order Date", z => z.ORDER_DATE.ToString("dd/MMM/yyyy"))
            .AddColumn("Request Cancel", z => z.CANCEL_REQUEST_DATE != null ? z.CANCEL_REQUEST_DATE.ToString("dd/MMM/yyyy") : z.CANCEL_REQUEST_DATE)
            .AddColumn("Item", z => z.ITEM_NO == null ? z.ITEM_NO : int.Parse(z.ITEM_NO.ToString()))
            .AddColumn("Part No", z => z.ORDER_PART_NO)
            .AddColumn("Part Name", z => z.PART_NAME_TH)
            .AddColumn("Order Qty", z => z.ORDER_QTY == null ? z.ORDER_QTY : int.Parse(z.ORDER_QTY.ToString()))
            .AddColumn("Cancel Qty", z => z.CANCEL_QTY == null ? z.CANCEL_QTY : int.Parse(z.CANCEL_QTY.ToString()))
            .AddColumn("Cancel Reason", z => z.CANCEL_REASON)
            .AddColumn("Reject Reason", z => z.REASON_CODE)
            .AddColumn("Cost amount", z => z.COST == null ? z.COST : double.Parse(z.COST.ToString()))
            .AddColumn("Net Amount", z => z.NET_AMT_B == null ? z.NET_AMT_B : double.Parse(z.NET_AMT_B.ToString()))
            );
            byte[] excel = excel2;
            try
            {
                MemoryStream stream = new MemoryStream(excel);
                string mimeType = MimeMapping.GetMimeMapping(fileName);
                StreamContent x = new StreamContent(stream);
                response.Content = x;
                response.Content.Headers.ContentType = new MediaTypeHeaderValue(mimeType);
                response.Content.Headers.ContentDisposition = new ContentDispositionHeaderValue("attachment");
                response.Content.Headers.ContentDisposition.FileName = fileName;
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE206_ExportExcel");
                return Request.CreateResponse(HttpStatusCode.BadRequest, ex);
            }
            return response;
        }


        #endregion

        #region SNSE042A4
        //[HttpGet, Route("SNSE042NA4_getPickLotDate/{pickLotDate}/{NoOfShowRecords}")]
        //[Authorize]
        //public IHttpActionResult SNSE042NA4_getPickLotDate(string pickLotDate, string NoOfShowRecords)
        //{
        //    try
        //    {
        //        DataSet ds = new DataSet();
        //        pickLotDate = ismoUtil.escapeCharacter(pickLotDate);
        //        OracleCommand cmd = new OracleCommand();
        //        cmd.CommandText = "SP_NP_SNSE042NA4_PICK_LOT_DATE";
        //        cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
        //        cmd.Parameters.Add("P_PICK_LOT_DATE", OracleDbType.Varchar2).Value = pickLotDate.ToUpper() == "NULL" ? (object)DBNull.Value : pickLotDate;
        //        //cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = NoOfShowRecords;
        //        cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
        //        ds = oOracledb.SqlQueryDS_CMD(cmd);
        //        return Json(DTFM.convertToList(ds));
        //    }
        //    catch (Exception ex)
        //    {
        //        return Json(new { Status = "Error" });

        //    }
        //}

        //[HttpPost, Route("SNSE039PInvoiceSeqList/{InvoiceSeq}/{noofshowrec}")]
        //[Authorize]
        //public IHttpActionResult SNSE042NA4_GetPickLotSeq(string InvoiceSeq, int noofshowrec, [FromBody] dynamic pParam)

        [HttpPost, Route("SNSE042NA4_GetPickLotSeq/{pickLotSeq}/{NoOfShowRecords}")]
        [Authorize]
        public IHttpActionResult SNSE042NA4_GetPickLotSeq(string pickLotSeq, string NoOfShowRecords, [FromBody] dynamic pParam)
        {
            try
            {
                DataSet ds = new DataSet();
                var pData = pParam["DATA"];

                pickLotSeq = ismoUtil.escapeCharacter(pickLotSeq);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE042NA4_PICK_LOT_SEQ";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PICK_LOT_DATE", OracleDbType.Varchar2).Value = pData["PICK_LOT_DATE"].ToString("dd-MM-yyyy");
                cmd.Parameters.Add("P_PICK_LOT_SEQ", OracleDbType.Varchar2).Value = pickLotSeq.ToUpper() == "NULL" ? (object)DBNull.Value : pickLotSeq;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = NoOfShowRecords;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE042NA4_GetPickLotSeq");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));


            }
        }

        [HttpPost, Route("SNSE042NA4_CheckDataReport")]
        [Authorize]
        public IHttpActionResult SNSE042NA4_CheckDataReport([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE039R_3_A4_RPT";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_INVOICE_NO_FROM", OracleDbType.Varchar2).Value = pParam.BEGIN_INVOICE_NO;
                cmd.Parameters.Add("P_INVOICE_NO_TO", OracleDbType.Varchar2).Value = pParam.END_INVOICE_NO;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE042NA4_CheckDataReport");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE042NA4_SetPrintData")]
        [Authorize]
        public IHttpActionResult SNSE042NA4_SetPrintData([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE042NA4_SET_PRINT";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_INVOICE_NO_FROM", OracleDbType.Varchar2).Value = pParam.BEGIN_INVOICE_NO;
                cmd.Parameters.Add("P_INVOICE_NO_TO", OracleDbType.Varchar2).Value = pParam.END_INVOICE_NO;
                cmd.Parameters.Add("P_PRINT_RECORD", OracleDbType.Int32).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);
                int returnRecords;

                if (Int32.TryParse(cmd.Parameters["P_PRINT_RECORD"].Value.ToString(), out returnRecords))
                {
                    if (returnRecords > 0)
                    {
                        return Ok("Exists");
                    }
                    return Ok("No record");
                }

                return Ok("Unable to print");
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE042NA4_SetPrintData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNSE042NA4_GetInitData")]
        [Authorize]
        public IHttpActionResult SNSE042NA4_GetInitData()
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE042NA4_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE042NA4_GetInitData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        #endregion

        #region SNSQ026_001
        [HttpGet, Route("SNSQ026_001_getVendorCode/{getVendorCode}/{NoOfShowRecords}")]
        [Authorize]
        public IHttpActionResult SNSQ026_001_getVendorCode(string getVendorCode, string NoOfShowRecords)
        {
            try
            {
                DataSet ds = new DataSet();
                getVendorCode = ismoUtil.escapeCharacter(getVendorCode);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ026_001_VENDOR_CODE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = getVendorCode.ToUpper() == "NULL" ? (object)DBNull.Value : getVendorCode;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = NoOfShowRecords;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ026_001_getVendorCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));

            }
        }

        [HttpPost, Route("SNSQ026_001_getPartNo/{orderPartNo}/{NoOfShowRecords}")]
        [Authorize]
        public IHttpActionResult SNSQ026_001_getPartNo(string orderPartNo, int NoOfShowRecords, [FromBody] dynamic pVendorCode)
        {
            DataSet ds = new DataSet();

            orderPartNo = ismoUtil.escapeCharacter(orderPartNo);
            var pData = pVendorCode["DATA"];

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ026_001_PART_NO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];
                cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = orderPartNo.ToUpper() == "NULL" ? (object)DBNull.Value : orderPartNo;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = NoOfShowRecords;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ026_001_getPartNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSQ026_getCustomerCode/{customerCode}/{NoOfShowRecords}")]
        [Authorize]
        public IHttpActionResult SNSQ026_getCustomerCode(string customerCode, int NoOfShowRecords, [FromBody] dynamic pCustomerCode)
        {
            DataSet ds = new DataSet();

            customerCode = ismoUtil.escapeCharacter(customerCode);
            var pData = pCustomerCode["DATA"];
            Console.WriteLine(pData);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ026_CUSTOMER_CODE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData["VENDOR_CODE"];
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = customerCode.ToUpper() == "NULL" ? (object)DBNull.Value : customerCode;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = NoOfShowRecords;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ026_getCustomerCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSQ026_001_getSearch")]
        [Authorize]
        public IHttpActionResult SNSQ026_001_getSearch([FromBody] dynamic pData)
        {
            try
            {

                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ026_001_SUMMARY";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData.VENDOR_CODE;
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pData.PART_NO;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                // cmd.Parameters.Add("P_CUSTOMER_CODE_E", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE_E;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ026_001_getSearch");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSQ026_001_Export_Excel")]
        public HttpResponseMessage SNSQ026_001_Export_Excel(List<DataModel.SNSQ026> dModel)
        {
            DataTable dt = new DataTable();
            string x = string.Empty;
            string filename = "SNSQ026_BOAmount.csv";
            string floderLocation = "SNSQ026";
            dt.Columns.Add("colum1");

            dt.Rows.Add("ID;PO2C");
            dt.Rows.Add("B");
            dt.Rows.Add("C;X1;Y1;K\"Seq.\"");
            dt.Rows.Add("C;X2;Y1;K\"Part No.\"");
            dt.Rows.Add("C;X3;Y1;K\"Part Name\"");
            dt.Rows.Add("C;X4;Y1;K\"Staff Incharge\"");
            dt.Rows.Add("C;X5;Y1;K\"Vendor Code\"");
            dt.Rows.Add("C;X6;Y1;K\"Model\"");
            dt.Rows.Add("C;X7;Y1;K\"Unit Price\"");
            dt.Rows.Add("C;X8;Y1;K\"Lead Time\"");
            dt.Rows.Add("C;X9;Y1;K\"Category Code\"");
            dt.Rows.Add("C;X10;Y1;K\"PO Qty\"");
            dt.Rows.Add("C;X11;Y1;K\"BO Qty\"");
            dt.Rows.Add("C;X12;Y1;K\"BO Line\"");
            dt.Rows.Add("C;X13;Y1;K\"BO Amount\"");
            dt.Rows.Add("C;X14;Y1;K\"Currency Code\"");
            dt.Rows.Add("C;X15;Y1;K\"Customer Code Domestic\"");
            dt.Rows.Add("C;X16;Y1;K\"Customer Code Export\"");
            dt.Rows.Add("C;X17;Y1;K\"Customer Name\"");

            int countrow = 0;
            int itemno = 0;
            for (var i = 0; i < dModel.Count; i++)
            {
                countrow = i + 2;
                itemno = i + 1;
                dt.Rows.Add("C;X1;Y" + countrow + ";K" + '"' + dModel[i].SEQ + '"');
                dt.Rows.Add("C;X2;Y" + countrow + ";K" + '"' + dModel[i].ORDER_PART_NO + '"');
                dt.Rows.Add("C;X3;Y" + countrow + ";K" + '"' + dModel[i].PART_NAME_EN + '"');
                dt.Rows.Add("C;X4;Y" + countrow + ";K" + '"' + dModel[i].STAFF_INCHARGE + '"');
                dt.Rows.Add("C;X5;Y" + countrow + ";K" + '"' + dModel[i].VENDOR_CODE + '"');
                dt.Rows.Add("C;X6;Y" + countrow + ";K" + '"' + dModel[i].MAIN_MODEL + '"');
                dt.Rows.Add("C;X7;Y" + countrow + ";K" + '"' + dModel[i].BASE_PRICE + '"');
                dt.Rows.Add("C;X8;Y" + countrow + ";K" + '"' + dModel[i].LEAD_TIME_ON + '"');
                dt.Rows.Add("C;X9;Y" + countrow + ";K" + '"' + dModel[i].PART_CATEGORY + '"');
                dt.Rows.Add("C;X10;Y" + countrow + ";K" + '"' + dModel[i].SUM_PO_QTY + '"');
                dt.Rows.Add("C;X11;Y" + countrow + ";K" + '"' + dModel[i].SUM_BO_QTY + '"');
                dt.Rows.Add("C;X12;Y" + countrow + ";K" + '"' + dModel[i].CNT_BO_LINE + '"');
                dt.Rows.Add("C;X13;Y" + countrow + ";K" + '"' + dModel[i].BO_AMOUNT + '"');
                dt.Rows.Add("C;X14;Y" + countrow + ";K" + '"' + dModel[i].CURRENCY_CODE + '"');
                dt.Rows.Add("C;X15;Y" + countrow + ";K" + '"' + dModel[i].CUSTOMER_CODE_D + '"');
                dt.Rows.Add("C;X16;Y" + countrow + ";K" + '"' + dModel[i].CUSTOMER_CODE_E + '"');
                dt.Rows.Add("C;X17;Y" + countrow + ";K" + '"' + dModel[i].CUST_NAME_EN + '"');
            }

            dt.Rows.Add("E");

            x = DataService.WriteCSVNoColumn(dt, filename, floderLocation);
            Controllers.SystemApiController response = new Controllers.SystemApiController();
            var File = response.downloadFile(x.Substring(1), filename);
            return File;
        }

        #endregion

        #region SNSQ046_001
        [HttpGet, Route("SNSQ046_001_BUYER_CODE/{buyerCode}/{NoOfShowRecords}")]
        [Authorize]
        public IHttpActionResult SNSQ046_001_BUYER_CODE(string buyerCode, string NoOfShowRecords)
        {
            try
            {
                DataSet ds = new DataSet();
                buyerCode = ismoUtil.escapeCharacter(buyerCode);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ046_001_BUYER_CODE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_BUYER_CODE", OracleDbType.Varchar2).Value = buyerCode.ToUpper() == "NULL" ? (object)DBNull.Value : buyerCode;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ046_001_BUYER_CODE");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));

            }
        }

        [HttpPost, Route("SNSQ046_001_getSearch")]
        [Authorize]
        public IHttpActionResult SNSQ046_001_getSearch([FromBody] dynamic pData)
        {
            try
            {

                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ046_SEARCH";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_BUYER_CODE", OracleDbType.Varchar2).Value = pData.BUYER_CODE;
                cmd.Parameters.Add("P_FROM_DATE", OracleDbType.Date).Value = pData.FROM_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(pData.FROM_DATE);
                cmd.Parameters.Add("P_TO_DATE", OracleDbType.Date).Value = pData.TO_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(pData.TO_DATE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ046_001_getSearch");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpPost, Route("SNSQ046_001_Export_Excel")]
        public HttpResponseMessage SNSQ046_001_Export_Excel(List<DataModel.SNSQ046_001> dModel)
        {

            DataTable dt = new DataTable();
            string x = string.Empty;
            Console.WriteLine();
            string filename = "SNSQ046.csv";
            string floderLocation = "SNSQ046";


            dt.Columns.Add("colum1");

            dt.Rows.Add("ID;PO2C");
            dt.Rows.Add("B");
            dt.Rows.Add("C;X1;Y1;K\"Seq\"");
            dt.Rows.Add("C;X2;Y1;K\"Country\"");
            dt.Rows.Add("C;X3;Y1;K\"Buyer\"");
            dt.Rows.Add("C;X4;Y1;K\"Part Number\"");
            dt.Rows.Add("C;X5;Y1;K\"Part Name\"");
            dt.Rows.Add("C;X6;Y1;K\"Pricing Role\"");
            dt.Rows.Add("C;X7;Y1;K\"Itp Parts Flag\"");
            dt.Rows.Add("C;X8;Y1;K\"Model Code\"");
            dt.Rows.Add("C;X9;Y1;K\"GLP\"");
            dt.Rows.Add("C;X10;Y1;K\"GLP Currency\"");
            dt.Rows.Add("C;X11;Y1;K\"GLP Establisment Category\"");
            dt.Rows.Add("C;X12;Y1;K\"GLP App\"");
            dt.Rows.Add("C;X13;Y1;K\"Description Code\"");
            dt.Rows.Add("C;X14;Y1;K\"Sales Classification Code\"");
            dt.Rows.Add("C;X15;Y1;K\"New Model Setting Due Date\"");
            dt.Rows.Add("C;X16;Y1;K\"Price Rivison Setting Due Date\"");
            dt.Rows.Add("C;X17;Y1;K\"Purchase FOB Bp\"");
            dt.Rows.Add("C;X18;Y1;K\"Purchase FOB Sp\"");
            dt.Rows.Add("C;X19;Y1;K\"Purchase FOB Currency\"");
            dt.Rows.Add("C;X20;Y1;K\"Purchase FOB Price App\"");
            dt.Rows.Add("C;X21;Y1;K\"Purchase FOB Sp Start\"");
            dt.Rows.Add("C;X22;Y1;K\"Purchase FOB Sp End\"");
            dt.Rows.Add("C;X23;Y1;K\"Purchase Cost\"");
            dt.Rows.Add("C;X24;Y1;K\"Purchase Cost Currency\"");
            dt.Rows.Add("C;X25;Y1;K\"Purchase Cost Tentative Flag\"");
            dt.Rows.Add("C;X26;Y1;K\"Into Store Cost\"");
            dt.Rows.Add("C;X27;Y1;K\"Isc Currency\"");
            dt.Rows.Add("C;X28;Y1;K\"Selling FOB BP\"");
            dt.Rows.Add("C;X29;Y1;K\"Selling FOB SP\"");
            dt.Rows.Add("C;X30;Y1;K\"Selling FOB Currency\"");
            dt.Rows.Add("C;X31;Y1;K\"Selling FOB Price App\"");
            dt.Rows.Add("C;X32;Y1;K\"Selling FOB SP Start\"");
            dt.Rows.Add("C;X33;Y1;K\"Selling FOB SP End\"");
            dt.Rows.Add("C;X34;Y1;K\"Acc Sv Unique Flag\"");
            dt.Rows.Add("C;X35;Y1;K\"Reason Code\"");
            dt.Rows.Add("C;X36;Y1;K\"Update Status\"");
            dt.Rows.Add("C;X37;Y1;K\"Send Status\"");
            dt.Rows.Add("C;X38;Y1;K\"Create By\"");
            dt.Rows.Add("C;X39;Y1;K\"Create Date\"");
            dt.Rows.Add("C;X40;Y1;K\"Last Upd Pgm\"");
            dt.Rows.Add("C;X41;Y1;K\"Last Upd By\"");
            dt.Rows.Add("C;X42;Y1;K\"Last Upd Date\"");


            int countrow = 0;
            int itemno = 0;

            for (var i = 0; i < dModel.Count; i++)
            {
                countrow = i + 2;
                itemno = i + 1;
                dt.Rows.Add("C;X1;Y" + countrow + ";K" + '"' + dModel[i].SEQ + '"');
                dt.Rows.Add("C;X2;Y" + countrow + ";K" + '"' + dModel[i].COUNTRY_CODE + '"');
                dt.Rows.Add("C;X3;Y" + countrow + ";K" + '"' + dModel[i].BUYER_CODE + '"');
                dt.Rows.Add("C;X4;Y" + countrow + ";K" + '"' + dModel[i].PART_NUMBER + '"');
                dt.Rows.Add("C;X5;Y" + countrow + ";K" + '"' + dModel[i].PART_NAME + '"');
                dt.Rows.Add("C;X6;Y" + countrow + ";K" + '"' + dModel[i].PRICING_ROLE + '"');
                dt.Rows.Add("C;X7;Y" + countrow + ";K" + '"' + dModel[i].ITP_PARTS_FLAG + '"');
                dt.Rows.Add("C;X8;Y" + countrow + ";K" + '"' + dModel[i].MODEL_COE + '"');
                dt.Rows.Add("C;X9;Y" + countrow + ";K" + '"' + dModel[i].GLP + '"');
                dt.Rows.Add("C;X10;Y" + countrow + ";K" + '"' + dModel[i].GLP_CURRENCY + '"');
                dt.Rows.Add("C;X11;Y" + countrow + ";K" + '"' + dModel[i].GLP_ESTABLISMENT_CATEGORY + '"');
                dt.Rows.Add("C;X12;Y" + countrow + ";K" + '"' + dModel[i].GLP_APP + '"');
                dt.Rows.Add("C;X13;Y" + countrow + ";K" + '"' + dModel[i].DESCRIPTION_CODE + '"');
                dt.Rows.Add("C;X14;Y" + countrow + ";K" + '"' + dModel[i].SALES_CLASSIFICATION_CODE + '"');
                dt.Rows.Add("C;X15;Y" + countrow + ";K" + '"' + dModel[i].NEW_MODEL_SETTING_DUE_DATE + '"');
                dt.Rows.Add("C;X16;Y" + countrow + ";K" + '"' + dModel[i].PRICE_RIVISON_SETTING_DUE_DATE + '"');
                dt.Rows.Add("C;X17;Y" + countrow + ";K" + '"' + dModel[i].PURCHASE_FOB_BP + '"');
                dt.Rows.Add("C;X18;Y" + countrow + ";K" + '"' + dModel[i].PURCHASE_FOB_SP + '"');
                dt.Rows.Add("C;X19;Y" + countrow + ";K" + '"' + dModel[i].PURCHASE_FOB_CURRENCY + '"');
                dt.Rows.Add("C;X20;Y" + countrow + ";K" + '"' + dModel[i].PURCHASE_FOB_PRICE_APP + '"');
                dt.Rows.Add("C;X21;Y" + countrow + ";K" + '"' + dModel[i].PURCHASE_FOB_SP_START + '"');
                dt.Rows.Add("C;X22;Y" + countrow + ";K" + '"' + dModel[i].PURCHASE_FOB_SP_END + '"');
                dt.Rows.Add("C;X23;Y" + countrow + ";K" + '"' + dModel[i].PURCHASE_COST + '"');
                dt.Rows.Add("C;X24;Y" + countrow + ";K" + '"' + dModel[i].PURCHASE_COST_CURRENCY + '"');
                dt.Rows.Add("C;X25;Y" + countrow + ";K" + '"' + dModel[i].PURCHASE_COST_TENTATIVE_FLAG + '"');
                dt.Rows.Add("C;X26;Y" + countrow + ";K" + '"' + dModel[i].INTO_STORE_COST + '"');
                dt.Rows.Add("C;X27;Y" + countrow + ";K" + '"' + dModel[i].ISC_CURRENCY + '"');
                dt.Rows.Add("C;X28;Y" + countrow + ";K" + '"' + dModel[i].SELLING_FOB_BP + '"');
                dt.Rows.Add("C;X29;Y" + countrow + ";K" + '"' + dModel[i].SELLING_FOB_SP + '"');
                dt.Rows.Add("C;X30;Y" + countrow + ";K" + '"' + dModel[i].SELLING_FOB_CURRENCY + '"');
                dt.Rows.Add("C;X31;Y" + countrow + ";K" + '"' + dModel[i].SELLING_FOB_PRICE_APP + '"');
                dt.Rows.Add("C;X32;Y" + countrow + ";K" + '"' + dModel[i].SELLING_FOB_SP_START + '"');
                dt.Rows.Add("C;X33;Y" + countrow + ";K" + '"' + dModel[i].SELLING_FOB_SP_END + '"');
                dt.Rows.Add("C;X34;Y" + countrow + ";K" + '"' + dModel[i].ACC_SV_UNIQUE_FLAG + '"');
                dt.Rows.Add("C;X35;Y" + countrow + ";K" + '"' + dModel[i].REASON_CODE + '"');
                dt.Rows.Add("C;X36;Y" + countrow + ";K" + '"' + dModel[i].UPDATE_STATUS + '"');
                dt.Rows.Add("C;X37;Y" + countrow + ";K" + '"' + dModel[i].SEND_STATUS + '"');
                dt.Rows.Add("C;X38;Y" + countrow + ";K" + '"' + dModel[i].CREATE_BY + '"');
                dt.Rows.Add("C;X39;Y" + countrow + ";K" + '"' + dModel[i].CREATE_DATE + '"');
                dt.Rows.Add("C;X40;Y" + countrow + ";K" + '"' + dModel[i].LAST_UPD_PGM + '"');
                dt.Rows.Add("C;X41;Y" + countrow + ";K" + '"' + dModel[i].LAST_UPD_BY + '"');
                dt.Rows.Add("C;X42;Y" + countrow + ";K" + '"' + dModel[i].LAST_UPD_DATE + '"');
            }

            dt.Rows.Add("E");

            x = DataService.WriteCSVNoColumn(dt, filename, floderLocation);
            Controllers.SystemApiController response = new Controllers.SystemApiController();
            var File = response.downloadFile(x.Substring(1), filename);
            return File;
        }

        #endregion

        #region SNSQ048

        [HttpPost, Route("SNSQ048_getPartNo/{getPartNo}/{NoOfShowRecords}")]
        [Authorize]
        public IHttpActionResult SNSQ048_getPartNo(string getPartNo, string NoOfShowRecords, [FromBody] dynamic pParam)
        {
            try
            {
                DataSet ds = new DataSet();
                getPartNo = ismoUtil.escapeCharacter(getPartNo);
                var pData = pParam["DATA"];

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ048_PART_NO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = getPartNo.ToUpper() == "NULL" ? (object)DBNull.Value : getPartNo;
                cmd.Parameters.Add("P_FROM_DATE", OracleDbType.Date).Value = pData.FROM_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(pData.FROM_DATE);
                cmd.Parameters.Add("P_TO_DATE", OracleDbType.Date).Value = pData.TO_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(pData.TO_DATE);
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = NoOfShowRecords;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ048_getPartNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));

            }
        }


        [HttpPost, Route("SNSQ048_getSearch")]
        [Authorize]
        public IHttpActionResult SNSQ048_getSearch([FromBody] dynamic pData)
        {
            try
            {

                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ048_SEARCH";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NUMBER", OracleDbType.Varchar2).Value = pData.PART_NUMBER;
                cmd.Parameters.Add("P_FROM_DATE", OracleDbType.Date).Value = pData.FROM_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(pData.FROM_DATE);
                cmd.Parameters.Add("P_TO_DATE", OracleDbType.Date).Value = pData.TO_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(pData.TO_DATE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ048_getSearch");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSQ048_Export_Excel")]
        public HttpResponseMessage SNSQ048_Export_Excel(List<DataModel.SNSQ048> dModel)
        {

            DataTable dt = new DataTable();
            string x = string.Empty;
            Console.WriteLine();
            string filename = "SNSQ048.csv";
            string floderLocation = "SNSQ048";


            dt.Columns.Add("colum1");

            dt.Rows.Add("ID;PO2C");
            dt.Rows.Add("B");
            dt.Rows.Add("C;X1;Y1;K\"Seq\"");
            dt.Rows.Add("C;X2;Y1;K\"Part Number\"");
            dt.Rows.Add("C;X3;Y1;K\"Reason Code\"");
            dt.Rows.Add("C;X4;Y1;K\"Error Code\"");
            dt.Rows.Add("C;X5;Y1;K\"Error Message\"");
            dt.Rows.Add("C;X6;Y1;K\"Country Code\"");
            dt.Rows.Add("C;X7;Y1;K\"Buyer Code\"");
            dt.Rows.Add("C;X8;Y1;K\"Pricing Role\"");
            dt.Rows.Add("C;X9;Y1;K\"ITP Parts Flag\"");
            dt.Rows.Add("C;X10;Y1;K\"Model Code\"");
            dt.Rows.Add("C;X11;Y1;K\"GLP\"");
            dt.Rows.Add("C;X12;Y1;K\"GLP Currency\"");
            dt.Rows.Add("C;X13;Y1;K\"GLP Establishment Category\"");
            dt.Rows.Add("C;X14;Y1;K\"GLP App\"");
            dt.Rows.Add("C;X15;Y1;K\"Description Code\"");
            dt.Rows.Add("C;X16;Y1;K\"Sales Classification Code\"");
            dt.Rows.Add("C;X17;Y1;K\"New Model Setting Due Date\"");
            dt.Rows.Add("C;X18;Y1;K\"Price Rivison Setting Due Date\"");
            dt.Rows.Add("C;X19;Y1;K\"Purchase FOB BP\"");
            dt.Rows.Add("C;X20;Y1;K\"Purchase FOB SP\"");
            dt.Rows.Add("C;X21;Y1;K\"Purchase FOB Currency\"");
            dt.Rows.Add("C;X22;Y1;K\"Purchase FOB Price App\"");
            dt.Rows.Add("C;X23;Y1;K\"Purchase FOB SP Start\"");
            dt.Rows.Add("C;X24;Y1;K\"Purchase FOB SP End\"");
            dt.Rows.Add("C;X25;Y1;K\"Purchase Cost\"");
            dt.Rows.Add("C;X26;Y1;K\"Purchase Cost Currency\"");
            dt.Rows.Add("C;X27;Y1;K\"Purchase Cost Tentative Flag\"");
            dt.Rows.Add("C;X28;Y1;K\"Into Store Cost\"");
            dt.Rows.Add("C;X29;Y1;K\"ISC Currency\"");
            dt.Rows.Add("C;X30;Y1;K\"Selling FOB BP\"");
            dt.Rows.Add("C;X31;Y1;K\"Selling FOB SP\"");
            dt.Rows.Add("C;X32;Y1;K\"Selling FOB Currency\"");
            dt.Rows.Add("C;X33;Y1;K\"Selling FOB Price App\"");
            dt.Rows.Add("C;X34;Y1;K\"Selling FOB SP Start\"");
            dt.Rows.Add("C;X35;Y1;K\"Selling FOB SP End\"");
            dt.Rows.Add("C;X36;Y1;K\"ACC SV Unique Flag\"");
            dt.Rows.Add("C;X37;Y1;K\"Status\"");
            dt.Rows.Add("C;X38;Y1;K\"Create By\"");
            dt.Rows.Add("C;X39;Y1;K\"Create Date\"");
            dt.Rows.Add("C;X40;Y1;K\"Last Upd Pgm\"");
            dt.Rows.Add("C;X41;Y1;K\"Last Upd By\"");
            dt.Rows.Add("C;X42;Y1;K\"Last Upd Date\"");


            int countrow = 0;
            int itemno = 0;

            for (var i = 0; i < dModel.Count; i++)
            {
                countrow = i + 2;
                itemno = i + 1;
                dt.Rows.Add("C;X1;Y" + countrow + ";K" + '"' + dModel[i].SEQ + '"');
                dt.Rows.Add("C;X2;Y" + countrow + ";K" + '"' + dModel[i].PART_NUMBER + '"');
                dt.Rows.Add("C;X3;Y" + countrow + ";K" + '"' + dModel[i].REASON_CODE + '"');
                dt.Rows.Add("C;X4;Y" + countrow + ";K" + '"' + dModel[i].ERROR_CODE + '"');
                dt.Rows.Add("C;X5;Y" + countrow + ";K" + '"' + dModel[i].ERROR_MESSAGE + '"');
                dt.Rows.Add("C;X6;Y" + countrow + ";K" + '"' + dModel[i].COUNTRY_CODE + '"');
                dt.Rows.Add("C;X7;Y" + countrow + ";K" + '"' + dModel[i].BUYER_CODE + '"');
                dt.Rows.Add("C;X8;Y" + countrow + ";K" + '"' + dModel[i].PRICING_ROLE + '"');
                dt.Rows.Add("C;X9;Y" + countrow + ";K" + '"' + dModel[i].ITP_PARTS_FLAG + '"');
                dt.Rows.Add("C;X10;Y" + countrow + ";K" + '"' + dModel[i].MODEL_CODE + '"');
                dt.Rows.Add("C;X11;Y" + countrow + ";K" + '"' + dModel[i].GLP + '"');
                dt.Rows.Add("C;X12;Y" + countrow + ";K" + '"' + dModel[i].GLP_CURRENCY + '"');
                dt.Rows.Add("C;X13;Y" + countrow + ";K" + '"' + dModel[i].GLP_ESTABLISHMENT_CATEGORY + '"');
                dt.Rows.Add("C;X14;Y" + countrow + ";K" + '"' + dModel[i].GLP_APP + '"');
                dt.Rows.Add("C;X15;Y" + countrow + ";K" + '"' + dModel[i].DESCRIPTION_CODE + '"');
                dt.Rows.Add("C;X16;Y" + countrow + ";K" + '"' + dModel[i].SALES_CLASSIFICATION_CODE + '"');
                dt.Rows.Add("C;X17;Y" + countrow + ";K" + '"' + dModel[i].NEW_MODEL_SETTING_DUE_DATE + '"');
                dt.Rows.Add("C;X18;Y" + countrow + ";K" + '"' + dModel[i].PRICE_RIVISON_SETTING_DUE_DATE + '"');
                dt.Rows.Add("C;X19;Y" + countrow + ";K" + '"' + dModel[i].PURCHASE_FOB_BP + '"');
                dt.Rows.Add("C;X20;Y" + countrow + ";K" + '"' + dModel[i].PURCHASE_FOB_SP + '"');
                dt.Rows.Add("C;X21;Y" + countrow + ";K" + '"' + dModel[i].PURCHASE_FOB_CURRENCY + '"');
                dt.Rows.Add("C;X22;Y" + countrow + ";K" + '"' + dModel[i].PURCHASE_FOB_PRICE_APP + '"');
                dt.Rows.Add("C;X23;Y" + countrow + ";K" + '"' + dModel[i].PURCHASE_FOB_SP_START + '"');
                dt.Rows.Add("C;X24;Y" + countrow + ";K" + '"' + dModel[i].PURCHASE_FOB_SP_END + '"');
                dt.Rows.Add("C;X25;Y" + countrow + ";K" + '"' + dModel[i].PURCHASE_COST + '"');
                dt.Rows.Add("C;X26;Y" + countrow + ";K" + '"' + dModel[i].PURCHASE_COST_CURRENCY + '"');
                dt.Rows.Add("C;X27;Y" + countrow + ";K" + '"' + dModel[i].PURCHASE_COST_TENTATIVE_FLAG + '"');
                dt.Rows.Add("C;X28;Y" + countrow + ";K" + '"' + dModel[i].INTO_STORE_COST + '"');
                dt.Rows.Add("C;X29;Y" + countrow + ";K" + '"' + dModel[i].ISC_CURRENCY + '"');
                dt.Rows.Add("C;X30;Y" + countrow + ";K" + '"' + dModel[i].SELLING_FOB_BP + '"');
                dt.Rows.Add("C;X31;Y" + countrow + ";K" + '"' + dModel[i].SELLING_FOB_SP + '"');
                dt.Rows.Add("C;X32;Y" + countrow + ";K" + '"' + dModel[i].SELLING_FOB_CURRENCY + '"');
                dt.Rows.Add("C;X33;Y" + countrow + ";K" + '"' + dModel[i].SELLING_FOB_PRICE_APP + '"');
                dt.Rows.Add("C;X34;Y" + countrow + ";K" + '"' + dModel[i].SELLING_FOB_SP_START + '"');
                dt.Rows.Add("C;X35;Y" + countrow + ";K" + '"' + dModel[i].SELLING_FOB_SP_END + '"');
                dt.Rows.Add("C;X36;Y" + countrow + ";K" + '"' + dModel[i].ACC_SV_UNIQUE_FLAG + '"');
                dt.Rows.Add("C;X37;Y" + countrow + ";K" + '"' + dModel[i].STATUS + '"');
                dt.Rows.Add("C;X38;Y" + countrow + ";K" + '"' + dModel[i].CREATE_BY + '"');
                dt.Rows.Add("C;X39;Y" + countrow + ";K" + '"' + dModel[i].CREATE_DATE + '"');
                dt.Rows.Add("C;X40;Y" + countrow + ";K" + '"' + dModel[i].LAST_UPD_PGM + '"');
                dt.Rows.Add("C;X41;Y" + countrow + ";K" + '"' + dModel[i].LAST_UPD_BY + '"');
                dt.Rows.Add("C;X42;Y" + countrow + ";K" + '"' + dModel[i].LAST_UPD_DATE + '"');
            }

            dt.Rows.Add("E");

            x = DataService.WriteCSVNoColumn(dt, filename, floderLocation);
            Controllers.SystemApiController response = new Controllers.SystemApiController();
            var File = response.downloadFile(x.Substring(1), filename);
            return File;
        }

        #endregion

        #region SNSQ074_MR

        [HttpPost, Route("SNSQ074MR_GET_ALLSALES")]
        [Authorize]
        public IHttpActionResult SNSQ074MR_GET_ALLSALES([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ074_MR_ALLSALES";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                //cmd.Parameters.Add("P_SALES_CHANNEL", OracleDbType.Varchar2).Value = pParam["DSP_SALES_CHANNEL"];
                cmd.Parameters.Add("P_PROCESS_YEAR", OracleDbType.Int32).Value = pParam["PROCESS_YEAR"];
                cmd.Parameters.Add("P_PROCESS_MONTH", OracleDbType.Int32).Value = pParam["PROCESS_MONTH"];
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ074MR_GET_ALLSALES");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSQ074MR_GET_ALLZONE")]
        [Authorize]
        public IHttpActionResult SNSQ074MR_GET_ALLZONE([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ074_MR_ALLZONE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                //cmd.Parameters.Add("P_DSP_ZONE", OracleDbType.Varchar2).Value = pParam["DSP_ZONE"];
                cmd.Parameters.Add("P_PROCESS_YEAR", OracleDbType.Int32).Value = pParam["PROCESS_YEAR"];
                cmd.Parameters.Add("P_PROCESS_MONTH", OracleDbType.Int32).Value = pParam["PROCESS_MONTH"];
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ074MR_GET_ALLZONE");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSQ074MR_GET_ALLCUSCODE")]
        [Authorize]
        public IHttpActionResult SNSQ074MR_GET_ALLCUSCODE([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ074_MR_ALLCUSCODE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                //cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam["DSP_CUSTOMER"];
                cmd.Parameters.Add("P_PROCESS_YEAR", OracleDbType.Int32).Value = pParam["PROCESS_YEAR"];
                cmd.Parameters.Add("P_PROCESS_MONTH", OracleDbType.Int32).Value = pParam["PROCESS_MONTH"];
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ074MR_GET_ALLCUSCODE");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSQ074MR_GET_ALL_PRODUCT")]
        [Authorize]
        public IHttpActionResult SNSQ074MR_GET_ALL_PRODUCT([FromBody] dynamic pData)
        {

            try
            {

                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ074_MR_ALL_PRODCODE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                //cmd.Parameters.Add("P_PRODUCT_CODE", OracleDbType.Varchar2).Value = pData.DSP_PRODUCT;
                cmd.Parameters.Add("P_PROCESS_YEAR", OracleDbType.Int32).Value = pData.PROCESS_YEAR;
                cmd.Parameters.Add("P_PROCESS_MONTH", OracleDbType.Int32).Value = pData.PROCESS_MONTH;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ074MR_GET_ALL_PRODUCT");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSQ074MR_GET_ALL_DESC")]
        [Authorize]
        public IHttpActionResult SNSQ074MR_GET_ALL_DESC([FromBody] dynamic pData)
        {

            try
            {

                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ074_MR_ALLDESC";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                //cmd.Parameters.Add("P_DESCRIPT_CODE", OracleDbType.Varchar2).Value = pData.DSP_DESC;
                cmd.Parameters.Add("P_PROCESS_YEAR", OracleDbType.Int32).Value = pData.PROCESS_YEAR;
                cmd.Parameters.Add("P_PROCESS_MONTH", OracleDbType.Int32).Value = pData.PROCESS_MONTH;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ074MR_GET_ALL_DESC");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSQ074MR_GET_ALL_PART")]
        [Authorize]
        public IHttpActionResult SNSQ074MR_GET_ALL_PART([FromBody] dynamic pData)
        {

            try
            {

                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ074_MR_ALLPARTNO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                //cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pData.DSP_PARTNO;
                cmd.Parameters.Add("P_PROCESS_YEAR", OracleDbType.Int32).Value = pData.PROCESS_YEAR;
                cmd.Parameters.Add("P_PROCESS_MONTH", OracleDbType.Int32).Value = pData.PROCESS_MONTH;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ074MR_GET_ALL_PART");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSQ074MR_GET_SALSE_ZONE")]
        [Authorize]
        public IHttpActionResult SNSQ074MR_GET_SALSE_ZONE([FromBody] dynamic pData)
        {

            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ074_MR_SALSEZONE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PROCESS_YEAR", OracleDbType.Int32).Value = pData.PROCESS_YEAR;
                cmd.Parameters.Add("P_PROCESS_MONTH", OracleDbType.Int32).Value = pData.PROCESS_MONTH;
                //cmd.Parameters.Add("P_DSP_SALES_CHANNEL", OracleDbType.Varchar2).Value = pData.DSP_SALES_CHANNEL;
                //cmd.Parameters.Add("P_DSP_ZONE", OracleDbType.Varchar2).Value = pData.DSP_ZONE;
                cmd.Parameters.Add("P_SALES_CHANNEL", OracleDbType.Varchar2).Value = pData.SALES_CHANNEL;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ074MR_GET_SALSE_ZONE");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSQ074MR_GET_ZONE_CUST")]
        [Authorize]
        public IHttpActionResult SNSQ074MR_GET_ZONE_CUST([FromBody] dynamic pData)
        {

            try
            {
                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ074_MR_ZONECUST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PROCESS_YEAR", OracleDbType.Int32).Value = pData.PROCESS_YEAR;
                cmd.Parameters.Add("P_PROCESS_MONTH", OracleDbType.Int32).Value = pData.PROCESS_MONTH;
                //cmd.Parameters.Add("P_DSP_ZONE", OracleDbType.Varchar2).Value = pData.DSP_ZONE;
                //cmd.Parameters.Add("P_DSP_CUSTOMER", OracleDbType.Varchar2).Value = pData.DSP_CUSTOMER;
                cmd.Parameters.Add("P_ZONE_AREA", OracleDbType.Varchar2).Value = pData.ZONE_AREA;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ074MR_GET_ZONE_CUST");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSQ074MR_GET_CUST_PROD")]
        [Authorize]
        public IHttpActionResult SNSQ074MR_GET_CUST_PROD([FromBody] dynamic pData)
        {

            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ074_MR_CUSTPROD";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PROCESS_YEAR", OracleDbType.Int32).Value = pData.PROCESS_YEAR;
                cmd.Parameters.Add("P_PROCESS_MONTH", OracleDbType.Int32).Value = pData.PROCESS_MONTH;
                //cmd.Parameters.Add("P_DSP_CUSTOMER", OracleDbType.Varchar2).Value = pData.DSP_CUSTOMER;
                //cmd.Parameters.Add("P_DSP_PRODUCT", OracleDbType.Varchar2).Value = pData.DSP_PRODUCT;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ074MR_GET_CUST_PROD");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSQ074MR_GET_PROD_DESC")]
        [Authorize]
        public IHttpActionResult SNSQ074MR_GET_PROD_DESC([FromBody] dynamic pData)
        {

            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ074_MR_PRODDESC";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PROCESS_YEAR", OracleDbType.Int32).Value = pData.PROCESS_YEAR;
                cmd.Parameters.Add("P_PROCESS_MONTH", OracleDbType.Int32).Value = pData.PROCESS_MONTH;
                //cmd.Parameters.Add("P_DSP_PRODUCT", OracleDbType.Varchar2).Value = pData.DSP_PRODUCT;
                //cmd.Parameters.Add("P_DSP_DESC", OracleDbType.Varchar2).Value = pData.DSP_DESC;
                cmd.Parameters.Add("P_PRODUCT_CODE", OracleDbType.Varchar2).Value = pData.PRODUCT_CODE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ074MR_GET_PROD_DESC");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSQ074MR_GET_DESC_PART")]
        [Authorize]
        public IHttpActionResult SNSQ074MR_GET_DESC_PART([FromBody] dynamic pData)
        {

            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ074_MR_DESCPART";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PROCESS_YEAR", OracleDbType.Int32).Value = pData.PROCESS_YEAR;
                cmd.Parameters.Add("P_PROCESS_MONTH", OracleDbType.Int32).Value = pData.PROCESS_MONTH;
                //cmd.Parameters.Add("P_DSP_DESC", OracleDbType.Varchar2).Value = pData.DSP_DESC;
                cmd.Parameters.Add("P_DESCRIPT_CODE", OracleDbType.Varchar2).Value = pData.DESCRIPT_CODE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ074MR_GET_DESC_PART");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSQ074MR_GET_SALE_CUST")]
        [Authorize]
        public IHttpActionResult SNSQ074MR_GET_SALE_CUST([FromBody] dynamic pData)
        {

            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ074_MR_SALESCUST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PROCESS_YEAR", OracleDbType.Int32).Value = pData.PROCESS_YEAR;
                cmd.Parameters.Add("P_PROCESS_MONTH", OracleDbType.Int32).Value = pData.PROCESS_MONTH;
                //cmd.Parameters.Add("P_DSP_SALES_CHANNEL", OracleDbType.Varchar2).Value = pData.DSP_SALES_CHANNEL;
                //cmd.Parameters.Add("P_DSP_CUSTOMER", OracleDbType.Varchar2).Value = pData.DSP_CUSTOMER;
                cmd.Parameters.Add("P_SALES_CHANNEL", OracleDbType.Varchar2).Value = pData.SALES_CHANNEL;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ074MR_GET_SALE_CUST");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSQ074MR_GET_SALE_PROD")]
        [Authorize]
        public IHttpActionResult SNSQ074MR_GET_SALE_PROD([FromBody] dynamic pData)
        {

            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ074_MR_SALESPROD";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PROCESS_YEAR", OracleDbType.Int32).Value = pData.PROCESS_YEAR;
                cmd.Parameters.Add("P_PROCESS_MONTH", OracleDbType.Varchar2).Value = pData.PROCESS_MONTH;
                //cmd.Parameters.Add("P_DSP_SALES_CHANNEL", OracleDbType.Varchar2).Value = pData.DSP_SALES_CHANNEL;
                //cmd.Parameters.Add("P_DSP_PRODUCT", OracleDbType.Varchar2).Value = pData.DSP_PRODUCT;
                cmd.Parameters.Add("P_SALES_CHANNEL", OracleDbType.Varchar2).Value = pData.SALES_CHANNEL;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ074MR_GET_SALE_PROD");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSQ074MR_GET_SALE_DESC")]
        [Authorize]
        public IHttpActionResult SNSQ074MR_GET_SALE_DESC([FromBody] dynamic pData)
        {

            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ074_MR_SALESDESC";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PROCESS_YEAR", OracleDbType.Int32).Value = pData.PROCESS_YEAR;
                cmd.Parameters.Add("P_PROCESS_MONTH", OracleDbType.Varchar2).Value = pData.PROCESS_MONTH;
                //cmd.Parameters.Add("P_DSP_SALES_CHANNEL", OracleDbType.Varchar2).Value = pData.DSP_SALES_CHANNEL;
                //cmd.Parameters.Add("P_DSP_DESC", OracleDbType.Varchar2).Value = pData.DSP_DESC;
                cmd.Parameters.Add("P_SALES_CHANNEL", OracleDbType.Varchar2).Value = pData.SALES_CHANNEL;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SP_NP_SNSQ074_MR_SALESDESC");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSQ074MR_GET_SALE_PART")]
        [Authorize]
        public IHttpActionResult SNSQ074MR_GET_SALE_PART([FromBody] dynamic pData)
        {

            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ074_MR_SALESPART";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PROCESS_YEAR", OracleDbType.Int32).Value = pData.PROCESS_YEAR;
                cmd.Parameters.Add("P_PROCESS_MONTH", OracleDbType.Int32).Value = pData.PROCESS_MONTH;
                //cmd.Parameters.Add("P_DSP_SALES_CHANNEL", OracleDbType.Varchar2).Value = pData.DSP_SALES_CHANNEL;
                cmd.Parameters.Add("P_SALES_CHANNEL", OracleDbType.Varchar2).Value = pData.SALES_CHANNEL;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ074MR_GET_SALE_PART");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSQ074MR_GET_ZONE_PROD")]
        [Authorize]
        public IHttpActionResult SNSQ074MR_GET_ZONE_PROD([FromBody] dynamic pData)
        {

            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ074_MR_ZONEPROD";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PROCESS_YEAR", OracleDbType.Int32).Value = pData.PROCESS_YEAR;
                cmd.Parameters.Add("P_PROCESS_MONTH", OracleDbType.Int32).Value = pData.PROCESS_MONTH;
                //cmd.Parameters.Add("P_DSP_ZONE", OracleDbType.Varchar2).Value = pData.DSP_ZONE;
                //cmd.Parameters.Add("P_DSP_PRODUCT", OracleDbType.Varchar2).Value = pData.DSP_PRODUCT;
                cmd.Parameters.Add("P_ZONE_AREA", OracleDbType.Varchar2).Value = pData.ZONE_AREA;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ074MR_GET_ZONE_PROD");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSQ074MR_GET_ZONE_DESC")]
        [Authorize]
        public IHttpActionResult SNSQ074MR_GET_ZONE_DESC([FromBody] dynamic pData)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ074_MR_ZONEDESC";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PROCESS_YEAR", OracleDbType.Int32).Value = pData.PROCESS_YEAR;
                cmd.Parameters.Add("P_PROCESS_MONTH", OracleDbType.Int32).Value = pData.PROCESS_MONTH;
                //cmd.Parameters.Add("P_DSP_ZONE", OracleDbType.Varchar2).Value = pData.DSP_ZONE;
                //cmd.Parameters.Add("P_DSP_DESC", OracleDbType.Varchar2).Value = pData.DSP_DESC;
                cmd.Parameters.Add("P_ZONE_AREA", OracleDbType.Varchar2).Value = pData.ZONE_AREA;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ074MR_GET_ZONE_DESC");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSQ074MR_GET_ZONE_PART")]
        [Authorize]
        public IHttpActionResult SNSQ074MR_GET_ZONE_PART([FromBody] dynamic pData)
        {

            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ074_MR_ZONEPART";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PROCESS_YEAR", OracleDbType.Int32).Value = pData.PROCESS_YEAR;
                cmd.Parameters.Add("P_PROCESS_MONTH", OracleDbType.Int32).Value = pData.PROCESS_MONTH;
                //cmd.Parameters.Add("P_DSP_ZONE", OracleDbType.Varchar2).Value = pData.DSP_ZONE;
                cmd.Parameters.Add("P_ZONE_AREA", OracleDbType.Varchar2).Value = pData.ZONE_AREA;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ074MR_GET_ZONE_PART");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSQ074MR_GET_CUST_DESC")]
        [Authorize]
        public IHttpActionResult SNSQ074MR_GET_CUST_DESC([FromBody] dynamic pData)
        {

            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ074_MR_CUSTDESC";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PROCESS_YEAR", OracleDbType.Int32).Value = pData.PROCESS_YEAR;
                cmd.Parameters.Add("P_PROCESS_MONTH", OracleDbType.Int32).Value = pData.PROCESS_MONTH;
                //cmd.Parameters.Add("P_DSP_CUSTOMER", OracleDbType.Varchar2).Value = pData.DSP_CUSTOMER;
                //cmd.Parameters.Add("P_DSP_DESC", OracleDbType.Varchar2).Value = pData.DSP_DESC;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ074MR_GET_CUST_DESC");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSQ074MR_GET_CUST_PART")]
        [Authorize]
        public IHttpActionResult SNSQ074MR_GET_CUST_PART([FromBody] dynamic pData)
        {

            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ074_MR_CUSTPART";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PROCESS_YEAR", OracleDbType.Int32).Value = pData.PROCESS_YEAR;
                cmd.Parameters.Add("P_PROCESS_MONTH", OracleDbType.Int32).Value = pData.PROCESS_MONTH;
                //cmd.Parameters.Add("P_DSP_CUSTOMER", OracleDbType.Varchar2).Value = pData.DSP_CUSTOMER;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ074MR_GET_CUST_DESC");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSQ074MR_GET_PROD_PART")]
        [Authorize]
        public IHttpActionResult SNSQ074MR_GET_PROD_PART([FromBody] dynamic pData)
        {

            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ074_MR_PRODPART";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PROCESS_YEAR", OracleDbType.Int32).Value = pData.PROCESS_YEAR;
                cmd.Parameters.Add("P_PROCESS_MONTH", OracleDbType.Int32).Value = pData.PROCESS_MONTH;
                //cmd.Parameters.Add("P_DSP_PRODUCT", OracleDbType.Varchar2).Value = pData.DSP_PRODUCT;
                cmd.Parameters.Add("P_PRODUCT_CODE", OracleDbType.Varchar2).Value = pData.PRODUCT_CODE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ074MR_GET_PROD_PART");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSQ074MR_GET_PROD_SALES")]
        [Authorize]
        public IHttpActionResult SNSQ074MR_GET_PROD_SALES([FromBody] dynamic pData)
        {

            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ074_MR_PRODSALES";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PROCESS_YEAR", OracleDbType.Int32).Value = pData.PROCESS_YEAR;
                cmd.Parameters.Add("P_PROCESS_MONTH", OracleDbType.Int32).Value = pData.PROCESS_MONTH;
                //cmd.Parameters.Add("P_DSP_PRODUCT", OracleDbType.Varchar2).Value = pData.DSP_PRODUCT;
                //cmd.Parameters.Add("P_DSP_SALES_CHANNEL", OracleDbType.Varchar2).Value = pData.DSP_SALES_CHANNEL;
                cmd.Parameters.Add("P_PRODUCT_CODE", OracleDbType.Varchar2).Value = pData.PRODUCT_CODE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ074MR_GET_PROD_SALES");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        #endregion

        #region SNSQ005_W

        [HttpGet, Route("SNSQ005_W_GET_PO/{PoNo=null}/{NoOfShowRecords=null}")]
        [Authorize]
        public IHttpActionResult SNSQ005_W_GET_PO(string PoNo = null, string NoOfShowRecords = null)
        {
            try
            {
                DataSet ds = new DataSet();
                PoNo = ismoUtil.escapeCharacter(PoNo);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ005W_GET_PO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PO_NO", OracleDbType.Varchar2).Value = PoNo.ToUpper() == "NULL" ? (object)DBNull.Value : PoNo;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = NoOfShowRecords;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ005_W_GET_PO");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));

            }
        }

        [HttpGet, Route("SNSQ005_W_GET_VENDOR_CODE/{VendorCode=null}/{NoOfShowRecords=null}")]
        [Authorize]
        public IHttpActionResult SNSQ005_W_GET_VENDOR_CODE(string VendorCode = null, string NoOfShowRecords = null)
        {
            try
            {
                DataSet ds = new DataSet();
                VendorCode = ismoUtil.escapeCharacter(VendorCode);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ005W_GET_VENDOR_CODE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = VendorCode.ToUpper() == "NULL" ? (object)DBNull.Value : VendorCode;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = NoOfShowRecords;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ005_W_GET_VENDOR_CODE");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));

            }
        }

        [HttpPost, Route("SNSQ005W_GET_DATA")]
        [Authorize]
        public IHttpActionResult SNSQ005W_GET_DATA([FromBody] dynamic pData)
        {

            try
            {

                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ005W_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PO_NO", OracleDbType.Varchar2).Value = pData.PO_NO;
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData.VENDOR_CODE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ005W_GET_DATA");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        #endregion

        #region SNM125P

        [HttpGet, Route("SNM125P_GET_PART_NO/{PartNo=null}/{NoOfShowRecords=null}")]
        [Authorize]
        public IHttpActionResult SNM125P_GET_PART_NO(string PartNo = null, string NoOfShowRecords = null)
        {
            try
            {
                DataSet ds = new DataSet();
                PartNo = ismoUtil.escapeCharacter(PartNo);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM125P_GET_PART_NO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = PartNo.ToUpper() == "NULL" ? (object)DBNull.Value : PartNo;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = NoOfShowRecords;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM125P_GET_PART_NO");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));

            }
        }

        [HttpGet, Route("SNM125P_GET_PART_CODE/{PartCode=null}/{NoOfShowRecords=null}")]
        [Authorize]
        public IHttpActionResult SNM125P_GET_PART_CODE(string PartCode = null, string NoOfShowRecords = null)
        {
            try
            {
                DataSet ds = new DataSet();
                PartCode = ismoUtil.escapeCharacter(PartCode);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM125P_GET_PART_CODE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_CODE", OracleDbType.Varchar2).Value = PartCode.ToUpper() == "NULL" ? (object)DBNull.Value : PartCode;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = NoOfShowRecords;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM125P_GET_PART_CODE");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));

            }
        }

        [HttpGet, Route("SNM125P_GET_PART_CODE_ADD/{PartNo=null}/{NoOfShowRecords=null}")]
        [Authorize]
        public IHttpActionResult SNM125P_GET_PART_CODE_ADD(string PartNo = null, string NoOfShowRecords = null)
        {
            try
            {
                DataSet ds = new DataSet();
                PartNo = ismoUtil.escapeCharacter(PartNo);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM125P_GET_PART_NO_ADD";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = PartNo.ToUpper() == "NULL" ? (object)DBNull.Value : PartNo;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = NoOfShowRecords;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM125P_GET_PART_CODE_ADD");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));

            }
        }

        [HttpPost, Route("SNM125P_GET_DATA")]
        [Authorize]
        public IHttpActionResult SNM125P_GET_DATA([FromBody] dynamic pData)
        {

            try
            {

                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM125P_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pData.PART_NO;
                cmd.Parameters.Add("P_PART_CODE", OracleDbType.Varchar2).Value = pData.PART_CODE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM125P_GET_DATA");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM125P_INSERT")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM112P_INSERT([FromBody] dynamic pData)
        {
            try
            {

                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM125P_INSERT";

                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pData.PART_NO;
                cmd.Parameters.Add("P_PART_NAME_TH", OracleDbType.Varchar2).Value = pData.PART_NAME_TH;
                cmd.Parameters.Add("P_PART_CODE", OracleDbType.Varchar2).Value = pData.PART_CODE;
                cmd.Parameters.Add("P_PACKAGING_CODE", OracleDbType.Varchar2).Value = pData.PACKAGING_CODE;
                cmd.Parameters.Add("P_EXPENSIVE_CODE", OracleDbType.Varchar2).Value = pData.EXPENSIVE_CODE;
                cmd.Parameters.Add("P_ATTRIBUTE1", OracleDbType.Varchar2).Value = pData.ATTRIBUTE1;
                cmd.Parameters.Add("P_ATTRIBUTE2", OracleDbType.Varchar2).Value = pData.ATTRIBUTE2;
                cmd.Parameters.Add("P_ATTRIBUTE3", OracleDbType.Varchar2).Value = pData.ATTRIBUTE3;
                cmd.Parameters.Add("P_ATTRIBUTE4", OracleDbType.Varchar2).Value = pData.ATTRIBUTE4;
                cmd.Parameters.Add("P_ATTRIBUTE5", OracleDbType.Varchar2).Value = pData.ATTRIBUTE5;
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM125P";
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_IP_ADDR", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM125P_INSERT");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM125P_Update")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM125P_Update([FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM125P_UPDATE";

                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pData.PART_NO;
                cmd.Parameters.Add("P_PART_NAME_TH", OracleDbType.Varchar2).Value = pData.PART_NAME_TH;
                cmd.Parameters.Add("P_PART_CODE", OracleDbType.Varchar2).Value = pData.PART_CODE;
                cmd.Parameters.Add("P_PACKAGING_CODE", OracleDbType.Varchar2).Value = pData.PACKAGING_CODE;
                cmd.Parameters.Add("P_EXPENSIVE_CODE", OracleDbType.Varchar2).Value = pData.EXPENSIVE_CODE;
                cmd.Parameters.Add("P_ATTRIBUTE1", OracleDbType.Varchar2).Value = pData.ATTRIBUTE1;
                cmd.Parameters.Add("P_ATTRIBUTE2", OracleDbType.Varchar2).Value = pData.ATTRIBUTE2;
                cmd.Parameters.Add("P_ATTRIBUTE3", OracleDbType.Varchar2).Value = pData.ATTRIBUTE3;
                cmd.Parameters.Add("P_ATTRIBUTE4", OracleDbType.Varchar2).Value = pData.ATTRIBUTE4;
                cmd.Parameters.Add("P_ATTRIBUTE5", OracleDbType.Varchar2).Value = pData.ATTRIBUTE5;
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM125P";
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_IP_ADDR", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM125P_Update");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNM125P_DELETE")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM112P_DELETE([FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM125P_DELETE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pData.PART_NO;
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM125P";
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_IP_ADDR", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM125P_DELETE");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNM125P_GetDangerousCode")]
        [Authorize]
        public IHttpActionResult SNM125P_GetDangerousCode()
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM125P_GET_DANGEROUS_CODE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM125P_GetDangerousCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        #endregion

        #region RUNBAT1
        [HttpPost, Route("RUNBAT1_Process")]
        [Authorize]
        public IHttpActionResult RUNBAT1_Process()
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_RUNBAT1_BEGIN_PROCESS";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                if (ds.Tables.Count > 0)
                {
                    DataTable dt = ds.Tables[0];
                    var returnJson = new
                    {
                        TotalOrder = dt.Rows[0]["TOTAL_ORDER"],
                        TotalPart = dt.Rows[0]["TOTAL_PART"],
                        TotalAllocate = dt.Rows[0]["TOTAL_ALLOCATE"]
                    };

                    return Ok(new { status = "Done", data = returnJson });
                }
                else
                {
                    return Ok(new { status = "Fail", message = "Found eror, please contact support team." });
                }

            }
            catch (OracleException ex)
            {
                oOracledb.WriteLog("OracleException", ex.Message, "RUNBAT1_Process");

                if (ex.Number == 20001)
                {
                    return Ok(new { status = "Fail", message = "Can not start Allocate case start_date and end_date is not null" });
                }
                else if (ex.Number == 20002)
                {
                    return Ok(new { status = "Fail", message = "Can not start Allocate case start_date is not null" });
                }
                else if (ex.Number == 20003)
                {
                    return Ok(new { status = "Fail", message = "Can not start Allocate case status != R" });
                }

                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "RUNBAT1_Process");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        #endregion

        #region RUNBAT1_BR
        [HttpPost, Route("RUNBAT1_BR_Process")]
        [Authorize]
        public IHttpActionResult RUNBAT1_BR_Process()
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_RUNBAT1_BR_BEGIN_PROCESS";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                oOracledb.SqlQueryDS_CMD(cmd);

                return Ok(new { status = "Done" });
            }
            catch (OracleException ex)
            {
                oOracledb.WriteLog("OracleException", ex.Message, "RUNBAT1_BR_Process");

                if (ex.Number == 20001)
                {
                    return Ok(new { status = "Fail", message = "Can not start Allocate case start_date and end_date is not null" });
                }
                else if (ex.Number == 20002)
                {
                    return Ok(new { status = "Fail", message = "Can not start Allocate case start_date is not null" });
                }
                else if (ex.Number == 20003)
                {
                    return Ok(new { status = "Fail", message = "Can not start Allocate case status != R" });
                }

                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "RUNBAT1_BR_Process");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        #endregion

        #region RUNBAT1_W
        [HttpPost, Route("RUNBAT1_W_Process")]
        [Authorize]
        public IHttpActionResult RUNBAT1_W_Process()
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_RUNBAT1_W_BEGIN_PROCESS";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                if (ds.Tables.Count > 0)
                {
                    DataTable dt = ds.Tables[0];
                    var returnJson = new
                    {
                        TotalOrder = dt.Rows[0]["TOTAL_ORDER"],
                        TotalPart = dt.Rows[0]["TOTAL_PART"],
                        TotalAllocate = dt.Rows[0]["TOTAL_ALLOCATE"]
                    };
                    return Ok(new { status = "Done", data = returnJson });
                }
                else
                {
                    return Ok(new { status = "Fail", message = "Found eror, please contact support team." });
                }
            }
            catch (OracleException ex)
            {
                oOracledb.WriteLog("OracleException", ex.Message, "RUNBAT1_W_Process");
                if (ex.Number == 20001)
                {
                    return Ok(new { status = "Fail", message = "Can not start Allocate case start_date and end_date is not null" });
                }
                else if (ex.Number == 20002)
                {
                    return Ok(new { status = "Fail", message = "Can not start Allocate case start_date is not null" });
                }
                else if (ex.Number == 20003)
                {
                    return Ok(new { status = "Fail", message = "Can not start Allocate case status != R" });
                }
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "RUNBAT1_W_Process");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        #endregion
        #region SNM032E

        [HttpPost, Route("SNM032E_GetExchange_Rate")]
        [Authorize]
        public IHttpActionResult SNM032E_GetExchange_Rate([FromBody] dynamic pDataList)
        {
            string[] RateType = new string[0];
            if (pDataList["RATE_TYPE"] != null)
            {
                RateType = pDataList["RATE_TYPE"].ToString().Split(':');
            }
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM032E_GET_EXC_RATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                //cmd.Parameters.Add("P_EFFECTIVE_DATE", OracleDbType.Varchar2).Value = Convert.ToDateTime(pDataList["EFFECTIVE_DATE"]).ToString("dd-MMM-yyyy").Trim() ;
                cmd.Parameters.Add("P_EFFECTIVE_DATE", OracleDbType.Date).Value = pDataList["EFFECTIVE_DATE"] != null ? Convert.ToDateTime(pDataList["EFFECTIVE_DATE"]) : (Object)DBNull.Value;

                cmd.Parameters.Add("P_RATE_TYPE", OracleDbType.Varchar2).Value = RateType.Length > 0 && RateType[0].ToString() != "" ? RateType[0].Trim() : "";
                cmd.Parameters.Add("P_CURRENCY_CODE", OracleDbType.Varchar2).Value = pDataList["CURRENCY_CODE"];
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM032E_GetExchange_Rate");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNM032E_Get_Rate_Type/{RateType}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM032E_Get_Rate_Type(string RateType, int noofshowrec)
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM032P_GET_RATE_TYPE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_RATE_TYPE", OracleDbType.Varchar2).Value = RateType != "null" ? RateType : null;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM032E_Get_Rate_Type");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNM032E_Get_Rate_Type_Add/{RateType}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM032E_Get_Rate_Type_Add(string RateType, int noofshowrec)
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM032E_GET_RATE_TYPE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_RATE_TYPE", OracleDbType.Varchar2).Value = RateType != "null" ? RateType : null;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM032E_Get_Rate_Type_Add");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNM032E_Get_Currency/{CurrencyCode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM032E_Get_Currency(string CurrencyCode, int noofshowrec)
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM032E_GET_CURRENCY";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CURRENCY_CODE", OracleDbType.Varchar2).Value = CurrencyCode != "null" ? CurrencyCode : null;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM032E_Get_Currency");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM032E_Exchange_Rate_Save")]
        [Authorize]
        public IHttpActionResult SNM032E_Exchange_Rate_Save([FromBody] dynamic pDataList)
        {
            string[] RateType = new string[0];
            if (pDataList["RATE_TYPE_DESC"] != null)
            {
                RateType = pDataList["RATE_TYPE_DESC"].ToString().Split(':');
            }
            DataSet ds = new DataSet();
            // DataTable dt = new DataTable();
            // var x = HttpContext.Current.Request.UserHostAddress.ToString();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM032E_SAVE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_EFFECTIVE_DATE", OracleDbType.Date).Value = pDataList["EFFECTIVE_DATE"];
                cmd.Parameters.Add("P_RATE_TYPE", OracleDbType.Varchar2).Value = RateType.Length > 0 && RateType[0].ToString() != "" ? RateType[0].Trim() : "";
                cmd.Parameters.Add("P_CURRENCY_CODE", OracleDbType.Varchar2).Value = pDataList["CURRENCY_CODE"];
                cmd.Parameters.Add("P_EXCHANGE_RATE", OracleDbType.Decimal).Value = pDataList["EXCHANGE_RATE"];
                cmd.Parameters.Add("P_EXCHANGE_UNIT", OracleDbType.Decimal).Value = pDataList["EXCHANGE_UNIT"];
                cmd.Parameters.Add("P_CREATE_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM032E";
                cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_LAST_IP_ADDR", OracleDbType.Varchar2).Value = pDataList["LAST_IP_ADDR"];
                cmd.Parameters.Add("P_MOD_EDIT", OracleDbType.Varchar2).Value = pDataList["ModEdit"] == true ? "Y" : "N";
                cmd.Parameters.Add("C_RE", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM032E_Exchange_Rate_Save");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM032E_GetExchange_Delete")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM032E_GetExchange_Delete([FromBody] dynamic pDataList)
        {
            DataSet ds = new DataSet();
            // DataTable dt = new DataTable();
            //  var x = HttpContext.Current.Request.UserHostAddress.ToString();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM032P_DELETE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_EFFECTIVE_DATE", OracleDbType.Date).Value = pDataList["EFFECTIVE_DATE"];
                cmd.Parameters.Add("P_RATE_TYPE", OracleDbType.Varchar2).Value = pDataList["RATE_TYPE"];
                cmd.Parameters.Add("P_CURRENCY_CODE", OracleDbType.Varchar2).Value = pDataList["CURRENCY_CODE"];
                cmd.Parameters.Add("P_EXCHANGE_RATE", OracleDbType.Varchar2).Value = pDataList["EXCHANGE_RATE"];
                cmd.Parameters.Add("P_EXCHANGE_UNIT", OracleDbType.Varchar2).Value = pDataList["EXCHANGE_UNIT"];
                cmd.Parameters.Add("P_CREATE_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM032E";
                cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_LAST_IP_ADDR", OracleDbType.Varchar2).Value = pDataList["LAST_IP_ADDR"];
                cmd.Parameters.Add("C_RE", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM032E_GetExchange_Delete");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }



        [HttpPost, Route("SNM032E_Upload")]
        [Authorize]
        public async Task<IHttpActionResult> SNM032E_Upload(HttpRequestMessage request)
        {

            if (!request.Content.IsMimeMultipartContent())
            {
                throw new HttpResponseException(HttpStatusCode.UnsupportedMediaType);
            }

            var reqParts = await request.Content.ReadAsMultipartAsync();
            List<string> ErrorList = new List<string>();
            try
            {


                string fileLocation = string.Empty;
                foreach (var file in reqParts.Contents)
                {
                    var buffer = await file.ReadAsByteArrayAsync();
                    string fileName = Path.GetFileNameWithoutExtension(file.Headers.ContentDisposition.FileName.Trim('\"'));
                    string ext = Path.GetExtension(file.Headers.ContentDisposition.FileName.Trim('\"'));
                    string FullfileName = String.Concat(fileName, "-", DateTime.Now.ToString("yyyyMMddHHmm"), ext);
                    string outputLocation = "~/tempFile/" + "SNM032E" + "/";
                    string outputPath = System.Web.HttpContext.Current.Server.MapPath(outputLocation);
                    fileLocation = String.Concat(outputPath, FullfileName);

                    DataService.SaveBytesToFileByLocation(buffer
                                                    , fileLocation
                                                    , fileLocation);
                }


                ErrorList = SNM032E_InsertDataFromExcelFile(fileLocation);
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM032E_Upload");
                throw new HttpResponseException(HttpStatusCode.NotAcceptable);
            }

            return Json(new { Error = ErrorList });
        }


        public List<string> SNM032E_InsertDataFromExcelFile(string fileLocation)
        {

            string fileLocationN = fileLocation;
            DataTable dt = new DataTable();
            SystemApiController sys = new SystemApiController();
            dt = sys.ReadDataExcel(fileLocationN);
            List<string> ErrList = new List<string>();
            int countrow = 1;
            foreach (DataRow row in dt.Rows)
            {

                //bool isDecimal_Type1 = CheckDecimal(row["Transfer(Type1)"].ToString().Trim());
                bool isDecimal_Type2 = CheckDecimal(row["Export exchange rate"].ToString().Trim());
                int n;
                bool isNumeric = int.TryParse(row["Exchange unit"].ToString().Trim(), out n);
                DateTime d;
                bool isDatetime = DateTime.TryParse(row["EFFECTIVE_DATE(DD-MMM-YYYY)"].ToString().Trim(), out d);

                if (isDecimal_Type2 && isNumeric && isDatetime)
                {


                    OracleCommand cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNM032E_SAVE_FROMFILE";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_EFFECTIVE_DATE", OracleDbType.Varchar2).Value = Convert.ToDateTime(row["EFFECTIVE_DATE(DD-MMM-YYYY)"]).ToString("ddMMyyyy").Trim();
                    //cmd.Parameters.Add("P_RATE_TYPE_1", OracleDbType.Varchar2).Value = "1";
                    cmd.Parameters.Add("P_CURRENCY_CODE", OracleDbType.Varchar2).Value = row["Currency"].ToString().Trim();
                    // cmd.Parameters.Add("P_EXCHANGE_RATE_TYPE1", OracleDbType.Varchar2).Value = row["Transfer(Type1)"].ToString().Trim();
                    cmd.Parameters.Add("P_EXPORT_EXC_RATE", OracleDbType.Varchar2).Value = row["Export exchange rate"].ToString().Trim();
                    cmd.Parameters.Add("P_EXCHANGE_UNIT", OracleDbType.Varchar2).Value = row["Exchange unit"].ToString().Trim();
                    cmd.Parameters.Add("P_CREATE_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM032E";
                    cmd.Parameters.Add("P_LAST_IP_ADDR", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                    cmd.Parameters.Add("P_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;

                    cmd = oOracledb.SqlQueryReturnValue(cmd);

                    string status = cmd.Parameters["P_STATUS"].Value.ToString();
                    //if(status == "DONE")
                    //{
                    //    ErrList.Add("Row " + countrow + " Upload complete");
                    //}
                    if (status == "DUPLICATE")
                    {

                        ErrList.Add("Row " + countrow + " " + "Currency Code :" + " " + row["Currency"].ToString().Trim() + " dupplicate record in data base");
                    }
                    else if (status == "FAIL")
                    {
                        ErrList.Add("Row " + countrow + " " + "Currency Code :" + " " + row["Currency"].ToString().Trim() + " error please recheck");
                    }

                }
                else
                {
                    ErrList.Add("Row " + countrow + " " + "Currency Code :" + " " + row["Currency"].ToString().Trim() + " error please recheck");
                }

                countrow++;
            }

            return ErrList;

        }

        public bool CheckDecimal(string value)
        {

            try
            {
                Decimal.Parse(value);
                if (CountDecimalDigits(decimal.Parse(value)) <= 5)
                {
                    return true;
                }
                else
                {
                    return false;
                }

            }
            catch
            {
                return false;
            }
        }

        public int CountDecimalDigits(decimal n)
        {
            return n.ToString(System.Globalization.CultureInfo.InvariantCulture)
                    //.TrimEnd('0') uncomment if you don't want to count trailing zeroes
                    .SkipWhile(c => c != '.')
                    .Skip(1)
                    .Count();
        }

        #endregion

        #region SNM123


        [HttpGet, Route("SNM123_GetCustomer/{cusID}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM123_GetCustomer(string cusID, int noofshowrec)
        {
            DataSet ds = new DataSet();

            cusID = ismoUtil.escapeCharacter(cusID);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM123_GET_CUSTOMER";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = cusID.ToUpper() == "NULL" ? (object)DBNull.Value : cusID;
            cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }


        [HttpGet, Route("SNM123_GetModelConfigure")]
        [Authorize]
        public IHttpActionResult SNM123_GetModelConfigure()
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM123_GET_MODEL_CONFIG";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM123_GetModelConfigure");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM123_GetData")]
        [Authorize]
        public IHttpActionResult SNM123_GetData([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();

            try
            {
                cmd.CommandText = "SP_NP_SNM123_GET_NO_PRICE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pParam.ORDER_NO;
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pParam.PART_NO;
                cmd.Parameters.Add("P_FILTER_NO", OracleDbType.Varchar2).Value = pParam.FILTER_NO;

                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM123_GetData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpPost, Route("SNM123_GetExportPriceInfo")]
        [Authorize]
        public IHttpActionResult SNM123_GetExportPriceInfo([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();

            try
            {
                cmd.CommandText = "SP_NP_SNM123_EXPORT_PRICE_INFO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pParam.PART_NO;
                cmd.Parameters.Add("P_ORDER_DATE", OracleDbType.Date).Value = pParam.ORDER_DATE;
                cmd.Parameters.Add("P_RECAL_FLAG", OracleDbType.Varchar2).Value = pParam.RECAL_FLAG;

                cmd.Parameters.Add("P_CFG_ACCESSORIES", OracleDbType.Int32).Value = pParam.CFG_ACCESSORIES;
                cmd.Parameters.Add("P_CFG_COUNTERMESURE", OracleDbType.Int32).Value = pParam.CFG_COUNTERMESURE;
                cmd.Parameters.Add("P_CFG_PRODUCTION_COUNTRY", OracleDbType.Int32).Value = pParam.CFG_PRODUCTION_COUNTRY;
                cmd.Parameters.Add("P_CFG_LOGISTICS", OracleDbType.Int32).Value = pParam.CFG_LOGISTICS;
                cmd.Parameters.Add("P_CFG_PROCUREMENT", OracleDbType.Int32).Value = pParam.CFG_PROCUREMENT;
                cmd.Parameters.Add("P_CFG_FIX", OracleDbType.Int32).Value = pParam.CFG_FIX;
                //cmd.Parameters.Add("V_MSG", OracleDbType.Varchar2, 900).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                //cmd = oOracledb.SqlQueryReturnValue(cmd);

                // string status = cmd.Parameters["P_STATUS"].Value.ToString();


                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM123_GetExportPriceInfo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }


        [HttpPost, Route("SNM123_InsertSettingTemp")]
        [Authorize]
        public IHttpActionResult SNM123_InsertSettingTemp([FromBody] dynamic pParam)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM123_SP_INSERT_TEMP";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pParam.PART_NO;
                cmd.Parameters.Add("P_CURRENCY_CODE", OracleDbType.Varchar2).Value = pParam.CURRENCY_CODE;
                cmd.Parameters.Add("P_NEW_SETTING", OracleDbType.Varchar2).Value = pParam.NEW_SETTING;
                cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM123_InsertSettingTemp");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNM123_Save")]
        [Authorize]
        public IHttpActionResult SNM123_Save([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();


            try
            {
                cmd.CommandText = "SP_NP_SNM123_SAVE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);

                cmd.Parameters.Add("P_EFFECTIVE_DATE", OracleDbType.Date).Value = pParam.EFFECTIVE_DATE;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pParam.PART_NO;
                cmd.Parameters.Add("P_CURRENCY_CODE", OracleDbType.Varchar2).Value = pParam.CURRENCY_CODE;
                cmd.Parameters.Add("P_NEW_PRICE", OracleDbType.Decimal).Value = pParam.NEW_PRICE;
                // cmd.Parameters.Add("P_FORMER_NET", OracleDbType.Decimal).Value = pParam.EXPORT_PRICE;
                cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                // cmd.Parameters.Add("P_IP_ADDRESS", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress;
                cmd.Parameters.Add("P_FILTER_NO", OracleDbType.Varchar2).Value = pParam.FILTER_NO;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM123_Save");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpPost, Route("SNM123_GetUploadData")]
        [Authorize]
        public IHttpActionResult SNM123_GetUploadData([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();

            try
            {
                cmd.CommandText = "SP_NP_SNM123_SP_GET_UPLOAD";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CREATE_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM123_GetUploadData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNM123_Upload")]
        [Authorize]
        public async Task<IHttpActionResult> SNM123_Upload(HttpRequestMessage request)
        {

            if (!request.Content.IsMimeMultipartContent())
            {
                throw new HttpResponseException(HttpStatusCode.UnsupportedMediaType);
            }

            var reqParts = await request.Content.ReadAsMultipartAsync();
            List<string> ErrorList = new List<string>();
            try
            {
                string fileLocation = string.Empty;
                foreach (var file in reqParts.Contents)
                {
                    var buffer = await file.ReadAsByteArrayAsync();
                    string fileName = Path.GetFileNameWithoutExtension(file.Headers.ContentDisposition.FileName.Trim('\"'));
                    string ext = Path.GetExtension(file.Headers.ContentDisposition.FileName.Trim('\"'));
                    string FullfileName = String.Concat(fileName, "-", DateTime.Now.ToString("yyyyMMddHHmm"), ext);
                    string outputLocation = "~/tempFile/" + "SNM123" + "/Suplement/";
                    string outputPath = System.Web.HttpContext.Current.Server.MapPath(outputLocation);
                    fileLocation = String.Concat(outputPath, FullfileName);

                    DataService.SaveBytesToFileByLocation(buffer
                                                    , fileLocation
                                                    , fileLocation);
                }
                ErrorList = SNM123_InsertDataFromExcelFile(fileLocation);
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM123_Upload");
                throw new HttpResponseException(HttpStatusCode.NotAcceptable);
            }

            return Json(new { Error = ErrorList });
        }


        public List<string> SNM123_InsertDataFromExcelFile(string fileLocation)
        {

            string fileLocationN = fileLocation;
            DataTable dt = new DataTable();
            SystemApiController sys = new SystemApiController();
            dt = sys.ReadDataExcel(fileLocationN);
            List<string> ErrList = new List<string>();
            int countrow = 1;
            OracleCommand cmd = new OracleCommand();
            DataSet ds = new DataSet();

            try
            {
                cmd.CommandText = "SP_NP_SNM123_SP_INIT_UPLOAD";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CREATE_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM123_InsertDataFromExcelFile");
                ErrList.Add("Error : SP_NP_SNM123_Init_Upload please recheck");
                return ErrList;
            }


            foreach (DataRow row in dt.Rows)
            {

                DateTime d;
                bool isDatetime = DateTime.TryParse(row["EFFECTIVE_DATE"].ToString().Trim(), out d);

                if (isDatetime)
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNM123_SP_SAVE_FROMFILE";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_RNO", OracleDbType.Varchar2).Value = row["NO"].ToString().Trim();
                    cmd.Parameters.Add("P_EFFECTIVE_DATE", OracleDbType.Varchar2).Value = Convert.ToDateTime(row["EFFECTIVE_DATE"]).ToString("dd-MMM-yyyy").Trim();
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = row["CUSTOMER_CODE"].ToString().Trim();
                    cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = row["PART_NO"].ToString().Trim();
                    cmd.Parameters.Add("P_CREATE_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_LAST_IP_ADDR", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                    cmd.Parameters.Add("P_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;

                    cmd = oOracledb.SqlQueryReturnValue(cmd);

                    string status = cmd.Parameters["P_STATUS"].Value.ToString();

                    if (status == "DUPLICATE")
                    {
                        ErrList.Add("Row " + countrow + " " + "Part no. :" + " " + row["PART_NO"].ToString().Trim() + " dupplicate record in data base");
                    }
                    else if (status == "FAIL")
                    {
                        ErrList.Add("Row " + countrow + " " + "Part no. :" + " " + row["PART_NO"].ToString().Trim() + " error please recheck");
                    }

                }
                else
                {
                    ErrList.Add("Row " + countrow + " " + "Part no :" + " " + row["PART_NO"].ToString().Trim() + " error please recheck");
                }

                countrow++;
            }

            return ErrList;

        }


        [HttpPost, Route("SNM123_SaveToTemp")]
        [Authorize]
        public IHttpActionResult SNM123_SaveToTemp([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();

            try
            {
                cmd.CommandText = "SP_NP_SNM123_SP_SAVE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);

                cmd.Parameters.Add("P_EFFECTIVE_DATE", OracleDbType.Varchar2).Value = pParam.EFFECTIVE_DATE.ToString("dd-MMM-yyyy");
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pParam.PART_NO;

                cmd.Parameters.Add("P_CFG_ACCESSORIES", OracleDbType.Int32).Value = pParam.CFG_ACCESSORIES;
                cmd.Parameters.Add("P_CFG_COUNTERMESURE", OracleDbType.Int32).Value = pParam.CFG_COUNTERMESURE;
                cmd.Parameters.Add("P_CFG_PRODUCTION_COUNTRY", OracleDbType.Int32).Value = pParam.CFG_PRODUCTION_COUNTRY;
                cmd.Parameters.Add("P_CFG_LOGISTICS", OracleDbType.Int32).Value = pParam.CFG_LOGISTICS;
                cmd.Parameters.Add("P_CFG_PROCUREMENT", OracleDbType.Int32).Value = pParam.CFG_PROCUREMENT;
                cmd.Parameters.Add("P_CFG_FIX", OracleDbType.Int32).Value = pParam.CFG_FIX;

                cmd.Parameters.Add("P_CREATE_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                cmd.Parameters.Add("P_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string status = cmd.Parameters["P_STATUS"].Value.ToString();

                return Json(new { Result = status });

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM123_SaveToTemp");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM123_DeleteUploadSP")]
        [Authorize]
        public IHttpActionResult SNM123_DeleteUploadSP([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();

            try
            {
                cmd.CommandText = "SP_NP_SNM123_SP_DELETE_TEMP";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_EFFECTIVE_DATE", OracleDbType.Varchar2).Value = pParam.EFFECTIVE_DATE.ToString("dd-MM-yyyy");
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pParam.PART_NO;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM123_DeleteUploadSP");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNM123_Annual_GetUploadData")]
        [Authorize]
        public IHttpActionResult SNM123_Annual_GetUploadData([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();

            try
            {
                cmd.CommandText = "SP_NP_SNM123_AN_GET_UPLOAD";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_CREATE_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM123_Annual_GetUploadData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }


        [HttpPost, Route("SNM123_Annual_Upload")]
        [Authorize]
        public async Task<IHttpActionResult> SNM123_Annual_Upload(HttpRequestMessage request)
        {

            if (!request.Content.IsMimeMultipartContent())
            {
                throw new HttpResponseException(HttpStatusCode.UnsupportedMediaType);
            }

            var reqParts = await request.Content.ReadAsMultipartAsync();
            List<string> ErrorList = new List<string>();
            try
            {
                string fileLocation = string.Empty;
                foreach (var file in reqParts.Contents)
                {
                    var buffer = await file.ReadAsByteArrayAsync();
                    string fileName = Path.GetFileNameWithoutExtension(file.Headers.ContentDisposition.FileName.Trim('\"'));
                    string ext = Path.GetExtension(file.Headers.ContentDisposition.FileName.Trim('\"'));
                    string FullfileName = String.Concat(fileName, "-", DateTime.Now.ToString("yyyyMMddHHmm"), ext);
                    string outputLocation = "~/tempFile/" + "SNM123" + "/Annual/";
                    string outputPath = System.Web.HttpContext.Current.Server.MapPath(outputLocation);
                    fileLocation = String.Concat(outputPath, FullfileName);

                    DataService.SaveBytesToFileByLocation(buffer
                                                    , fileLocation
                                                    , fileLocation);
                }
                ErrorList = SNM123_Annual_InsertDataFromExcelFile(fileLocation);
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM123_Upload_Annual");
                throw new HttpResponseException(HttpStatusCode.NotAcceptable);
            }

            return Json(new { Error = ErrorList });
        }


        public List<string> SNM123_Annual_InsertDataFromExcelFile(string fileLocation)
        {

            string fileLocationN = fileLocation;
            DataTable dt = new DataTable();
            SystemApiController sys = new SystemApiController();
            dt = sys.ReadDataExcel(fileLocationN);
            List<string> ErrList = new List<string>();
            int countrow = 1;
            OracleCommand cmd = new OracleCommand();
            DataSet ds = new DataSet();

            try
            {
                cmd.CommandText = "SP_NP_SNM123_AN_INIT_UPLOAD";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CREATE_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM123_Annual_InsertDataFromExcelFile");
                ErrList.Add("Error : SP_NP_SNM123_Init_Upload please recheck");
                return ErrList;
            }


            foreach (DataRow row in dt.Rows)
            {

                DateTime d;
                bool isDatetime = DateTime.TryParse(row["EFFECTIVE_DATE"].ToString().Trim(), out d);

                if (isDatetime)
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNM123_AN_SAVE_FROMFILE";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_RNO", OracleDbType.Varchar2).Value = row["NO"].ToString().Trim();
                    cmd.Parameters.Add("P_EFFECTIVE_DATE", OracleDbType.Varchar2).Value = Convert.ToDateTime(row["EFFECTIVE_DATE"]).ToString("dd-MMM-yyyy").Trim();
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = row["CUSTOMER_CODE"].ToString().Trim();
                    cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = row["PART_NO"].ToString().Trim();
                    cmd.Parameters.Add("P_CREATE_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_LAST_IP_ADDR", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                    cmd.Parameters.Add("P_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;

                    cmd = oOracledb.SqlQueryReturnValue(cmd);

                    string status = cmd.Parameters["P_STATUS"].Value.ToString();

                    if (status == "DUPLICATE")
                    {
                        ErrList.Add("Row " + countrow + " " + "Part no. :" + " " + row["PART_NO"].ToString().Trim() + " dupplicate record in data base");
                    }
                    else if (status == "FAIL")
                    {
                        ErrList.Add("Row " + countrow + " " + "Part no. :" + " " + row["PART_NO"].ToString().Trim() + " error please recheck");
                    }

                }
                else
                {
                    ErrList.Add("Row " + countrow + " " + "Part no :" + " " + row["PART_NO"].ToString().Trim() + " error please recheck");
                }

                countrow++;
            }

            return ErrList;

        }


        [HttpPost, Route("SNM123_Annual_SaveToTemp")]
        [Authorize]
        public IHttpActionResult SNM123_Annual_SaveToTemp([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();

            try
            {
                cmd.CommandText = "SP_NP_SNM123_AN_SAVE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_EFFECTIVE_DATE", OracleDbType.Varchar2).Value = pParam.EFFECTIVE_DATE.ToString("dd-MMM-yyyy");
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pParam.PART_NO;
                cmd.Parameters.Add("P_CFG_ACCESSORIES", OracleDbType.Int32).Value = pParam.CFG_ACCESSORIES;
                cmd.Parameters.Add("P_CFG_COUNTERMESURE", OracleDbType.Int32).Value = pParam.CFG_COUNTERMESURE;
                cmd.Parameters.Add("P_CFG_PRODUCTION_COUNTRY", OracleDbType.Int32).Value = pParam.CFG_PRODUCTION_COUNTRY;
                cmd.Parameters.Add("P_CFG_LOGISTICS", OracleDbType.Int32).Value = pParam.CFG_LOGISTICS;
                cmd.Parameters.Add("P_CFG_PROCUREMENT", OracleDbType.Int32).Value = pParam.CFG_PROCUREMENT;
                cmd.Parameters.Add("P_CFG_FIX", OracleDbType.Int32).Value = pParam.CFG_FIX;
                cmd.Parameters.Add("P_CREATE_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string status = cmd.Parameters["P_STATUS"].Value.ToString();

                return Json(new { Result = status });

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM123_Annual_SaveToTemp");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM123_Annual_Fetch")]
        [Authorize]
        public IHttpActionResult SNM123_Annual_Fetch([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();

            try
            {
                cmd.CommandText = "SP_NP_SNM123_AN_PROCESS";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CALCULATE_TYPE", OracleDbType.Varchar2).Value = pParam.CALCULATE_TYPE;
                cmd.Parameters.Add("P_NON_CON_FLAG", OracleDbType.Varchar2).Value = pParam.NON_CON_FLAG;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_EFFECTIVE_DATE", OracleDbType.Varchar2).Value = pParam.EFFECTIVE_DATE.ToString("dd-MMM-yyyy");
                //cmd.Parameters.Add("P_MONTH", OracleDbType.Int16).Value = pParam.NO_OF_MONTH;

                cmd.Parameters.Add("P_CREATE_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                // cmd.Parameters.Add("P_LAST_IP_ADDR", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("P_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string status = cmd.Parameters["P_STATUS"].Value.ToString();

                return Json(new { Result = status });

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM123_Annual_Fetch");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM123_Annual_Clear")]
        [Authorize]
        public IHttpActionResult SNM123_Annual_Clear([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            try
            {
                cmd.CommandText = "SP_NP_SNM123_AN_CLEAR";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_EFFECTIVE_DATE", OracleDbType.Varchar2).Value = pParam.EFFECTIVE_DATE.ToString("dd-MMM-yyyy");
                cmd.Parameters.Add("P_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);
                string status = cmd.Parameters["P_STATUS"].Value.ToString();
                return Json(new { Result = status });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM123_Annual_Clear");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNM123_GetPartNo/{partNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM123_GetPartNo(string partNo = null, string noofshowrec = null)
        {
            try
            {
                DataSet ds = new DataSet();
                partNo = ismoUtil.escapeCharacter(partNo);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM123_GET_PART_NO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = partNo;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM123_GetPartNo");
                return Json(new { Status = "Error" });

            }
        }


        [HttpPost, Route("SNM123_GetGroupList")]
        [Authorize]
        public IHttpActionResult SNM123_GetGroupList([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();

            try
            {
                cmd.CommandText = "SP_NP_SNM123_AN_GET_CUST_GRP";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CREATE_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM123_GetGroupList");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }


        [HttpPost, Route("SNM123_Add_Model")]
        [Authorize]
        public IHttpActionResult SNM123_Add_Model([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            try
            {
                cmd.CommandText = "SP_NP_SNM123_ADD_MODEL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_MODEL_NAME", OracleDbType.Varchar2).Value = pParam.MODEL_NAME;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BASE_MODEL", OracleDbType.Varchar2).Value = pParam.BASE_MODEL;
                cmd.Parameters.Add("P_ITP_TYPE", OracleDbType.Varchar2).Value = pParam.ITP_TYPE;
                cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                //  return Json(DTFM.convertToList(ds));

                if (ds.Tables.Count > 0)
                {
                    if (ds.Tables[0].Rows[0]["STATUS"].ToString() != "DONE" && ds.Tables[0].Rows[0]["STATUS"].ToString() != "DUPLICATE")
                    {
                        string errString = "Error : " + ds.Tables[0].Rows[0]["STATUS"].ToString();
                        oOracledb.WriteLog("ErrorHandle", errString, "SNM123_Add_Model");
                        return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, errString));
                    }
                }
                else
                {
                    string errString = "Error : No data found";
                    oOracledb.WriteLog("ErrorHandle", errString, "SNM123_Add_Model");
                    return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, errString));
                }

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM123_Add_Model");
                throw new HttpResponseException(HttpStatusCode.NotAcceptable);
            }

            return Ok(ds.Tables[0].Rows[0]["STATUS"].ToString());
        }

        [HttpPost, Route("SNM123_Update_Model")]
        [Authorize]
        public IHttpActionResult SNM123_Update_Model([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();

            Console.WriteLine(pParam.CUSTOMER_CODE.ToString());
            try
            {
                cmd.CommandText = "SP_NP_SNM123_UPDATE_MODEL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_MODEL_NAME", OracleDbType.Varchar2).Value = pParam.MODEL_NAME;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BASE_MODEL", OracleDbType.Varchar2).Value = pParam.BASE_MODEL;
                cmd.Parameters.Add("P_ITP_TYPE", OracleDbType.Varchar2).Value = pParam.ITP_TYPE;
                cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM123_Update_Model");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM123_Delete_Model")]
        [Authorize]
        public IHttpActionResult SNM123_Delete_Model([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();

            //Console.WriteLine(pParam.CUSTOMER_CODE.ToString());
            try
            {
                cmd.CommandText = "SP_NP_SNM123_DELETE_MODEL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_MODEL_NAME", OracleDbType.Varchar2).Value = pParam.MODEL_NAME;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM123_Delete_Model");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNSE123_UploadFile")]
        [Authorize]
        public async Task<IHttpActionResult> SNSE123_UploadFile(HttpRequestMessage request)
        {
            if (!request.Content.IsMimeMultipartContent())
            {
                oOracledb.WriteLog("ErrorHandle", "UnsupportedMediaType", "SNSE123_UploadFile");
                throw new HttpResponseException(HttpStatusCode.UnsupportedMediaType);
            }
            int itemNo = 0;
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM123_PR_CLEAR_TEMP";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SP_NP_SNM123_CLEAR_TEMP");
            }

            try
            {
                var provider = new MultipartMemoryStreamProvider();
                var reqParts = await request.Content.ReadAsMultipartAsync(provider);

                foreach (var file in reqParts.Contents)
                {
                    var timestamp = DateTime.Now.ToString("yyyyMMdd");
                    var buffer = await file.ReadAsByteArrayAsync();
                    string gFileName = file.Headers.ContentDisposition.FileName.Trim('\"');

                    string gRootPath = ConfigurationManager.AppSettings["RootSharedPath"].ToString();
                    string gUploadLication = System.IO.Path.Combine(gRootPath, "tempFile/SNM123");
                    string gFileLocation = System.IO.Path.Combine(gUploadLication, gFileName);

                    DataService.SaveBytesToFileByLocation(buffer
                                                    , gUploadLication
                                                    , gFileLocation);

                    //Begin read file
                    Stream stream = new MemoryStream(buffer);
                    using (StreamReader reader = new StreamReader(stream, Encoding.UTF8))
                    {
                        string line;
                        int rno = 0;
                        while ((line = reader.ReadLine()) != null)
                        {
                            string sep = "\t";
                            string[] splitContent = line.Split(sep.ToCharArray());
                            var lenght = splitContent.Length;
                            rno++;

                            //line
                            //read line from memorystream
                            OracleCommand cmd = new OracleCommand();
                            cmd.CommandText = "SP_NP_SNM123_PR_EXTRACT_FILE";
                            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                            cmd.Parameters.Add("P_FILE_NAME", OracleDbType.Varchar2).Value = gFileName;
                            cmd.Parameters.Add("P_LINE_DATA", OracleDbType.Varchar2).Value = line;
                            cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                            cmd.Parameters.Add("P_RNO", OracleDbType.Varchar2).Value = rno;
                            cmd.Parameters.Add("P_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;

                            cmd = oOracledb.SqlQueryReturnValue(cmd);

                            string status = cmd.Parameters["P_STATUS"].Value.ToString();
                            if (status == "DONE")
                            {

                            }
                            else
                            {
                                oOracledb.WriteLog("ErrorHandle", status, "SP_NP_SNM123_EXTRACT_FILE");
                                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, status));
                            }

                        }
                    }

                    try
                    {
                        OracleCommand cmd = new OracleCommand();
                        cmd.CommandText = "SP_NP_SNM123_PR_GET_UPLOAD";
                        cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                        cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                        cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                        ds = oOracledb.SqlQueryDS_CMD(cmd);

                        return Json(DTFM.convertToList(ds));
                    }
                    catch (Exception ex)
                    {
                        oOracledb.WriteLog("ErrorHandle", ex.Message, "SP_NP_SNM123_CLEAR_TEMP");
                        return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
                    }
                }
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message + ' ' + itemNo, "SNSE123_UploadFile");
                throw new HttpResponseException(HttpStatusCode.NotAcceptable);
            }

            throw new Exception("Something wrong");

        }


        [HttpPost, Route("SNM123_Save_Confirm")]
        [Authorize]
        public IHttpActionResult SNM123_Save_Confirm([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            try
            {
                cmd.CommandText = "SP_NP_SNM123_SAVE_CONFIRM";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_FILE_NAME", OracleDbType.Varchar2).Value = pParam.FILE_NAME;
                cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string status = cmd.Parameters["P_STATUS"].Value.ToString();
                if (status == "DONE")
                {
                    return Ok("DONE");
                }
                else
                {
                    oOracledb.WriteLog("ErrorHandle", status, "SNM123_PR_Confirm");
                    return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, status));
                }

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM123_PR_Confirm");
                throw new HttpResponseException(HttpStatusCode.NotAcceptable);
            }


        }



        [HttpPost, Route("SNM123_PR_UploadExcel")]
        [Authorize]
        public async Task<IHttpActionResult> SNM123_PR_UploadExcel(HttpRequestMessage request)
        {

            if (!request.Content.IsMimeMultipartContent())
            {
                throw new HttpResponseException(HttpStatusCode.UnsupportedMediaType);
            }

            var reqParts = await request.Content.ReadAsMultipartAsync();
            List<string> ErrorList = new List<string>();
            try
            {
                string fileLocation = string.Empty;
                string gFileName = string.Empty;
                foreach (var file in reqParts.Contents)
                {
                    var buffer = await file.ReadAsByteArrayAsync();
                    string fileName = Path.GetFileNameWithoutExtension(file.Headers.ContentDisposition.FileName.Trim('\"'));
                    string ext = Path.GetExtension(file.Headers.ContentDisposition.FileName.Trim('\"'));
                    string FullfileName = String.Concat(fileName, "-", DateTime.Now.ToString("yyyyMMddHHmm"), ext);
                    string outputLocation = "~/tempFile/" + "SNM123" + "/Suplement/";
                    string outputPath = System.Web.HttpContext.Current.Server.MapPath(outputLocation);
                    fileLocation = String.Concat(outputPath, FullfileName);
                    gFileName = file.Headers.ContentDisposition.FileName.Trim('\"');

                    DataService.SaveBytesToFileByLocation(buffer
                                                    , fileLocation
                                                    , fileLocation);
                }
                ErrorList = SNM123_InsertDataPRFromExcelFile(fileLocation, gFileName);
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM123_PR_UploadExcel");
                throw new HttpResponseException(HttpStatusCode.NotAcceptable);
            }

            return Json(new { Error = ErrorList });
        }


        public List<string> SNM123_InsertDataPRFromExcelFile(string fileLocation, string gFileName)
        {

            string fileLocationN = fileLocation;
            string gFileNameN = gFileName;
            DataTable dt = new DataTable();
            SystemApiController sys = new SystemApiController();
            dt = sys.ReadDataExcel(fileLocationN);
            List<string> ErrList = new List<string>();
            int countrow = 1;
            OracleCommand cmd = new OracleCommand();
            DataSet ds = new DataSet();

            try
            {
                cmd.CommandText = "SP_NP_SNM123_PR_CLEAR_TEMP";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM123_InsertDataPRFromExcelFile");
                ErrList.Add("Error : SP_NP_SNM123_PR_CLEAR_TEMP please recheck");
                return ErrList;
            }


            foreach (DataRow row in dt.Rows)
            {

                DateTime d;
                bool isDatetime = DateTime.TryParse(row["EFFECTIVE_DATE"].ToString().Trim(), out d);

                if (isDatetime)
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNM123_PR_SAVE_FROMFILE";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_RNO", OracleDbType.Varchar2).Value = row["NO"].ToString().Trim();
                    cmd.Parameters.Add("P_EFFECTIVE_DATE", OracleDbType.Varchar2).Value = Convert.ToDateTime(row["EFFECTIVE_DATE"]).ToString("dd-MMM-yyyy").Trim();
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = row["CUSTOMER_CODE"].ToString().Trim();
                    cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = row["PART_NO"].ToString().Trim();
                    cmd.Parameters.Add("P_NET_PRICE", OracleDbType.Varchar2).Value = row["NET_PRICE"].ToString().Trim();
                    cmd.Parameters.Add("P_FILE_NAME", OracleDbType.Varchar2).Value = gFileNameN;
                    cmd.Parameters.Add("P_CREATE_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;

                    cmd = oOracledb.SqlQueryReturnValue(cmd);

                    string status = cmd.Parameters["P_STATUS"].Value.ToString();
                }
                else
                {
                    ErrList.Add("Row " + countrow + " " + "Part no :" + " " + row["PART_NO"].ToString().Trim() + " error please recheck");
                }

                countrow++;
            }

            return ErrList;

        }

        [HttpGet, Route("SNM123_PR_GetData")]
        [Authorize]
        public IHttpActionResult SNM123_PR_GetData()
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();

            try
            {
                cmd.CommandText = "SP_NP_SNM123_PR_GET_UPLOAD";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SP_NP_SNM123_PR_GET_UPLOAD");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM123_PR_GetData_Upload")]
        [Authorize]
        public IHttpActionResult SNM123_PR_GetData_Upload([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();

            try
            {
                cmd.CommandText = "SP_NP_SNM123_PR_GET_UPLOAD_LOG";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_FILE_NAME", OracleDbType.Varchar2).Value = pParam.FILE_NAME;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM123_PR_GetData_Upload");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM123_PR_Confirm_Upload")]
        [Authorize]
        public IHttpActionResult SNM123_PR_Confirm_Upload([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            try
            {
                cmd.CommandText = "SP_NP_SNM123_PR_CONFIRM_UPLOAD";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_FILE_NAME", OracleDbType.Varchar2).Value = pParam.FILE_NAME;
                cmd.Parameters.Add("P_USER_ID", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string status = cmd.Parameters["P_STATUS"].Value.ToString();
                if (status == "DONE")
                {
                    return Ok("DONE");
                }
                else
                {
                    oOracledb.WriteLog("ErrorHandle", status, "SNM123_PR_Confirm_Upload");
                    return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, status));
                }

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM123_PR_Confirm_Upload");
                throw new HttpResponseException(HttpStatusCode.NotAcceptable);
            }


        }
        #endregion

        #region SNM134P

        [HttpGet, Route("SNM134P_GET_PART_NO/{partNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM134P_GET_DATA_PART_NO(string partNo, int noofshowrec)
        {
            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM134P_GET_DATA_PART_NO";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = partNo;
            cmd.Parameters.Add("p_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }
        [HttpPost, Route("SNM134P_GET_DATA")]
        [Authorize]
        public IHttpActionResult SNM134P_GET_DATA([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();

            try
            {
                cmd.CommandText = "SP_NP_SNM134P_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pParam.PART_NO;
                cmd.Parameters.Add("P_PART_TYPE", OracleDbType.Varchar2).Value = pParam.PART_TYPE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM134P_GET_DATA");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNM134P_INSERT")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM134P_INSERT([FromBody] dynamic pData)
        {
            try
            {

                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM134P_INSERT";

                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pData.PART_NO;
                cmd.Parameters.Add("P_PART_TYPE", OracleDbType.Varchar2).Value = pData.PART_TYPE;
                cmd.Parameters.Add("P_REMARKS", OracleDbType.Varchar2).Value = pData.REMARKS;
                cmd.Parameters.Add("P_ATTRIBUTE1", OracleDbType.Varchar2).Value = pData.ATTRIBUTE1;
                cmd.Parameters.Add("P_ATTRIBUTE2", OracleDbType.Varchar2).Value = pData.ATTRIBUTE2;
                cmd.Parameters.Add("P_ATTRIBUTE3", OracleDbType.Varchar2).Value = pData.ATTRIBUTE3;
                cmd.Parameters.Add("P_ATTRIBUTE4", OracleDbType.Varchar2).Value = pData.ATTRIBUTE4;
                cmd.Parameters.Add("P_ATTRIBUTE5", OracleDbType.Varchar2).Value = pData.ATTRIBUTE5;

                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM134P";
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                //cmd.Parameters.Add("P_IP_ADDR", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM134P_INSERT");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNM134P_UPDATE")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM134P_UPDATE([FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM134P_UPDATE";

                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pData.PART_NO;
                cmd.Parameters.Add("P_PART_TYPE", OracleDbType.Varchar2).Value = pData.PART_TYPE;
                cmd.Parameters.Add("P_REMARKS", OracleDbType.Varchar2).Value = pData.REMARKS;
                cmd.Parameters.Add("P_ATTRIBUTE1", OracleDbType.Varchar2).Value = pData.ATTRIBUTE1;
                cmd.Parameters.Add("P_ATTRIBUTE2", OracleDbType.Varchar2).Value = pData.ATTRIBUTE2;
                cmd.Parameters.Add("P_ATTRIBUTE3", OracleDbType.Varchar2).Value = pData.ATTRIBUTE3;
                cmd.Parameters.Add("P_ATTRIBUTE4", OracleDbType.Varchar2).Value = pData.ATTRIBUTE4;
                cmd.Parameters.Add("P_ATTRIBUTE5", OracleDbType.Varchar2).Value = pData.ATTRIBUTE5;

                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM134P";
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                //cmd.Parameters.Add("P_IP_ADDR", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM134P_UPDATE");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }
        [HttpPost, Route("SNM134P_DELETE")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM134P_DELETE([FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM134P_DELETE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pData.PART_NO;
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM134P";

                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                //cmd.Parameters.Add("P_IP_ADDR", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM134P_DELETE");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        #endregion
        #region SNM124

        [HttpPost, Route("SNM124_GET_DATA")]
        [Authorize]
        public IHttpActionResult SNM124_GET_DATA([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();

            try
            {
                cmd.CommandText = "SP_NP_SNM124_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_EFFECTIVE_DATE", OracleDbType.Date).Value = pParam.EFFECTIVE_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(pParam.EFFECTIVE_DATE);
                cmd.Parameters.Add("P_FY", OracleDbType.Varchar2).Value = pParam.FY == null ? (object)DBNull.Value : pParam.FY;
                cmd.Parameters.Add("P_ROUND_NO", OracleDbType.Varchar2).Value = pParam.ROUND_NO == null ? (object)DBNull.Value : pParam.ROUND_NO;

                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM124_GET_DATA");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNM124_INSERT")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM124_INSERT([FromBody] dynamic pData)
        {
            try
            {

                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM124_INSERT";

                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_EFFECTIVE_DATE", OracleDbType.Date).Value = pData.EFFECTIVE_DATE;
                cmd.Parameters.Add("P_FY", OracleDbType.Varchar2).Value = pData.FY;
                cmd.Parameters.Add("P_ROUND_NO", OracleDbType.Varchar2).Value = pData.ROUND_NO;
                cmd.Parameters.Add("P_CFG_ACCESSORIES", OracleDbType.Varchar2).Value = pData.CFG_ACCESSORIES;
                cmd.Parameters.Add("P_CFG_COUNTERMESURE", OracleDbType.Varchar2).Value = pData.CFG_COUNTERMESURE;
                cmd.Parameters.Add("P_CFG_PRODUCTION_COUNTRY", OracleDbType.Varchar2).Value = pData.CFG_PRODUCTION_COUNTRY;
                cmd.Parameters.Add("P_CFG_LOGISTICS", OracleDbType.Varchar2).Value = pData.CFG_LOGISTICS;
                cmd.Parameters.Add("P_CFG_PROCUREMENT", OracleDbType.Varchar2).Value = pData.CFG_PROCUREMENT;
                cmd.Parameters.Add("P_CFG_FIX", OracleDbType.Varchar2).Value = pData.CFG_FIX;
                cmd.Parameters.Add("P_REMARKS", OracleDbType.Varchar2).Value = pData.REMARKS;
                cmd.Parameters.Add("P_ATTRIBUTE1", OracleDbType.Varchar2).Value = pData.ATTRIBUTE1;
                cmd.Parameters.Add("P_ATTRIBUTE2", OracleDbType.Varchar2).Value = pData.ATTRIBUTE2;
                cmd.Parameters.Add("P_ATTRIBUTE3", OracleDbType.Varchar2).Value = pData.ATTRIBUTE3;
                cmd.Parameters.Add("P_ATTRIBUTE4", OracleDbType.Varchar2).Value = pData.ATTRIBUTE4;
                cmd.Parameters.Add("P_ATTRIBUTE5", OracleDbType.Varchar2).Value = pData.ATTRIBUTE5;

                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM124";
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_IP_ADDR", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM124_INSERT");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM124_UPDATE")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM124_UPDATE([FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM124_UPDATE";

                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_EFFECTIVE_DATE", OracleDbType.Date).Value = pData.EFFECTIVE_DATE;
                cmd.Parameters.Add("P_FY", OracleDbType.Varchar2).Value = pData.FY;
                cmd.Parameters.Add("P_ROUND_NO", OracleDbType.Varchar2).Value = pData.ROUND_NO;
                cmd.Parameters.Add("P_CFG_ACCESSORIES", OracleDbType.Varchar2).Value = pData.CFG_ACCESSORIES;
                cmd.Parameters.Add("P_CFG_COUNTERMESURE", OracleDbType.Varchar2).Value = pData.CFG_COUNTERMESURE;
                cmd.Parameters.Add("P_CFG_PRODUCTION_COUNTRY", OracleDbType.Varchar2).Value = pData.CFG_PRODUCTION_COUNTRY;
                cmd.Parameters.Add("P_CFG_LOGISTICS", OracleDbType.Varchar2).Value = pData.CFG_LOGISTICS;
                cmd.Parameters.Add("P_CFG_PROCUREMENT", OracleDbType.Varchar2).Value = pData.CFG_PROCUREMENT;
                cmd.Parameters.Add("P_CFG_FIX", OracleDbType.Varchar2).Value = pData.CFG_FIX;
                cmd.Parameters.Add("P_REMARKS", OracleDbType.Varchar2).Value = pData.REMARKS;
                cmd.Parameters.Add("P_ATTRIBUTE1", OracleDbType.Varchar2).Value = pData.ATTRIBUTE1;
                cmd.Parameters.Add("P_ATTRIBUTE2", OracleDbType.Varchar2).Value = pData.ATTRIBUTE2;
                cmd.Parameters.Add("P_ATTRIBUTE3", OracleDbType.Varchar2).Value = pData.ATTRIBUTE3;
                cmd.Parameters.Add("P_ATTRIBUTE4", OracleDbType.Varchar2).Value = pData.ATTRIBUTE4;
                cmd.Parameters.Add("P_ATTRIBUTE5", OracleDbType.Varchar2).Value = pData.ATTRIBUTE5;

                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM124";
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_IP_ADDR", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM124_UPDATE");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNM124_DELETE")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM124_DELETE([FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM124_DELETE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_EFFECTIVE_DATE", OracleDbType.Date).Value = pData.EFFECTIVE_DATE;

                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM124";

                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_IP_ADDR", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM124_DELETE");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM124_GET_EXCHANGE_RATE")]
        [Authorize]
        public IHttpActionResult SNM124_GET_EXCHANGE_RATE([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();

            try
            {
                cmd.CommandText = "SP_NP_SNM124_GET_EXC_RATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_EFFECTIVE_DATE", OracleDbType.Date).Value = pParam.EFFECTIVE_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(pParam.EFFECTIVE_DATE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM124_GET_EXCHANGE_RATE");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM124_UPDATE_EXCHANGE_RATE")]
        [Authorize]
        public IHttpActionResult SNM124_UPDATE_EXCHANGE_RATE([FromBody] dynamic pDataList)
        {
            string[] RateType = new string[0];
            if (pDataList["RATE_TYPE_DESC"] != null)
            {
                RateType = pDataList["RATE_TYPE_DESC"].ToString().Split(':');
            }
            DataSet ds = new DataSet();
            // DataTable dt = new DataTable();
            // var x = HttpContext.Current.Request.UserHostAddress.ToString();
            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM032E_SAVE";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_EFFECTIVE_DATE", OracleDbType.Date).Value = pDataList["EFFECTIVE_DATE"];
            cmd.Parameters.Add("P_RATE_TYPE", OracleDbType.Varchar2).Value = RateType.Length > 0 && RateType[0].ToString() != "" ? RateType[0].Trim() : "";
            cmd.Parameters.Add("P_CURRENCY_CODE", OracleDbType.Varchar2).Value = pDataList["CURRENCY_CODE"];
            cmd.Parameters.Add("P_EXCHANGE_RATE", OracleDbType.Decimal).Value = pDataList["EXCHANGE_RATE"];
            cmd.Parameters.Add("P_EXCHANGE_UNIT", OracleDbType.Decimal).Value = pDataList["EXCHANGE_UNIT"];
            cmd.Parameters.Add("P_CREATE_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
            cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM124P";
            cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
            cmd.Parameters.Add("P_LAST_IP_ADDR", OracleDbType.Varchar2).Value = pDataList["LAST_IP_ADDR"];
            cmd.Parameters.Add("P_MOD_EDIT", OracleDbType.Varchar2).Value = pDataList["ModEdit"] == true ? "Y" : "N";
            cmd.Parameters.Add("C_RE", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        #endregion

        [HttpPost, Route("SNSE214_GetData")]
        [Authorize]
        public IHttpActionResult SNSE214_GetData([FromBody] dynamic pParam)
        {

            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();

            try
            {

                cmd.CommandText = "SP_NP_SNSE214_GET_DATA";
                cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                //cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                //cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam.BRANCH_CODE;
                cmd.Parameters.Add("P_FROM_DATE", OracleDbType.Date).Value = pParam.FROM_DATE == null ? (object)DBNull.Value : pParam.FROM_DATE;
                cmd.Parameters.Add("P_TO_DATE", OracleDbType.Date).Value = pParam.TO_DATE == null ? (object)DBNull.Value : pParam.TO_DATE;

                //cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pParam.ORDER_NO;

                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                // cmd.Parameters.Add("C_RETURN2", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM124_UPDATE");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE214_Attachedfile")]
        [Authorize]
        public async Task<IHttpActionResult> SNSE005_Attachedfile(HttpRequestMessage request)
        {

            DataSet ds = new DataSet();

            if (!request.Content.IsMimeMultipartContent())
            {
                throw new HttpResponseException(HttpStatusCode.UnsupportedMediaType);
            }

            var reqParts = await request.Content.ReadAsMultipartAsync();
            List<string> ErrorList = new List<string>();
            try
            {
                string fileLocation = string.Empty;
                string fileName = "";
                string FullfileName = "";

                string ORDER_NO = "ORDER", ORDER_DATE = "2020-10-23", CUSTOMER_CODE = "1JPN";

                var file = reqParts.Contents[0];



                var name = reqParts.Contents[1].ReadAsByteArrayAsync();
                CUSTOMER_CODE = Encoding.UTF8.GetString(name.Result, 0, name.Result.Length);

                name = reqParts.Contents[2].ReadAsByteArrayAsync();
                ORDER_NO = Encoding.UTF8.GetString(name.Result, 0, name.Result.Length);

                name = reqParts.Contents[3].ReadAsByteArrayAsync();
                ORDER_DATE = Encoding.UTF8.GetString(name.Result, 0, name.Result.Length).Substring(0, 10);

                var buffer = await file.ReadAsByteArrayAsync();
                fileName = Path.GetFileNameWithoutExtension(file.Headers.ContentDisposition.FileName.Trim('\"'));
                string ext = Path.GetExtension(file.Headers.ContentDisposition.FileName.Trim('\"'));
                FullfileName = String.Concat(fileName, ext);

                string gRootPath = ConfigurationManager.AppSettings["RootSharedPath"].ToString();
                string outputLocation = System.IO.Path.Combine(gRootPath, "P_Shared/PMS_Upload/SNSE005_Attached/" + CUSTOMER_CODE + "/" + ORDER_NO + "/");
                string outputPath = outputLocation; //System.Web.HttpContext.Current.Server.MapPath(outputLocation);
                fileLocation = String.Concat(outputPath, FullfileName);

                DataService.SaveBytesToFileByLocation(buffer
                                                , fileLocation
                                                , fileLocation);


                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE214_N_SAVE_ATTACHED";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = CUSTOMER_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = ORDER_NO;
                cmd.Parameters.Add("P_ORDER_DATE", OracleDbType.Varchar2).Value = ORDER_DATE;
                cmd.Parameters.Add("P_FILE_PATH", OracleDbType.Varchar2).Value = System.IO.Path.Combine(gRootPath, "P_Shared/PMS_Upload/SNSE005_Attached/" + CUSTOMER_CODE + "/" + ORDER_NO + "/" + FullfileName);
                cmd.Parameters.Add("P_FILE_NAME", OracleDbType.Varchar2).Value = FullfileName;
                cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE005_N";
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                if (ds.Tables.Count > 0)
                {
                    if (ds.Tables[0].Rows[0]["STATUS"].ToString() != "DONE")
                    {
                        string errString = "Error : " + ds.Tables[0].Rows[0]["STATUS"].ToString();
                        oOracledb.WriteLog("ErrorHandle", errString, "SNSE005_Attachedfile");
                        return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, errString));
                    }
                }
                else
                {
                    string errString = "Error : No data found";
                    oOracledb.WriteLog("ErrorHandle", errString, "SNSE005_Attachedfile");
                    return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, errString));
                }

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE005_Attachedfile");
                throw new HttpResponseException(HttpStatusCode.NotAcceptable);
            }

            return Ok("DONE");
        }

        #region SNSE015_V

        [HttpGet, Route("SNSE015_V_GetCustomer/{cusID}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE015_V_GetCustomer(string cusID, int noofshowrec)
        {
            DataSet ds = new DataSet();

            cusID = ismoUtil.escapeCharacter(cusID);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "PK_NP_SNSE015_V.GET_CUST_LOOKUP";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = cusID.ToUpper() == "NULL" ? (object)DBNull.Value : cusID;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE015_V_GetCustomer");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE015_V_GetOrder/{orderNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE015_V_GetOrder(string orderNo, int noofshowrec, [FromBody] dynamic pCust)
        {
            DataSet ds = new DataSet();

            orderNo = ismoUtil.escapeCharacter(orderNo);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "PK_NP_SNSE015_V.GET_ORDER_LOOKUP";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pCust.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pCust.BRANCH_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = orderNo.ToUpper() == "NULL" ? (object)DBNull.Value : orderNo;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE015_V_GetOrder");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNSE015_V_GetOrderDetail/{customerCode}/{branchCode}/{orderNo}")]
        [Authorize]
        public IHttpActionResult SNSE015_V_GetOrderDetail(string customerCode, string branchCode, string orderNo)
        {
            OracleCommand cmd;
            DataSet ds;
            DataTable orderHeader, orderDetail;

            try
            {
                cmd = new OracleCommand();
                cmd.CommandText = "PK_NP_SNSE015_V.GET_ORDER_HEADER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = customerCode;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = branchCode;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = orderNo;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                orderHeader = ds.Tables[0];

                cmd = new OracleCommand();
                cmd.CommandText = "PK_NP_SNSE015_V.GET_ORDER_STATUS";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = customerCode;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = branchCode;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = orderNo;
                cmd.Parameters.Add("P_IP_ADDRESS", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_IP_ADDRESS);
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                orderDetail = ds.Tables[0];

                return Json(new
                {
                    Header = orderHeader,
                    Detail = orderDetail
                });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE015_V_GetOrderDetail");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpPost, Route("SNSE015_V_BeginAllocate")]
        [Authorize]
        public IHttpActionResult SNSE015_V_BeginAllocate([FromBody] dynamic pData)
        {
            OracleCommand cmd;
            DataSet ds;

            try
            {
                cmd = new OracleCommand();
                cmd.CommandText = "PK_NP_SNSE015_V.BEGIN_PROCESS";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pData.BRANCH_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pData.ORDER_NO;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_CHK_PRINT_PICKING_CARD", OracleDbType.Varchar2).Value = pData.CHK_PRINT_PICKING_CARD;
                cmd.Parameters.Add("R_INVOICE_FROM", OracleDbType.Varchar2, 20).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_INVOICE_TO", OracleDbType.Varchar2, 20).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_INVOICE_AMT", OracleDbType.Decimal).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_COMPLETE_MSG", OracleDbType.Varchar2, 200).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                string Message = cmd.Parameters["R_COMPLETE_MSG"].Value.ToString();
                string InvoiceFrom = cmd.Parameters["R_INVOICE_FROM"].Value.ToString();
                string InvoiceTo = cmd.Parameters["R_INVOICE_TO"].Value.ToString();
                string InvoiceAmount = cmd.Parameters["R_INVOICE_AMT"].Value.ToString();

                return Json(new
                {
                    Status = "Done",
                    Message = Message == "null" ? null : Message,
                    InvoiceFrom = InvoiceFrom == "null" ? null : InvoiceFrom,
                    InvoiceTo = InvoiceTo == "null" ? null : InvoiceTo,
                    InvoiceAmount = Convert.ToDecimal(InvoiceAmount)
                });
            }
            catch (OracleException ex)
            {
                string message = ex.Message;

                if (ex.Number >= 20001 && ex.Number <= 20010)
                {
                    message = message.Substring(11, message.IndexOf('\n') - 11);
                    return Json(new
                    {
                        Status = "Fail",
                        Message = message
                    });
                }

                throw (ex);
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE015_V_BeginAllocate");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        #endregion

        #region SNSE015_VW

        [HttpGet, Route("SNSE015_VW_GetCustomer/{cusID}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE015_VW_GetCustomer(string cusID, int noofshowrec)
        {
            DataSet ds = new DataSet();

            cusID = ismoUtil.escapeCharacter(cusID);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "PK_NP_SNSE015_VW.GET_CUST_LOOKUP";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = cusID.ToUpper() == "NULL" ? (object)DBNull.Value : cusID;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE015_VW_GetCustomer");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE015_VW_GetOrder/{orderNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE015_VW_GetOrder(string orderNo, int noofshowrec, [FromBody] dynamic pCust)
        {
            DataSet ds = new DataSet();

            orderNo = ismoUtil.escapeCharacter(orderNo);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "PK_NP_SNSE015_VW.GET_ORDER_LOOKUP";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pCust.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pCust.BRANCH_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = orderNo.ToUpper() == "NULL" ? (object)DBNull.Value : orderNo;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE015_VW_GetOrder");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNSE015_VW_GetOrderDetail/{customerCode}/{branchCode}/{orderNo}")]
        [Authorize]
        public IHttpActionResult SNSE015_VW_GetOrderDetail(string customerCode, string branchCode, string orderNo)
        {
            OracleCommand cmd;
            DataSet ds;
            DataTable orderHeader, orderDetail;

            try
            {
                cmd = new OracleCommand();
                cmd.CommandText = "PK_NP_SNSE015_VW.GET_ORDER_HEADER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = customerCode;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = branchCode;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = orderNo;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                orderHeader = ds.Tables[0];

                cmd = new OracleCommand();
                cmd.CommandText = "PK_NP_SNSE015_VW.GET_ORDER_STATUS";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = customerCode;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = branchCode;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = orderNo;
                cmd.Parameters.Add("P_IP_ADDRESS", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_IP_ADDRESS);
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                orderDetail = ds.Tables[0];

                return Json(new
                {
                    Header = orderHeader,
                    Detail = orderDetail
                });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE015_VW_GetOrderDetail");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpPost, Route("SNSE015_VW_BeginAllocate")]
        [Authorize]
        public IHttpActionResult SNSE015_VW_BeginAllocate([FromBody] dynamic pData)
        {
            OracleCommand cmd;
            DataSet ds;

            try
            {
                cmd = new OracleCommand();
                cmd.CommandText = "PK_NP_SNSE015_VW.BEGIN_PROCESS";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pData.BRANCH_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pData.ORDER_NO;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_CHK_PRINT_PICKING_CARD", OracleDbType.Varchar2).Value = pData.CHK_PRINT_PICKING_CARD;
                cmd.Parameters.Add("R_INVOICE_FROM", OracleDbType.Varchar2, 20).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_INVOICE_TO", OracleDbType.Varchar2, 20).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_INVOICE_AMT", OracleDbType.Decimal).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_COMPLETE_MSG", OracleDbType.Varchar2, 200).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                string Message = cmd.Parameters["R_COMPLETE_MSG"].Value.ToString();
                string InvoiceFrom = cmd.Parameters["R_INVOICE_FROM"].Value.ToString();
                string InvoiceTo = cmd.Parameters["R_INVOICE_TO"].Value.ToString();
                string InvoiceAmount = cmd.Parameters["R_INVOICE_AMT"].Value.ToString();

                return Json(new
                {
                    Status = "Done",
                    Message = Message == "null" ? null : Message,
                    InvoiceFrom = InvoiceFrom == "null" ? null : InvoiceFrom,
                    InvoiceTo = InvoiceTo == "null" ? null : InvoiceTo,
                    InvoiceAmount = Convert.ToDecimal(InvoiceAmount)
                });
            }
            catch (OracleException ex)
            {
                string message = ex.Message;

                if (ex.Number >= 20001 && ex.Number <= 20010)
                {
                    message = message.Substring(11, message.IndexOf('\n') - 11);
                    return Json(new
                    {
                        Status = "Fail",
                        Message = message
                    });
                }

                throw (ex);
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE015_VW_BeginAllocate");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        #endregion
        #region SNSE198
        [HttpPost, Route("SNSE198_PROCESS/{param=null}/{noofshowrec=null}")]
        [Authorize]
        public IHttpActionResult SNSE198_PROCESS(string param = null, string noofshowrec = null)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE198_PROCESS";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = null;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = null;
                cmd.Parameters.Add("P_PGM_NAME", OracleDbType.Varchar2).Value = "SNSE198";
                cmd.Parameters.Add("P_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 500).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string RETURN_MESSAGE = cmd.Parameters["R_MESSAGE"].Value.ToString();
                return Json(new { STATUS_MESSAGE = RETURN_MESSAGE });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE198_PROCESS");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.ExpectationFailed, ex.Message));
            }
        }

        #endregion

        #region SNSE204_S
        [HttpGet, Route("SNSE204_S_GetBRNo/{brNO}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE204_S_GetBRNo(string brNO, int noofshowrec)
        {
            DataSet ds = new DataSet();
            brNO = ismoUtil.escapeCharacter(brNO);
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "PK_NP_SNSE204_S.GET_BR_NO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_BR_NO", OracleDbType.Varchar2).Value = brNO.ToUpper() == "NULL" ? (object)DBNull.Value : brNO;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE204_S_GetCustomer");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpGet, Route("SNSE204_S_GetCustomer/{cusID}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE204_S_GetCustomer(string cusID, int noofshowrec)
        {
            DataSet ds = new DataSet();

            cusID = ismoUtil.escapeCharacter(cusID);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "PK_NP_SNSE204_S.GET_CUST_LOOKUP";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = cusID.ToUpper() == "NULL" ? (object)DBNull.Value : cusID;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE204_S_GetCustomer");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE204_S_GetBranch/{branchCode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE204_S_GetBranch(string branchCode, int noofshowrec, [FromBody] dynamic pData)
        {
            DataSet ds = new DataSet();

            branchCode = ismoUtil.escapeCharacter(branchCode);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "PK_NP_SNSE204_S.GET_BRANCH_LOOKUP";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = branchCode.ToUpper() == "NULL" ? (object)DBNull.Value : branchCode;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE204_S_GetBranch");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE204_S_GetOrder/{orderNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE204_S_GetOrder(string orderNo, int noofshowrec, [FromBody] dynamic pData)
        {
            DataSet ds = new DataSet();

            orderNo = ismoUtil.escapeCharacter(orderNo);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "PK_NP_SNSE204_S.GET_ORDER_LOOKUP";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pData.BRANCH_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = orderNo.ToUpper() == "NULL" ? (object)DBNull.Value : orderNo;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE204_S_GetOrder");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE204_S_GetPartList")]
        [Authorize]
        public IHttpActionResult SNSE204_S_GetPartList([FromBody] dynamic pParams)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "PK_NP_SNSE204_S.GET_PART_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParams["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParams["BRANCH_CODE"];
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pParams["ORDER_NO"];
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE204_S_GetPartList");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNSE204_S_GetPart/{partName}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE204_S_GetPart(string partName, int noofshowrec, [FromBody] dynamic pData)
        {
            DataSet ds = new DataSet();

            partName = ismoUtil.escapeCharacter(partName);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "PK_NP_SNSE204_S.GET_PART_NO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pData.BRANCH_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pData.ORDER_NO;
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = partName.ToUpper() == "NULL" ? (object)DBNull.Value : partName;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE204_S_GetPart");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE204_S_GetBorrowList")]
        [Authorize]
        public IHttpActionResult SNSE204_S_GetBorrowList([FromBody] dynamic pParams)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "PK_NP_SNSE204_S.GET_BORROW_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParams["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParams["BRANCH_CODE"];
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pParams["ORDER_NO"];
                cmd.Parameters.Add("P_BR_NO", OracleDbType.Varchar2).Value = pParams["BR_NO"];
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE204_S_GetBorrowList");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE204_S_GetDetail")]
        [Authorize]
        public IHttpActionResult SNSE204_S_GetDetail([FromBody] dynamic pParams)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "PK_NP_SNSE204_S.GET_BORROW_DETAIL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParams["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParams["BRANCH_CODE"];
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pParams["ORDER_NO"];
                cmd.Parameters.Add("P_BR_NO", OracleDbType.Varchar2).Value = pParams["BR_NO"];
                cmd.Parameters.Add("C_RETURN_H", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN_D", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE204_S_GetDetail");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE204_S_Save")]
        [Authorize]
        public IHttpActionResult SNSE204_S_Save([FromBody] dynamic pData)
        {
            string customerCode = pData["CUSTOMER_CODE"];
            string branchCode = pData["BRANCH_CODE"];
            string orderNo = pData["ORDER_NO"];
            string vorStatus = pData["VOR_STATUS"];
            int countItem = 0;

            var partList = new List<dynamic>(pData.PART_LIST);

            OracleCommand cmd;

            try
            {
                cmd = new OracleCommand();
                cmd.CommandText = "PK_NP_SNSE204_S.DELETE_TEMP_DETAIL";

                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = customerCode;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = branchCode;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = orderNo;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                oOracledb.SqlQueryReturnValue(cmd);

                foreach (var part in partList)
                {
                    countItem++;

                    cmd = new OracleCommand();
                    cmd.CommandText = "PK_NP_SNSE204_S.INSERT_TEMP_DETAIL";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Decimal).Value = countItem;
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = customerCode;
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = branchCode;
                    cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = orderNo;
                    cmd.Parameters.Add("P_REF_ITEM_NO", OracleDbType.Decimal).Value = part.ITEM_NO;
                    cmd.Parameters.Add("P_REF_SUB_ITEM_NO", OracleDbType.Varchar2).Value = part.SUB_ITEM_NO;
                    cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = part.ORDER_PART_NO;
                    cmd.Parameters.Add("P_OH_QTY", OracleDbType.Decimal).Value = part.OH_QTY;
                    cmd.Parameters.Add("P_LOCATION", OracleDbType.Varchar2).Value = part.LOCATION_NO;
                    cmd.Parameters.Add("P_ORDER_QTY", OracleDbType.Decimal).Value = part.ORDER_QTY;
                    cmd.Parameters.Add("P_BORROW_QTY", OracleDbType.Decimal).Value = part.BORROW_QTY;
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                    oOracledb.SqlQueryReturnValue(cmd);

                }

                cmd = new OracleCommand();
                cmd.CommandText = "PK_NP_SNSE204_S.BEGIN_PROCESS";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = customerCode;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = branchCode;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = orderNo;
                cmd.Parameters.Add("P_VOR_STATUS", OracleDbType.Varchar2).Value = vorStatus;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_BR_NO", OracleDbType.Varchar2, 13).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string brNo = cmd.Parameters["R_BR_NO"].Value.ToString();

                return Json(new
                {
                    BR_NO = brNo
                });

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE204_S_Save");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        #endregion

        #region SNM126P

        [HttpGet, Route("SNM126P_GetCurrencyCode/{CurrencyCode=null}/{NoOfShowRecords=null}")]
        [Authorize]
        public IHttpActionResult SNM126P_GetCurrencyCode(string CurrencyCode = null, string NoOfShowRecords = null)
        {
            try
            {
                DataSet ds = new DataSet();
                CurrencyCode = ismoUtil.escapeCharacter(CurrencyCode);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM126P_CURRENCY_CODE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CURRENCY_CODE", OracleDbType.Varchar2).Value = CurrencyCode.ToUpper() == "NULL" ? (object)DBNull.Value : CurrencyCode;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = NoOfShowRecords;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM126P_GetCurrencyCode");
                return Json(new { Status = "Error" });

            }
        }

        [HttpGet, Route("SNM126P_GetCustomerCode/{CustomerCode=null}/{NoOfShowRecords=null}")]
        [Authorize]
        public IHttpActionResult SNM126P_GetCustomerCode(string CustomerCode = null, string NoOfShowRecords = null)
        {
            try
            {
                DataSet ds = new DataSet();
                CustomerCode = ismoUtil.escapeCharacter(CustomerCode);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM126P_GET_CUST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = CustomerCode.ToUpper() == "NULL" ? (object)DBNull.Value : CustomerCode;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = NoOfShowRecords;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM126P_GetCustomerCode");
                return Json(new { Status = "Error" });

            }
        }

        [HttpPost, Route("SNM126P_Insert")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM126P_Insert([FromBody] dynamic pData)
        {
            try
            {

                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM126P_INSERT";

                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_GROUP_NAME", OracleDbType.Varchar2).Value = pData.GROUP_NAME;
                cmd.Parameters.Add("P_MAIN_CUSTOMER", OracleDbType.Varchar2).Value = pData.MAIN_CUSTOMER;
                cmd.Parameters.Add("P_CURRENCY_CODE", OracleDbType.Varchar2).Value = pData.CURRENCY_CODE;
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM126P";
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_IP_ADDR", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM126P_Insert");
                return Ok("Error");
            }
        }

        [HttpPost, Route("SNM126P_GetData")]
        [Authorize]
        public IHttpActionResult SNM125P_GetData([FromBody] dynamic pData)
        {

            try
            {

                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM126P_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_GROUP_NAME", OracleDbType.Varchar2).Value = pData.GROUP_NAME;
                cmd.Parameters.Add("P_MAIN_CUSTOMER", OracleDbType.Varchar2).Value = pData.MAIN_CUSTOMER;
                cmd.Parameters.Add("P_CURRENCY_CODE", OracleDbType.Varchar2).Value = pData.CURRENCY_CODE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM126P_GetData");
                return Json(new { Status = "Error" });
            }
        }

        [HttpPost, Route("SNM126P_Delete")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM126P_Delete([FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM126P_DELETE";

                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_GROUP_NAME", OracleDbType.Varchar2).Value = pData.GROUP_NAME;
                cmd.Parameters.Add("P_MAIN_CUSTOMER", OracleDbType.Varchar2).Value = pData.MAIN_CUSTOMER;
                cmd.Parameters.Add("P_CURRENCY_CODE", OracleDbType.Varchar2).Value = pData.CURRENCY_CODE;
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM126P";
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_IP_ADDR", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM126P_Delete");
                return Ok("Error");
            }
        }

        [HttpPost, Route("SNM126P_Update")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM126P_Update([FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM126P_UPDATE";

                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_GROUP_NAME", OracleDbType.Varchar2).Value = pData.GROUP_NAME;
                cmd.Parameters.Add("P_MAIN_CUSTOMER", OracleDbType.Varchar2).Value = pData.MAIN_CUSTOMER;
                cmd.Parameters.Add("P_CURRENCY_CODE", OracleDbType.Varchar2).Value = pData.CURRENCY_CODE;
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM126P";
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_IP_ADDR", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM126P_Update");
                return Ok("Error");
            }

        }

        #endregion

        #region SNSE210
        [HttpPost, Route("SNSE210_GetCustomerCode/{customerCode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE210_GetCustomerCode(string customerCode, int noofshowrec, [FromBody] dynamic pData)
        {
            DataSet ds = new DataSet();

            customerCode = ismoUtil.escapeCharacter(customerCode);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE210_GET_CUSTOMER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = customerCode.ToUpper() == "NULL" ? (object)DBNull.Value : customerCode;
                cmd.Parameters.Add("P_GROUP_PART_NO", OracleDbType.Varchar2).Value = pData.GROUP_PART_NO;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE210_GetCustomerCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE210_GetBranch/{branchCode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE210_GetBranch(string branchCode, int noofshowrec, [FromBody] dynamic pData)
        {
            DataSet ds = new DataSet();

            branchCode = ismoUtil.escapeCharacter(branchCode);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE210_GET_BRANCH";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = branchCode.ToUpper() == "NULL" ? (object)DBNull.Value : branchCode;
                cmd.Parameters.Add("P_GROUP_PART_NO", OracleDbType.Varchar2).Value = pData.GROUP_PART_NO;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE210_GetBranch");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE210_GetGroupPart/{groupPart}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE210_GetGroupPart(string groupPart, int noofshowrec, [FromBody] dynamic pData)
        {
            DataSet ds = new DataSet();

            groupPart = ismoUtil.escapeCharacter(groupPart);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE210_GET_GROUP_PART";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pData.BRANCH_CODE;
                cmd.Parameters.Add("P_GROUP_PART_NO", OracleDbType.Varchar2).Value = groupPart.ToUpper() == "NULL" ? (object)DBNull.Value : groupPart;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE210_GetGroupPart");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE210_GetData")]
        [Authorize]
        public IHttpActionResult SNSE210_GetData([FromBody] dynamic pParams)
        {
            try
            {
                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE210_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParams.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParams.BRANCH_CODE;
                cmd.Parameters.Add("P_GROUP_PART_NO", OracleDbType.Varchar2).Value = pParams.GROUP_PART_NO;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE210_GetData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE210_Update")]
        [Authorize]
        [RunBatchGuard]
        public IHttpActionResult SNSE210_Update([FromBody] dynamic[] pDataList)
        {
            DataSet ds = new DataSet();

            try
            {
                foreach (var data in pDataList)
                {
                    OracleCommand cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE210_UPDATE";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = data.BRANCH_CODE;
                    cmd.Parameters.Add("P_GROUP_PART_NO", OracleDbType.Varchar2).Value = data.GROUP_PART_NO;
                    cmd.Parameters.Add("P_LAST_MONTH", OracleDbType.Varchar2).Value = data.LAST_MONTH;
                    cmd.Parameters.Add("P_ADJUST_UNIT", OracleDbType.Varchar2).Value = data.ADJUST_UNIT;
                    cmd.Parameters.Add("P_REMAIN_UNIT", OracleDbType.Varchar2).Value = data.REMAIN_UNIT;
                    cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                    ds = oOracledb.SqlQueryDS_CMD(cmd);

                }
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE210_Update");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNSE210_Upload")]
        [Authorize]
        public async Task<IHttpActionResult> SNSE210_Upload(HttpRequestMessage request)
        {
            if (!request.Content.IsMimeMultipartContent())
            {
                throw new HttpResponseException(HttpStatusCode.UnsupportedMediaType);
            }

            var reqParts = await request.Content.ReadAsMultipartAsync();
            List<string> ErrorList = new List<string>();
            try
            {
                string fileLocation = string.Empty;
                foreach (var file in reqParts.Contents)
                {
                    var buffer = await file.ReadAsByteArrayAsync();
                    string fileName = Path.GetFileNameWithoutExtension(file.Headers.ContentDisposition.FileName.Trim('\"'));
                    string ext = Path.GetExtension(file.Headers.ContentDisposition.FileName.Trim('\"'));
                    string FullfileName = String.Concat(fileName, "-", DateTime.Now.ToString("yyyyMMddHHmm"), ext);
                    string outputLocation = "~/tempFile/" + "SNSE210" + "/";
                    string outputPath = System.Web.HttpContext.Current.Server.MapPath(outputLocation);
                    fileLocation = String.Concat(outputPath, FullfileName);

                    DataService.SaveBytesToFileByLocation(buffer
                                                    , fileLocation
                                                    , fileLocation);
                }

                ErrorList = SNSE210_InsertDataFromExcelFile(fileLocation);
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ01E_Upload");
                throw new HttpResponseException(HttpStatusCode.NotAcceptable);
            }

            return Json(new { Error = ErrorList });
        }

        public string SNSE210_ClearTemp()
        {
            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSE210_DEL_TEMP";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return "DONE";
        }

        public List<string> SNSE210_InsertDataFromExcelFile(string fileLocation)
        {
            string fileLocationN = fileLocation;
            DataTable dt = new DataTable();
            SystemApiController sys = new SystemApiController();
            dt = sys.ReadDataExcel(fileLocationN);
            List<string> ErrList = new List<string>();
            int countrow = 1;

            SNSE210_ClearTemp();

            foreach (DataRow row in dt.Rows)
            {
                string p1 = null;
                string p2 = null;
                string p3 = null;
                string p4 = null;
                string p5 = null;
                string p6 = null;

                p1 = row["Customer Code"].ToString().Trim();
                p2 = row["Branch Code"].ToString().Trim();
                p3 = row["Part No"].ToString().Trim();
                p4 = row["Forecast Year"].ToString().Trim();
                p5 = row["Forecast Month"].ToString().Trim();
                p6 = row["Forcast Qty"].ToString().Trim();

                if (p1 != null && p2 != null && p3 != null && p4 != null && p5 != null && p6 != null)
                {
                    OracleCommand cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE210_INS_TEMP_UPLOAD";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = row["Customer Code"].ToString().Trim();
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = row["Branch Code"].ToString().Trim();
                    cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = row["Part No"].ToString().Trim();
                    cmd.Parameters.Add("P_FORECAST_YYYY", OracleDbType.Varchar2).Value = row["Forecast Year"].ToString().Trim();
                    cmd.Parameters.Add("P_FORECAST_MM", OracleDbType.Varchar2).Value = row["Forecast Month"].ToString().Trim();
                    cmd.Parameters.Add("P_FORECAST_QTY", OracleDbType.Varchar2).Value = row["Forcast Qty"].ToString().Trim();
                    cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;

                    cmd = oOracledb.SqlQueryReturnValue(cmd);

                    string status = cmd.Parameters["P_STATUS"].Value.ToString();
                    if (status == "DONE")
                    {
                        ErrList.Add("Row " + countrow + " Upload complete");
                    }
                    else if (status == "FAIL")
                    {
                        ErrList.Add("Row " + countrow + " " + "Part No. :" + " " + row["Part No"].ToString().Trim() + " error please recheck");
                    }

                }
                else
                {
                    ErrList.Add("Row " + countrow + " " + "Part No. :" + " " + row["Part No"].ToString().Trim() + " error please recheck");
                }

                countrow++;
            }
            return ErrList;
        }

        [HttpGet, Route("SNSE210_GetTempData")]
        [Authorize]
        public IHttpActionResult SNSE210_GetTempData()
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE210_GET_TEMP_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE210_GetTempData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNSE210_Save_Forecast")]
        [Authorize]
        public IHttpActionResult SNSE210_Save_Forecast()
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE210_UPDATE_FORECAST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE210_Save_Forecast");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        #endregion

        #region SNSE209_D
        [HttpPost, Route("SNSE209_D_GetCustomerCode/{customerCode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE209_D_GetCustomerCode(string customerCode, int noofshowrec, [FromBody] dynamic pData)
        {
            DataSet ds = new DataSet();

            customerCode = ismoUtil.escapeCharacter(customerCode);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE209_D_GET_CUSTOMER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = customerCode.ToUpper() == "NULL" ? (object)DBNull.Value : customerCode;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pData.ORDER_NO;
                cmd.Parameters.Add("P_ACCEPT_PART_NO", OracleDbType.Varchar2).Value = pData.ACCEPT_PART_NO;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE209_D_GetCustomerCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE209_D_GetBranch/{branchCode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE209_D_GetBranch(string branchCode, int noofshowrec, [FromBody] dynamic pData)
        {
            DataSet ds = new DataSet();

            branchCode = ismoUtil.escapeCharacter(branchCode);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE209_D_GET_BRANCH";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = branchCode.ToUpper() == "NULL" ? (object)DBNull.Value : branchCode;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pData.ORDER_NO;
                cmd.Parameters.Add("P_ACCEPT_PART_NO", OracleDbType.Varchar2).Value = pData.ACCEPT_PART_NO;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE209_D_GetBranch");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE209_D_GetOrderNo/{orderNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE209_D_GetOrderNo(string orderNo, int noofshowrec, [FromBody] dynamic pData)
        {
            DataSet ds = new DataSet();

            orderNo = ismoUtil.escapeCharacter(orderNo);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE209_D_GET_ORDER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pData.BRANCH_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = orderNo.ToUpper() == "NULL" ? (object)DBNull.Value : orderNo;
                cmd.Parameters.Add("P_ACCEPT_PART_NO", OracleDbType.Varchar2).Value = pData.ACCEPT_PART_NO;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE209_D_GetOrderNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE209_D_GetPart/{partNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE209_D_GetPart(string partNo, int noofshowrec, [FromBody] dynamic pData)
        {
            DataSet ds = new DataSet();

            partNo = ismoUtil.escapeCharacter(partNo);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE209_D_GET_PART";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pData.BRANCH_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pData.ORDER_NO;
                cmd.Parameters.Add("P_ACCEPT_PART_NO", OracleDbType.Varchar2).Value = partNo.ToUpper() == "NULL" ? (object)DBNull.Value : partNo;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE209_D_GetPart");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE209_D_GetData")]
        [Authorize]
        public IHttpActionResult SNSE209_D_GetData([FromBody] dynamic pParams)
        {
            try
            {
                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE209_D_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParams.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParams.BRANCH_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pParams.ORDER_NO;
                cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = pParams.ORDER_CLASS;
                cmd.Parameters.Add("P_ACCEPT_PART_NO", OracleDbType.Varchar2).Value = pParams.ACCEPT_PART_NO;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE209_D_GetData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE209_D_Update")]
        [Authorize]
        [RunBatchGuard]
        public IHttpActionResult SNSE209_D_Update([FromBody] dynamic[] pDataList)
        {
            DataSet ds = new DataSet();

            try
            {
                foreach (var data in pDataList)
                {
                    OracleCommand cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE209_D_UPDATE";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = data.BRANCH_CODE;
                    cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = data.ORDER_NO;
                    cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Varchar2).Value = data.ITEM_NO;
                    cmd.Parameters.Add("P_SUB_ITEM_NO", OracleDbType.Varchar2).Value = data.SUB_ITEM_NO;
                    cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE209_D";
                    cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                    ds = oOracledb.SqlQueryDS_CMD(cmd);

                }
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE210_Update");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        #endregion

        #region SNSE209_E
        [HttpPost, Route("SNSE209_E_GetCustomerCode/{customerCode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE209_E_GetCustomerCode(string customerCode, int noofshowrec, [FromBody] dynamic pData)
        {
            DataSet ds = new DataSet();

            customerCode = ismoUtil.escapeCharacter(customerCode);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE209_E_GET_CUSTOMER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = customerCode.ToUpper() == "NULL" ? (object)DBNull.Value : customerCode;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pData.ORDER_NO;
                cmd.Parameters.Add("P_ACCEPT_PART_NO", OracleDbType.Varchar2).Value = pData.ACCEPT_PART_NO;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE209_E_GetCustomerCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE209_E_GetBranch/{branchCode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE209_E_GetBranch(string branchCode, int noofshowrec, [FromBody] dynamic pData)
        {
            DataSet ds = new DataSet();

            branchCode = ismoUtil.escapeCharacter(branchCode);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE209_E_GET_BRANCH";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = branchCode.ToUpper() == "NULL" ? (object)DBNull.Value : branchCode;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pData.ORDER_NO;
                cmd.Parameters.Add("P_ACCEPT_PART_NO", OracleDbType.Varchar2).Value = pData.ACCEPT_PART_NO;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE209_E_GetBranch");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE209_E_GetOrderNo/{orderNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE209_E_GetOrderNo(string orderNo, int noofshowrec, [FromBody] dynamic pData)
        {
            DataSet ds = new DataSet();

            orderNo = ismoUtil.escapeCharacter(orderNo);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE209_E_GET_ORDER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pData.BRANCH_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = orderNo.ToUpper() == "NULL" ? (object)DBNull.Value : orderNo;
                cmd.Parameters.Add("P_ACCEPT_PART_NO", OracleDbType.Varchar2).Value = pData.ACCEPT_PART_NO;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE209_E_GetOrderNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE209_E_GetPart/{partNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE209_E_GetPart(string partNo, int noofshowrec, [FromBody] dynamic pData)
        {
            DataSet ds = new DataSet();

            partNo = ismoUtil.escapeCharacter(partNo);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE209_E_GET_PART";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pData.BRANCH_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pData.ORDER_NO;
                cmd.Parameters.Add("P_ACCEPT_PART_NO", OracleDbType.Varchar2).Value = partNo.ToUpper() == "NULL" ? (object)DBNull.Value : partNo;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE209_E_GetPart");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE209_E_GetData")]
        [Authorize]
        public IHttpActionResult SNSE209_E_GetData([FromBody] dynamic pParams)
        {
            try
            {
                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE209_E_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParams.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParams.BRANCH_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pParams.ORDER_NO;
                cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = pParams.ORDER_CLASS;
                cmd.Parameters.Add("P_ACCEPT_PART_NO", OracleDbType.Varchar2).Value = pParams.ACCEPT_PART_NO;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE209_E_GetData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE209_E_Update")]
        [Authorize]
        [RunBatchGuard]
        public IHttpActionResult SNSE209_E_Update([FromBody] dynamic[] pDataList)
        {
            DataSet ds = new DataSet();

            try
            {
                foreach (var data in pDataList)
                {
                    OracleCommand cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE209_E_UPDATE";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = data.BRANCH_CODE;
                    cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = data.ORDER_NO;
                    cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Varchar2).Value = data.ITEM_NO;
                    cmd.Parameters.Add("P_SUB_ITEM_NO", OracleDbType.Varchar2).Value = data.SUB_ITEM_NO;
                    cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE209_E";
                    cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                    ds = oOracledb.SqlQueryDS_CMD(cmd);

                }
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE210_Update");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        #endregion

        #region SNBA009_USS
        [HttpGet, Route("SNBA009_USS_GetGroupPartNo/{groupPart}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNBA009_USS_GetGroupPartNo(string groupPart, int noofshowrec)
        {
            DataSet ds = new DataSet();

            groupPart = ismoUtil.escapeCharacter(groupPart);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNBA009_USS_GET_PART_NO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_GROUP_PART_NO", OracleDbType.Varchar2).Value = groupPart.ToUpper() == "NULL" ? (object)DBNull.Value : groupPart;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNBA009_USS_GetGroupPartNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }



        [HttpGet, Route("SNBA009_USS_GetControl")]
        [Authorize]
        public IHttpActionResult SNBA009_USS_GetControl()
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNBA009_USS_GET_CONTROL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNBA009_USS_GetControl");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNBA009_USS_GetData")]
        [Authorize]
        public IHttpActionResult SNBA009_USS_GetData([FromBody] dynamic pParams)
        {
            try
            {
                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNBA009_USS_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_GROUP_PART_NO", OracleDbType.Varchar2).Value = pParams.GROUP_PART_NO;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNBA009_USS_GetData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNBA009_USS_GetPercent")]
        [Authorize]
        public IHttpActionResult SNBA009_USS_GetPercent()
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNBA009_USS_GET_PERCENT";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNBA009_USS_GetPercent");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNBA009_USS_UpdateControl")]
        [Authorize]
        [RunBatchGuard]
        public IHttpActionResult SNBA009_USS_UpdateControl([FromBody] dynamic pParams)
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNBA009_USS_CON_UPDATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_BATCH_ID", OracleDbType.Varchar2).Value = pParams.BATCH_ID;
                cmd.Parameters.Add("P_BATCH_STATUS", OracleDbType.Varchar2).Value = pParams.BATCH_STATUS;
                cmd.Parameters.Add("P_START_DATE", OracleDbType.Date).Value = pParams.START_DATE;
                cmd.Parameters.Add("P_END_DATE", OracleDbType.Date).Value = pParams.END_DATE;
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNBA009_USS";
                cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNBA009_USS_UpdateControl");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNBA009_USS_UpdateAllPart")]
        [Authorize]
        [RunBatchGuard]
        public IHttpActionResult SNBA009_USS_UpdateAllPart([FromBody] dynamic pParams)
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNBA009_USS_ALL_UPDATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PERCENT", OracleDbType.Varchar2).Value = pParams.USS_PERCENT;
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNBA009_USS";
                cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNBA009_USS_UpdateAllPart");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNBA009_USS_UpdateGroupPart")]
        [Authorize]
        [RunBatchGuard]
        public IHttpActionResult SNBA009_USS_UpdateGroupPart([FromBody] dynamic[] pDataList)
        {
            DataSet ds = new DataSet();

            try
            {
                foreach (var data in pDataList)
                {
                    OracleCommand cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNBA009_USS_PART_UPDATE";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_GROUP_PART_NO", OracleDbType.Varchar2).Value = data.GROUP_PART_NO;
                    cmd.Parameters.Add("P_BCONFIRM_PERCENT", OracleDbType.Varchar2).Value = data.BCONFIRM_PERCENT;
                    cmd.Parameters.Add("P_BCONFIRM_QTY", OracleDbType.Varchar2).Value = data.BCONFIRM_QTY;
                    cmd.Parameters.Add("P_AQTY", OracleDbType.Varchar2).Value = data.AQTY;
                    cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNBA009_USS";
                    cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                    ds = oOracledb.SqlQueryDS_CMD(cmd);

                }
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNBA009_USS_UpdateGroupPart");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        #endregion
        #region SNSJ003N2
        [HttpPost, Route("SNSJ003N2_Process")]
        [Authorize]
        public IHttpActionResult SNSJ003N2_Process()
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            try
            {
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSJ003_PROCESS";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Ok(new { status = "Done" });
            }
            catch (OracleException ex)
            {
                oOracledb.WriteLog("OracleException", ex.Message, "SNSJ003N2_Process");
                if (ex.Number == 20000)
                {
                    return Ok(new { status = "Fail", message = "Can not start Allocate batch start date and end date is not null." });
                }
                if (ex.Number == 20002)
                {
                    return Ok(new { status = "Fail", message = "Can not start Allocate batch start date is not null and end date is null." });
                }
                if (ex.Number == 20003)
                {
                    return Ok(new { status = "Fail", message = "Can not start Allocate batch status != R" });
                }
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSJ003N2_Process");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        #endregion
        #region SNM133P
        [HttpGet, Route("SNM133P_GetPartNo/{partNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM133P_GetPartNo(string partNo, int noofshowrec)
        {
            DataSet ds = new DataSet();

            partNo = ismoUtil.escapeCharacter(partNo);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM133P_GET_PART_NO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = partNo.ToUpper() == "NULL" ? (object)DBNull.Value : partNo;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM133P_GetPartNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNM133P_GetCustomerCode/{customerCode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM133P_GetCustomerCode(string customerCode, int noofshowrec)
        {
            DataSet ds = new DataSet();

            customerCode = ismoUtil.escapeCharacter(customerCode);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM133P_GET_CUSTOMER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = customerCode.ToUpper() == "NULL" ? (object)DBNull.Value : customerCode;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM133P_GetCustomerCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM133P_GetBranchCode/{branchCode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM133P_GetBranchCode(string branchCode, int noofshowrec, [FromBody] dynamic Pparam)
        {
            DataSet ds = new DataSet();

            branchCode = ismoUtil.escapeCharacter(branchCode);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM133P_GET_BRANCH";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = Pparam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = branchCode.ToUpper() == "NULL" ? (object)DBNull.Value : branchCode;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM133P_GetCustomerCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNM133P_GetData")]
        [Authorize]
        public IHttpActionResult SNM133P_GetData([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM133P_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = param.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = param.BRANCH_CODE;
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = param.PART_NO;
                cmd.Parameters.Add("P_EFFECTIVE_DATE", OracleDbType.Date).Value = param.EFFECTIVE_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(param.EFFECTIVE_DATE);
                cmd.Parameters.Add("P_DISABLE_DATE", OracleDbType.Date).Value = param.DISABLE_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(param.DISABLE_DATE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN2", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM133P_GetData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM133P_Save")]
        [Authorize]
        public IHttpActionResult SNM133P_Save([FromBody] dynamic[] pDataList)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();
            OracleCommand cmd = new OracleCommand();
            try
            {
                cmd.CommandText = "SP_NP_SNM133P_CLEAR_SAVE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pDataList[0].CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pDataList[0].BRANCH_CODE;
                cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = pDataList[0].ORDER_CLASS;
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pDataList[0].PART_NO;
                cmd.Parameters.Add("P_EFFECTIVE_DATE", OracleDbType.Date).Value = pDataList[0].EFFECTIVE_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(pDataList[0].EFFECTIVE_DATE);
                cmd.Parameters.Add("P_DISABLE_DATE", OracleDbType.Date).Value = pDataList[0].DISABLE_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(pDataList[0].DISABLE_DATE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                foreach (var data in pDataList)
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNM133P_SAVE";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = data.BRANCH_CODE;
                    cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = data.ORDER_CLASS;
                    cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = data.PART_NO;
                    cmd.Parameters.Add("P_EFFECTIVE_DATE", OracleDbType.Date).Value = data.EFFECTIVE_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(data.EFFECTIVE_DATE);
                    cmd.Parameters.Add("P_DISABLE_DATE", OracleDbType.Date).Value = data.DISABLE_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(data.DISABLE_DATE);
                    cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM133P";
                    cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                    ds = oOracledb.SqlQueryDS_CMD(cmd);
                    dt = ds.Tables[0];
                }
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM133P_Save");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM133P_SaveUpload")]
        [Authorize]
        public IHttpActionResult SNM133P_SaveUpload([FromBody] dynamic pDataList)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM133P_SAVE_UPLOAD";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM133P";
                cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM133P_SaveUpload");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        public string SNM133P_ClearTemp()
        {
            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM133P_CLEAR_TEMP";
            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return "DONE";
        }









        [HttpPost, Route("SNM133P_UploadExcel")]
        [Authorize]
        public async Task<IHttpActionResult> SNM133P_UploadExcel(HttpRequestMessage request)
        {

            if (!request.Content.IsMimeMultipartContent())
            {
                throw new HttpResponseException(HttpStatusCode.UnsupportedMediaType);
            }

            var reqParts = await request.Content.ReadAsMultipartAsync();
            List<string> ErrorList = new List<string>();
            try
            {
                string fileLocation = string.Empty;
                foreach (var file in reqParts.Contents)
                {
                    var buffer = await file.ReadAsByteArrayAsync();
                    string fileName = Path.GetFileNameWithoutExtension(file.Headers.ContentDisposition.FileName.Trim('\"'));
                    string ext = Path.GetExtension(file.Headers.ContentDisposition.FileName.Trim('\"'));
                    string FullfileName = String.Concat(fileName, "-", DateTime.Now.ToString("yyyyMMddHHmm"), ext);
                    string outputLocation = "~/tempFile/" + "SNM133P";
                    string outputPath = System.Web.HttpContext.Current.Server.MapPath(outputLocation);
                    fileLocation = String.Concat(outputPath, FullfileName);

                    DataService.SaveBytesToFileByLocation(buffer
                                                    , fileLocation
                                                    , fileLocation);
                }
                ErrorList = SNM133P_InsertDataFromExcelFile(fileLocation);
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM133P_UploadExcel");
                throw new HttpResponseException(HttpStatusCode.NotAcceptable);
            }

            return Json(new { Error = ErrorList });
        }


        public List<string> SNM133P_InsertDataFromExcelFile(string fileLocation)
        {

            string fileLocationN = fileLocation;
            DataTable dt = new DataTable();
            SystemApiController sys = new SystemApiController();
            dt = sys.ReadDataExcel(fileLocationN);
            List<string> ErrList = new List<string>();
            int countrow = 1;
            OracleCommand cmd = new OracleCommand();
            DataSet ds = new DataSet();

            try
            {
                cmd.CommandText = "SP_NP_SNM133P_CLEAR_TEMP";
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM133P_InsertDataFromExcelFile");
                ErrList.Add("Error : SP_NP_SNM133P_CLEAR_TEMP please recheck");
                return ErrList;
            }


            foreach (DataRow row in dt.Rows)
            {

                DateTime e;
                DateTime d;
                bool isEffDatetime = DateTime.TryParse(row["EFFECTIVE_DATE"].ToString().Trim(), out e);
                bool isDisDate = DateTime.TryParse(row["DISABLE_DATE"].ToString().Trim(), out d);

                if (isEffDatetime && isDisDate)
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNM133P_UPLOAD";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_RNO", OracleDbType.Varchar2).Value = row["NO"].ToString().Trim();
                    cmd.Parameters.Add("P_EFFECTIVE_DATE", OracleDbType.Varchar2).Value = Convert.ToDateTime(row["EFFECTIVE_DATE"]).ToString("dd-MMM-yyyy").Trim();
                    cmd.Parameters.Add("P_DISABLE_DATE", OracleDbType.Varchar2).Value = Convert.ToDateTime(row["DISABLE_DATE"]).ToString("dd-MMM-yyyy").Trim();
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = row["CUSTOMER_CODE"].ToString().Trim();
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = row["BRANCH_CODE"].ToString().Trim();
                    cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = row["ORDER_CLASS"].ToString().Trim();
                    cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = row["PART_NO"].ToString().Trim();
                    cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;

                    cmd = oOracledb.SqlQueryReturnValue(cmd);

                    string status = cmd.Parameters["P_STATUS"].Value.ToString();
                }
                else
                {
                    ErrList.Add("Row " + countrow + "Customer Code : " + row["CUSTOMER_CODE"].ToString().Trim() + "Part no : " + row["PART_NO"].ToString().Trim() + " error please recheck");
                }

                countrow++;
            }
            return ErrList;
        }

        [HttpGet, Route("SNM133P_GetTempData")]
        [Authorize]
        public IHttpActionResult SNM133P_GetTempData()
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM133P_GET_TEMP_DATA";
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM133P_GetTempData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM133P_Delete")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM133P_Delete([FromBody] dynamic pDataList)
        {
            try
            {
                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM133P_DELETE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pDataList.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pDataList.BRANCH_CODE;
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pDataList.PART_NO;
                cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM133P_Delete");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }



        [HttpGet, Route("SNM133P_GetPartNoAdd/{partNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM133P_GetPartNoAdd(string partNo, int noofshowrec)
        {
            DataSet ds = new DataSet();

            partNo = ismoUtil.escapeCharacter(partNo);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM133P_GET_PART_NO_ADD";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = partNo.ToUpper() == "NULL" ? (object)DBNull.Value : partNo;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM133P_GetPartNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNM133P_GetCustomerCodeAdd/{customerCode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM133P_GetCustomerCodeAdd(string customerCode, int noofshowrec)
        {
            DataSet ds = new DataSet();

            customerCode = ismoUtil.escapeCharacter(customerCode);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM133P_GET_CUSTOMER_ADD";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = customerCode.ToUpper() == "NULL" ? (object)DBNull.Value : customerCode;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM133P_GetCustomerCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM133P_GetBranchCodeAdd/{branchCode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM133P_GetBranchCodeAdd(string branchCode, int noofshowrec, [FromBody] dynamic pParams)
        {
            DataSet ds = new DataSet();

            branchCode = ismoUtil.escapeCharacter(branchCode);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM133P_GET_BRANCH_ADD";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParams.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = branchCode.ToUpper() == "NULL" ? (object)DBNull.Value : branchCode;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM133P_GetBranchCodeAdd");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        //Except Quota-------------------------------------------------------------------------------------------------------
        [HttpGet, Route("SNM133P_GetDataExcept")]
        [Authorize]
        public IHttpActionResult SNM133P_GetDataExcept()
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM133P_GET_DATA_EXCEPT";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN2", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM133P_GetData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpPost, Route("SNM133P_SaveExcept")]
        [Authorize]
        public IHttpActionResult SNM133P_SaveExcept([FromBody] dynamic[] pDataList)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();
            OracleCommand cmd = new OracleCommand();
            try
            {
                cmd.CommandText = "SP_NP_SNM133P_CLEAR_SAVE_EXCEPT";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pDataList[0].CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pDataList[0].BRANCH_CODE;
                cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = pDataList[0].ORDER_CLASS;
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pDataList[0].PART_NO;
                cmd.Parameters.Add("P_EFFECTIVE_DATE", OracleDbType.Date).Value = pDataList[0].EFFECTIVE_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(pDataList[0].EFFECTIVE_DATE);
                cmd.Parameters.Add("P_DISABLE_DATE", OracleDbType.Date).Value = pDataList[0].DISABLE_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(pDataList[0].DISABLE_DATE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                foreach (var data in pDataList)
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNM133P_SAVE_EXCEPT";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = data.BRANCH_CODE;
                    cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = data.ORDER_CLASS;
                    cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = data.PART_NO;
                    cmd.Parameters.Add("P_EFFECTIVE_DATE", OracleDbType.Date).Value = data.EFFECTIVE_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(data.EFFECTIVE_DATE);
                    cmd.Parameters.Add("P_DISABLE_DATE", OracleDbType.Date).Value = data.DISABLE_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(data.DISABLE_DATE);
                    cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM133P";
                    cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                    ds = oOracledb.SqlQueryDS_CMD(cmd);
                    dt = ds.Tables[0];
                }
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM133P_Save");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM133P_DeleteExcept")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM133P_DeleteExcept([FromBody] dynamic pDataList)
        {
            try
            {
                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM133P_DELETE_EXCEPT";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pDataList.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pDataList.BRANCH_CODE;
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pDataList.PART_NO;
                cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM133P_DeleteExcept");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNM133P_SaveUploadExcept")]
        [Authorize]
        public IHttpActionResult SNM133P_SaveUploadExcept([FromBody] dynamic pDataList)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM133P_SAVE_UPLOAD_EXCEPT";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM133P";
                cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM133P_SaveUploadExcept");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        public string SNM133P_ClearTempExcept()
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM133P_CLEAR_TEMP_EXCEPT";
            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return "DONE";
        }
        [HttpPost, Route("SNM133P_UploadExcelExcept")]
        [Authorize]
        public async Task<IHttpActionResult> SNM133P_UploadExcelExcept(HttpRequestMessage request)
        {
            if (!request.Content.IsMimeMultipartContent())
            {
                throw new HttpResponseException(HttpStatusCode.UnsupportedMediaType);
            }
            var reqParts = await request.Content.ReadAsMultipartAsync();
            List<string> ErrorList = new List<string>();
            try
            {
                string fileLocation = string.Empty;
                foreach (var file in reqParts.Contents)
                {
                    var buffer = await file.ReadAsByteArrayAsync();
                    string fileName = Path.GetFileNameWithoutExtension(file.Headers.ContentDisposition.FileName.Trim('\"'));
                    string ext = Path.GetExtension(file.Headers.ContentDisposition.FileName.Trim('\"'));
                    string FullfileName = String.Concat(fileName, "-", DateTime.Now.ToString("yyyyMMddHHmm"), ext);
                    string outputLocation = "~/tempFile/" + "SNM133P";
                    string outputPath = System.Web.HttpContext.Current.Server.MapPath(outputLocation);
                    fileLocation = String.Concat(outputPath, FullfileName);
                    DataService.SaveBytesToFileByLocation(buffer
                                                    , fileLocation
                                                    , fileLocation);
                }
                ErrorList = SNM133P_InsertDataFromExcelFileExcept(fileLocation);
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM133P_UploadExcelExcept");
                throw new HttpResponseException(HttpStatusCode.NotAcceptable);
            }
            return Json(new { Error = ErrorList });
        }
        public List<string> SNM133P_InsertDataFromExcelFileExcept(string fileLocation)
        {
            string fileLocationN = fileLocation;
            DataTable dt = new DataTable();
            SystemApiController sys = new SystemApiController();
            dt = sys.ReadDataExcel(fileLocationN);
            List<string> ErrList = new List<string>();
            int countrow = 1;
            OracleCommand cmd = new OracleCommand();
            DataSet ds = new DataSet();
            try
            {
                cmd.CommandText = "SP_NP_SNM133P_CLEAR_TEMP_EXCEPT";
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM133P_InsertDataFromExcelFileExcept");
                ErrList.Add("Error : SP_NP_SNM133P_CLEAR_TEMP_EXCEPT please recheck");
                return ErrList;
            }
            foreach (DataRow row in dt.Rows)
            {
                DateTime e;
                DateTime d;
                bool isEffDatetime = DateTime.TryParse(row["EFFECTIVE_DATE"].ToString().Trim(), out e);
                bool isDisDate = DateTime.TryParse(row["DISABLE_DATE"].ToString().Trim(), out d);
                if (isEffDatetime && isDisDate)
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNM133P_UPLOAD_EXCEPT";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_RNO", OracleDbType.Varchar2).Value = row["NO"].ToString().Trim();
                    cmd.Parameters.Add("P_EFFECTIVE_DATE", OracleDbType.Varchar2).Value = Convert.ToDateTime(row["EFFECTIVE_DATE"]).ToString("dd-MMM-yyyy").Trim();
                    cmd.Parameters.Add("P_DISABLE_DATE", OracleDbType.Varchar2).Value = Convert.ToDateTime(row["DISABLE_DATE"]).ToString("dd-MMM-yyyy").Trim();
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = row["CUSTOMER_CODE"].ToString().Trim();
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = row["BRANCH_CODE"].ToString().Trim();
                    cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = row["ORDER_CLASS"].ToString().Trim();
                    cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = row["PART_NO"].ToString().Trim();
                    cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                    cmd = oOracledb.SqlQueryReturnValue(cmd);
                    string status = cmd.Parameters["P_STATUS"].Value.ToString();
                }
                else
                {
                    ErrList.Add("Row " + countrow + "Customer Code : " + row["CUSTOMER_CODE"].ToString().Trim() + "Part no : " + row["PART_NO"].ToString().Trim() + " error please recheck");
                }
                countrow++;
            }
            return ErrList;
        }
        [HttpGet, Route("SNM133P_GetTempDataExcept")]
        [Authorize]
        public IHttpActionResult SNM133P_GetTempDataExcept()
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM133P_GET_TEMP_DATA_EXCEPT";
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM133P_GetTempDataExcept");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        #endregion
        #region SNSE219
        [HttpGet, Route("SNSE219_GetData")]
        [Authorize]
        public IHttpActionResult SNSE219_GetData()
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE219_GET_DATA";
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE219_GetData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE219_UpdateStatus")]
        [Authorize]
        [RunBatchGuard]
        public IHttpActionResult SNSE219_UpdateStatus([FromBody] dynamic pParams)
        {
            try
            {
                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE219_UPDATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CONFIG_VALUE", OracleDbType.Varchar2).Value = pParams.CONFIG_VALUE;
                cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE219";
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE219_Save");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        #endregion
        #region SNSE152
        [HttpGet, Route("SNSE152_getCustomerCode/{cusID}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE152_getCustomerCode(string cusID, int noofshowrec)
        {
            DataSet ds = new DataSet();
            cusID = ismoUtil.escapeCharacter(cusID);
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE152_CUSTOMER_CODE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = cusID.ToUpper() == "NULL" ? (object)DBNull.Value : cusID;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE152_getCustomerCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpGet, Route("SNSE152_getPartNo/{partNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE152_getPartNo(string partNo, int noofshowrec)
        {
            DataSet ds = new DataSet();
            partNo = ismoUtil.escapeCharacter(partNo);
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE152_PART_NO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = partNo.ToUpper() == "NULL" ? (object)DBNull.Value : partNo;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE152_getPartNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpGet, Route("SNSE152_getModelCode/{modelCode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE152_getModelCode(string modelCode, int noofshowrec)
        {
            DataSet ds = new DataSet();
            modelCode = ismoUtil.escapeCharacter(modelCode);
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE152_MODEL_CODE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_MODEL_CODE", OracleDbType.Varchar2).Value = modelCode.ToUpper() == "NULL" ? (object)DBNull.Value : modelCode;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE152_getModelCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpGet, Route("SNSE152_getPricingRole/{pricingRole}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE152_getPricingRole(string pricingRole, int noofshowrec)
        {
            DataSet ds = new DataSet();
            pricingRole = ismoUtil.escapeCharacter(pricingRole);
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE152_PRICING_ROLE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PRICING_ROLE", OracleDbType.Varchar2).Value = pricingRole.ToUpper() == "NULL" ? (object)DBNull.Value : pricingRole;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE152_getPricingRole");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNSE152_getData")]
        [Authorize]
        public IHttpActionResult SNSE152_getData([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE152_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pParam.PART_NO;
                cmd.Parameters.Add("P_MODEL_CODE", OracleDbType.Varchar2).Value = pParam.MODEL_CODE;
                cmd.Parameters.Add("P_GR_RATE", OracleDbType.Int32).Value = pParam.GR_RATE == "" ? pParam.GR_RATE = null : pParam.GR_RATE;
                cmd.Parameters.Add("P_CAL_DIS_RATE", OracleDbType.Varchar2).Value = pParam.CAL_DIS_RATE;
                cmd.Parameters.Add("P_SETTING_TYPE", OracleDbType.Varchar2).Value = pParam.SETTING_TYPE == "" ? pParam.SETTING_TYPE = null : pParam.SETTING_TYPE;
                cmd.Parameters.Add("P_CONFIRM_STATUS", OracleDbType.Varchar2).Value = pParam.CONFIRM_STATUS == "" ? pParam.CONFIRM_STATUS = null : pParam.CONFIRM_STATUS;
                cmd.Parameters.Add("P_OPP_GR_RATE", OracleDbType.Varchar2).Value = pParam.OPP_GR_RATE;
                cmd.Parameters.Add("P_COMPLETE_STATUS", OracleDbType.Varchar2).Value = pParam.COMPLETE_STATUS == "" ? pParam.COMPLETE_STATUS = null : pParam.COMPLETE_STATUS;
                cmd.Parameters.Add("P_ITP_PARTS_FLAG", OracleDbType.Varchar2).Value = pParam.ITP_FLAG == "" ? pParam.ITP_FLAG = null : pParam.ITP_FLAG;
                cmd.Parameters.Add("P_PRICING_ROLE", OracleDbType.Varchar2).Value = pParam.PRICING_ROLE == "" ? pParam.PRICING_ROLE = null : pParam.PRICING_ROLE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE152_getData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNSE152_CalAdjRate")]
        [Authorize]
        public IHttpActionResult SNSE152_CalAdjRate([FromBody] dynamic[] pDataList)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();
            try
            {
                foreach (var data in pDataList)
                {
                    OracleCommand cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE0152_CAL_ADJ_RATE";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_PART_ON", OracleDbType.Varchar2).Value = data.PART_NO;
                    cmd.Parameters.Add("P_PMS_ADJ_RATE", OracleDbType.Varchar2).Value = data.PMS_ADJ_RATE;
                    cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE152";
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                    ds = oOracledb.SqlQueryDS_CMD(cmd);
                    dt = ds.Tables[0];
                }
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE152_CalAdjRate");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNSE152_CalDisRate")]
        [Authorize]
        public IHttpActionResult SNSE152_CalDisRate([FromBody] dynamic[] pDataList)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();
            try
            {
                foreach (var data in pDataList)
                {
                    OracleCommand cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE152_CAL_DC_SETTING";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_BUYER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = data.PART_NO;
                    cmd.Parameters.Add("P_DESCRIPTION_CODE", OracleDbType.Varchar2).Value = data.DESCRIPTION_CODE;
                    cmd.Parameters.Add("P_SALES_CLASS_CODE", OracleDbType.Varchar2).Value = data.SALES_CLASS_CODE;
                    cmd.Parameters.Add("P_CAL_DIS_RATE", OracleDbType.Varchar2).Value = data.CAL_DIS_RATE;
                    cmd.Parameters.Add("P_RETAIL_PRICE", OracleDbType.Varchar2).Value = data.RETAIL_PRICE;
                    cmd.Parameters.Add("P_GDR_TYPE", OracleDbType.Varchar2).Value = data.GDR_TYPE;
                    cmd.Parameters.Add("P_GLOBAL_DISCOUNT_RATE", OracleDbType.Varchar2).Value = data.GLOBAL_DISCOUNT_RATE;
                    cmd.Parameters.Add("P_COMPLETE_STATUS", OracleDbType.Varchar2).Value = data.COMPLETE_STATUS;
                    cmd.Parameters.Add("P_PMS_FOB_PRICE", OracleDbType.Varchar2).Value = data.PMS_FOB_PRICE;
                    cmd.Parameters.Add("P_STANDARD_COST", OracleDbType.Varchar2).Value = data.STANDARD_COST;
                    cmd.Parameters.Add("P_PMS_ADJ_RATE", OracleDbType.Varchar2).Value = data.PMS_ADJ_RATE;
                    cmd.Parameters.Add("P_GLOBAL_LIST_PRICE_JPY", OracleDbType.Varchar2).Value = data.GLOBAL_LIST_PRICE_JPY;
                    cmd.Parameters.Add("P_GLOBAL_FOB_PRICE", OracleDbType.Varchar2).Value = data.GLOBAL_FOB_PRICE;
                    cmd.Parameters.Add("P_EXCHANGE_RATE_JPY", OracleDbType.Varchar2).Value = data.EXCHANGE_RATE_JPY;
                    cmd.Parameters.Add("P_TEMP_COST", OracleDbType.Varchar2).Value = data.TEMP_COST;
                    cmd.Parameters.Add("P_ITP_PARTS_FLAG", OracleDbType.Varchar2).Value = data.ITP_PARTS_FLAG;
                    cmd.Parameters.Add("P_ADJ_FOB_PRICE", OracleDbType.Varchar2).Value = data.ADJ_FOB_PRICE;
                    cmd.Parameters.Add("P_CONFIRM_STATUS", OracleDbType.Varchar2).Value = data.CONFIRM_STATUS;
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                    ds = oOracledb.SqlQueryDS_CMD(cmd);
                    dt = ds.Tables[0];
                }
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE152_CalDisRate");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNSE152_RegisConfirm")]
        [Authorize]
        public IHttpActionResult SNSE152_RegisConfirm([FromBody] dynamic[] pDataList)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();
            try
            {
                foreach (var data in pDataList)
                {
                    OracleCommand cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE152_REGIS_COMFIRM";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_BUYER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = data.PART_NO;
                    cmd.Parameters.Add("P_COMPLETE_STATUS", OracleDbType.Varchar2).Value = data.COMPLETE_STATUS;
                    cmd.Parameters.Add("P_STANDARD_COST", OracleDbType.Varchar2).Value = data.STANDARD_COST;
                    cmd.Parameters.Add("P_GLOBAL_LIST_PRICE", OracleDbType.Varchar2).Value = data.GLOBAL_LIST_PRICE;
                    cmd.Parameters.Add("P_TEMP_COST", OracleDbType.Varchar2).Value = data.TEMP_COST;
                    cmd.Parameters.Add("P_ITP_PARTS_FLAG", OracleDbType.Varchar2).Value = data.ITP_PARTS_FLAG;
                    cmd.Parameters.Add("P_CONFIRM_STATUS", OracleDbType.Varchar2).Value = data.CONFIRM_STATUS;
                    cmd.Parameters.Add("P_FOB_PRICE_FC", OracleDbType.Varchar2).Value = data.FOB_PRICE_FC;
                    cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_PRICE_APP", OracleDbType.Date).Value = data.EFFECTIVE_DATE;
                    cmd.Parameters.Add("P_CURRENCY_CODE", OracleDbType.Varchar2).Value = data.CURRENCY_CODE;
                    cmd.Parameters.Add("P_GP_RATE", OracleDbType.Varchar2).Value = data.GP_RATE;
                    cmd.Parameters.Add("P_FOB_PRICE_FLAG", OracleDbType.Varchar2).Value = data.FOB_PRICE_FLAG;
                    cmd.Parameters.Add("P_PMS_GP_RATE", OracleDbType.Varchar2).Value = data.PMS_GP_RATE;
                    cmd.Parameters.Add("P_ADJ_GP_RATE", OracleDbType.Varchar2).Value = data.ADJ_GP_RATE;
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                    ds = oOracledb.SqlQueryDS_CMD(cmd);
                    dt = ds.Tables[0];
                }
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE152_RegisConfirm");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNSE152_ExportExcel")]
        [Authorize]
        public HttpResponseMessage SNSE152_ExportExcel(List<DataModel.SNSE152> Data)
        {
            HttpResponseMessage response = new HttpResponseMessage(HttpStatusCode.OK);
            string gLocation = "~/tempFile/" + "SNSE152" + "/";
            string gWriteLocation = System.Web.HttpContext.Current.Server.MapPath(gLocation);
            FileInfo filePath = new FileInfo(gWriteLocation);
            filePath.Directory.Create();
            string gFileLocation = Path.Combine(gWriteLocation, "SNSE152.xlsx");
            string gReturnLocation = Path.Combine(gLocation, "SNSE152.xlsx");
            var excel2 = Data.ToExcel(x => x
            .AddColumn("Seq.", z => z.SEQ)
            .AddColumn("Customer Code", z => z.CUSTOMER_CODE)
            .AddColumn("Part No.", z => z.PART_NO)
            .AddColumn("Part Name", z => z.PART_NAME)
            .AddColumn("ITP Part Flag", z => z.ITP_PARTS_FLAG)
            .AddColumn("Pricing Role", z => z.PRICING_ROLE)
            .AddColumn("Pricing Role GP Rate", z => z.DSP_GP_RATE_PRICING_ROLE)
            .AddColumn("Calculate By", z => z.CALCULATE_BY)
            .AddColumn("In Store Cost(THB)", z => z.STANDARD_COST)
            .AddColumn("Average Cost(THB)", z => z.AVERAGE_COST)
            .AddColumn("Retail Price(THB)", z => z.RETAIL_PRICE)
            .AddColumn("Temp Cost(THB)", z => z.TEMP_COST)
            .AddColumn("Model Code", z => z.MODEL_CODE)
            .AddColumn("FOB Price(THB)", z => z.GLOBAL_FOB_PRICE)
            .AddColumn("GP Rate(%)", z => z.GP_RATE)
            .AddColumn("GP Amount(THB)", z => z.DIF_GLOBAL_COST)
            .AddColumn("FOB Adjust(THB)", z => z.ADJ_FOB_PRICE)
            .AddColumn("GP Rate(%)", z => z.ADJ_GP_RATE)
            .AddColumn("GP Amount(THB)", z => z.DIF_ADJ_COST)
            .AddColumn("FOB Price Customer", z => z.FOB_PRICE_FC)
            .AddColumn("Currency Code Customer", z => z.CURRENCY_CODE)
            .AddColumn("Exchange Rate(THB-->??)", z => z.EXCHANGE_RATE)
            .AddColumn("Effective Date", z => z.PRICE_APP == null ? (object)DBNull.Value : Convert.ToDateTime(z.PRICE_APP).ToString("dd-MM-yyyy"))
            .AddColumn("FOB Price Flag", z => z.DSC_FOB_PRICE_FLAG)
            .AddColumn("FOB Price Registered", z => z.FOB_PRICE_CONFIRM)
            .AddColumn("GP Rate(%) Registered", z => z.GP_RATE_COMFIRM)
            .AddColumn("Price Info Type", z => z.SETTING_TYPE_D)
            .AddColumn("Price Info Due Date", z => z.SETTING_DUE_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(z.SETTING_DUE_DATE).ToString("dd-MM-yyyy"))
            .AddColumn("Export Net Price", z => z.EXPORT_NET_PRICE)
            .AddColumn("Export Currency Code", z => z.EXPORT_CURRENCY_CODE)
            .AddColumn("Registration Status", z => z.DSP_CONFIRM_STATUS)
            .AddColumn("Create By", z => z.CREATE_BY)
            .AddColumn("Create Date", z => z.CREATE_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(z.CREATE_DATE).ToString("dd-MM-yyyy"))
            .AddColumn("Last Update by Program ", z => z.LAST_UPD_PGM)
            .AddColumn("Last Update by User ", z => z.LAST_UPD_BY)
            .AddColumn("Last Update Date ", z => z.LAST_UPD_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(z.LAST_UPD_DATE).ToString("dd-MM-yyyy"))
            .AddColumn("Message Error", z => z.MESSAGE_ERROR)
            );
            byte[] excel = excel2;
            File.WriteAllBytes(gFileLocation, excel);
            string path = HttpContext.Current.Server.MapPath("~/" + gReturnLocation.Substring(1));
            if (!File.Exists(path))
            {
                return Request.CreateErrorResponse(HttpStatusCode.NotFound, "File not found.");
            }
            try
            {
                string mimeType = MimeMapping.GetMimeMapping("SNSE152.xlsx");
                StreamContent x = new StreamContent(new FileStream(path, FileMode.Open, FileAccess.Read));
                response.Content = x;
                response.Content.Headers.ContentType = new MediaTypeHeaderValue(mimeType);
                response.Content.Headers.ContentDisposition = new ContentDispositionHeaderValue("attachment");
                response.Content.Headers.ContentDisposition.FileName = "SNSE152.xlsx";
            }
            catch (Exception ex)
            {
                return Request.CreateResponse(HttpStatusCode.BadRequest, ex);
            }
            return response;
        }
        #endregion
        #region SNM145P
        [HttpGet, Route("SNM145P_GetCustomerCode/{customerCode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM145P_GetCustomerCode(string customerCode, int noofshowrec)
        {
            DataSet ds = new DataSet();
            customerCode = ismoUtil.escapeCharacter(customerCode);
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM145P_GET_CUSTOMER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = customerCode.ToUpper() == "NULL" ? (object)DBNull.Value : customerCode;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM145P_GetCustomerCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNM145P_GetCustomerCodeAdd/{customerCode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM145P_GetCustomerCodeAdd(string customerCode, int noofshowrec)
        {
            DataSet ds = new DataSet();
            customerCode = ismoUtil.escapeCharacter(customerCode);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM145P_GET_CUSTOMER_ADD";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = customerCode.ToUpper() == "NULL" ? (object)DBNull.Value : customerCode;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM145P_GetCustomerCodeAdd");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNM145P_GetData")]
        [Authorize]
        public IHttpActionResult SNM145P_GetData([FromBody] dynamic pParams)
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM145P_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParams["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_ISSUE_STATUS", OracleDbType.Varchar2).Value = pParams["ISSUE_STATUS"];
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM145P_GetData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNM145P_SaveAdd")]
        [Authorize]
        public IHttpActionResult SNM145P_SaveAdd([FromBody] dynamic pParams)
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM145P_INSERT";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParams.CUSTOMER_CODE;
                cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = pParams.SENT_BY;
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM145P";
                cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM145P_SaveAdd");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNM145P_UpdateStatus")]
        [Authorize]
        public IHttpActionResult SNM145P_UpdateStatus([FromBody] dynamic[] pDataList)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();
            try
            {
                foreach (var data in pDataList)
                {
                    OracleCommand cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNM145P_UPD_STATUS";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = data.SENT_BY;
                    cmd.Parameters.Add("P_ISSUE_STATUS", OracleDbType.Varchar2).Value = data.V_ISSUE_STATUS;
                    cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM145P";
                    cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                    ds = oOracledb.SqlQueryDS_CMD(cmd);
                }
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM145P_UpdateStatus");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("CheckIssueInvoiceRunning")]
        [Authorize]
        public IHttpActionResult CheckIssueInvoiceRunning([FromBody] dynamic pParams)
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_CHECK_ISSUE_INV_RUNNING";
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParams.CUSTOMER_CODE;
                cmd.Parameters.Add("P_SENT_BY", OracleDbType.Varchar2).Value = pParams.SENT_BY;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (OracleException oex)
            {
                string err_massage = string.Empty;
                if (oex.Number == 20001)
                {
                    err_massage = "Someone is working on this customer. Please try it later";
                }
                oOracledb.WriteLog("ErrorHandle", oex.Number + " - " + err_massage, "CheckIssueInvoiceRunning");
                return Ok(new { status = "fail", massage = err_massage });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "CheckIssueInvoiceRunning");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.NotAcceptable, ex.Message));
            }
        }
        #endregion


        #region SNSE222


        [HttpPost, Route("SNSE222_GetInvoiceList")]
        [Authorize]
        public IHttpActionResult SNSE222_GetInvoiceList([FromBody] dynamic pParam)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE222_GET_INVOICE_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_START_DATE", OracleDbType.Date).Value = pParam.START_DATE;
                cmd.Parameters.Add("P_END_DATE", OracleDbType.Date).Value = pParam.END_DATE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE222_GetInvoiceList");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        //[HttpPost, Route("SNSE222_GetInvoiceDetail")]
        //[Authorize]
        //public IHttpActionResult SNSE222_GetInvoiceDetail([FromBody] dynamic pParam)
        //{
        //    DataSet ds = new DataSet();
        //    OracleCommand cmd = new OracleCommand();
        //    try
        //    {
        //        cmd.CommandText = "SP_NP_SNWR023_GET_INVOICE_DETAIL";
        //        cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
        //        cmd.Parameters.Add("P_DOCUMENT_NO", OracleDbType.Varchar2).Value = pParam.DOCUMENT_NO;
        //        cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
        //        cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
        //        ds = oOracledb.SqlQueryDS_CMD(cmd);
        //        return Json(DTFM.convertToList(ds));
        //    }
        //    catch (Exception ex)
        //    {
        //        oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE222_GetInvoiceDetail");
        //        return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
        //    }
        //}

        #endregion
        #region SNSE223
        [HttpPost, Route("SNSE223_GetOrderList")]
        [Authorize]
        public IHttpActionResult SNSE223_GetOrderList([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE223_ORDER_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam.BRANCH_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pParam.ORDER_NO;
                cmd.Parameters.Add("P_ORDER_DATE", OracleDbType.Date).Value = pParam.ORDER_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(pParam.ORDER_DATE);
                cmd.Parameters.Add("P_DOCUMENT_NO", OracleDbType.Varchar2).Value = pParam.DOCUMENT_NO;
                cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE223_ORDER_LIST");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNSE223_GetOrderDetail")]
        [Authorize]
        public IHttpActionResult SNSE223_GetOrderDetail([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE223_GET_DETAIL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam.BRANCH_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pParam.ORDER_NO;
                cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("c_Return2", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE223_GET_DETAIL");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpGet, Route("SNSE223_GetBOList/{customerCode}/{branchCode}/{orderNo}/{orderDate}")]
        [Authorize]
        public IHttpActionResult SNSE223_GetBOList(string customerCode, string branchCode, string orderNo, string orderDate)
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE223_BO_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = customerCode;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = branchCode;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = orderNo;
                cmd.Parameters.Add("P_ORDER_DATE", OracleDbType.Date).Value = Convert.ToDateTime(orderDate);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE223_BO_LIST");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpGet, Route("SNSE223_GetETPList/{customerCode}/{branchCode}/{orderNo}/{orderDate}")]
        [Authorize]
        public IHttpActionResult SNSE223_GetETPList(string customerCode, string branchCode, string orderNo, string orderDate)
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE223_ETP_List";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = customerCode;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = branchCode;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = orderNo;
                cmd.Parameters.Add("P_ORDER_DATE", OracleDbType.Date).Value = Convert.ToDateTime(orderDate);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE223_ETP_List");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpGet, Route("SNSE223_GetCustomer/{cusID}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE223_GetCustomer(string cusID, int noofshowrec)
        {
            DataSet ds = new DataSet();
            cusID = ismoUtil.escapeCharacter(cusID);
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE223_CUSTOMER";
                cmd.Parameters.Add("PCOMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("PCUSTOMER_CODE", OracleDbType.Varchar2).Value = cusID.ToUpper() == "NULL" ? (object)DBNull.Value : cusID;
                cmd.Parameters.Add("PSHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("CRETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE223_CUSTOMER");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpGet, Route("SNSE223_GetSearchCust/{cusID}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE223_GetSearchCust(string cusID, int noofshowrec)
        {
            DataSet ds = new DataSet();
            cusID = ismoUtil.escapeCharacter(cusID);
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE223_GET_CUSTOMER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = cusID.ToUpper() == "NULL" ? (object)DBNull.Value : cusID;
                cmd.Parameters.Add("P_SHOW_REC", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE223_GET_CUSTOMER");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNSE223_GetParts/{partNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE223_GetParts(string partNo, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            partNo = ismoUtil.escapeCharacter(partNo);
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE223_GETPARTS";
                cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParam["BRANCH_CODE"];
                cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = pParam["ORDER_CLASS"];
                cmd.Parameters.Add("p_PART_NO", OracleDbType.Varchar2).Value = (partNo != null && partNo == "null" ? "" : partNo);
                cmd.Parameters.Add("p_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE223_GETPARTS");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNSE223_GetSearchBrach/{branchCode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE223_GetSearchBrach(string branchCode, int noofshowrec, [FromBody] dynamic pData)
        {
            DataSet ds = new DataSet();
            branchCode = ismoUtil.escapeCharacter(branchCode);
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE223_GET_BRANCH";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = branchCode.ToUpper() == "NULL" ? (object)DBNull.Value : branchCode;
                cmd.Parameters.Add("P_SHOW_REC", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE223_GET_BRANCH");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNSE223_GetBranchCode/{branchCode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE223_GetBranchCode(string branchCode, int noofshowrec, [FromBody] dynamic pCustomer)
        {
            DataSet ds = new DataSet();
            branchCode = ismoUtil.escapeCharacter(branchCode);
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE223_BRANCH_CODE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pCustomer["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = branchCode.ToUpper() == "NULL" ? (object)DBNull.Value : branchCode;
                cmd.Parameters.Add("P_SHOW_REC", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE223_BRANCH_CODE");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNSE223_GetCustCodeOrderNo/{OrderNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE223_GetCustCodeOrderNo(string OrderNo, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            var pData = pParam["DATA"];
            OrderNo = ismoUtil.escapeCharacter(OrderNo);
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE223_ORDER_NO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData["CUSTOMER_CODE"];
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pData["BRANCH_CODE"];
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = OrderNo.ToUpper() == "NULL" ? (object)DBNull.Value : OrderNo;
                cmd.Parameters.Add("P_SHOW_REC", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE223_ORDER_NO");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpGet, Route("SNSE223_GetDocumentNo/{DocumentNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE223_GetDocumentNo(string DocumentNo, int noofshowrec)
        {
            DataSet ds = new DataSet();
            DocumentNo = ismoUtil.escapeCharacter(DocumentNo);
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE223_GET_DOC";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DOCUMENT_NO", OracleDbType.Varchar2).Value = DocumentNo.ToUpper() == "NULL" ? (object)DBNull.Value : DocumentNo;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE223_GET_DOC");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNSE223_CheckOrder")]
        [Authorize]
        public IHttpActionResult SNSE223_CheckOrder([FromBody] dynamic pParam)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE223_CHECK_ORDER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pParam.ORDER_NO;
                cmd.Parameters.Add("R_ERROR_MSG", OracleDbType.Varchar2, 255).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);
                if (cmd.Parameters["R_ERROR_MSG"].Value.ToString() != "Done")
                {
                    return Json(new
                    {
                        Result = "Fail",
                        Message = cmd.Parameters["R_ERROR_MSG"].Value.ToString()
                    });
                }
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE223_CHECK_ORDER");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
            return Ok();
        }
        [HttpGet, Route("SNSE223_SNSC003/{customerCode}/{orderDate}/{partNo}/{orderClass}/{vendorCode}")]
        [Authorize]
        public IHttpActionResult SNSE223_SNSC003(string customerCode, string orderDate, string partNo, string orderClass, string vendorCode)
        {
            decimal RETAIL_D
                , RETAIL_E
                , UNIT_D
                , UNIT_E
                , DISC_1
                , DISC_2
                , DISC_3
                , AVG_COST
                , STD_COST;
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SNSC003A";
                cmd.Parameters.Add("CUST_CODE", OracleDbType.Varchar2).Value = customerCode;
                cmd.Parameters.Add("ORDER_DATE", OracleDbType.Date).Value = Convert.ToDateTime(orderDate);
                cmd.Parameters.Add("PARTNO", OracleDbType.Varchar2).Value = partNo;
                cmd.Parameters.Add("OR_CLASS", OracleDbType.Varchar2).Value = orderClass;
                cmd.Parameters.Add("VD_CODE", OracleDbType.Varchar2).Value = vendorCode;
                cmd.Parameters.Add("OVERSEAFLAG", OracleDbType.Varchar2, 1).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("RETAIL_D", OracleDbType.Decimal).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("RETAIL_E", OracleDbType.Decimal).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("UNIT_D", OracleDbType.Decimal).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("UNIT_E", OracleDbType.Decimal).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("DISC_1", OracleDbType.Decimal).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("DISC_2", OracleDbType.Decimal).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("DISC_3", OracleDbType.Decimal).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("DISC_GROUP", OracleDbType.Varchar2, 3).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("DISC_CODE", OracleDbType.Varchar2, 2).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("AVG_COST", OracleDbType.Decimal).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("STD_COST", OracleDbType.Decimal).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);
                return Json(new
                {
                    OVERSEAFLAG = cmd.Parameters["OVERSEAFLAG"].Value.ToString(),
                    RETAIL_D = Decimal.TryParse(cmd.Parameters["RETAIL_D"].Value.ToString(), out RETAIL_D) ? RETAIL_D : 0,
                    RETAIL_E = Decimal.TryParse(cmd.Parameters["RETAIL_E"].Value.ToString(), out RETAIL_E) ? RETAIL_E : 0,
                    UNIT_D = Decimal.TryParse(cmd.Parameters["UNIT_D"].Value.ToString(), out UNIT_D) ? UNIT_D : 0,
                    UNIT_E = Decimal.TryParse(cmd.Parameters["UNIT_E"].Value.ToString(), out UNIT_E) ? UNIT_E : 0,
                    DISC_1 = Decimal.TryParse(cmd.Parameters["DISC_1"].Value.ToString(), out DISC_1) ? DISC_1 : 0,
                    DISC_2 = Decimal.TryParse(cmd.Parameters["DISC_2"].Value.ToString(), out DISC_2) ? DISC_2 : 0,
                    DISC_3 = Decimal.TryParse(cmd.Parameters["DISC_3"].Value.ToString(), out DISC_3) ? DISC_3 : 0,
                    DISC_GROUP = cmd.Parameters["DISC_GROUP"].Value.ToString(),
                    DISC_CODE = cmd.Parameters["DISC_CODE"].Value.ToString(),
                    AVG_COST = Decimal.TryParse(cmd.Parameters["AVG_COST"].Value.ToString(), out AVG_COST) ? AVG_COST : 0,
                    STD_COST = Decimal.TryParse(cmd.Parameters["STD_COST"].Value.ToString(), out STD_COST) ? STD_COST : 0
                });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE223_SNSC003A");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNSE223_GetCampaignSpecialDiscount")]
        [Authorize]
        public IHttpActionResult SNSE223_GetCampaignSpecialDiscount([FromBody] dynamic pParams)
        {
            decimal DISCOUNT_RATE1
                , DISCOUNT_RATE2
                , DISCOUNT_RATE3
                , UNIT_PRICE_B
                , UNIT_PRICE;
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE223_GET_CMP_DIS";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pParams.ORDER_NO;
                cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = pParams.ORDER_CLASS;
                cmd.Parameters.Add("P_ORDER_DATE", OracleDbType.Date).Value = Convert.ToDateTime(pParams.ORDER_DATE);
                cmd.Parameters.Add("P_SALES_CHANNEL", OracleDbType.Varchar2).Value = pParams.SALES_CHANNEL;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParams.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParams.BRANCH_CODE;
                cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = pParams.ORDER_PART_NO;
                cmd.Parameters.Add("P_ORDER_QTY", OracleDbType.Decimal).Value = pParams.ORDER_QTY;
                cmd.Parameters.Add("P_PART_LIST", OracleDbType.Varchar2).Value = pParams.PART_LIST;
                cmd.Parameters.Add("R_DISCOUNT_RATE1", OracleDbType.Decimal).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_DISCOUNT_RATE2", OracleDbType.Decimal).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_DISCOUNT_RATE3", OracleDbType.Decimal).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_UNIT_PRICE_B", OracleDbType.Decimal).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_UNIT_PRICE", OracleDbType.Decimal).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_DISCOUNT_CODE", OracleDbType.Varchar2, 10).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_CAMPAIGN_ID", OracleDbType.Varchar2, 10).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_CAMPAIGN_TYPE", OracleDbType.Varchar2, 1).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_CAMPAIGN_REF", OracleDbType.Varchar2, 10).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_DSP_CAMPAIGN_NAME", OracleDbType.Varchar2, 100).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);
                return Json(new
                {
                    DISCOUNT_RATE1 = Decimal.TryParse(cmd.Parameters["R_DISCOUNT_RATE1"].Value.ToString(), out DISCOUNT_RATE1) ? DISCOUNT_RATE1 : 0,
                    DISCOUNT_RATE2 = Decimal.TryParse(cmd.Parameters["R_DISCOUNT_RATE2"].Value.ToString(), out DISCOUNT_RATE2) ? DISCOUNT_RATE2 : 0,
                    DISCOUNT_RATE3 = Decimal.TryParse(cmd.Parameters["R_DISCOUNT_RATE3"].Value.ToString(), out DISCOUNT_RATE3) ? DISCOUNT_RATE3 : 0,
                    UNIT_PRICE_B = Decimal.TryParse(cmd.Parameters["R_UNIT_PRICE_B"].Value.ToString(), out UNIT_PRICE_B) ? UNIT_PRICE_B : 0,
                    UNIT_PRICE = Decimal.TryParse(cmd.Parameters["R_UNIT_PRICE"].Value.ToString(), out UNIT_PRICE) ? UNIT_PRICE : 0,
                    DISCOUNT_CODE = cmd.Parameters["R_DISCOUNT_CODE"].Value.ToString(),
                    CAMPAIGN_ID = cmd.Parameters["R_CAMPAIGN_ID"].Value.ToString(),
                    CAMPAIGN_TYPE = cmd.Parameters["R_CAMPAIGN_TYPE"].Value.ToString(),
                    CAMPAIGN_REF = cmd.Parameters["R_CAMPAIGN_REF"].Value.ToString(),
                    DSP_CAMPAIGN_NAME = cmd.Parameters["R_DSP_CAMPAIGN_NAME"].Value.ToString()
                });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE223_GET_CMP_DIS");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNSE223_SaveDocument")]
        [Authorize]
        public IHttpActionResult SNSE223_SaveDocument([FromBody] dynamic pData)
        {
            int iCount = 0;
            OracleCommand cmd = new OracleCommand();
            var partList = new List<dynamic>(pData.PART_LIST);
            if (partList.Count > 0)
            {
                try
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE223_CHG_ROUTE";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pData.BRANCH_CODE;
                    cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pData.ORDER_NO;
                    cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = pData.ORDER_CLASS;
                    cmd.Parameters.Add("P_ORDER_DATE", OracleDbType.Date).Value = Convert.ToDateTime(pData._ORDER_DATE);
                    cmd.Parameters.Add("P_CUR_ROUTE_CODE", OracleDbType.Varchar2).Value = pData.CUR_ROUTE_CODE;
                    cmd.Parameters.Add("P_CUR_TRANSPORT_CODE", OracleDbType.Varchar2).Value = pData.CUR_TRANSPORT_CODE;
                    cmd.Parameters.Add("P_CHANGE_ROUTE_CUSTOMER", OracleDbType.Varchar2).Value = pData.CHANGE_ROUTE_CUSTOMER;
                    cmd.Parameters.Add("P_CHANGE_ROUTE_BRANCH", OracleDbType.Varchar2).Value = pData.CHANGE_ROUTE_BRANCH;
                    cmd.Parameters.Add("P_ROUTE_CODE", OracleDbType.Varchar2).Value = pData.ROUTE_CODE;
                    cmd.Parameters.Add("P_TRANSPORT_CODE", OracleDbType.Varchar2).Value = pData.TRANSPORT_CODE;
                    cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE223_M";
                    oOracledb.SqlQueryReturnValue(cmd);
                    foreach (var detail in partList)
                    {
                        iCount++;
                        cmd = new OracleCommand();
                        cmd.CommandText = "SP_NP_SNSE223_SAVE_DETAIL_TEMP";
                        cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                        cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                        cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pData.BRANCH_CODE;
                        cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pData.ORDER_NO;
                        cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Decimal).Value = iCount;
                        cmd.Parameters.Add("P_ORDER_QTY", OracleDbType.Decimal).Value = detail.ORDER_QTY;
                        cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = detail.ORDER_PART_NO;
                        cmd.Parameters.Add("P_SALES_PACKAGE", OracleDbType.Varchar2).Value = detail.SALES_PACKAGE;
                        cmd.Parameters.Add("P_OH_QTY", OracleDbType.Varchar2).Value = detail.OH_QTY;
                        cmd.Parameters.Add("P_OVERSEA_FLAG_C", OracleDbType.Varchar2).Value = detail.OVERSEA_FLAG_C;
                        cmd.Parameters.Add("P_RETAIL_PRICE_B", OracleDbType.Decimal).Value = detail.RETAIL_PRICE_B;
                        cmd.Parameters.Add("P_RETAIL_PRICE_C", OracleDbType.Decimal).Value = detail.RETAIL_PRICE_C;
                        cmd.Parameters.Add("P_UNIT_PRICE_B", OracleDbType.Decimal).Value = detail.UNIT_PRICE_B;
                        cmd.Parameters.Add("P_UNIT_PRICE_C", OracleDbType.Decimal).Value = detail.UNIT_PRICE_C;
                        cmd.Parameters.Add("P_UNIT_PRICE_EDP", OracleDbType.Decimal).Value = detail.UNIT_EDP;
                        cmd.Parameters.Add("P_DISCOUNT_RATE1", OracleDbType.Decimal).Value = detail.DISCOUNT_RATE1;
                        cmd.Parameters.Add("P_DISCOUNT_RATE2", OracleDbType.Decimal).Value = detail.DISCOUNT_RATE2;
                        cmd.Parameters.Add("P_DISCOUNT_RATE3", OracleDbType.Decimal).Value = detail.DISCOUNT_RATE3;
                        cmd.Parameters.Add("P_DISCOUNT_GROUP", OracleDbType.Varchar2).Value = detail.DISCOUNT_GROUP;
                        cmd.Parameters.Add("P_DISCOUNT_CODE", OracleDbType.Varchar2).Value = detail.DISCOUNT_CODE;
                        cmd.Parameters.Add("P_AVERAGE_COST", OracleDbType.Decimal).Value = detail.AVERAGE_COST;
                        cmd.Parameters.Add("P_STANDARD_COST", OracleDbType.Decimal).Value = detail.STANDARD_COST;
                        cmd.Parameters.Add("P_STATUS", OracleDbType.Varchar2).Value = detail.STATUS;
                        cmd.Parameters.Add("P_REASON_CODE", OracleDbType.Decimal).Value = detail.REASON_CODE;
                        cmd.Parameters.Add("P_CAMPAIGN_ID", OracleDbType.Varchar2).Value = detail.CAMPAIGN_ID;
                        cmd.Parameters.Add("P_CAMPAIGN_REF", OracleDbType.Varchar2).Value = detail.CAMPAIGN_REF;
                        cmd.Parameters.Add("P_CLEAR_DATA", OracleDbType.Varchar2).Value = iCount == 1 ? "Y" : "N";
                        cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                        oOracledb.SqlQueryReturnValue(cmd);
                    }
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE223_SAVE_TRANSACTION";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_CUSTOMER_BRANCH", OracleDbType.Varchar2).Value = pData.BRANCH_CODE;
                    cmd.Parameters.Add("P_CUSTOMER_NAME", OracleDbType.Varchar2).Value = pData.CUSTOMER_NAME;
                    cmd.Parameters.Add("P_CUST_ADDR1", OracleDbType.Varchar2).Value = pData.CUST_ADDR1;
                    cmd.Parameters.Add("P_CUST_ADDR2", OracleDbType.Varchar2).Value = pData.CUST_ADDR2;
                    cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pData.ORDER_NO;
                    cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = pData.ORDER_CLASS;
                    cmd.Parameters.Add("P_ORDER_TYPE", OracleDbType.Varchar2).Value = pData.ORDER_TYPE;
                    cmd.Parameters.Add("P_ORDER_DATE", OracleDbType.Date).Value = Convert.ToDateTime(pData._ORDER_DATE);
                    cmd.Parameters.Add("P_DELIVERY_CODE", OracleDbType.Varchar2).Value = pData.DELIVERY_CODE;
                    cmd.Parameters.Add("P_ROUTE_CODE", OracleDbType.Varchar2).Value = pData.ROUTE_CODE;
                    cmd.Parameters.Add("P_TRANSPORT_CODE", OracleDbType.Varchar2).Value = pData.TRANSPORT_CODE;
                    cmd.Parameters.Add("P_DISCOUNT_GROUP", OracleDbType.Varchar2).Value = pData.DISCOUNT_GROUP;
                    cmd.Parameters.Add("P_BO_FLAG", OracleDbType.Varchar2).Value = pData.BO_FLAG;
                    cmd.Parameters.Add("P_PRINT_PICKING_CARD", OracleDbType.Varchar2).Value = (pData.PRINT_PICKING_CARD == "true" ? "Y" : "N");
                    cmd.Parameters.Add("P_PRINT_REMARK", OracleDbType.Varchar2).Value = (pData.PRINT_REMARK == "true" ? "Y" : "N");
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_DOC_NO", OracleDbType.Varchar2, 20).Direction = ParameterDirection.Output;
                    oOracledb.SqlQueryReturnValue(cmd);
                    return Json(new
                    {
                        DOCUMENT_NO = cmd.Parameters["P_DOC_NO"].Value.ToString()
                    });
                }
                catch (Exception ex)
                {
                    oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE223_SAVE_TRANSACTION");
                    return ResponseMessage(Request.CreateResponse(HttpStatusCode.BadRequest, "Something went wrong."));
                }
            }
            return ResponseMessage(Request.CreateResponse(HttpStatusCode.BadRequest, "No row to process."));
        }
        [HttpPost, Route("SNSE223_CheckAcceptQuantity")]
        [Authorize]
        public IHttpActionResult SNSE223_CheckAcceptQuantity([FromBody] dynamic pParams)
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE223_SNSC001_SCR";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pParams.PART_NO;
                cmd.Parameters.Add("P_ORDER_QTY", OracleDbType.Varchar2).Value = pParams.ORDER_QTY;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParams.CUSTOMER_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pParams.ORDER_NO;
                cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Varchar2).Value = pParams.ITEM_NO;
                cmd.Parameters.Add("P_SUB_ITEM", OracleDbType.Varchar2).Value = pParams.SUB_ITEM;
                cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = pParams.ORDER_CLASS;
                cmd.Parameters.Add("P_STATUS", OracleDbType.Varchar2).Value = pParams.STATUS;
                cmd.Parameters.Add("P_BO_FLAG", OracleDbType.Varchar2).Value = pParams.BO_FLAG;
                cmd.Parameters.Add("P_AUTO_FLAG", OracleDbType.Varchar2).Value = pParams.AUTO_FLAG;
                cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE223_M";
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE223_CheckAcceptQuantity");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        #endregion

        #region SNM163P
        [HttpGet, Route("SNM163P_GetCustomerCode/{customerCode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM163P_GetCustomerCode(string customerCode, int noofshowrec)
        {
            DataSet ds = new DataSet();

            customerCode = ismoUtil.escapeCharacter(customerCode);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM163P_GET_CUSTOMER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = customerCode.ToUpper() == "NULL" ? (object)DBNull.Value : customerCode;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM163P_GetCustomerCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM163P_GetBranchCode/{branchCode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM163P_GetBranchCode(string branchCode, int noofshowrec, [FromBody] dynamic Pparam)
        {
            DataSet ds = new DataSet();

            branchCode = ismoUtil.escapeCharacter(branchCode);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM163P_GET_BRANCH";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = Pparam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = branchCode.ToUpper() == "NULL" ? (object)DBNull.Value : branchCode;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM163P_GetCustomerCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNM163P_GetPartNo/{partNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM163P_GetPartNo(string partNo, int noofshowrec)
        {
            DataSet ds = new DataSet();

            partNo = ismoUtil.escapeCharacter(partNo);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM163P_GET_PART_NO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = partNo.ToUpper() == "NULL" ? (object)DBNull.Value : partNo;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM163P_GetPartNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNM163P_GetCustomerCodeForAdd/{customerCode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM163P_GetCustomerCodeForAdd(string customerCode, int noofshowrec)
        {
            DataSet ds = new DataSet();

            customerCode = ismoUtil.escapeCharacter(customerCode);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM163P_GET_CUSTOMER_FOR_ADD";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = customerCode.ToUpper() == "NULL" ? (object)DBNull.Value : customerCode;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM163P_GetCustomerCodeForAdd");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM163P_GetBranchCodeForAdd/{branchCode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM163P_GetBranchCodeForAdd(string branchCode, int noofshowrec, [FromBody] dynamic pParams)
        {
            DataSet ds = new DataSet();

            branchCode = ismoUtil.escapeCharacter(branchCode);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM163P_GET_BRANCH_FOR_ADD";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParams.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = branchCode.ToUpper() == "NULL" ? (object)DBNull.Value : branchCode;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM163P_GetBranchCodeForAdd");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNM163P_GetPartNoForAdd/{partNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM163P_GetPartNoForAdd(string partNo, int noofshowrec)
        {
            DataSet ds = new DataSet();

            partNo = ismoUtil.escapeCharacter(partNo);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM163P_GET_PART_NO_FOR_ADD";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = partNo.ToUpper() == "NULL" ? (object)DBNull.Value : partNo;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM163P_GetPartNoForAdd");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM163P_GetData")]
        [Authorize]
        public IHttpActionResult SNM163P_GetData([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM163P_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = param.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = param.BRANCH_CODE;
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = param.PART_NO;
                cmd.Parameters.Add("P_EFFECTIVE_DATE", OracleDbType.Date).Value = param.EFFECTIVE_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(param.EFFECTIVE_DATE);
                cmd.Parameters.Add("P_DISABLE_DATE", OracleDbType.Date).Value = param.DISABLE_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(param.DISABLE_DATE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM163P_GetData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM163P_Save")]
        [Authorize]
        public IHttpActionResult SNM163P_Save([FromBody] dynamic pParams)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            try
            {
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM163P_SAVE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParams.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pParams.BRANCH_CODE;
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pParams.PART_NO;
                cmd.Parameters.Add("P_EFFECTIVE_DATE", OracleDbType.Date).Value = pParams.EFFECTIVE_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(pParams.EFFECTIVE_DATE);
                cmd.Parameters.Add("P_DISABLE_DATE", OracleDbType.Date).Value = pParams.DISABLE_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(pParams.DISABLE_DATE);
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM163P";
                cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM163P_Save");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM163P_SaveUpload")]
        [Authorize]
        public IHttpActionResult SNM163P_SaveUpload([FromBody] dynamic pDataList)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM163P_SAVE_UPLOAD";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM163P";
                cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM163P_SaveUpload");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        public string SNM163P_ClearTemp()
        {
            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM163P_CLEAR_TEMP";
            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return "DONE";
        }

        [HttpPost, Route("SNM163P_UploadExcel")]
        [Authorize]
        public async Task<IHttpActionResult> SNM163P_UploadExcel(HttpRequestMessage request)
        {

            if (!request.Content.IsMimeMultipartContent())
            {
                throw new HttpResponseException(HttpStatusCode.UnsupportedMediaType);
            }

            var reqParts = await request.Content.ReadAsMultipartAsync();
            List<string> ErrorList = new List<string>();
            try
            {
                string fileLocation = string.Empty;
                foreach (var file in reqParts.Contents)
                {
                    var buffer = await file.ReadAsByteArrayAsync();
                    string fileName = Path.GetFileNameWithoutExtension(file.Headers.ContentDisposition.FileName.Trim('\"'));
                    string ext = Path.GetExtension(file.Headers.ContentDisposition.FileName.Trim('\"'));
                    string FullfileName = String.Concat(fileName, "-", DateTime.Now.ToString("yyyyMMddHHmm"), ext);
                    string outputLocation = "~/tempFile/" + "SNM163P";
                    string outputPath = System.Web.HttpContext.Current.Server.MapPath(outputLocation);
                    fileLocation = String.Concat(outputPath, FullfileName);

                    DataService.SaveBytesToFileByLocation(buffer
                                                    , fileLocation
                                                    , fileLocation);
                }
                ErrorList = SNM163P_InsertDataFromExcelFile(fileLocation);
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM163P_UploadExcel");
                throw new HttpResponseException(HttpStatusCode.NotAcceptable);
            }

            return Json(new { Error = ErrorList });
        }

        public List<string> SNM163P_InsertDataFromExcelFile(string fileLocation)
        {

            string fileLocationN = fileLocation;
            DataTable dt = new DataTable();
            SystemApiController sys = new SystemApiController();
            dt = sys.ReadDataExcel(fileLocationN);
            List<string> ErrList = new List<string>();
            int countrow = 1;
            OracleCommand cmd = new OracleCommand();
            DataSet ds = new DataSet();

            try
            {
                cmd.CommandText = "SP_NP_SNM163P_CLEAR_TEMP";
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM163P_InsertDataFromExcelFile");
                ErrList.Add("Error : SP_NP_SNM163P_CLEAR_TEMP please recheck");
                return ErrList;
            }

            foreach (DataRow row in dt.Rows)
            {

                DateTime e;
                DateTime d;
                bool isEffDatetime = DateTime.TryParse(row["EFFECTIVE_DATE"].ToString().Trim(), out e);
                bool isDisDate = DateTime.TryParse(row["DISABLE_DATE"].ToString().Trim(), out d);

                if (isEffDatetime && isDisDate)
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNM163P_UPLOAD";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_RNO", OracleDbType.Varchar2).Value = row["NO"].ToString().Trim();
                    cmd.Parameters.Add("P_EFFECTIVE_DATE", OracleDbType.Varchar2).Value = Convert.ToDateTime(row["EFFECTIVE_DATE"]).ToString("dd-MMM-yyyy").Trim();
                    cmd.Parameters.Add("P_DISABLE_DATE", OracleDbType.Varchar2).Value = Convert.ToDateTime(row["DISABLE_DATE"]).ToString("dd-MMM-yyyy").Trim();
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = row["CUSTOMER_CODE"].ToString().Trim();
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = row["BRANCH_CODE"].ToString().Trim();
                    cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = row["PART_NO"].ToString().Trim();
                    cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;

                    cmd = oOracledb.SqlQueryReturnValue(cmd);

                    string status = cmd.Parameters["P_STATUS"].Value.ToString();
                }
                else
                {
                    ErrList.Add("Row " + countrow + "Customer Code : " + row["CUSTOMER_CODE"].ToString().Trim() + "Part no : " + row["PART_NO"].ToString().Trim() + " error please recheck");
                }

                countrow++;
            }
            return ErrList;
        }

        [HttpGet, Route("SNM163P_GetTempData")]
        [Authorize]
        public IHttpActionResult SNM163P_GetTempData()
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM163P_GET_TEMP_DATA";
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM163P_GetTempData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM163P_Delete")]
        [Authorize]
        public IHttpActionResult SNM163P_Delete([FromBody] dynamic pDataList)
        {
            try
            {
                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM163P_DELETE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pDataList.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pDataList.BRANCH_CODE;
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pDataList.PART_NO;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM163P_Delete");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

       

        [HttpGet, Route("SNM163P_GetStatus")]
        [Authorize]
        public IHttpActionResult SNM163P_GetStatus()
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM163P_GET_STATUS";
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM163P_GetStatus");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM163P_UpdateStatus")]
        [Authorize]
        [RunBatchGuard]
        public IHttpActionResult SNM163P_UpdateStatus([FromBody] dynamic pParams)
        {
            try
            {
                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM163P_UPDATE_STATUS";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CONFIG_VALUE", OracleDbType.Varchar2).Value = pParams.CONFIG_VALUE;
                cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM163P";
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM163P_UpdateStatus");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        #endregion

        #region SNM168

        [HttpGet, Route("SNM168_GetInvoiceExportMasterGroup/{customerGroupCode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM168_GetInvoiceExportMasterGroup(string customerGroupCode, int noofshowrec)
        {
            DataSet ds = new DataSet();

            customerGroupCode = ismoUtil.escapeCharacter(customerGroupCode);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM168_GET_INVOICE_EXPORT_MASTER_GROUP";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_GROUP_CODE", OracleDbType.Varchar2).Value = customerGroupCode.ToUpper() == "NULL" ? (object)DBNull.Value : customerGroupCode;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM168_GetInvoiceExportMasterGroup");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM168_GetInvoiceExportMaster")]
        [Authorize]
        public IHttpActionResult SNM168_GetInvoiceExportMaster([FromBody] dynamic param)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM168_GET_INVOICE_EXPORT_MASTER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_GROUP_CODE", OracleDbType.Varchar2).Value = param.CUSTOMER_GROUP_CODE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM168_GetInvoiceExportMaster");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNM168_GetDataInvoiceExportMaster/{customerGroupCode}")]
        [Authorize]
        public IHttpActionResult SNM168_GetDataInvoiceExportMaster(string customerGroupCode)
        {
            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM168_GET_DATA_INVOICE_EXPORT_MASTER";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_CUSTOMER_GROUP_CODE", OracleDbType.Varchar2).Value = customerGroupCode;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }

        [HttpGet, Route("SNM168_GetInvoiceExportCustomerCode/{customerCode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM168_GetInvoiceExportCustomerCode(string customerCode, int noofshowrec)
        {
            DataSet ds = new DataSet();

            customerCode = ismoUtil.escapeCharacter(customerCode);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM168_GET_INVOICE_EXPORT_CUSTOMER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = customerCode.ToUpper() == "NULL" ? (object)DBNull.Value : customerCode;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM168_GetInvoiceExportCustomerCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM168_GetDataInvoiceExportCustomerGroup")]
        [Authorize]
        public IHttpActionResult SNM168_GetDataInvoiceExportCustomerGroup([FromBody] dynamic param)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM168_GET_INVOICE_EXPORT_CUSTOMER_GROUP";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = param.CUSTOMER_CODE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM168_GetDataInvoiceExportCustomerGroup");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNM168_GetCustomerCodeForAdd/{customerCode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM168_GetCustomerCodeForAdd(string customerCode, int noofshowrec)
        {
            DataSet ds = new DataSet();

            customerCode = ismoUtil.escapeCharacter(customerCode);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM168_GET_CUSTOMER_FOR_ADD";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = customerCode.ToUpper() == "NULL" ? (object)DBNull.Value : customerCode;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM168_GetCustomerCodeForAdd");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM168_InsertInvoiceExportMaster")]
        [Authorize]
        public IHttpActionResult SNM168_InsertInvoiceExportMaster([FromBody] dynamic pParams)
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM168_INSERT_INVOICE_EXPORT_MASTER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_GROUP_CODE)", OracleDbType.Varchar2).Value = pParams.CUSTOMER_GROUP_CODE;
                cmd.Parameters.Add("P_CUSTOMER_GROUP_NAME)", OracleDbType.Varchar2).Value = pParams.CUSTOMER_GROUP_NAME;
                cmd.Parameters.Add("P_CUSTOMER_GROUP_REMARK)", OracleDbType.Varchar2).Value = pParams.CUSTOMER_GROUP_REMARK;
                cmd.Parameters.Add("P_SHIPPER1", OracleDbType.Varchar2).Value = pParams.SHIPPER1;
                cmd.Parameters.Add("P_SHIPPER2", OracleDbType.Varchar2).Value = pParams.SHIPPER2;
                cmd.Parameters.Add("P_SHIPPER3", OracleDbType.Varchar2).Value = pParams.SHIPPER3;
                cmd.Parameters.Add("P_SHIPPER4", OracleDbType.Varchar2).Value = pParams.SHIPPER4;
                cmd.Parameters.Add("P_SHIPPER_TAX_ID", OracleDbType.Varchar2).Value = pParams.SHIPPER_TAX_ID;
                cmd.Parameters.Add("P_CONSIGNED_MESSRS1", OracleDbType.Varchar2).Value = pParams.CONSIGNED_MESSRS1;
                cmd.Parameters.Add("P_CONSIGNED_MESSRS2", OracleDbType.Varchar2).Value = pParams.CONSIGNED_MESSRS2;
                cmd.Parameters.Add("P_CONSIGNED_MESSRS3", OracleDbType.Varchar2).Value = pParams.CONSIGNED_MESSRS3;
                cmd.Parameters.Add("P_CONSIGNED_MESSRS4", OracleDbType.Varchar2).Value = pParams.CONSIGNED_MESSRS4;
                cmd.Parameters.Add("P_NOTIFY_PARTY1", OracleDbType.Varchar2).Value = pParams.NOTIFY_PARTY1;
                cmd.Parameters.Add("P_NOTIFY_PARTY2", OracleDbType.Varchar2).Value = pParams.NOTIFY_PARTY2;
                cmd.Parameters.Add("P_NOTIFY_PARTY3", OracleDbType.Varchar2).Value = pParams.NOTIFY_PARTY3;
                cmd.Parameters.Add("P_NOTIFY_PARTY4", OracleDbType.Varchar2).Value = pParams.NOTIFY_PARTY4;
                cmd.Parameters.Add("P_NOTIFY_PARTY_TEL", OracleDbType.Varchar2).Value = pParams.NOTIFY_PARTY_TEL;
                cmd.Parameters.Add("P_BANK_OF_NMT1", OracleDbType.Varchar2).Value = pParams.BANK_OF_NMT1;
                cmd.Parameters.Add("P_BANK_OF_NMT2", OracleDbType.Varchar2).Value = pParams.BANK_OF_NMT2;
                cmd.Parameters.Add("P_BANK_OF_NMT3", OracleDbType.Varchar2).Value = pParams.BANK_OF_NMT3;
                cmd.Parameters.Add("P_BANK_OF_NMT_TEL", OracleDbType.Varchar2).Value = pParams.BANK_OF_NMT_TEL;
                cmd.Parameters.Add("P_BANK_OF_NMT_FAX", OracleDbType.Varchar2).Value = pParams.BANK_OF_NMT_FAX;
                cmd.Parameters.Add("P_BANK_OF_NMT_SWIFT_CODE", OracleDbType.Varchar2).Value = pParams.BANK_OF_NMT_SWIFT_CODE;
                cmd.Parameters.Add("P_BANK_OF_NMT_ACOUNT_ACCOUNT_NO", OracleDbType.Varchar2).Value = pParams.BANK_OF_NMT_ACOUNT_ACCOUNT_NO;
                cmd.Parameters.Add("P_SHIPPED_PER", OracleDbType.Varchar2).Value = pParams.SHIPPED_PER;
                cmd.Parameters.Add("P_PORT_OF_DISCHARGE", OracleDbType.Varchar2).Value = pParams.PORT_OF_DISCHARGE;
                cmd.Parameters.Add("P_PORT_OF_LOADING", OracleDbType.Varchar2).Value = pParams.PORT_OF_LOADING;
                cmd.Parameters.Add("P_FINAL_DESTINATION_FOR_REFERENCE_ONLY", OracleDbType.Varchar2).Value = pParams.FINAL_DESTINATION_FOR_REFERENCE_ONLY;
                cmd.Parameters.Add("P_DESCRIPTION_OF_GOODS1", OracleDbType.Varchar2).Value = pParams.DESCRIPTION_OF_GOODS1;
                cmd.Parameters.Add("P_DESCRIPTION_OF_GOODS2", OracleDbType.Varchar2).Value = pParams.DESCRIPTION_OF_GOODS2;
                cmd.Parameters.Add("P_DESCRIPTION_OF_GOODS3", OracleDbType.Varchar2).Value = pParams.DESCRIPTION_OF_GOODS3;
                cmd.Parameters.Add("P_DESCRIPTION_OF_GOODS4", OracleDbType.Varchar2).Value = pParams.DESCRIPTION_OF_GOODS4;
                cmd.Parameters.Add("P_F_O_B", OracleDbType.Varchar2).Value = pParams.F_O_B;
                cmd.Parameters.Add("P_UNIT_PRICE", OracleDbType.Varchar2).Value = pParams.UNIT_PRICE;
                cmd.Parameters.Add("P_FREIGHT", OracleDbType.Varchar2).Value = pParams.FREIGHT;
                cmd.Parameters.Add("P_PLACE", OracleDbType.Varchar2).Value = pParams.PLACE;
                cmd.Parameters.Add("P_TERMS_METHOD_OF_PAYMENT", OracleDbType.Varchar2).Value = pParams.TERMS_METHOD_OF_PAYMENT;
                cmd.Parameters.Add("P_SIGNATURE1", OracleDbType.Varchar2).Value = pParams.SIGNATURE1;
                cmd.Parameters.Add("P_SIGNATURE2", OracleDbType.Varchar2).Value = pParams.SIGNATURE2;
                cmd.Parameters.Add("P_SIGNATURE3", OracleDbType.Varchar2).Value = pParams.SIGNATURE3;
                cmd.Parameters.Add("P_FLAG_CONFIRM_TEMPLATE", OracleDbType.Varchar2).Value = pParams.FLAG_CONFIRM_TEMPLATE;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM168";
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM168_InsertInvoiceExportMaster");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpPost, Route("SNM168_UpdateInvoiceExportMaster")]
        [Authorize]
        public IHttpActionResult SNM168_UpdateInvoiceExportMaster([FromBody] dynamic pParams)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM168_UPDATE_INVOICE_EXPORT_MASTER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_GROUP_CODE)", OracleDbType.Varchar2).Value = pParams.CUSTOMER_GROUP_CODE;
                cmd.Parameters.Add("P_CUSTOMER_GROUP_NAME)", OracleDbType.Varchar2).Value = pParams.CUSTOMER_GROUP_NAME;
                cmd.Parameters.Add("P_CUSTOMER_GROUP_REMARK)", OracleDbType.Varchar2).Value = pParams.CUSTOMER_GROUP_REMARK;
                cmd.Parameters.Add("P_SHIPPER1", OracleDbType.Varchar2).Value = pParams.SHIPPER1;
                cmd.Parameters.Add("P_SHIPPER2", OracleDbType.Varchar2).Value = pParams.SHIPPER2;
                cmd.Parameters.Add("P_SHIPPER3", OracleDbType.Varchar2).Value = pParams.SHIPPER3;
                cmd.Parameters.Add("P_SHIPPER4", OracleDbType.Varchar2).Value = pParams.SHIPPER4;
                cmd.Parameters.Add("P_SHIPPER_TAX_ID", OracleDbType.Varchar2).Value = pParams.SHIPPER_TAX_ID;
                cmd.Parameters.Add("P_CONSIGNED_MESSRS1", OracleDbType.Varchar2).Value = pParams.CONSIGNED_MESSRS1;
                cmd.Parameters.Add("P_CONSIGNED_MESSRS2", OracleDbType.Varchar2).Value = pParams.CONSIGNED_MESSRS2;
                cmd.Parameters.Add("P_CONSIGNED_MESSRS3", OracleDbType.Varchar2).Value = pParams.CONSIGNED_MESSRS3;
                cmd.Parameters.Add("P_CONSIGNED_MESSRS4", OracleDbType.Varchar2).Value = pParams.CONSIGNED_MESSRS4;
                cmd.Parameters.Add("P_NOTIFY_PARTY1", OracleDbType.Varchar2).Value = pParams.NOTIFY_PARTY1;
                cmd.Parameters.Add("P_NOTIFY_PARTY2", OracleDbType.Varchar2).Value = pParams.NOTIFY_PARTY2;
                cmd.Parameters.Add("P_NOTIFY_PARTY3", OracleDbType.Varchar2).Value = pParams.NOTIFY_PARTY3;
                cmd.Parameters.Add("P_NOTIFY_PARTY4", OracleDbType.Varchar2).Value = pParams.NOTIFY_PARTY4;
                cmd.Parameters.Add("P_NOTIFY_PARTY_TEL", OracleDbType.Varchar2).Value = pParams.NOTIFY_PARTY_TEL;
                cmd.Parameters.Add("P_BANK_OF_NMT1", OracleDbType.Varchar2).Value = pParams.BANK_OF_NMT1;
                cmd.Parameters.Add("P_BANK_OF_NMT2", OracleDbType.Varchar2).Value = pParams.BANK_OF_NMT2;
                cmd.Parameters.Add("P_BANK_OF_NMT3", OracleDbType.Varchar2).Value = pParams.BANK_OF_NMT3;
                cmd.Parameters.Add("P_BANK_OF_NMT_TEL", OracleDbType.Varchar2).Value = pParams.BANK_OF_NMT_TEL;
                cmd.Parameters.Add("P_BANK_OF_NMT_FAX", OracleDbType.Varchar2).Value = pParams.BANK_OF_NMT_FAX;
                cmd.Parameters.Add("P_BANK_OF_NMT_SWIFT_CODE", OracleDbType.Varchar2).Value = pParams.BANK_OF_NMT_SWIFT_CODE;
                cmd.Parameters.Add("P_BANK_OF_NMT_ACOUNT_ACCOUNT_NO", OracleDbType.Varchar2).Value = pParams.BANK_OF_NMT_ACOUNT_ACCOUNT_NO;
                cmd.Parameters.Add("P_SHIPPED_PER", OracleDbType.Varchar2).Value = pParams.SHIPPED_PER;
                cmd.Parameters.Add("P_PORT_OF_DISCHARGE", OracleDbType.Varchar2).Value = pParams.PORT_OF_DISCHARGE;
                cmd.Parameters.Add("P_PORT_OF_LOADING", OracleDbType.Varchar2).Value = pParams.PORT_OF_LOADING;
                cmd.Parameters.Add("P_FINAL_DESTINATION_FOR_REFERENCE_ONLY", OracleDbType.Varchar2).Value = pParams.FINAL_DESTINATION_FOR_REFERENCE_ONLY;
                cmd.Parameters.Add("P_DESCRIPTION_OF_GOODS1", OracleDbType.Varchar2).Value = pParams.DESCRIPTION_OF_GOODS1;
                cmd.Parameters.Add("P_DESCRIPTION_OF_GOODS2", OracleDbType.Varchar2).Value = pParams.DESCRIPTION_OF_GOODS2;
                cmd.Parameters.Add("P_DESCRIPTION_OF_GOODS3", OracleDbType.Varchar2).Value = pParams.DESCRIPTION_OF_GOODS3;
                cmd.Parameters.Add("P_DESCRIPTION_OF_GOODS4", OracleDbType.Varchar2).Value = pParams.DESCRIPTION_OF_GOODS4;
                cmd.Parameters.Add("P_F_O_B", OracleDbType.Varchar2).Value = pParams.F_O_B;
                cmd.Parameters.Add("P_UNIT_PRICE", OracleDbType.Varchar2).Value = pParams.UNIT_PRICE;
                cmd.Parameters.Add("P_FREIGHT", OracleDbType.Varchar2).Value = pParams.FREIGHT;
                cmd.Parameters.Add("P_PLACE", OracleDbType.Varchar2).Value = pParams.PLACE;
                cmd.Parameters.Add("P_TERMS_METHOD_OF_PAYMENT", OracleDbType.Varchar2).Value = pParams.TERMS_METHOD_OF_PAYMENT;
                cmd.Parameters.Add("P_SIGNATURE1", OracleDbType.Varchar2).Value = pParams.SIGNATURE1;
                cmd.Parameters.Add("P_SIGNATURE2", OracleDbType.Varchar2).Value = pParams.SIGNATURE2;
                cmd.Parameters.Add("P_SIGNATURE3", OracleDbType.Varchar2).Value = pParams.SIGNATURE3;
                cmd.Parameters.Add("P_FLAG_CONFIRM_TEMPLATE", OracleDbType.Varchar2).Value = pParams.FLAG_CONFIRM_TEMPLATE;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM168";
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM168_UpdateInvoiceExportMaster");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM168_DeleteInvoiceExportMaster")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM168_DeleteInvoiceExportMaster([FromBody] dynamic pParams)
        {
            DataSet ds = new DataSet();

            try
            {

                OracleCommand cmd = new OracleCommand();

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM168_DELETE_INVOICE_EXPORT_MASTER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_GROUP_CODE", OracleDbType.Varchar2).Value = pParams.CUSTOMER_GROUP_CODE;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM168";
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string R_STATUS = cmd.Parameters["R_STATUS"].Value.ToString();
                string R_MESSAGE = cmd.Parameters["R_MESSAGE"].Value.ToString();

                if (R_STATUS == "FAIL")
                {
                    oOracledb.WriteLog("ErrorHandle", R_MESSAGE, "SNM168_DeleteInvoiceExportMaster");
                }

                return Json(new { STATUS_MSG = R_STATUS, });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM168_DeleteInvoiceExportMaster");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM168_InsertInvoiceExportCustomerGroup")]
        [Authorize]
        public IHttpActionResult SNM168_InsertInvoiceExportCustomerGroup([FromBody] dynamic pParams)
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM168_INSERT_INVOICE_EXPORT_CUSTOMER_GROUP";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_GROUP_CODE", OracleDbType.Varchar2).Value = pParams.CUSTOMER_GROUP_CODE;
                cmd.Parameters.Add("P_CUSTOMER_GROUP_NAME", OracleDbType.Varchar2).Value = pParams.CUSTOMER_GROUP_NAME;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParams.CUSTOMER_CODE;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM168";
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM168_InsertInvoiceExportCustomerGroup");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM168_UpdateInvoiceExportCustomerGroup")]
        [Authorize]
        public IHttpActionResult SNM168_UpdateInvoiceExportCustomerGroup([FromBody] dynamic pParams)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM168_UPDATE_INVOICE_EXPORT_CUSTOMER_GROUP";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_GROUP_CODE", OracleDbType.Varchar2).Value = pParams.CUSTOMER_GROUP_CODE;
                cmd.Parameters.Add("P_CUSTOMER_GROUP_CODE_OLD", OracleDbType.Varchar2).Value = pParams.CUSTOMER_GROUP_CODE_OLD;
                cmd.Parameters.Add("P_CUSTOMER_GROUP_NAME", OracleDbType.Varchar2).Value = pParams.CUSTOMER_GROUP_NAME;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParams.CUSTOMER_CODE;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM168";
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM168_UpdateInvoiceExportCustomerGroup");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM168_DeleteInvoiceExportCustomerGroup")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM168_DeleteInvoiceExportCustomerGroup([FromBody] dynamic pParams)
        {
            DataSet ds = new DataSet();

            try
            {

                OracleCommand cmd = new OracleCommand();

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM168_DELETE_INVOICE_EXPORT_CUSTOMER_GROUP";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParams.CUSTOMER_CODE;
                cmd.Parameters.Add("P_CUSTOMER_GROUP_CODE", OracleDbType.Varchar2).Value = pParams.CUSTOMER_GROUP_CODE;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM168";
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string R_STATUS = cmd.Parameters["R_STATUS"].Value.ToString();
                string R_MESSAGE = cmd.Parameters["R_MESSAGE"].Value.ToString();

                if (R_STATUS == "FAIL")
                {
                    oOracledb.WriteLog("ErrorHandle", R_MESSAGE, "SNM168_DeleteInvoiceExportCustomerGroup");
                }

                return Json(new { STATUS_MSG = R_STATUS, });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM168_DeleteInvoiceExportCustomerGroup");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM168_CheckGruopCode")]
        [Authorize]
        public IHttpActionResult SNM168_CheckGruopCode([FromBody] dynamic pParams)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM168_CHECK_GROUP_CODE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_GROUP_CODE", OracleDbType.Varchar2).Value = pParams.CUSTOMER_GROUP_CODE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM168_CheckGruopCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM168_CheckCustomerGroupMapping")]
        [Authorize]
        public IHttpActionResult SNM168_CheckAddCustomerGroupMapping([FromBody] dynamic pParams)
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM168_CHECK_CUSTOMER_GROUP_MAPPING";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParams.CUSTOMER_CODE;
                cmd.Parameters.Add("P_CUSTOMER_GROUP_CODE", OracleDbType.Varchar2).Value = pParams.CUSTOMER_GROUP_CODE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM168_CheckCustomerGroupMapping");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        #endregion

        #region SNSE225


        [HttpPost, Route("SNSE225_Update_Invoice_Export")]
        [Authorize]
        public IHttpActionResult SNSE225_Update_Invoice_Export([FromBody] dynamic pParams)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE225_UPDATE_INVOICE_EXPORT";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pParams.INVOICE_NO;
                cmd.Parameters.Add("P_CUSTOMER_GROUP_CODE", OracleDbType.Varchar2).Value = pParams.CUSTOMER_GROUP_CODE;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParams.CUSTOMER_CODE;
                cmd.Parameters.Add("P_PLACE_TEXT", OracleDbType.Varchar2).Value = pParams.PLACE_TEXT;
                cmd.Parameters.Add("P_NISSAN_REFERENCE_NO", OracleDbType.Varchar2).Value = pParams.NISSAN_REFERENCE_NO;
                cmd.Parameters.Add("P_EXPORT_DECLARATION_NO", OracleDbType.Varchar2).Value = pParams.EXPORT_DECLARATION_NO;
                cmd.Parameters.Add("P_INDENT_NO", OracleDbType.Varchar2).Value = pParams.INDENT_NO;
                cmd.Parameters.Add("P_BANK_CERTIFICATE_NO", OracleDbType.Varchar2).Value = pParams.BANK_CERTIFICATE_NO;
                cmd.Parameters.Add("P_EXPORT_LICENSE_NO", OracleDbType.Varchar2).Value = pParams.EXPORT_LICENSE_NO;
                cmd.Parameters.Add("P_PLACE_OF_DELIVERY", OracleDbType.Varchar2).Value = pParams.PLACE_OF_DELIVERY;
                cmd.Parameters.Add("P_TERMS_METHOD_PAYMENT", OracleDbType.Varchar2).Value = pParams.TERMS_METHOD_PAYMENT;
                cmd.Parameters.Add("P_SHIPPED_PER", OracleDbType.Varchar2).Value = pParams.SHIPPED_PER;
                cmd.Parameters.Add("P_PORT_OF_DISCHARGE", OracleDbType.Varchar2).Value = pParams.PORT_OF_DISCHARGE;
                cmd.Parameters.Add("P_PORT_OF_LOADING", OracleDbType.Varchar2).Value = pParams.PORT_OF_LOADING;
                cmd.Parameters.Add("P_FINAL_DESTINATION_FOR_REFERENCE_ONLY", OracleDbType.Varchar2).Value = pParams.FINAL_DESTINATION_FOR_REFERENCE_ONLY;
                cmd.Parameters.Add("P_TH_ONLY_FLAG", OracleDbType.Varchar2).Value = pParams.TH_ONLY_FLAG;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string R_STATUS = cmd.Parameters["R_STATUS"].Value.ToString();
                string R_MESSAGE = cmd.Parameters["R_MESSAGE"].Value.ToString();

                if (R_STATUS == "FAIL")
                {
                    oOracledb.WriteLog("ErrorHandle", R_MESSAGE, "SNSE225_Update_Inviice_Export");
                }

                return Json(new { STATUS_MSG = R_STATUS, });

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE225_Update_Invoice_Export");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }




        [HttpPost, Route("SNSE225_GetData_Master")]
        [Authorize]
        public IHttpActionResult SNSE225_GetData_Master([FromBody] dynamic pParams)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE225_GET_DATA_MASTER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_GROUP_CODE", OracleDbType.Varchar2).Value = pParams.CUSTOMER_GROUP_CODE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE225_GetData_Master");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE225_GetData_Master_Customer_Group")]
        [Authorize]
        public IHttpActionResult SNSE225_GetData_Master_Customer_Group([FromBody] dynamic param)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE225_GET_DATA_MASTER_CUSTOMER_GROUP";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = param.CUSTOMER_CODE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE225_GetData_Master_Customer_Group");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE225_GetData_Master_List")]
        [Authorize]
        public IHttpActionResult SNSE225_GetData_Master_List([FromBody] dynamic pParams)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE225_GET_DATA_MASTER_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pParams.INVOICE_NO;
                cmd.Parameters.Add("P_INVOICE_TYPE", OracleDbType.Varchar2).Value = pParams.INVOICE_TYPE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE225_GetData_Master_List");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }



        [HttpGet, Route("SNSE225_SearchInvoiceMaster/{CustomerGroup}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE225_SearchInvoiceMaster(string CustomerGroup, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            CustomerGroup = ismoUtil.escapeCharacter(CustomerGroup);
            var pData = pParam["DATA"];

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE225_GET_INVOICE_EXPORT_MASTER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_GROUP_CODE", OracleDbType.Varchar2).Value = CustomerGroup.ToUpper() == "NULL" ? (object)DBNull.Value : CustomerGroup;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE225_SearchInvoiceMaster");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE225_GetSearchInv/{InvNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE225_GetSearchInv(string InvNo, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            InvNo = ismoUtil.escapeCharacter(InvNo);
            var pData = pParam["DATA"];

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE225_GET_INVOICE_NO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = InvNo.ToUpper() == "NULL" ? (object)DBNull.Value : InvNo;
                cmd.Parameters.Add("P_INVOICE_TYPE", OracleDbType.Varchar2).Value = pData.INVOICE_TYPE;
                cmd.Parameters.Add("P_SHOW_REC", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE225_GetSearchInv");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpPost, Route("SNSE225_PROCESS")]
        [Authorize]
        public IHttpActionResult SNSE225_PROCESS([FromBody] dynamic pParams)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE225_PROCESS";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pParams.INVOICE_NO;
                cmd.Parameters.Add("P_PGM_NAME", OracleDbType.Varchar2).Value = "SNSE225";
                cmd.Parameters.Add("P_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 500).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string RETURN_MESSAGE = cmd.Parameters["R_MESSAGE"].Value.ToString();
                return Json(new { STATUS_MESSAGE = RETURN_MESSAGE });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE225_PROCESS");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.ExpectationFailed, ex.Message));
            }
        }


        [HttpPost, Route("SNSE225_PROCESS_PRINT")]
        [Authorize]
        public IHttpActionResult SNSE225_PROCESS_PRINT([FromBody] dynamic pParams)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE225_PROCESS_PRINT";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pParams.INVOICE_NO;
                cmd.Parameters.Add("P_PGM_NAME", OracleDbType.Varchar2).Value = "SNSE225";
                cmd.Parameters.Add("P_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 500).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string RETURN_MESSAGE = cmd.Parameters["R_MESSAGE"].Value.ToString();
                return Json(new { STATUS_MESSAGE = RETURN_MESSAGE });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE225_PROCESS_PRINT");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.ExpectationFailed, ex.Message));
            }
        }


        [HttpPost, Route("SNSE225_GetData")]
        [Authorize]
        public IHttpActionResult SNSE225_GetData([FromBody] dynamic pParams)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE225_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_INVOICE_NO", OracleDbType.Varchar2).Value = pParams.INVOICE_NO;
                cmd.Parameters.Add("P_INVOICE_TYPE", OracleDbType.Varchar2).Value = pParams.INVOICE_TYPE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE225_GetData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        #endregion SNSE225

        #region GetCSIDiscount

        [HttpPost, Route("GetCSIDiscount")]
        [Authorize]
        public IHttpActionResult GetCSIDiscount([FromBody] dynamic pData)
        {

            int iCount = 0;
            OracleCommand cmd = new OracleCommand();
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();

            var partList = new List<dynamic>(pData.PART_LIST);

            if (partList.Count > 0)
            {
                //Run insert detail
                foreach (var detail in partList)
                {
                    iCount++;

                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_GET_CSI_DISCOUNT";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pData.BRANCH_CODE;
                    cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pData.ORDER_NO;
                    cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = pData.ORDER_CLASS;
                    cmd.Parameters.Add("P_ORDER_DATE", OracleDbType.Date).Value = pData.ORDER_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(pData.ORDER_DATE);
                    cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Decimal).Value = iCount;
                    cmd.Parameters.Add("P_ORDER_QTY", OracleDbType.Decimal).Value = detail.ORDER_QTY;
                    cmd.Parameters.Add("P_ORDER_PART_NO", OracleDbType.Varchar2).Value = detail.ORDER_PART_NO;
                    cmd.Parameters.Add("P_DISCOUNT_GROUP", OracleDbType.Varchar2).Value = detail.DISCOUNT_GROUP;
                    cmd.Parameters.Add("P_DISCOUNT_CODE", OracleDbType.Varchar2).Value = detail.DISCOUNT_CODE;
                    cmd.Parameters.Add("P_CLEAR_DATA", OracleDbType.Varchar2).Value = iCount == 1 ? "Y" : "N";
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    oOracledb.SqlQueryReturnValue(cmd);
                }
            }

            try
            {
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_GET_CSI_DISCOUNT_TEMP";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pData.CUSTOMER_CODE;
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = pData.BRANCH_CODE;
                cmd.Parameters.Add("P_ORDER_NO", OracleDbType.Varchar2).Value = pData.ORDER_NO;
                cmd.Parameters.Add("P_ORDER_DATE", OracleDbType.Date).Value = pData.ORDER_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(pData.ORDER_DATE);
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GetCSIDiscount");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        #endregion GetCSIDiscount
    }
}