﻿using Oracle.ManagedDataAccess.Client;
using SYS_Core.aControl;
using System;
using System.Collections.Generic;
using System.Configuration;
using System.Data;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Web.Http;
using System.Web.Http.Cors;
using System.Collections;
using System.Web;
using System.Net.Sockets;
using System.Threading.Tasks;
using System.IO;
using System.Text;
using SYS_Core.aBiz;
using System.Globalization;
using SYSCore.ReportingService;
using System.Text.RegularExpressions;
using SYSCore.aControl;
using System.Net.Http.Headers;
using RandomSolutions;
using SYSCore.Attributes;

namespace SYS_Core.Controllers
{

    [EnableCors("*", "*", "*")]
    [RoutePrefix("api/TI")]
    public class TechnicalController : ApiController
    {

        private DBOracle oOracledb = new DBOracle();

        private DataModel DataModel = new DataModel();

        #region SNSE142
        [HttpPost, Route("SNSE142_ExportExcel")]
        public HttpResponseMessage SNSE142_ExportExcel(List<DataModel.SNSE142> Data)
        {
            HttpResponseMessage response = new HttpResponseMessage(HttpStatusCode.OK);
            string gLocation = "~/tempFile/" + "SNSE142" + "/";
            string gWriteLocation = System.Web.HttpContext.Current.Server.MapPath(gLocation);
            FileInfo filePath = new FileInfo(gWriteLocation);
            filePath.Directory.Create();
            string gFileLocation = Path.Combine(gWriteLocation, "SNSE142.xlsx");
            string gReturnLocation = Path.Combine(gLocation, "SNSE142.xlsx");
            var excel2 = Data.ToExcel(x => x
            .AddColumn("Seq.", z => z.SEQ)
            .AddColumn("Part No.", z => z.PART_NUMBER)
            .AddColumn("Piece Price", z => z.PIECE_PRICE)
            .AddColumn("Base Price", z => z.BASE_PRICE)
            .AddColumn("Currency", z => z.CURRENCY)
            .AddColumn("Adopted Date From", z => z.ADOPTED_DATE_FROM_PRINT)
            .AddColumn("Adopted Date To", z => z.ADOPTED_DATE_TO_PRINT)
            .AddColumn("Global Supplier Code", z => z.SUPPLIER_CODE_GLOBAL)
            .AddColumn("Local Supplier Code", z => z.SUPPLIER_CODE_LOCAL)
            .AddColumn("Part Name", z => z.PART_NAME)
            .AddColumn("Company Code", z => z.COMPANY_CODE)
            .AddColumn("I/F Reason Code", z => z.IF_REASON_CODE)
            .AddColumn("New Price I/F Reson Code", z => z.NEW_PRICE_IF_REASON_CODE)
            .AddColumn("Category Code", z => z.CATEGORY_CODE)
            .AddColumn("PO Np.", z => z.PO_NO)
            .AddColumn("Global Company Code RC", z => z.GLOBAL_COMPANY_CODE_RC)
            .AddColumn("Global Company Code", z => z.GLOBAL_COMPANY_CODE)
            .AddColumn("Commodity Code", z => z.COMMODITY_CODE)
            .AddColumn("Received Status", z => z.RECEIVED_STATUS)
            );
            byte[] excel = excel2;
            File.WriteAllBytes(gFileLocation, excel);
            string path = HttpContext.Current.Server.MapPath("~/" + gReturnLocation.Substring(1));
            if (!File.Exists(path))
            {
                return Request.CreateErrorResponse(HttpStatusCode.NotFound, "File not found.");
            }
            try
            {
                string mimeType = MimeMapping.GetMimeMapping("SNSE142.xlsx");
                StreamContent x = new StreamContent(new FileStream(path, FileMode.Open, FileAccess.Read));
                response.Content = x;
                response.Content.Headers.ContentType = new MediaTypeHeaderValue(mimeType);
                response.Content.Headers.ContentDisposition = new ContentDispositionHeaderValue("attachment");
                response.Content.Headers.ContentDisposition.FileName = "SNSE142.xlsx";
            }
            catch (Exception ex)
            {
                return Request.CreateResponse(HttpStatusCode.BadRequest, ex);
            }
            return response;
        }
        [HttpGet, Route("SNSE142_GET_VENDOR/{vendor}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE142_GET_VENDOR(string vendor, int noofshowrec)
        {
            DataSet ds = new DataSet();
            vendor = ismoUtil.escapeCharacter(vendor);
            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSE142_GET_VENDOR";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = vendor.ToUpper() == "NULL" ? (object)DBNull.Value : vendor;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }
        [HttpPost, Route("SNSE142_GET_PART/{part}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE142_GET_PART(string part, int noofshowrec, [FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            part = ismoUtil.escapeCharacter(part);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSE142_GET_PART";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = param.VENDOR_CODE;
            cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = part.ToUpper() == "NULL" ? (object)DBNull.Value : part;
            // cmd.Parameters.Add("p_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }


        [HttpPost, Route("SNSE142_SEARCH")]
        [Authorize]
        public IHttpActionResult SNSE142_SEARCH([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            
            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSE142_SEARCH";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = param.VENDOR_CODE;
            cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = param.PART_NO;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNSE142_Save")]
        [Authorize]
        public IHttpActionResult SNSE142_Save([FromBody] dynamic data)
        {

            DataSet ds = new DataSet();
            DataTable dt = new DataTable();

            OracleCommand cmd = new OracleCommand();
            int iCount = 0;
            try
            {

                foreach (dynamic d in data.DETAIL)
                {
                    iCount++;
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE142_SAVE_TEMP";
                    cmd.Parameters.Add("P_PART_NUMBER", OracleDbType.Varchar2).Value = d["PART_NUMBER"];
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_FILE_NAME", OracleDbType.Varchar2).Value = d["FILE_NAME"];
                    cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Varchar2).Value = d["ITEM_NO"];
                    cmd.Parameters.Add("P_ADOPTED_DATE_FROM", OracleDbType.Varchar2).Value = d["ADOPTED_DATE_FROM"];
                    cmd.Parameters.Add("P_ADOPTED_DATE_TO", OracleDbType.Varchar2).Value = d["ADOPTED_DATE_TO"];
                    cmd.Parameters.Add("P_SUPPLIER_CODE_GLOBAL", OracleDbType.Varchar2).Value = d["SUPPLIER_CODE_GLOBAL"];
                    cmd.Parameters.Add("P_SUPPLIER_CODE_LOCAL", OracleDbType.Varchar2).Value = d["SUPPLIER_CODE_LOCAL"];
                    cmd.Parameters.Add("P_PIECE_PRICE", OracleDbType.Varchar2).Value = d["PIECE_PRICE"];
                    cmd.Parameters.Add("P_COMMODITY_CODE", OracleDbType.Varchar2).Value = d["COMMODITY_CODE"];
                    cmd.Parameters.Add("P_BUYER_CODE_GLOBAL", OracleDbType.Varchar2).Value = d["BUYER_CODE_GLOBAL"];
                    cmd.Parameters.Add("P_CREATE_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_CLEAR_DATA", OracleDbType.Varchar2).Value = iCount == 1 ? "Y" : "N";
                    cmd = oOracledb.SqlQueryReturnValue(cmd);
                }
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE142_UPDATE_PRICE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_IP", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 200).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);
                string r_status = cmd.Parameters["R_STATUS"].Value.ToString();
                string r_msg = cmd.Parameters["R_MESSAGE"].Value.ToString();

                if (r_status != "DONE")
                {
                    oOracledb.WriteLog("ErrorHandle", r_msg, "SNSE142_Save");
                }

                return Json(new { STATUS = r_status, MESSAGE = r_msg });
            }
            catch (Exception ex)
            {
                //--- Roll back ---
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE142_Save");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

            //return Json(data);
        }

        [HttpGet, Route("SNSE142_GetProcessLog")]
        [Authorize]
        public IHttpActionResult SNSE142_GetProcessLog()
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE142_GET_PROCESS_LOG";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                oOracledb.SqlQueryReturnValue(cmd);

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE142_GetProcessLog");
                throw new Exception(ex.Message);
            }

        }

        #endregion
        #region SNM113P


        [HttpPost, Route("SNM113P_SAVE")]
        [Authorize]
        public IHttpActionResult SNM113P_SAVE([FromBody] dynamic data)
        {

            DataSet ds = new DataSet();
            DataTable dt = new DataTable();

            OracleCommand cmd = new OracleCommand();
            int iCount = 0;
            try
            {
                if (data.CustomerList.Count > 0)
                {
                    foreach (dynamic d in data.CustomerList)
                    {
                        iCount++;
                        cmd = new OracleCommand();
                        cmd.CommandText = "SP_NP_SNM113P_SAVE_TEMP";
                        cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                        cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = d["CUSTOMER_CODE"];
                        cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = d["BRANCH_CODE"];
                        cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = d["PART_NO"];
                        cmd.Parameters.Add("P_FLAG", OracleDbType.Varchar2).Value = "INS_CUS";
                        cmd.Parameters.Add("P_CLEAR_DATA", OracleDbType.Varchar2).Value = iCount == 1 ? "Y" : "N";
                        cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                        cmd = oOracledb.SqlQueryReturnValue(cmd);
                    }

                }

                if (data.PartList.Count > 0)
                {
                    foreach (dynamic d in data.PartList)
                    {
                        iCount++;
                        cmd = new OracleCommand();
                        cmd.CommandText = "SP_NP_SNM113P_SAVE_TEMP";
                        cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                        cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = d["CUSTOMER_CODE"];
                        cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = d["BRANCH_CODE"];
                        cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = d["PART_NO"];
                        cmd.Parameters.Add("P_FLAG", OracleDbType.Varchar2).Value = "INS_PART";
                        cmd.Parameters.Add("P_CLEAR_DATA", OracleDbType.Varchar2).Value = iCount == 1 ? "Y" : "N";
                        cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                        cmd = oOracledb.SqlQueryReturnValue(cmd);
                    }

                }


                if (data.DeleteCustomer.Count > 0)
                {
                    foreach (dynamic d in data.DeleteCustomer)
                    {
                        iCount++;
                        cmd = new OracleCommand();
                        cmd.CommandText = "SP_NP_SNM113P_SAVE_TEMP";
                        cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                        cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = d["CUSTOMER_CODE"];
                        cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = d["BRANCH_CODE"];
                        cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = d["PART_NO"];
                        cmd.Parameters.Add("P_FLAG", OracleDbType.Varchar2).Value = "DEL_CUS";
                        cmd.Parameters.Add("P_CLEAR_DATA", OracleDbType.Varchar2).Value = iCount == 1 ? "Y" : "N";
                        cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                        cmd = oOracledb.SqlQueryReturnValue(cmd);
                    }

                }


                if (data.DeletePart.Count > 0)
                {
                    foreach (dynamic d in data.DeletePart)
                    {
                        iCount++;
                        cmd = new OracleCommand();
                        cmd.CommandText = "SP_NP_SNM113P_SAVE_TEMP";
                        cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                        cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = d["CUSTOMER_CODE"];
                        cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = d["BRANCH_CODE"];
                        cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = d["PART_NO"];
                        cmd.Parameters.Add("P_FLAG", OracleDbType.Varchar2).Value = "DEL_PART";
                        cmd.Parameters.Add("P_CLEAR_DATA", OracleDbType.Varchar2).Value = iCount == 1 ? "Y" : "N";
                        cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                        cmd = oOracledb.SqlQueryReturnValue(cmd);
                    }

                }


                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNM113P_SAVE";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CONDITION_KEY", OracleDbType.Varchar2).Value = data.CONDITION_KEY;
                    cmd.Parameters.Add("P_DESCRIPTION", OracleDbType.Varchar2).Value = data.DESCRIPTION;
                    cmd.Parameters.Add("P_FLAG", OracleDbType.Varchar2).Value = data.FLAG;
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                    cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;

                    cmd = oOracledb.SqlQueryReturnValue(cmd);
                    string r_status = cmd.Parameters["R_STATUS"].Value.ToString();
                    string r_msg = cmd.Parameters["R_MESSAGE"].Value.ToString();

                    if (r_status != "DONE")
                    {
                        oOracledb.WriteLog("ErrorHandle", r_msg, "SNM113P_SAVE");
                    }

                    return Json(new { STATUS = r_status, MESSAGE = r_msg });
            }
            catch (Exception ex)
            {
                //--- Roll back ---
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM113P_SAVE");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

            //return Json(data);
        }


        [HttpPost, Route("SNM113P_DELETE")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM113P_DELETE([FromBody] dynamic data)
        {

            DataSet ds = new DataSet();
            DataTable dt = new DataTable();

            OracleCommand cmd = new OracleCommand();
           
            try
            {
              

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM113P_DELETE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CONDITION_KEY", OracleDbType.Varchar2).Value = data.CONDITION_KEY;
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);
                string r_status = cmd.Parameters["R_STATUS"].Value.ToString();
                string r_msg = cmd.Parameters["R_MESSAGE"].Value.ToString();

                if (r_status != "DONE")
                {
                    oOracledb.WriteLog("ErrorHandle", r_msg, "SP_NP_SNM113P_DELETE");
                }

                return Json(new { STATUS = r_status, MESSAGE = r_msg });
            }
            catch (Exception ex)
            {
                //--- Roll back ---
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM113P_DELETE");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

            //return Json(data);
        }

        [HttpPost, Route("SNM113P_CHECK_CONDITION_KEY")]
        [Authorize]
        public IHttpActionResult SNM113P_CHECK_CONDITION_KEY([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM113P_CHECK_CONDITION";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_CONDITION_KEY", OracleDbType.Varchar2).Value = param.CONDITION_KEY;
            cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2,2).Direction = ParameterDirection.Output;

            cmd = oOracledb.SqlQueryReturnValue(cmd);

            string status = cmd.Parameters["R_STATUS"].Value.ToString();

            return Json(new { STATUS = status});
        }

        [HttpPost, Route("SNM113P_SEARCH")]
        [Authorize]
        public IHttpActionResult SNM113P_SEARCH([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM113P_SEARCH";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_CONDITION_KEY", OracleDbType.Varchar2).Value = param.CONDITION_KEY;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }


        [HttpPost, Route("SNM113P_GETDETAIL")]
        [Authorize]
        public IHttpActionResult SNM113P_GETDETAIL([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM113P_GETDETAIL";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_CONDITION_KEY", OracleDbType.Varchar2).Value = param.CONDITION_KEY;
            cmd.Parameters.Add("C_RETURN_1", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
            cmd.Parameters.Add("C_RETURN_2", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
            cmd.Parameters.Add("C_RETURN_3", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        [HttpGet, Route("SNM113P_CONDITION_KEY/{condition}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM113P_CONDITION_KEY(string condition, int noofshowrec)
        {
            DataSet ds = new DataSet();

            condition = ismoUtil.escapeCharacter(condition);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM113P_GET_CONDITION";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_CONDITION", OracleDbType.Varchar2).Value = condition.ToUpper() == "NULL" ? (object)DBNull.Value : condition;
           // cmd.Parameters.Add("p_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        [HttpGet, Route("SNM113P_CUSTOMER_CODE/{customer}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM113P_CUSTOMER_CODE(string customer, int noofshowrec)
        {
            DataSet ds = new DataSet();

            customer = ismoUtil.escapeCharacter(customer);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM113P_GET_CUSTOMER";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = customer.ToUpper() == "NULL" ? (object)DBNull.Value : customer;
            // cmd.Parameters.Add("p_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNM113P_BRANCH_CODE/{branch}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM113P_BRANCH_CODE(string branch, int noofshowrec , [FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            branch = ismoUtil.escapeCharacter(branch);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM113P_GET_BRANCH";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = param.CUSTOMER_CODE;
            cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = branch.ToUpper() == "NULL" ? (object)DBNull.Value : branch;
            // cmd.Parameters.Add("p_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        [HttpGet, Route("SNM113P_GETPART/{part}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM113P_GETPART(string part, int noofshowrec)
        {
            DataSet ds = new DataSet();

            part = ismoUtil.escapeCharacter(part);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM113P_GET_PARTS";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = part.ToUpper() == "NULL" ? (object)DBNull.Value : part;
            // cmd.Parameters.Add("p_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        #endregion
        #region SNM135P
        [HttpPost, Route("SNM135P_Upload")]
        [Authorize]
        public async Task<IHttpActionResult> SNM135P_Upload(HttpRequestMessage request)
        {

            if (!request.Content.IsMimeMultipartContent())
            {
                throw new HttpResponseException(HttpStatusCode.UnsupportedMediaType);
            }

            var reqParts = await request.Content.ReadAsMultipartAsync();
            DataTable dt = new DataTable();
            DataSet ds = new DataSet();
            try
            {

                string fileLocation = string.Empty;

                //var name = reqParts.Contents[1].ReadAsByteArrayAsync();
                //var enname = Encoding.UTF8.GetString(name.Result, 0, name.Result.Length);

                foreach (var file in reqParts.Contents)
                {
                    var buffer = await file.ReadAsByteArrayAsync();
                    //string fileName = Path.GetFileNameWithoutExtension(file.Headers.ContentDisposition.FileName.Trim('\"'));
                    //string ext = Path.GetExtension(file.Headers.ContentDisposition.FileName.Trim('\"'));
                    string FullfileName = String.Concat("SNM135P", "-", DateTime.Now.ToString("yyyyMMddHHmm"), ".CSV");
                    string outputLocation = "~/tempFile/" + "SNM135P" + "/";
                    string outputPath = System.Web.HttpContext.Current.Server.MapPath(outputLocation);
                    fileLocation = String.Concat(outputPath, FullfileName);

                    DataService.SaveBytesToFileByLocation(buffer
                                                    , fileLocation
                                                    , fileLocation);
                }


                //ErrorList = SNM032P_InsertDataFromExcelFile(fileLocation);
                string Header = "PART_NO,NEW_BASE_PRICE,NEW_STANDARD_COST";
                dt = DataService.ConvertCSVtoDataTable(fileLocation, Header);
                dt.Columns.Add("SEQ");
                dt.Columns.Add("PART_NAME_EN");
                dt.Columns.Add("BASE_PRICE");
                dt.Columns.Add("STANDARD_COST");
                dt.Columns.Add("MSG");
                dt.Columns.Add("ERR_FLAG_PART");
                dt.Columns.Add("ERR_FLAG");


                //dt = SNM135P_Validate(dt);
                ds.Tables.Add(dt);

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM135P_Upload");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

            return Json(DTFM.convertToList(ds));

        }

        [HttpPost, Route("SNM135P_UploadExcel")]
        [Authorize]
        public async Task<IHttpActionResult> SNM135P_UploadExcel(HttpRequestMessage request)
        {
            if (!request.Content.IsMimeMultipartContent())
            {
                oOracledb.WriteLog("ErrorHandle", "UnsupportedMediaType", "SNM135P_UploadExcel");
                throw new HttpResponseException(HttpStatusCode.UnsupportedMediaType);
            }
            var reqParts = await request.Content.ReadAsMultipartAsync();
            try
            {
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();
                string USER_ID = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                var file = reqParts.Contents[0];
                var buffer = await file.ReadAsByteArrayAsync();
                string gFileName = file.Headers.ContentDisposition.FileName.Trim('\"');
                string name = Path.GetFileNameWithoutExtension(gFileName);
                string ext = Path.GetExtension(gFileName);
                string gFileNameExtend = String.Concat(name, '_', USER_ID, ext);
                string gRootPath = ConfigurationManager.AppSettings["RootSharedPath"].ToString();
                string gUploadLication = System.IO.Path.Combine(gRootPath, "tempFile/SNM135P");
                string gFileLocation = System.IO.Path.Combine(gUploadLication, gFileName);
                DataService.SaveBytesToFileByLocation(buffer
                                                , gUploadLication
                                                , gFileLocation);
                SystemApiController sys = new SystemApiController();
                dt = SNM135P_Validate(gFileLocation);
                return Json(DTFM.convertToList(dt));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM135P_UploadExcel");
                throw new HttpResponseException(HttpStatusCode.NotAcceptable);
            }
            throw new Exception("Something wrong");
        }
        public DataTable SNM135P_Validate(string filelocation)
        {

            DataTable dt = new DataTable();
            SystemApiController sys = new SystemApiController();
            dt = sys.ReadDataExcel(filelocation);
            dt.Columns.Add("SEQ");
            dt.Columns.Add("PART_NAME_EN");
            // dt.Columns.Add("BASE_PRICE");
            //dt.Columns.Add("STANDARD_COST");
            dt.Columns.Add("MSG");
            dt.Columns.Add("ERR_FLAG_PART");
            dt.Columns.Add("ERR_FLAG");
            int iCount = 0;
            Hashtable hTable = new Hashtable();
            OracleCommand cmd = new OracleCommand();
            DataColumnCollection columns = dt.Columns;
            foreach (DataRow d in dt.Rows)
            {
                if (!string.IsNullOrEmpty(d["PART_NO"].ToString()))
                {
                    iCount++;
                    try
                    {

                        cmd = new OracleCommand();
                        cmd.CommandText = "SP_NP_SNM135P_VALIDATE";
                        cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                        cmd.Parameters.Add("P_PART_NUMBER", OracleDbType.Varchar2).Value = d["PART_NO"];
                        cmd.Parameters.Add("P_BASE_PRICE", OracleDbType.Varchar2).Value = d["NEW_BASE_PRICE"];
                        cmd.Parameters.Add("P_RETAIL_PRICE", OracleDbType.Varchar2).Value = d["RETAIL_PRICE"];
                        cmd.Parameters.Add("P_STANDARD_COST", OracleDbType.Varchar2).Value = d["NEW_STANDARD_COST"];
                        cmd.Parameters.Add("P_PRICING_CODE", OracleDbType.Varchar2).Value = d["NEW_PRICING_CODE"];
                        cmd.Parameters.Add("P_PO_UNIT_PRICE", OracleDbType.Varchar2).Value = columns.Contains("PO_UNIT_PRICE") ? d["PO_UNIT_PRICE"] : null ;
                        cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = columns.Contains("VENDOR_CODE") ? d["VENDOR_CODE"] : null;
                        cmd.Parameters.Add("P_PO_NO", OracleDbType.Varchar2).Value = columns.Contains("PO_NO") ? d["PO_NO"] : null;
                        //cmd.Parameters.Add("R_CURRENT_BASE_PRICE", OracleDbType.Double).Direction = ParameterDirection.Output;
                        //cmd.Parameters.Add("R_CURRENT_STANDARD_COST", OracleDbType.Double).Direction = ParameterDirection.Output;

                        cmd.Parameters.Add("R_ERROR_MESSAGE", OracleDbType.Varchar2, 500).Direction = ParameterDirection.Output;
                        cmd.Parameters.Add("R_ERROR_FLAG_PART_NO", OracleDbType.Varchar2, 1).Direction = ParameterDirection.Output;
                        cmd.Parameters.Add("R_PART_NAME", OracleDbType.Varchar2, 500).Direction = ParameterDirection.Output;
                        cmd.Parameters.Add("R_ERROR_FLAG", OracleDbType.Varchar2, 1).Direction = ParameterDirection.Output;

                        cmd = oOracledb.SqlQueryReturnValue(cmd);

                        //string BASE_PRICE = cmd.Parameters["R_CURRENT_BASE_PRICE"].Value.ToString() == "null" ? "" : cmd.Parameters["R_CURRENT_BASE_PRICE"].Value.ToString();
                        //string STANDARD_COST = cmd.Parameters["R_CURRENT_STANDARD_COST"].Value.ToString() == "null" ? "" : cmd.Parameters["R_CURRENT_STANDARD_COST"].Value.ToString();
                        string MSG = cmd.Parameters["R_ERROR_MESSAGE"].Value.ToString() == "null" ? "" : cmd.Parameters["R_ERROR_MESSAGE"].Value.ToString();
                        string ERROR_FLAG_PART_NO = cmd.Parameters["R_ERROR_FLAG_PART_NO"].Value.ToString();
                        string ERROR_FLAG = cmd.Parameters["R_ERROR_FLAG"].Value.ToString();
                        string PART_NAME_EN = cmd.Parameters["R_PART_NAME"].Value.ToString() == "null" ? "" : cmd.Parameters["R_PART_NAME"].Value.ToString(); ;
                        d["SEQ"] = iCount;
                        d["PART_NAME_EN"] = PART_NAME_EN;
                        //d["BASE_PRICE"] = BASE_PRICE;
                        //d["STANDARD_COST"] = STANDARD_COST;
                        d["MSG"] = MSG;
                        d["ERR_FLAG_PART"] = ERROR_FLAG_PART_NO;
                        d["ERR_FLAG"] = ERROR_FLAG;

                        if (hTable.Contains(d["PART_NO"].ToString().Trim()))
                        {
                            d["SEQ"] = iCount;
                            d["PART_NAME_EN"] = PART_NAME_EN;
                            //d["BASE_PRICE"] = BASE_PRICE;
                            //d["STANDARD_COST"] = STANDARD_COST;
                            d["MSG"] = "Duplicate Part No at Seq " + hTable[d["PART_NO"]]; ;
                            d["ERR_FLAG_PART"] = "Y";
                            d["ERR_FLAG"] = "Y";
                        }
                        else
                        {
                            hTable.Add(d["PART_NO"].ToString().Trim(), iCount);
                        }


                    }
                    catch (Exception ex)
                    {
                        //--- Roll back ---
                        oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM135P_Validate");
                        throw new HttpResponseException(HttpStatusCode.NotAcceptable);
                    }
                }
            }
            return dt;
        }

        [HttpPost, Route("SNM135P_UpdatePrice")]
        [Authorize]
        public IHttpActionResult SNM135P_UpdatePrice([FromBody] dynamic data)
        {

            DataSet ds = new DataSet();
            DataTable dt = new DataTable();

            OracleCommand cmd = new OracleCommand();
            int iCount = 0;
            try
            {

                foreach (dynamic d in data.DETAIL)
                {
                    iCount++;
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNM135P_SAVE_TEMP";
                    cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = d["PART_NO"];
                    cmd.Parameters.Add("P_BASE_PRICE", OracleDbType.Varchar2).Value = d["NEW_BASE_PRICE"];
                    cmd.Parameters.Add("P_RETAIL_PRICE", OracleDbType.Varchar2).Value = d["RETAIL_PRICE"];
                    cmd.Parameters.Add("P_PRICING_CODE", OracleDbType.Varchar2).Value = d["NEW_PRICING_CODE"];
                    cmd.Parameters.Add("P_STANDARD_COST", OracleDbType.Varchar2).Value = d["NEW_STANDARD_COST"];
                    cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = d["VENDOR_CODE"];
                    cmd.Parameters.Add("P_PO_NO", OracleDbType.Varchar2).Value = d["PO_NO"];
                    cmd.Parameters.Add("P_PO_UNIT_PRICE", OracleDbType.Varchar2).Value = d["PO_UNIT_PRICE"];
                    cmd.Parameters.Add("P_REQUEST_BY", OracleDbType.Varchar2).Value = d["REQUEST_BY"];
                    cmd.Parameters.Add("P_CLEAR_DATA", OracleDbType.Varchar2).Value = iCount == 1 ? "Y" : "N";
                    cmd = oOracledb.SqlQueryReturnValue(cmd);
                }
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM135P_UPDATE_PRICE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_IS_BASE_PRICE", OracleDbType.Varchar2).Value = data.IS_BASE_PRICE;
                cmd.Parameters.Add("P_IS_RETAIL_PRICE", OracleDbType.Varchar2).Value = data.IS_RETAIL_PRICE;
                cmd.Parameters.Add("P_IS_STANDARD_COST", OracleDbType.Varchar2).Value = data.IS_STANDARD_COST;
                cmd.Parameters.Add("P_IS_PRICING_CODE", OracleDbType.Varchar2).Value = data.IS_PRICING_CODE;
                cmd.Parameters.Add("P_IS_PO_UNIT_PRICE", OracleDbType.Varchar2).Value = data.IS_PO_UNIT_PRICE;
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_IP", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);
                string r_status = cmd.Parameters["R_STATUS"].Value.ToString();
                string r_msg = cmd.Parameters["R_MESSAGE"].Value.ToString();

                if (r_status != "DONE")
                {
                    oOracledb.WriteLog("ErrorHandle", r_msg, "SNM135P_UpdatePrice");
                }

                return Json(new { STATUS = r_status, MESSAGE = r_msg });
            }
            catch (Exception ex)
            {
                //--- Roll back ---
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM135P_UpdatePrice");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

            //return Json(data);
        }

        [HttpPost, Route("SNM135P_SaveDataForBatch")]
        [Authorize]
        public IHttpActionResult SNM135P_SaveDataForBatch([FromBody] dynamic[] data)
        {

            DataSet ds = new DataSet();
            DataTable dt = new DataTable();

            OracleCommand cmd = new OracleCommand();
            int iCount = 0;
            try
            {
                string GUID = Guid.NewGuid().ToString();
                foreach (dynamic d in data)
                {
                    iCount++;
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNM135P_SAVE_DATA_BATCH";
                    cmd.Parameters.Add("P_ID", OracleDbType.Varchar2).Value = GUID;
                    cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = d["PART_NO"];
                    cmd.Parameters.Add("P_BASE_PRICE", OracleDbType.Varchar2).Value = d["NEW_BASE_PRICE"];
                    cmd.Parameters.Add("P_RETAIL_PRICE", OracleDbType.Varchar2).Value = d["RETAIL_PRICE"];
                    cmd.Parameters.Add("P_PRICING_CODE", OracleDbType.Varchar2).Value = d["NEW_PRICING_CODE"];
                    cmd.Parameters.Add("P_STANDARD_COST", OracleDbType.Varchar2).Value = d["NEW_STANDARD_COST"];
                    cmd.Parameters.Add("P_EFFECTIVE_DATE", OracleDbType.Varchar2).Value = d["EFFECTIVE_DATE(DD-MM-YYYY)"];
                    cmd.Parameters.Add("P_REQUEST_BY", OracleDbType.Varchar2).Value = d["REQUEST_BY"];
                    cmd = oOracledb.SqlQueryReturnValue(cmd);
                }

                return Ok("DONE");
            }
            catch (Exception ex)
            {
                //--- Roll back ---
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM135P_SaveDataForBatch");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

            //return Json(data);
        }

        [HttpPost, Route("SNM135P_Download")]
        public HttpResponseMessage SNM135P_Download([FromBody] dynamic[] data)
        {
            DataTable dt = new DataTable();
            dt.Columns.Add("SEQ");
            dt.Columns.Add("PART_NO");
            dt.Columns.Add("BASE_PRICE");
            dt.Columns.Add("RETAIL_PRICE");
            dt.Columns.Add("STANDARD_COST");
            dt.Columns.Add("PRICING_CODE");
            DataSet ds = new DataSet();
            try
            {
                    OracleCommand cmd = new OracleCommand();
                string x = string.Empty;
                string filename = "SNM135P.csv";
                string floderLocation = "SNM135P";
                    int iCount = 0;

                foreach (var d in data)
                {
                    if (d["ERR_FLAG"] != 'Y')
                    {
                        iCount++;
                        cmd = new OracleCommand();
                        cmd.CommandText = "SP_NP_SNM135P_EXPORT_EXCEL";
                        cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                        cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = d["PART_NO"];
                        cmd.Parameters.Add("R_BASE_PRICE", OracleDbType.Int16).Direction = ParameterDirection.Output;
                        cmd.Parameters.Add("R_STANDARD_COST", OracleDbType.Int16).Direction = ParameterDirection.Output;
                        cmd.Parameters.Add("R_RETAIL_PRICE", OracleDbType.Int16).Direction = ParameterDirection.Output;
                        cmd.Parameters.Add("R_PRICING_CODE", OracleDbType.Varchar2, 10).Direction = ParameterDirection.Output;
                        cmd = oOracledb.SqlQueryReturnValue(cmd);
                        double r_base_price = double.Parse(cmd.Parameters["R_BASE_PRICE"].Value.ToString());
                        double r_standard_cost = double.Parse(cmd.Parameters["R_STANDARD_COST"].Value.ToString());
                        double r_retail_price = double.Parse(cmd.Parameters["R_RETAIL_PRICE"].Value.ToString());
                        string r_pricing_code = cmd.Parameters["R_PRICING_CODE"].Value.ToString();
                        DataRow dr = dt.NewRow();
                        dr["SEQ"] = d["SEQ"];
                        dr["PART_NO"] = d["PART_NO"];
                        dr["BASE_PRICE"] = r_base_price;
                        dr["RETAIL_PRICE"] = r_retail_price;
                        dr["STANDARD_COST"] = r_standard_cost;
                        dr["PRICING_CODE"] = r_pricing_code;
                        dt.Rows.Add(dr);
                    }
                    else
                    {
                        continue;
                    }
                }
                x = DataService.WriteCSV(dt, filename, floderLocation, "SNM135P");
                Controllers.SystemApiController response = new Controllers.SystemApiController();
                var File = response.downloadFile(x.Substring(1), filename);

                return File;
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM135P_Download");
                    return Request.CreateResponse(HttpStatusCode.BadRequest);
                }
        } 
        [HttpPost, Route("SNM135P_Download_PO")]
        public HttpResponseMessage SNM135P_Download_PO([FromBody] dynamic[] data)
        {
            DataTable dt = new DataTable();
            DataTable dt2 = new DataTable();
            DataSet ds = new DataSet();
                try
                {
                    OracleCommand cmd = new OracleCommand();
                    string x = string.Empty;
                    string filename = "SNM135P_POcsv";
                    string floderLocation = "SNM135P_PO";
                    int iCount = 0;
                foreach (var d in data)
                {
                    if (d["ERR_FLAG"] == 'N') {
                        iCount++;
                        cmd = new OracleCommand();
                        cmd.CommandText = "SP_NP_SNM135P_EXPORT_EXCEL_PO";
                        cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                        cmd.Parameters.Add("P_SEQ", OracleDbType.Varchar2).Value = d["SEQ"];
                        cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = d["PART_NO"];
                        cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                        ds = oOracledb.SqlQueryDS_CMD(cmd);
                        dt2 = ds.Tables[0];
                        dt.Merge(dt2, false);
                    }
                    else if(d["ERR_FLAG"] == 'Y')
                    {
                        continue;
                    }
                }
                    x = DataService.WriteCSV(dt, filename, floderLocation, "SNM135P_PO");
                    Controllers.SystemApiController response = new Controllers.SystemApiController();
                    var File = response.downloadFile(x.Substring(1), filename);
                    return File;
                }
                catch (Exception ex)
                {
                    oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM135P_Download_PO");
                return Request.CreateResponse(HttpStatusCode.BadRequest);
            }
        }
        #endregion
        #region SNM121P

        [HttpGet, Route("SNM121P_Family_Code/{family}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM121P_Family_Code(string family, int noofshowrec)
        {
            DataSet ds = new DataSet();

            family = ismoUtil.escapeCharacter(family);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM121P_GET_FAMILY_CODE";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_FAMILY_CODE", OracleDbType.Varchar2).Value = family.ToUpper() == "NULL" ? (object)DBNull.Value : family;
            //cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }


        [HttpPost, Route("SNM121P_Search")]
        [Authorize]
        public IHttpActionResult SNM121P_Search([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM121P_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_FAMILY_CODE", OracleDbType.Varchar2).Value = param.FAMILY_CODE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM121P_Search");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNM121P_Insert")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM121P_Insert([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM121P_INSERT";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_FAMILY_COE", OracleDbType.Varchar2).Value = param.FAMILY_CODE;
                cmd.Parameters.Add("P_FAMILY_NAME", OracleDbType.Varchar2).Value = param.FAMILY_NAME;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string R_STATUS = cmd.Parameters["R_STATUS"].Value.ToString();
                string R_MESSAGE = cmd.Parameters["R_MESSAGE"].Value.ToString();

                if (R_STATUS == "FAIL")
                {
                    oOracledb.WriteLog("ErrorHandle", R_MESSAGE, "SNM121P_Insert");
                }

                return Json(new { STATUS_MSG = R_STATUS, });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM121P_Insert");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNM121P_Update")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM121P_Update([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM121P_UPDATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_FAMILY_CODE", OracleDbType.Varchar2).Value = param.FAMILY_CODE;
                cmd.Parameters.Add("P_FAMILY_NAME", OracleDbType.Varchar2).Value = param.FAMILY_NAME;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string R_STATUS = cmd.Parameters["R_STATUS"].Value.ToString();
                string R_MESSAGE = cmd.Parameters["R_MESSAGE"].Value.ToString();

                if (R_STATUS == "FAIL")
                {
                    oOracledb.WriteLog("ErrorHandle", R_MESSAGE, "SNM121P_Update");
                }

                return Json(new { STATUS_MSG = R_STATUS, });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM121P_Update");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }


        [HttpPost, Route("SNM121P_Delete")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM121P_Delete([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM121P_DELETE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_FAMILY_CODE", OracleDbType.Varchar2).Value = param.FAMILY_CODE;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string R_STATUS = cmd.Parameters["R_STATUS"].Value.ToString();
                string R_MESSAGE = cmd.Parameters["R_MESSAGE"].Value.ToString();

                if (R_STATUS == "FAIL")
                {
                    oOracledb.WriteLog("ErrorHandle", R_MESSAGE, "SNM121P_Delete");
                }

                return Json(new { STATUS_MSG = R_STATUS, });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM121P_Delete");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }
        #endregion SNM121P

        #region SNSE197_M

        [HttpGet, Route("SNSE197_M_GetPartNo/{partNo}/{NoOfShowRecords}")]
        [Authorize]
        public IHttpActionResult SNSE197_M_GetPartNo(string partNo, string NoOfShowRecords)
        {
            try
            {
                DataSet ds = new DataSet();
                partNo = ismoUtil.escapeCharacter(partNo);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE197_M_PART_NO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = partNo.ToUpper() == "NULL" ? (object)DBNull.Value : partNo;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE197_M_GetPartNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE197_M_GetactiveList")]
        [Authorize]
        public IHttpActionResult SNSE197_M_GetactiveList([FromBody] dynamic param)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE197_M_GET_ACTIVE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = param.VENDOR_CODE;
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = param.PART_NO;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE197_M_GetactiveList");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE197_M_DisableNetPrice")]
        [Authorize]
        public IHttpActionResult SNSE197_M_DisableNetPrice([FromBody] dynamic param)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE197_M_DISABLE_NET";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = param.VENDOR_CODE;
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = param.PART_NO;
                cmd.Parameters.Add("P_EFFECTIVE_DATE", OracleDbType.Date).Value = param.EFFECTIVE_DATE;
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_CURRENT_FORM", OracleDbType.Varchar2).Value = "SNSE197_M";
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string R_STATUS = cmd.Parameters["R_STATUS"].Value.ToString();
                string R_MESSAGE = cmd.Parameters["R_MESSAGE"].Value.ToString();

                if (R_STATUS == "FAIL")
                {
                    oOracledb.WriteLog("ErrorHandle", R_MESSAGE, "SNSE197_M_DisableNetPrice");
                }

                return Json(new { STATUS_MSG = R_STATUS, });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE197_M_DisableNetPrice");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE197_M_CheckOverlap")]
        [Authorize]
        public IHttpActionResult SNSE197_M_CheckOverlap([FromBody] dynamic param)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE197_M_CHECK_OVERLAP";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = param.VENDOR_CODE;
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = param.PART_NO;
                cmd.Parameters.Add("P_EFFECTIVE_DATE", OracleDbType.Date).Value = param.EFFECTIVE_DATE;
                cmd.Parameters.Add("P_DISABLE_DATE", OracleDbType.Date).Value = param.DISABLE_DATE;
                cmd.Parameters.Add("R_ERR_MSG", OracleDbType.Varchar2, 2000).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_ERR_MSG_2", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string R_ERR_MSG = cmd.Parameters["R_ERR_MSG"].Value.ToString();
                string R_ERR_MSG_2 = cmd.Parameters["R_ERR_MSG_2"].Value.ToString();

                return Json(new { STATUS_MSG = R_ERR_MSG, ERROR_MSG = R_ERR_MSG_2 });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE197_M_CheckOverlap");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE197_M_SaveByPart")]
        [Authorize]
        public IHttpActionResult SNSE197_M_SaveByPart([FromBody] dynamic param)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE197_M_SAVE_BY_PART";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = param["PART_NO"];
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = param["VENDOR_CODE"];
                cmd.Parameters.Add("P_NET_PRICE1", OracleDbType.Varchar2).Value = param["NET_PRICE1"];
                cmd.Parameters.Add("P_NET_PRICE2", OracleDbType.Varchar2).Value = param["NET_PRICE2"];
                cmd.Parameters.Add("P_EFFECTIVE_DATE", OracleDbType.Date).Value = param["EFFECTIVE_DATE"];
                cmd.Parameters.Add("P_DISABLE_DATE", OracleDbType.Date).Value = param["DISABLE_DATE"];
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_CURRENT_FORM", OracleDbType.Varchar2).Value = "SNSE197_M";
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string R_STATUS = cmd.Parameters["R_STATUS"].Value.ToString();
                string R_MESSAGE = cmd.Parameters["R_MESSAGE"].Value.ToString();

                if (R_STATUS == "FAIL")
                {
                    oOracledb.WriteLog("ErrorHandle", R_MESSAGE, "SNSE197_M_SaveByPart");
                }

                return Json(new { STATUS = R_STATUS });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE197_M_SaveByPart");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE197_M_SaveByUpload")]
        [Authorize]
        public IHttpActionResult SNSE197_M_SaveByUpload([FromBody] dynamic[] param)
        {
            DataSet ds = new DataSet();

            try
            {
                int iCount = 0;
                OracleCommand cmd = new OracleCommand();

                foreach (var detail in param)
                {
                    iCount++;

                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE197_M_SAVE_TEMP";
                    cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = detail["PART_NO"];
                    cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = detail["VENDOR_CODE"];
                    cmd.Parameters.Add("P_NET_PRICE1", OracleDbType.Varchar2).Value = detail["NET_PRICE1"];
                    cmd.Parameters.Add("P_NET_PRICE2", OracleDbType.Varchar2).Value = detail["NET_PRICE2"];
                    cmd.Parameters.Add("P_EFFECTIVE_DATE", OracleDbType.Varchar2).Value = detail["EFFECTIVE_DATE(DD-MM-YYYY)"];
                    cmd.Parameters.Add("P_DISABLE_DATE", OracleDbType.Varchar2).Value = detail["DISABLE_DATE(DD-MM-YYYY)"];
                    cmd.Parameters.Add("P_CLEAR_DATA", OracleDbType.Varchar2).Value = iCount == 1 ? "Y" : "N";
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                    oOracledb.SqlQueryReturnValue(cmd);

                }

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE197_M_SAVE_BY_UPD";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_CURRENT_FORM", OracleDbType.Varchar2).Value = "SNSE197_M";
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string R_STATUS = cmd.Parameters["R_STATUS"].Value.ToString();
                string R_MESSAGE = cmd.Parameters["R_MESSAGE"].Value.ToString();

                if (R_STATUS == "FAIL")
                {
                    oOracledb.WriteLog("ErrorHandle", R_MESSAGE, "SNSE197_M_SaveByUpload");
                }

                return Json(new { STATUS_MSG = R_STATUS, });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE197_M_SaveByUpload");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNSE197_M_UploadFile")]
        [Authorize]
        public async Task<IHttpActionResult> SNSE197_M_UploadFile(HttpRequestMessage request)
        {
            if (!request.Content.IsMimeMultipartContent())
            {
                throw new HttpResponseException(HttpStatusCode.UnsupportedMediaType);
            }
            var reqParts = await request.Content.ReadAsMultipartAsync();

            try
            {
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();
                var file = reqParts.Contents[0];
                var buffer = await file.ReadAsByteArrayAsync();
                string gFileName = file.Headers.ContentDisposition.FileName.Trim('\"');
                string gRootPath = ConfigurationManager.AppSettings["RootSharedPath"].ToString();
                string gUploadLication = System.IO.Path.Combine(gRootPath, "tempFile/SNSE197_M");
                string gFileLocation = System.IO.Path.Combine(gUploadLication, gFileName);

                DataService.SaveBytesToFileByLocation(buffer
                                                , gUploadLication
                                                , gFileLocation);

                dt = SNSE_197_M_ValidateData(gFileLocation);
                //ds.Tables.Add(dt);

                return Json(DTFM.convertToList(dt));

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE197_M_UploadFile");
                throw new HttpResponseException(HttpStatusCode.NotAcceptable);
            }

            throw new Exception("Something wrong");

        }

        public DataTable SNSE_197_M_ValidateData(string fileLocation)
        {

            string fileLocationN = fileLocation;
            DataTable dt = new DataTable();
            SystemApiController sys = new SystemApiController();
            dt = sys.ReadDataExcel(fileLocationN);
            int countrow = 1;
            Hashtable hTable = new Hashtable();
            dt.Columns.Add("PART_NAME");
            dt.Columns.Add("STATUS");
            dt.Columns.Add("ERROR_MSG");
            OracleCommand cmd = new OracleCommand();
            foreach (DataRow row in dt.Rows)
            {

                int index = dt.Rows.IndexOf(row);

                if (hTable.Contains(row["PART_NO"]))
                {

                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE197_M_GET_PART_NAME";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = row["PART_NO"] == null ? "" : row["PART_NO"].ToString().Trim();
                    cmd.Parameters.Add("P_PART_NAME", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;
                    cmd = oOracledb.SqlQueryReturnValue(cmd);
                    string part_name = cmd.Parameters["P_PART_NAME"].Value.ToString();

                    row["PART_NAME"] = part_name;
                    row["STATUS"] = "Error";
                    row["ERROR_MSG"] = "Duplicate Part No at Seq " + hTable[row["PART_NO"]];

                }
                else
                {
                    hTable.Add(row["PART_NO"], countrow);
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE197_M_VALID_UPLOAD";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = row["PART_NO"] == null ? "" : row["PART_NO"].ToString().Trim();
                    cmd.Parameters.Add("P_NET_PRICE1", OracleDbType.Varchar2).Value = row["NET_PRICE1"] == null ? "" : row["NET_PRICE1"].ToString().Trim();
                    cmd.Parameters.Add("P_NET_PRICE2", OracleDbType.Varchar2).Value = row["NET_PRICE2"] == null ? "" : row["NET_PRICE2"].ToString().Trim();
                    cmd.Parameters.Add("P_EFFECTIVE_DATE", OracleDbType.Varchar2).Value = row["EFFECTIVE_DATE(DD-MM-YYYY)"] == null ? "" : row["EFFECTIVE_DATE(DD-MM-YYYY)"].ToString().Trim();
                    cmd.Parameters.Add("P_DISABLE_DATE", OracleDbType.Varchar2).Value = row["DISABLE_DATE(DD-MM-YYYY)"] == null ? "" : row["DISABLE_DATE(DD-MM-YYYY)"].ToString().Trim();
                    cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;
                    cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;
                    cmd.Parameters.Add("R_PART_NAME", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;

                    cmd = oOracledb.SqlQueryReturnValue(cmd);

                    string status = cmd.Parameters["R_STATUS"].Value.ToString();
                    string message = cmd.Parameters["R_MESSAGE"].Value.ToString();
                    string part_name = cmd.Parameters["R_PART_NAME"].Value.ToString();

                    row["PART_NAME"] = part_name;
                    row["STATUS"] = status;
                    row["ERROR_MSG"] = message;
                }




                countrow++;
            }

            return dt;

        }

        public bool CheckDecimal(string value)
        {

            try
            {
                Decimal.Parse(value);
                if (CountDecimalDigits(decimal.Parse(value)) <= 5)
                {
                    return true;
                }
                else
                {
                    return false;
                }

            }
            catch
            {
                return false;
            }
        }

        public int CountDecimalDigits(decimal n)
        {
            return n.ToString(System.Globalization.CultureInfo.InvariantCulture)
                    //.TrimEnd('0') uncomment if you don't want to count trailing zeroes
                    .SkipWhile(c => c != '.')
                    .Skip(1)
                    .Count();
        }


        #endregion SNSE197_M


        #region SNSE147

        [HttpGet, Route("SNSE147_GetDetailByFileName/{fileName}")]
        [Authorize]
        public IHttpActionResult SNSE147_GetDetailByFileName(string fileName)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE147_DETAILBYNAME";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_FILE_NAME", OracleDbType.Varchar2).Value = fileName;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE147_GetDetailByFileName");
                throw new Exception(ex.Message);
            }

        }

        [HttpPost, Route("SNSE147_TransferData/{fileName}")]
        [Authorize]
        public IHttpActionResult SNSE147_TransferData(string fileName)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE147_CONFIRM";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_FILE_NAME", OracleDbType.Varchar2).Value = fileName;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN_MSG", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN_ERROR", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;

                oOracledb.SqlQueryReturnValue(cmd);

                string R_STATUS = cmd.Parameters["C_RETURN_MSG"].Value.ToString();
                string R_ERROR = cmd.Parameters["C_RETURN_ERROR"].Value.ToString();

                if (R_STATUS == "ERROR")
                {
                    oOracledb.WriteLog("ErrorHandle", R_ERROR, "SNSE147_TransferData");
                }

                return Ok();
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE147_TransferData");
                throw new Exception(ex.Message);
            }

        }

        [HttpPost, Route("SNSE147_UploadFile")]
        [Authorize]
        public async Task<IHttpActionResult> SNSE147_UploadFile(HttpRequestMessage request)
        {
            if (!request.Content.IsMimeMultipartContent())
            {
                throw new HttpResponseException(HttpStatusCode.UnsupportedMediaType);
            }


            try
            {
                var reqParts = await request.Content.ReadAsMultipartAsync();
                foreach (var file in reqParts.Contents)
                {
                    var buffer = await file.ReadAsByteArrayAsync();
                    string gFileName = file.Headers.ContentDisposition.FileName.Trim('\"');


                    try
                    {
                        OracleCommand cmd = new OracleCommand();
                        cmd.CommandText = "SP_NP_SNSE147_CHECK_FILE_NAME";
                        cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                        cmd.Parameters.Add("P_FILE_NAME", OracleDbType.Varchar2).Value = gFileName;
                        cmd.Parameters.Add("C_RETURN_MSG", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;
                        oOracledb.SqlQueryReturnValue(cmd);
                        string STATUS_MSG = cmd.Parameters["C_RETURN_MSG"].Value.ToString();
                        if (STATUS_MSG == "DUPLICATE")
                        {
                            return Ok(STATUS_MSG);
                        }
                    }
                    catch (Exception ex)
                    {
                        oOracledb.WriteLog("ErrorHandle", ex.Message, "SP_NP_SNSE147_CHECK_FILE_NAME");
                        return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
                    //string gFileNameExtend = String.Concat(gFileName);
                    }
                    string gRootPath = ConfigurationManager.AppSettings["RootSharedPath"].ToString();
                    string gUploadLication = System.IO.Path.Combine(gRootPath, "tempFile/SNSE147");
                    string gFileLocation = System.IO.Path.Combine(gUploadLication, gFileName);

                    DataService.SaveBytesToFileByLocation(buffer
                                                    , gUploadLication
                                                    , gFileLocation);

                    try
                    {
                        OracleCommand cmd = new OracleCommand();
                        cmd.CommandText = "PK_NP_SNSE147.REMOVE_FORECAST";
                        cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                        cmd.Parameters.Add("P_FILE_NAME", OracleDbType.Varchar2).Value = gFileName;

                        oOracledb.SqlQueryReturnValue(cmd);
                    }
                    catch (Exception ex)
                    {
                        oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE147_UploadFile_REMOVE_FORECAST");
                        return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
                    }

                    //Begin read file
                    Stream stream = new MemoryStream(buffer);

                    using (StreamReader reader = new StreamReader(stream, Encoding.UTF8))
                    {
                        string line;
                        int itemNo = 0;
                        while ((line = reader.ReadLine()) != null)
                        {
                            if (itemNo == 0)
                            {
                                itemNo++;
                                continue;
                            }  // Skip first line

                            string sep = "\t";
                            string[] splitContent = line.Split(sep.ToCharArray());
                            var lenght = splitContent.Length;
                            //read line from memorystream
                            OracleCommand cmd = new OracleCommand();
                            cmd.CommandText = "PK_NP_SNSE147.EXTRACT_FILE";
                            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                            cmd.Parameters.Add("P_FILE_NAME", OracleDbType.Varchar2).Value = gFileName;
                            cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Decimal).Value = itemNo;
                            cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                            cmd.Parameters.Add("P_YOUR_CODE", OracleDbType.Varchar2).Value = splitContent[0];
                            cmd.Parameters.Add("P_ROLE_COUNTRY_CODE", OracleDbType.Varchar2).Value = splitContent[1];
                            cmd.Parameters.Add("P_ROLE_NUMBER", OracleDbType.Varchar2).Value = splitContent[2];
                            cmd.Parameters.Add("P_RECOMMEND_ID", OracleDbType.Varchar2).Value = splitContent[3];
                            cmd.Parameters.Add("P_PROJECT_ID", OracleDbType.Varchar2).Value = splitContent[4];
                            cmd.Parameters.Add("P_MODEL_CODE", OracleDbType.Varchar2).Value = splitContent[5];
                            cmd.Parameters.Add("P_MODEL_YEAR", OracleDbType.Varchar2).Value = splitContent[6];
                            cmd.Parameters.Add("P_MODEL_ADMINISTRATION_CODE", OracleDbType.Varchar2).Value = splitContent[7];
                            cmd.Parameters.Add("P_CAR_PRODUCT_COUNTRY", OracleDbType.Varchar2).Value = splitContent[8];
                            cmd.Parameters.Add("P_SALES_COUNTRY", OracleDbType.Varchar2).Value = splitContent[9];
                            cmd.Parameters.Add("P_PROCURMENT_COUNTRY", OracleDbType.Varchar2).Value = splitContent[10];
                            cmd.Parameters.Add("P_DESIGN_SYMBOL", OracleDbType.Varchar2).Value = splitContent[11];
                            cmd.Parameters.Add("P_PART_NUMBER", OracleDbType.Varchar2).Value = splitContent[12];
                            cmd.Parameters.Add("P_PART_NAME", OracleDbType.Varchar2).Value = splitContent[13];
                            cmd.Parameters.Add("P_FORECASTED_QTY", OracleDbType.Varchar2).Value = splitContent[14];
                            cmd.Parameters.Add("P_MODIFICATION_COMPLETION", OracleDbType.Varchar2).Value = splitContent[15];
                            cmd.Parameters.Add("P_MODIFICATION_REASON", OracleDbType.Varchar2).Value = splitContent[16];
                            cmd.Parameters.Add("P_FIRM_FORECAST_DATE", OracleDbType.Varchar2).Value = splitContent[17];
                            cmd.Parameters.Add("P_SHIPPING_DUE_DATE", OracleDbType.Varchar2).Value = splitContent[18];
                            cmd.Parameters.Add("P_FINAL_DESTINATION", OracleDbType.Varchar2).Value = splitContent[19];
                            cmd.Parameters.Add("P_UPDATE_DATE", OracleDbType.Varchar2).Value = splitContent[20];
                            

                            cmd = oOracledb.SqlQueryReturnValue(cmd);
                            itemNo++;

                        }
                    }
                    return Ok(gFileName);
                }
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE147_UploadFile");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

            throw new Exception("Something wrong");

        }
        [HttpPost, Route("SNSE147_GetProcessLog")]
        [Authorize]
        public IHttpActionResult SNSE147_GetProcessLog([FromBody] dynamic param)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE147_GET_PROCESS_LOG";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_FILE_NAME", OracleDbType.Varchar2).Value = param.file_name;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                oOracledb.SqlQueryReturnValue(cmd);
                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE145_GetProcessLog");
                throw new Exception(ex.Message);
            }
        }
        #endregion SNSE147

        #region SNSE218

        [HttpGet, Route("SNSE218_GetPart/{PartNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE218_GetPart(string PartNo, string noofshowrec)
        {
            try
            {
                DataSet ds = new DataSet();
                PartNo = ismoUtil.escapeCharacter(PartNo);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE218_GET_PART";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = PartNo != "NULL" ? PartNo : (object)DBNull.Value;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE218_GetPart");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE218_Search")]
        [Authorize]
        public IHttpActionResult SNSE218_Search([FromBody] dynamic param)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE218_SEARCH";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = param.PART_NO;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE218_Search");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE218_Upload")]
        [Authorize]
        public async Task<IHttpActionResult> SNSE218_Upload(HttpRequestMessage request)
        {
            if (!request.Content.IsMimeMultipartContent())
            {
                throw new HttpResponseException(HttpStatusCode.UnsupportedMediaType);
            }
            var reqParts = await request.Content.ReadAsMultipartAsync();

            try
            {
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();
                string SPPS_CODE = string.Empty;
                var file = reqParts.Contents[0];
                //var content = reqParts.Contents[1].ReadAsByteArrayAsync();
                //SPPS_CODE = Encoding.UTF8.GetString(content.Result, 0, content.Result.Length);

                //var timestamp = DateTime.Now.ToString("yyyyMMdd");
                var buffer = await file.ReadAsByteArrayAsync();
                string gFileName = file.Headers.ContentDisposition.FileName.Trim('\"');
                //string name = Path.GetFileNameWithoutExtension(gFileName);
                //string ext = Path.GetExtension(gFileName);
                //string gFileNameExtend = String.Concat(name, '_', SPPS_CODE, ext);
                string gRootPath = ConfigurationManager.AppSettings["RootSharedPath"].ToString();
                string gUploadLication = System.IO.Path.Combine(gRootPath, "tempFile/SNSE218");
                string gFileLocation = System.IO.Path.Combine(gUploadLication, gFileName);

                DataService.SaveBytesToFileByLocation(buffer
                                                , gUploadLication
                                                , gFileLocation);


                SystemApiController sys = new SystemApiController();
                dt = sys.ReadDataExcel(gFileLocation);
                return Json(DTFM.convertToList(dt));

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE218_Upload");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.NotAcceptable, ex.Message));
            }

            throw new Exception("Something wrong");

        }


        [HttpPost, Route("SNSE218_Save")]
        [Authorize]
        [RunBatchGuard]
        public IHttpActionResult SNSE218_Save([FromBody] dynamic[] param)
        {

            try
            {
                int iCount = 0;
                OracleCommand cmd = new OracleCommand();
                foreach (var row in param)
                {
                    iCount++;
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE218_SAVE_TEMP";
                    cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = row["PART_NO"];
                    cmd.Parameters.Add("P_QTY", OracleDbType.Varchar2).Value = row["DEFAULT_ALLOWED_QTY"];
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_CLEAR_DATA", OracleDbType.Varchar2).Value = iCount == 1 ? "Y" : "N";
                    cmd.Parameters.Add("C_RETURN_STATUS", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;
                    cmd.Parameters.Add("C_RETURN_MSG", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;

                    cmd = oOracledb.SqlQueryReturnValue(cmd);

                    string status = cmd.Parameters["C_RETURN_STATUS"].Value.ToString();
                    string Message = cmd.Parameters["C_RETURN_MSG"].Value.ToString();

                    if (status == "FAIL")
                    {
                        oOracledb.WriteLog("ErrorHandle", Message, "SNSE218_SAVE_TEMP");
                        return ResponseMessage(Request.CreateResponse(HttpStatusCode.NotAcceptable, Message));
                    }



                }

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE218_INSERT";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN_STATUS", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN_MSG", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string status2 = cmd.Parameters["C_RETURN_STATUS"].Value.ToString();
                string Message2 = cmd.Parameters["C_RETURN_MSG"].Value.ToString();

                if (status2 == "FAIL")
                {
                    oOracledb.WriteLog("ErrorHandle", Message2, "SP_NP_SNSE218_SAVE");
                }

                return Json(new { STATUS_MSG = status2, });

                //return Json(DTFM.convertToList(dt));

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE218_Save");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.NotAcceptable, ex.Message));
            }

            throw new Exception("Something wrong");

        }



        [HttpPost, Route("SNSE218_Delete")]
        [Authorize]
        [RunBatchGuard]
        public IHttpActionResult SNSE218_Delete([FromBody] dynamic[] param)
        {

            try
            {
                int iCount = 0;
                OracleCommand cmd = new OracleCommand();
                foreach (var row in param)
                {
                    iCount++;
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE218_SAVE_TEMP";
                    cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = row["PART_NO"];
                    cmd.Parameters.Add("P_QTY", OracleDbType.Varchar2).Value = row["DEFAULT_ALLOWED_QTY"];
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_CLEAR_DATA", OracleDbType.Varchar2).Value = iCount == 1 ? "Y" : "N";
                    cmd.Parameters.Add("C_RETURN_STATUS", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;
                    cmd.Parameters.Add("C_RETURN_MSG", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;

                    cmd = oOracledb.SqlQueryReturnValue(cmd);

                    string status = cmd.Parameters["C_RETURN_STATUS"].Value.ToString();
                    string Message = cmd.Parameters["C_RETURN_MSG"].Value.ToString();

                    if (status == "FAIL")
                    {
                        oOracledb.WriteLog("ErrorHandle", Message, "SNSE218_DELETE_TEMP");
                        return ResponseMessage(Request.CreateResponse(HttpStatusCode.NotAcceptable, Message));
                    }



                }

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE218_DELETE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN_STATUS", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN_MSG", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string status2 = cmd.Parameters["C_RETURN_STATUS"].Value.ToString();
                string Message2 = cmd.Parameters["C_RETURN_MSG"].Value.ToString();

                if (status2 == "FAIL")
                {
                    oOracledb.WriteLog("ErrorHandle", Message2, "SP_NP_SNSE218_DELETE");
                }

                return Json(new { STATUS_MSG = status2, });

                //return Json(DTFM.convertToList(dt));

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE218_Delete");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.NotAcceptable, ex.Message));
            }

            throw new Exception("Something wrong");

        }



        #endregion

        #region SNM001P_M_MC



        //////////////////////////////////////////// AUTO COMPLETE ////////////////////////////////////////////////////////////////
        [HttpGet, Route("SNM001P_M_MC_Format_Part/{farmat}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM001P_M_MC_Formal_Part(string farmat, int noofshowrec)
        {
            DataSet ds = new DataSet();

            farmat = ismoUtil.escapeCharacter(farmat);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM001P_M_MC_FORMAT_PART";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_PRN_PART_FORMAT", OracleDbType.Varchar2).Value = farmat.ToUpper() == "NULL" ? (object)DBNull.Value : farmat;
            cmd.Parameters.Add("p_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }


        [HttpGet, Route("SNM001P_M_MC_Part_Type/{part_type}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM001P_M_MC_Part_Type(string part_type, int noofshowrec)
        {
            DataSet ds = new DataSet();

            part_type = ismoUtil.escapeCharacter(part_type);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM001P_M_MC_PARTTYPE";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_PART_TYPE", OracleDbType.Varchar2).Value = part_type.ToUpper() == "NULL" ? (object)DBNull.Value : part_type.ToUpper();
            cmd.Parameters.Add("p_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        [HttpGet, Route("SNM001P_M_MC_Part_Type_Old/{part_type}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM001P_M_MC_Part_Type_Old(string part_type, int noofshowrec)
        {
            DataSet ds = new DataSet();
            part_type = ismoUtil.escapeCharacter(part_type);
            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM001P_M_MC_PARTTYPE_OLD";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_PART_TYPE", OracleDbType.Varchar2).Value = part_type.ToUpper() == "NULL" ? (object)DBNull.Value : part_type.ToUpper();
            cmd.Parameters.Add("p_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }

        [HttpGet, Route("SNM001P_M_MC_Location/{location}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM001P_M_MC_Location(string location, int noofshowrec)
        {
            DataSet ds = new DataSet();

            location = ismoUtil.escapeCharacter(location);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM001P_M_MC_LOCATION";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_LOCATION", OracleDbType.Varchar2).Value = location.ToUpper() == "NULL" ? (object)DBNull.Value : location;
            cmd.Parameters.Add("p_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        [HttpGet, Route("SNM001P_M_MC_Tariff/{tariff}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM001P_M_MC_Tariff(string tariff, int noofshowrec)
        {
            DataSet ds = new DataSet();

            tariff = ismoUtil.escapeCharacter(tariff);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM001P_M_MC_TARIFF";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_TARIFF", OracleDbType.Varchar2).Value = tariff.ToUpper() == "NULL" ? (object)DBNull.Value : tariff;
            cmd.Parameters.Add("p_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        [HttpGet, Route("SNM001P_M_MC_Vendor/{vendor}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM001P_M_MC_Vendor(string vendor, int noofshowrec)
        {
            DataSet ds = new DataSet();

            vendor = ismoUtil.escapeCharacter(vendor);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM001P_M_MC_VENDOR";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = vendor.ToUpper() == "NULL" ? (object)DBNull.Value : vendor;
            cmd.Parameters.Add("p_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        [HttpGet, Route("SNM001P_M_MC_Currency/{currency}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM001P_M_MC_Currency(string currency, int noofshowrec)
        {
            DataSet ds = new DataSet();

            currency = ismoUtil.escapeCharacter(currency);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM001P_M_MC_CURRENCY";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_CURRENCY", OracleDbType.Varchar2).Value = currency.ToUpper() == "NULL" ? (object)DBNull.Value : currency;
            cmd.Parameters.Add("p_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        [HttpGet, Route("SNM001P_M_MC_Category/{category}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM001P_M_MC_Category(string category, int noofshowrec)
        {
            DataSet ds = new DataSet();

            category = ismoUtil.escapeCharacter(category);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM001P_M_MC_CATEGORY";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_CATEGORY", OracleDbType.Varchar2).Value = category.ToUpper() == "NULL" ? (object)DBNull.Value : category;
            cmd.Parameters.Add("p_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        [HttpGet, Route("SNM001P_M_MC_Pricing/{pricing}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM001P_M_MC_Pricing(string pricing, int noofshowrec)
        {
            DataSet ds = new DataSet();

            pricing = ismoUtil.escapeCharacter(pricing);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM001P_M_MC_PRICING";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_PRICING", OracleDbType.Varchar2).Value = pricing.ToUpper() == "NULL" ? (object)DBNull.Value : pricing;
            cmd.Parameters.Add("p_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }


        [HttpGet, Route("SNM001P_M_MC_Product/{product}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM001P_M_MC_Product(string product, int noofshowrec)
        {
            DataSet ds = new DataSet();

            product = ismoUtil.escapeCharacter(product);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM001P_M_MC_PRODUCT";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_PRODUCT", OracleDbType.Varchar2).Value = product.ToUpper() == "NULL" ? (object)DBNull.Value : product;
            cmd.Parameters.Add("p_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        [HttpGet, Route("SNM001P_M_MC_MainModel/{mainmodel}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM001P_M_MC_MainModel(string mainmodel, int noofshowrec)
        {
            DataSet ds = new DataSet();

            mainmodel = ismoUtil.escapeCharacter(mainmodel);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM001P_M_MC_MAIN_MODEL";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_MAIN_MODEL", OracleDbType.Varchar2).Value = mainmodel.ToUpper() == "NULL" ? (object)DBNull.Value : mainmodel;
            cmd.Parameters.Add("p_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        [HttpGet, Route("SNM001P_M_MC_PartCode/{partcode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM001P_M_MC_PartCode(string partcode, int noofshowrec)
        {
            DataSet ds = new DataSet();

            partcode = ismoUtil.escapeCharacter(partcode);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM001P_M_MC_PART_CODE";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_PART_CODE", OracleDbType.Varchar2).Value = partcode.ToUpper() == "NULL" ? (object)DBNull.Value : partcode;
            cmd.Parameters.Add("p_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }


        [HttpGet, Route("SNM001P_M_MC_Descript/{descript}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM001P_M_MC_Descript(string descript, int noofshowrec)
        {
            DataSet ds = new DataSet();

            descript = ismoUtil.escapeCharacter(descript);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM001P_M_MC_DESCRIPT";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_DESCRIPT", OracleDbType.Varchar2).Value = descript.ToUpper() == "NULL" ? (object)DBNull.Value : descript;
            cmd.Parameters.Add("p_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        [HttpGet, Route("SNM001P_M_MC_Salesclass/{sales}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM001P_M_MC_Salesclass(string sales, int noofshowrec)
        {
            DataSet ds = new DataSet();

            sales = ismoUtil.escapeCharacter(sales);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM001P_M_MC_SALESCLASS";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_SALES_CLASS", OracleDbType.Varchar2).Value = sales.ToUpper() == "NULL" ? (object)DBNull.Value : sales;
            cmd.Parameters.Add("p_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        [HttpGet, Route("SNM001P_M_MC_EDC/{edc}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM001P_M_MC_EDC(string edc, int noofshowrec)
        {
            DataSet ds = new DataSet();

            edc = ismoUtil.escapeCharacter(edc);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM001P_M_MC_EDC";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_EDC", OracleDbType.Varchar2).Value = edc.ToUpper() == "NULL" ? (object)DBNull.Value : edc;
            cmd.Parameters.Add("p_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNM001P_M_MC_ADC/{adc}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM001P_M_MC_ADC(string adc, int noofshowrec, [FromBody] dynamic pparam)
        {
            DataSet ds = new DataSet();

            adc = ismoUtil.escapeCharacter(adc);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM001P_M_MC_ADC";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_EDC", OracleDbType.Varchar2).Value = pparam.EDC;
            cmd.Parameters.Add("P_ADC", OracleDbType.Varchar2).Value = adc.ToUpper() == "NULL" ? (object)DBNull.Value : adc;
            cmd.Parameters.Add("p_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNM001P_M_MC_FormerPart/{formerpart}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM001P_M_MC_FormerPart(string formerpart, int noofshowrec, [FromBody] dynamic pparam)
        {
            DataSet ds = new DataSet();

            formerpart = ismoUtil.escapeCharacter(formerpart);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM001P_M_MC_FORMER_P";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pparam.PART_NO;
            cmd.Parameters.Add("P_NEW_PART_NO", OracleDbType.Varchar2).Value = pparam.NEW_PART_NO;
            cmd.Parameters.Add("P_GROUP_PART_NO", OracleDbType.Varchar2).Value = pparam.GROUP_PART_NO;
            cmd.Parameters.Add("P_FORMER_PART", OracleDbType.Varchar2).Value = formerpart.ToUpper() == "NULL" ? (object)DBNull.Value : formerpart;
            cmd.Parameters.Add("p_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNM001P_M_MC_NewPart/{newpart}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM001P_M_MC_NewPart(string newpart, int noofshowrec, [FromBody] dynamic pparam)
        {
            DataSet ds = new DataSet();

            newpart = ismoUtil.escapeCharacter(newpart);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM001P_M_MC_NEW_PART";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pparam.PART_NO;
            cmd.Parameters.Add("P_NEW_PART", OracleDbType.Varchar2).Value = newpart.ToUpper() == "NULL" ? (object)DBNull.Value : newpart;
            cmd.Parameters.Add("P_GROUP_PART_NO", OracleDbType.Varchar2).Value = pparam.GROUP_PART_NO;
            cmd.Parameters.Add("P_FORMER_PART", OracleDbType.Varchar2).Value = pparam.FORMER_PART;
            cmd.Parameters.Add("p_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }


        [HttpGet, Route("SNM001P_M_MC_GroupPart/{grouppart}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM001P_M_MC_GroupPart(string grouppart, int noofshowrec)
        {
            DataSet ds = new DataSet();

            grouppart = ismoUtil.escapeCharacter(grouppart);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM001P_M_MC_GROUP_PART";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_GROUP_PART", OracleDbType.Varchar2).Value = grouppart.ToUpper() == "NULL" ? (object)DBNull.Value : grouppart;
            cmd.Parameters.Add("p_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }


        [HttpGet, Route("SNM001P_M_MC_Family/{family}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM001P_M_MC_Family(string family, int noofshowrec)
        {
            DataSet ds = new DataSet();

            family = ismoUtil.escapeCharacter(family);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM001P_M_MC_FAMILY";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_FAMILY", OracleDbType.Varchar2).Value = family.ToUpper() == "NULL" ? (object)DBNull.Value : family;
            cmd.Parameters.Add("p_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        [HttpGet, Route("SNM001P_M_MC_LockPO/{lockpo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM001P_M_MC_LockPO(string lockpo, int noofshowrec)
        {
            DataSet ds = new DataSet();

            lockpo = ismoUtil.escapeCharacter(lockpo);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM001P_M_MC_LOCKPO";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_LOCK_PO", OracleDbType.Varchar2).Value = lockpo.ToUpper() == "NULL" ? (object)DBNull.Value : lockpo;
            cmd.Parameters.Add("p_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        [HttpGet, Route("SNM001P_M_MC_Origin/{attribute5}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM001P_M_MC_Origin(string attribute5, int noofshowrec)
        {
            DataSet ds = new DataSet();

            attribute5 = ismoUtil.escapeCharacter(attribute5);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM001P_M_MC_ORIGIN";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_ATTRIBUTE5", OracleDbType.Varchar2).Value = attribute5.ToUpper() == "NULL" ? (object)DBNull.Value : attribute5;
            cmd.Parameters.Add("p_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }


        [HttpPost, Route("SNM001P_M_MC_LRPart/{lrpart}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM001P_M_MC_LRPart(string lrpart, int noofshowrec, [FromBody] dynamic pparam)
        {
            DataSet ds = new DataSet();

            lrpart = ismoUtil.escapeCharacter(lrpart);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM001P_M_MC_LR_PART";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pparam.PART_NO;
            cmd.Parameters.Add("P_NEW_PART", OracleDbType.Varchar2).Value = lrpart.ToUpper() == "NULL" ? (object)DBNull.Value : lrpart;
            cmd.Parameters.Add("p_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }


        /////////////////////////////////////////// END AUTO COMPLETE //////////////////////////////////////////////////////////////


        [HttpPost, Route("SNM001P_M_MC_InsertPartMasterTI")]
        [Authorize]
        public IHttpActionResult SNM001P_M_MC_InsertPartMasterTI([FromBody] dynamic pDataList)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM001P_M_INSERT_TI";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pDataList["PART_NO"].ToString().ToUpper();
                cmd.Parameters.Add("P_PART_NAME_EN", OracleDbType.Varchar2).Value = pDataList["PART_NAME_EN"];
                cmd.Parameters.Add("P_PART_NAME_TH", OracleDbType.Varchar2).Value = pDataList["PART_NAME_TH"];
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pDataList["VENDOR_CODE"];
                cmd.Parameters.Add("P_VENDOR_TYPE", OracleDbType.Varchar2).Value = pDataList["VENDOR_TYPE"];
                cmd.Parameters.Add("P_CURRENCY_CODE", OracleDbType.Varchar2).Value = pDataList["CURRENCY_CODE"];
                cmd.Parameters.Add("P_KIT_FLAG", OracleDbType.Varchar2).Value = pDataList["KIT_FLAG"];
                cmd.Parameters.Add("P_NET_WEIGHT", OracleDbType.Double).Value = string.IsNullOrEmpty(pDataList["NET_WEIGHT"].ToString()) ? 0 : pDataList["NET_WEIGHT"];
                cmd.Parameters.Add("P_LR_PART_NO", OracleDbType.Varchar2).Value = pDataList["LR_PART_NO"];
                cmd.Parameters.Add("P_LOCATION_NO", OracleDbType.Varchar2).Value = pDataList["LOCATION_NO"];
                cmd.Parameters.Add("P_DIMENSION_MAX", OracleDbType.Double).Value = string.IsNullOrEmpty(pDataList["DIMENSION_MAX"].ToString()) ? 0 : pDataList["DIMENSION_MAX"];
                cmd.Parameters.Add("P_DIMENSION_MID", OracleDbType.Double).Value = string.IsNullOrEmpty(pDataList["DIMENSION_MID"].ToString()) ? 0 : pDataList["DIMENSION_MID"];
                cmd.Parameters.Add("P_DIMENSION_MIN", OracleDbType.Double).Value = string.IsNullOrEmpty(pDataList["DIMENSION_MIN"].ToString()) ? 0 : pDataList["DIMENSION_MIN"];
                cmd.Parameters.Add("P_EDC_CODE", OracleDbType.Varchar2).Value = pDataList["EDC_CODE"];
                cmd.Parameters.Add("P_ADC_CODE", OracleDbType.Varchar2).Value = pDataList["ADC_CODE"];
                cmd.Parameters.Add("P_KEY_DES_FLAG", OracleDbType.Varchar2).Value = pDataList["KEY_DES_FLAG"];
                cmd.Parameters.Add("P_PART_NO_PRINT", OracleDbType.Varchar2).Value = pDataList["PART_NO_PRINT"];
                cmd.Parameters.Add("P_AVERAGE_DEMAND", OracleDbType.Double).Value = string.IsNullOrEmpty(pDataList["AVERAGE_DEMAND"].ToString()) ? 0 : pDataList["AVERAGE_DEMAND"];
                cmd.Parameters.Add("P_RETAIL_PRICE_E", OracleDbType.Double).Value = string.IsNullOrEmpty(pDataList["RETAIL_PRICE_E"].ToString()) ? 0 : pDataList["RETAIL_PRICE_E"];
                cmd.Parameters.Add("P_PART_APPLY_DATE", OracleDbType.Varchar2).Value = pDataList["PART_APPLY_DATE"] == DateTime.MinValue ? DBNull.Value : pDataList["PART_APPLY_DATE"].ToString("yyyy-MM-dd");
                cmd.Parameters.Add("P_FAMILY_CODE", OracleDbType.Varchar2).Value = pDataList["FAMILY_CODE"];

                cmd.Parameters.Add("P_PRICING_CODE", OracleDbType.Varchar2).Value = pDataList["PRICING_CODE"];
                cmd.Parameters.Add("P_ONLINE_FLAG", OracleDbType.Varchar2).Value = pDataList["ONLINE_FLAG"];
                cmd.Parameters.Add("P_PRODUCT_CODE", OracleDbType.Varchar2).Value = pDataList["PRODUCT_CODE"];
                cmd.Parameters.Add("P_PART_CODE", OracleDbType.Varchar2).Value = pDataList["PART_CODE"];
                cmd.Parameters.Add("P_MAIN_MODEL", OracleDbType.Varchar2).Value = pDataList["MAIN_MODEL"];
                cmd.Parameters.Add("P_DESCRIPT_CODE", OracleDbType.Varchar2).Value = pDataList["DESCRIPT_CODE"];
                cmd.Parameters.Add("P_SALES_CLASS", OracleDbType.Varchar2).Value = pDataList["SALES_CLASS"];
                cmd.Parameters.Add("P_SALES_OPTION", OracleDbType.Varchar2).Value = pDataList["SALES_OPTION"];
                cmd.Parameters.Add("P_ATTRIBUTE5", OracleDbType.Varchar2).Value = pDataList["ATTRIBUTE5"];
                cmd.Parameters.Add("P_PART_TYPE", OracleDbType.Varchar2).Value = pDataList["PART_TYPE"];

                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM001P_M_MC";
                cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_IP_ADDRESS", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("C_RETURN_STATUS", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN_MSG", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string R_STATUS = cmd.Parameters["C_RETURN_STATUS"].Value.ToString();
                string R_MESSAGE = cmd.Parameters["C_RETURN_MSG"].Value.ToString();

                if (R_STATUS == "FAIL")
                {
                    oOracledb.WriteLog("ErrorHandle", R_MESSAGE, "SNM001P_M_MC_InsertPartMasterTI");
                }

                return Json(new { STATUS_MSG = R_STATUS, });

            }

            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM001P_M_MC_InsertPartMasterTI");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));

            }
        }


        [HttpPost, Route("SNM001P_M_MC_UpdatePartMasterTI")]
        [Authorize]
        public IHttpActionResult SNM001P_M_MC_UpdatePartMasterTI([FromBody] dynamic pDataList)
        {
            try
            {

                var xxx = string.IsNullOrEmpty(pDataList["NET_WEIGHT"].ToString()) ? 0 : pDataList["NET_WEIGHT"];
                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM001P_M_UPDATE_TI";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pDataList["PART_NO"];
                cmd.Parameters.Add("P_PART_NAME_EN", OracleDbType.Varchar2).Value = pDataList["PART_NAME_EN"];
                cmd.Parameters.Add("P_PART_NAME_TH", OracleDbType.Varchar2).Value = pDataList["PART_NAME_TH"];
                cmd.Parameters.Add("P_PART_TYPE", OracleDbType.Varchar2).Value = pDataList["PART_TYPE"];
                cmd.Parameters.Add("P_BRAND_CODE", OracleDbType.Varchar2).Value = pDataList["BRAND_CODE"];
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pDataList["VENDOR_CODE"];
                cmd.Parameters.Add("P_VENDOR_TYPE", OracleDbType.Varchar2).Value = pDataList["VENDOR_TYPE"];
                cmd.Parameters.Add("P_CURRENCY_CODE", OracleDbType.Varchar2).Value = pDataList["CURRENCY_CODE"];
                cmd.Parameters.Add("P_PART_CATEGORY", OracleDbType.Varchar2).Value = pDataList["PART_CATEGORY"];
                cmd.Parameters.Add("P_MAIN_MODEL", OracleDbType.Varchar2).Value = pDataList["MAIN_MODEL"];
                cmd.Parameters.Add("P_ONLINE_FLAG", OracleDbType.Varchar2).Value = pDataList["ONLINE_FLAG"];
                cmd.Parameters.Add("P_PRODUCT_CODE", OracleDbType.Varchar2).Value = pDataList["PRODUCT_CODE"];
                cmd.Parameters.Add("P_DESCRIPT_CODE", OracleDbType.Varchar2).Value = pDataList["DESCRIPT_CODE"];
                cmd.Parameters.Add("P_PART_CODE", OracleDbType.Varchar2).Value = pDataList["PART_CODE"]; ;
                cmd.Parameters.Add("P_KIT_FLAG", OracleDbType.Varchar2).Value = pDataList["KIT_FLAG"];
                cmd.Parameters.Add("P_NET_WEIGHT", OracleDbType.Double).Value = string.IsNullOrEmpty(pDataList["NET_WEIGHT"].ToString()) ? 0 : pDataList["NET_WEIGHT"];
                cmd.Parameters.Add("P_TARIFF_CODE", OracleDbType.Varchar2).Value = pDataList["TARIFF_CODE"];
                cmd.Parameters.Add("P_DUTY_RATE", OracleDbType.Double).Value = string.IsNullOrEmpty(pDataList["DUTY_RATE"].ToString()) ? 0 : pDataList["DUTY_RATE"];
                cmd.Parameters.Add("P_LOCK_PO", OracleDbType.Varchar2).Value = pDataList["LOCK_PO"];
                cmd.Parameters.Add("P_LOCK_SALES", OracleDbType.Varchar2).Value = pDataList["LOCK_SALES"];
                cmd.Parameters.Add("P_GROUP_PART_NO", OracleDbType.Varchar2).Value = pDataList["GROUP_PART_NO"];
                cmd.Parameters.Add("P_FORMER_PART_NO", OracleDbType.Varchar2).Value = pDataList["FORMER_PART_NO"];
                cmd.Parameters.Add("P_NEW_PART_NO", OracleDbType.Varchar2).Value = pDataList["NEW_PART_NO"];
                cmd.Parameters.Add("P_LR_PART_NO", OracleDbType.Varchar2).Value = pDataList["LR_PART_NO"];
                cmd.Parameters.Add("P_FORMER_ICA", OracleDbType.Varchar2).Value = pDataList["FORMER_ICA"];
                cmd.Parameters.Add("P_NEW_ICA", OracleDbType.Varchar2).Value = pDataList["NEW_ICA"];
                cmd.Parameters.Add("P_LOCATION_NO", OracleDbType.Varchar2).Value = pDataList["LOCATION_NO"];
                cmd.Parameters.Add("P_DIMENSION_MAX", OracleDbType.Double).Value = string.IsNullOrEmpty(pDataList["DIMENSION_MAX"].ToString()) ? 0 : pDataList["DIMENSION_MAX"];
                cmd.Parameters.Add("P_DIMENSION_MID", OracleDbType.Double).Value = string.IsNullOrEmpty(pDataList["DIMENSION_MID"].ToString()) ? 0 : pDataList["DIMENSION_MID"];
                cmd.Parameters.Add("P_DIMENSION_MIN", OracleDbType.Double).Value = string.IsNullOrEmpty(pDataList["DIMENSION_MIN"].ToString()) ? 0 : pDataList["DIMENSION_MIN"];
                cmd.Parameters.Add("P_PO_PACKAGE", OracleDbType.Varchar2).Value = pDataList["PO_PACKAGE"];
                cmd.Parameters.Add("P_SALES_PACKAGE", OracleDbType.Varchar2).Value = pDataList["SALES_PACKAGE"];
                cmd.Parameters.Add("P_STICKER_PACKAGE", OracleDbType.Varchar2).Value = pDataList["STICKER_PACKAGE"];
                cmd.Parameters.Add("P_PACKAGE_SIZE", OracleDbType.Varchar2).Value = pDataList["PACKAGE_SIZE"];
                cmd.Parameters.Add("P_PACKAGE_NAME", OracleDbType.Varchar2).Value = pDataList["PACKAGE_NAME"];
                cmd.Parameters.Add("P_SALES_OPTION", OracleDbType.Varchar2).Value = pDataList["SALES_OPTION"];
                cmd.Parameters.Add("P_PRICING_CODE", OracleDbType.Varchar2).Value = pDataList["PRICING_CODE"];
                cmd.Parameters.Add("P_SALES_CLASS", OracleDbType.Varchar2).Value = pDataList["SALES_CLASS"];
                cmd.Parameters.Add("P_EDC_CODE", OracleDbType.Varchar2).Value = pDataList["EDC_CODE"];
                cmd.Parameters.Add("P_ADC_CODE", OracleDbType.Varchar2).Value = pDataList["ADC_CODE"];
                cmd.Parameters.Add("P_LIST_PRICE", OracleDbType.Varchar2).Value = pDataList["LIST_PRICE"];

                cmd.Parameters.Add("P_BASE_PRICE", OracleDbType.Varchar2).Value = pDataList["BASE_PRICE"];
                cmd.Parameters.Add("P_RETAIL_PRICE", OracleDbType.Varchar2).Value = pDataList["RETAIL_PRICE"];
                cmd.Parameters.Add("P_KEY_DES_FLAG", OracleDbType.Varchar2).Value = pDataList["KEY_DES_FLAG"];
                cmd.Parameters.Add("P_PRN_PART_FORMAT", OracleDbType.Varchar2).Value = pDataList["PRN_PART_FORMAT"];
                cmd.Parameters.Add("P_PART_NO_PRINT", OracleDbType.Varchar2).Value = pDataList["PART_NO_PRINT"];
                cmd.Parameters.Add("P_MODEL_NAME", OracleDbType.Varchar2).Value = pDataList["MODEL_NAME"];

                cmd.Parameters.Add("P_AVERAGE_DEMAND", OracleDbType.Double).Value = string.IsNullOrEmpty(pDataList["AVERAGE_DEMAND"].ToString()) ? 0 : pDataList["AVERAGE_DEMAND"];
                cmd.Parameters.Add("P_FORMER_LIST", OracleDbType.Double).Value = string.IsNullOrEmpty(pDataList["FORMER_LIST"].ToString()) ? 0 : pDataList["FORMER_LIST"];
                cmd.Parameters.Add("P_FORMER_BASE", OracleDbType.Double).Value = string.IsNullOrEmpty(pDataList["FORMER_BASE"].ToString()) ? 0 : pDataList["FORMER_BASE"];
                cmd.Parameters.Add("P_FORMER_RETAIL", OracleDbType.Double).Value = string.IsNullOrEmpty(pDataList["FORMER_RETAIL"].ToString()) ? 0 : pDataList["FORMER_RETAIL"];
                cmd.Parameters.Add("P_RETAIL_PRICE_E", OracleDbType.Double).Value = string.IsNullOrEmpty(pDataList["RETAIL_PRICE_E"].ToString()) ? 0 : pDataList["RETAIL_PRICE_E"];
                cmd.Parameters.Add("P_FORMER_RETAIL_E", OracleDbType.Double).Value = string.IsNullOrEmpty(pDataList["FORMER_RETAIL_E"].ToString()) ? 0 : pDataList["FORMER_RETAIL_E"];
                cmd.Parameters.Add("P_ATTRIBUTE2", OracleDbType.Varchar2).Value = pDataList["ATTRIBUTE2"];
                cmd.Parameters.Add("P_LAST_PO_DATE", OracleDbType.Varchar2).Value = pDataList["LAST_PO_DATE"] == DateTime.MinValue ? DBNull.Value : pDataList["LAST_PO_DATE"].ToString("yyyy-MM-dd");
                cmd.Parameters.Add("P_ATTRIBUTE1", OracleDbType.Varchar2).Value = pDataList["ATTRIBUTE1"];
                cmd.Parameters.Add("P_LAST_SALES_DATE", OracleDbType.Varchar2).Value = pDataList["LAST_SALES_DATE"] == DateTime.MinValue ? DBNull.Value : pDataList["LAST_SALES_DATE"].ToString("yyyy-MM-dd");
                cmd.Parameters.Add("P_PART_APPLY_DATE", OracleDbType.Varchar2).Value = pDataList["PART_APPLY_DATE"] == DateTime.MinValue ? DBNull.Value : pDataList["PART_APPLY_DATE"].ToString("yyyy-MM-dd");


                cmd.Parameters.Add("P_STANDARD_COST", OracleDbType.Varchar2).Value = pDataList["STANDARD_COST"];
                cmd.Parameters.Add("P_ATTRIBUTE5", OracleDbType.Varchar2).Value = pDataList["ATTRIBUTE5"];
                cmd.Parameters.Add("P_FAMILY_CODE", OracleDbType.Varchar2).Value = pDataList["FAMILY_CODE"];
             


                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM001P_M_MC";
                cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_IP_ADDRESS", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString(); ;
                cmd.Parameters.Add("C_RETRUN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));

            }

            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM001P_M_MC_SavePartMasterTI");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));

            }
        }

        [HttpPost, Route("SNM001P_M_MC_UpdateNetWeight")]
        [Authorize]
        public IHttpActionResult SNM001P_M_MC_UpdateNetWeight([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM001P_M_MC_UPD_TR_DET";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = param.PART_NO;
                cmd.Parameters.Add("P_NET_WEIGHT", OracleDbType.Varchar2).Value = param.NET_WEIGHT;
                cmd.Parameters.Add("P_TARIFF_CODE", OracleDbType.Varchar2).Value = param.TARIFF_CODE;
                cmd.Parameters.Add("C_RETURN_STATUS", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN_MSG", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string R_STATUS = cmd.Parameters["C_RETURN_STATUS"].Value.ToString();
                string R_MESSAGE = cmd.Parameters["C_RETURN_MSG"].Value.ToString();

                if (R_STATUS == "FAIL")
                {
                    oOracledb.WriteLog("ErrorHandle", R_MESSAGE, "SNM001P_M_MC_UpdateNetWeight");
                }
                else
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNM001P_M_MC_UPD_ORDER";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = param.PART_NO;
                    cmd.Parameters.Add("P_NET_WEIGHT", OracleDbType.Varchar2).Value = param.NET_WEIGHT;
                    cmd.Parameters.Add("P_TARIFF_CODE", OracleDbType.Varchar2).Value = param.TARIFF_CODE;
                    cmd.Parameters.Add("C_RETURN_STATUS2", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;
                    cmd.Parameters.Add("C_RETURN_MSG2", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;

                    cmd = oOracledb.SqlQueryReturnValue(cmd);

                    R_STATUS = cmd.Parameters["C_RETURN_STATUS2"].Value.ToString();
                    R_MESSAGE = cmd.Parameters["C_RETURN_MSG2"].Value.ToString();

                    if (R_STATUS == "FAIL")
                    {
                        oOracledb.WriteLog("ErrorHandle", R_MESSAGE, "SNM001P_M_MC_UpdateNetWeight");
                    }
                }

                return Json(new { STATUS_MSG = R_STATUS, });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM001P_M_MC_UpdateNetWeight");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNM001P_M_MC_ValidateParts")]
        [Authorize]
        public IHttpActionResult SNM001P_M_MC_ValidateParts([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM001P_M_MC_VALIDATE";
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = param.PART_NO;
                cmd.Parameters.Add("C_RETURN_MSG", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string R_STATUS = cmd.Parameters["C_RETURN_MSG"].Value.ToString();
                return Json(new { STATUS_MSG = R_STATUS, });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM001P_M_MC_ValidateParts");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }





        [HttpGet, Route("SNM001P_M_MC_LookupDesignNote/{DesignNote}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM001P_M_MC_LookupDesignNote(string DesignNote, int noofshowrec)
        {
            DataSet ds = new DataSet();

            DesignNote = ismoUtil.escapeCharacter(DesignNote);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM001P_M_GET_DESIGNNOTE";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_DESIGN_NOTE", OracleDbType.Varchar2).Value = DesignNote.ToUpper() == "NULL" ? (object)DBNull.Value : DesignNote;
            cmd.Parameters.Add("p_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        [HttpGet, Route("SNM001P_M_MC_GetDesignNote/{partNo}")]
        [Authorize]
        public IHttpActionResult SNM001P_M_MC_GetDesignNote(string partNo)
        {
            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM001_M_MC_DESING_NOTE";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = partNo;
            cmd.Parameters.Add("C_RETRUN1", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }


        [HttpPost, Route("SNM001P_M_MC_SaveDesignNote")]
        [Authorize]
        public IHttpActionResult SNM001P_M_MC_SaveDesignNote([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            try
            {
                int iCount = 0;
                OracleCommand cmd = new OracleCommand();

                foreach (var detail in param.Detail)
                {
                    iCount++;

                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNM001P_M_MC_SAVE_TEMP";
                    cmd.Parameters.Add("P_DESIGN_NOTE_NUMBER", OracleDbType.Varchar2).Value = detail["DESIGN_NOTE_NUMBER"];
                    cmd.Parameters.Add("P_FILE_NAME", OracleDbType.Varchar2).Value = detail["FILE_NAME"];
                    cmd.Parameters.Add("P_FLAG", OracleDbType.Varchar2).Value = detail["FLAG"];
                    cmd.Parameters.Add("P_CLEAR_DATA", OracleDbType.Varchar2).Value = iCount == 1 ? "Y" : "N";
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                    oOracledb.SqlQueryReturnValue(cmd);

                }

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM001P_M_MC_SAVE_DN";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = param.PART_NO;
                cmd.Parameters.Add("P_PART_NAME_EN", OracleDbType.Varchar2).Value = param.PART_NAME_EN;
                cmd.Parameters.Add("P_PART_NAME_TH", OracleDbType.Varchar2).Value = param.PART_NAME_TH;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string R_STATUS = cmd.Parameters["R_STATUS"].Value.ToString();
                string R_MESSAGE = cmd.Parameters["R_MESSAGE"].Value.ToString();

                if (R_STATUS == "FAIL")
                {
                    oOracledb.WriteLog("ErrorHandle", R_MESSAGE, "SNM001P_M_MC_SaveDesignNote");
                }

                return Json(new { STATUS_MSG = R_STATUS, });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM001P_M_MC_SaveDesignNote");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }
        [HttpGet, Route("SNM001P_M_MC_CheckOH/{partNo}")]
        [Authorize]
        public IHttpActionResult SNM001P_M_MC_CheckOH(string partNo)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM001P_M_CHECK_OH";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = partNo;
            cmd.Parameters.Add("C_RETRUN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }
        #endregion

        #region SNM132P
        [HttpPost, Route("SNM132P_FIndPartsMaster")]
        [Authorize]
        public IHttpActionResult SNM132P_FIndPartsMaster([FromBody] dynamic param)
        {
            try
            {
                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM132P_FIND_PART";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = param.PART_COMPONENT_NO;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM132P_FIndPartsMaster");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNM132P_GetPArt/{PartNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM132P_GetPArt(string PartNo, string noofshowrec)
        {
            try
            {
                DataSet ds = new DataSet();
                PartNo = ismoUtil.escapeCharacter(PartNo);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM132P_GET_PART_MASTER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = PartNo.ToUpper() != "NULL" ? PartNo.ToUpper() : (object)DBNull.Value;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM132P_GetPArt");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNM132P_GetPArtKit/{PartNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM132P_GetPArtKit(string PartNo, string noofshowrec)
        {
            try
            {
                DataSet ds = new DataSet();
                PartNo = ismoUtil.escapeCharacter(PartNo);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM132P_GET_PART_KIT";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = PartNo.ToUpper() != "NULL" ? PartNo.ToUpper() : (object)DBNull.Value;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM132P_GetPArtKit");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));

            }
        }

        [HttpPost, Route("SNM132P_SearchHeader")]
        [Authorize]
        public IHttpActionResult SNM132P_SearchHeader([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM132P_GET_HEADER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = param.PART_NO;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM132P_SearchHeader");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNM132P_SearchDetail")]
        [Authorize]
        public IHttpActionResult SNM132P_SearchDetail([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM132P_GET_DETAIL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = param.PART_KIT_NO;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM132P_SearchDetail");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNM132P_Insert")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM132P_Insert([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            try
            {
                int iCount = 0;
                OracleCommand cmd = new OracleCommand();

                if(param.Detail.Count > 0)
                {
                foreach (var detail in param.Detail)
                {
                    iCount++;

                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNM132P_SAVE_TEMP";
                    cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = detail["PART_COMPONENT_NO"];
                    cmd.Parameters.Add("P_ITEM_QTY", OracleDbType.Int32).Value = detail["ITEM_QTY"];
                    cmd.Parameters.Add("P_REMARKS", OracleDbType.Varchar2).Value = detail["REMARKS"];
                    cmd.Parameters.Add("P_CLEAR_DATA", OracleDbType.Varchar2).Value = iCount == 1 ? "Y" : "N";
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                    oOracledb.SqlQueryReturnValue(cmd);

                    }
                }
                else
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNM132P_DELETE_TEMP";
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    oOracledb.SqlQueryReturnValue(cmd);
                }

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM132P_INSERT";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = param.PART_KIT_NO;
                cmd.Parameters.Add("P_SPPS_CODE", OracleDbType.Varchar2).Value = param.SPPS_CODE;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_IP", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                //cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string R_STATUS = cmd.Parameters["R_STATUS"].Value.ToString();
                string R_MESSAGE = cmd.Parameters["R_MESSAGE"].Value.ToString();

                if (R_STATUS == "FAIL")
                {
                    oOracledb.WriteLog("ErrorHandle", R_MESSAGE, "SNM132P_Insert");
                }

                return Json(new { STATUS_MSG = R_STATUS, });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM132P_Insert");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }


        [HttpPost, Route("SNM132P_Update")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM132P_Update([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            try
            {
                int iCount = 0;
                OracleCommand cmd = new OracleCommand();

                foreach (var detail in param.Detail)
                {
                    iCount++;

                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNM132P_SAVE_TEMP_UPD";
                    cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = detail["PART_COMPONENT_NO"];
                    cmd.Parameters.Add("P_ITEM_QTY", OracleDbType.Varchar2).Value = detail["ITEM_QTY"];
                    cmd.Parameters.Add("P_REMARKS", OracleDbType.Varchar2).Value = detail["REMARKS"];
                    cmd.Parameters.Add("P_FLAG", OracleDbType.Varchar2).Value = detail["FLAG"];
                    cmd.Parameters.Add("P_CLEAR_DATA", OracleDbType.Varchar2).Value = iCount == 1 ? "Y" : "N";
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                    oOracledb.SqlQueryReturnValue(cmd);

                }

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM132P_UPDATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = param.PART_KIT_NO;
                cmd.Parameters.Add("P_SPPS_CODE", OracleDbType.Varchar2).Value = param.SPPS_CODE;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string R_STATUS = cmd.Parameters["R_STATUS"].Value.ToString();
                string R_MESSAGE = cmd.Parameters["R_MESSAGE"].Value.ToString();

                if (R_STATUS == "FAIL")
                {
                    oOracledb.WriteLog("ErrorHandle", R_MESSAGE, "SNM132P_Update");
                }

                return Json(new { STATUS_MSG = R_STATUS, });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM132P_Update");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNM132P_Delete")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM132P_Delete([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            try
            {

                OracleCommand cmd = new OracleCommand();

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM132P_DELETE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = param.PART_KIT_NO;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string R_STATUS = cmd.Parameters["R_STATUS"].Value.ToString();
                string R_MESSAGE = cmd.Parameters["R_MESSAGE"].Value.ToString();

                if (R_STATUS == "FAIL")
                {
                    oOracledb.WriteLog("ErrorHandle", R_MESSAGE, "SNM132P_Delete");
                }

                return Json(new { STATUS_MSG = R_STATUS, });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM132P_Delete");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }


        [HttpPost, Route("SNM132P_UploadFile")]
        [Authorize]
        public async Task<IHttpActionResult> SNM132P_UploadFile(HttpRequestMessage request)
        {
            if (!request.Content.IsMimeMultipartContent())
            {
                throw new HttpResponseException(HttpStatusCode.UnsupportedMediaType);
            }
            var reqParts = await request.Content.ReadAsMultipartAsync();

            try
            {
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();
                string SPPS_CODE = string.Empty;
                var file = reqParts.Contents[0];
                //var content = reqParts.Contents[1].ReadAsByteArrayAsync();
                //SPPS_CODE = Encoding.UTF8.GetString(content.Result, 0, content.Result.Length);

                //var timestamp = DateTime.Now.ToString("yyyyMMdd");
                var buffer = await file.ReadAsByteArrayAsync();
                string gFileName = file.Headers.ContentDisposition.FileName.Trim('\"');
                //string name = Path.GetFileNameWithoutExtension(gFileName);
                //string ext = Path.GetExtension(gFileName);
                //string gFileNameExtend = String.Concat(name, '_', SPPS_CODE, ext);
                string gRootPath = ConfigurationManager.AppSettings["RootSharedPath"].ToString();
                string gUploadLication = System.IO.Path.Combine(gRootPath, "tempFile/SNM132P");
                string gFileLocation = System.IO.Path.Combine(gUploadLication, gFileName);

                DataService.SaveBytesToFileByLocation(buffer
                                                , gUploadLication
                                                , gFileLocation);

                dt = ValidateData(gFileLocation);
                //ds.Tables.Add(dt);

                return Json(DTFM.convertToList(dt));

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM132P_UploadFile");
                throw new HttpResponseException(HttpStatusCode.NotAcceptable);
            }

            throw new Exception("Something wrong");

        }

        public DataTable ValidateData(string filelocation)
        {
            DataTable dt = new DataTable();
            SystemApiController sys = new SystemApiController();
            dt = sys.ReadDataExcel(filelocation);
            //int countrow = 1;
            dt.Columns.Add("EDIT");
            //dt.Columns.Add("PART_COMPONENT_NO");
            dt.Columns.Add("COMPONENT_NAME_EN");
            dt.Columns.Add("COMPONENT_NAME_TH");
            dt.Columns.Add("UPDATE_BY");
            dt.Columns.Add("UPDATE_DATE");
            dt.Columns.Add("MSG");
            dt.Columns.Add("FLAG");

            foreach (DataRow row in dt.Rows)
            {

                OracleCommand cmd = new OracleCommand();
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM132P_GET_PART_NAME";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = row["PART_COMPONENT_NO"];
                cmd.Parameters.Add("R_PART_NAME_EN", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_PART_NAME_TH", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_ERROR", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                row["EDIT"] = "DEL";
                //row["PART_COMPONENT_NO"] = row["PART_COMPONENT_NO"];
                row["COMPONENT_NAME_EN"] = cmd.Parameters["R_PART_NAME_EN"].Value;
                row["COMPONENT_NAME_TH"] = cmd.Parameters["R_PART_NAME_TH"].Value;
                row["MSG"] = cmd.Parameters["R_ERROR"].Value;
                row["FLAG"] = "A";
                //row["UPDATE_BY"] = DataService.GetIdentity(User, Cons.SS_USER_NAME_EN);
                //row["UPDATE_DATE"] = DateTime.Now.ToString("yyyy-MM-dd hh:mm:ss");

            }

            return dt;
        }


        [HttpPost, Route("SNM132P_UploadMultiParts")]
        [Authorize]
        public async Task<IHttpActionResult> SNM132P_UploadMultiParts(HttpRequestMessage request)
        {
            if (!request.Content.IsMimeMultipartContent())
            {
                throw new HttpResponseException(HttpStatusCode.UnsupportedMediaType);
            }
            var reqParts = await request.Content.ReadAsMultipartAsync();

            try
            {
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();
                string USER_ID = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                var file = reqParts.Contents[0];

                var buffer = await file.ReadAsByteArrayAsync();
                string gFileName = file.Headers.ContentDisposition.FileName.Trim('\"');
                string name = Path.GetFileNameWithoutExtension(gFileName);
                string ext = Path.GetExtension(gFileName);
                string gFileNameExtend = String.Concat(name, '_', USER_ID, ext);
                string gRootPath = ConfigurationManager.AppSettings["RootSharedPath"].ToString();
                string gUploadLication = System.IO.Path.Combine(gRootPath, "tempFile/SNM132P");
                string gFileLocation = System.IO.Path.Combine(gUploadLication, gFileName);

                DataService.SaveBytesToFileByLocation(buffer
                                                , gUploadLication
                                                , gFileLocation);

                SystemApiController sys = new SystemApiController();
                //dt = sys.ReadDataExcel(gFileLocation);
                dt = SNM132P_ValidateData(gFileLocation);
                return Json(DTFM.convertToList(dt));

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM132P_UploadFile");
                throw new HttpResponseException(HttpStatusCode.NotAcceptable);
            }

            throw new Exception("Something wrong");

        }


        public DataTable SNM132P_ValidateData(string filelocation)
        {
            DataTable dt = new DataTable();
            SystemApiController sys = new SystemApiController();
            dt = sys.ReadDataExcel(filelocation);
            dt.Columns.Add("MSG");


            foreach (DataRow row in dt.Rows)
            {

                OracleCommand cmd = new OracleCommand();
                if (!string.IsNullOrEmpty(row["PART_KIT_NO"].ToString()))
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNM132P_VALIDATE_PART";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_PART_KIT_NO", OracleDbType.Varchar2).Value = row["PART_KIT_NO"];
                    cmd.Parameters.Add("P_SPPS_CODE", OracleDbType.Varchar2).Value = row["SPPS_CODE"];
                    cmd.Parameters.Add("P_PART_COMPONENT_NO", OracleDbType.Varchar2).Value = row["PART_COMPONENT_NO"];
                    cmd.Parameters.Add("C_RETURN_MSG", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;

                    cmd = oOracledb.SqlQueryReturnValue(cmd);


                    //row["MSG"] = cmd.Parameters["C_RETURN_MSG"].Value.ToString();

                    if (cmd.Parameters["C_RETURN_MSG"].Value.ToString() == "null")
                    {
                        row["MSG"] = DBNull.Value;
                    }
                    else
                    {
                        row["MSG"] = cmd.Parameters["C_RETURN_MSG"].Value;
                    }

                    int n;
                    bool isNumeric = int.TryParse(row["QUANTITY"].ToString().Trim(), out n);

                    if (!isNumeric)
                    {
                        if (!string.IsNullOrEmpty(row["MSG"].ToString()))
                        {
                            row["MSG"] = row["MSG"] + " - " + "Quantity is not number";
                        }
                        else
                        {
                            row["MSG"] = "Quantity is not number";
                        }
                    }
                }
                else
                {
                    row["MSG"] = "Part kit not found";
                }


            }

            return dt;
        }


        [HttpPost, Route("SNM132P_SaveMultipart")]
        [Authorize]
        public IHttpActionResult SNM132P_SaveMultipart([FromBody] dynamic[] param)
        {
            DataSet ds = new DataSet();

            try
            {
                int iCount = 0;
                OracleCommand cmd = new OracleCommand();

                foreach (var detail in param)
                {
                    iCount++;

                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNM132P_SAVE_TEMP_MULTI";
                    cmd.Parameters.Add("P_MAIN_PART_NO", OracleDbType.Varchar2).Value = detail["PART_KIT_NO"];
                    cmd.Parameters.Add("P_SPPS_CODE", OracleDbType.Varchar2).Value = detail["SPPS_CODE"];
                    cmd.Parameters.Add("P_KIT_PART_NO", OracleDbType.Varchar2).Value = detail["PART_COMPONENT_NO"];
                    cmd.Parameters.Add("P_QTY", OracleDbType.Varchar2).Value = detail["QUANTITY"];
                    cmd.Parameters.Add("P_REMARKS", OracleDbType.Varchar2).Value = detail["REMARKS"];
                    cmd.Parameters.Add("P_CLEAR_DATA", OracleDbType.Varchar2).Value = iCount == 1 ? "Y" : "N";
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                    oOracledb.SqlQueryReturnValue(cmd);

                }

                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM132P_INSERT_MULTI";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string R_STATUS = cmd.Parameters["R_STATUS"].Value.ToString();
                string R_MESSAGE = cmd.Parameters["R_MESSAGE"].Value.ToString();

                if (R_STATUS == "FAIL")
                {
                    oOracledb.WriteLog("ErrorHandle", R_MESSAGE, "SNM132P_SaveMultipart");
                }

                return Json(new { STATUS_MSG = R_STATUS, });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM132P_SaveMultipart");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }
        #endregion

        #region SNM142P
        [HttpGet, Route("SNM142P_GetPart/{PartNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM142P_GetPart(string PartNo, string noofshowrec)
        {
            try
            {
                DataSet ds = new DataSet();
                PartNo = ismoUtil.escapeCharacter(PartNo);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM142P_GET_PART_MASTER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = PartNo.ToUpper() != "NULL" ? PartNo.ToUpper() : (object)DBNull.Value;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM142P_GetPart");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpGet, Route("SNM142P_GetPartKit/{PartNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM142P_GetPartKit(string PartNo, string noofshowrec)
        {
            try
            {
                DataSet ds = new DataSet();
                PartNo = ismoUtil.escapeCharacter(PartNo);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM142P_GET_PART_KIT";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = PartNo.ToUpper() != "NULL" ? PartNo.ToUpper() : (object)DBNull.Value;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM142P_GetPartKit");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNM142P_SearchHeader")]
        [Authorize]
        public IHttpActionResult SNM142P_SearchHeader([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM142P_GET_HEADER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = param.PART_NO;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM142P_SearchHeader");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNM142P_SearchDetail")]
        [Authorize]
        public IHttpActionResult SNM142P_SearchDetail([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM142P_GET_DETAIL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = param.PART_KIT_NO;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM142P_SearchDetail");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNM142P_Insert")]
        [Authorize]
        public IHttpActionResult SNM142P_Insert([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();
            try
            {
                int iCount = 0;
                OracleCommand cmd = new OracleCommand();
                if (param.Detail.Count > 0)
                {
                    foreach (var detail in param.Detail)
                    {
                        iCount++;
                        cmd = new OracleCommand();
                        cmd.CommandText = "SP_NP_SNM142P_SAVE_TEMP";
                        cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = detail["PART_COMPONENT_NO"];
                        cmd.Parameters.Add("P_ITEM_QTY", OracleDbType.Int32).Value = detail["ITEM_QTY"];
                        cmd.Parameters.Add("P_STOCK_TYPE", OracleDbType.Varchar2).Value = detail["STOCK_TYPE"];
                        cmd.Parameters.Add("P_REMARKS", OracleDbType.Varchar2).Value = detail["REMARKS"];
                        cmd.Parameters.Add("P_CLEAR_DATA", OracleDbType.Varchar2).Value = iCount == 1 ? "Y" : "N";
                        cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                        oOracledb.SqlQueryReturnValue(cmd);
                    }
                }
                else
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNM142P_DELETE_TEMP";
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    oOracledb.SqlQueryReturnValue(cmd);
                }
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM142P_INSERT";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = param.PART_KIT_NO;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_IP", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);
                string R_STATUS = cmd.Parameters["R_STATUS"].Value.ToString();
                string R_MESSAGE = cmd.Parameters["R_MESSAGE"].Value.ToString();
                if (R_STATUS == "FAIL")
                {
                    oOracledb.WriteLog("ErrorHandle", R_MESSAGE, "SNM142P_Insert");
                }
                return Json(new { STATUS_MSG = R_STATUS, });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM142P_Insert");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNM142P_Update")]
        [Authorize]
        public IHttpActionResult SNM142P_Update([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();
            try
            {
                int iCount = 0;
                OracleCommand cmd = new OracleCommand();
                foreach (var detail in param.Detail)
                {
                    iCount++;
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNM142P_SAVE_TEMP_UPD";
                    cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = detail["PART_COMPONENT_NO"];
                    cmd.Parameters.Add("P_ITEM_QTY", OracleDbType.Varchar2).Value = detail["ITEM_QTY"];
                    cmd.Parameters.Add("P_STOCK_TYPE", OracleDbType.Varchar2).Value = detail["STOCK_TYPE"];
                    cmd.Parameters.Add("P_REMARKS", OracleDbType.Varchar2).Value = detail["REMARKS"];
                    cmd.Parameters.Add("P_FLAG", OracleDbType.Varchar2).Value = detail["FLAG"];
                    cmd.Parameters.Add("P_CLEAR_DATA", OracleDbType.Varchar2).Value = iCount == 1 ? "Y" : "N";
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    oOracledb.SqlQueryReturnValue(cmd);
                }
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM142P_UPDATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = param.PART_KIT_NO;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);
                string R_STATUS = cmd.Parameters["R_STATUS"].Value.ToString();
                string R_MESSAGE = cmd.Parameters["R_MESSAGE"].Value.ToString();
                if (R_STATUS == "FAIL")
                {
                    oOracledb.WriteLog("ErrorHandle", R_MESSAGE, "SNM142P_Update");
                }
                return Json(new { STATUS_MSG = R_STATUS, });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM142P_Update");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNM142P_Delete")]
        [Authorize]
        public IHttpActionResult SNM142P_Delete([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM142P_DELETE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = param.PART_KIT_NO;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);
                string R_STATUS = cmd.Parameters["R_STATUS"].Value.ToString();
                string R_MESSAGE = cmd.Parameters["R_MESSAGE"].Value.ToString();
                if (R_STATUS == "FAIL")
                {
                    oOracledb.WriteLog("ErrorHandle", R_MESSAGE, "SNM142P_Delete");
                }
                return Json(new { STATUS_MSG = R_STATUS, });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM142P_Delete");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        [HttpPost, Route("SNM142P_UploadMultiParts")]
        [Authorize]
        public async Task<IHttpActionResult> SNM142P_UploadMultiParts(HttpRequestMessage request)
        {
            if (!request.Content.IsMimeMultipartContent())
            {
                throw new HttpResponseException(HttpStatusCode.UnsupportedMediaType);
            }
            var reqParts = await request.Content.ReadAsMultipartAsync();
            try
            {
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();
                string USER_ID = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                var file = reqParts.Contents[0];
                var buffer = await file.ReadAsByteArrayAsync();
                string gFileName = file.Headers.ContentDisposition.FileName.Trim('\"');
                string name = Path.GetFileNameWithoutExtension(gFileName);
                string ext = Path.GetExtension(gFileName);
                string gFileNameExtend = String.Concat(name, '_', USER_ID, ext);
                string gRootPath = ConfigurationManager.AppSettings["RootSharedPath"].ToString();
                string gUploadLication = System.IO.Path.Combine(gRootPath, "tempFile/SNM142P");
                string gFileLocation = System.IO.Path.Combine(gUploadLication, gFileName);
                DataService.SaveBytesToFileByLocation(buffer
                                                , gUploadLication
                                                , gFileLocation);
                SystemApiController sys = new SystemApiController();
                dt = SNM142P_ValidateData(gFileLocation);
                return Json(DTFM.convertToList(dt));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM142P_UploadFile");
                throw new HttpResponseException(HttpStatusCode.NotAcceptable);
            }
            throw new Exception("Something wrong");
        }
        public DataTable SNM142P_ValidateData(string filelocation)
        {
            DataTable dt = new DataTable();
            SystemApiController sys = new SystemApiController();
            dt = sys.ReadDataExcel(filelocation);
            dt.Columns.Add("MSG");
            foreach (DataRow row in dt.Rows)
            {
                OracleCommand cmd = new OracleCommand();
                if (!string.IsNullOrEmpty(row["PART_KIT_NO"].ToString()))
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNM142P_VALIDATE_PART";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_PART_KIT_NO", OracleDbType.Varchar2).Value = row["PART_KIT_NO"];
                    cmd.Parameters.Add("P_STOCK_TYPE", OracleDbType.Varchar2).Value = row["STOCK_TYPE"].ToString().Trim();
                    cmd.Parameters.Add("P_PART_COMPONENT_NO", OracleDbType.Varchar2).Value = row["PART_COMPONENT_NO"];
                    cmd.Parameters.Add("C_RETURN_MSG", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;
                    cmd = oOracledb.SqlQueryReturnValue(cmd);
                    if (cmd.Parameters["C_RETURN_MSG"].Value.ToString() == "null")
                    {
                        row["MSG"] = DBNull.Value;
                    }
                    else
                    {
                        row["MSG"] = cmd.Parameters["C_RETURN_MSG"].Value;
                    }
                    int n;
                    bool isNumeric = int.TryParse(row["QUANTITY"].ToString().Trim(), out n);                    
                    if (!isNumeric)
                    {
                        if (!string.IsNullOrEmpty(row["MSG"].ToString()))
                        {
                            row["MSG"] = row["MSG"] + " - " + "Quantity is not number";
                        }
                        else
                        {
                            row["MSG"] = "Quantity is not number";
                        }
                    }
                    if (string.IsNullOrEmpty(row["STOCK_TYPE"].ToString().Trim()))
                    {
                        if (!string.IsNullOrEmpty(row["MSG"].ToString()))
                        {
                            row["MSG"] = row["MSG"] + " - " + "Stock type must not be null.";
                        }
                        else
                        {
                            row["MSG"] = "Stock type must not be null.";
                        }
                    }
                }
                else
                {
                    row["MSG"] = "Part kit not found";
                }
            }
            return dt;
        }
        [HttpPost, Route("SNM142P_SaveMultipart")]
        [Authorize]
        public IHttpActionResult SNM142P_SaveMultipart([FromBody] dynamic[] param)
        {
            DataSet ds = new DataSet();
            try
            {
                int iCount = 0;
                OracleCommand cmd = new OracleCommand();
                foreach (var detail in param)
                {
                    iCount++;
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNM142P_SAVE_TEMP_MULTI";
                    cmd.Parameters.Add("P_PART_KIT_NO", OracleDbType.Varchar2).Value = detail["PART_KIT_NO"];
                    cmd.Parameters.Add("P_STOCK_TYPE", OracleDbType.Varchar2).Value = detail["STOCK_TYPE"];
                    cmd.Parameters.Add("P_PART_COMPONENT_NO", OracleDbType.Varchar2).Value = detail["PART_COMPONENT_NO"];
                    cmd.Parameters.Add("P_QTY", OracleDbType.Varchar2).Value = detail["QUANTITY"];
                    cmd.Parameters.Add("P_REMARKS", OracleDbType.Varchar2).Value = detail["REMARKS"];
                    cmd.Parameters.Add("P_CLEAR_DATA", OracleDbType.Varchar2).Value = iCount == 1 ? "Y" : "N";
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    oOracledb.SqlQueryReturnValue(cmd);
                }
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM142P_INSERT_MULTI";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;
                cmd = oOracledb.SqlQueryReturnValue(cmd);
                string R_STATUS = cmd.Parameters["R_STATUS"].Value.ToString();
                string R_MESSAGE = cmd.Parameters["R_MESSAGE"].Value.ToString();
                if (R_STATUS == "FAIL")
                {
                    oOracledb.WriteLog("ErrorHandle", R_MESSAGE, "SNM142P_SaveMultipart");
                }
                return Json(new { STATUS_MSG = R_STATUS, });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM142P_SaveMultipart");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        #endregion
        #region SNM131P
        [HttpPost, Route("SNM131P_UploadFile")]
        [Authorize]
        public async Task<IHttpActionResult> SNM131P_UploadFile(HttpRequestMessage request)
        {
            if (!request.Content.IsMimeMultipartContent())
            {
                throw new HttpResponseException(HttpStatusCode.UnsupportedMediaType);
            }
            var reqParts = await request.Content.ReadAsMultipartAsync();

            try
            {
                string SPPS_CODE = string.Empty;
                var file = reqParts.Contents[0];
                var content = reqParts.Contents[1].ReadAsByteArrayAsync();
                SPPS_CODE = Encoding.UTF8.GetString(content.Result, 0, content.Result.Length);

                //var timestamp = DateTime.Now.ToString("yyyyMMdd");
                var buffer = await file.ReadAsByteArrayAsync();
                string gFileName = file.Headers.ContentDisposition.FileName.Trim('\"');
                string name = Path.GetFileNameWithoutExtension(gFileName);
                string ext = Path.GetExtension(gFileName);
                string gFileNameExtend = String.Concat(name, '_', SPPS_CODE, ext);
                string gRootPath = ConfigurationManager.AppSettings["RootSharedPath"].ToString();
                string gUploadLication = System.IO.Path.Combine(gRootPath, "tempFile/SNM131P");
                string gFileLocation = System.IO.Path.Combine(gUploadLication, gFileNameExtend);

                DataService.SaveBytesToFileByLocation(buffer
                                                , gUploadLication
                                                , gFileLocation);
                return Ok(gFileNameExtend);

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM131P_UploadFile");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

            throw new Exception("Something wrong");

        }

        [HttpPost, Route("SNM131P_Download")]
        public HttpResponseMessage SNM131P_Download([FromBody] dynamic pParam)
        {
            try
            {
                DataTable dt = new DataTable();

                string filename = pParam.filename;

                string gRootPath = ConfigurationManager.AppSettings["RootSharedPath"].ToString();
                string gUploadLication = System.IO.Path.Combine(gRootPath, "tempFile/SNM131P");
                string gFileLocation = System.IO.Path.Combine(gUploadLication, filename);
                Controllers.SystemApiController response = new Controllers.SystemApiController();
                var File = response.downloadFileFromServer(gFileLocation, filename);

                return File;
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM131P_Download");
                return Request.CreateResponse(HttpStatusCode.InternalServerError);
            }

        }

        [HttpGet, Route("SNM131P_DesignNote/{design}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM131P_DesignNote(string design, int noofshowrec)
        {
            DataSet ds = new DataSet();

            design = ismoUtil.escapeCharacter(design);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM131P_GET_DESIGN_NOTE";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_DESIGN_NOTE", OracleDbType.Varchar2).Value = design.ToUpper() == "NULL" ? (object)DBNull.Value : design;
            //cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }


        [HttpPost, Route("SNM131P_Search")]
        [Authorize]
        public IHttpActionResult SNM131P_Search([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM131P_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DESIGN_NOTE", OracleDbType.Varchar2).Value = param.DESIGN_NOTE_NUMBER;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM131P_Search");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNM131P_Insert")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM131P_Insert([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM131P_INSERT";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DESIGN_NOTE", OracleDbType.Varchar2).Value = param.DESIGN_NOTE_NUMBER;
                cmd.Parameters.Add("FILE_NAME", OracleDbType.Varchar2).Value = param.FILE_NAME;
                cmd.Parameters.Add("P_PGM", OracleDbType.Varchar2).Value = "SNM131P";
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_IP", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string R_STATUS = cmd.Parameters["R_STATUS"].Value.ToString();
                string R_MESSAGE = cmd.Parameters["R_MESSAGE"].Value.ToString();

                if (R_STATUS == "FAIL")
                {
                    oOracledb.WriteLog("ErrorHandle", R_MESSAGE, "SNM131P_Insert");
                }

                return Json(new { STATUS_MSG = R_STATUS, });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM131P_Insert");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNM131P_Update")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM131P_Update([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM131P_UPDATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DESIGN_NOTE", OracleDbType.Varchar2).Value = param.DESIGN_NOTE_NUMBER;
                cmd.Parameters.Add("FILE_NAME", OracleDbType.Varchar2).Value = param.FILE_NAME;
                cmd.Parameters.Add("P_PGM", OracleDbType.Varchar2).Value = "SNM131P";
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_IP", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string R_STATUS = cmd.Parameters["R_STATUS"].Value.ToString();
                string R_MESSAGE = cmd.Parameters["R_MESSAGE"].Value.ToString();

                if (R_STATUS == "FAIL")
                {
                    oOracledb.WriteLog("ErrorHandle", R_MESSAGE, "SNM131P_Update");
                }

                return Json(new { STATUS_MSG = R_STATUS, });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM131P_Update");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }


        [HttpPost, Route("SNM131P_Delete")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM131P_Delete([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM131P_DELETE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DESIGN_NOTE", OracleDbType.Varchar2).Value = param.DESIGN_NOTE_NUMBER;
                cmd.Parameters.Add("FILE_NAME", OracleDbType.Varchar2).Value = param.FILE_NAME;
                cmd.Parameters.Add("P_PGM", OracleDbType.Varchar2).Value = "SNM131P";
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_IP", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string R_STATUS = cmd.Parameters["R_STATUS"].Value.ToString();
                string R_MESSAGE = cmd.Parameters["R_MESSAGE"].Value.ToString();

                if (R_STATUS == "FAIL")
                {
                    oOracledb.WriteLog("ErrorHandle", R_MESSAGE, "SNM131P_Delete");
                }

                return Json(new { STATUS_MSG = R_STATUS, });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM131P_Delete");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }
        #endregion

        #region SNSE127_1
        [HttpGet, Route("SNSE127_1_GetVendor/{vendor}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE127_1_GetVendor(string vendor, int noofshowrec)
        {
            DataSet ds = new DataSet();

            vendor = ismoUtil.escapeCharacter(vendor);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSE127_1_VENDOR_CODE";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = vendor.ToUpper() == "NULL" ? (object)DBNull.Value : vendor;
            cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }

        [HttpGet, Route("SNSE127_1_GetPricing/{pricing}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM009P_GetPricing(string pricing, int noofshowrec)
        {
            DataSet ds = new DataSet();

            pricing = ismoUtil.escapeCharacter(pricing);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSE127_1_PRICING_CODE";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_PRICING_CODE", OracleDbType.Varchar2).Value = pricing.ToUpper() == "NULL" ? (object)DBNull.Value : pricing;
            cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }


        [HttpPost, Route("SNSE127_1_CalByVendor")]
        [Authorize]
        public IHttpActionResult SNSE127_1_CalByVendor([FromBody] dynamic pData)
        {
            try
            {

                OracleCommand cmd = new OracleCommand();

                cmd = new OracleCommand();
                cmd.CommandText = "PK_NP_SNSE127_1.PRINT_PRICE_BY_VENDOR";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PROCESS_YEAR", OracleDbType.Varchar2).Value = pData.YEAR;
                cmd.Parameters.Add("P_PROCESS_MONTH", OracleDbType.Varchar2).Value = pData.MONTH;
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData.VENDOR_CODE;
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 100).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string R_STATUS = cmd.Parameters["R_STATUS"].Value.ToString();
                string R_MESSAGE = cmd.Parameters["R_MESSAGE"].Value.ToString();

                if (R_STATUS == "FAIL")
                {
                    oOracledb.WriteLog("ErrorHandle", R_MESSAGE, "SNSE127_1_CalByVendor");
                }

                return Json(new { STATUS_MSG = R_STATUS, });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE127_1_CalByVendor");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE127_1_CalByPricing")]
        [Authorize]
        public IHttpActionResult SNSE127_1_CalByPricing([FromBody] dynamic pData)
        {
            try
            {

                OracleCommand cmd = new OracleCommand();

                cmd = new OracleCommand();
                cmd.CommandText = "PK_NP_SNSE127_1.PRINT_PRICE_BY_PRICING";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PROCESS_YEAR", OracleDbType.Varchar2).Value = pData.YEAR;
                cmd.Parameters.Add("P_PROCESS_MONTH", OracleDbType.Varchar2).Value = pData.MONTH;
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData.PRICING_CODE;
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 100).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string R_STATUS = cmd.Parameters["R_STATUS"].Value.ToString();
                string R_MESSAGE = cmd.Parameters["R_MESSAGE"].Value.ToString();

                if (R_STATUS == "FAIL")
                {
                    oOracledb.WriteLog("ErrorHandle", R_MESSAGE, "SNSE127_1_CalByPricing");
                }

                return Json(new { STATUS_MSG = R_STATUS, });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE127_1_CalByPricing");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE127_1_CalPrice")]
        [Authorize]
        public IHttpActionResult SNSE127_1_CalPrice([FromBody] dynamic pData)
        {
            try
            {

                OracleCommand cmd = new OracleCommand();

                cmd = new OracleCommand();
                cmd.CommandText = "PK_NP_SNSE127_1.PRINT_PAPER";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PROCESS_YEAR", OracleDbType.Varchar2).Value = pData.YEAR;
                cmd.Parameters.Add("P_PROCESS_MONTH", OracleDbType.Varchar2).Value = pData.MONTH;
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 100).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string R_STATUS = cmd.Parameters["R_STATUS"].Value.ToString();
                string R_MESSAGE = cmd.Parameters["R_MESSAGE"].Value.ToString();

                if (R_STATUS == "FAIL")
                {
                    oOracledb.WriteLog("ErrorHandle", R_MESSAGE, "SNSE127_1_CalPrice");
                }

                return Json(new { STATUS_MSG = R_STATUS, });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE127_1_CalPrice");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE127_EXPORT_1")]
        public HttpResponseMessage SNSE127_EXPORT_1([FromBody] dynamic pParams)
        {
            DataTable dt = new DataTable();
            DataSet ds = new DataSet();



            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSE127_1_EX_PRICE01";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_PROCESS_YEAR", OracleDbType.Varchar2).Value = pParams.PROCESS_YEAR;
            cmd.Parameters.Add("P_PROCESS_MONTH", OracleDbType.Int16).Value = pParams.PROCESS_MONTH;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
            ds = oOracledb.SqlQueryDS_CMD(cmd);

            dt = ds.Tables[0];

            string x = string.Empty;
            string filename = "PRICE01.txt";
            string floderLocation = "SNSE127";

            x = DataService.WriteTSVNoHeader(dt, filename, floderLocation);
            Controllers.SystemApiController response = new Controllers.SystemApiController();
            var File = response.downloadFile(x.Substring(1), filename);
            return File;
        }

        [HttpPost, Route("SNSE127_EXPORT_2")]
        public HttpResponseMessage SNSE127_EXPORT_2([FromBody] dynamic pParams)
        {
            DataTable dt = new DataTable();
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE127_1_EX_PARTS";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PROCESS_YEAR", OracleDbType.Varchar2).Value = pParams.PROCESS_YEAR;
                cmd.Parameters.Add("P_PROCESS_MONTH", OracleDbType.Int16).Value = pParams.PROCESS_MONTH;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                dt = ds.Tables[0];

                string x = string.Empty;
                string filename = "PARTS.txt";
                string floderLocation = "SNSE127";

                x = DataService.WriteTSVNoHeader(dt, filename, floderLocation);
                Controllers.SystemApiController response = new Controllers.SystemApiController();
                var File = response.downloadFile(x.Substring(1), filename);
                return File;


            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE127_EXPORT_2");
                return Request.CreateResponse(HttpStatusCode.BadRequest, ex);
            }
        }

        [HttpPost, Route("SNSE127_EXPORT_3")]
        public HttpResponseMessage SNSE127_EXPORT_3([FromBody] dynamic pParams)
        {
            DataTable dt = new DataTable();
            DataSet ds = new DataSet();


            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE127_1_EX_CAR_D";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PROCESS_YEAR", OracleDbType.Varchar2).Value = pParams.PROCESS_YEAR;
                cmd.Parameters.Add("P_PROCESS_MONTH", OracleDbType.Int16).Value = pParams.PROCESS_MONTH;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                dt = ds.Tables[0];

                string x = string.Empty;
                string filename = "CAR_D.txt";
                string floderLocation = "SNSE127";

                x = DataService.WriteTSVNoHeader(dt, filename, floderLocation);
                Controllers.SystemApiController response = new Controllers.SystemApiController();
                var File = response.downloadFile(x.Substring(1), filename);
                return File;


            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE127_EXPORT_3");
                return Request.CreateResponse(HttpStatusCode.BadRequest, ex);
            }
        }
        #endregion


        #region SNM130P
        [HttpPost, Route("SNM130P_UploadFile")]
        [Authorize]
        public async Task<IHttpActionResult> SNM130P_UploadFile(HttpRequestMessage request)
        {
            if (!request.Content.IsMimeMultipartContent())
            {
                throw new HttpResponseException(HttpStatusCode.UnsupportedMediaType);
            }
            var reqParts = await request.Content.ReadAsMultipartAsync();

            try
            {
                string SPPS_CODE = string.Empty;
                var file = reqParts.Contents[0];
                var content = reqParts.Contents[1].ReadAsByteArrayAsync();
                SPPS_CODE = Encoding.UTF8.GetString(content.Result, 0, content.Result.Length);

                //var timestamp = DateTime.Now.ToString("yyyyMMdd");
                var buffer = await file.ReadAsByteArrayAsync();
                string gFileName = file.Headers.ContentDisposition.FileName.Trim('\"');
                string name = Path.GetFileNameWithoutExtension(gFileName);
                string ext = Path.GetExtension(gFileName);
                string gFileNameExtend = String.Concat(name, '_', SPPS_CODE, ext);
                string gRootPath = ConfigurationManager.AppSettings["RootSharedPath"].ToString();
                string gUploadLication = System.IO.Path.Combine(gRootPath, "tempFile/SNM130P");
                string gFileLocation = System.IO.Path.Combine(gUploadLication, gFileNameExtend);

                DataService.SaveBytesToFileByLocation(buffer
                                                , gUploadLication
                                                , gFileLocation);
                return Ok(gFileNameExtend);

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM130P_UploadFile");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

            throw new Exception("Something wrong");

        }

        [HttpPost, Route("SNM130P_Download")]
        public HttpResponseMessage SNM130P_Download([FromBody] dynamic pParam)
        {

            try
            {
                DataTable dt = new DataTable();

                string filename = pParam.filename;

                string gRootPath = ConfigurationManager.AppSettings["RootSharedPath"].ToString();
                string gUploadLication = System.IO.Path.Combine(gRootPath, "tempFile/SNM130P");
                string gFileLocation = System.IO.Path.Combine(gUploadLication, filename);
                Controllers.SystemApiController response = new Controllers.SystemApiController();
                var File = response.downloadFileFromServer(gFileLocation, filename);

                return File;
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM130P_Download");
                return Request.CreateResponse(HttpStatusCode.InternalServerError);
            }

        }

        [HttpGet, Route("SNM130P_Spps_Code/{spps}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM130P_Spps_Code(string spps, int noofshowrec)
        {
            DataSet ds = new DataSet();

            spps = ismoUtil.escapeCharacter(spps);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM130P_GET_SPPS";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_SPPS_CODE", OracleDbType.Varchar2).Value = spps.ToUpper() == "NULL" ? (object)DBNull.Value : spps;
            //cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }


        [HttpPost, Route("SNM130P_Search")]
        [Authorize]
        public IHttpActionResult SNM130P_Search([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM130P_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_SPPS_CODE", OracleDbType.Varchar2).Value = param.SPPS_CODE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM130P_Search");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNM130P_Insert")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM130P_Insert([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM130P_INSERT";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("SPPS_CODE", OracleDbType.Varchar2).Value = param.SPPS_CODE;
                cmd.Parameters.Add("FILE_NAME", OracleDbType.Varchar2).Value = param.FILE_NAME;
                cmd.Parameters.Add("P_PGM", OracleDbType.Varchar2).Value = "SNM130P";
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_IP", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string R_STATUS = cmd.Parameters["R_STATUS"].Value.ToString();
                string R_MESSAGE = cmd.Parameters["R_MESSAGE"].Value.ToString();

                if (R_STATUS == "FAIL")
                {
                    oOracledb.WriteLog("ErrorHandle", R_MESSAGE, "SNM130P_Insert");
                }

                return Json(new { STATUS_MSG = R_STATUS, });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM130P_Insert");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNM130P_Update")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM130P_Update([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM130P_UPDATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("SPPS_CODE", OracleDbType.Varchar2).Value = param.SPPS_CODE;
                cmd.Parameters.Add("FILE_NAME", OracleDbType.Varchar2).Value = param.FILE_NAME;
                cmd.Parameters.Add("P_PGM", OracleDbType.Varchar2).Value = "SNM130P";
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_IP", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string R_STATUS = cmd.Parameters["R_STATUS"].Value.ToString();
                string R_MESSAGE = cmd.Parameters["R_MESSAGE"].Value.ToString();

                if (R_STATUS == "FAIL")
                {
                    oOracledb.WriteLog("ErrorHandle", R_MESSAGE, "SNM130P_Update");
                }

                return Json(new { STATUS_MSG = R_STATUS, });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM130P_Update");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }


        [HttpPost, Route("SNM130P_Delete")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM130P_Delete([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM130P_DELETE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("SPPS_CODE", OracleDbType.Varchar2).Value = param.SPPS_CODE;
                cmd.Parameters.Add("FILE_NAME", OracleDbType.Varchar2).Value = param.FILE_NAME;
                cmd.Parameters.Add("P_PGM", OracleDbType.Varchar2).Value = "SNM130P";
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_IP", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string R_STATUS = cmd.Parameters["R_STATUS"].Value.ToString();
                string R_MESSAGE = cmd.Parameters["R_MESSAGE"].Value.ToString();

                if (R_STATUS == "FAIL")
                {
                    oOracledb.WriteLog("ErrorHandle", R_MESSAGE, "SNM130P_Delete");
                }

                return Json(new { STATUS_MSG = R_STATUS, });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM130P_Delete");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }
        #endregion

        #region SNSE145

        [HttpGet, Route("SNSE145_GetDetailByFileName/{fileName}")]
        [Authorize]
        public IHttpActionResult SNSE145_GetDetailByFileName(string fileName)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE145_DETAILBYNAME";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_FILE_NAME", OracleDbType.Varchar2).Value = fileName;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE145_GetDetailByFileName");
                throw new Exception(ex.Message);
            }

        }

        [HttpPost, Route("SNSE145_TransferData/{fileName}")]
        [Authorize]
        public IHttpActionResult SNSE145_TransferData(string fileName)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "PK_NP_SNSE145.CONFIRM_WVLH_FOR_REGIS";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_FILE_NAME", OracleDbType.Varchar2).Value = fileName;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                oOracledb.SqlQueryReturnValue(cmd);

                return Ok();
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE145_TransferData");
                throw new Exception(ex.Message);
            }

        }

        [HttpPost, Route("SNSE145_GetProcessLog")]
        [Authorize]
        public IHttpActionResult SNSE145_GetProcessLog([FromBody] dynamic param)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE145_GET_PROCESS_LOG";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_FILE_NAME", OracleDbType.Varchar2).Value = param.file_name;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                oOracledb.SqlQueryReturnValue(cmd);

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE145_GetProcessLog");
                throw new Exception(ex.Message);
            }

        }


        [HttpPost, Route("SNSE145_UploadFile")]
        [Authorize]
        public async Task<IHttpActionResult> SNSE145_UploadFile(HttpRequestMessage request)
        {
            if (!request.Content.IsMimeMultipartContent())
            {
                oOracledb.WriteLog("ErrorHandle", "UnsupportedMediaType", "SNSE145_EXTRACT_FILE");
                throw new HttpResponseException(HttpStatusCode.UnsupportedMediaType);
            }
            int itemNo = 0;

            try
            {
                var provider = new MultipartMemoryStreamProvider();
                var reqParts = await request.Content.ReadAsMultipartAsync(provider);

                foreach (var file in reqParts.Contents)
                {
                    var timestamp = DateTime.Now.ToString("yyyyMMdd");
                    var buffer = await file.ReadAsByteArrayAsync();
                    string gFileName = file.Headers.ContentDisposition.FileName.Trim('\"');


                    try
                    {
                        OracleCommand cmd = new OracleCommand();
                        cmd.CommandText = "SP_NP_SNSE145_CHECK_FILE_NAME";
                        cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                        cmd.Parameters.Add("P_FILE_NAME", OracleDbType.Varchar2).Value = gFileName;
                        cmd.Parameters.Add("C_RETURN_MSG", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;

                        oOracledb.SqlQueryReturnValue(cmd);


                        string STATUS_MSG = cmd.Parameters["C_RETURN_MSG"].Value.ToString();

                        if (STATUS_MSG == "DUPLICATE")
                        {
                            return Ok(STATUS_MSG);
                        }


                    }
                    catch (Exception ex)
                    {
                        oOracledb.WriteLog("ErrorHandle", ex.Message, "SP_NP_SNSE145_CHECK_FILE_NAME");
                        return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
                        //throw new HttpResponseException(HttpStatusCode.InternalServerError);
                    }




                    //string gFileNameExtend = String.Concat(gFileName, '_', timestamp);
                    string gRootPath = ConfigurationManager.AppSettings["RootSharedPath"].ToString();
                    string gUploadLication = System.IO.Path.Combine(gRootPath, "tempFile/SNSE145");
                    string gFileLocation = System.IO.Path.Combine(gUploadLication, gFileName);

                    DataService.SaveBytesToFileByLocation(buffer
                                                    , gUploadLication
                                                    , gFileLocation);

                    try
                    {
                        OracleCommand cmd = new OracleCommand();
                        cmd.CommandText = "PK_NP_SNSE145.REMOVE_WHOLE_VEHICLE";
                        cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                        cmd.Parameters.Add("P_FILE_NAME", OracleDbType.Varchar2).Value = gFileName;

                        oOracledb.SqlQueryReturnValue(cmd);
                    }
                    catch (Exception ex)
                    {
                        oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE145_REMOVE_WHOLE_VEHICLE");
                        throw new HttpResponseException(HttpStatusCode.InternalServerError);
                    }

                    //Begin read file
                    Stream stream = new MemoryStream(buffer);

                    using (StreamReader reader = new StreamReader(stream, Encoding.UTF8))
                    {
                        string line;

                        while ((line = reader.ReadLine()) != null)
                        {
                            if (itemNo < 1)
                            {
                                itemNo++;
                                continue;
                            }  // Skip first line

                            string sep = "\t";
                            string[] splitContent = line.Split(sep.ToCharArray());
                            var lenght = splitContent.Length;
                            //read line from memorystream
                            OracleCommand cmd = new OracleCommand();
                            cmd.CommandText = "PK_NP_SNSE145.EXTRACT_FILE";
                            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                            cmd.Parameters.Add("P_FILE_NAME", OracleDbType.Varchar2).Value = gFileName;
                            cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Decimal).Value = itemNo;
                            cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                            cmd.Parameters.Add("P_PUB_NO", OracleDbType.Varchar2).Value = splitContent[0].Trim();
                            cmd.Parameters.Add("P_CATALOG_DESTINATION", OracleDbType.Varchar2).Value = splitContent[1].Trim();
                            cmd.Parameters.Add("P_SECTION_NUMBER", OracleDbType.Varchar2).Value = splitContent[2].Trim();
                            cmd.Parameters.Add("P_SECTION_DIFFERENCE", OracleDbType.Varchar2).Value = splitContent[3].Trim();
                            cmd.Parameters.Add("P_PART_CODE", OracleDbType.Varchar2).Value = splitContent[4].Trim();
                            cmd.Parameters.Add("P_PART_NUMBER", OracleDbType.Varchar2).Value = splitContent[5].Trim();
                            cmd.Parameters.Add("P_CATALOG_NC_CODE", OracleDbType.Varchar2).Value = splitContent[6].Trim();
                            cmd.Parameters.Add("P_ADOPTION_MONTH", OracleDbType.Varchar2).Value = splitContent[7].Trim();
                            cmd.Parameters.Add("P_ADOPTION_YEAR", OracleDbType.Varchar2).Value = splitContent[8].Trim();
                            cmd.Parameters.Add("P_ABOLISH_MONTH", OracleDbType.Varchar2).Value = splitContent[9].Trim();
                            cmd.Parameters.Add("P_ABOLISH_YEAR", OracleDbType.Varchar2).Value = splitContent[10].Trim();
                            cmd.Parameters.Add("P_COUNTRY_CODE1", OracleDbType.Varchar2).Value = splitContent[11].Trim();
                            cmd.Parameters.Add("P_COUNTRY_CODE2", OracleDbType.Varchar2).Value = splitContent[12].Trim();
                            cmd.Parameters.Add("P_COUNTRY_CODE3", OracleDbType.Varchar2).Value = splitContent[13].Trim();
                            cmd.Parameters.Add("P_COUNTRY_CODE4", OracleDbType.Varchar2).Value = splitContent[14].Trim();
                            cmd.Parameters.Add("P_COUNTRY_CODE5", OracleDbType.Varchar2).Value = splitContent[15].Trim();
                            cmd.Parameters.Add("P_COUNTRY_CODE6", OracleDbType.Varchar2).Value = splitContent[16].Trim();
                            cmd.Parameters.Add("P_USAGE", OracleDbType.Varchar2).Value = splitContent[17].Trim();
                            cmd.Parameters.Add("P_PART_SPECIFICATION", OracleDbType.Varchar2).Value = splitContent[18].Trim();
                            cmd.Parameters.Add("P_PART_SPECIFICATION1", OracleDbType.Varchar2).Value = splitContent[19].Trim();
                            cmd.Parameters.Add("P_PART_SPECIFICATION2", OracleDbType.Varchar2).Value = splitContent[20].Trim();
                            cmd.Parameters.Add("P_PART_SPECIFICATION3", OracleDbType.Varchar2).Value = splitContent[21].Trim();
                            cmd.Parameters.Add("P_PART_SPECIFICATION4", OracleDbType.Varchar2).Value = splitContent[22].Trim();
                            cmd.Parameters.Add("P_FORMER_ICA_CONDITION", OracleDbType.Varchar2).Value = splitContent[23].Trim();
                            cmd.Parameters.Add("P_FORMER_ICA_FORMER_PART", OracleDbType.Varchar2).Value = splitContent[24].Trim();
                            cmd.Parameters.Add("P_NEW_ICA_CONDITION", OracleDbType.Varchar2).Value = splitContent[25].Trim();
                            cmd.Parameters.Add("P_NEW_ICA_FORMER_PART_NUMBER", OracleDbType.Varchar2).Value = splitContent[26].Trim();
                            cmd.Parameters.Add("P_ORIGINAL_MODEL_SERIES_CODE", OracleDbType.Varchar2).Value = splitContent[27].Trim();
                            cmd.Parameters.Add("P_S_CODE", OracleDbType.Varchar2).Value = splitContent[28].Trim();
                            cmd.Parameters.Add("P_QUANTITY", OracleDbType.Varchar2).Value = splitContent[29].Trim();
                            cmd.Parameters.Add("P_UNIT_PACKAGE", OracleDbType.Varchar2).Value = splitContent[30].Trim();
                            cmd.Parameters.Add("P_DESCRIPTION_CODE", OracleDbType.Varchar2).Value = splitContent[31].Trim();
                            cmd.Parameters.Add("P_LIST_PRICE", OracleDbType.Varchar2).Value = splitContent[32].Trim();
                            cmd.Parameters.Add("P_PART_NAME_ENGLISH", OracleDbType.Varchar2).Value = splitContent[33].Replace("\"", "");
                            cmd.Parameters.Add("P_CATALOG_MODEL", OracleDbType.Varchar2).Value = splitContent[34].Trim();
                            cmd.Parameters.Add("P_CHASSY_NO_CODE", OracleDbType.Varchar2).Value = splitContent[35].Trim();
                            cmd.Parameters.Add("P_PART_LETTER_NUMBER", OracleDbType.Varchar2).Value = splitContent[36].Trim();
                            cmd.Parameters.Add("P_REMARKS", OracleDbType.Varchar2).Value = splitContent[37].Trim();
                            cmd.Parameters.Add("P_INFINITY_SYMBOL", OracleDbType.Varchar2).Value = splitContent[38].Trim();
                            cmd.Parameters.Add("P_TRIM_COLOR1", OracleDbType.Varchar2).Value = splitContent[39].Trim();
                            cmd.Parameters.Add("P_TRIM_COLOR2", OracleDbType.Varchar2).Value = splitContent[40].Trim();
                            cmd.Parameters.Add("P_TRIM_COLOR3", OracleDbType.Varchar2).Value = splitContent[41].Trim();
                            cmd.Parameters.Add("P_TRIM_COLOR4", OracleDbType.Varchar2).Value = splitContent[42].Trim();
                            cmd.Parameters.Add("P_TRIM_COLOR5", OracleDbType.Varchar2).Value = splitContent[43].Trim();
                            cmd.Parameters.Add("P_TRIM_COLOR6", OracleDbType.Varchar2).Value = splitContent[44].Trim();
                            cmd.Parameters.Add("P_TRIM_COLOR7", OracleDbType.Varchar2).Value = splitContent[45].Trim();
                            cmd.Parameters.Add("P_TRIM_COLOR8", OracleDbType.Varchar2).Value = splitContent[46].Trim();
                            cmd.Parameters.Add("P_TRIM_COLOR9", OracleDbType.Varchar2).Value = splitContent[47].Trim();
                            cmd.Parameters.Add("P_TRIM_COLOR10", OracleDbType.Varchar2).Value = splitContent[48].Trim();
                            cmd.Parameters.Add("P_BODY_COLOR1", OracleDbType.Varchar2).Value = splitContent[49].Trim();
                            cmd.Parameters.Add("P_BODY_COLOR2", OracleDbType.Varchar2).Value = splitContent[50].Trim();
                            cmd.Parameters.Add("P_BODY_COLOR3", OracleDbType.Varchar2).Value = splitContent[51].Trim();
                            cmd.Parameters.Add("P_BODY_COLOR4", OracleDbType.Varchar2).Value = splitContent[52].Trim();
                            cmd.Parameters.Add("P_BODY_COLOR5", OracleDbType.Varchar2).Value = splitContent[53].Trim();
                            cmd.Parameters.Add("P_BODY_COLOR6", OracleDbType.Varchar2).Value = splitContent[54].Trim();
                            cmd.Parameters.Add("P_BODY_COLOR7", OracleDbType.Varchar2).Value = splitContent[55].Trim();
                            cmd.Parameters.Add("P_BODY_COLOR8", OracleDbType.Varchar2).Value = splitContent[56].Trim();
                            cmd.Parameters.Add("P_BODY_COLOR9", OracleDbType.Varchar2).Value = splitContent[57].Trim();
                            cmd.Parameters.Add("P_BODY_COLOR10", OracleDbType.Varchar2).Value = splitContent[58].Trim();
                            cmd.Parameters.Add("P_BODY_COLOR11", OracleDbType.Varchar2).Value = splitContent[59].Trim();
                            cmd.Parameters.Add("P_BODY_COLOR12", OracleDbType.Varchar2).Value = splitContent[60].Trim();
                            cmd.Parameters.Add("P_BODY_COLOR13", OracleDbType.Varchar2).Value = splitContent[61].Trim();
                            cmd.Parameters.Add("P_BODY_COLOR14", OracleDbType.Varchar2).Value = splitContent[62].Trim();
                            cmd.Parameters.Add("P_BODY_COLOR15", OracleDbType.Varchar2).Value = splitContent[63].Trim();
                            cmd.Parameters.Add("P_BODY_COLOR16", OracleDbType.Varchar2).Value = splitContent[64].Trim();
                            cmd.Parameters.Add("P_BODY_COLOR17", OracleDbType.Varchar2).Value = splitContent[65].Trim();
                            cmd.Parameters.Add("P_BODY_COLOR18", OracleDbType.Varchar2).Value = splitContent[66].Trim();
                            cmd.Parameters.Add("P_BODY_COLOR19", OracleDbType.Varchar2).Value = splitContent[67].Trim();
                            cmd.Parameters.Add("P_BODY_COLOR20", OracleDbType.Varchar2).Value = splitContent[68].Trim();
                            cmd.Parameters.Add("P_CAR_PRODUCTION_LOCATION", OracleDbType.Varchar2).Value = splitContent[69].Trim();
                            cmd.Parameters.Add("P_UNIT_PRODUCTION_LOCATION", OracleDbType.Varchar2).Value = splitContent[70].Trim();
                            cmd.Parameters.Add("P_SOURCING_CODE", OracleDbType.Varchar2).Value = splitContent[71].Trim();
                            cmd.Parameters.Add("P_MODEL_SERIES_CODE_CAR", OracleDbType.Varchar2).Value = splitContent[72].Trim();
                            cmd.Parameters.Add("P_MODEL_YEAR", OracleDbType.Varchar2).Value = splitContent[73].Trim();
                            cmd.Parameters.Add("P_MODEL_CODE_13_DIGIT", OracleDbType.Varchar2).Value = splitContent[74].Trim();
                            cmd.Parameters.Add("P_PROCUREMENT_ADDR_LOC_CODE", OracleDbType.Varchar2).Value = splitContent[75].Trim();
                            cmd.Parameters.Add("P_PROCUREMENT_HUB_ADDR_LOC", OracleDbType.Varchar2).Value = splitContent[76].Trim();
                            cmd.Parameters.Add("P_SUPPLY_HUB_ADDR1_LOC_CODE", OracleDbType.Varchar2).Value = splitContent[77].Trim();
                            cmd.Parameters.Add("P_SUPPLY_HUB_ADDR2_LOC_CODE", OracleDbType.Varchar2).Value = splitContent[78].Trim();
                            cmd.Parameters.Add("P_SUPPLY_HUB_ADDR3_LOC_CODE", OracleDbType.Varchar2).Value = splitContent[79].Trim();
                            cmd.Parameters.Add("P_SUPPLY_HUB_ADDR4_LOC_CODE", OracleDbType.Varchar2).Value = splitContent[80].Trim();
                            cmd.Parameters.Add("P_SALES_ADDR_COUNTRY_CODE", OracleDbType.Varchar2).Value = splitContent[81].Trim();
                            cmd.Parameters.Add("P_REINVOICE_COUNTRY_CODE", OracleDbType.Varchar2).Value = splitContent[82].Trim();

                            cmd = oOracledb.SqlQueryReturnValue(cmd);

                            //if(cmd.Parameters["P_STATUS"].Value.ToString() == "OK")
                            //{
                            itemNo++;
                            //}
                        }
                    }
                    return Ok(gFileName);
                }
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message + ' ' + itemNo, "SNSE145_EXTRACT_FILE");
                throw new HttpResponseException(HttpStatusCode.NotAcceptable);
            }

            throw new Exception("Something wrong");

        }
        #endregion SNSE145

        #region SNPE143

        [HttpPost, Route("SNSE143_UploadFile")]
        [Authorize]
        public async Task<IHttpActionResult> SNSE143_UploadFile(HttpRequestMessage request)
        {
            if (!request.Content.IsMimeMultipartContent())
            {
                throw new HttpResponseException(HttpStatusCode.UnsupportedMediaType);
            }


            try
            {
                var reqParts = await request.Content.ReadAsMultipartAsync();
                foreach (var file in reqParts.Contents)
                {
                    var timestamp = DateTime.Now.ToString("yyyyMMdd");
                    var buffer = await file.ReadAsByteArrayAsync();
                    string gFileName = file.Headers.ContentDisposition.FileName.Trim('\"');
                    //string name = Path.GetFileNameWithoutExtension(gFileName);
                    //string ext = Path.GetExtension(gFileName);



                    try
                    {
                        OracleCommand cmd = new OracleCommand();
                        cmd.CommandText = "SP_NP_SNSE143_F2_CHK_FILE_NAME";
                        cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                        cmd.Parameters.Add("P_FILE_NAME", OracleDbType.Varchar2).Value = gFileName;
                        cmd.Parameters.Add("C_RETURN_MSG", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;

                        oOracledb.SqlQueryReturnValue(cmd);


                        string STATUS_MSG = cmd.Parameters["C_RETURN_MSG"].Value.ToString();

                        if (STATUS_MSG == "DUPLICATE")
                        {
                            return Ok(STATUS_MSG);
                        }


                    }
                    catch (Exception ex)
                    {
                        oOracledb.WriteLog("ErrorHandle", ex.Message, "SP_NP_SNSE143_F2_CHK_FILE_NAME");
                        return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
                        //throw new HttpResponseException(HttpStatusCode.InternalServerError);
                    }






                    //string gFileNameExtend = String.Concat(gFileName);
                    //string gFileNameExtend = String.Concat(name, '_', timestamp, ext);
                    string gRootPath = ConfigurationManager.AppSettings["RootSharedPath"].ToString();
                    string gUploadLication = System.IO.Path.Combine(gRootPath, "tempFile/SNSE143");
                    string gFileLocation = System.IO.Path.Combine(gUploadLication, gFileName);

                    DataService.SaveBytesToFileByLocation(buffer
                                                    , gUploadLication
                                                    , gFileLocation);

                    try
                    {
                        OracleCommand cmd = new OracleCommand();
                        cmd.CommandText = "PK_NP_SNSE143_F2.REMOVE_PFPM";
                        cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                        cmd.Parameters.Add("P_FILE_NAME", OracleDbType.Varchar2).Value = gFileName;

                        oOracledb.SqlQueryReturnValue(cmd);
                    }
                    catch (Exception ex)
                    {
                        oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE143_UploadFile_REMOVE_PFPM");
                        return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
                    }

                    //Begin read file
                    Stream stream = new MemoryStream(buffer);

                    using (StreamReader reader = new StreamReader(stream, Encoding.UTF8))
                    {
                        string line;
                        int itemNo = 0;
                        while ((line = reader.ReadLine()) != null)
                        {
                            if (itemNo == 0)
                            {
                                itemNo++;
                                continue;
                            }  // Skip first line

                            string sep = "\t";
                            string[] splitContent = line.Split(sep.ToCharArray());
                            var lenght = splitContent.Length;
                            //read line from memorystream
                            OracleCommand cmd = new OracleCommand();
                            cmd.CommandText = "PK_NP_SNSE143_F2.EXTRACT_PFPM_2";
                            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                            cmd.Parameters.Add("P_FILE_NAME", OracleDbType.Varchar2).Value = gFileName;
                            cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Decimal).Value = itemNo;
                            cmd.Parameters.Add("P_TEXT_DATA", OracleDbType.Varchar2).Value = line;
                            cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                            cmd.Parameters.Add("P_MODEL_SERIES_CODE", OracleDbType.Varchar2).Value = splitContent[0];
                            cmd.Parameters.Add("P_BLOCK", OracleDbType.Varchar2).Value = splitContent[1];
                            cmd.Parameters.Add("P_PART_NUMBER", OracleDbType.Varchar2).Value = splitContent[2];
                            cmd.Parameters.Add("P_SALES_CODE", OracleDbType.Varchar2).Value = splitContent[3];
                            cmd.Parameters.Add("P_SALES_CONDITIONS", OracleDbType.Varchar2).Value = splitContent[4];
                            cmd.Parameters.Add("P_PARENT_PART_NUMBER", OracleDbType.Varchar2).Value = splitContent[5];
                            cmd.Parameters.Add("P_MODEL_INFO_JPN_ID", OracleDbType.Varchar2).Value = splitContent[6];
                            cmd.Parameters.Add("P_MODEL_INFO_CATALOG_MODEL", OracleDbType.Varchar2).Value = splitContent[7];
                            cmd.Parameters.Add("P_MODEL_INFO_MODEL_YEAR", OracleDbType.Varchar2).Value = splitContent[8];
                            cmd.Parameters.Add("P_MODEL_INFO_UNIT_SYMBOL", OracleDbType.Varchar2).Value = splitContent[9];
                            cmd.Parameters.Add("P_PATTERN_NUMBER", OracleDbType.Varchar2).Value = splitContent[10];
                            cmd.Parameters.Add("P_SALES_CHANNEL_N", OracleDbType.Varchar2).Value = splitContent[11];
                            cmd.Parameters.Add("P_SALES_CHANNEL_M", OracleDbType.Varchar2).Value = splitContent[12];
                            cmd.Parameters.Add("P_SALES_CHANNEL_S", OracleDbType.Varchar2).Value = splitContent[13];
                            cmd.Parameters.Add("P_SALES_CHANNEL_P", OracleDbType.Varchar2).Value = splitContent[14];
                            cmd.Parameters.Add("P_PART_CODE", OracleDbType.Varchar2).Value = splitContent[15];
                            cmd.Parameters.Add("P_ENGLISH_PART_NAME", OracleDbType.Varchar2).Value = splitContent[16];
                            cmd.Parameters.Add("P_JAPANESE_PART_NAME", OracleDbType.Varchar2).Value = splitContent[17];
                            cmd.Parameters.Add("P_DESCRIPTION_CODE", OracleDbType.Varchar2).Value = splitContent[18];
                            cmd.Parameters.Add("P_SQUARES", OracleDbType.Varchar2).Value = splitContent[19];
                            cmd.Parameters.Add("P_PRE_LIST_ITEM", OracleDbType.Varchar2).Value = splitContent[20];
                            cmd.Parameters.Add("P_SQUARES_A", OracleDbType.Varchar2).Value = splitContent[21];
                            cmd.Parameters.Add("P_SQUARES_B", OracleDbType.Varchar2).Value = splitContent[22];
                            cmd.Parameters.Add("P_NEW_ITEM_SYMBOL", OracleDbType.Varchar2).Value = splitContent[23];
                            cmd.Parameters.Add("P_ADOPTION_YEAR_MONTH", OracleDbType.Varchar2).Value = splitContent[24];
                            cmd.Parameters.Add("P_ABOLISH_YEAR_MONTH", OracleDbType.Varchar2).Value = splitContent[25];
                            cmd.Parameters.Add("P_JAPAN_PARTS_LETTER", OracleDbType.Varchar2).Value = splitContent[26];
                            cmd.Parameters.Add("P_SPPS", OracleDbType.Varchar2).Value = splitContent[27];
                            cmd.Parameters.Add("P_DESIGN_NOTE", OracleDbType.Varchar2).Value = splitContent[28];
                            cmd.Parameters.Add("P_QUANTITY", OracleDbType.Varchar2).Value = splitContent[29];
                            cmd.Parameters.Add("P_DRAWING_DRAWING_NUMBER", OracleDbType.Varchar2).Value = splitContent[30];
                            cmd.Parameters.Add("P_DRAWING_DRAWING_CHANGE", OracleDbType.Varchar2).Value = splitContent[31];
                            cmd.Parameters.Add("P_DESIGN_PART_NAME", OracleDbType.Varchar2).Value = splitContent[32];
                            cmd.Parameters.Add("P_PARTS_SPECIFICATION", OracleDbType.Varchar2).Value = splitContent[33];
                            cmd.Parameters.Add("P_FORMER_ALTERNATE", OracleDbType.Varchar2).Value = splitContent[34];
                            cmd.Parameters.Add("P_ICA_NIO_1", OracleDbType.Varchar2).Value = splitContent[35];
                            cmd.Parameters.Add("P_ICA_OIN_1", OracleDbType.Varchar2).Value = splitContent[36];
                            cmd.Parameters.Add("P_COEFF_JP_P", OracleDbType.Varchar2).Value = splitContent[37];
                            cmd.Parameters.Add("P_COEFF_OTHERS_P", OracleDbType.Varchar2).Value = splitContent[38];
                            cmd.Parameters.Add("P_VITAL_PARTS", OracleDbType.Varchar2).Value = splitContent[39];
                            cmd.Parameters.Add("P_SPC_BUKAN", OracleDbType.Varchar2).Value = splitContent[40];
                            cmd.Parameters.Add("P_TRIM_COLOR_1", OracleDbType.Varchar2).Value = splitContent[41];
                            cmd.Parameters.Add("P_TRIM_COLOR_2", OracleDbType.Varchar2).Value = splitContent[42];
                            cmd.Parameters.Add("P_TRIM_COLOR_3", OracleDbType.Varchar2).Value = splitContent[43];
                            cmd.Parameters.Add("P_TRIM_COLOR_4", OracleDbType.Varchar2).Value = splitContent[44];
                            cmd.Parameters.Add("P_TRIM_COLOR_5", OracleDbType.Varchar2).Value = splitContent[45];
                            cmd.Parameters.Add("P_TRIM_COLOR_6", OracleDbType.Varchar2).Value = splitContent[46];
                            cmd.Parameters.Add("P_TRIM_COLOR_7", OracleDbType.Varchar2).Value = splitContent[47];
                            cmd.Parameters.Add("P_TRIM_COLOR_8", OracleDbType.Varchar2).Value = splitContent[48];
                            cmd.Parameters.Add("P_TRIM_COLOR_9", OracleDbType.Varchar2).Value = splitContent[49];
                            cmd.Parameters.Add("P_TRIM_COLOR_10", OracleDbType.Varchar2).Value = splitContent[50];
                            cmd.Parameters.Add("GENESIS_SOURCING_CODE", OracleDbType.Varchar2).Value = splitContent[51];
                            cmd.Parameters.Add("GENESIS_RESP_GROUP_CODE", OracleDbType.Varchar2).Value = splitContent[52];
                            cmd.Parameters.Add("ESTABLISHMENT_LOCATION", OracleDbType.Varchar2).Value = splitContent[53];
                            cmd.Parameters.Add("CAR_PRODUCTION_LOCATION", OracleDbType.Varchar2).Value = splitContent[54];
                            cmd.Parameters.Add("UNIT_PRODUCTION_LOCATION", OracleDbType.Varchar2).Value = splitContent[55];
                            cmd.Parameters.Add("UNIT_PACKAGE_P", OracleDbType.Varchar2).Value = splitContent[56];
                            cmd.Parameters.Add("NET_WEIGHT_P", OracleDbType.Varchar2).Value = splitContent[57];
                            cmd.Parameters.Add("NEW_ALTERNATE", OracleDbType.Varchar2).Value = splitContent[58];
                            cmd.Parameters.Add("ICA_OIN_2", OracleDbType.Varchar2).Value = splitContent[59];
                            cmd.Parameters.Add("ICA_NIO_2", OracleDbType.Varchar2).Value = splitContent[60];
                            cmd.Parameters.Add("DESIGN_PART_NUMBER", OracleDbType.Varchar2).Value = splitContent[61];
                            cmd.Parameters.Add("AHQ_CODE", OracleDbType.Varchar2).Value = splitContent[62];
                            cmd.Parameters.Add("MARKET_CODE_DM", OracleDbType.Varchar2).Value = splitContent[63];
                            cmd.Parameters.Add("MARKET_CODE_EL", OracleDbType.Varchar2).Value = splitContent[64];
                            cmd.Parameters.Add("MARKET_CODE_ER", OracleDbType.Varchar2).Value = splitContent[65];
                            cmd.Parameters.Add("MARKET_CODE_GL", OracleDbType.Varchar2).Value = splitContent[66];
                            cmd.Parameters.Add("MARKET_CODE_GR", OracleDbType.Varchar2).Value = splitContent[67];
                            cmd.Parameters.Add("MARKET_CODE_US", OracleDbType.Varchar2).Value = splitContent[68];
                            cmd.Parameters.Add("MARKET_CODE_CA", OracleDbType.Varchar2).Value = splitContent[69];
                            cmd.Parameters.Add("MARKET_CODE_AR", OracleDbType.Varchar2).Value = splitContent[70];
                            cmd.Parameters.Add("ORIGINAL_PART_NUMBER", OracleDbType.Varchar2).Value = splitContent[71];
                            cmd.Parameters.Add("ICA_ORIGINAL", OracleDbType.Varchar2).Value = splitContent[72];
                            cmd.Parameters.Add("VENDER_CODE", OracleDbType.Varchar2).Value = splitContent[73];
                            cmd.Parameters.Add("PLANT_ADOPT_STAT", OracleDbType.Varchar2).Value = splitContent[74];
                            cmd.Parameters.Add("SECTION_NUMBER", OracleDbType.Varchar2).Value = splitContent[75];
                            cmd.Parameters.Add("L_SERIES_CODE_EUR", OracleDbType.Varchar2).Value = splitContent[76];
                            cmd.Parameters.Add("ALTERNATE_SOURCING_CODE", OracleDbType.Varchar2).Value = splitContent[77];
                            cmd.Parameters.Add("GSC", OracleDbType.Varchar2).Value = splitContent[78];
                            cmd.Parameters.Add("PROCUREMENT_HUB", OracleDbType.Varchar2).Value = splitContent[79];
                            cmd.Parameters.Add("DESTINATION", OracleDbType.Varchar2).Value = splitContent[80];
                            cmd.Parameters.Add("FILLER", OracleDbType.Varchar2).Value = splitContent[81];

                            cmd = oOracledb.SqlQueryReturnValue(cmd);

                            //if(cmd.Parameters["P_STATUS"].Value.ToString() == "OK")
                            //{
                            itemNo++;
                            //}
                        }
                    }
                    return Ok(gFileName);
                }
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE143_UploadFile");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

            throw new Exception("Something wrong");

        }
        #endregion SNPE143


        #region SNM115P
        [HttpGet, Route("SNM115P_CampaignID/{campaign}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM115P_CampaignID(string campaign, int noofshowrec)
        {
            DataSet ds = new DataSet();

            campaign = ismoUtil.escapeCharacter(campaign);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM115P_CAMPAIGN_ID";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_CAMPAIGN_ID", OracleDbType.Varchar2).Value = campaign.ToUpper() == "NULL" ? (object)DBNull.Value : campaign;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNM115P_CampaignList")]
        [Authorize]
        public IHttpActionResult SNM115P_CampaignList([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM115P_CAMPAIGN_LIST";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_CAMPAIGN_ID", OracleDbType.Varchar2).Value = param.CAMPAIGN_ID;
            cmd.Parameters.Add("P_STATUS", OracleDbType.Varchar2).Value = param.STATUS;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }


        [HttpPost, Route("SNM115P_Active")]
        [Authorize]
        public IHttpActionResult SNM115P_Active([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM115P_ACTIVE_CAMPAIGN";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_CAMPAIGN_ID", OracleDbType.Varchar2).Value = param.CAMPAIGN_ID;
            cmd.Parameters.Add("P_DISABLE_DATE", OracleDbType.Varchar2).Value = param.DISABLE_DATE.ToString("yyyy-MM-dd");
            cmd.Parameters.Add("P_DISCOUNT_TYPE", OracleDbType.Varchar2).Value = param.DISCOUNT_TYPE;
            cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
            cmd.Parameters.Add("R_ORDER_MSG", OracleDbType.Varchar2, 500).Direction = ParameterDirection.Output;
            cmd.Parameters.Add("R_CUST_FACTOR_MSG", OracleDbType.Varchar2, 500).Direction = ParameterDirection.Output;
            cmd.Parameters.Add("R_DISCOUNT_DETAIL_MSG", OracleDbType.Varchar2, 500).Direction = ParameterDirection.Output;
            cmd.Parameters.Add("R_DISABLE_DATE_MSG", OracleDbType.Varchar2, 500).Direction = ParameterDirection.Output;
            cmd.Parameters.Add("R_ERROR_FLAG", OracleDbType.Varchar2, 500).Direction = ParameterDirection.Output;
            cmd.Parameters.Add("R_FLAG_RESULT", OracleDbType.Varchar2, 500).Direction = ParameterDirection.Output;

            cmd = oOracledb.SqlQueryReturnValue(cmd);

            string ORDER_MSG = cmd.Parameters["R_ORDER_MSG"].Value.ToString();
            string CUST_MSG = cmd.Parameters["R_CUST_FACTOR_MSG"].Value.ToString();
            string DISCOUNT_MSG = cmd.Parameters["R_DISCOUNT_DETAIL_MSG"].Value.ToString();
            string DISABLE_MSG = cmd.Parameters["R_DISABLE_DATE_MSG"].Value.ToString() == "null" ? "" : cmd.Parameters["R_DISABLE_DATE_MSG"].Value.ToString();
            string RESULT_MSG = cmd.Parameters["R_ERROR_FLAG"].Value.ToString();
            string FLAG_RESULT = cmd.Parameters["R_FLAG_RESULT"].Value.ToString();

            var returnObject = Json(new
            {
                N_ORDER_MSG = ORDER_MSG,
                N_CUST_MSG = CUST_MSG,
                N_DISCOUNT_MSG = DISCOUNT_MSG,
                N_DISABLE_MSG = DISABLE_MSG,
                N_RESULT_MSG = RESULT_MSG,
                N_FLAG_RESULT = FLAG_RESULT
            });

            return returnObject;
        }


        [HttpPost, Route("SNM115P_Inactive")]
        [Authorize]
        public IHttpActionResult SNM115P_Inactive([FromBody] dynamic[] data)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            foreach (dynamic d in data)
            {
                try
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNM115P_INACTIVE";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CAMPAIGN_ID", OracleDbType.Varchar2).Value = d["CAMPAIGN_ID"].ToString();
                    cmd.Parameters.Add("P_STATUS", OracleDbType.Varchar2).Value = d["STATUS"].ToString();
                    cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("R_MSG", OracleDbType.Varchar2, 500).Direction = ParameterDirection.Output;
                    //cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = d["LAST_UPD_PGM"].ToString();
                    //cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                    cmd = oOracledb.SqlQueryReturnValue(cmd);
                }
                catch (Exception ex)
                {
                    //--- Roll back ---
                    string errString = d["CAMPAIGN_ID"].ToString() + ex.Message.ToString();
                    return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, errString));
                }
            }
            return Ok("DONE");
        }


        [HttpPost, Route("SNM115P_DownLoad")]
        public HttpResponseMessage SNM115P_DownLoad([FromBody] dynamic param)
        {
            DataTable dt = new DataTable();
            DataSet ds = new DataSet();
            DataTable dt2 = new DataTable();

            string x = string.Empty;
            string filename = "SNM115P.slk";
            string floderLocation = "SNM115P";
            OracleCommand cmd = new OracleCommand();
            //dt = DTFM.ToDatatable<DataModel.SNPE044>(dModel);
            dt.Columns.Add("colum1");
            dt.Rows.Add("ID;PO2C");
            dt.Rows.Add("B");

            if (param.DISCOUNT_TYPE == 'D' || param.DISCOUNT_TYPE == 'P' || param.DISCOUNT_TYPE == 'V')
            {


                ///////////////////////CAMPAIGN_HEADER///////////////////////////////////////

                dt.Rows.Add("C;X1;Y1;K\" * **** Campaign Header Info. *****\"");
                dt.Rows.Add("C;X1;Y2;K\"CAMPAIGN_ID\"");
                dt.Rows.Add("C;X2;Y2;K\"CAMPAIGN_NAME\"");
                dt.Rows.Add("C;X3;Y2;K\"EFFECTIVE_DATE\"");
                dt.Rows.Add("C;X4;Y2;K\"DISABLE_DATE\"");
                dt.Rows.Add("C;X5;Y2;K\"STATUS\"");

                dt.Rows.Add("C;X1;Y3;K" + '"' + param.CAMPAIGN_ID + '"');
                dt.Rows.Add("C;X2;Y3;K" + '"' + param.CAMPAIGN_NAME + '"');
                dt.Rows.Add("C;X3;Y3;K" + '"' + param.EFFECTIVE_DATE.ToString("dd-MMM-yyyy") + '"');
                dt.Rows.Add("C;X4;Y3;K" + '"' + param.DISABLE_DATE.ToString("dd-MMM-yyyy") + '"');
                dt.Rows.Add("C;X5;Y3;K" + '"' + param.STATUS_SHOW + '"');


                ///////////////////////END CAMPAIGN_HEADER///////////////////////////////////////
                ///


                //////////////////////campaign customers factor//////////////////////////////
                ///


                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM115P_CUSTOMER_FACTOR";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CAMPAIGN_ID", OracleDbType.Varchar2).Value = param.CAMPAIGN_ID;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                dt2 = ds.Tables[0];

                dt.Rows.Add("C;X1;Y6;K\" ***** Campaign Customer Factor Info. *****\"");

                dt.Rows.Add("C;X1;Y7;K\"SALES_CHANNEL\"");
                dt.Rows.Add("C;X2;Y7;K\"CHANNEL_NAME\"");
                dt.Rows.Add("C;X3;Y7;K\"CUSTOMER_CODE\"");
                dt.Rows.Add("C;X4;Y7;K\"CUSTOMER_NAME\"");
                dt.Rows.Add("C;X5;Y7;K\"BRANCH_CODE\"");
                dt.Rows.Add("C;X6;Y7;K\"BRANCH_NAME\"");

                if (dt2.Rows.Count > 0)
                {
                    int countCust = 8;
                    foreach (DataRow d in dt2.Rows)
                    {
                        dt.Rows.Add("C;X1;Y" + countCust + ";K" + '"' + d["SALES_CHANNEL"] + '"');
                        dt.Rows.Add("C;X2;Y" + countCust + ";K" + '"' + d["CHANNEL_NAME"] + '"');
                        dt.Rows.Add("C;X3;Y" + countCust + ";K" + '"' + d["CUSTOMER_CODE"] + '"');
                        dt.Rows.Add("C;X4;Y" + countCust + ";K" + '"' + d["CUSTOMER_NAME"] + '"');
                        dt.Rows.Add("C;X5;Y" + countCust + ";K" + '"' + d["BRANCH_CODE"] + '"');
                        dt.Rows.Add("C;X6;Y" + countCust + ";K" + '"' + d["BRANCH_NAME"] + '"');
                        countCust++;
                    }
                }
                else
                {
                    dt.Rows.Add("C;X1;Y8;K\"There is no customer factor setting.\"");
                }

                //////////////////////End campaign customers factor//////////////////////////////


                /////////////////////campaign order factor///////////////////////////////////
                ///
                int CustomerRow = dt2.Rows.Count + 10;
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM115P_ORDER_FACTOR";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CAMPAIGN_ID", OracleDbType.Varchar2).Value = param.CAMPAIGN_ID;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                dt2 = ds.Tables[0];


                dt.Rows.Add("C;X1;Y" + CustomerRow + ";K\" ***** Campaign Order Factor Info. *****\"");

                int rowHeadOrder = CustomerRow + 1;
                dt.Rows.Add("C;X1;Y" + rowHeadOrder + ";K\"ORDER_CLASS\"");
                dt.Rows.Add("C;X2;Y" + rowHeadOrder + ";K\"ORDER_DESCRIPT\"");
                dt.Rows.Add("C;X3;Y" + rowHeadOrder + ";K\"ORDER_PREFIX\"");


                if (dt2.Rows.Count > 0)
                {
                    int countOrder = CustomerRow + 2;
                    foreach (DataRow d in dt2.Rows)
                    {
                        dt.Rows.Add("C;X1;Y" + countOrder + ";K" + '"' + d["ORDER_CLASS"] + '"');
                        dt.Rows.Add("C;X2;Y" + countOrder + ";K" + '"' + d["ORDER_DESCRIPT"] + '"');
                        dt.Rows.Add("C;X3;Y" + countOrder + ";K" + '"' + d["ORDER_PREFIX"] + '"');

                        countOrder++;
                    }
                }
                else
                {
                    int rowFooterOrder = CustomerRow + 2;
                    dt.Rows.Add("C;X6;Y" + rowFooterOrder + "; K\"There is no Order factor setting.\"");
                }


                ////////////////////////////////End campaign order factor/////////////////////////////////////////////
                ///


                ///////////////////////////////campaign discount factor////////////////////////////////////////////////


                int OrderRow = CustomerRow + dt2.Rows.Count + 3;
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM115P_DISCOUNT_FACTOR";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CAMPAIGN_ID", OracleDbType.Varchar2).Value = param.CAMPAIGN_ID;
                cmd.Parameters.Add("P_DISCOUNT_TYPE", OracleDbType.Varchar2).Value = param.DISCOUNT_TYPE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                dt2 = ds.Tables[0];


                dt.Rows.Add("C;X1;Y" + OrderRow + ";K\" ***** Campaign Discount Factor Info. *****\"");
                int h, f;
                h = OrderRow + 1;
                f = OrderRow + 2;
                if (param.DISCOUNT_TYPE == 'D')
                {

                    dt.Rows.Add("C;X1;Y" + h + ";K\"PART_NO\"");
                    dt.Rows.Add("C;X2;Y" + h + ";K\"PART_NAME_EN\"");
                    dt.Rows.Add("C;X3;Y" + h + ";K\"DISCOUNT_RATE1\"");
                    dt.Rows.Add("C;X4;Y" + h + ";K\"DISCOUNT_RATE2\"");
                    dt.Rows.Add("C;X5;Y" + h + ";K\"DISCOUNT_RATE3\"");
                    dt.Rows.Add("C;X6;Y" + h + ";K\"NET_PRICE\"");

                    if (dt2.Rows.Count > 0)
                    {
                        int countOrder = OrderRow + 2;
                        foreach (DataRow d in dt2.Rows)
                        {
                            dt.Rows.Add("C;X1;Y" + countOrder + ";K" + '"' + d["PART_NO"] + '"');
                            dt.Rows.Add("C;X2;Y" + countOrder + ";K" + '"' + d["PART_NAME_EN"] + '"');
                            dt.Rows.Add("C;X3;Y" + countOrder + ";K" + '"' + d["DISCOUNT_RATE1"] + '"');
                            dt.Rows.Add("C;X4;Y" + countOrder + ";K" + '"' + d["DISCOUNT_RATE2"] + '"');
                            dt.Rows.Add("C;X5;Y" + countOrder + ";K" + '"' + d["DISCOUNT_RATE3"] + '"');
                            dt.Rows.Add("C;X6;Y" + countOrder + ";K" + '"' + d["NET_PRICE"] + '"');
                            countOrder++;
                        }
                    }
                    else
                    {

                        dt.Rows.Add("C;X6;Y" + f + ";K\"\"");
                    }
                }
                else if (param.DISCOUNT_TYPE == 'P')
                {
                    dt.Rows.Add("C;X1;Y" + h + ";K\"PACKAGE_ID\"");
                    dt.Rows.Add("C;X2;Y" + h + ";K\"PACKAGE_NAME\"");
                    dt.Rows.Add("C;X3;Y" + h + ";K\"PART_NO\"");
                    dt.Rows.Add("C;X4;Y" + h + ";K\"PART_NAME_EN\"");
                    dt.Rows.Add("C;X5;Y" + h + ";K\"DISCOUNT_RATE1\"");
                    dt.Rows.Add("C;X6;Y" + h + ";K\"DISCOUNT_RATE2\"");
                    dt.Rows.Add("C;X7;Y" + h + ";K\"DISCOUNT_RATE3\"");
                    dt.Rows.Add("C;X8;Y" + h + ";K\"NET_PRICE\"");

                    if (dt2.Rows.Count > 0)
                    {
                        int countOrder = OrderRow + 2;
                        foreach (DataRow d in dt2.Rows)
                        {
                            dt.Rows.Add("C;X1;Y" + countOrder + ";K" + '"' + d["PACKAGE_ID"] + '"');
                            dt.Rows.Add("C;X2;Y" + countOrder + ";K" + '"' + d["PACKAGE_NAME"] + '"');
                            dt.Rows.Add("C;X3;Y" + countOrder + ";K" + '"' + d["PART_NO"] + '"');
                            dt.Rows.Add("C;X4;Y" + countOrder + ";K" + '"' + d["PART_NAME_EN"] + '"');
                            dt.Rows.Add("C;X5;Y" + countOrder + ";K" + '"' + d["DISCOUNT_RATE1"] + '"');
                            dt.Rows.Add("C;X6;Y" + countOrder + ";K" + '"' + d["DISCOUNT_RATE2"] + '"');
                            dt.Rows.Add("C;X7;Y" + countOrder + ";K" + '"' + d["DISCOUNT_RATE3"] + '"');
                            dt.Rows.Add("C;X8;Y" + countOrder + ";K" + '"' + d["NET_PRICE"] + '"');
                            countOrder++;
                        }
                    }
                    else
                    {
                        dt.Rows.Add("C;X6;Y" + f + ";K\"\"");
                    }


                }
                else if (param.DISCOUNT_TYPE == 'V')
                {


                    dt.Rows.Add("C;X1;Y" + h + ";K\"PART_NO\"");
                    dt.Rows.Add("C;X2;Y" + h + ";K\"PART_NAME_EN\"");
                    dt.Rows.Add("C;X3;Y" + h + ";K\"MAX_VOLUME\"");
                    dt.Rows.Add("C;X4;Y" + h + ";K\"DISCOUNT_RATE1\"");
                    dt.Rows.Add("C;X5;Y" + h + ";K\"DISCOUNT_RATE2\"");
                    dt.Rows.Add("C;X6;Y" + h + ";K\"DISCOUNT_RATE3\"");
                    dt.Rows.Add("C;X7;Y" + h + ";K\"NET_PRICE\"");

                    if (dt2.Rows.Count > 0)
                    {
                        int countOrder = OrderRow + 2;
                        foreach (DataRow d in dt2.Rows)
                        {
                            dt.Rows.Add("C;X1;Y" + countOrder + ";K" + '"' + d["PART_NO"] + '"');
                            dt.Rows.Add("C;X2;Y" + countOrder + ";K" + '"' + d["PART_NAME_EN"] + '"');
                            dt.Rows.Add("C;X3;Y" + countOrder + ";K" + '"' + d["MAX_VOLUME"] + '"');
                            dt.Rows.Add("C;X4;Y" + countOrder + ";K" + '"' + d["DISCOUNT_RATE1"] + '"');
                            dt.Rows.Add("C;X5;Y" + countOrder + ";K" + '"' + d["DISCOUNT_RATE2"] + '"');
                            dt.Rows.Add("C;X6;Y" + countOrder + ";K" + '"' + d["DISCOUNT_RATE3"] + '"');
                            dt.Rows.Add("C;X7;Y" + countOrder + ";K" + '"' + d["NET_PRICE"] + '"');
                            countOrder++;
                        }
                    }
                    else
                    {
                        dt.Rows.Add("C;X6;Y" + f + ";K\"\"");
                    }
                }


                ///////////////////////////////END campaign discount factor//////////////////////////////////////////////
                ///
            }
            else
            {
                dt.Rows.Add("C;X1;Y1;K\"\"");
            }

            dt.Rows.Add("E");

            x = DataService.WriteCSVNoColumn(dt, filename, floderLocation);
            Controllers.SystemApiController response = new Controllers.SystemApiController();
            var File = response.downloadFile(x.Substring(1), filename);
            return File;
        }

        #endregion


        #region SNM034P
        [HttpGet, Route("SNM034P_GetDiscountGroup/{discount}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM034P_GetDiscountGroup(string discount, int noofshowrec)
        {
            DataSet ds = new DataSet();

            discount = ismoUtil.escapeCharacter(discount);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM034P_GET_DISCOUNT";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_DISCOUNT", OracleDbType.Varchar2).Value = discount.ToUpper() == "NULL" ? (object)DBNull.Value : discount;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }


        [HttpGet, Route("SNM034P_GetDiscountType/{type}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM034P_GetDiscountType(string type, int noofshowrec)
        {
            DataSet ds = new DataSet();

            type = ismoUtil.escapeCharacter(type);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM034P_DISCOUNT_TYPE";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_DISCOUNT_TYPE", OracleDbType.Varchar2).Value = type.ToUpper() == "NULL" ? (object)DBNull.Value : type;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNM034P_GetDiscountList")]
        [Authorize]
        public IHttpActionResult SNM034P_GetDiscountList([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM034P_DISCOUNT_SEARCH";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_DISCOUNT_GROUP", OracleDbType.Varchar2).Value = param.DISCOUNT_GROUP;
            cmd.Parameters.Add("P_DISCOUNT_TYPE", OracleDbType.Varchar2).Value = param.DISCOUNT_TYPE;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }

        [HttpGet, Route("SNM034P_GetDetail/{discountgroup}")]
        [Authorize]
        public IHttpActionResult SNM034P_GetDetail(string discountgroup)
        {
            DataSet ds = new DataSet();



            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM034P_DISCOUNT_DETAIL";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_DISCOUNT_GROUP", OracleDbType.Varchar2).Value = discountgroup;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }
        #endregion

        #region SNM038P

        [HttpGet, Route("SNM038P_GetMainModel/{mainmodel}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM038P_GetMainModel(string mainmodel, int noofshowrec)
        {
            DataSet ds = new DataSet();

            mainmodel = ismoUtil.escapeCharacter(mainmodel);
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM038P_GET_MAIN_MODEL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_MAIN_MODEL", OracleDbType.Varchar2).Value = mainmodel.ToUpper() == "NULL" ? (object)DBNull.Value : mainmodel;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM038P_GetMainModel");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM038P_GetMainModelList")]
        [Authorize]
        public IHttpActionResult SNM038P_GetMainModelList([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM038P_GET_MODEL_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_MAIN_MODEL", OracleDbType.Varchar2).Value = param.MAIN_MODEL;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM038P_GetMainModelList");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM038P_Insert")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM038P_Insert([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM038P_INSERT";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_MAIN_MODEL", OracleDbType.Varchar2).Value = param.MAIN_MODEL;
                cmd.Parameters.Add("P_MAIN_MODEL_NAME", OracleDbType.Varchar2).Value = param.MAIN_MODEL_NAME;
                cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Int16).Value = param.ITEM_NO;
                cmd.Parameters.Add("P_CAR_MARKET_TYPE", OracleDbType.Varchar2).Value = param.CAR_MARKET_TYPE;
                cmd.Parameters.Add("P_CAR_VENDOR_TYPE", OracleDbType.Varchar2).Value = param.CAR_VENDOR_TYPE;
                cmd.Parameters.Add("P_CAR_PROD_TYPE", OracleDbType.Varchar2).Value = param.CAR_PROD_TYPE;
                cmd.Parameters.Add("P_PGM", OracleDbType.Varchar2).Value = "SNM038P";
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_IP", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM038P_Insert");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM038P_Update")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM038P_Update([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM038P_UPDATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_MAIN_MODEL", OracleDbType.Varchar2).Value = param.MAIN_MODEL;
                cmd.Parameters.Add("P_MAIN_MODEL_NAME", OracleDbType.Varchar2).Value = param.MAIN_MODEL_NAME;
                cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Int16).Value = param.ITEM_NO;
                cmd.Parameters.Add("P_CAR_MARKET_TYPE", OracleDbType.Varchar2).Value = param.CAR_MARKET_TYPE;
                cmd.Parameters.Add("P_CAR_VENDOR_TYPE", OracleDbType.Varchar2).Value = param.CAR_VENDOR_TYPE;
                cmd.Parameters.Add("P_CAR_PROD_TYPE", OracleDbType.Varchar2).Value = param.CAR_PROD_TYPE;
                cmd.Parameters.Add("P_PGM", OracleDbType.Varchar2).Value = "SNM038P";
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_IP", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM038P_Update");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }



        [HttpGet, Route("SNM038P_DELETE/{mainmodel}")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM041P_DELETE(string mainmodel)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM038P_DELETE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_MAIN_MODEL", OracleDbType.Varchar2).Value = mainmodel;
                cmd.Parameters.Add("P_PGM", OracleDbType.Varchar2).Value = "SNM038P";
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_IP", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM038P_DELETE");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        #endregion

        #region SNM105P
        [HttpPost, Route("SNM105P_Upload")]
        [Authorize]
        public async Task<IHttpActionResult> SNM105P_Upload(HttpRequestMessage request)
        {

            if (!request.Content.IsMimeMultipartContent())
            {
                throw new HttpResponseException(HttpStatusCode.UnsupportedMediaType);
            }

            var reqParts = await request.Content.ReadAsMultipartAsync();
            DataTable dt = new DataTable();
            DataSet ds = new DataSet();
            try
            {

                string fileLocation = string.Empty;

                //var name = reqParts.Contents[1].ReadAsByteArrayAsync();
                //var enname = Encoding.UTF8.GetString(name.Result, 0, name.Result.Length);

                foreach (var file in reqParts.Contents)
                {
                    var buffer = await file.ReadAsByteArrayAsync();
                    //string fileName = Path.GetFileNameWithoutExtension(file.Headers.ContentDisposition.FileName.Trim('\"'));
                    //string ext = Path.GetExtension(file.Headers.ContentDisposition.FileName.Trim('\"'));
                    string FullfileName = String.Concat("SNM105P", "-", DateTime.Now.ToString("yyyyMMddHHmm"), ".CSV");
                    string outputLocation = "~/tempFile/" + "SNM105P" + "/";
                    string outputPath = System.Web.HttpContext.Current.Server.MapPath(outputLocation);
                    fileLocation = String.Concat(outputPath, FullfileName);

                    DataService.SaveBytesToFileByLocation(buffer
                                                    , fileLocation
                                                    , fileLocation);
                }


                //ErrorList = SNM032P_InsertDataFromExcelFile(fileLocation);
                string Header = "PART_NO,NEW_BASE_PRICE,NEW_STANDARD_COST";
                dt = DataService.ConvertCSVtoDataTable(fileLocation, Header);
                dt.Columns.Add("SEQ");
                dt.Columns.Add("PART_NAME_EN");
                dt.Columns.Add("BASE_PRICE");
                dt.Columns.Add("STANDARD_COST");
                dt.Columns.Add("MSG");
                dt.Columns.Add("ERR_FLAG_PART");
                dt.Columns.Add("ERR_FLAG");


                //dt = SNM105P_Validate(dt);
                ds.Tables.Add(dt);

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM105P_Upload");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

            return Json(DTFM.convertToList(ds));

        }

        [HttpPost, Route("SNM105P_UploadExcel")]
        [Authorize]
        public async Task<IHttpActionResult> SNM105P_UploadExcel(HttpRequestMessage request)
        {
            if (!request.Content.IsMimeMultipartContent())
            {
                oOracledb.WriteLog("ErrorHandle","UnsupportedMediaType", "SNM105P_UploadExcel");
                throw new HttpResponseException(HttpStatusCode.UnsupportedMediaType);
            }
            var reqParts = await request.Content.ReadAsMultipartAsync();
            try
            {
                DataSet ds = new DataSet();
                DataTable dt = new DataTable();
                string USER_ID = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                var file = reqParts.Contents[0];
                var buffer = await file.ReadAsByteArrayAsync();
                string gFileName = file.Headers.ContentDisposition.FileName.Trim('\"');
                string name = Path.GetFileNameWithoutExtension(gFileName);
                string ext = Path.GetExtension(gFileName);
                string gFileNameExtend = String.Concat(name, '_', USER_ID, ext);
                string gRootPath = ConfigurationManager.AppSettings["RootSharedPath"].ToString();
                string gUploadLication = System.IO.Path.Combine(gRootPath, "tempFile/SNM105P");
                string gFileLocation = System.IO.Path.Combine(gUploadLication, gFileName);
                DataService.SaveBytesToFileByLocation(buffer
                                                , gUploadLication
                                                , gFileLocation);
                SystemApiController sys = new SystemApiController();
                dt = SNM105P_Validate(gFileLocation);
                return Json(DTFM.convertToList(dt));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM105P_UploadExcel");
                throw new HttpResponseException(HttpStatusCode.NotAcceptable);
            }
            throw new Exception("Something wrong");
        }
        public DataTable SNM105P_Validate(string filelocation)
        {

            DataTable dt = new DataTable();
            SystemApiController sys = new SystemApiController();
            dt = sys.ReadDataExcel(filelocation);
            dt.Columns.Add("SEQ");
            dt.Columns.Add("PART_NAME_EN");
            dt.Columns.Add("BASE_PRICE");
            dt.Columns.Add("STANDARD_COST");
            dt.Columns.Add("MSG");
            dt.Columns.Add("ERR_FLAG_PART");
            dt.Columns.Add("ERR_FLAG");
            dt.Columns.Add("ERR_FLAG_BP");
            dt.Columns.Add("ERR_FLAG_STD");
            dt.Columns.Add("ERR_MSG_BP");
            dt.Columns.Add("ERR_MSG_STD");
            int iCount = 0;
            Hashtable hTable = new Hashtable();
            OracleCommand cmd = new OracleCommand();
            foreach (DataRow d in dt.Rows)
            {
                if (!string.IsNullOrEmpty(d["PART_NO"].ToString()))
                {
                    iCount++;
                    try
                    {

                        cmd = new OracleCommand();
                        cmd.CommandText = "SP_NP_SNM105P_VALIDATE";
                        cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                        cmd.Parameters.Add("P_PART_NUMBER", OracleDbType.Varchar2).Value = d["PART_NO"];
                        cmd.Parameters.Add("P_BASE_PRICE", OracleDbType.Varchar2).Value = d["NEW_BASE_PRICE"];
                        cmd.Parameters.Add("P_STANDARD_COST", OracleDbType.Varchar2).Value = d["NEW_STANDARD_COST"];                    
                        

                        cmd.Parameters.Add("R_CURRENT_BASE_PRICE", OracleDbType.Double).Direction = ParameterDirection.Output;
                        cmd.Parameters.Add("R_CURRENT_STANDARD_COST", OracleDbType.Double).Direction = ParameterDirection.Output;

                        cmd.Parameters.Add("R_ERROR_MESSAGE", OracleDbType.Varchar2, 500).Direction = ParameterDirection.Output;
                        cmd.Parameters.Add("R_ERROR_FLAG_PART_NO", OracleDbType.Varchar2, 1).Direction = ParameterDirection.Output;
                        cmd.Parameters.Add("R_PART_NAME", OracleDbType.Varchar2, 500).Direction = ParameterDirection.Output;
                        cmd.Parameters.Add("R_ERROR_FLAG", OracleDbType.Varchar2, 1).Direction = ParameterDirection.Output;
                        cmd.Parameters.Add("R_ERROR_FLAG_BP", OracleDbType.Varchar2, 1).Direction = ParameterDirection.Output;
                        cmd.Parameters.Add("R_ERROR_FLAG_STD", OracleDbType.Varchar2, 1).Direction = ParameterDirection.Output;
                        cmd.Parameters.Add("R_ERROR_MSG_BP", OracleDbType.Varchar2, 500).Direction = ParameterDirection.Output;
                        cmd.Parameters.Add("R_ERROR_MSG_STD", OracleDbType.Varchar2, 500).Direction = ParameterDirection.Output;

                        cmd = oOracledb.SqlQueryReturnValue(cmd);

                        string BASE_PRICE = cmd.Parameters["R_CURRENT_BASE_PRICE"].Value.ToString() == "null" ? "" : cmd.Parameters["R_CURRENT_BASE_PRICE"].Value.ToString();
                        string STANDARD_COST = cmd.Parameters["R_CURRENT_STANDARD_COST"].Value.ToString() == "null" ? "" : cmd.Parameters["R_CURRENT_STANDARD_COST"].Value.ToString();
                        string MSG = cmd.Parameters["R_ERROR_MESSAGE"].Value.ToString() == "null" ? (string)null : cmd.Parameters["R_ERROR_MESSAGE"].Value.ToString();
                        string ERROR_FLAG_PART_NO = cmd.Parameters["R_ERROR_FLAG_PART_NO"].Value.ToString();
                        string ERROR_FLAG = cmd.Parameters["R_ERROR_FLAG"].Value.ToString();
                        string PART_NAME_EN = cmd.Parameters["R_PART_NAME"].Value.ToString() == "null" ? "" : cmd.Parameters["R_PART_NAME"].Value.ToString();
                        string ERROR_FLAG_BP = cmd.Parameters["R_ERROR_FLAG_BP"].Value.ToString();
                        string ERROR_FLAG_STD = cmd.Parameters["R_ERROR_FLAG_STD"].Value.ToString();
                        string MSG_BP = cmd.Parameters["R_ERROR_MSG_BP"].Value.ToString() == "null" ? "" : cmd.Parameters["R_ERROR_MSG_BP"].Value.ToString();
                        string MSG_STD = cmd.Parameters["R_ERROR_MSG_STD"].Value.ToString() == "null" ? "" : cmd.Parameters["R_ERROR_MSG_STD"].Value.ToString();
                        d["SEQ"] = iCount;
                        d["PART_NAME_EN"] = PART_NAME_EN;
                        d["BASE_PRICE"] = BASE_PRICE;
                        d["STANDARD_COST"] = STANDARD_COST;
                        d["MSG"] = MSG;
                        d["ERR_FLAG_PART"] = ERROR_FLAG_PART_NO;
                        d["ERR_FLAG"] = ERROR_FLAG;
                        d["ERR_FLAG_BP"] = ERROR_FLAG_BP;
                        d["ERR_FLAG_STD"] = ERROR_FLAG_STD;
                        d["ERR_MSG_BP"] = MSG_BP;
                        d["ERR_MSG_STD"] = MSG_STD;

                        if (hTable.Contains(d["PART_NO"].ToString().Trim()))
                        {
                            d["SEQ"] = iCount;
                            d["PART_NAME_EN"] = PART_NAME_EN;
                            d["BASE_PRICE"] = BASE_PRICE;
                            d["STANDARD_COST"] = STANDARD_COST;
                            d["MSG"] = "Duplicate Part No at Seq " + hTable[d["PART_NO"]]; 
                            //d["ERR_FLAG_PART"] = "Y";
                            d["ERR_FLAG"] = "Y";
                            d["ERR_FLAG_BP"] = "Y";
                            d["ERR_FLAG_STD"] = "Y";
                            d["ERR_MSG_BP"] = "Duplicate Part No at Seq " + hTable[d["PART_NO"]]; 
                            d["ERR_MSG_STD"] = "Duplicate Part No at Seq " + hTable[d["PART_NO"]];
                        }
                        else
                        {
                            hTable.Add(d["PART_NO"].ToString().Trim(), iCount);
                        }


                }
                catch (Exception ex)
                {
                    //--- Roll back ---
                    oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM105P_Validate");
                    throw new HttpResponseException(HttpStatusCode.NotAcceptable);
                    }
                }
            }
            return dt;
        }

        [HttpPost, Route("SNM105P_UpdatePrice")]
        [Authorize]
        public IHttpActionResult SNM105P_UpdatePrice([FromBody] dynamic data)
        {

            DataSet ds = new DataSet();
            DataTable dt = new DataTable();

            OracleCommand cmd = new OracleCommand();
            int iCount = 0;
            try
            {

                foreach (dynamic d in data.DETAIL)
                {
                    iCount++;
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNM105P_SAVE_TEMP";
                    cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = d["PART_NO"];
                    cmd.Parameters.Add("P_BASE_PRICE", OracleDbType.Varchar2).Value = d["NEW_BASE_PRICE"];
                    cmd.Parameters.Add("P_STANDARD_COST", OracleDbType.Varchar2).Value = d["NEW_STANDARD_COST"];
                    cmd.Parameters.Add("P_ERR_FLAG_BP", OracleDbType.Varchar2).Value = d["ERR_FLAG_BP"];
                    cmd.Parameters.Add("P_ERR_FLAG_STD", OracleDbType.Varchar2).Value = d["ERR_FLAG_STD"];
                    cmd.Parameters.Add("P_CLEAR_DATA", OracleDbType.Varchar2).Value = iCount == 1 ? "Y" : "N";
                    cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd = oOracledb.SqlQueryReturnValue(cmd);
                }
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM105P_UPDATE_PRICE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_IS_BASE_PRICE", OracleDbType.Varchar2).Value = data.IS_BASE_PRICE;
                cmd.Parameters.Add("P_IS_STANDARD_COST", OracleDbType.Varchar2).Value = data.IS_STANDARD_COST;                
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_IP", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("R_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("R_MESSAGE", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);
                string r_status = cmd.Parameters["R_STATUS"].Value.ToString();
                string r_msg = cmd.Parameters["R_MESSAGE"].Value.ToString();

                    if (r_status != "DONE")
                    {
                        oOracledb.WriteLog("ErrorHandle", r_msg, "SNM105P_UpdatePrice");
                    }

                return Json(new { STATUS = r_status,MESSAGE = r_msg });
                }
                catch (Exception ex)
                {
                    //--- Roll back ---
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM105P_UpdatePrice");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
                }

            //return Json(data);
        }

        [HttpPost, Route("SNM105P_Download")]
        public HttpResponseMessage SNM105P_Download(List<DataModel.SNM105P> dModel)
        {
            DataTable dt = new DataTable();
            try
            {
                string x = string.Empty;
                string filename = "SNM105P.csv";
                string floderLocation = "SNM105P";
                dt = DTFM.ToDatatable<DataModel.SNM105P>(dModel);
                x = DataService.WriteCSV(dt, filename, floderLocation, "SNM105P");
                Controllers.SystemApiController response = new Controllers.SystemApiController();
                var File = response.downloadFile(x.Substring(1), filename);

                return File;
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM105P_Download");
                return Request.CreateResponse(HttpStatusCode.BadRequest);
            }
        }
        #endregion

        #region SNSE165
        [HttpGet, Route("SNSE165_GetBuyer/{buyer}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE165_GetBuyer(string buyer, int noofshowrec)
        {
            DataSet ds = new DataSet();

            buyer = ismoUtil.escapeCharacter(buyer);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE165_BUYER_CODE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_BUYER", OracleDbType.Varchar2).Value = buyer.ToUpper() == "NULL" ? (object)DBNull.Value : buyer;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE165_GetBuyer");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE165_Search")]
        [Authorize]
        public IHttpActionResult SNSE165_Search([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            try
            {
                //var xxx = param.FROM_DATE.ToString("dd-MM-yyyy");
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE165_SEARCH";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_FROM_DATE", OracleDbType.Varchar2).Value = param.FROM_DATE.ToString("dd-MM-yyyy");
                cmd.Parameters.Add("P_TO_DATE", OracleDbType.Varchar2).Value = param.TO_DATE.ToString("dd-MM-yyyy");
                cmd.Parameters.Add("P_RECORD_STATUS", OracleDbType.Varchar2).Value = param.RECORD_STATUS;
                cmd.Parameters.Add("P_BUYER_CODE", OracleDbType.Varchar2).Value = param.BUYER_CODE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE165_Search");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE165_UPdatePrice_list")]
        [Authorize]
        public IHttpActionResult SNSE165_UPdatePrice_list([FromBody] dynamic[] data)
        {

            DataSet ds = new DataSet();
            DataTable dt = new DataTable();

            OracleCommand cmd = new OracleCommand();

            foreach (dynamic d in data)
            {
                try
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE165_UPD_PRICE_LIST";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_YEAR", OracleDbType.Varchar2).Value = d["YEAR"].ToString();
                    cmd.Parameters.Add("P_MONTH", OracleDbType.Varchar2).Value = d["MONTH"].ToString();
                    cmd.Parameters.Add("P_PART_NUMBER", OracleDbType.Varchar2).Value = d["PART_NUMBER"];
                    cmd.Parameters.Add("P_PART_NAME", OracleDbType.Varchar2).Value = d["PART_NAME"];
                    cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = d["VENDOR_CODE"];
                    cmd.Parameters.Add("P_VENDOR_TYPE", OracleDbType.Varchar2).Value = d["VENDOR_TYPE"];
                    cmd.Parameters.Add("P_GLP", OracleDbType.Varchar2).Value = d["GLP"];
                    cmd.Parameters.Add("P_SALES_PACKAGE", OracleDbType.Varchar2).Value = d["SALES_PACKAGE"];
                    cmd.Parameters.Add("P_DESCRIPTION_CODE", OracleDbType.Varchar2).Value = d["DESCRIPTION_CODE"];
                    cmd.Parameters.Add("P_DESCRIPT_CODE", OracleDbType.Varchar2).Value = d["DESCRIPT_CODE"];
                    cmd.Parameters.Add("P_SALES_CLASSIFICATION_CODE", OracleDbType.Varchar2).Value = d["SALES_CLASSIFICATION_CODE"];
                    cmd.Parameters.Add("P_SALES_CLASS", OracleDbType.Varchar2).Value = d["SALES_CLASS"];
                    cmd.Parameters.Add("P_EDC_CODE", OracleDbType.Varchar2).Value = d["EDC_CODE"];
                    cmd.Parameters.Add("P_ADC_CODE", OracleDbType.Varchar2).Value = d["ADC_CODE"];
                    cmd.Parameters.Add("P_PURCHASE_FOB_BP", OracleDbType.Varchar2).Value = d["PURCHASE_FOB_BP"];
                    cmd.Parameters.Add("P_NET_WEIGHT", OracleDbType.Varchar2).Value = d["NET_WEIGHT"];
                    cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Varchar2).Value = d["ITEM_NO"];
                    cmd.Parameters.Add("P_COUNTRY_CODE", OracleDbType.Varchar2).Value = d["COUNTRY_CODE"];
                    cmd.Parameters.Add("P_BUYER_CODE", OracleDbType.Varchar2).Value = d["BUYER_CODE"];
                    cmd.Parameters.Add("P_UPDATE_STATUS", OracleDbType.Varchar2).Value = d["UPDATE_STATUS"];
                    cmd.Parameters.Add("P_FILE_NAME", OracleDbType.Varchar2).Value = d["FILE_NAME"];
                    cmd.Parameters.Add("P_CREATE_DATE", OracleDbType.Varchar2).Value = d.CREATE_DATE.ToString("yyyy-MM-dd");
                    cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    //cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = d["LAST_UPD_PGM"].ToString();
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                    ds = oOracledb.SqlQueryDS_CMD(cmd);

                    if (ds.Tables.Count > 0)
                    {
                        if (ds.Tables[0].Rows[0]["STATUS"].ToString() != "DONE")
                        {
                            string errString = "Error : " + d["PART_NUMBER"].ToString() + "/\n" + ds.Tables[0].Rows[0]["STATUS"].ToString();
                            oOracledb.WriteLog("ErrorHandle", errString, "SNSE165_UPdatePrice_list");
                            return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, errString));
                        }
                    }
                }
                catch (Exception ex)
                {
                    //--- Roll back ---
                    string errString = d["PART_NUMBER"].ToString() + ex.Message.ToString();
                    oOracledb.WriteLog("ErrorHandle", errString, "SNSE165_UPdatePrice_list");
                    return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, errString));
                }
            }
            return Ok("DONE");
        }


        [HttpPost, Route("SNSE165_ReUpdate")]
        [Authorize]
        public IHttpActionResult SNSE165_ReUpdate([FromBody] dynamic[] data)
        {

            DataSet ds = new DataSet();
            DataTable dt = new DataTable();

            OracleCommand cmd = new OracleCommand();

            foreach (dynamic d in data)
            {
                try
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE165_UPD_PRICE_MASTER";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_PRICING_CODE", OracleDbType.Varchar2).Value = d["PRICING_CODE"];
                    cmd.Parameters.Add("P_MSG_ERR", OracleDbType.Varchar2).Value = d["MSG_ERR"];
                    cmd.Parameters.Add("P_UPDATE_STATUS", OracleDbType.Varchar2).Value = d["UPDATE_STATUS"];
                    cmd.Parameters.Add("P_COUNTRY_CODE", OracleDbType.Varchar2).Value = d["COUNTRY_CODE"];
                    cmd.Parameters.Add("P_BUYER_CODE", OracleDbType.Varchar2).Value = d["BUYER_CODE"];
                    cmd.Parameters.Add("P_PART_NUMBER", OracleDbType.Varchar2).Value = d["PART_NUMBER"];
                    cmd.Parameters.Add("P_PURCHASE_FOB_BP", OracleDbType.Double).Value = d["PURCHASE_FOB_BP"];
                    cmd.Parameters.Add("P_PURCHASE_FOB_CURRENCY", OracleDbType.Varchar2).Value = d["PURCHASE_FOB_CURRENCY"];
                    cmd.Parameters.Add("P_PURCHASE_FOB_PRICE_APP", OracleDbType.Varchar2).Value = d["PURCHASE_FOB_PRICE_APP"];
                    cmd.Parameters.Add("P_PURCHASE_FOB_SP", OracleDbType.Double).Value = d["PURCHASE_FOB_SP"];
                    cmd.Parameters.Add("P_PURCHASE_FOB_SP_START", OracleDbType.Varchar2).Value = d["PURCHASE_FOB_SP_START"];
                    cmd.Parameters.Add("P_PURCHASE_FOB_SP_END", OracleDbType.Varchar2).Value = d["PURCHASE_FOB_SP_END"];
                    cmd.Parameters.Add("P_FILE_NAME", OracleDbType.Varchar2).Value = d["FILE_NAME"];
                    cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Int16).Value = d["ITEM_NO"];
                    cmd.Parameters.Add("P_CREATE_DATE", OracleDbType.Varchar2).Value = d["CREATE_DATE"].ToString("yyyy-MM-dd");
                    cmd.Parameters.Add("P_PURCHASE_COST_CURRENCY", OracleDbType.Varchar2).Value = d["PURCHASE_COST_CURRENCY"];
                    cmd.Parameters.Add("P_VENDOR_TYPE", OracleDbType.Varchar2).Value = d["VENDOR_TYPE"];
                    cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = d["VENDOR_CODE"];
                    cmd.Parameters.Add("P_DUTY_RATE", OracleDbType.Double).Value = d["DUTY_RATE"];
                    cmd.Parameters.Add("P_RETAIL_PRICE", OracleDbType.Double).Value = d["RETAIL_PRICE"];
                    cmd.Parameters.Add("P_GROUP_PART_NO", OracleDbType.Varchar2).Value = d["GROUP_PART_NO"];
                    cmd.Parameters.Add("P_BASE_PRICE", OracleDbType.Double).Value = d["BASE_PRICE"];
                    cmd.Parameters.Add("P_STANDARD_COST", OracleDbType.Varchar2).Value = d["STANDARD_COST"];
                    cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_IP", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress;
                    //cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = d["LAST_UPD_PGM"].ToString();
                    cmd.Parameters.Add("P_STATUS", OracleDbType.Varchar2, 10).Direction = ParameterDirection.Output;
                    cmd.Parameters.Add("P_MSG_ERR_OUT", OracleDbType.Varchar2, 2000).Direction = ParameterDirection.Output;

                    //ds = oOracledb.SqlQueryDS_CMD(cmd);
                    cmd = oOracledb.SqlQueryReturnValue(cmd);
                    string r_status = cmd.Parameters["P_STATUS"].Value.ToString();
                    string r_Msg = cmd.Parameters["P_MSG_ERR_OUT"].Value.ToString();

                    if (r_status != "DONE")
                    {
                        string errString = "Error : " + d["P_PART_NUMBER"].ToString() + "/\n" + r_status;
                        oOracledb.WriteLog("ErrorHandle", errString, "SNSE165_ReUpdate");
                        return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, errString));
                    }
                    else
                    {
                        d["MSG_ERR"] = r_Msg;
                    }

                }
                catch (Exception ex)
                {
                    //--- Roll back ---
                    string errString = d["P_PART_NUMBER"].ToString() + ex.Message.ToString();
                    oOracledb.WriteLog("ErrorHandle", errString, "SNSE165_ReUpdate");
                    return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, errString));
                }
            }
            return Ok("DONE");
            //return Json(data);
        }


        [HttpPost, Route("SNSE165_CancelPrice")]
        [Authorize]
        public IHttpActionResult SNSE165_CancelPrice([FromBody] dynamic[] data)
        {

            DataSet ds = new DataSet();
            DataTable dt = new DataTable();

            OracleCommand cmd = new OracleCommand();

            foreach (dynamic d in data)
            {
                try
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE165_CANCEL_PRICE";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_PART_NUMBER", OracleDbType.Varchar2).Value = d["PART_NUMBER"].ToString();
                    cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Int16).Value = d["ITEM_NO"];
                    cmd.Parameters.Add("P_COUNTRY_CODE", OracleDbType.Varchar2).Value = d["COUNTRY_CODE"].ToString();
                    cmd.Parameters.Add("P_BUYER_CODE", OracleDbType.Varchar2).Value = d["BUYER_CODE"].ToString();
                    cmd.Parameters.Add("P_UPDATE_STATUS", OracleDbType.Varchar2).Value = d["UPDATE_STATUS"].ToString();
                    cmd.Parameters.Add("P_FILE_NAME", OracleDbType.Varchar2).Value = d["FILE_NAME"].ToString();
                    cmd.Parameters.Add("P_CREATE_DATE", OracleDbType.Varchar2).Value = d["CREATE_DATE"].ToString("yyyy-MM-dd");
                    cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    //cmd.Parameters.Add("P_IP", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();                   
                    cmd.Parameters.Add("P_STATUS", OracleDbType.Varchar2, 10).Direction = ParameterDirection.Output;
                    //cmd.Parameters.Add("P_MSG_ERR_OUT", OracleDbType.Varchar2, 2000).Direction = ParameterDirection.Output;

                    //ds = oOracledb.SqlQueryDS_CMD(cmd);
                    cmd = oOracledb.SqlQueryReturnValue(cmd);
                    string r_status = cmd.Parameters["P_STATUS"].Value.ToString();
                    //string r_Msg = cmd.Parameters["P_MSG_ERR_OUT"].Value.ToString();

                    if (r_status != "DONE")
                    {
                        string errString = "Error : " + d["PART_NUMBER"].ToString() + "/\n" + r_status;
                        oOracledb.WriteLog("ErrorHandle", errString, "SNSE165_CancelPrice");
                        return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, errString));
                    }

                }
                catch (Exception ex)
                {
                    //--- Roll back ---
                    string errString = d["PART_NUMBER"].ToString() + ex.Message.ToString();
                    oOracledb.WriteLog("ErrorHandle", errString, "SNSE165_CancelPrice");
                    return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, errString));
                }
            }
            //return Ok("DONE");
            return Ok("DONE");
        }

        [HttpPost, Route("SNSE165_Export")]
        public HttpResponseMessage SNSE165_Export(List<DataModel.SNSE165_MODEL> dModel)
        {
            DataTable dt = new DataTable();
            try
            {
                string x = string.Empty;
                string filename = "SNSE165.csv";
                string floderLocation = "SNSE165";
                dt = DTFM.ToDatatable<DataModel.SNSE165_MODEL>(dModel);
                x = DataService.WriteCSV(dt, filename, floderLocation, "SNSE165");
                Controllers.SystemApiController response = new Controllers.SystemApiController();
                var File = response.downloadFile(x.Substring(1), filename);

                return File;
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE165_Export");
                return Request.CreateResponse(HttpStatusCode.BadRequest);
            }
        }
        #endregion

        #region SNM041P
        [HttpGet, Route("SNM041P_JobType/{jobtype}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM041P_JobType(string jobtype, int noofshowrec)
        {
            DataSet ds = new DataSet();

            jobtype = ismoUtil.escapeCharacter(jobtype);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM041P_GET_JOB_TYPE";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_JOB_TYPE", OracleDbType.Varchar2).Value = jobtype.ToUpper() == "NULL" ? (object)DBNull.Value : jobtype;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNM041P_OrderClass/{orderclass}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM041P_OrderClass(string orderclass, int noofshowrec, [FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            orderclass = ismoUtil.escapeCharacter(orderclass);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM041P_GET_ORDER_CLASS";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_JOB_TYPE", OracleDbType.Varchar2).Value = param.JOB_TYPE;
            cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = orderclass.ToUpper() == "NULL" ? (object)DBNull.Value : orderclass;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }


        [HttpPost, Route("SNM041P_OrderList")]
        [Authorize]
        public IHttpActionResult SNM041P_OrderList([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM041P_GET_ORDER_LIST";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_JOB_TYPE", OracleDbType.Varchar2).Value = param.JOB_TYPE;
            cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = param.ORDER_CLASS;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNM041P_Insert")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM041P_Insert([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM041P_INSERT";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_JOB_TYPE", OracleDbType.Varchar2).Value = param.JOB_TYPE;
            cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = param.ORDER_CLASS;
            cmd.Parameters.Add("P_ORDER_DESCRIPT", OracleDbType.Varchar2).Value = param.ORDER_DESCRIPT;
            cmd.Parameters.Add("P_PGM", OracleDbType.Varchar2).Value = "SNM041P";
            cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
            cmd.Parameters.Add("P_IP", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNM041P_UPDATE")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM041P_UPDATE([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM041P_UPDATE";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_JOB_TYPE", OracleDbType.Varchar2).Value = param.JOB_TYPE;
            cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = param.ORDER_CLASS;
            cmd.Parameters.Add("P_ORDER_DESCRIPT", OracleDbType.Varchar2).Value = param.ORDER_DESCRIPT;
            cmd.Parameters.Add("P_PGM", OracleDbType.Varchar2).Value = "SNM041P";
            cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
            cmd.Parameters.Add("P_IP", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }

        [HttpGet, Route("SNM041P_DELETE/{jobtype}/{orderclass}")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM041P_DELETE(string jobtype, string orderclass)
        {
            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM041P_DELETE";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_JOB_TYPE", OracleDbType.Varchar2).Value = jobtype;
            cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = orderclass;
            cmd.Parameters.Add("P_PGM", OracleDbType.Varchar2).Value = "SNM041P";
            cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
            cmd.Parameters.Add("P_IP", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }
        #endregion

        #region SNSQ038

        [HttpGet, Route("SNSQ038_GetPartNo/{partno}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSQ038_GetPartNo(string partno, int noofshowrec)
        {
            DataSet ds = new DataSet();

            partno = ismoUtil.escapeCharacter(partno);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSQ038_GET_PART";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_PART_NUMBER", OracleDbType.Varchar2).Value = partno.ToUpper() == "NULL" ? (object)DBNull.Value : partno;

            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNSQ038_GetData")]
        [Authorize]
        public IHttpActionResult SNSQ038_GetData([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSQ038_GET_DATA";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_DATE_FROM", OracleDbType.Varchar2).Value = param.DATE_FROM != null ? param.DATE_FROM.ToString("dd-MM-yyyy") : (object)DBNull.Value;
            cmd.Parameters.Add("P_DATE_TO", OracleDbType.Varchar2).Value = param.DATE_TO != null ? param.DATE_TO.ToString("dd-MM-yyyy") : (object)DBNull.Value;
            cmd.Parameters.Add("P_RECEIVE_TYPE", OracleDbType.Varchar2).Value = param.RECIEVE_TYPE;
            cmd.Parameters.Add("P_PART_NUMBER", OracleDbType.Varchar2).Value = param.PART_NUMBER;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));

        }


        [HttpPost, Route("SNSQ038_Download")]
        public HttpResponseMessage SNSQ038_Download(List<DataModel.SNSQ038> dModel)
        {
            try
            {
                DataTable dt = new DataTable();
                string x = string.Empty;
                string filename = "SNSQ038_FILE.csv";
                string floderLocation = "SNSQ038";
                dt = DTFM.ToDatatable<DataModel.SNSQ038>(dModel);
                //oOracledb.WriteLog("ErrorHandle", "SNSQ038_Download_01", "SNSQ038_Download");
                x = DataService.WriteCSV(dt, filename, floderLocation, "SNSQ038");
                //oOracledb.WriteLog("ErrorHandle", "SNSQ038_Download_02", "SNSQ038_Download");
                Controllers.SystemApiController response = new Controllers.SystemApiController();
                var File = response.downloadFile(x.Substring(1), filename);

                return File;


            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ038_Download");
                return Request.CreateResponse(HttpStatusCode.BadRequest, ex);
            }


        }

        #endregion



        #region SNM008P

        [HttpGet, Route("SNM008P_GetCategory/{cat}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM008P_GetCategory(string cat, int noofshowrec)
        {
            DataSet ds = new DataSet();

            cat = ismoUtil.escapeCharacter(cat);
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM008P_GET_CATEGORY";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CATEGORY", OracleDbType.Varchar2).Value = cat.ToUpper() == "NULL" ? (object)DBNull.Value : cat;
                //cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM008P_GetCategory");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM008P_GetCatDetail")]
        [Authorize]
        public IHttpActionResult SNM008P_GetCatDetail([FromBody] dynamic Param)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();

            try
            {
                cmd.CommandText = "SP_NP_SNM008P_GET_CAT_DETAIL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CATEGORY", OracleDbType.Varchar2).Value = Param.PART_CATEGORY;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM008P_GetCatDetail");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpPost, Route("SNM008P_Insert")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM008P_Insert([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM008P_INSERT";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_CATEGORY", OracleDbType.Varchar2).Value = param.PART_CATEGORY;
                cmd.Parameters.Add("P_CATEGORY_DESC", OracleDbType.Varchar2).Value = param.CATEGORY_DESC;
                cmd.Parameters.Add("P_CATEGORY_REM", OracleDbType.Varchar2).Value = param.CATEGORY_REM;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_PGM", OracleDbType.Varchar2).Value = "SNM008P";
                cmd.Parameters.Add("P_IP", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM008P_Insert");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpPost, Route("SNM008P_Update")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM008P_Update([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM008P_UPDATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_CATEGORY", OracleDbType.Varchar2).Value = param.PART_CATEGORY;
                cmd.Parameters.Add("P_CATEGORY_DESC", OracleDbType.Varchar2).Value = param.CATEGORY_DESC;
                cmd.Parameters.Add("P_CATEGORY_REM", OracleDbType.Varchar2).Value = param.CATEGORY_REM;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_PGM", OracleDbType.Varchar2).Value = "SNM008P";
                cmd.Parameters.Add("P_IP", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM008P_Update");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNM008P_Delete/{part_cat}")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM008P_Delete(string part_cat)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM008P_DELETE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_CATEGORY", OracleDbType.Varchar2).Value = part_cat;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_PGM", OracleDbType.Varchar2).Value = "SNM008P";
                cmd.Parameters.Add("P_IP", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM008P_Delete");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        #endregion


        #region SNM009P

        [HttpGet, Route("SNM009P_GetProductCode/{product}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM009P_GetProductCOde(string product, int noofshowrec)
        {
            DataSet ds = new DataSet();

            product = ismoUtil.escapeCharacter(product);
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM009P_GET_PRODUCT";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PRODUCT", OracleDbType.Varchar2).Value = product.ToUpper() == "NULL" ? (object)DBNull.Value : product;
                //cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM009P_GetProductCOde");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM009P_GetProductDetail")]
        [Authorize]
        public IHttpActionResult SNM009P_GetProductDetail([FromBody] dynamic Param)
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM009P_GET_PROD_DETAIL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PRODUCT", OracleDbType.Varchar2).Value = Param.PRODUCT_CODE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM009P_GetProductDetail");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }



        [HttpPost, Route("SNM009P_Insert")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM009P_Insert([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM009P_INSERT";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PRODUCT_CODE", OracleDbType.Varchar2).Value = param.PRODUCT_CODE;
                cmd.Parameters.Add("P_PRODUCT_NAME_EN", OracleDbType.Varchar2).Value = param.PRODUCT_NAME_EN;
                cmd.Parameters.Add("P_PRODUCT_NAME_TH", OracleDbType.Varchar2).Value = param.PRODUCT_NAME_TH;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_PGM", OracleDbType.Varchar2).Value = "SNM009P";
                cmd.Parameters.Add("P_IP", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM009P_Insert");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpPost, Route("SNM009P_Update")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM009P_Update([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM009P_UPDATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PRODUCT_CODE", OracleDbType.Varchar2).Value = param.PRODUCT_CODE;
                cmd.Parameters.Add("P_PRODUCT_NAME_EN", OracleDbType.Varchar2).Value = param.PRODUCT_NAME_EN;
                cmd.Parameters.Add("P_PRODUCT_NAME_TH", OracleDbType.Varchar2).Value = param.PRODUCT_NAME_TH;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_PGM", OracleDbType.Varchar2).Value = "SNM009P";
                cmd.Parameters.Add("P_IP", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM009P_Update");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNM009P_Delete/{product_code}")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM009P_Delete(string product_code)
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM009P_DELETE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PRODUCT_CODE", OracleDbType.Varchar2).Value = product_code;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_PGM", OracleDbType.Varchar2).Value = "SNM009P";
                cmd.Parameters.Add("P_IP", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM009P_Delete");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        #endregion


        #region SNSQ030

        [HttpGet, Route("SNSQ030_GetSupplier/{supplier}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSQ030_GetVendor(string supplier, int noofshowrec)
        {
            DataSet ds = new DataSet();

            supplier = ismoUtil.escapeCharacter(supplier);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSQ030_GET_VENDOR";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_VENDOR", OracleDbType.Varchar2).Value = supplier.ToUpper() == "NULL" ? (object)DBNull.Value : supplier;
            cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }


        [HttpPost, Route("SNSQ030_GetPartNo/{partno}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSQ030_GetPartNo(string partno, int noofshowrec, [FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            partno = ismoUtil.escapeCharacter(partno);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSQ030_GET_PART";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_SUPPLIER_CODE", OracleDbType.Varchar2).Value = param.SUPPLIER_CODE;
            cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = partno.ToUpper() == "NULL" ? (object)DBNull.Value : partno;

            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNSQ030_GetData")]
        [Authorize]
        public IHttpActionResult SNSQ030_GetData([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSQ030_GET_DATA";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_DATE_FROM", OracleDbType.Varchar2).Value = param.DATE_FROM != null ? param.DATE_FROM.ToString("dd-MM-yyyy") : (object)DBNull.Value;
            cmd.Parameters.Add("P_DATE_TO", OracleDbType.Varchar2).Value = param.DATE_TO != null ? param.DATE_TO.ToString("dd-MM-yyyy") : (object)DBNull.Value;
            cmd.Parameters.Add("P_SUPPLIER_CODE", OracleDbType.Varchar2).Value = param.SUPPLIER_CODE;
            cmd.Parameters.Add("P_PART_NUMBER", OracleDbType.Varchar2).Value = param.PART_NUMBER;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNSQ030_Download")]
        public HttpResponseMessage SNSQ030_Download(List<DataModel.SNSQ030> dModel)
        {
            try
            {
                DataTable dt = new DataTable();
                string x = string.Empty;
                string filename = "SNSQ030_FILE.csv";
                string floderLocation = "SNSQ030";
                dt = DTFM.ToDatatable<DataModel.SNSQ030>(dModel);
                x = DataService.WriteCSV(dt, filename, floderLocation, "SNSQ030");
                Controllers.SystemApiController response = new Controllers.SystemApiController();
                var File = response.downloadFile(x.Substring(1), filename);

                return File;

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE127_EXPORT_2");
                return Request.CreateResponse(HttpStatusCode.BadRequest, ex);
            }
        }
        [HttpPost, Route("SNSQ030_getDataForExport")]
        [Authorize]
        public HttpResponseMessage SNSQ030_getDataForExport([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSQ030_GET_DATA";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_DATE_FROM", OracleDbType.Varchar2).Value = param.DATE_FROM != null ? param.DATE_FROM.ToString("dd-MM-yyyy") : (object)DBNull.Value;
            cmd.Parameters.Add("P_DATE_TO", OracleDbType.Varchar2).Value = param.DATE_TO != null ? param.DATE_TO.ToString("dd-MM-yyyy") : (object)DBNull.Value;
            cmd.Parameters.Add("P_SUPPLIER_CODE", OracleDbType.Varchar2).Value = param.SUPPLIER_CODE;
            cmd.Parameters.Add("P_PART_NUMBER", OracleDbType.Varchar2).Value = param.PART_NUMBER;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            string x = SNSQ030_ExportExcel(ds.Tables[0], "SNSQ030.csv", "SNSQ030");
            SystemApiController sys = new SystemApiController();
            var respons = sys.downloadFile(x.Substring(1), "SNSQ030.csv");
            return respons;

        }

        public static string SNSQ030_ExportExcel(DataTable dt, string fileName, string FloderLocation)
        {
            bool Succes = false;

            string gLocation = "~/tempFile/" + FloderLocation + "/";
            string gWriteLocation = System.Web.HttpContext.Current.Server.MapPath(gLocation);

            FileInfo filePath = new FileInfo(gWriteLocation);
            filePath.Directory.Create();
            string gFileLocation = Path.Combine(gWriteLocation, fileName);
            string gReturnLocation = Path.Combine(gLocation, fileName);
            SystemApiController sys = new SystemApiController();
            try
            {
                StringBuilder sb = new StringBuilder();

                string[] HeaderForExport = new string[]
                {
                    "Seq.",
                    "Create Date",
                    "Part No",
                    "Part Name",
                    "Local Supplier Code",
                    "Piece Price",
                    "Adopted Date From",
                    "Currency",
                    "Create By",
                    "Last Upd Pgm",
                    "File Name",
                    "Item No"
                };

                IEnumerable<string> columnNames = HeaderForExport.Select(h => h.ToString());
                sb.AppendLine(string.Join(",", columnNames));

                foreach (DataRow row in dt.Rows)
                {
                    for (int i = 0; i < dt.Columns.Count; i++)
                    {
                        if (i == 1 || i == 6)
                        {
                            sb.Append(row[i].ToString() == "" ? "" : Convert.ToDateTime(row[i]).ToString("dd-MMM-yyyy"));
                        }
                        else
                        {
                            sb.Append(CsvEscape(row[i].ToString()));
                        }



                        sb.Append(i == dt.Columns.Count - 1 ? "\n" : ",");
                    }
                }
                File.WriteAllText(gFileLocation, sb.ToString());

                Succes = true;
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
            }
            if (Succes)
            {
                return gReturnLocation;
            }
            else
            {
                return "";
            }

        }

        static string CsvEscape(string value)
        {
            if (value.Contains(","))
            {
                return "\"" + value.Replace("\"", "\"\"") + "\"";
            }
            return value;
        }
        #endregion



        #region SNM010P

        [HttpGet, Route("SNM010P_GetPricingCode/{pricing}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM010P_GetPricingCode(string pricing, int noofshowrec)
        {
            DataSet ds = new DataSet();

            pricing = ismoUtil.escapeCharacter(pricing);
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM010P_GET_PRICING_CODE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PRICING_CODE", OracleDbType.Varchar2).Value = pricing.ToUpper() == "NULL" ? (object)DBNull.Value : pricing;
                //cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM010P_GetPricingCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM010P_GetPricingDetail")]
        [Authorize]
        public IHttpActionResult SNM010P_GetPricingDetail([FromBody] dynamic Param)
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM010P_GET_PRIC_DETAIL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PRICING_CODE", OracleDbType.Varchar2).Value = Param.PRICING_CODE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM010P_GetPricingDetail");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }



        [HttpPost, Route("SNM010P_Insert")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM010P_Insert([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM010P_INSERT";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PRICING_CODE", OracleDbType.Varchar2).Value = param.PRICING_CODE;
                cmd.Parameters.Add("P_MARK_UP", OracleDbType.Varchar2).Value = param.MARK_UP;
                cmd.Parameters.Add("P_CALCULATE_PRICE", OracleDbType.Varchar2).Value = param.CALCULATE_PRICE;
                cmd.Parameters.Add("P_PRINT_PRICE", OracleDbType.Varchar2).Value = param.PRINT_PRICE;
                cmd.Parameters.Add("P_ATTRIBUTE1", OracleDbType.Varchar2).Value = param.ATTRIBUTE1;
                cmd.Parameters.Add("P_ATTRIBUTE2", OracleDbType.Varchar2).Value = param.ATTRIBUTE2;
                cmd.Parameters.Add("P_ATTRIBUTE5", OracleDbType.Varchar2).Value = param.ATTRIBUTE5;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_PGM", OracleDbType.Varchar2).Value = "SNM010P";
                cmd.Parameters.Add("P_IP", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM010P_Insert");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpPost, Route("SNM010P_Update")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM010P_Update([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM010P_UPDATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PRICING_CODE", OracleDbType.Varchar2).Value = param.PRICING_CODE;
                cmd.Parameters.Add("P_MARK_UP", OracleDbType.Varchar2).Value = param.MARK_UP;
                cmd.Parameters.Add("P_CALCULATE_PRICE", OracleDbType.Varchar2).Value = param.CALCULATE_PRICE;
                cmd.Parameters.Add("P_PRINT_PRICE", OracleDbType.Varchar2).Value = param.PRINT_PRICE;
                cmd.Parameters.Add("P_ATTRIBUTE1", OracleDbType.Varchar2).Value = param.ATTRIBUTE1;
                cmd.Parameters.Add("P_ATTRIBUTE2", OracleDbType.Varchar2).Value = param.ATTRIBUTE2;
                cmd.Parameters.Add("P_ATTRIBUTE5", OracleDbType.Varchar2).Value = param.ATTRIBUTE5;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_PGM", OracleDbType.Varchar2).Value = "SNM010P";
                cmd.Parameters.Add("P_IP", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM010P_Update");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNM010P_Delete/{pricing_code}")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM010P_Delete(string pricing_code)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM010P_DELETE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PRICING_CODE", OracleDbType.Varchar2).Value = pricing_code;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_PGM", OracleDbType.Varchar2).Value = "SNM010P";
                cmd.Parameters.Add("P_IP", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM010P_Delete");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        #endregion

        #region SNSQ031
        [HttpPost, Route("SNSQ031_GetData")]
        [Authorize]
        public IHttpActionResult SNSQ031_GetData([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSQ031_GET_DATA";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_DATE_FROM", OracleDbType.Varchar2).Value = param.DATE_FROM != null ? param.DATE_FROM.ToString("dd-MM-yyyy") : (object)DBNull.Value;
            cmd.Parameters.Add("P_DATE_TO", OracleDbType.Varchar2).Value = param.DATE_TO != null ? param.DATE_TO.ToString("dd-MM-yyyy") : (object)DBNull.Value;
            cmd.Parameters.Add("P_SUPPLIER_CODE", OracleDbType.Varchar2).Value = param.SUPPLIER_CODE;
            cmd.Parameters.Add("P_PART_NUMBER", OracleDbType.Varchar2).Value = param.PART_NUMBER;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }


        [HttpPost, Route("SNSQ031_Download")]
        public HttpResponseMessage SNSQ031_Download(List<DataModel.SNSQ030> dModel)
        {

            try
            {
                DataTable dt = new DataTable();
                string x = string.Empty;
                string filename = "SNSQ031_FILE.csv";
                string floderLocation = "SNSQ031";
                dt = DTFM.ToDatatable<DataModel.SNSQ030>(dModel);
                x = DataService.WriteCSV(dt, filename, floderLocation, "SNSQ031");
                Controllers.SystemApiController response = new Controllers.SystemApiController();
                var File = response.downloadFile(x.Substring(1), filename);

                return File;

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ031_Download");
                return Request.CreateResponse(HttpStatusCode.BadRequest, ex);
            }
        }
        [HttpPost, Route("SNSQ031_getDataForExport")]
        [Authorize]
        public HttpResponseMessage SNSQ031_getDataForExport([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSQ031_GET_DATA";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_DATE_FROM", OracleDbType.Varchar2).Value = param.DATE_FROM != null ? param.DATE_FROM.ToString("dd-MM-yyyy") : (object)DBNull.Value;
            cmd.Parameters.Add("P_DATE_TO", OracleDbType.Varchar2).Value = param.DATE_TO != null ? param.DATE_TO.ToString("dd-MM-yyyy") : (object)DBNull.Value;
            cmd.Parameters.Add("P_SUPPLIER_CODE", OracleDbType.Varchar2).Value = param.SUPPLIER_CODE;
            cmd.Parameters.Add("P_PART_NUMBER", OracleDbType.Varchar2).Value = param.PART_NUMBER;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            string x = SNSQ031_ExportExcel(ds.Tables[0], "SNSQ031.csv", "SNSQ031");
            SystemApiController sys = new SystemApiController();
            var respons = sys.downloadFile(x.Substring(1), "SNSQ031.csv");
            return respons;

        }

        public static string SNSQ031_ExportExcel(DataTable dt, string fileName, string FloderLocation)
        {
            bool Succes = false;

            string gLocation = "~/tempFile/" + FloderLocation + "/";
            string gWriteLocation = System.Web.HttpContext.Current.Server.MapPath(gLocation);

            FileInfo filePath = new FileInfo(gWriteLocation);
            filePath.Directory.Create();
            string gFileLocation = Path.Combine(gWriteLocation, fileName);
            string gReturnLocation = Path.Combine(gLocation, fileName);
            SystemApiController sys = new SystemApiController();
            try
            {
                StringBuilder sb = new StringBuilder();

                string[] HeaderForExport = new string[]
                {
                    "Seq.",
                    "Create Date",
                    "Part No",
                    "Part Name",
                    "Local Supplier Code",
                    "Piece Price",
                    "Adopted Date From",
                    "Currency",
                    "Create By",
                    "Last Upd Pgm",
                    "File Name",
                    "Item No"
                };

                IEnumerable<string> columnNames = HeaderForExport.Select(h => h.ToString());
                sb.AppendLine(string.Join(",", columnNames));

                foreach (DataRow row in dt.Rows)
                {
                    for (int i = 0; i < dt.Columns.Count; i++)
                    {
                        if (i == 1 || i == 6)
                        {
                            sb.Append(row[i].ToString() == "" ? "" : Convert.ToDateTime(row[i]).ToString("dd-MMM-yyyy"));
                        }
                        else
                        {
                            sb.Append(CsvEscape(row[i].ToString()));
                        }



                        sb.Append(i == dt.Columns.Count - 1 ? "\n" : ",");
                    }
                }
                File.WriteAllText(gFileLocation, sb.ToString());

                Succes = true;
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
            }
            if (Succes)
            {
                return gReturnLocation;
            }
            else
            {
                return "";
            }
        }
        #endregion

        #region SNSQ032

        [HttpPost, Route("SNSQ032_ProcessGroup/{pg}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSQ032_ProcessGroup(string pg, int noofshowrec, [FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            pg = ismoUtil.escapeCharacter(pg);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSQ032_PROCESS_GROUP";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_PROCESS_GROUP", OracleDbType.Varchar2).Value = pg.ToUpper() == "NULL" ? (object)DBNull.Value : pg;
            cmd.Parameters.Add("P_SYSTEM_GROUP", OracleDbType.Varchar2).Value = param.SYSTEM_GROUP;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNSQ032_ProcessName/{pn}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSQ032_ProcessName(string pn, int noofshowrec, [FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            pn = ismoUtil.escapeCharacter(pn);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSQ032_PROCESS_NAME";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_PROCESS_GROUP", OracleDbType.Varchar2).Value = param.PROCESS_GROUP;
            cmd.Parameters.Add("P_SYSTEM_GROUP", OracleDbType.Varchar2).Value = param.SYSTEM_GROUP;
            cmd.Parameters.Add("P_PROCESS_NAME", OracleDbType.Varchar2).Value = pn.ToUpper() == "NULL" ? (object)DBNull.Value : pn;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNSQ032_GetData")]
        [Authorize]
        public IHttpActionResult SNSQ032_GetData([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSQ032_GET_DATA";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_DATE_FROM", OracleDbType.Varchar2).Value = param.DATE_FROM != null ? param.DATE_FROM.ToString("dd-MM-yyyy") : (object)DBNull.Value;
            cmd.Parameters.Add("P_DATE_TO", OracleDbType.Varchar2).Value = param.DATE_TO != null ? param.DATE_TO.ToString("dd-MM-yyyy") : (object)DBNull.Value;
            cmd.Parameters.Add("P_RECEIVED_STATUS", OracleDbType.Varchar2).Value = param.RECEIVED_STATUS;
            cmd.Parameters.Add("P_PROCESS_GROUP", OracleDbType.Varchar2).Value = param.PROCESS_GROUP;
            cmd.Parameters.Add("P_PROCESS_ID", OracleDbType.Varchar2).Value = param.PROCESS_NAME;
            cmd.Parameters.Add("P_SYSTEM_GROUP", OracleDbType.Varchar2).Value = param.SYSTEM_GROUP;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNSQ032_Download")]
        public HttpResponseMessage SNSQ032_Download(List<DataModel.SNSQ032> dModel)
        {

            try
            {
                DataTable dt = new DataTable();
                string x = string.Empty;
                string filename = "SNSQ032_FILE.csv";
                string floderLocation = "SNSQ032";
                dt = DTFM.ToDatatable<DataModel.SNSQ032>(dModel);
                x = DataService.WriteCSV(dt, filename, floderLocation, "SNSQ032");
                Controllers.SystemApiController response = new Controllers.SystemApiController();
                var File = response.downloadFile(x.Substring(1), filename);

                return File;
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ032_Download");
                return Request.CreateResponse(HttpStatusCode.BadRequest, ex);
            }
        }
        #endregion


        #region SNM020P

        [HttpGet, Route("SNM020P_Salesclass/{sales}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM0202P_Salesclass(string sales, int noofshowrec)
        {
            DataSet ds = new DataSet();

            sales = ismoUtil.escapeCharacter(sales);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM020P_SALESCLASS";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_SALES_CLASS", OracleDbType.Varchar2).Value = sales.ToUpper() == "NULL" ? (object)DBNull.Value : sales;
            cmd.Parameters.Add("p_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        [HttpGet, Route("SNM020P_Product/{product}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM020P_Product(string product, int noofshowrec)
        {
            DataSet ds = new DataSet();

            product = ismoUtil.escapeCharacter(product);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM020P_PRODUCT";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_PRODUCT_CODE", OracleDbType.Varchar2).Value = product.ToUpper() == "NULL" ? (object)DBNull.Value : product;
            cmd.Parameters.Add("p_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNM020P_GetDescriptionCode")]
        [Authorize]

        public IHttpActionResult SNM020P_GetDescriptionCode([FromBody] dynamic param)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM020P_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DESCRIPT_CODE", OracleDbType.Varchar2).Value = param.DESCRIPT_CODE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM020P_GetDescriptionCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNM020P_GetDESCRIPT/{partNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM020P_GetDESCRIPT(string partNo, int noofshowrec)
        {
            try
            {
                DataSet ds = new DataSet();

                partNo = ismoUtil.escapeCharacter(partNo);

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM020P_GET_DESCRIPT";
                cmd.Parameters.Add("pCOMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DESCRIPT_CODE", OracleDbType.Varchar2).Value = partNo.ToUpper() == "NULL" ? (object)DBNull.Value : partNo;
                cmd.Parameters.Add("pSHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("cReturn", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM020P_GetDESCRIPT");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM020P_Insert")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM020P_Insert([FromBody] dynamic pData)
        {
            try
            {

                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM020P_INSERT";

                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DESCRIPT_CODE", OracleDbType.Varchar2).Value = pData.DESCRIPT_CODE;
                cmd.Parameters.Add("P_DESCRIPT_NAME", OracleDbType.Varchar2).Value = pData.DESCRIPT_NAME;
                cmd.Parameters.Add("P_SALES_CLASS", OracleDbType.Varchar2).Value = pData.SALES_CLASS;
                cmd.Parameters.Add("P_DISCOUNT_RATE", OracleDbType.Varchar2).Value = pData.DISCOUNT_RATE;
                cmd.Parameters.Add("P_PRODUCT_CODE", OracleDbType.Varchar2).Value = pData.PRODUCT_CODE;
                cmd.Parameters.Add("P_PART_SIZE", OracleDbType.Varchar2).Value = pData.PART_SIZE;
                cmd.Parameters.Add("P_ATTRIBUTE1", OracleDbType.Varchar2).Value = pData.ATTRIBUTE1;
                cmd.Parameters.Add("P_ATTRIBUTE2", OracleDbType.Varchar2).Value = pData.ATTRIBUTE2;
                cmd.Parameters.Add("P_ATTRIBUTE3", OracleDbType.Varchar2).Value = pData.ATTRIBUTE3;
                cmd.Parameters.Add("P_ATTRIBUTE4", OracleDbType.Varchar2).Value = pData.ATTRIBUTE4;
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM020P";
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_IP_ADDR", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("cReturn", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM020P_INSERT");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM020P_Update")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM020P_Update([FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM020P_UPDATE";

                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DESCRIPT_CODE", OracleDbType.Varchar2).Value = pData.DESCRIPT_CODE;
                cmd.Parameters.Add("P_DESCRIPT_NAME", OracleDbType.Varchar2).Value = pData.DESCRIPT_NAME;
                cmd.Parameters.Add("P_SALES_CLASS", OracleDbType.Varchar2).Value = pData.SALES_CLASS;
                cmd.Parameters.Add("P_DISCOUNT_RATE", OracleDbType.Varchar2).Value = pData.DISCOUNT_RATE;
                cmd.Parameters.Add("P_PART_SIZE", OracleDbType.Varchar2).Value = pData.PART_SIZE;
                cmd.Parameters.Add("P_PRODUCT_CODE", OracleDbType.Varchar2).Value = pData.PRODUCT_CODE;
                cmd.Parameters.Add("P_ATTRIBUTE1", OracleDbType.Varchar2).Value = pData.ATTRIBUTE1;
                cmd.Parameters.Add("P_ATTRIBUTE2", OracleDbType.Varchar2).Value = pData.ATTRIBUTE2;
                cmd.Parameters.Add("P_ATTRIBUTE3", OracleDbType.Varchar2).Value = pData.ATTRIBUTE3;
                cmd.Parameters.Add("P_ATTRIBUTE4", OracleDbType.Varchar2).Value = pData.ATTRIBUTE4;
                cmd.Parameters.Add("P_LAST_PGM", OracleDbType.Varchar2).Value = "SNM020P";
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_IP_ADDR", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM020P_UPDATE");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNM020P_Delete")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM020P_Delete([FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM020P_DELETE";

                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DESCRIPT_CODE", OracleDbType.Varchar2).Value = pData.DESCRIPT_CODE;
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM020P";
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_IP_ADDR", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("cReturn", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM020P_DELETE");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM020P_Download")]
        [Authorize]
        public HttpResponseMessage SNM020P_Download(List<DataModel.SNM020P> Data)
        {
            HttpResponseMessage response = new HttpResponseMessage(HttpStatusCode.OK);
            string gLocation = "~/tempFile/" + "SNM020P" + "/";
            string gWriteLocation = System.Web.HttpContext.Current.Server.MapPath(gLocation);
            FileInfo filePath = new FileInfo(gWriteLocation);
            filePath.Directory.Create();
            string gFileLocation = Path.Combine(gWriteLocation, "SNM020P.xlsx");
            string gReturnLocation = Path.Combine(gLocation, "SNM020P.xlsx");
            var excel2 = Data.ToExcel(x => x
            .AddColumn("Description Code", z => z.DESCRIPT_CODE)
            .AddColumn("Description Name", z => z.DESCRIPT_NAME)
            .AddColumn("Sales Class", z => z.SALES_CLASS)
            .AddColumn("Discount Rate", z => z.DISCOUNT_RATE)
            .AddColumn("Product Code", z => z.PRODUCT_CODE)
            .AddColumn("Part Size", z => z.PART_SIZE)
            .AddColumn("Last Update By", z => z.LAST_UPD_BY)
            .AddColumn("Last Update Date", z => z.LAST_UPD_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(z.LAST_UPD_DATE).ToString("dd-MM-yyyy"))
            );
            byte[] excel = excel2;
            File.WriteAllBytes(gFileLocation, excel);
            string path = HttpContext.Current.Server.MapPath("~/" + gReturnLocation.Substring(1));
            if (!File.Exists(path))
            {
                return Request.CreateErrorResponse(HttpStatusCode.NotFound, "File not found.");
            }
            try
            {
                string mimeType = MimeMapping.GetMimeMapping("SNM020P.xlsx");
                StreamContent x = new StreamContent(new FileStream(path, FileMode.Open, FileAccess.Read));
                response.Content = x;
                response.Content.Headers.ContentType = new MediaTypeHeaderValue(mimeType);
                response.Content.Headers.ContentDisposition = new ContentDispositionHeaderValue("attachment");
                response.Content.Headers.ContentDisposition.FileName = "SNM020P.xlsx";
            }
            catch (Exception ex)
            {
                return Request.CreateResponse(HttpStatusCode.BadRequest, ex);
            }
            return response;
        }
        #endregion

        #region SNSE143_F2
        [HttpGet, Route("SNSE143_F2_GetDetailByFileName/{fileName}")]
        [Authorize]
        public IHttpActionResult SNSE143_F2_GetDetailByFileName(string fileName)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE143_F2_DETAILBYNAME";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_FILE_NAME", OracleDbType.Varchar2).Value = fileName;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }

        }

        [HttpGet, Route("SNSE143_F2_GetFileList")]
        [Authorize]
        public IHttpActionResult SNSE143_F2_GetFileList()
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE143_F2_FileList";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }

        }

        [HttpPost, Route("SNSE143_F2_TransferData/{fileName}")]
        [Authorize]
        public IHttpActionResult SNSE143_F2_TransferData(string fileName)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "PK_NP_SNSE143_F2.CONFIRM_PFPM_FOR_REGIS";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_FILE_NAME", OracleDbType.Varchar2).Value = fileName;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                oOracledb.SqlQueryReturnValue(cmd);

                return Ok();
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }

        }

        [HttpPost, Route("SNSE143_F2_UploadFile")]
        [Authorize]
        public async Task<IHttpActionResult> SNSE143_F2_UploadFile(HttpRequestMessage request)
        {
            if (!request.Content.IsMimeMultipartContent())
            {
                throw new HttpResponseException(HttpStatusCode.UnsupportedMediaType);
            }


            try
            {
                var reqParts = await request.Content.ReadAsMultipartAsync();
                foreach (var file in reqParts.Contents)
                {
                    var timestamp = DateTime.Now.ToString("yyyyMMdd");
                    var buffer = await file.ReadAsByteArrayAsync();
                    string gFileName = file.Headers.ContentDisposition.FileName.Trim('\"');


                    try
                    {
                        OracleCommand cmd = new OracleCommand();
                        cmd.CommandText = "SP_NP_SNSE143_F2_CHK_FILE_NAME";
                        cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                        cmd.Parameters.Add("P_FILE_NAME", OracleDbType.Varchar2).Value = gFileName;
                        cmd.Parameters.Add("C_RETURN_MSG", OracleDbType.Varchar2, 1000).Direction = ParameterDirection.Output;

                        oOracledb.SqlQueryReturnValue(cmd);


                        string STATUS_MSG = cmd.Parameters["C_RETURN_MSG"].Value.ToString();

                        if (STATUS_MSG == "DUPLICATE")
                        {
                            return Ok(STATUS_MSG);
                        }


                    }
                    catch (Exception ex)
                    {
                        oOracledb.WriteLog("ErrorHandle", ex.Message, "SP_NP_SNSE143_F2_CHECK_FILE_NAME");
                        return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
                        //throw new HttpResponseException(HttpStatusCode.InternalServerError);
                    }




                    //string gFileNameExtend = String.Concat(gFileName, '_', timestamp);
                    string gRootPath = ConfigurationManager.AppSettings["RootSharedPath"].ToString();
                    string gUploadLication = System.IO.Path.Combine(gRootPath, "tempFile/SNSE143_F2");
                    string gFileLocation = System.IO.Path.Combine(gUploadLication, gFileName);

                    DataService.SaveBytesToFileByLocation(buffer
                                                    , gUploadLication
                                                    , gFileLocation);

                    try
                    {
                        OracleCommand cmd = new OracleCommand();
                        cmd.CommandText = "PK_NP_SNSE143_F2.REMOVE_PFPM";
                        cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                        cmd.Parameters.Add("P_FILE_NAME", OracleDbType.Varchar2).Value = gFileName;

                        oOracledb.SqlQueryReturnValue(cmd);
                    }
                    catch (Exception ex)
                    {
                        oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE143_F2_UploadFile_REMOVE_PFPM");
                        throw new HttpResponseException(HttpStatusCode.InternalServerError);
                    }

                    //Begin read file
                    Stream stream = new MemoryStream(buffer);

                    using (StreamReader reader = new StreamReader(stream, Encoding.UTF8))
                    {
                        string line;
                        int itemNo = 1;
                        while ((line = reader.ReadLine()) != null)
                        {
                            //read line from memorystream
                            OracleCommand cmd = new OracleCommand();
                            cmd.CommandText = "PK_NP_SNSE143_F2.EXTRACT_PFPM";
                            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                            cmd.Parameters.Add("P_FILE_NAME", OracleDbType.Varchar2).Value = gFileName;
                            cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Decimal).Value = itemNo;
                            cmd.Parameters.Add("P_TEXT_DATA", OracleDbType.Varchar2).Value = line;
                            cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                            cmd = oOracledb.SqlQueryReturnValue(cmd);

                            //if(cmd.Parameters["P_STATUS"].Value.ToString() == "OK")
                            //{
                            itemNo++;
                            //}
                        }
                    }
                    return Ok(gFileName);
                }
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE143_F2_UploadFile_EXTRACT_PFPM");
                throw new HttpResponseException(HttpStatusCode.NotAcceptable);
            }

            throw new Exception("Something wrong");

        }
        #endregion

        #region SNSE197
        [HttpGet, Route("SNSE197_getPartNo/{partNo}/{NoOfShowRecords}")]
        [Authorize]
        public IHttpActionResult SNSE197_getPartNo(string partNo, string NoOfShowRecords)
        {
            try
            {
                DataSet ds = new DataSet();
                partNo = ismoUtil.escapeCharacter(partNo);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE197_PART_NO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = partNo.ToUpper() == "NULL" ? (object)DBNull.Value : partNo;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE197_getPartNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE197_All_Current")]
        [Authorize]

        public IHttpActionResult SNSE197_All_Current([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();
            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSE197_ALL_CURR";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = param.PART_NO;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNSE197_ShowInfo")]
        [Authorize]
        public IHttpActionResult SNSE197_ShowInfo([FromBody] dynamic pData)
        {
            try
            {

                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE197_SHOW_INFO1";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pData.PART_NO;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE197_Update_Net_Price")]
        [Authorize]
        public IHttpActionResult SNSE197_Update_Net_Price([FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE197_UPDATE_PRICE1";

                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART", OracleDbType.Varchar2).Value = pData.PART_NO;
                cmd.Parameters.Add("P_NET_PRICE1", OracleDbType.Int32).Value = pData.NET_PRICE_1;
                cmd.Parameters.Add("P_NET_PRICE2", OracleDbType.Int32).Value = pData.NET_PRICE_2;
                cmd.Parameters.Add("P_EFFECTIVE_DATE", OracleDbType.Date).Value = pData.EFFECTIVE_DATE;
                cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNSE197";
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE197_Update_Net_Price");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNSE197_Upload_CSV")]
        [Authorize]
        public async Task<IHttpActionResult> SNSE197_Upload_CSV(HttpRequestMessage request)
        {

            if (!request.Content.IsMimeMultipartContent())
            {
                throw new HttpResponseException(HttpStatusCode.UnsupportedMediaType);
            }

            var reqParts = await request.Content.ReadAsMultipartAsync();
            List<string> ErrorList = new List<string>();
            try
            {
                string fileLocation = string.Empty;
                foreach (var file in reqParts.Contents)
                {
                    var buffer = await file.ReadAsByteArrayAsync();
                    string fileName = Path.GetFileNameWithoutExtension(file.Headers.ContentDisposition.FileName.Trim('\"'));
                    string ext = Path.GetExtension(file.Headers.ContentDisposition.FileName.Trim('\"'));
                    string FullfileName = String.Concat(fileName, "-", DateTime.Now.ToString("yyyyMMddHHmm"), ext);
                    string outputLocation = "~/tempFile/" + "SNSE197" + "/";
                    string outputPath = System.Web.HttpContext.Current.Server.MapPath(outputLocation);
                    fileLocation = String.Concat(outputPath, FullfileName);

                    DataService.SaveBytesToFileByLocation(buffer
                                                    , fileLocation
                                                    , fileLocation);
                }


                ErrorList = SNSE197_InsertDataFromExcelFile(fileLocation);
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE197_Upload_CSV");
                throw new HttpResponseException(HttpStatusCode.NotAcceptable);
            }

            return Json(new { Error = ErrorList });
        }

        public string SNSE197_ClearTemp()
        {
            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSQ01E_DEL_TEMP";
            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return "DONE";
        }


        public List<string> SNSE197_InsertDataFromExcelFile(string fileLocation)
        {

            string fileLocationN = fileLocation;
            DataTable dt = new DataTable();
            SystemApiController sys = new SystemApiController();
            dt = sys.ReadDataExcel(fileLocationN);
            List<string> ErrList = new List<string>();
            int countrow = 1;

            SNSE197_ClearTemp();

            foreach (DataRow row in dt.Rows)
            {
                string p1 = null;
                string p2 = null;
                string p3 = null;
                string p4 = null;

                p1 = row["Part No."].ToString().Trim();
                p2 = row["Net Price 1"].ToString().Trim();
                p3 = row["Net Price 2"].ToString().Trim();
                p4 = row["Effective Date"].ToString().Trim();


                if (p1 != null && p2 != null && p3 != null && p4 != null)
                {
                    OracleCommand cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE197_INS_TEMP_UPLOAD";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = p1;
                    cmd.Parameters.Add("P_NET_PRICE_1", OracleDbType.Int32).Value = p2;
                    cmd.Parameters.Add("P_NET_PRICE_2", OracleDbType.Int32).Value = p3;
                    cmd.Parameters.Add("P_EFFECTIVE_DATE", OracleDbType.Date).Value = p4;

                    cmd = oOracledb.SqlQueryReturnValue(cmd);

                    string status = cmd.Parameters["P_STATUS"].Value.ToString();
                    if (status == "DONE")
                    {
                        ErrList.Add("Row " + countrow + " Upload complete");
                    }
                    else if (status == "FAIL")
                    {
                        ErrList.Add("Row " + countrow + " " + "Part No. :" + " " + row["Part No."].ToString().Trim() + " error please recheck");
                    }

                }
                else
                {
                    ErrList.Add("Row " + countrow + " " + "Part No. :" + " " + row["Part No."].ToString().Trim() + " error please recheck");
                }

                countrow++;
            }

            return ErrList;

        }


        [HttpGet, Route("SNSE197_GetTempData")]
        [Authorize]
        public IHttpActionResult SNSE197_GetTempData()
        {
            DataSet ds = new DataSet();



            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSE197_GET_TEMP_DATA";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        #endregion

        #region SNM039P
        [HttpPost, Route("SNM039P_getSearch")]
        [Authorize]
        public IHttpActionResult SNM039P_getSearch([FromBody] dynamic pData)
        {
            try
            {

                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM039P_SEARCH";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_SALES_CLASS", OracleDbType.Varchar2).Value = pData.SALES_CLASS;
                cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM039P_getSearch");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNM039P_Sales_Class/{saleClass}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM039P_Sales_Class(string saleClass, int noofshowrec)
        {
            try
            {
                DataSet ds = new DataSet();

                saleClass = ismoUtil.escapeCharacter(saleClass);

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM039P_SALE_CLASS";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_SALES_CLASS", OracleDbType.Varchar2).Value = saleClass.ToUpper() == "NULL" ? (object)DBNull.Value : saleClass;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM039P_Sales_Class");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM039P_Save_Class_Name")]
        [Authorize]
        public IHttpActionResult SNM039P_Save_Class_Name([FromBody] dynamic data)
        {
            try
            {
                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM039P_SAVE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_SALES_CLASS", OracleDbType.Varchar2).Value = data.SALES_CLASS;
                cmd.Parameters.Add("P_CLASS_NAME", OracleDbType.Varchar2).Value = data.CLASS_NAME;
                cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_LAST_UPD_DATE", OracleDbType.Date).Value = data.LAST_UPD_DATE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM039P_Save_Class_Name");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM039P_Delete")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM039P_Delete([FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM039P_DELETE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_SALES_CLASS", OracleDbType.Varchar2).Value = pData.SALES_CLASS;
                cmd.Parameters.Add("P_LAST_IP_ADDR", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM039P_Delete");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM039P_Insert")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM039P_Insert([FromBody] dynamic pData)
        {
            try
            {
                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM039P_INSERT";

                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_SALES_CLASS", OracleDbType.Varchar2).Value = pData.SALES_CLASS;
                cmd.Parameters.Add("P_CLASS_NAME", OracleDbType.Varchar2).Value = pData.CLASS_NAME;
                cmd.Parameters.Add("P_CREATE_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM039P";
                cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_ATTRIBUTE1", OracleDbType.Varchar2).Value = pData.ATTRIBUTE1;
                cmd.Parameters.Add("P_ATTRIBUTE2", OracleDbType.Varchar2).Value = pData.ATTRIBUTE2;
                cmd.Parameters.Add("P_ATTRIBUTE3", OracleDbType.Varchar2).Value = pData.ATTRIBUTE3;
                cmd.Parameters.Add("P_ATTRIBUTE4", OracleDbType.Varchar2).Value = pData.ATTRIBUTE4;
                cmd.Parameters.Add("P_ATTRIBUTE5", OracleDbType.Varchar2).Value = pData.ATTRIBUTE5;
                cmd.Parameters.Add("P_IP_ADDR", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM039P_INSERT");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM039P_Download")]
        [Authorize]
        public HttpResponseMessage SNM039P_Download(List<DataModel.SNM039P> Data)
        {
            HttpResponseMessage response = new HttpResponseMessage(HttpStatusCode.OK);
            string gLocation = "~/tempFile/" + "SNM039P" + "/";
            string gWriteLocation = System.Web.HttpContext.Current.Server.MapPath(gLocation);
            FileInfo filePath = new FileInfo(gWriteLocation);
            filePath.Directory.Create();
            string gFileLocation = Path.Combine(gWriteLocation, "SNM039P.xlsx");
            string gReturnLocation = Path.Combine(gLocation, "SNM039P.xlsx");

            var excel2 = Data.ToExcel(x => x
            .AddColumn("Sales Class", z => z.SALES_CLASS)
            .AddColumn("Class Name", z => z.CLASS_NAME)
            .AddColumn("Update By", z => z.LAST_UPD_BY)
            .AddColumn("Update Date", z => z.LAST_UPD_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(z.LAST_UPD_DATE).ToString("dd-MM-yyyy"))
            );

            byte[] excel = excel2;
            // byte[] excel = dt.ToExcel();
            File.WriteAllBytes(gFileLocation, excel);
            string path = HttpContext.Current.Server.MapPath("~/" + gReturnLocation.Substring(1));
            if (!File.Exists(path))
            {
                return Request.CreateErrorResponse(HttpStatusCode.NotFound, "File not found.");
            }
            try
            {
                string mimeType = MimeMapping.GetMimeMapping("SNM039P.xlsx");
                StreamContent x = new StreamContent(new FileStream(path, FileMode.Open, FileAccess.Read));
                response.Content = x;
                response.Content.Headers.ContentType = new MediaTypeHeaderValue(mimeType);
                response.Content.Headers.ContentDisposition = new ContentDispositionHeaderValue("attachment");
                response.Content.Headers.ContentDisposition.FileName = "SNM039P.xlsx";
            }
            catch (Exception ex)
            {
                return Request.CreateResponse(HttpStatusCode.BadRequest, ex);
            }
            return response;
        }

        #endregion


        #region SNSE144

        [HttpGet, Route("SNSE144_GSC/{gsc}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE144_VENDOR_CODE_LOCAL(string gsc, int noofshowrec)
        {
            DataSet ds = new DataSet();

            gsc = ismoUtil.escapeCharacter(gsc);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSE144_GET_GSC ";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_GSC", OracleDbType.Varchar2).Value = gsc.ToUpper() == "NULL" ? (object)DBNull.Value : gsc;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNSE144_VENDOR_CODE_LOCAL/{vendor}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE144_VENDOR_CODE_LOCAL(string vendor, int noofshowrec, [FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            vendor = ismoUtil.escapeCharacter(vendor);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSE144_GET_VENDOR_LOCAL";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = vendor.ToUpper() == "NULL" ? (object)DBNull.Value : vendor;
            cmd.Parameters.Add("P_GSC", OracleDbType.Varchar2).Value = param.SUPPLIER_CODE;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        [HttpGet, Route("SNSE_144_MAIN_MODEL/{mainmodel}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE_144_MAIN_MODEL(string mainmodel, int noofshowrec)
        {
            DataSet ds = new DataSet();

            mainmodel = ismoUtil.escapeCharacter(mainmodel);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSE144_GET_MAIN_MODEL";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_MAIN_MODEL", OracleDbType.Varchar2).Value = mainmodel.ToUpper() == "NULL" ? (object)DBNull.Value : mainmodel;
            //cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNSE144_PART_NO/{partno}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE144_PART_NO(string partno, int noofshowrec, [FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            partno = ismoUtil.escapeCharacter(partno);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSE144_GET_PART_NO";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_MAIN_MODEL", OracleDbType.Varchar2).Value = param.MAIN_MODEL;
            cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = partno.ToUpper() == "NULL" ? (object)DBNull.Value : partno;
            //cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNSE144_VENDOR_CODE/{vendor}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE144_VENDOR_CODE(string vendor, int noofshowrec, [FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            vendor = ismoUtil.escapeCharacter(vendor);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSE144_GET_VENDOR";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = vendor.ToUpper() == "NULL" ? (object)DBNull.Value : vendor;
            cmd.Parameters.Add("P_UPDATE_STATUS", OracleDbType.Varchar2).Value = param.UPDATE_STATUS;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        [HttpGet, Route("SNSE144_SOURCING_CODE/{sourcingCode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE144_SOURCING_CODE(string sourcingCode, int noofshowrec)
        {
            DataSet ds = new DataSet();

            sourcingCode = ismoUtil.escapeCharacter(sourcingCode);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSE144_SOURCING_CODE";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_SOURCING_CODE", OracleDbType.Varchar2).Value = sourcingCode.ToUpper() == "NULL" ? (object)DBNull.Value : sourcingCode;
            //cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNSE144_GET_FILE_NAME/{filename}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE144_GET_FILE_NAME(string filename, int noofshowrec, [FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            filename = ismoUtil.escapeCharacter(filename);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSE144_GET_FILE_NAME";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_FROM_TABLE", OracleDbType.Varchar2).Value = param.FROM_TABLE;
            cmd.Parameters.Add("P_FILE_NAME", OracleDbType.Varchar2).Value = filename.ToUpper() == "NULL" ? (object)DBNull.Value : filename;
            //cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNSE144_getSearch")]
        [Authorize]
        public IHttpActionResult SNSE144_getSearch([FromBody] dynamic pData)
        {
            try
            {

                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE144_SEARCH";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_MAIN_MODEL", OracleDbType.Varchar2).Value = pData.MAIN_MODEL;
                cmd.Parameters.Add("P_PART_NUMBER", OracleDbType.Varchar2).Value = pData.PART_NUMBER;
                cmd.Parameters.Add("P_COMPLETE_STATUS", OracleDbType.Varchar2).Value = pData.COMPLETE_STATUS;
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData.VENDOR_CODE;
                cmd.Parameters.Add("P_FROM_TABLE", OracleDbType.Varchar2).Value = pData.FROM_TABLE;
                cmd.Parameters.Add("P_UPDATE_STATUS", OracleDbType.Varchar2).Value = pData.UPDATE_STATUS;
                cmd.Parameters.Add("P_SOURCING_CODE", OracleDbType.Varchar2).Value = pData.SOURCING_CODE;
                cmd.Parameters.Add("P_SOURCING_TYPE", OracleDbType.Varchar2).Value = pData.SOURCING_TYPE;
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_FORM", OracleDbType.Varchar2).Value = "SNSE144";
                cmd.Parameters.Add("P_FILE_NAME", OracleDbType.Varchar2).Value = pData.FILE_NAME;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_RETURN2", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {

                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE144_getSearch");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNSE144_Vendor_Master/{vendorMaster}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNSE144_Vendor_Master(string vendorMaster, int noofshowrec)
        {
            DataSet ds = new DataSet();

            vendorMaster = ismoUtil.escapeCharacter(vendorMaster);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSE144_VENDOR_MASTER";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = vendorMaster.ToUpper() == "NULL" ? (object)DBNull.Value : vendorMaster;
            cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        [HttpGet, Route("SNSE144_GetLog")]
        [Authorize]
        public IHttpActionResult SNSE144_GetLog()
        {
            DataSet ds = new DataSet();



            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNSE144_GET_LOG";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNSE144_UpdateData")]
        [Authorize]
        public IHttpActionResult SNSE144_UpdateData([FromBody] dynamic pData)
        {
            try
            {
                int iCount = 0;
                OracleCommand cmd = new OracleCommand();
                foreach (var detail in pData.List)
                {
                    iCount++;

                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE144_SAVE_TEMP_UPLOAD";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = detail["COMPANY_CODE"];
                    cmd.Parameters.Add("P_FILE_NAME", OracleDbType.Varchar2).Value = detail["FILE_NAME"];
                    cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Varchar2).Value = detail["ITEM_NO"];
                    cmd.Parameters.Add("P_PART_NUMBER", OracleDbType.Varchar2).Value = detail["PART_NUMBER"];
                    cmd.Parameters.Add("P_PART_NAME_EN", OracleDbType.Varchar2).Value = detail["PART_NAME_EN"];
                    cmd.Parameters.Add("P_PART_NAME_TH", OracleDbType.Varchar2).Value = detail["PART_NAME_TH"];
                    cmd.Parameters.Add("P_PART_CODE", OracleDbType.Varchar2).Value = detail["PART_CODE"];
                    cmd.Parameters.Add("P_DESCRIPT_CODE", OracleDbType.Varchar2).Value = detail["DESCRIPT_CODE"];
                    cmd.Parameters.Add("P_PRODUCT_CODE", OracleDbType.Varchar2).Value = detail["PRODUCT_CODE"];
                    cmd.Parameters.Add("P_SALES_CLASS", OracleDbType.Varchar2).Value = detail["SALES_CLASS"];
                    cmd.Parameters.Add("P_TARIFF_CODE", OracleDbType.Varchar2).Value = detail["TARIFF_CODE"];
                    cmd.Parameters.Add("P_DUTY_RATE", OracleDbType.Varchar2).Value = detail["DUTY_RATE"];
                    cmd.Parameters.Add("P_PARENT_PART_NUMBER", OracleDbType.Varchar2).Value = detail["PARENT_PART_NUMBER"];
                    cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = detail["VENDOR_CODE"];
                    cmd.Parameters.Add("P_VENDOR_TYPE", OracleDbType.Varchar2).Value = detail["VENDOR_TYPE"];
                    cmd.Parameters.Add("P_SUPPLIER_CODE", OracleDbType.Varchar2).Value = detail["SUPPLIER_CODE"];
                    cmd.Parameters.Add("P_PRICING_CODE", OracleDbType.Varchar2).Value = detail["PRICING_CODE"];
                    cmd.Parameters.Add("P_CURRENCY_CODE", OracleDbType.Varchar2).Value = detail["CURRENCY_CODE"];
                    cmd.Parameters.Add("P_CATALOG_MODEL", OracleDbType.Varchar2).Value = detail["CATALOG_MODEL"];
                    cmd.Parameters.Add("P_MAIN_MODEL", OracleDbType.Varchar2).Value = detail["MAIN_MODEL"];
                    cmd.Parameters.Add("P_ONLINE_FLAG", OracleDbType.Varchar2).Value = detail["ONLINE_FLAG"];
                    cmd.Parameters.Add("P_SOURCING_CODE", OracleDbType.Varchar2).Value = detail["SOURCING_CODE"];
                    cmd.Parameters.Add("P_CAR_PRODUCTION_LOCATION", OracleDbType.Varchar2).Value = detail["CAR_PRODUCTION_LOCATION"];
                    cmd.Parameters.Add("P_UNIT_PRODUCTION_LOCATION", OracleDbType.Varchar2).Value = detail["UNIT_PRODUCTION_LOCATION"];
                    cmd.Parameters.Add("P_PROCUREMENT_HUB", OracleDbType.Varchar2).Value = detail["PROCUREMENT_HUB"];
                    cmd.Parameters.Add("P_FROM_TABLE", OracleDbType.Varchar2).Value = detail["FROM_TABLE"];
                    cmd.Parameters.Add("P_COMPLETE_STATUS", OracleDbType.Varchar2).Value = detail["COMPLETE_STATUS"];
                    cmd.Parameters.Add("P_UPDATE_STATUS", OracleDbType.Varchar2).Value = detail["UPDATE_STATUS"];
                    cmd.Parameters.Add("P_CLEAR_DATA", OracleDbType.Varchar2).Value = iCount == 1 ? "Y" : "N";
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                    oOracledb.SqlQueryReturnValue(cmd);

                }

                DataSet ds = new DataSet();
                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE144_UPDATE_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_MAIN_MODEL", OracleDbType.Varchar2).Value = pData.MAIN_MODEL;
                cmd.Parameters.Add("P_PART_NUMBER", OracleDbType.Varchar2).Value = pData.PART_NUMBER;
                cmd.Parameters.Add("P_COMPLETE_STATUS", OracleDbType.Varchar2).Value = pData.COMPLETE_STATUS;
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = pData.VENDOR_CODE;
                cmd.Parameters.Add("P_FROM_TABLE", OracleDbType.Varchar2).Value = pData.FROM_TABLE;
                cmd.Parameters.Add("P_UPDATE_STATUS", OracleDbType.Varchar2).Value = pData.UPDATE_STATUS;
                cmd.Parameters.Add("P_SOURCING_CODE", OracleDbType.Varchar2).Value = pData.SOURCING_CODE;
                cmd.Parameters.Add("P_SOURCING_TYPE", OracleDbType.Varchar2).Value = pData.SOURCING_TYPE;
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_FORM", OracleDbType.Varchar2).Value = "SNSE144";
                cmd.Parameters.Add("P_STATUS", OracleDbType.Varchar2, 100).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                return Ok("DONE");
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE144_UpdateData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpPost, Route("SNSE144_Register")]
        public IHttpActionResult SNSE144_Register([FromBody] dynamic[] pData)
        {
            DataSet ds = new DataSet();
            int iCount = 0;
            try
            {
                OracleCommand cmd = new OracleCommand();


                foreach (var detail in pData)
                {
                    iCount++;
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE144_SAVE_TEMP";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = detail["COMPANY_CODE"];
                    cmd.Parameters.Add("P_FILE_NAME", OracleDbType.Varchar2).Value = detail["FILE_NAME"];
                    cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Int16).Value = detail["ITEM_NO"];
                    cmd.Parameters.Add("P_UPDATE_STATUS", OracleDbType.Varchar2).Value = detail["UPDATE_STATUS"];
                    cmd.Parameters.Add("P_COMPLETE_STATUS", OracleDbType.Varchar2).Value = detail["COMPLETE_STATUS"];
                    cmd.Parameters.Add("P_MAIN_MODEL", OracleDbType.Varchar2).Value = detail["MAIN_MODEL"];
                    cmd.Parameters.Add("P_PART_NUMBER", OracleDbType.Varchar2).Value = detail["PART_NUMBER"];
                    cmd.Parameters.Add("P_PART_NAME_EN", OracleDbType.Varchar2).Value = detail["PART_NAME_EN"];
                    cmd.Parameters.Add("P_PART_NAME_TH", OracleDbType.Varchar2).Value = detail["PART_NAME_TH"];
                    cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = detail["VENDOR_CODE"];
                    cmd.Parameters.Add("P_VENDOR_TYPE", OracleDbType.Varchar2).Value = detail["VENDOR_TYPE"];
                    cmd.Parameters.Add("P_CURRENCY_CODE", OracleDbType.Varchar2).Value = detail["CURRENCY_CODE"];
                    cmd.Parameters.Add("P_ONLINE_FLAG", OracleDbType.Varchar2).Value = detail["ONLINE_FLAG"];
                    cmd.Parameters.Add("P_PRODUCT_CODE", OracleDbType.Varchar2).Value = detail["PRODUCT_CODE"];
                    cmd.Parameters.Add("P_DESCRIPT_CODE", OracleDbType.Varchar2).Value = detail["DESCRIPT_CODE"];
                    cmd.Parameters.Add("P_PART_CODE", OracleDbType.Varchar2).Value = detail["PART_CODE"];
                    cmd.Parameters.Add("P_TARIFF_CODE", OracleDbType.Varchar2).Value = detail["TARIFF_CODE"];
                    cmd.Parameters.Add("P_DUTY_RATE", OracleDbType.Varchar2).Value = detail["DUTY_RATE"];
                    cmd.Parameters.Add("P_SALES_CLASS", OracleDbType.Varchar2).Value = detail["SALES_CLASS"];
                    cmd.Parameters.Add("P_CATALOG_MODEL", OracleDbType.Varchar2).Value = detail["CATALOG_MODEL"];
                    cmd.Parameters.Add("P_FROM_TABLE", OracleDbType.Varchar2).Value = detail["FROM_TABLE"];
                    cmd.Parameters.Add("P_SOURCING_CODE", OracleDbType.Varchar2).Value = detail["SOURCING_CODE"];
                    cmd.Parameters.Add("P_SUPPLIER_CODE", OracleDbType.Varchar2).Value = detail["SUPPLIER_CODE"];
                    cmd.Parameters.Add("P_CLEAR_DATA", OracleDbType.Varchar2).Value = iCount == 1 ? "Y" : "N";
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                    oOracledb.SqlQueryReturnValue(cmd);

                }


                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE144_REGIS_NEW_PART";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_FORM", OracleDbType.Varchar2).Value = "SNSE144";
                cmd.Parameters.Add("P_IP", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress;
                cmd.Parameters.Add("P_STATUS", OracleDbType.Varchar2, 250).Direction = ParameterDirection.Output;

                oOracledb.SqlQueryReturnValue(cmd);

                return Ok("DONE");
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE144_Register");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE144_CancelUpdate")]
        public IHttpActionResult SNSE144_CancelUpdate([FromBody] dynamic[] pData)
        {
            DataSet ds = new DataSet();
            int iCount = 0;
            try
            {
                OracleCommand cmd = new OracleCommand();


                foreach (var detail in pData)
                {
                    iCount++;
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNSE144_SAVE_TEMP";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = detail["COMPANY_CODE"];
                    cmd.Parameters.Add("P_FILE_NAME", OracleDbType.Varchar2).Value = detail["FILE_NAME"];
                    cmd.Parameters.Add("P_ITEM_NO", OracleDbType.Int16).Value = detail["ITEM_NO"];
                    cmd.Parameters.Add("P_UPDATE_STATUS", OracleDbType.Varchar2).Value = detail["UPDATE_STATUS"];
                    cmd.Parameters.Add("P_COMPLETE_STATUS", OracleDbType.Varchar2).Value = detail["COMPLETE_STATUS"];
                    cmd.Parameters.Add("P_MAIN_MODEL", OracleDbType.Varchar2).Value = detail["MAIN_MODEL"];
                    cmd.Parameters.Add("P_PART_NUMBER", OracleDbType.Varchar2).Value = detail["PART_NUMBER"];
                    cmd.Parameters.Add("P_PART_NAME_EN", OracleDbType.Varchar2).Value = detail["PART_NAME_EN"];
                    cmd.Parameters.Add("P_PART_NAME_TH", OracleDbType.Varchar2).Value = detail["PART_NAME_TH"];
                    cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = detail["VENDOR_CODE"];
                    cmd.Parameters.Add("P_VENDOR_TYPE", OracleDbType.Varchar2).Value = detail["VENDOR_TYPE"];
                    cmd.Parameters.Add("P_CURRENCY_CODE", OracleDbType.Varchar2).Value = detail["CURRENCY_CODE"];
                    cmd.Parameters.Add("P_ONLINE_FLAG", OracleDbType.Varchar2).Value = detail["ONLINE_FLAG"];
                    cmd.Parameters.Add("P_PRODUCT_CODE", OracleDbType.Varchar2).Value = detail["PRODUCT_CODE"];
                    cmd.Parameters.Add("P_DESCRIPT_CODE", OracleDbType.Varchar2).Value = detail["DESCRIPT_CODE"];
                    cmd.Parameters.Add("P_PART_CODE", OracleDbType.Varchar2).Value = detail["PART_CODE"];
                    cmd.Parameters.Add("P_TARIFF_CODE", OracleDbType.Varchar2).Value = detail["TARIFF_CODE"];
                    cmd.Parameters.Add("P_DUTY_RATE", OracleDbType.Varchar2).Value = detail["DUTY_RATE"];
                    cmd.Parameters.Add("P_SALES_CLASS", OracleDbType.Varchar2).Value = detail["SALES_CLASS"];
                    cmd.Parameters.Add("P_CATALOG_MODEL", OracleDbType.Varchar2).Value = detail["CATALOG_MODEL"];
                    cmd.Parameters.Add("P_FROM_TABLE", OracleDbType.Varchar2).Value = detail["FROM_TABLE"];
                    cmd.Parameters.Add("P_SOURCING_CODE", OracleDbType.Varchar2).Value = detail["SOURCING_CODE"];
                    cmd.Parameters.Add("P_SUPPLIER_CODE", OracleDbType.Varchar2).Value = detail["SUPPLIER_CODE"];
                    cmd.Parameters.Add("P_CLEAR_DATA", OracleDbType.Varchar2).Value = iCount == 1 ? "Y" : "N";
                    cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);

                    oOracledb.SqlQueryReturnValue(cmd);

                }


                cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE144_CANCEL_UPDATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_FORM", OracleDbType.Varchar2).Value = "SNSE144";
                cmd.Parameters.Add("P_STATUS", OracleDbType.Varchar2, 250).Direction = ParameterDirection.Output;

                oOracledb.SqlQueryReturnValue(cmd);

                return Ok("DONE");
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE144_CancelUpdate");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }



        [HttpPost, Route("SNSE144_Download")]
        public HttpResponseMessage SNSE144_Download(List<DataModel.SNSE144> dModel)
        {
            try
            {
                DataTable dt = new DataTable();
                string x = string.Empty;
                string filename = "SNSE144.csv";
                string floderLocation = "SNSE144";
                dt = DTFM.ToDatatable<DataModel.SNSE144>(dModel);
                x = DataService.WriteCSV(dt, filename, floderLocation, "SNSE144");
                Controllers.SystemApiController response = new Controllers.SystemApiController();
                var File = response.downloadFile(x.Substring(1), filename);

                return File;
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE144_Download");
                return Request.CreateResponse(HttpStatusCode.BadRequest, ex);
            }



        }


        public static DataTable ConvertCSVtoDataTable(string strFilePath, string Header)
        {
            DataTable dt = new DataTable();
            using (StreamReader sr = new StreamReader(strFilePath))
            {
                string[] headers = Header.Split(',');
                foreach (string header in headers)
                {
                    dt.Columns.Add(header);
                }
                while (!sr.EndOfStream)
                {
                    string[] rows = sr.ReadLine().Split(',');
                    DataRow dr = dt.NewRow();
                    for (int i = 0; i < headers.Length; i++)
                    {
                        dr[i] = rows[i];
                    }
                    dt.Rows.Add(dr);
                }

            }
            return dt;
        }

        [HttpPost, Route("SNSE144_UploadFile")]
        [Authorize]
        public async Task<IHttpActionResult> SNSE144_UploadFile(HttpRequestMessage request)
        {
            if (!request.Content.IsMimeMultipartContent())
            {
                throw new HttpResponseException(HttpStatusCode.UnsupportedMediaType);
            }
            var reqParts = await request.Content.ReadAsMultipartAsync();
            DataTable dt = new DataTable();
            try
            {

                foreach (var file in reqParts.Contents)
                {
                    var timestamp = DateTime.Now.ToString("yyyyMMdd");
                    var buffer = await file.ReadAsByteArrayAsync();
                    string gFileName = file.Headers.ContentDisposition.FileName.Trim('\"');
                    string gFileNameExtend = String.Concat(gFileName, '_', timestamp);
                    string gRootPath = ConfigurationManager.AppSettings["RootSharedPath"].ToString();
                    string gUploadLication = System.IO.Path.Combine(gRootPath, "tempFile/SNSE144");
                    string gFileLocation = System.IO.Path.Combine(gUploadLication, gFileNameExtend);

                    DataService.SaveBytesToFileByLocation(buffer
                                                    , gUploadLication
                                                    , gFileLocation);
                    //Begin read file
                    Stream stream = new MemoryStream(buffer);

                    using (StreamReader reader = new StreamReader(stream, Encoding.UTF8))
                    {
                        string[] Header = new string[]{
                                         "COMPANY_CODE",
                                         "FILE_NAME",
                                         "ITEM_NO",
                                         "PART_NUMBER",
                                         "PART_NAME_EN",
                                         "PART_NAME_TH",
                                         "PART_CODE",
                                         "DESCRIPT_CODE",
                                         "PRODUCT_CODE",
                                         "SALES_CLASS",
                                         "TARIFF_CODE",
                                         "DUTY_RATE",
                                         "PARENT_PART_NUMBER",
                                         "SOURCING_CODE",
                                         "SUPPLIER_CODE",
                                         "VENDOR_CODE",
                                         "VENDOR_TYPE",
                                         "VENDOR_CODE_PART",
                                         "VENDOR_TYPE_PART" ,
                                         "PRICING_CODE" ,
                                         "CURRENCY_CODE" ,
                                         "CATALOG_MODEL" ,
                                         "MAIN_MODEL" ,
                                         "CAR_PRODUCTION_LOCATION" ,
                                         "UNIT_PRODUCTION_LOCATION" ,
                                         "PROCUREMENT_HUB",
                                         "ONLINE_FLAG",
                                         "DIS_FROM_TABLE" ,
                                         "COMPLETE_STATUS" ,
                                         "UPDATE_STATUS" ,
                                         "CREATE_BY",
                                         "CREATE_DATE",
                                         "LAST_UPD_PGM",
                                         "LAST_UPD_BY",
                                         "LAST_UPD_DATE",
                                         "PACKAGING_CODE",
                                         "MSG_ERROR"
                                        };

                        foreach (string header in Header)
                        {
                            dt.Columns.Add(header);
                        }
                        string line;
                        int itemNo = 0;
                        while ((line = reader.ReadLine()) != null)
                        {
                            if (itemNo == 0)
                            {
                                itemNo++;
                                continue;
                            }  // Skip first line

                            //read line from memorystream
                            //string[] rows = Regex.Split(line, ",(?=(?:[^\"]*\"[^\"]*\")*[^\"]*$)");
                            Regex CSVParser = new Regex(",(?=(?:[^\"]*\"[^\"]*\")*(?![^\"]*\"))");
                            String[] rows = CSVParser.Split(line);
                            DataRow dr = dt.NewRow();
                            for (int i = 0; i < Header.Length; i++)
                            {
                                dr[i] = rows[i].Replace("\"", "");
                            }
                            dt.Rows.Add(dr);


                            itemNo++;

                        }
                    }

                }

                return Json(DTFM.convertToList(dt));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE144_UploadFile");
                throw new HttpResponseException(HttpStatusCode.NotAcceptable);
            }

            throw new Exception("Something wrong");

        }


        [HttpPost, Route("SNSE144_UploadExcel")]
        [Authorize]
        public async Task<IHttpActionResult> SNSE144_UploadExcel(HttpRequestMessage request)
        {

            if (!request.Content.IsMimeMultipartContent())
            {
                throw new HttpResponseException(HttpStatusCode.UnsupportedMediaType);
            }

            var reqParts = await request.Content.ReadAsMultipartAsync();
            DataTable dt = new DataTable();
            DataTable dt2 = new DataTable();

            dt2.Columns.Add("CHECK");
            dt2.Columns.Add("SEQ");
            dt2.Columns.Add("COMPANY_CODE");
            dt2.Columns.Add("FILE_NAME");
            dt2.Columns.Add("ITEM_NO");
            dt2.Columns.Add("PART_NUMBER");
            dt2.Columns.Add("PART_NAME_EN");
            dt2.Columns.Add("PART_NAME_TH");
            dt2.Columns.Add("PART_CODE");
            dt2.Columns.Add("DESCRIPT_CODE");
            dt2.Columns.Add("PRODUCT_CODE");
            dt2.Columns.Add("SALES_CLASS");
            dt2.Columns.Add("TARIFF_CODE");
            dt2.Columns.Add("DUTY_RATE");
            dt2.Columns.Add("PARENT_PART_NUMBER");
            dt2.Columns.Add("SOURCING_CODE");
            dt2.Columns.Add("SUPPLIER_CODE");
            dt2.Columns.Add("VENDOR_CODE");
            dt2.Columns.Add("VENDOR_TYPE");
            dt2.Columns.Add("VENDOR_CODE_PART");
            dt2.Columns.Add("VENDOR_TYPE_PART");
            dt2.Columns.Add("PRICING_CODE");
            dt2.Columns.Add("CURRENCY_CODE");
            dt2.Columns.Add("CATALOG_MODEL");
            dt2.Columns.Add("MAIN_MODEL");
            dt2.Columns.Add("CAR_PRODUCTION_LOCATION");
            dt2.Columns.Add("UNIT_PRODUCTION_LOCATION");
            dt2.Columns.Add("PROCUREMENT_HUB");
            dt2.Columns.Add("ONLINE_FLAG");
            dt2.Columns.Add("DIS_FROM_TABLE");
            dt2.Columns.Add("COMPLETE_STATUS");
            dt2.Columns.Add("UPDATE_STATUS");
            dt2.Columns.Add("CREATE_BY");
            dt2.Columns.Add("CREATE_DATE");
            dt2.Columns.Add("LAST_UPD_PGM");
            dt2.Columns.Add("LAST_UPD_BY");
            dt2.Columns.Add("LAST_UPD_DATE");
            dt2.Columns.Add("PACKAGING_CODE");
            dt2.Columns.Add("MSG_ERROR");
            try
            {


                string fileLocation = string.Empty;
                foreach (var file in reqParts.Contents)
                {
                    var buffer = await file.ReadAsByteArrayAsync();
                    string fileName = Path.GetFileNameWithoutExtension(file.Headers.ContentDisposition.FileName.Trim('\"'));
                    //string ext = Path.GetExtension(file.Headers.ContentDisposition.FileName.Trim('\"'));
                    //string FullfileName = String.Concat(fileName, "-", DateTime.Now.ToString("yyyyMMddHHmm"), ext);
                    string outputLocation = "~/tempFile/" + "SNSE144" + "/";
                    string outputPath = System.Web.HttpContext.Current.Server.MapPath(outputLocation);
                    fileLocation = String.Concat(outputPath, fileName);

                    DataService.SaveBytesToFileByLocation(buffer
                                                    , fileLocation
                                                    , fileLocation);
                }


                SystemApiController sys = new SystemApiController();
                dt = sys.ReadDataExcel(fileLocation);
                int itemNo = 0;
                foreach (DataRow rows in dt.Rows)
                {
                    if (itemNo == 0)
                    {
                        itemNo++;
                        continue;
                    }  // Skip first line

                    DataRow dr = dt2.NewRow();
                    for (int i = 2; i < 38; i++)
                    {
                        int j = i - 1;
                        dr[i] = rows[j];
                    }

                    if (dr["COMPLETE_STATUS"].ToString() == "Complete")
                    {
                        dr["COMPLETE_STATUS"] = "Y";
                    }
                    else
                    {
                        dr["COMPLETE_STATUS"] = "N";
                    }


                    if (dr["UPDATE_STATUS"].ToString() == "Register")
                    {
                        dr["UPDATE_STATUS"] = "Y";
                    }
                    else if (dr["UPDATE_STATUS"].ToString() == "Duplicate")
                    {
                        dr["UPDATE_STATUS"] = "D";
                    }
                    else if (dr["UPDATE_STATUS"].ToString() == "Dup-Dif")
                    {
                        dr["UPDATE_STATUS"] = "P";
                    }
                    else if (dr["UPDATE_STATUS"].ToString() == "Dup-Dif Updated")
                    {
                        dr["UPDATE_STATUS"] = "U";
                    }
                    else if (dr["UPDATE_STATUS"].ToString() == "Dup-Dif Cancelled")
                    {
                        dr["UPDATE_STATUS"] = "C";
                    }
                    else
                    {
                        dr["UPDATE_STATUS"] = "N";
                    }


                    dt2.Rows.Add(dr);
                }


                //ErrorList = SNM032P_InsertDataFromExcelFile(fileLocation);
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE144_UploadExcel");
                throw new HttpResponseException(HttpStatusCode.NotAcceptable);
            }

            return Json(DTFM.convertToList(dt2));
        }


        [HttpGet, Route("SNSE144_GenerateReport/{reportName}/{fileName}")]
        [Authorize]
        public async Task<IHttpActionResult> SNSE144_GenerateReport(string reportName, string fileName)
        {

            //Get querystring
            var queryStrings = Request.GetQueryNameValuePairs();

            var reportParams = new List<ParameterValue>();
            //Set report type EXCELOPENXML or PDF
            var fileType = "EXCELOPENXML";

            //Set output location & filename
            var outputLocation = "~/tempFile/" + "SNSE144" + "/";
            //string gFileName = file.Headers.ContentDisposition.FileName.Trim('\"');
            //string gFileNameExtend = String.Concat(fileName, '_', timestamp);
            //string gRootPath = ConfigurationManager.AppSettings["RootSharedPath"].ToString();
            //string gUploadLication = System.IO.Path.Combine(gRootPath, "tempFile/SNSE144");
            string gFileLocation = System.IO.Path.Combine(outputLocation, fileName);

            //Create report parameters
            foreach (var qs in queryStrings)
            {

                reportParams.Add(new ParameterValue
                {
                    Name = qs.Key,
                    Value = qs.Value,
                    Label = qs.Key
                });
            }

            try
            {
                //Generate report
                await ReportServices.GenerateReport(reportName, fileType, reportParams, fileName, outputLocation);

            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE144_GenerateReport");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }


            return Ok(gFileLocation);
        }


        [HttpPost, Route("SNSE144_DownloadExcel")]
        public HttpResponseMessage SNSE144_DownloadExcel([FromBody] dynamic pparam)
        {

            //string gRootPath = ConfigurationManager.AppSettings["RootSharedPath"].ToString();
            //string gUploadLication = System.IO.Path.Combine(gRootPath, "tempFile/SNSE144");
            string filename = pparam.filename;
            var outputLocation = "/tempFile/" + "SNSE144" + "/" + filename;

            Controllers.SystemApiController response = new Controllers.SystemApiController();
            var File = response.downloadFile(outputLocation, filename);

            return File;
        }



        #endregion

        #region SNSE162
        [HttpPost, Route("SNSE162_Insert")]
        [Authorize]
        [RunBatchGuard]
        public IHttpActionResult SNSE162_Insert([FromBody] dynamic param)
        {
            try
            {
                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE162_INSERT";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_SOURCING_CODE", OracleDbType.Varchar2).Value = param.SOURCING_CODE;
                cmd.Parameters.Add("P_SOURCING_GROUP", OracleDbType.Varchar2).Value = param.SOURCING_GROUP;
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = param.VENDOR_CODE;
                cmd.Parameters.Add("P_ORIGIN_CODE", OracleDbType.Varchar2).Value = param.ORIGIN_CODE;
                cmd.Parameters.Add("P_DESCRIPTION", OracleDbType.Varchar2).Value = param.DESCRIPTION;
                cmd.Parameters.Add("P_PFPM_REGIS_STATUS", OracleDbType.Varchar2).Value = param.PFPM_REGIS_STATUS;
                cmd.Parameters.Add("P_WVL_REGIS_STATUS", OracleDbType.Varchar2).Value = param.WVL_REGIS_STATUS;
                cmd.Parameters.Add("P_COMMENTS", OracleDbType.Varchar2).Value = param.COMMENTS;
                cmd.Parameters.Add("P_PGM", OracleDbType.Varchar2).Value = "SNSE162";
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_IP", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE162_Insert");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        [HttpPost, Route("SNSE162_Update")]
        [Authorize]
        [RunBatchGuard]
        public IHttpActionResult SNSE162_Update([FromBody] dynamic param)
        {
            try
            {
                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE162_UPDATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_SOURCING_CODE", OracleDbType.Varchar2).Value = param.SOURCING_CODE;
                cmd.Parameters.Add("P_SOURCING_GROUP", OracleDbType.Varchar2).Value = param.SOURCING_GROUP;
                cmd.Parameters.Add("P_VENDOR_CODE", OracleDbType.Varchar2).Value = param.VENDOR_CODE;
                cmd.Parameters.Add("P_ORIGIN_CODE", OracleDbType.Varchar2).Value = param.ORIGIN_CODE;
                cmd.Parameters.Add("P_DESCRIPTION", OracleDbType.Varchar2).Value = param.DESCRIPTION;
                cmd.Parameters.Add("P_PFPM_REGIS_STATUS", OracleDbType.Varchar2).Value = param.PFPM_REGIS_STATUS;
                cmd.Parameters.Add("P_WVL_REGIS_STATUS", OracleDbType.Varchar2).Value = param.WVL_REGIS_STATUS;
                cmd.Parameters.Add("P_COMMENTS", OracleDbType.Varchar2).Value = param.COMMENTS;
                cmd.Parameters.Add("P_PGM", OracleDbType.Varchar2).Value = "SNSE162";
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_IP", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE162_Update");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNSE162_Delete/{sorcing_code}")]
        [Authorize]
        [RunBatchGuard]
        public IHttpActionResult SNSE162_Delete(string sorcing_code)
        {
            try
            {
                DataSet ds = new DataSet();

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE162_DELETE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_SOURCING_CODE", OracleDbType.Varchar2).Value = sorcing_code;
                cmd.Parameters.Add("P_SYSTEM_USER", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_PGM", OracleDbType.Varchar2).Value = "SNSE162";
                cmd.Parameters.Add("P_IP", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE162_Delete");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE162_Download")]
        [Authorize]
        public HttpResponseMessage SNSE162_Download(List<DataModel.SNSE162> Data)
        {
            HttpResponseMessage response = new HttpResponseMessage(HttpStatusCode.OK);
            string gLocation = "~/tempFile/" + "SNSE162" + "/";
            string gWriteLocation = System.Web.HttpContext.Current.Server.MapPath(gLocation);
            FileInfo filePath = new FileInfo(gWriteLocation);
            filePath.Directory.Create();
            string gFileLocation = Path.Combine(gWriteLocation, "SNSE162.xlsx");
            string gReturnLocation = Path.Combine(gLocation, "SNSE162.xlsx");

            var excel2 = Data.ToExcel(x => x
            .AddColumn("Seq.", z => z.SEQ)
            .AddColumn("Sourcing Code", z => z.SOURCING_CODE)
            .AddColumn("Sourcing Group", z => z.SOURCING_GROUP)
            .AddColumn("Description", z => z.DESCRIPTION)
            .AddColumn("Vendor Code", z => z.VENDOR_CODE)
            .AddColumn("Origin Code", z => z.ORIGIN_CODE)
            .AddColumn("PFPM Regis Status", z => z.PFPM_REGIS_STATUS)
            .AddColumn("WVL Regis Status", z => z.WVL_REGIS_STATUS)
            .AddColumn("Comments", z => z.COMMENTS)
            .AddColumn("Create By", z => z.CREATE_BY)
            .AddColumn("Create Date", z => z.CREATE_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(z.CREATE_DATE).ToString("dd-MM-yyyy"))
            .AddColumn("Last Update By", z => z.LAST_UPD_BY)
            .AddColumn("Last Update Date", z => z.LAST_UPD_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(z.LAST_UPD_DATE).ToString("dd-MM-yyyy"))
            );

            byte[] excel = excel2;
            // byte[] excel = dt.ToExcel();
            File.WriteAllBytes(gFileLocation, excel);
            string path = HttpContext.Current.Server.MapPath("~/" + gReturnLocation.Substring(1));
            if (!File.Exists(path))
            {
                return Request.CreateErrorResponse(HttpStatusCode.NotFound, "File not found.");
            }
            try
            {
                string mimeType = MimeMapping.GetMimeMapping("SNSE162.xlsx");
                StreamContent x = new StreamContent(new FileStream(path, FileMode.Open, FileAccess.Read));
                response.Content = x;
                response.Content.Headers.ContentType = new MediaTypeHeaderValue(mimeType);
                response.Content.Headers.ContentDisposition = new ContentDispositionHeaderValue("attachment");
                response.Content.Headers.ContentDisposition.FileName = "SNSE162.xlsx";
            }
            catch (Exception ex)
            {
                return Request.CreateResponse(HttpStatusCode.BadRequest, ex);
            }
            return response;
        }

        #endregion

        #region SNM054P

        [HttpGet, Route("SNM054P_Discount_Code/{discountCode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM054P_Discount_Code(string discountCode, int noofshowrec)
        {
            DataSet ds = new DataSet();

            discountCode = ismoUtil.escapeCharacter(discountCode);

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM054P_DISCOUNT_CODE";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_DISCOUNT_CODE", OracleDbType.Varchar2).Value = discountCode.ToUpper() == "NULL" ? (object)DBNull.Value : discountCode;
            cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return Json(DTFM.convertToList(ds));
        }

        [HttpPost, Route("SNM054P_getSearch")]
        [Authorize]
        public IHttpActionResult SNM054P_getSearch([FromBody] dynamic pData)
        {
            try
            {

                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM054P_SEARCH";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DISCOUNT_CODE", OracleDbType.Varchar2).Value = pData.DISCOUNT_CODE;
                cmd.Parameters.Add("P_DISCOUNT_NAME", OracleDbType.Varchar2).Value = pData.DISCOUNT_NAME;
                cmd.Parameters.Add("P_CREATE_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_CREATE_DATE", OracleDbType.Varchar2).Value = pData.CREATE_DATE;
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM054P";
                cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_LAST_UPD_DATE", OracleDbType.Varchar2).Value = pData.LAST_UPD_DATE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM054P_Delete")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM054P_Delete([FromBody] dynamic pData)
        {

            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM054P_DELETE";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_DISCOUNT_CODE", OracleDbType.Varchar2).Value = pData.DISCOUNT_CODE;
            cmd.Parameters.Add("P_IP_ADDR", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));

        }

        [HttpPost, Route("SNM054P_Insert")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM054P_Insert([FromBody] dynamic pData)
        {

            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM054P_INSERT";

            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_DISCOUNT_CODE", OracleDbType.Varchar2).Value = pData.DISCOUNT_CODE;
            cmd.Parameters.Add("P_DISCOUNT_NAME", OracleDbType.Varchar2).Value = pData.DISCOUNT_NAME;
            cmd.Parameters.Add("P_CREATE_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
            cmd.Parameters.Add("P_CREATE_DATE", OracleDbType.Varchar2).Value = pData.CREATE_DATE;
            cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM054P";
            cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
            cmd.Parameters.Add("P_LAST_UPD_DATE", OracleDbType.Varchar2).Value = pData.LAST_UPD_DATE;
            cmd.Parameters.Add("P_ATTRIBUTE1", OracleDbType.Varchar2).Value = pData.ATTRIBUTE1;
            cmd.Parameters.Add("P_ATTRIBUTE2", OracleDbType.Varchar2).Value = pData.ATTRIBUTE2;
            cmd.Parameters.Add("P_ATTRIBUTE3", OracleDbType.Varchar2).Value = pData.ATTRIBUTE3;
            cmd.Parameters.Add("P_ATTRIBUTE4", OracleDbType.Varchar2).Value = pData.ATTRIBUTE4;
            cmd.Parameters.Add("P_ATTRIBUTE5", OracleDbType.Varchar2).Value = pData.ATTRIBUTE5;
            cmd.Parameters.Add("P_IP_ADDR", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));

        }

        [HttpPost, Route("SNM054P_Save")]
        [Authorize]
        public IHttpActionResult SNM054P_Save([FromBody] dynamic data)
        {
            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM054P_SAVE";
            cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
            cmd.Parameters.Add("P_DISCOUNT_CODE", OracleDbType.Varchar2).Value = data.DISCOUNT_CODE;
            cmd.Parameters.Add("P_DISCOUNT_NAME", OracleDbType.Varchar2).Value = data.DISCOUNT_NAME;
            //cmd.Parameters.Add("P_CREATE_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
            //cmd.Parameters.Add("P_CREATE_DATE", OracleDbType.Varchar2).Value = data.CREATE_DATE;
            //cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = data.LAST_UPD_PGM;
            //cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
            //cmd.Parameters.Add("P_LAST_UPD_DATE", OracleDbType.Date).Value = data.LAST_UPD_DATE;
            cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

            ds = oOracledb.SqlQueryDS_CMD(cmd);
            return Json(DTFM.convertToList(ds));
        }
        #endregion

        #region SNM114P
        [HttpGet, Route("SNM114P_GetCampaignID/{campaignId}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM114P_GetCampaignID(string campaignId, int noofshowrec)
        {
            DataSet ds = new DataSet();

            campaignId = ismoUtil.escapeCharacter(campaignId);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM114P_GET_CAMPAIGN_ID";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CAMPAIGN_ID", OracleDbType.Varchar2).Value = campaignId.ToUpper() == "NULL" ? (object)DBNull.Value : campaignId;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM114P_GetCampaignID");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM114P_InsertCampaignMaster")]
        [Authorize]
        public IHttpActionResult SNM114P_InsertCampaignMaster([FromBody] dynamic pParams)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();
            try
            {

                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM114P_INS_CAMPAIGN_MT";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CAMPAIGN_ID", OracleDbType.Varchar2).Value = pParams.CAMPAIGN_ID;
                cmd.Parameters.Add("P_CAMPAIGN_NAME", OracleDbType.Varchar2).Value = pParams.CAMPAIGN_NAME;
                cmd.Parameters.Add("P_EFFECTIVE_DATE", OracleDbType.Date).Value = pParams.EFFECTIVE_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(pParams.EFFECTIVE_DATE);
                cmd.Parameters.Add("P_DISABLE_DATE", OracleDbType.Date).Value = pParams.DISABLE_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(pParams.DISABLE_DATE);
                cmd.Parameters.Add("P_DISCOUNT_TYPE", OracleDbType.Varchar2).Value = pParams.DISCOUNT_TYPE;
                cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM114P";
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM114P_InsertCampaignMaster");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }



        //Order Factor (Tab 1)
        [HttpGet, Route("SNM114P_GetOrderClass/{orderClass}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM114P_GetOrderClass(string orderClass, int noofshowrec)
        {
            DataSet ds = new DataSet();

            orderClass = ismoUtil.escapeCharacter(orderClass);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM114P_GET_ORDER_CLASS";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = orderClass.ToUpper() == "NULL" ? (object)DBNull.Value : orderClass;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM114P_GetOrderClass");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM114P_GetOrderFactor")]
        [Authorize]
        public IHttpActionResult SNM114P_GetOrderFactor([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM114P_GET_ORDER_FACTOR";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CAMPAIGN_ID", OracleDbType.Varchar2).Value = pParam.CAMPAIGN_ID;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM114P_GetOrderFactor");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM114P_SaveOrderFactor")]
        [Authorize]
        public IHttpActionResult SNM114P_SaveOrderFactor([FromBody] dynamic[] pDataList)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();
            try
            {
                foreach (var data in pDataList)
                {
                    OracleCommand cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNM114P_INS_ORDER_FACTOR";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CAMPAIGN_ID", OracleDbType.Varchar2).Value = data.CAMPAIGN_ID;
                    cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = data.ORDER_CLASS;
                    cmd.Parameters.Add("P_ORDER_PREFIX", OracleDbType.Varchar2).Value = data.ORDER_PREFIX;
                    cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM114P";
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                    ds = oOracledb.SqlQueryDS_CMD(cmd);
                }
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM114P_SaveOrderFactor");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM114P_DeleteOrderFactor")]
        [Authorize]
        public IHttpActionResult SNM114P_DeleteOrderFactor([FromBody] dynamic[] pDataList)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();
            try
            {
                foreach (var data in pDataList)
                {
                    OracleCommand cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNM114P_DEL_ORDER_FACTOR";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CAMPAIGN_ID", OracleDbType.Varchar2).Value = data.CAMPAIGN_ID;
                    cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = data.ORDER_CLASS;
                    cmd.Parameters.Add("P_ORDER_PREFIX", OracleDbType.Varchar2).Value = data.ORDER_PREFIX;
                    cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                    ds = oOracledb.SqlQueryDS_CMD(cmd);
                }
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM114P_DeleteOrderFactor");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        //Customer Factor (Tab 2)
        [HttpPost, Route("SNM114P_GetCustomerFactor")]
        [Authorize]
        public IHttpActionResult SNM114P_GetCustomerFactor([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM114P_GET_CUST_FACTOR";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CAMPAIGN_ID", OracleDbType.Varchar2).Value = pParam.CAMPAIGN_ID;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM114P_GetCustomerFactor");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNM114P_GetSalesChannel/{salesChannel}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM114P_GetSalesChannel(string salesChannel, int noofshowrec)
        {
            DataSet ds = new DataSet();

            salesChannel = ismoUtil.escapeCharacter(salesChannel);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM114P_SALES_CHANNEL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_SALES_CHANNEL", OracleDbType.Varchar2).Value = salesChannel.ToUpper() == "NULL" ? (object)DBNull.Value : salesChannel;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM114P_GetSalesChannel");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM114P_GetCustomerCode/{customerCode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM114P_GetCustomerCode(string customerCode, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            customerCode = ismoUtil.escapeCharacter(customerCode);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM114P_CUSTOMER_CODE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = customerCode.ToUpper() == "NULL" ? (object)DBNull.Value : customerCode;
                cmd.Parameters.Add("P_SALES_CHANNEL", OracleDbType.Varchar2).Value = pParam.SALES_CHANNEL;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE009M_GetOrderNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM114P_GetBranchCode/{branchCode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM114P_GetBranchCode(string branchCode, int noofshowrec, [FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();

            branchCode = ismoUtil.escapeCharacter(branchCode);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM114_BRANCH_CODE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = branchCode.ToUpper() == "NULL" ? (object)DBNull.Value : branchCode;
                cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = pParam.CUSTOMER_CODE;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM114P_GetOrderClass");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM114P_SaveCustomerFactor")]
        [Authorize]
        public IHttpActionResult SNM114P_SaveCustomerFactor([FromBody] dynamic[] pDataList)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();
            try
            {
                foreach (var data in pDataList)
                {
                    OracleCommand cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNM114P_INS_CUST_FACTOR";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CAMPAIGN_ID", OracleDbType.Varchar2).Value = data.CAMPAIGN_ID;
                    cmd.Parameters.Add("P_SALES_CHANNEL", OracleDbType.Varchar2).Value = data.SALES_CHANNEL;
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = data.BRANCH_CODE;
                    cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM114P";
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                    ds = oOracledb.SqlQueryDS_CMD(cmd);
                }
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM114P_SaveCustomerFactor");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM114P_DeleteCustomerFactor")]
        [Authorize]
        public IHttpActionResult SNM114P_DeleteCustomerFactor([FromBody] dynamic[] pDataList)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();
            try
            {
                foreach (var data in pDataList)
                {
                    OracleCommand cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNM114P_DEL_CUST_FACTOR";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CAMPAIGN_ID", OracleDbType.Varchar2).Value = data.CAMPAIGN_ID;
                    cmd.Parameters.Add("P_SALES_CHANNEL", OracleDbType.Varchar2).Value = data.SALES_CHANNEL;
                    cmd.Parameters.Add("P_CUSTOMER_CODE", OracleDbType.Varchar2).Value = data.CUSTOMER_CODE;
                    cmd.Parameters.Add("P_BRANCH_CODE", OracleDbType.Varchar2).Value = data.BRANCH_CODE;
                    cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                    ds = oOracledb.SqlQueryDS_CMD(cmd);
                }
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM114P_DeleteCustomerFactor");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        //Discount By Defalut (Tab 3)
        [HttpPost, Route("SNM114P_GetDiscountDefault")]
        [Authorize]
        public IHttpActionResult SNM114P_GetDiscountDefault([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM114P_GET_DISC_DEFAULT";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CAMPAIGN_ID", OracleDbType.Varchar2).Value = pParam.CAMPAIGN_ID;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM114P_GetDiscountDefault");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNM114P_GetPartNo/{partNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM114P_GetPartNo(string partNo, int noofshowrec)
        {
            DataSet ds = new DataSet();

            partNo = ismoUtil.escapeCharacter(partNo);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM114P_GET_PART_NO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = partNo.ToUpper() == "NULL" ? (object)DBNull.Value : partNo;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM114P_GetPartNo");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNM114P_GetPartDef/{partNo}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM114P_GetPartDef(string partNo, int noofshowrec)
        {
            DataSet ds = new DataSet();

            partNo = ismoUtil.escapeCharacter(partNo);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM114P_GET_PART_DEF";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = partNo.ToUpper() == "NULL" ? (object)DBNull.Value : partNo;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM114P_GetPartDef");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM114P_SaveDiscountDefault")]
        [Authorize]
        public IHttpActionResult SNM114P_SaveDiscountDefault([FromBody] dynamic[] pDataList)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();
            try
            {
                foreach (var data in pDataList)
                {
                    OracleCommand cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNM114P_INS_DISC_DEFAULT";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CAMPAIGN_ID", OracleDbType.Varchar2).Value = data.CAMPAIGN_ID;
                    cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = data.PART_NO;
                    cmd.Parameters.Add("P_DISCOUNT_RATE1", OracleDbType.Int16).Value = data.DISCOUNT_RATE1;
                    cmd.Parameters.Add("P_DISCOUNT_RATE2", OracleDbType.Int16).Value = data.DISCOUNT_RATE2;
                    cmd.Parameters.Add("P_DISCOUNT_RATE3", OracleDbType.Int16).Value = data.DISCOUNT_RATE3;
                    cmd.Parameters.Add("P_NET_PRICE", OracleDbType.Varchar2).Value = data.NET_PRICE;
                    cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM114P";
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                    ds = oOracledb.SqlQueryDS_CMD(cmd);
                }
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM114P_SaveDiscountDefault");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM114P_DeleteDiscountDefault")]
        [Authorize]
        public IHttpActionResult SNM114P_DeleteDiscountDefault([FromBody] dynamic[] pDataList)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();
            try
            {
                foreach (var data in pDataList)
                {
                    OracleCommand cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNM114P_DEL_DISC_DEFAULT";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CAMPAIGN_ID", OracleDbType.Varchar2).Value = data.CAMPAIGN_ID;
                    cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = data.PART_NO;
                    cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                    ds = oOracledb.SqlQueryDS_CMD(cmd);
                }
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM114P_DeleteDiscountDefault");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }


        //Discount By Volume (Tab 4)
        [HttpPost, Route("SNM114P_GetDiscountVolumeHeader")]
        [Authorize]
        public IHttpActionResult SNM114P_GetDiscountVolumeHeader([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM114P_GET_DISC_VOL_H";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CAMPAIGN_ID", OracleDbType.Varchar2).Value = pParam.CAMPAIGN_ID;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM114P_GetDiscountVolumeHeader");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM114P_GetDiscountVolumeDetail")]
        [Authorize]
        public IHttpActionResult SNM114P_GetDiscountVolumeDetail([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM114P_GET_DISC_VOL_D";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CAMPAIGN_ID", OracleDbType.Varchar2).Value = pParam.CAMPAIGN_ID;
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pParam.PART_NO;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM114P_GetDiscountVolumeDetail");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM114P_GetMaxVolume/{maxVolume}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM114P_GetMaxVolume(string maxVolume, int noofshowrec)
        {
            DataSet ds = new DataSet();

            maxVolume = ismoUtil.escapeCharacter(maxVolume);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM114_MAX_VOLUME";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_MAX_VOLUME", OracleDbType.Varchar2).Value = maxVolume.ToUpper() == "NULL" ? (object)DBNull.Value : maxVolume;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM114P_GetMaxVolume");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM114P_SaveDiscountVolumeHead")]
        [Authorize]
        public IHttpActionResult SNM114P_SaveDiscountVolumeHead([FromBody] dynamic[] pDataList)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();
            try
            {
                foreach (var data in pDataList)
                {
                    OracleCommand cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNM114P_INS_DISC_VOL_H";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CAMPAIGN_ID", OracleDbType.Varchar2).Value = data.CAMPAIGN_ID;
                    cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = data.PART_NO;
                    cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM114P";
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                    ds = oOracledb.SqlQueryDS_CMD(cmd);
                }
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM114P_SaveDiscountVolumeHead");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }



        [HttpPost, Route("SNM114P_SaveDiscountVolumeDetail")]
        [Authorize]
        public IHttpActionResult SNM114P_SaveDiscountVolumeDetail([FromBody] dynamic[] pDataList)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();
            try
            {
                foreach (var data in pDataList)
                {
                    OracleCommand cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNM114P_INS_DISC_VOL_D";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CAMPAIGN_ID", OracleDbType.Varchar2).Value = data.CAMPAIGN_ID;
                    cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = data.PART_NO;
                    cmd.Parameters.Add("P_MAX_VOLUME", OracleDbType.Varchar2).Value = data.MAX_VOLUME;
                    cmd.Parameters.Add("P_DISCOUNT_RATE1", OracleDbType.Int16).Value = data.DISCOUNT_RATE1;
                    cmd.Parameters.Add("P_DISCOUNT_RATE2", OracleDbType.Int16).Value = data.DISCOUNT_RATE2;
                    cmd.Parameters.Add("P_DISCOUNT_RATE3", OracleDbType.Int16).Value = data.DISCOUNT_RATE3;
                    cmd.Parameters.Add("P_NET_PRICE", OracleDbType.Int16).Value = data.NET_PRICE;
                    cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM114P";
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                    ds = oOracledb.SqlQueryDS_CMD(cmd);
                }
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM114P_SaveDiscountVolumeDetail");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM114P_DeleteDiscountVolumeHead")]
        [Authorize]
        public IHttpActionResult SNM114P_DeleteDiscountVolumeHead([FromBody] dynamic[] pDataList)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();
            try
            {
                foreach (var data in pDataList)
                {
                    OracleCommand cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNM114P_DEL_DISC_VOL_H";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CAMPAIGN_ID", OracleDbType.Varchar2).Value = data.CAMPAIGN_ID;
                    cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = data.PART_NO;
                    cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                    ds = oOracledb.SqlQueryDS_CMD(cmd);
                }
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM114P_DeleteDiscountVolumeHead");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM114P_DeleteDiscountVolumeDetail")]
        [Authorize]
        public IHttpActionResult SNM114P_DeleteDiscountVolumeDetail([FromBody] dynamic[] pDataList)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();
            try
            {
                foreach (var data in pDataList)
                {
                    OracleCommand cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNM114P_DEL_DISC_VOL_D";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CAMPAIGN_ID", OracleDbType.Varchar2).Value = data.CAMPAIGN_ID;
                    cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = data.PART_NO;
                    cmd.Parameters.Add("P_MAX_VOLUME", OracleDbType.Varchar2).Value = data.MAX_VOLUME;
                    cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                    ds = oOracledb.SqlQueryDS_CMD(cmd);
                }
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM114P_DeleteDiscountVolumeDetail");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        //Discount By Package (Tab 5)
        [HttpPost, Route("SNM114P_GetDiscountPackageHeader")]
        [Authorize]
        public IHttpActionResult SNM114P_GetDiscountPackageHeader([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM114P_GET_DISC_PAC_H";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CAMPAIGN_ID", OracleDbType.Varchar2).Value = pParam.CAMPAIGN_ID;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM114P_GetDiscountPackageHeader");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM114P_GetDiscountPackageDetail")]
        [Authorize]
        public IHttpActionResult SNM114P_GetDiscountPackageDetail([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM114P_GET_DISC_PAC_D";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_CAMPAIGN_ID", OracleDbType.Varchar2).Value = pParam.CAMPAIGN_ID;
                cmd.Parameters.Add("P_PACKAGE_ID", OracleDbType.Varchar2).Value = pParam.PACKAGE_ID;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM114P_GetDiscountPackageDetail");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM114P_SaveDiscountPackageHead")]
        [Authorize]
        public IHttpActionResult SNM114P_SaveDiscountPackageHead([FromBody] dynamic[] pDataList)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();
            try
            {
                foreach (var data in pDataList)
                {
                    OracleCommand cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNM114P_INS_DISC_PAC_H";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CAMPAIGN_ID", OracleDbType.Varchar2).Value = data.CAMPAIGN_ID;
                    cmd.Parameters.Add("P_PACKAGE_ID", OracleDbType.Varchar2).Value = data.PACKAGE_ID;
                    cmd.Parameters.Add("P_PACKAGE_NAME", OracleDbType.Varchar2).Value = data.PACKAGE_NAME;
                    cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM114P";
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                    ds = oOracledb.SqlQueryDS_CMD(cmd);
                }
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM114P_SaveDiscountPackageHead");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM114P_SaveDiscountPackageDetail")]
        [Authorize]
        public IHttpActionResult SNM114P_SaveDiscountPackageDetail([FromBody] dynamic[] pDataList)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();
            try
            {
                foreach (var data in pDataList)
                {
                    OracleCommand cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNM114P_INS_DISC_PAC_D";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CAMPAIGN_ID", OracleDbType.Varchar2).Value = data.CAMPAIGN_ID;
                    cmd.Parameters.Add("P_PACKAGE_ID", OracleDbType.Varchar2).Value = data.PACKAGE_ID;
                    cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = data.PART_NO;
                    cmd.Parameters.Add("P_DISCOUNT_RATE1", OracleDbType.Int16).Value = data.DISCOUNT_RATE1;
                    cmd.Parameters.Add("P_DISCOUNT_RATE2", OracleDbType.Int16).Value = data.DISCOUNT_RATE2;
                    cmd.Parameters.Add("P_DISCOUNT_RATE3", OracleDbType.Int16).Value = data.DISCOUNT_RATE3;
                    cmd.Parameters.Add("P_NET_PRICE", OracleDbType.Int16).Value = data.NET_PRICE;
                    cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM114P";
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                    ds = oOracledb.SqlQueryDS_CMD(cmd);
                }
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM114P_SaveDiscountPackageDetail");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM114P_DeleteDiscountPackageHead")]
        [Authorize]
        public IHttpActionResult SNM114P_DeleteDiscountPackageHead([FromBody] dynamic[] pDataList)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();
            try
            {
                foreach (var data in pDataList)
                {
                    OracleCommand cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNM114P_DEL_DISC_PAC_H";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CAMPAIGN_ID", OracleDbType.Varchar2).Value = data.CAMPAIGN_ID;
                    cmd.Parameters.Add("P_PACKAGE_ID", OracleDbType.Varchar2).Value = data.PACKAGE_ID;
                    cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                    ds = oOracledb.SqlQueryDS_CMD(cmd);
                }
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM114P_DeleteDiscountPackageHead");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM114P_DeleteDiscountPackageDetail")]
        [Authorize]
        public IHttpActionResult SNM114P_DeleteDiscountPackageDetail([FromBody] dynamic[] pDataList)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();
            try
            {
                foreach (var data in pDataList)
                {
                    OracleCommand cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNM114P_DEL_DISC_PAC_D";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_CAMPAIGN_ID", OracleDbType.Varchar2).Value = data.CAMPAIGN_ID;
                    cmd.Parameters.Add("P_PACKAGE_ID", OracleDbType.Varchar2).Value = data.PACKAGE_ID;
                    cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = data.PART_NO;
                    cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                    ds = oOracledb.SqlQueryDS_CMD(cmd);
                }
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM114P_DeleteDiscountPackageDetail");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }




        [HttpPost, Route("SNM114P_ValidateNetPrice")]
        [Authorize]
        public IHttpActionResult SNM114P_ValidateNetPrice([FromBody] dynamic pParam)
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM114P_VALID_NET_PRICE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pParam.PART_NO;
                cmd.Parameters.Add("P_DISCOUNT_RATE1", OracleDbType.Varchar2).Value = pParam.DISCOUNT_RATE1;
                cmd.Parameters.Add("P_DISCOUNT_RATE2", OracleDbType.Varchar2).Value = pParam.DISCOUNT_RATE2;
                cmd.Parameters.Add("P_DISCOUNT_RATE3", OracleDbType.Varchar2).Value = pParam.DISCOUNT_RATE3;
                cmd.Parameters.Add("P_NET_PRICE", OracleDbType.Varchar2).Value = pParam.NET_PRICE;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM114P_ValidateNetPrice");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }




        [HttpPost, Route("SNM114P_Upload")]
        [Authorize]
        public async Task<IHttpActionResult> SNM114P_Upload(HttpRequestMessage request)
        {

            if (!request.Content.IsMimeMultipartContent())
            {
                throw new HttpResponseException(HttpStatusCode.UnsupportedMediaType);
            }

            var reqParts = await request.Content.ReadAsMultipartAsync();
            List<string> ErrorList = new List<string>();
            try
            {
                string fileLocation = string.Empty;
                foreach (var file in reqParts.Contents)
                {
                    var buffer = await file.ReadAsByteArrayAsync();
                    string fileName = Path.GetFileNameWithoutExtension(file.Headers.ContentDisposition.FileName.Trim('\"'));
                    string ext = Path.GetExtension(file.Headers.ContentDisposition.FileName.Trim('\"'));
                    string FullfileName = String.Concat(fileName, "-", DateTime.Now.ToString("yyyyMMddHHmm"), ext);
                    string outputLocation = "~/tempFile/" + "SNM114P" + "/";
                    string outputPath = System.Web.HttpContext.Current.Server.MapPath(outputLocation);
                    fileLocation = String.Concat(outputPath, FullfileName);

                    DataService.SaveBytesToFileByLocation(buffer
                                                    , fileLocation
                                                    , fileLocation);
                }

                ErrorList = SNM114P_InsertDataFromExcelFile(fileLocation);
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM114P_Upload");
                throw new HttpResponseException(HttpStatusCode.NotAcceptable);
            }

            return Json(new { Error = ErrorList });
        }

        public string SNM114P_ClearTemp()
        {
            DataSet ds = new DataSet();

            OracleCommand cmd = new OracleCommand();
            cmd.CommandText = "SP_NP_SNM114P_DEL_TEMP";
            ds = oOracledb.SqlQueryDS_CMD(cmd);

            return "DONE";
        }


        public List<string> SNM114P_InsertDataFromExcelFile(string fileLocation)
        {

            string fileLocationN = fileLocation;
            DataTable dt = new DataTable();
            SystemApiController sys = new SystemApiController();
            dt = sys.ReadDataExcel(fileLocationN);
            List<string> ErrList = new List<string>();
            int countrow = 0;

            SNM114P_ClearTemp();

            foreach (DataRow row in dt.Rows)
            {
                string p1 = null;
                string p2 = null;
                string p3 = null;
                string p4 = null;
                string p5 = null;

                p1 = row[0].ToString().Trim();
                p2 = row[1].ToString().Trim();
                p3 = row[2].ToString().Trim();
                p4 = row[3].ToString().Trim();
                p5 = row[4].ToString().Trim();


                if (p1 != null && p2 != null && p3 != null && p4 != null && p5 != null)
                {
                    OracleCommand cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNM114P_INS_TEMP_UPLOAD";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = p1;
                    cmd.Parameters.Add("P_DISCOUNT_RATE1", OracleDbType.Varchar2).Value = p2;
                    cmd.Parameters.Add("P_DISCOUNT_RATE2", OracleDbType.Varchar2).Value = p3;
                    cmd.Parameters.Add("P_DISCOUNT_RATE3", OracleDbType.Varchar2).Value = p4;
                    cmd.Parameters.Add("P_NET_PRICE", OracleDbType.Varchar2).Value = p5;
                    cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("P_STATUS", OracleDbType.Varchar2, 50).Direction = ParameterDirection.Output;

                    cmd = oOracledb.SqlQueryReturnValue(cmd);

                    string status = cmd.Parameters["P_STATUS"].Value.ToString();
                    if (status == "DONE")
                    {
                        ErrList.Add("Row " + countrow + " Upload complete");
                    }
                    else if (status == "FAIL")
                    {
                        ErrList.Add("Row " + countrow + " " + "Part No. :" + " " + row[0].ToString().Trim() + " error please recheck");
                    }

                }
                else
                {
                    ErrList.Add("Row " + countrow + " " + "Part No. :" + " " + row[0].ToString().Trim() + " error please recheck");
                }

                countrow++;
            }

            return ErrList;

        }


        [HttpGet, Route("SNM114P_GetTempData")]
        [Authorize]
        public IHttpActionResult SNM114P_GetTempData()
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM114P_GET_TEMP_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM114P_GetTempData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM114P_ExportExcel")]
        public HttpResponseMessage SNM114P_ExportExcel(List<DataModel.SNM114P> Data)
        {
            HttpResponseMessage response = new HttpResponseMessage(HttpStatusCode.OK);
            string gLocation = "~/tempFile/" + "SNM114P" + "/";
            string gWriteLocation = System.Web.HttpContext.Current.Server.MapPath(gLocation);
            FileInfo filePath = new FileInfo(gWriteLocation);
            filePath.Directory.Create();
            string gFileLocation = Path.Combine(gWriteLocation, "SNM114P.xlsx");
            string gReturnLocation = Path.Combine(gLocation, "SNM114P.xlsx");

            var excel2 = Data.ToExcel(x => x
            .AddColumn("Seq.", z => z.SEQ)
            .AddColumn("Part No.", z => z.PART_NO)
            .AddColumn("Discount Rate 1", z => z.DISCOUNT_RATE1)
            .AddColumn("Discount Rate 2", z => z.DISCOUNT_RATE2)
            .AddColumn("Discount Rate 3", z => z.DISCOUNT_RATE3)
            .AddColumn("Net Price", z => z.NET_PRICE)
            .AddColumn("Error Message", z => z.ERROR_MSG)
            );


            byte[] excel = excel2;
            //byte[] excel = Data.ToExcel();
            File.WriteAllBytes(gFileLocation, excel);
            string path = HttpContext.Current.Server.MapPath("~/" + gReturnLocation.Substring(1));
            if (!File.Exists(path))
            {
                return Request.CreateErrorResponse(HttpStatusCode.NotFound, "File not found.");
            }
            try
            {
                string mimeType = MimeMapping.GetMimeMapping("SNM114P.xlsx");
                StreamContent x = new StreamContent(new FileStream(path, FileMode.Open, FileAccess.Read));
                response.Content = x;
                response.Content.Headers.ContentType = new MediaTypeHeaderValue(mimeType);
                response.Content.Headers.ContentDisposition = new ContentDispositionHeaderValue("attachment");
                response.Content.Headers.ContentDisposition.FileName = "SNM114P.xlsx";
            }
            catch (Exception ex)
            {
                return Request.CreateResponse(HttpStatusCode.BadRequest, ex);
            }
            return response;
        }


        #endregion

        #region SNM122P
        [HttpGet, Route("SNM122P_SearchDiscountGroup/{discGroup}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM122P_SearchDiscountGroup(string discGroup, int noofshowrec)
        {
            DataSet ds = new DataSet();
            discGroup = ismoUtil.escapeCharacter(discGroup);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM122P_GET_DISC_GROUP";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DISCOUNT_GROUP", OracleDbType.Varchar2).Value = discGroup.ToUpper() == "NULL" ? (object)DBNull.Value : discGroup;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM122P_SearchDiscountGroup");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNM122P_SearchSalesChannel/{salesChannel}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM122P_SearchSalesChannel(string salesChannel, int noofshowrec)
        {
            DataSet ds = new DataSet();
            salesChannel = ismoUtil.escapeCharacter(salesChannel);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM122P_GET_SALE_CHANNEL";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_SALES_CHANNEL", OracleDbType.Varchar2).Value = salesChannel.ToUpper() == "NULL" ? (object)DBNull.Value : salesChannel;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM122P_SearchSalesChannel");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNM122P_SearchOrderClass/{orderClass}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM122P_SearchOrderClass(string orderClass, int noofshowrec)
        {
            DataSet ds = new DataSet();
            orderClass = ismoUtil.escapeCharacter(orderClass);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM122P_GET_ORDER_CLASS";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = orderClass.ToUpper() == "NULL" ? (object)DBNull.Value : orderClass;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM122P_SearchOrderClass");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("SNM122P_SearchFamilyCode/{familyCode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult SNM122P_SearchFamilyCode(string familyCode, int noofshowrec)
        {
            DataSet ds = new DataSet();
            familyCode = ismoUtil.escapeCharacter(familyCode);

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM122P_GET_FAMILY_CODE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_FAMILY_CODE", OracleDbType.Varchar2).Value = familyCode.ToUpper() == "NULL" ? (object)DBNull.Value : familyCode;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM122P_SearchFamilyCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM122P_GetData")]
        [Authorize]
        public IHttpActionResult SNM122P_GetData([FromBody] dynamic param)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM122P_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_DISCOUNT_GROUP", OracleDbType.Varchar2).Value = param.DISCOUNT_GROUP;
                cmd.Parameters.Add("P_SALES_CHANNEL", OracleDbType.Varchar2).Value = param.SALES_CHANNEL;
                cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = param.ORDER_CLASS;
                cmd.Parameters.Add("P_FAMILY_CODE", OracleDbType.Varchar2).Value = param.DISCOUNT_FAMILY;
                cmd.Parameters.Add("P_DISCOUNT_STATUS", OracleDbType.Varchar2).Value = param.DISCOUNT_STATUS;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM122P_GetData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM122P_Save")]
        public IHttpActionResult SNM122P_Save([FromBody] dynamic[] pData)
        {
            DataSet ds = new DataSet();
            try
            {
                OracleCommand cmd = new OracleCommand();

                foreach (var detail in pData)
                {
                    cmd = new OracleCommand();
                    cmd.CommandText = "SP_NP_SNM122P_SAVE";
                    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                    cmd.Parameters.Add("P_DISCOUNT_GROUP", OracleDbType.Varchar2).Value = detail.DISCOUNT_GROUP;
                    cmd.Parameters.Add("P_SALES_CHANNEL", OracleDbType.Varchar2).Value = detail.SALES_CHANNEL;
                    cmd.Parameters.Add("P_ORDER_CLASS", OracleDbType.Varchar2).Value = detail.ORDER_CLASS;
                    cmd.Parameters.Add("P_DISCOUNT_FAMILY", OracleDbType.Varchar2).Value = detail.DISCOUNT_FAMILY;
                    cmd.Parameters.Add("P_EFFECTIVE_DATE", OracleDbType.Date).Value = detail.EFFECTIVE_DATE;
                    cmd.Parameters.Add("P_DISABLE_DATE", OracleDbType.Date).Value = detail.DISABLE_DATE;
                    cmd.Parameters.Add("P_DISCOUNT_RATE1", OracleDbType.Varchar2).Value = detail.DISCOUNT_RATE1;
                    cmd.Parameters.Add("P_DISCOUNT_RATE2", OracleDbType.Varchar2).Value = detail.DISCOUNT_RATE2;
                    cmd.Parameters.Add("P_DISCOUNT_RATE3", OracleDbType.Varchar2).Value = detail.DISCOUNT_RATE3;
                    cmd.Parameters.Add("P_DISCOUNT_MODE", OracleDbType.Varchar2).Value = detail.DISCOUNT_MODE;
                    cmd.Parameters.Add("P_DISCOUNT_EFFECT", OracleDbType.Varchar2).Value = detail.DISCOUNT_EFFECT;
                    cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                    ds = oOracledb.SqlQueryDS_CMD(cmd);
                }
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM122P_Save");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        #endregion
        #region SNM023P

        [HttpPost, Route("SNM023P_GetData")]
        [Authorize]
        public IHttpActionResult SNM023P_GetData([FromBody] dynamic pDataList)
        {
            try
            {
                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM023P_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_CODE", OracleDbType.Varchar2).Value = pDataList["PART_CODE"];
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM023P_GetData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM023P_Export")]
        [Authorize]
        public HttpResponseMessage SNM023P_Export(List<DataModel.SNM023P> Data)
        {
            HttpResponseMessage response = new HttpResponseMessage(HttpStatusCode.OK);
            string gLocation = "~/tempFile/" + "SNM023P" + "/";
            string gWriteLocation = System.Web.HttpContext.Current.Server.MapPath(gLocation);
            FileInfo filePath = new FileInfo(gWriteLocation);
            filePath.Directory.Create();
            string gFileLocation = Path.Combine(gWriteLocation, "SNM023P.xlsx");
            string gReturnLocation = Path.Combine(gLocation, "SNM023P.xlsx");

            var excel2 = Data.ToExcel(x => x
            .AddColumn("Part Code", z => z.PART_CODE)
            .AddColumn("Part Name", z => z.PART_NAME)
            .AddColumn("Dimension Max", z => z.DIMENSION_MAX)
            .AddColumn("Dimension Mid", z => z.DIMENSION_MID)
            .AddColumn("Dimension Min", z => z.DIMENSION_MIN)
            .AddColumn("Description Code", z => z.DESCRIPTION_CODE)
            .AddColumn("Tariff Code", z => z.TARIFF_CODE)
            .AddColumn("Packaging Code", z => z.PACKAGING_CODE)
            .AddColumn("Expensive Code", z => z.EXPENSIVE_CODE)
            .AddColumn("Last Update By", z => z.UPDATE_BY_NAME)
            .AddColumn("Last Update Date", z => z.LAST_UPD_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(z.LAST_UPD_DATE).ToString("dd-MM-yyyy"))
            );

            byte[] excel = excel2;
            // byte[] excel = dt.ToExcel();
            File.WriteAllBytes(gFileLocation, excel);
            string path = HttpContext.Current.Server.MapPath("~/" + gReturnLocation.Substring(1));
            if (!File.Exists(path))
            {
                oOracledb.WriteLog("ErrorHandle", "File not found.", "SNM023P_Export");
                return Request.CreateErrorResponse(HttpStatusCode.NotFound, "File not found.");
            }
            try
            {
                string mimeType = MimeMapping.GetMimeMapping("SNM023P.xlsx");
                StreamContent x = new StreamContent(new FileStream(path, FileMode.Open, FileAccess.Read));
                response.Content = x;
                response.Content.Headers.ContentType = new MediaTypeHeaderValue(mimeType);
                response.Content.Headers.ContentDisposition = new ContentDispositionHeaderValue("attachment");
                response.Content.Headers.ContentDisposition.FileName = "SNM023P.xlsx";
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM023P_Export");
                return Request.CreateResponse(HttpStatusCode.BadRequest, ex);
            }
            return response;
        }

        [HttpGet, Route("GetDescriptCode/{DescrpitCode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult GetDescriptCode(string DescrpitCode, int noofshowrec)
        {
            try
            {
                DataSet ds = new DataSet();
                DescrpitCode = ismoUtil.escapeCharacter(DescrpitCode);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_MT_DESCRIPT_CODE";
                cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                //cmd.Parameters.Add("p_DESCRIPT_CODE", OracleDbType.Varchar2).Value = (DescrpitCode != null && DescrpitCode == "null" ? "" : DescrpitCode);
                cmd.Parameters.Add("p_DESCRIPT_CODE", OracleDbType.Varchar2).Value = DescrpitCode.ToUpper() != "NULL" ? DescrpitCode.ToUpper() : (object)DBNull.Value;
                cmd.Parameters.Add("p_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GetDescriptCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("GetExpensiveCode/{ExpensiveCode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult GetExpensiveCode(string ExpensiveCode, int noofshowrec)
        {
            try
            {
                DataSet ds = new DataSet();
                ExpensiveCode = ismoUtil.escapeCharacter(ExpensiveCode);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM023P_EXPENSIVE_CODE";
                cmd.Parameters.Add("p_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                //cmd.Parameters.Add("p_EXPENSIVE_CODE", OracleDbType.Varchar2).Value = (ExpensiveCode != null && ExpensiveCode == "null" ? "" : ExpensiveCode);
                cmd.Parameters.Add("p_EXPENSIVE_CODE", OracleDbType.Varchar2).Value = ExpensiveCode.ToUpper() != "NULL" ? ExpensiveCode.ToUpper() : (object)DBNull.Value;
                cmd.Parameters.Add("p_SHOW_RECORD", OracleDbType.Int16).Value = noofshowrec;
                cmd.Parameters.Add("c_Return", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GetExpensiveCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpGet, Route("GetTariffCode/{TariffCode}/{noofshowrec}")]
        [Authorize]
        public IHttpActionResult GetTariffCode(string TariffCode, string noofshowrec)
        {
            try
            {
                DataSet ds = new DataSet();
                TariffCode = ismoUtil.escapeCharacter(TariffCode);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_MT_TARIFF_LIST";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2, DataService.GetIdentity(User, Cons.SS_COMPANY_CODE), ParameterDirection.Input);
                //cmd.Parameters.Add("P_NAME_FILTER", OracleDbType.Varchar2).Value = (TariffCode != null && TariffCode == "null" ? "" : TariffCode);
                cmd.Parameters.Add("P_NAME_FILTER", OracleDbType.Varchar2).Value = TariffCode.ToUpper() != "NULL" ? TariffCode.ToUpper() : (object)DBNull.Value;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor, ParameterDirection.Output);

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "GetTariffCode");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNM023P_PartCode_Insert")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM023P_PartCode_Insert([FromBody] dynamic pDataList)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM023P_INSERT";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_CODE", OracleDbType.Varchar2).Value = pDataList["PART_CODE"];
                cmd.Parameters.Add("P_PART_NAME", OracleDbType.Varchar2).Value = pDataList["PART_NAME_TH"];
                cmd.Parameters.Add("P_DIMENSION_MAX", OracleDbType.Varchar2).Value = pDataList["DIMENSION_MAX"];
                cmd.Parameters.Add("P_DIMENSION_MID", OracleDbType.Varchar2).Value = pDataList["DIMENSION_MID"];
                cmd.Parameters.Add("P_DIMENSION_MIN", OracleDbType.Varchar2).Value = pDataList["DIMENSION_MIN"];
                cmd.Parameters.Add("P_DESCRIPTION_CODE", OracleDbType.Varchar2).Value = pDataList["DESCRIPTION_CODE"];
                cmd.Parameters.Add("P_TARIFF_CODE", OracleDbType.Varchar2).Value = pDataList["TARIFF_CODE"];
                cmd.Parameters.Add("P_PACKAGING_CODE", OracleDbType.Varchar2).Value = pDataList["PACKAGING_CODE"];
                cmd.Parameters.Add("P_EXPENSIVE_CODE", OracleDbType.Varchar2).Value = pDataList["EXPENSIVE_CODE"];
                cmd.Parameters.Add("P_CREATE_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM023P";
                cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_LAST_IP_ADDR", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("C_RE", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM023P_PartCode_Insert");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNM023P_PartCode_Update")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM023P_PartCode_Update([FromBody] dynamic pDataList)
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM023P_UPDATE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_CODE", OracleDbType.Varchar2).Value = pDataList["PART_CODE"];
                cmd.Parameters.Add("P_PART_NAME", OracleDbType.Varchar2).Value = pDataList["PART_NAME_TH"];
                cmd.Parameters.Add("P_DIMENSION_MAX", OracleDbType.Varchar2).Value = pDataList["DIMENSION_MAX"];
                cmd.Parameters.Add("P_DIMENSION_MID", OracleDbType.Varchar2).Value = pDataList["DIMENSION_MID"];
                cmd.Parameters.Add("P_DIMENSION_MIN", OracleDbType.Varchar2).Value = pDataList["DIMENSION_MIN"];
                cmd.Parameters.Add("P_DESCRIPTION_CODE", OracleDbType.Varchar2).Value = pDataList["DESCRIPTION_CODE"];
                cmd.Parameters.Add("P_TARIFF_CODE", OracleDbType.Varchar2).Value = pDataList["TARIFF_CODE"];
                cmd.Parameters.Add("P_PACKAGING_CODE", OracleDbType.Varchar2).Value = pDataList["PACKAGING_CODE"];
                cmd.Parameters.Add("P_EXPENSIVE_CODE", OracleDbType.Varchar2).Value = pDataList["EXPENSIVE_CODE"];
                cmd.Parameters.Add("P_CREATE_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM023P";
                cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_LAST_IP_ADDR", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("C_RE", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM023P_PartCode_Update");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        [HttpPost, Route("SNM023P_PartCode_Delete")]
        [Authorize]
        // [RunBatchGuard]
        public IHttpActionResult SNM023P_PartCode_Delete([FromBody] dynamic pDataList)
        {
            DataSet ds = new DataSet();

            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNM023P_DELETE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_CODE", OracleDbType.Varchar2).Value = pDataList["PART_CODE"];
                cmd.Parameters.Add("P_LAST_UPD_PGM", OracleDbType.Varchar2).Value = "SNM023P";
                cmd.Parameters.Add("P_LAST_UPD_BY", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_USER_NAME);
                cmd.Parameters.Add("P_LAST_IP_ADDR", OracleDbType.Varchar2).Value = HttpContext.Current.Request.UserHostAddress.ToString();
                cmd.Parameters.Add("C_RE", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNM023P_PartCode_Delete");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }

        }

        //[HttpPost, Route("SNM023P_getData_Export")]
        //[Authorize]
        //public HttpResponseMessage SNM023P_getData_Export([FromBody] dynamic pParam)
        //{
        //    DataSet ds = new DataSet();

        //    OracleCommand cmd = new OracleCommand();
        //    cmd.CommandText = "SP_NP_SNM023P_GET_DATA";
        //    cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
        //    cmd.Parameters.Add("P_PART_CODE", OracleDbType.Varchar2).Value = pParam["PART_CODE"];
        //    cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

        //    ds = oOracledb.SqlQueryDS_CMD(cmd);

        //    string x = SNM023P_Export(ds.Tables[0], "SNM023P.xlsx", "SNM023P");
        //    SystemApiController sys = new SystemApiController();
        //    var respons = sys.downloadFile(x.Substring(1), "SNM023P.xlsx");
        //    return respons;
        //}

        //public static string SNM023P_Export(DataTable dt, string fileName, string FloderLocation)
        //{

        //    bool Succes = false;

        //    string gLocation = "~/tempFile/" + FloderLocation + "/";
        //    string gWriteLocation = System.Web.HttpContext.Current.Server.MapPath(gLocation);

        //    FileInfo filePath = new FileInfo(gWriteLocation);
        //    filePath.Directory.Create();
        //    string gFileLocation = Path.Combine(gWriteLocation, fileName);
        //    string gReturnLocation = Path.Combine(gLocation, fileName);
        //    SystemApiController sys = new SystemApiController();
        //    try
        //    {
        //        StringBuilder sb = new StringBuilder();

        //        IEnumerable<string> columnNames = dt.Columns.Cast<DataColumn>().Select(column => column.ColumnName);
        //        sb.AppendLine(string.Join(",", columnNames));

        //        foreach (DataRow row in dt.Rows)
        //        {
        //            for (int i = 0; i < dt.Columns.Count; i++)
        //            {

        //                sb.Append(CsvEscape(row[i].ToString()));
        //                sb.Append(i == dt.Columns.Count - 1 ? "\n" : ",");
        //            }
        //        }

        //        File.WriteAllText(gFileLocation, sb.ToString());

        //        Succes = true;
        //    }
        //    catch (Exception ex)
        //    {
        //        Console.WriteLine(ex);
        //    }

        //    if (Succes)
        //    {
        //        return gReturnLocation;
        //    }
        //    else
        //    {

        //        return "";
        //    }

        //}

        #endregion


        #region SNSQ039_001

        [HttpGet, Route("SNSQ039_GetPartNo/{PartNumber=null}/{NoOfShowRecords=null}")]
        [Authorize]
        public IHttpActionResult SNSQ039_GetPartNo(string PartNumber = null, string NoOfShowRecords = null)
        {
            try
            {
                DataSet ds = new DataSet();
                PartNumber = ismoUtil.escapeCharacter(PartNumber);
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ039_GET_PART_NO";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_PART_NUMBER", OracleDbType.Varchar2).Value = PartNumber.ToUpper() == "NULL" ? (object)DBNull.Value : PartNumber;
                cmd.Parameters.Add("P_SHOW_RECORD", OracleDbType.Int16).Value = NoOfShowRecords;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);
                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ039_GetPartNo");
                return Json(new { Status = "Error" });

            }
        }

        [HttpPost, Route("SNSQ039_GetData")]
        [Authorize]
        public IHttpActionResult SNSQ039_GetData([FromBody] dynamic pData)
        {

            try
            {

                DataSet ds = new DataSet();
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSQ039_GET_DATA";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_FROM_DATE", OracleDbType.Date).Value = pData.FROM_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(pData.FROM_DATE);
                cmd.Parameters.Add("P_TO_DATE", OracleDbType.Date).Value = pData.TO_DATE == null ? (object)DBNull.Value : Convert.ToDateTime(pData.TO_DATE);
                cmd.Parameters.Add("P_PART_NUMBER", OracleDbType.Varchar2).Value = pData.PART_NUMBER;
                cmd.Parameters.Add("C_RETURN", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSQ039_GetData");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSQ039_Export_Excle")]
        public HttpResponseMessage SNSQ039_Export_Excle(List<DataModel.SNSQ039_EXPORT> dModel)
        {

            DataTable dt = new DataTable();
            string x = string.Empty;
            Console.WriteLine();
            string filename = "SNSQ039.csv";
            string floderLocation = "SNSQ039";


            dt.Columns.Add("colum1");

            dt.Rows.Add("ID;PO2C");
            dt.Rows.Add("B");
            dt.Rows.Add("C;X1;Y1;K\"Seq.\"");
            dt.Rows.Add("C;X2;Y1;K\"Part Number\"");
            dt.Rows.Add("C;X3;Y1;K\"Part Name\"");
            dt.Rows.Add("C;X4;Y1;K\"Catalog Model\"");
            dt.Rows.Add("C;X5;Y1;K\"Model Year\"");
            dt.Rows.Add("C;X6;Y1;K\"Catalog Destination\"");
            dt.Rows.Add("C;X7;Y1;K\"Section Number\"");
            dt.Rows.Add("C;X8;Y1;K\"Section Differrence\"");
            dt.Rows.Add("C;X9;Y1;K\"Part Code\"");
            dt.Rows.Add("C;X10;Y1;K\"Part Specification\"");
            dt.Rows.Add("C;X11;Y1;K\"Trim Color1\"");
            dt.Rows.Add("C;X12;Y1;K\"Trim Color2\"");
            dt.Rows.Add("C;X13;Y1;K\"Trim Color3\"");
            dt.Rows.Add("C;X14;Y1;K\"Trim Color4\"");
            dt.Rows.Add("C;X15;Y1;K\"Trim Color5\"");
            dt.Rows.Add("C;X16;Y1;K\"Trim Color6\"");
            dt.Rows.Add("C;X17;Y1;K\"Trim Color7\"");
            dt.Rows.Add("C;X18;Y1;K\"Trim Color8\"");
            dt.Rows.Add("C;X19;Y1;K\"Trim Color9\"");
            dt.Rows.Add("C;X20;Y1;K\"Trim Color10\"");
            dt.Rows.Add("C;X21;Y1;K\"Body Color1\"");
            dt.Rows.Add("C;X22;Y1;K\"Body Color2\"");
            dt.Rows.Add("C;X23;Y1;K\"Body Color3\"");
            dt.Rows.Add("C;X24;Y1;K\"Body Color4\"");
            dt.Rows.Add("C;X25;Y1;K\"Body Color5\"");
            dt.Rows.Add("C;X26;Y1;K\"Body Color6\"");
            dt.Rows.Add("C;X27;Y1;K\"Body Color7\"");
            dt.Rows.Add("C;X28;Y1;K\"Body Color8\"");
            dt.Rows.Add("C;X29;Y1;K\"Body Color9\"");
            dt.Rows.Add("C;X30;Y1;K\"Body Color10\"");
            dt.Rows.Add("C;X31;Y1;K\"Body Color11\"");
            dt.Rows.Add("C;X32;Y1;K\"Body Color12\"");
            dt.Rows.Add("C;X33;Y1;K\"Body Color13\"");
            dt.Rows.Add("C;X34;Y1;K\"Body Color14\"");
            dt.Rows.Add("C;X35;Y1;K\"Body Color15\"");
            dt.Rows.Add("C;X36;Y1;K\"Body Color16\"");
            dt.Rows.Add("C;X37;Y1;K\"Body Color17\"");
            dt.Rows.Add("C;X38;Y1;K\"Body Color18\"");
            dt.Rows.Add("C;X39;Y1;K\"Body Color19\"");
            dt.Rows.Add("C;X40;Y1;K\"Body Color20\"");
            dt.Rows.Add("C;X41;Y1;K\"Adoption Year\"");
            dt.Rows.Add("C;X42;Y1;K\"Adoption Month\"");
            dt.Rows.Add("C;X43;Y1;K\"Abolish Year\"");
            dt.Rows.Add("C;X44;Y1;K\"Abolish Month\"");
            dt.Rows.Add("C;X45;Y1;K\"Unit Production Location\"");
            dt.Rows.Add("C;X46;Y1;K\"Car Production Location\"");
            dt.Rows.Add("C;X47;Y1;K\"Model Code 13 Digit\"");
            dt.Rows.Add("C;X48;Y1;K\"Sourcing Code\"");
            dt.Rows.Add("C;X49;Y1;K\"Model Series Code Car\"");
            dt.Rows.Add("C;X50;Y1;K\"Procurement Addr Loc Code\"");
            dt.Rows.Add("C;X51;Y1;K\"Procurement Hub Addr Loc Code\"");
            dt.Rows.Add("C;X52;Y1;K\"Procurement Hub Addr1 Loc Code\"");
            dt.Rows.Add("C;X53;Y1;K\"Procurement Hub Addr2 Loc Code\"");
            dt.Rows.Add("C;X54;Y1;K\"Procurement Hub Addr3 Loc Code\"");
            dt.Rows.Add("C;X55;Y1;K\"Procurement Hub Addr4 Loc Code\"");
            dt.Rows.Add("C;X56;Y1;K\"Chassy No Code\"");
            dt.Rows.Add("C;X57;Y1;K\"Sales Addr Country Code\"");
            dt.Rows.Add("C;X58;Y1;K\"Reinvoice country\"");
            dt.Rows.Add("C;X59;Y1;K\"Pub No\"");
            dt.Rows.Add("C;X60;Y1;K\"Catalog NC Code\"");
            dt.Rows.Add("C;X61;Y1;K\"Country code1\"");
            dt.Rows.Add("C;X62;Y1;K\"Country code2\"");
            dt.Rows.Add("C;X63;Y1;K\"Country code3\"");
            dt.Rows.Add("C;X64;Y1;K\"Country code4\"");
            dt.Rows.Add("C;X65;Y1;K\"Country code5\"");
            dt.Rows.Add("C;X66;Y1;K\"Country code6\"");
            dt.Rows.Add("C;X67;Y1;K\"Usage\"");
            dt.Rows.Add("C;X68;Y1;K\"Spec Specification1\"");
            dt.Rows.Add("C;X69;Y1;K\"Spec Specification2\"");
            dt.Rows.Add("C;X70;Y1;K\"Spec Specification3\"");
            dt.Rows.Add("C;X71;Y1;K\"Spec Specification4\"");
            dt.Rows.Add("C;X72;Y1;K\"Former Ica Condition\"");
            dt.Rows.Add("C;X73;Y1;K\"Foemer Ica Former Part Number\"");
            dt.Rows.Add("C;X74;Y1;K\"New Ica Condition\"");
            dt.Rows.Add("C;X75;Y1;K\"New Ica Former Part Number\"");
            dt.Rows.Add("C;X76;Y1;K\"Original Model Series Code\"");
            dt.Rows.Add("C;X77;Y1;K\"S Code\"");
            dt.Rows.Add("C;X78;Y1;K\"Quantity\"");
            dt.Rows.Add("C;X79;Y1;K\"Unit Package\"");
            dt.Rows.Add("C;X80;Y1;K\"Description Code\"");
            dt.Rows.Add("C;X81;Y1;K\"List Price\"");
            dt.Rows.Add("C;X82;Y1;K\"Part Letter Number\"");
            dt.Rows.Add("C;X83;Y1;K\"Remarks\"");
            dt.Rows.Add("C;X84;Y1;K\"Infinity Symbol\"");
            dt.Rows.Add("C;X85;Y1;K\"Trasfer status\"");
            dt.Rows.Add("C;X86;Y1;K\"Create By\"");
            dt.Rows.Add("C;X87;Y1;K\"Create Date\"");
            dt.Rows.Add("C;X88;Y1;K\"Last Upt Pgm\"");
            dt.Rows.Add("C;X89;Y1;K\"Last Upt By\"");
            dt.Rows.Add("C;X90;Y1;K\"Last Upt Date\"");
            dt.Rows.Add("C;X91;Y1;K\"Upload Status\"");

            int countrow = 0;
            int itemno = 0;
            for (var i = 0; i < dModel.Count; i++)
            {
                countrow = i + 2;
                itemno = i + 1;

                dt.Rows.Add("C;X1;Y" + countrow + ";K" + '"' + dModel[i].SEQ + '"');
                dt.Rows.Add("C;X2;Y" + countrow + ";K" + '"' + dModel[i].PART_NUMBER + '"');
                dt.Rows.Add("C;X3;Y" + countrow + ";K" + '"' + dModel[i].PART_NAME_ENGLISH + '"');
                dt.Rows.Add("C;X4;Y" + countrow + ";K" + '"' + dModel[i].CATALOG_MODEL + '"');
                dt.Rows.Add("C;X5;Y" + countrow + ";K" + '"' + dModel[i].MODEL_YEAR + '"');
                dt.Rows.Add("C;X6;Y" + countrow + ";K" + '"' + dModel[i].CATALOG_DESTINATION + '"');
                dt.Rows.Add("C;X7;Y" + countrow + ";K" + '"' + dModel[i].SECTION_NUMBER + '"');
                dt.Rows.Add("C;X8;Y" + countrow + ";K" + '"' + dModel[i].SECTION_DIFFERENCE + '"');
                dt.Rows.Add("C;X9;Y" + countrow + ";K" + '"' + dModel[i].PART_CODE + '"');
                dt.Rows.Add("C;X10;Y" + countrow + ";K" + '"' + dModel[i].PART_SPECIFICATION + '"');
                dt.Rows.Add("C;X11;Y" + countrow + ";K" + '"' + dModel[i].TRIM_COLOR1 + '"');
                dt.Rows.Add("C;X12;Y" + countrow + ";K" + '"' + dModel[i].TRIM_COLOR2 + '"');
                dt.Rows.Add("C;X13;Y" + countrow + ";K" + '"' + dModel[i].TRIM_COLOR3 + '"');
                dt.Rows.Add("C;X14;Y" + countrow + ";K" + '"' + dModel[i].TRIM_COLOR4 + '"');
                dt.Rows.Add("C;X15;Y" + countrow + ";K" + '"' + dModel[i].TRIM_COLOR5 + '"');
                dt.Rows.Add("C;X16;Y" + countrow + ";K" + '"' + dModel[i].TRIM_COLOR6 + '"');
                dt.Rows.Add("C;X17;Y" + countrow + ";K" + '"' + dModel[i].TRIM_COLOR7 + '"');
                dt.Rows.Add("C;X18;Y" + countrow + ";K" + '"' + dModel[i].TRIM_COLOR8 + '"');
                dt.Rows.Add("C;X19;Y" + countrow + ";K" + '"' + dModel[i].TRIM_COLOR9 + '"');
                dt.Rows.Add("C;X20;Y" + countrow + ";K" + '"' + dModel[i].TRIM_COLOR10 + '"');
                dt.Rows.Add("C;X21;Y" + countrow + ";K" + '"' + dModel[i].BODY_COLOR1 + '"');
                dt.Rows.Add("C;X22;Y" + countrow + ";K" + '"' + dModel[i].BODY_COLOR2 + '"');
                dt.Rows.Add("C;X23;Y" + countrow + ";K" + '"' + dModel[i].BODY_COLOR3 + '"');
                dt.Rows.Add("C;X24;Y" + countrow + ";K" + '"' + dModel[i].BODY_COLOR4 + '"');
                dt.Rows.Add("C;X25;Y" + countrow + ";K" + '"' + dModel[i].BODY_COLOR5 + '"');
                dt.Rows.Add("C;X26;Y" + countrow + ";K" + '"' + dModel[i].BODY_COLOR6 + '"');
                dt.Rows.Add("C;X27;Y" + countrow + ";K" + '"' + dModel[i].BODY_COLOR7 + '"');
                dt.Rows.Add("C;X28;Y" + countrow + ";K" + '"' + dModel[i].BODY_COLOR8 + '"');
                dt.Rows.Add("C;X29;Y" + countrow + ";K" + '"' + dModel[i].BODY_COLOR9 + '"');
                dt.Rows.Add("C;X30;Y" + countrow + ";K" + '"' + dModel[i].BODY_COLOR10 + '"');
                dt.Rows.Add("C;X31;Y" + countrow + ";K" + '"' + dModel[i].BODY_COLOR11 + '"');
                dt.Rows.Add("C;X32;Y" + countrow + ";K" + '"' + dModel[i].BODY_COLOR12 + '"');
                dt.Rows.Add("C;X33;Y" + countrow + ";K" + '"' + dModel[i].BODY_COLOR13 + '"');
                dt.Rows.Add("C;X34;Y" + countrow + ";K" + '"' + dModel[i].BODY_COLOR14 + '"');
                dt.Rows.Add("C;X35;Y" + countrow + ";K" + '"' + dModel[i].BODY_COLOR15 + '"');
                dt.Rows.Add("C;X36;Y" + countrow + ";K" + '"' + dModel[i].BODY_COLOR16 + '"');
                dt.Rows.Add("C;X37;Y" + countrow + ";K" + '"' + dModel[i].BODY_COLOR17 + '"');
                dt.Rows.Add("C;X38;Y" + countrow + ";K" + '"' + dModel[i].BODY_COLOR18 + '"');
                dt.Rows.Add("C;X39;Y" + countrow + ";K" + '"' + dModel[i].BODY_COLOR19 + '"');
                dt.Rows.Add("C;X40;Y" + countrow + ";K" + '"' + dModel[i].BODY_COLOR20 + '"');
                dt.Rows.Add("C;X41;Y" + countrow + ";K" + '"' + dModel[i].ADOPTION_YEAR + '"');
                dt.Rows.Add("C;X42;Y" + countrow + ";K" + '"' + dModel[i].ADOPTION_MONTH + '"');
                dt.Rows.Add("C;X43;Y" + countrow + ";K" + '"' + dModel[i].ABOLISH_YEAR + '"');
                dt.Rows.Add("C;X44;Y" + countrow + ";K" + '"' + dModel[i].ABOLISH_MONTH + '"');
                dt.Rows.Add("C;X45;Y" + countrow + ";K" + '"' + dModel[i].UNIT_PRODUCTION_LOCATION + '"');
                dt.Rows.Add("C;X46;Y" + countrow + ";K" + '"' + dModel[i].CAR_PRODUCTION_LOCATION + '"');
                dt.Rows.Add("C;X47;Y" + countrow + ";K" + '"' + dModel[i].MODEL_CODE_13_DIGIT + '"');
                dt.Rows.Add("C;X48;Y" + countrow + ";K" + '"' + dModel[i].SOURCING_CODE + '"');
                dt.Rows.Add("C;X49;Y" + countrow + ";K" + '"' + dModel[i].MODEL_SERIES_CODE_CAR + '"');
                dt.Rows.Add("C;X50;Y" + countrow + ";K" + '"' + dModel[i].PROCUREMENT_ADDR_LOC_CODE + '"');
                dt.Rows.Add("C;X51;Y" + countrow + ";K" + '"' + dModel[i].PROCUREMENT_HUB_ADDR_LOC_CODE + '"');
                dt.Rows.Add("C;X52;Y" + countrow + ";K" + '"' + dModel[i].PROCUREMENT_HUB_ADDR1_LOC_CODE + '"');
                dt.Rows.Add("C;X53;Y" + countrow + ";K" + '"' + dModel[i].PROCUREMENT_HUB_ADDR2_LOC_CODE + '"');
                dt.Rows.Add("C;X54;Y" + countrow + ";K" + '"' + dModel[i].PROCUREMENT_HUB_ADDR3_LOC_CODE + '"');
                dt.Rows.Add("C;X55;Y" + countrow + ";K" + '"' + dModel[i].PROCUREMENT_HUB_ADDR4_LOC_CODE + '"');
                dt.Rows.Add("C;X56;Y" + countrow + ";K" + '"' + dModel[i].CHASSY_NO_CODE + '"');
                dt.Rows.Add("C;X57;Y" + countrow + ";K" + '"' + dModel[i].SALES_ADDR_COUNTRY_CODE + '"');
                dt.Rows.Add("C;X58;Y" + countrow + ";K" + '"' + dModel[i].REINVOICE_COUNTRY_CODE + '"');
                dt.Rows.Add("C;X59;Y" + countrow + ";K" + '"' + dModel[i].PUB_NO + '"');
                dt.Rows.Add("C;X60;Y" + countrow + ";K" + '"' + dModel[i].CATALOG_NC_CODE + '"');
                dt.Rows.Add("C;X61;Y" + countrow + ";K" + '"' + dModel[i].COUNTRY_CODE1 + '"');
                dt.Rows.Add("C;X62;Y" + countrow + ";K" + '"' + dModel[i].COUNTRY_CODE2 + '"');
                dt.Rows.Add("C;X63;Y" + countrow + ";K" + '"' + dModel[i].COUNTRY_CODE3 + '"');
                dt.Rows.Add("C;X64;Y" + countrow + ";K" + '"' + dModel[i].COUNTRY_CODE4 + '"');
                dt.Rows.Add("C;X65;Y" + countrow + ";K" + '"' + dModel[i].COUNTRY_CODE5 + '"');
                dt.Rows.Add("C;X66;Y" + countrow + ";K" + '"' + dModel[i].COUNTRY_CODE6 + '"');
                dt.Rows.Add("C;X67;Y" + countrow + ";K" + '"' + dModel[i].USAGE + '"');
                dt.Rows.Add("C;X68;Y" + countrow + ";K" + '"' + dModel[i].SPEC_SPECIFICATION1 + '"');
                dt.Rows.Add("C;X69;Y" + countrow + ";K" + '"' + dModel[i].SPEC_SPECIFICATION2 + '"');
                dt.Rows.Add("C;X70;Y" + countrow + ";K" + '"' + dModel[i].SPEC_SPECIFICATION3 + '"');
                dt.Rows.Add("C;X71;Y" + countrow + ";K" + '"' + dModel[i].SPEC_SPECIFICATION4 + '"');
                dt.Rows.Add("C;X72;Y" + countrow + ";K" + '"' + dModel[i].FORMER_ICA_CONDITION + '"');
                dt.Rows.Add("C;X73;Y" + countrow + ";K" + '"' + dModel[i].FORMER_ICA_FORMER_PART_NUMBER + '"');
                dt.Rows.Add("C;X74;Y" + countrow + ";K" + '"' + dModel[i].NEW_ICA_CONDITION + '"');
                dt.Rows.Add("C;X75;Y" + countrow + ";K" + '"' + dModel[i].NEW_ICA_FORMER_PART_NUMBER + '"');
                dt.Rows.Add("C;X76;Y" + countrow + ";K" + '"' + dModel[i].ORIGINAL_MODEL_SERIES_CODE + '"');
                dt.Rows.Add("C;X77;Y" + countrow + ";K" + '"' + dModel[i].S_CODE + '"');
                dt.Rows.Add("C;X78;Y" + countrow + ";K" + '"' + dModel[i].QUANTITY + '"');
                dt.Rows.Add("C;X79;Y" + countrow + ";K" + '"' + dModel[i].UNIT_PACKAGE + '"');
                dt.Rows.Add("C;X80;Y" + countrow + ";K" + '"' + dModel[i].DESCRIPTION_CODE + '"');
                dt.Rows.Add("C;X81;Y" + countrow + ";K" + '"' + dModel[i].LIST_PRICE + '"');
                dt.Rows.Add("C;X82;Y" + countrow + ";K" + '"' + dModel[i].PART_LETTER_NUMBER + '"');
                dt.Rows.Add("C;X83;Y" + countrow + ";K" + '"' + dModel[i].REMARKS + '"');
                dt.Rows.Add("C;X84;Y" + countrow + ";K" + '"' + dModel[i].INFINITY_SYMBOL + '"');
                dt.Rows.Add("C;X85;Y" + countrow + ";K" + '"' + dModel[i].RECEIVED_STATUS + '"');
                dt.Rows.Add("C;X86;Y" + countrow + ";K" + '"' + dModel[i].CREATE_BY + '"');
                dt.Rows.Add("C;X87;Y" + countrow + ";K" + '"' + dModel[i].CREATE_DATE + '"');
                dt.Rows.Add("C;X88;Y" + countrow + ";K" + '"' + dModel[i].LAST_UPD_PGM + '"');
                dt.Rows.Add("C;X89;Y" + countrow + ";K" + '"' + dModel[i].LAST_UPD_BY + '"');
                dt.Rows.Add("C;X90;Y" + countrow + ";K" + '"' + dModel[i].LAST_UPD_DATE + '"');
                dt.Rows.Add("C;X91;Y" + countrow + ";K" + '"' + dModel[i].ATTRIBUTE1 + '"');
            }

            dt.Rows.Add("E");

            x = DataService.WriteCSVNoColumn(dt, filename, floderLocation);
            Controllers.SystemApiController response = new Controllers.SystemApiController();
            var File = response.downloadFile(x.Substring(1), filename);
            return File;
        }

        #endregion


        #region SNSE140
        [HttpPost, Route("SNSE140_GetPrice")]
        [Authorize]
        public IHttpActionResult SNSE140_GetPrice([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE140_GET_PRICE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_FROM_DATE", OracleDbType.Date).Value = Convert.ToDateTime(pData["FROM_DATE"]);
                cmd.Parameters.Add("P_TO_DATE", OracleDbType.Date).Value = Convert.ToDateTime(pData["TO_DATE"]);
                cmd.Parameters.Add("P_SUPPLIER_CODE", OracleDbType.Varchar2).Value = pData["SUPPLIER_CODE"];
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pData["PART_NO"];
                cmd.Parameters.Add("P_RECEIVED_STATUS", OracleDbType.Varchar2).Value = pData["RECEIVED_STATUS"];
                cmd.Parameters.Add("C_CURRENT_PRICE", OracleDbType.RefCursor).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("C_FUTURE_PRICE", OracleDbType.RefCursor).Direction = ParameterDirection.Output;

                DataSet ds = oOracledb.SqlQueryDS_CMD(cmd);

                return Json(DTFM.convertToList(ds));
            } catch(Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE140_GetPrice");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }

        [HttpPost, Route("SNSE140_SavePrice")]
        [Authorize]
        public IHttpActionResult SNSE140_SavePrice([FromBody] dynamic pData)
        {
            try
            {
                OracleCommand cmd = new OracleCommand();
                cmd.CommandText = "SP_NP_SNSE140_SAVE_PRICE";
                cmd.Parameters.Add("P_COMPANY_CODE", OracleDbType.Varchar2).Value = DataService.GetIdentity(User, Cons.SS_COMPANY_CODE);
                cmd.Parameters.Add("P_FROM_DATE", OracleDbType.Date).Value = Convert.ToDateTime(pData["FROM_DATE"]);
                cmd.Parameters.Add("P_TO_DATE", OracleDbType.Date).Value = Convert.ToDateTime(pData["TO_DATE"]);
                cmd.Parameters.Add("P_SUPPLIER_CODE", OracleDbType.Varchar2).Value = pData["SUPPLIER_CODE"];
                cmd.Parameters.Add("P_PART_NO", OracleDbType.Varchar2).Value = pData["PART_NO"];
                cmd.Parameters.Add("P_RECEIVED_STATUS", OracleDbType.Varchar2).Value = pData["RECEIVED_STATUS"];
                cmd.Parameters.Add("P_COMPLETE_CNT", OracleDbType.Decimal).Direction = ParameterDirection.Output;
                cmd.Parameters.Add("P_ERROR_CNT", OracleDbType.Decimal).Direction = ParameterDirection.Output;

                cmd = oOracledb.SqlQueryReturnValue(cmd);

                string completedCount = cmd.Parameters["P_COMPLETE_CNT"].ToString();
                string errorCount = cmd.Parameters["P_ERROR_CNT"].ToString();

                return Ok(new { completed = completedCount, error = errorCount });
            }
            catch (Exception ex)
            {
                oOracledb.WriteLog("ErrorHandle", ex.Message, "SNSE140_SavePrice");
                return ResponseMessage(Request.CreateResponse(HttpStatusCode.InternalServerError, ex.Message));
            }
        }
        #endregion
    }
}